/********************************************************************************
** Form generated from reading UI file 'introwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INTROWIZARDPAGE_H
#define UI_INTROWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_IntroPage
{
public:
    QHBoxLayout *horizontalLayout;
    QSpacerItem *leftSpacer;
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QFrame *line_hor;
    QHBoxLayout *horizontalLayout_radio;
    QSpacerItem *horizontalSpacer_mid;
    QGridLayout *gridLayout_2;
    QRadioButton *m_profilesBank;
    QRadioButton *m_profilesInvest;
    QRadioButton *m_profilesStockPrices;
    QRadioButton *m_profilesCurrencyPrices;
    QSpacerItem *horizontalSpacer_2;
    QGridLayout *gridLayout;
    QPushButton *m_add;
    QCheckBox *m_skipSetup;
    QPushButton *m_remove;
    QPushButton *m_rename;
    QComboBox *m_profiles;
    QSpacerItem *rightSpacer;

    void setupUi(QWidget *IntroPage)
    {
        if (IntroPage->objectName().isEmpty())
            IntroPage->setObjectName("IntroPage");
        IntroPage->resize(417, 281);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(IntroPage->sizePolicy().hasHeightForWidth());
        IntroPage->setSizePolicy(sizePolicy);
        horizontalLayout = new QHBoxLayout(IntroPage);
        horizontalLayout->setObjectName("horizontalLayout");
        leftSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(leftSpacer);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        label = new QLabel(IntroPage);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignCenter);
        label->setWordWrap(true);

        verticalLayout_2->addWidget(label);

        line_hor = new QFrame(IntroPage);
        line_hor->setObjectName("line_hor");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(line_hor->sizePolicy().hasHeightForWidth());
        line_hor->setSizePolicy(sizePolicy1);
        line_hor->setFrameShape(QFrame::Shape::HLine);
        line_hor->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line_hor);

        horizontalLayout_radio = new QHBoxLayout();
        horizontalLayout_radio->setObjectName("horizontalLayout_radio");
        horizontalSpacer_mid = new QSpacerItem(13, 22, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_radio->addItem(horizontalSpacer_mid);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        m_profilesBank = new QRadioButton(IntroPage);
        m_profilesBank->setObjectName("m_profilesBank");
        m_profilesBank->setEnabled(true);
        m_profilesBank->setMinimumSize(QSize(0, 28));

        gridLayout_2->addWidget(m_profilesBank, 0, 0, 1, 1);

        m_profilesInvest = new QRadioButton(IntroPage);
        m_profilesInvest->setObjectName("m_profilesInvest");
        m_profilesInvest->setEnabled(true);
        m_profilesInvest->setMinimumSize(QSize(0, 28));

        gridLayout_2->addWidget(m_profilesInvest, 1, 0, 1, 1);

        m_profilesStockPrices = new QRadioButton(IntroPage);
        m_profilesStockPrices->setObjectName("m_profilesStockPrices");
        m_profilesStockPrices->setMinimumSize(QSize(0, 28));

        gridLayout_2->addWidget(m_profilesStockPrices, 1, 1, 1, 1);

        m_profilesCurrencyPrices = new QRadioButton(IntroPage);
        m_profilesCurrencyPrices->setObjectName("m_profilesCurrencyPrices");

        gridLayout_2->addWidget(m_profilesCurrencyPrices, 0, 1, 1, 1);


        horizontalLayout_radio->addLayout(gridLayout_2);

        horizontalSpacer_2 = new QSpacerItem(13, 22, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_radio->addItem(horizontalSpacer_2);


        verticalLayout_2->addLayout(horizontalLayout_radio);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        m_add = new QPushButton(IntroPage);
        m_add->setObjectName("m_add");
        m_add->setEnabled(false);
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(m_add->sizePolicy().hasHeightForWidth());
        m_add->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(m_add, 2, 0, 1, 1);

        m_skipSetup = new QCheckBox(IntroPage);
        m_skipSetup->setObjectName("m_skipSetup");
        sizePolicy2.setHeightForWidth(m_skipSetup->sizePolicy().hasHeightForWidth());
        m_skipSetup->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(m_skipSetup, 1, 3, 1, 1);

        m_remove = new QPushButton(IntroPage);
        m_remove->setObjectName("m_remove");
        m_remove->setEnabled(false);
        sizePolicy2.setHeightForWidth(m_remove->sizePolicy().hasHeightForWidth());
        m_remove->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(m_remove, 2, 1, 1, 1);

        m_rename = new QPushButton(IntroPage);
        m_rename->setObjectName("m_rename");
        m_rename->setEnabled(false);
        sizePolicy2.setHeightForWidth(m_rename->sizePolicy().hasHeightForWidth());
        m_rename->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(m_rename, 2, 2, 1, 1);

        m_profiles = new QComboBox(IntroPage);
        m_profiles->setObjectName("m_profiles");
        m_profiles->setEnabled(false);
        sizePolicy2.setHeightForWidth(m_profiles->sizePolicy().hasHeightForWidth());
        m_profiles->setSizePolicy(sizePolicy2);
        m_profiles->setEditable(true);
        m_profiles->setInsertPolicy(QComboBox::InsertAtCurrent);

        gridLayout->addWidget(m_profiles, 1, 0, 1, 3);


        verticalLayout_2->addLayout(gridLayout);

        verticalLayout_2->setStretch(0, 20);
        verticalLayout_2->setStretch(1, 1);
        verticalLayout_2->setStretch(2, 13);

        horizontalLayout->addLayout(verticalLayout_2);

        rightSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(rightSpacer);

        horizontalLayout->setStretch(0, 1);
        horizontalLayout->setStretch(1, 10);
        horizontalLayout->setStretch(2, 1);
        QWidget::setTabOrder(m_profilesBank, m_profilesInvest);
        QWidget::setTabOrder(m_profilesInvest, m_profilesCurrencyPrices);
        QWidget::setTabOrder(m_profilesCurrencyPrices, m_profilesStockPrices);
        QWidget::setTabOrder(m_profilesStockPrices, m_profiles);
        QWidget::setTabOrder(m_profiles, m_skipSetup);
        QWidget::setTabOrder(m_skipSetup, m_add);
        QWidget::setTabOrder(m_add, m_remove);
        QWidget::setTabOrder(m_remove, m_rename);

        retranslateUi(IntroPage);

        m_profiles->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(IntroPage);
    } // setupUi

    void retranslateUi(QWidget *IntroPage)
    {
        IntroPage->setWindowTitle(tr2i18n("Intro Wizard Page", nullptr));
        label->setText(tr2i18n("Click either Banking, Investment or Prices.\n"
"Then select existing profile or create new\n"
"by entering its name and selecting 'Add'.\n"
"Then open the file you wish to import.", nullptr));
        m_profilesBank->setText(tr2i18n("Ban&king", nullptr));
        m_profilesInvest->setText(tr2i18n("In&vestment", nullptr));
        m_profilesStockPrices->setText(tr2i18n("Stock p&rices", nullptr));
        m_profilesCurrencyPrices->setText(tr2i18n("C&urrency prices", nullptr));
#if QT_CONFIG(tooltip)
        m_add->setToolTip(tr2i18n("After entering new profile name, press this button to add it.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_add->setText(tr2i18n("Add", nullptr));
#if QT_CONFIG(tooltip)
        m_skipSetup->setToolTip(tr2i18n("Select to bypass profile setup.\n"
"Only useful if a profile has already been setup.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_skipSetup->setText(tr2i18n("Skip setup", nullptr));
#if QT_CONFIG(tooltip)
        m_remove->setToolTip(tr2i18n("Press this button to remove selected profile.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_remove->setText(tr2i18n("Remove", nullptr));
#if QT_CONFIG(tooltip)
        m_rename->setToolTip(tr2i18n("Press this button after renaming your profile.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_rename->setText(tr2i18n("Rename", nullptr));
#if QT_CONFIG(tooltip)
        m_profiles->setToolTip(tr2i18n("Select your importing profile here.", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class IntroPage: public Ui_IntroPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // INTROWIZARDPAGE_H

