/****************************************************************************
** Meta object code from reading C++ file 'pivotgrid.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../../kmymoney-5.2.2/kmymoney/plugins/views/reports/core/pivotgrid.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pivotgrid.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7reportsE_t {};
} // unnamed namespace

template <> constexpr inline auto reports::qt_create_metaobjectdata<qt_meta_tag_ZN7reportsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "reports",
        "ERowType",
        "eActual",
        "eBudget",
        "eBudgetDiff",
        "eForecast",
        "eAverage",
        "ePrice"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ERowType'
        QtMocHelpers::EnumData<ERowType>(1, 1, QMC::EnumFlags{}).add({
            {    2, ERowType::eActual },
            {    3, ERowType::eBudget },
            {    4, ERowType::eBudgetDiff },
            {    5, ERowType::eForecast },
            {    6, ERowType::eAverage },
            {    7, ERowType::ePrice },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN7reportsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN7reportsE =
    reports::qt_create_metaobjectdata<qt_meta_tag_ZN7reportsE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN7reportsE =
    qt_staticMetaObjectContent_ZN7reportsE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN7reportsE =
    qt_staticMetaObjectContent_ZN7reportsE.relocatingData;

Q_CONSTINIT const QMetaObject reports::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN7reportsE.stringdata,
    qt_staticMetaObjectStaticContent_ZN7reportsE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN7reportsE.metaTypes,
    nullptr
} };

QT_WARNING_POP
