/********************************************************************************
** Form generated from reading UI file 'newtransactioneditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NEWTRANSACTIONEDITOR_H
#define UI_NEWTRANSACTIONEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "creditdebitedit.h"
#include "kdatecombobox.h"
#include "kmymoneyaccountcombo.h"
#include "kmymoneydateedit.h"
#include "ktagcontainer.h"

QT_BEGIN_NAMESPACE

class Ui_NewTransactionEditor
{
public:
    QGridLayout *gridLayout;
    QLabel *accountLabel;
    QLineEdit *numberEdit;
    QLabel *statusLabel;
    QHBoxLayout *hs3;
    QSpacerItem *horizontalSpacer_3;
    QLabel *amountLabel;
    CreditDebitEdit *creditDebitEdit;
    QLabel *postdateLabel;
    KTagContainer *tagContainer;
    QLabel *label;
    QPlainTextEdit *memoEdit;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QToolButton *enterButton;
    QToolButton *cancelButton;
    KMyMoneyAccountCombo *accountCombo;
    QLabel *memoLabel;
    QSpacerItem *verticalSpacer_2;
    QComboBox *costCenterCombo;
    QWidget *rightMarginWidget;
    QLabel *tagsLabel;
    QLabel *categoryLabel;
    QComboBox *statusCombo;
    QLabel *costCenterLabel;
    QLabel *numberLabel;
    QHBoxLayout *horizontalLayout_4;
    KMyMoneyDateEdit *dateEdit;
    QSpacerItem *hs2;
    KMyMoneyAccountCombo *categoryCombo;
    QComboBox *payeeEdit;
    QWidget *topMarginWidget;
    QLabel *linkLabel;

    void setupUi(QWidget *NewTransactionEditor)
    {
        if (NewTransactionEditor->objectName().isEmpty())
            NewTransactionEditor->setObjectName("NewTransactionEditor");
        NewTransactionEditor->resize(679, 314);
        gridLayout = new QGridLayout(NewTransactionEditor);
        gridLayout->setObjectName("gridLayout");
        accountLabel = new QLabel(NewTransactionEditor);
        accountLabel->setObjectName("accountLabel");

        gridLayout->addWidget(accountLabel, 1, 0, 1, 1);

        numberEdit = new QLineEdit(NewTransactionEditor);
        numberEdit->setObjectName("numberEdit");
        numberEdit->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(numberEdit, 3, 4, 1, 1);

        statusLabel = new QLabel(NewTransactionEditor);
        statusLabel->setObjectName("statusLabel");

        gridLayout->addWidget(statusLabel, 5, 3, 1, 1);

        hs3 = new QHBoxLayout();
        hs3->setSpacing(6);
        hs3->setObjectName("hs3");
        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hs3->addItem(horizontalSpacer_3);

        amountLabel = new QLabel(NewTransactionEditor);
        amountLabel->setObjectName("amountLabel");

        hs3->addWidget(amountLabel);

        creditDebitEdit = new CreditDebitEdit(NewTransactionEditor);
        creditDebitEdit->setObjectName("creditDebitEdit");
        creditDebitEdit->setMinimumSize(QSize(220, 0));
        creditDebitEdit->setProperty("kmm_taborder", QVariant(true));

        hs3->addWidget(creditDebitEdit);


        gridLayout->addLayout(hs3, 2, 2, 1, 3);

        postdateLabel = new QLabel(NewTransactionEditor);
        postdateLabel->setObjectName("postdateLabel");

        gridLayout->addWidget(postdateLabel, 2, 0, 1, 1);

        tagContainer = new KTagContainer(NewTransactionEditor);
        tagContainer->setObjectName("tagContainer");
        tagContainer->setFocusPolicy(Qt::WheelFocus);
        tagContainer->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(tagContainer, 5, 1, 1, 2);

        label = new QLabel(NewTransactionEditor);
        label->setObjectName("label");

        gridLayout->addWidget(label, 3, 0, 1, 1);

        memoEdit = new QPlainTextEdit(NewTransactionEditor);
        memoEdit->setObjectName("memoEdit");
        memoEdit->setTabChangesFocus(true);
        memoEdit->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(memoEdit, 6, 1, 2, 2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        enterButton = new QToolButton(NewTransactionEditor);
        enterButton->setObjectName("enterButton");
        enterButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        enterButton->setProperty("kmm_taborder", QVariant(true));

        horizontalLayout->addWidget(enterButton);

        cancelButton = new QToolButton(NewTransactionEditor);
        cancelButton->setObjectName("cancelButton");
        cancelButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        cancelButton->setProperty("kmm_taborder", QVariant(true));

        horizontalLayout->addWidget(cancelButton);


        gridLayout->addLayout(horizontalLayout, 7, 3, 1, 2);

        accountCombo = new KMyMoneyAccountCombo(NewTransactionEditor);
        accountCombo->setObjectName("accountCombo");
        accountCombo->setEditable(true);
        accountCombo->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(accountCombo, 1, 1, 1, 2);

        memoLabel = new QLabel(NewTransactionEditor);
        memoLabel->setObjectName("memoLabel");
        memoLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout->addWidget(memoLabel, 6, 0, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 48, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer_2, 6, 4, 1, 1);

        costCenterCombo = new QComboBox(NewTransactionEditor);
        costCenterCombo->setObjectName("costCenterCombo");
        costCenterCombo->setEditable(true);
        costCenterCombo->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(costCenterCombo, 4, 4, 1, 1);

        rightMarginWidget = new QWidget(NewTransactionEditor);
        rightMarginWidget->setObjectName("rightMarginWidget");

        gridLayout->addWidget(rightMarginWidget, 0, 5, 8, 1);

        tagsLabel = new QLabel(NewTransactionEditor);
        tagsLabel->setObjectName("tagsLabel");

        gridLayout->addWidget(tagsLabel, 5, 0, 1, 1);

        categoryLabel = new QLabel(NewTransactionEditor);
        categoryLabel->setObjectName("categoryLabel");

        gridLayout->addWidget(categoryLabel, 4, 0, 1, 1);

        statusCombo = new QComboBox(NewTransactionEditor);
        statusCombo->addItem(QString());
        statusCombo->setObjectName("statusCombo");
        statusCombo->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(statusCombo, 5, 4, 1, 1);

        costCenterLabel = new QLabel(NewTransactionEditor);
        costCenterLabel->setObjectName("costCenterLabel");

        gridLayout->addWidget(costCenterLabel, 4, 3, 1, 1);

        numberLabel = new QLabel(NewTransactionEditor);
        numberLabel->setObjectName("numberLabel");

        gridLayout->addWidget(numberLabel, 3, 3, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        dateEdit = new KMyMoneyDateEdit(NewTransactionEditor);
        dateEdit->setObjectName("dateEdit");
        dateEdit->setProperty("kmm_taborder", QVariant(true));

        horizontalLayout_4->addWidget(dateEdit);

        hs2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(hs2);


        gridLayout->addLayout(horizontalLayout_4, 2, 1, 1, 1);

        categoryCombo = new KMyMoneyAccountCombo(NewTransactionEditor);
        categoryCombo->setObjectName("categoryCombo");
        categoryCombo->setEditable(true);
        categoryCombo->setIconSize(QSize(0, 0));
        categoryCombo->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(categoryCombo, 4, 1, 1, 2);

        payeeEdit = new QComboBox(NewTransactionEditor);
        payeeEdit->setObjectName("payeeEdit");
        payeeEdit->setEditable(true);
        payeeEdit->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(payeeEdit, 3, 1, 1, 2);

        topMarginWidget = new QWidget(NewTransactionEditor);
        topMarginWidget->setObjectName("topMarginWidget");

        gridLayout->addWidget(topMarginWidget, 0, 0, 1, 5);

        linkLabel = new QLabel(NewTransactionEditor);
        linkLabel->setObjectName("linkLabel");
        linkLabel->setEnabled(true);

        gridLayout->addWidget(linkLabel, 7, 0, 1, 1);

        gridLayout->setColumnStretch(2, 5);
        QWidget::setTabOrder(accountCombo, dateEdit);
        QWidget::setTabOrder(dateEdit, payeeEdit);
        QWidget::setTabOrder(payeeEdit, numberEdit);
        QWidget::setTabOrder(numberEdit, categoryCombo);
        QWidget::setTabOrder(categoryCombo, costCenterCombo);
        QWidget::setTabOrder(costCenterCombo, tagContainer);
        QWidget::setTabOrder(tagContainer, statusCombo);
        QWidget::setTabOrder(statusCombo, memoEdit);
        QWidget::setTabOrder(memoEdit, enterButton);
        QWidget::setTabOrder(enterButton, cancelButton);

        retranslateUi(NewTransactionEditor);

        QMetaObject::connectSlotsByName(NewTransactionEditor);
    } // setupUi

    void retranslateUi(QWidget *NewTransactionEditor)
    {
        accountLabel->setText(tr2i18n("Account", nullptr));
#if QT_CONFIG(tooltip)
        numberEdit->setToolTip(tr2i18n("The check number used for this transaction.", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
        statusLabel->setText(tr2i18n("Status", nullptr));
        amountLabel->setText(tr2i18n("Amount", nullptr));
#if QT_CONFIG(tooltip)
        creditDebitEdit->setToolTip(tr2i18n("Enter the amount of the transaction.", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
        postdateLabel->setText(tr2i18n("Date", nullptr));
        label->setText(tr2i18n("Payer/Payee", nullptr));
        enterButton->setText(tr2i18n("Enter", nullptr));
        cancelButton->setText(tr2i18n("Cancel", nullptr));
        memoLabel->setText(tr2i18n("Memo", nullptr));
#if QT_CONFIG(tooltip)
        costCenterCombo->setToolTip(tr2i18n("The cost center this transaction should be assigned to.", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        rightMarginWidget->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
        tagsLabel->setText(tr2i18n("Tags", nullptr));
        categoryLabel->setText(tr2i18n("Category", nullptr));
        statusCombo->setItemText(0, tr2i18n("Cleared", nullptr));

        costCenterLabel->setText(tr2i18n("Costcenter", nullptr));
        numberLabel->setText(tr2i18n("Number", nullptr));
#if QT_CONFIG(tooltip)
        dateEdit->setToolTip(tr2i18n("The posting date of the transaction.", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        categoryCombo->setToolTip(tr2i18n("Enter the category or counter account of the transaction.", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        topMarginWidget->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
        linkLabel->setText(tr2i18n("Link", nullptr));
        (void)NewTransactionEditor;
    } // retranslateUi

};

namespace Ui {
    class NewTransactionEditor: public Ui_NewTransactionEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // NEWTRANSACTIONEDITOR_H

