/****************************************************************************
** Meta object code from reading C++ file 'kmymoney.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kmymoney-5.2.2/kmymoney/kmymoney.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kmymoney.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11KMyMoneyAppE_t {};
} // unnamed namespace

template <> constexpr inline auto KMyMoneyApp::qt_create_metaobjectdata<qt_meta_tag_ZN11KMyMoneyAppE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KMyMoneyApp",
        "fileLoaded",
        "",
        "QUrl",
        "url",
        "transactionSelected",
        "accountId",
        "transactionId",
        "selectAllTransactions",
        "institutionSelected",
        "MyMoneyInstitution",
        "institution",
        "scheduleSelected",
        "MyMoneySchedule",
        "schedule",
        "startMatchTransaction",
        "MyMoneyTransaction",
        "t",
        "cancelMatchTransaction",
        "kmmFilePlugin",
        "slotInstallConsistencyCheckContextMenu",
        "slotShowContextMenuForConsistencyCheck",
        "QPoint",
        "slotSelectionChanged",
        "SelectedObjects",
        "selection",
        "slotAddSharedAction",
        "eMenu::Action",
        "action",
        "QAction*",
        "defaultAction",
        "slotToggleMenuBar",
        "showMenuBar",
        "pauseAutoSave",
        "id",
        "pause",
        "slotAutoSave",
        "slotEnableMessages",
        "slotGetOnlineHelp",
        "slotWhatsNew",
        "slotVisitWebsite",
        "slotPerformanceTest",
        "slotGenerateSql",
        "slotToggleTimers",
        "slotFileViewPersonal",
        "slotLoadAccountTemplates",
        "slotSaveAccountTemplates",
        "slotSettings",
        "slotShowCredits",
        "slotBackupFile",
        "slotBackupMount",
        "slotBackupWriteFile",
        "slotBackupUnmount",
        "slotBackupFinish",
        "slotBackupHandleEvents",
        "slotUpdateConfiguration",
        "dialogName",
        "slotNewFeature",
        "slotEditTabOrder",
        "slotDateChanged",
        "slotDataChanged",
        "slotScheduleNew",
        "eMyMoney::Schedule::Occurrence",
        "occurrence",
        "slotStatusProgressDone",
        "slotCheckSchedules",
        "slotDeleteTransactions",
        "slotDuplicateTransactions",
        "slotDisplayTransactionDetails",
        "slotCopySplits",
        "slotMarkTransactions",
        "slotMoveTransactionTo",
        "slotTransactionOpenURL",
        "openUrl",
        "slotMoveToToday",
        "slotMatchTransaction",
        "slotAcceptTransaction",
        "slotCreateScheduledTransaction",
        "slotStartReconciliation",
        "slotReportReconciliation",
        "slotEnterOverdueSchedules",
        "slotFindTransaction",
        "slotCloseAccount",
        "slotReopenAccount",
        "slotExecuteAction",
        "slotExecuteActionWithData",
        "createFactoryObject",
        "parent",
        "objectName",
        "slotFileInfoDialog",
        "slotFileConsistencyCheck",
        "slotPriceDialog",
        "slotCurrencyDialog",
        "slotShowTransactionDetail",
        "slotHideReconciledTransactions",
        "slotHideUnusedCategories",
        "slotShowAllAccounts",
        "slotStatusMsg",
        "text",
        "slotStatusProgressBar",
        "current",
        "total",
        "slotEquityPriceUpdate",
        "slotInvestmentNew",
        "MyMoneyAccount&",
        "account",
        "MyMoneyAccount",
        "slotCategoryNew",
        "slotPayeeNew",
        "newnameBase",
        "QString&",
        "slotToolsStartKCalc",
        "slotAccountNew",
        "webConnect",
        "sourceUrl",
        "asn_id",
        "webConnectUrl",
        "slotFileNew",
        "slotFileOpen",
        "slotFileOpenRecent",
        "slotFileSave",
        "slotFileSaveAs",
        "slotFileClose",
        "slotCloseViewOrFile",
        "slotFileQuit"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fileLoaded'
        QtMocHelpers::SignalData<void(const QUrl &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'transactionSelected'
        QtMocHelpers::SignalData<void(const QString, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'selectAllTransactions'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'institutionSelected'
        QtMocHelpers::SignalData<void(const MyMoneyInstitution &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'scheduleSelected'
        QtMocHelpers::SignalData<void(const MyMoneySchedule &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'startMatchTransaction'
        QtMocHelpers::SignalData<void(const MyMoneyTransaction &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'cancelMatchTransaction'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'kmmFilePlugin'
        QtMocHelpers::SignalData<void(unsigned int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 2 },
        }}),
        // Slot 'slotInstallConsistencyCheckContextMenu'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowContextMenuForConsistencyCheck'
        QtMocHelpers::SlotData<void(const QPoint &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 2 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const SelectedObjects &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'slotAddSharedAction'
        QtMocHelpers::SlotData<void(eMenu::Action, QAction *)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 27, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Slot 'slotToggleMenuBar'
        QtMocHelpers::SlotData<void(bool)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 32 },
        }}),
        // Slot 'pauseAutoSave'
        QtMocHelpers::SlotData<void(void *, bool)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::VoidStar, 34 }, { QMetaType::Bool, 35 },
        }}),
        // Slot 'slotAutoSave'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEnableMessages'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotGetOnlineHelp'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotWhatsNew'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotVisitWebsite'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotPerformanceTest'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotGenerateSql'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotToggleTimers'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotFileViewPersonal'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotLoadAccountTemplates'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSaveAccountTemplates'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSettings'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotShowCredits'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotBackupFile'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotBackupMount'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotBackupWriteFile'
        QtMocHelpers::SlotData<bool()>(51, 2, QMC::AccessProtected, QMetaType::Bool),
        // Slot 'slotBackupUnmount'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotBackupFinish'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotBackupHandleEvents'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotUpdateConfiguration'
        QtMocHelpers::SlotData<void(const QString &)>(55, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 56 },
        }}),
        // Slot 'slotNewFeature'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEditTabOrder'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDateChanged'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDataChanged'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotScheduleNew'
        QtMocHelpers::SlotData<void(const MyMoneyTransaction &, eMyMoney::Schedule::Occurrence)>(61, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { 0x80000000 | 62, 63 },
        }}),
        // Slot 'slotScheduleNew'
        QtMocHelpers::SlotData<void(const MyMoneyTransaction &)>(61, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotStatusProgressDone'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCheckSchedules'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDeleteTransactions'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDuplicateTransactions'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDisplayTransactionDetails'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCopySplits'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotMarkTransactions'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotMoveTransactionTo'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotTransactionOpenURL'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'openUrl'
        QtMocHelpers::SlotData<bool(const QUrl &)>(73, 2, QMC::AccessProtected, QMetaType::Bool, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotMoveToToday'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotMatchTransaction'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAcceptTransaction'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCreateScheduledTransaction'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotStartReconciliation'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotReportReconciliation'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEnterOverdueSchedules'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotFindTransaction'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCloseAccount'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotReopenAccount'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotExecuteAction'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotExecuteActionWithData'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'createFactoryObject'
        QtMocHelpers::SlotData<QObject *(QObject *, const QString &)>(86, 2, QMC::AccessPublic, QMetaType::QObjectStar, {{
            { QMetaType::QObjectStar, 87 }, { QMetaType::QString, 88 },
        }}),
        // Slot 'slotFileInfoDialog'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileConsistencyCheck'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPriceDialog'
        QtMocHelpers::SlotData<void()>(91, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCurrencyDialog'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowTransactionDetail'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHideReconciledTransactions'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHideUnusedCategories'
        QtMocHelpers::SlotData<void()>(95, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowAllAccounts'
        QtMocHelpers::SlotData<void()>(96, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotStatusMsg'
        QtMocHelpers::SlotData<QString(const QString &)>(97, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 98 },
        }}),
        // Slot 'slotStatusProgressBar'
        QtMocHelpers::SlotData<void(int, int)>(99, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 100 }, { QMetaType::Int, 101 },
        }}),
        // Slot 'slotStatusProgressBar'
        QtMocHelpers::SlotData<void(int)>(99, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 100 },
        }}),
        // Slot 'slotEquityPriceUpdate'
        QtMocHelpers::SlotData<void()>(102, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotInvestmentNew'
        QtMocHelpers::SlotData<void(MyMoneyAccount &, const MyMoneyAccount &)>(103, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 104, 105 }, { 0x80000000 | 106, 87 },
        }}),
        // Slot 'slotCategoryNew'
        QtMocHelpers::SlotData<void(MyMoneyAccount &, const MyMoneyAccount &)>(107, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 104, 105 }, { 0x80000000 | 106, 87 },
        }}),
        // Slot 'slotCategoryNew'
        QtMocHelpers::SlotData<void(MyMoneyAccount &)>(107, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 104, 105 },
        }}),
        // Slot 'slotPayeeNew'
        QtMocHelpers::SlotData<void(const QString &, QString &)>(108, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 109 }, { 0x80000000 | 110, 34 },
        }}),
        // Slot 'slotToolsStartKCalc'
        QtMocHelpers::SlotData<void()>(111, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAccountNew'
        QtMocHelpers::SlotData<void(MyMoneyAccount &)>(112, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 104, 2 },
        }}),
        // Slot 'webConnect'
        QtMocHelpers::SlotData<void(const QString &, const QByteArray &)>(113, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 114 }, { QMetaType::QByteArray, 115 },
        }}),
        // Slot 'webConnectUrl'
        QtMocHelpers::SlotData<void(const QUrl)>(116, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotFileNew'
        QtMocHelpers::SlotData<bool()>(117, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'slotFileOpen'
        QtMocHelpers::SlotData<void()>(118, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileOpenRecent'
        QtMocHelpers::SlotData<bool(const QUrl &)>(119, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotFileSave'
        QtMocHelpers::SlotData<bool()>(120, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'slotFileSaveAs'
        QtMocHelpers::SlotData<bool()>(121, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'slotFileClose'
        QtMocHelpers::SlotData<bool()>(122, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'slotCloseViewOrFile'
        QtMocHelpers::SlotData<bool()>(123, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'slotFileQuit'
        QtMocHelpers::SlotData<void()>(124, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KMyMoneyApp, qt_meta_tag_ZN11KMyMoneyAppE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KMyMoneyApp::staticMetaObject = { {
    QMetaObject::SuperData::link<KXmlGuiWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KMyMoneyAppE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KMyMoneyAppE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11KMyMoneyAppE_t>.metaTypes,
    nullptr
} };

void KMyMoneyApp::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KMyMoneyApp *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fileLoaded((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 1: _t->transactionSelected((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->selectAllTransactions(); break;
        case 3: _t->institutionSelected((*reinterpret_cast<std::add_pointer_t<MyMoneyInstitution>>(_a[1]))); break;
        case 4: _t->scheduleSelected((*reinterpret_cast<std::add_pointer_t<MyMoneySchedule>>(_a[1]))); break;
        case 5: _t->startMatchTransaction((*reinterpret_cast<std::add_pointer_t<MyMoneyTransaction>>(_a[1]))); break;
        case 6: _t->cancelMatchTransaction(); break;
        case 7: _t->kmmFilePlugin((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 8: _t->slotInstallConsistencyCheckContextMenu(); break;
        case 9: _t->slotShowContextMenuForConsistencyCheck((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 10: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 11: _t->slotAddSharedAction((*reinterpret_cast<std::add_pointer_t<eMenu::Action>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[2]))); break;
        case 12: _t->slotToggleMenuBar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->pauseAutoSave((*reinterpret_cast<std::add_pointer_t<void*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 14: _t->slotAutoSave(); break;
        case 15: _t->slotEnableMessages(); break;
        case 16: _t->slotGetOnlineHelp(); break;
        case 17: _t->slotWhatsNew(); break;
        case 18: _t->slotVisitWebsite(); break;
        case 19: _t->slotPerformanceTest(); break;
        case 20: _t->slotGenerateSql(); break;
        case 21: _t->slotToggleTimers(); break;
        case 22: _t->slotFileViewPersonal(); break;
        case 23: _t->slotLoadAccountTemplates(); break;
        case 24: _t->slotSaveAccountTemplates(); break;
        case 25: _t->slotSettings(); break;
        case 26: _t->slotShowCredits(); break;
        case 27: _t->slotBackupFile(); break;
        case 28: _t->slotBackupMount(); break;
        case 29: { bool _r = _t->slotBackupWriteFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 30: _t->slotBackupUnmount(); break;
        case 31: _t->slotBackupFinish(); break;
        case 32: _t->slotBackupHandleEvents(); break;
        case 33: _t->slotUpdateConfiguration((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 34: _t->slotNewFeature(); break;
        case 35: _t->slotEditTabOrder(); break;
        case 36: _t->slotDateChanged(); break;
        case 37: _t->slotDataChanged(); break;
        case 38: _t->slotScheduleNew((*reinterpret_cast<std::add_pointer_t<MyMoneyTransaction>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<eMyMoney::Schedule::Occurrence>>(_a[2]))); break;
        case 39: _t->slotScheduleNew((*reinterpret_cast<std::add_pointer_t<MyMoneyTransaction>>(_a[1]))); break;
        case 40: _t->slotStatusProgressDone(); break;
        case 41: _t->slotCheckSchedules(); break;
        case 42: _t->slotDeleteTransactions(); break;
        case 43: _t->slotDuplicateTransactions(); break;
        case 44: _t->slotDisplayTransactionDetails(); break;
        case 45: _t->slotCopySplits(); break;
        case 46: _t->slotMarkTransactions(); break;
        case 47: _t->slotMoveTransactionTo(); break;
        case 48: _t->slotTransactionOpenURL(); break;
        case 49: { bool _r = _t->openUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 50: _t->slotMoveToToday(); break;
        case 51: _t->slotMatchTransaction(); break;
        case 52: _t->slotAcceptTransaction(); break;
        case 53: _t->slotCreateScheduledTransaction(); break;
        case 54: _t->slotStartReconciliation(); break;
        case 55: _t->slotReportReconciliation(); break;
        case 56: _t->slotEnterOverdueSchedules(); break;
        case 57: _t->slotFindTransaction(); break;
        case 58: _t->slotCloseAccount(); break;
        case 59: _t->slotReopenAccount(); break;
        case 60: _t->slotExecuteAction(); break;
        case 61: _t->slotExecuteActionWithData(); break;
        case 62: { QObject* _r = _t->createFactoryObject((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 63: _t->slotFileInfoDialog(); break;
        case 64: _t->slotFileConsistencyCheck(); break;
        case 65: _t->slotPriceDialog(); break;
        case 66: _t->slotCurrencyDialog(); break;
        case 67: _t->slotShowTransactionDetail(); break;
        case 68: _t->slotHideReconciledTransactions(); break;
        case 69: _t->slotHideUnusedCategories(); break;
        case 70: _t->slotShowAllAccounts(); break;
        case 71: { QString _r = _t->slotStatusMsg((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 72: _t->slotStatusProgressBar((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 73: _t->slotStatusProgressBar((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 74: _t->slotEquityPriceUpdate(); break;
        case 75: _t->slotInvestmentNew((*reinterpret_cast<std::add_pointer_t<MyMoneyAccount&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MyMoneyAccount>>(_a[2]))); break;
        case 76: _t->slotCategoryNew((*reinterpret_cast<std::add_pointer_t<MyMoneyAccount&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MyMoneyAccount>>(_a[2]))); break;
        case 77: _t->slotCategoryNew((*reinterpret_cast<std::add_pointer_t<MyMoneyAccount&>>(_a[1]))); break;
        case 78: _t->slotPayeeNew((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[2]))); break;
        case 79: _t->slotToolsStartKCalc(); break;
        case 80: _t->slotAccountNew((*reinterpret_cast<std::add_pointer_t<MyMoneyAccount&>>(_a[1]))); break;
        case 81: _t->webConnect((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 82: _t->webConnectUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 83: { bool _r = _t->slotFileNew();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 84: _t->slotFileOpen(); break;
        case 85: { bool _r = _t->slotFileOpenRecent((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 86: { bool _r = _t->slotFileSave();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 87: { bool _r = _t->slotFileSaveAs();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 88: { bool _r = _t->slotFileClose();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 89: { bool _r = _t->slotCloseViewOrFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 90: _t->slotFileQuit(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MyMoneyTransaction >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MyMoneyTransaction >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< eMyMoney::Schedule::Occurrence >(); break;
            }
            break;
        case 39:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MyMoneyTransaction >(); break;
            }
            break;
        case 75:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MyMoneyAccount >(); break;
            }
            break;
        case 76:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MyMoneyAccount >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyApp::*)(const QUrl & )>(_a, &KMyMoneyApp::fileLoaded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyApp::*)(const QString , const QString & )>(_a, &KMyMoneyApp::transactionSelected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyApp::*)()>(_a, &KMyMoneyApp::selectAllTransactions, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyApp::*)(const MyMoneyInstitution & )>(_a, &KMyMoneyApp::institutionSelected, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyApp::*)(const MyMoneySchedule & )>(_a, &KMyMoneyApp::scheduleSelected, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyApp::*)(const MyMoneyTransaction & )>(_a, &KMyMoneyApp::startMatchTransaction, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyApp::*)()>(_a, &KMyMoneyApp::cancelMatchTransaction, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyApp::*)(unsigned int )>(_a, &KMyMoneyApp::kmmFilePlugin, 7))
            return;
    }
}

const QMetaObject *KMyMoneyApp::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KMyMoneyApp::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KMyMoneyAppE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "IMyMoneyProcessingCalendar"))
        return static_cast< IMyMoneyProcessingCalendar*>(this);
    if (!strcmp(_clname, "MyMoneyFactory"))
        return static_cast< MyMoneyFactory*>(this);
    return KXmlGuiWindow::qt_metacast(_clname);
}

int KMyMoneyApp::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KXmlGuiWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 91)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 91;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 91)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 91;
    }
    return _id;
}

// SIGNAL 0
void KMyMoneyApp::fileLoaded(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KMyMoneyApp::transactionSelected(const QString _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KMyMoneyApp::selectAllTransactions()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KMyMoneyApp::institutionSelected(const MyMoneyInstitution & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KMyMoneyApp::scheduleSelected(const MyMoneySchedule & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KMyMoneyApp::startMatchTransaction(const MyMoneyTransaction & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KMyMoneyApp::cancelMatchTransaction()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KMyMoneyApp::kmmFilePlugin(unsigned int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
