/********************************************************************************
** Form generated from reading UI file 'kloanpaymentpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KLOANPAYMENTPAGE_H
#define UI_KLOANPAYMENTPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KLoanPaymentPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel1;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacer10;
    QGridLayout *gridLayout;
    QLabel *textLabel1;
    QLabel *m_basePayment;
    QLabel *textLabel3;
    QPushButton *m_additionalFeesButton;
    QLabel *m_additionalFees;
    QLabel *textLabel5;
    QLabel *m_totalPayment;
    QSpacerItem *spacer15;
    QSpacerItem *spacer12;

    void setupUi(QWidget *KLoanPaymentPage)
    {
        if (KLoanPaymentPage->objectName().isEmpty())
            KLoanPaymentPage->setObjectName("KLoanPaymentPage");
        KLoanPaymentPage->resize(525, 373);
        vboxLayout = new QVBoxLayout(KLoanPaymentPage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        qLabel1 = new QLabel(KLoanPaymentPage);
        qLabel1->setObjectName("qLabel1");
        qLabel1->setFocusPolicy(Qt::NoFocus);
        qLabel1->setWordWrap(true);

        vboxLayout->addWidget(qLabel1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        spacer10 = new QSpacerItem(63, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer10);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        textLabel1 = new QLabel(KLoanPaymentPage);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 2);

        m_basePayment = new QLabel(KLoanPaymentPage);
        m_basePayment->setObjectName("m_basePayment");
        QFont font;
        font.setBold(true);
        m_basePayment->setFont(font);
        m_basePayment->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        m_basePayment->setWordWrap(false);

        gridLayout->addWidget(m_basePayment, 0, 2, 1, 1);

        textLabel3 = new QLabel(KLoanPaymentPage);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout->addWidget(textLabel3, 1, 0, 1, 1);

        m_additionalFeesButton = new QPushButton(KLoanPaymentPage);
        m_additionalFeesButton->setObjectName("m_additionalFeesButton");

        gridLayout->addWidget(m_additionalFeesButton, 1, 1, 1, 1);

        m_additionalFees = new QLabel(KLoanPaymentPage);
        m_additionalFees->setObjectName("m_additionalFees");
        m_additionalFees->setFont(font);
        m_additionalFees->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        m_additionalFees->setWordWrap(false);

        gridLayout->addWidget(m_additionalFees, 1, 2, 1, 1);

        textLabel5 = new QLabel(KLoanPaymentPage);
        textLabel5->setObjectName("textLabel5");
        textLabel5->setWordWrap(false);

        gridLayout->addWidget(textLabel5, 2, 0, 1, 2);

        m_totalPayment = new QLabel(KLoanPaymentPage);
        m_totalPayment->setObjectName("m_totalPayment");
        m_totalPayment->setFont(font);
        m_totalPayment->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        m_totalPayment->setWordWrap(false);

        gridLayout->addWidget(m_totalPayment, 2, 2, 1, 1);


        hboxLayout->addLayout(gridLayout);

        spacer15 = new QSpacerItem(62, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer15);


        vboxLayout->addLayout(hboxLayout);

        spacer12 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer12);


        retranslateUi(KLoanPaymentPage);

        QMetaObject::connectSlotsByName(KLoanPaymentPage);
    } // setupUi

    void retranslateUi(QWidget *KLoanPaymentPage)
    {
        qLabel1->setText(tr2i18n("Now enter the information required for the periodic loan payments. If your regular payments contains any additional fees, click on the <b>Additional fees</b> button to enter them.", nullptr));
        textLabel1->setText(tr2i18n("Principal + Interest", nullptr));
        m_basePayment->setText(tr2i18n("xxx", nullptr));
        textLabel3->setText(tr2i18n("+", nullptr));
        m_additionalFeesButton->setText(tr2i18n("Additional Fees", nullptr));
        m_additionalFees->setText(tr2i18n("xxx", nullptr));
        textLabel5->setText(tr2i18n("= periodical payments", nullptr));
        m_totalPayment->setText(tr2i18n("xxx", nullptr));
        (void)KLoanPaymentPage;
    } // retranslateUi

};

namespace Ui {
    class KLoanPaymentPage: public Ui_KLoanPaymentPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KLOANPAYMENTPAGE_H

