/********************************************************************************
** Form generated from reading UI file 'kinvestmenttypewizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KINVESTMENTTYPEWIZARDPAGE_H
#define UI_KINVESTMENTTYPEWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <kinvestmenttypewizardpage.h>
#include <klocalizedstring.h>
#include "kcombobox.h"

QT_BEGIN_NAMESPACE

class Ui_KInvestmentTypeWizardPage
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *m_introLabel;
    QSpacerItem *spacer8;
    QLabel *textLabel1;
    QSpacerItem *spacer3;
    QGridLayout *gridLayout;
    QSpacerItem *spacer1;
    KComboBox *m_securityType;
    QLabel *textLabel2;
    QLabel *label;
    QLineEdit *m_accountName;
    QSpacerItem *spacer2;

    void setupUi(KInvestmentTypeWizardPage *KInvestmentTypeWizardPage)
    {
        if (KInvestmentTypeWizardPage->objectName().isEmpty())
            KInvestmentTypeWizardPage->setObjectName("KInvestmentTypeWizardPage");
        KInvestmentTypeWizardPage->resize(357, 275);
        verticalLayout = new QVBoxLayout(KInvestmentTypeWizardPage);
        verticalLayout->setObjectName("verticalLayout");
        m_introLabel = new QLabel(KInvestmentTypeWizardPage);
        m_introLabel->setObjectName("m_introLabel");
        m_introLabel->setAlignment(Qt::AlignTop);
        m_introLabel->setWordWrap(true);

        verticalLayout->addWidget(m_introLabel);

        spacer8 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout->addItem(spacer8);

        textLabel1 = new QLabel(KInvestmentTypeWizardPage);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setAlignment(Qt::AlignTop);
        textLabel1->setWordWrap(true);

        verticalLayout->addWidget(textLabel1);

        spacer3 = new QSpacerItem(20, 30, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacer3);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        spacer1 = new QSpacerItem(61, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacer1, 1, 2, 1, 1);

        m_securityType = new KComboBox(KInvestmentTypeWizardPage);
        m_securityType->setObjectName("m_securityType");

        gridLayout->addWidget(m_securityType, 1, 1, 1, 1);

        textLabel2 = new QLabel(KInvestmentTypeWizardPage);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 1, 0, 1, 1);

        label = new QLabel(KInvestmentTypeWizardPage);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        m_accountName = new QLineEdit(KInvestmentTypeWizardPage);
        m_accountName->setObjectName("m_accountName");

        gridLayout->addWidget(m_accountName, 0, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        spacer2 = new QSpacerItem(20, 60, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacer2);


        retranslateUi(KInvestmentTypeWizardPage);

        QMetaObject::connectSlotsByName(KInvestmentTypeWizardPage);
    } // setupUi

    void retranslateUi(KInvestmentTypeWizardPage *KInvestmentTypeWizardPage)
    {
        KInvestmentTypeWizardPage->setTitle(tr2i18n("Investment Type", nullptr));
        m_introLabel->setText(tr2i18n("This wizard allows you to create a new investment.", nullptr));
        textLabel1->setText(tr2i18n("The first step in this process requires to provide an account name for the investment and select the type of investment. The following steps collect further details about the investment from you.", nullptr));
        textLabel2->setText(tr2i18n("Type of investment", nullptr));
        label->setText(tr2i18n("Account name", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KInvestmentTypeWizardPage: public Ui_KInvestmentTypeWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KINVESTMENTTYPEWIZARDPAGE_H

