/********************************************************************************
** Form generated from reading UI file 'kbaccountsettings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KBACCOUNTSETTINGS_H
#define UI_KBACCOUNTSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "keditlistwidget.h"
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KBAccountSettings
{
public:
    QVBoxLayout *verticalLayout_3;
    QTabWidget *tabWidget2;
    QWidget *payeeTab;
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QRadioButton *m_usePayeeAsIsButton;
    QRadioButton *m_extractPayeeButton;
    QFrame *m_memoFrame;
    QGridLayout *gridLayout;
    QLabel *textLabel1;
    QLabel *textLabel2;
    KLineEdit *m_memoRegExpEdit;
    KLineEdit *m_payeeRegExpEdit;
    KEditListWidget *m_payeeExceptions;
    QLabel *textLabel3;
    QWidget *downloadTab;
    QVBoxLayout *vboxLayout;
    QCheckBox *m_transactionDownload;
    QGridLayout *gridLayout1;
    KComboBox *m_preferredStatementDate;
    QLabel *textLabel2_2;
    QSpacerItem *spacer2;
    QWidget *memoTab;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *m_removeLineBreaksFromMemo;
    QCheckBox *m_includePayeeDetails;
    QSpacerItem *verticalSpacer;
    QSpacerItem *spacer1;

    void setupUi(QWidget *KBAccountSettings)
    {
        if (KBAccountSettings->objectName().isEmpty())
            KBAccountSettings->setObjectName("KBAccountSettings");
        KBAccountSettings->resize(561, 499);
        verticalLayout_3 = new QVBoxLayout(KBAccountSettings);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        tabWidget2 = new QTabWidget(KBAccountSettings);
        tabWidget2->setObjectName("tabWidget2");
        payeeTab = new QWidget();
        payeeTab->setObjectName("payeeTab");
        verticalLayout_2 = new QVBoxLayout(payeeTab);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox = new QGroupBox(payeeTab);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        m_usePayeeAsIsButton = new QRadioButton(groupBox);
        m_usePayeeAsIsButton->setObjectName("m_usePayeeAsIsButton");
        m_usePayeeAsIsButton->setChecked(false);

        verticalLayout->addWidget(m_usePayeeAsIsButton);

        m_extractPayeeButton = new QRadioButton(groupBox);
        m_extractPayeeButton->setObjectName("m_extractPayeeButton");
        m_extractPayeeButton->setChecked(true);

        verticalLayout->addWidget(m_extractPayeeButton);


        verticalLayout_2->addWidget(groupBox);

        m_memoFrame = new QFrame(payeeTab);
        m_memoFrame->setObjectName("m_memoFrame");
        m_memoFrame->setEnabled(true);
        m_memoFrame->setFrameShape(QFrame::NoFrame);
        m_memoFrame->setFrameShadow(QFrame::Sunken);
        gridLayout = new QGridLayout(m_memoFrame);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        textLabel1 = new QLabel(m_memoFrame);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 1, 0, 1, 1);

        textLabel2 = new QLabel(m_memoFrame);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 2, 0, 1, 1);

        m_memoRegExpEdit = new KLineEdit(m_memoFrame);
        m_memoRegExpEdit->setObjectName("m_memoRegExpEdit");

        gridLayout->addWidget(m_memoRegExpEdit, 2, 1, 1, 1);

        m_payeeRegExpEdit = new KLineEdit(m_memoFrame);
        m_payeeRegExpEdit->setObjectName("m_payeeRegExpEdit");

        gridLayout->addWidget(m_payeeRegExpEdit, 1, 1, 1, 1);

        m_payeeExceptions = new KEditListWidget(m_memoFrame);
        m_payeeExceptions->setObjectName("m_payeeExceptions");
        m_payeeExceptions->setEnabled(true);
        m_payeeExceptions->setButtons(KEditListWidget::All);

        gridLayout->addWidget(m_payeeExceptions, 4, 1, 1, 1);

        textLabel3 = new QLabel(m_memoFrame);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout->addWidget(textLabel3, 0, 1, 1, 1);


        verticalLayout_2->addWidget(m_memoFrame);

        tabWidget2->addTab(payeeTab, QString());
        downloadTab = new QWidget();
        downloadTab->setObjectName("downloadTab");
        vboxLayout = new QVBoxLayout(downloadTab);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        m_transactionDownload = new QCheckBox(downloadTab);
        m_transactionDownload->setObjectName("m_transactionDownload");
        m_transactionDownload->setChecked(false);

        vboxLayout->addWidget(m_transactionDownload);

        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setObjectName("gridLayout1");
        m_preferredStatementDate = new KComboBox(downloadTab);
        m_preferredStatementDate->addItem(QString());
        m_preferredStatementDate->addItem(QString());
        m_preferredStatementDate->addItem(QString());
        m_preferredStatementDate->addItem(QString());
        m_preferredStatementDate->setObjectName("m_preferredStatementDate");

        gridLayout1->addWidget(m_preferredStatementDate, 0, 1, 1, 1);

        textLabel2_2 = new QLabel(downloadTab);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setWordWrap(true);

        gridLayout1->addWidget(textLabel2_2, 0, 0, 1, 1);


        vboxLayout->addLayout(gridLayout1);

        spacer2 = new QSpacerItem(20, 160, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer2);

        tabWidget2->addTab(downloadTab, QString());
        memoTab = new QWidget();
        memoTab->setObjectName("memoTab");
        verticalLayout_4 = new QVBoxLayout(memoTab);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName("verticalLayout_4");
        m_removeLineBreaksFromMemo = new QCheckBox(memoTab);
        m_removeLineBreaksFromMemo->setObjectName("m_removeLineBreaksFromMemo");

        verticalLayout_4->addWidget(m_removeLineBreaksFromMemo);

        m_includePayeeDetails = new QCheckBox(memoTab);
        m_includePayeeDetails->setObjectName("m_includePayeeDetails");

        verticalLayout_4->addWidget(m_includePayeeDetails);

        verticalSpacer = new QSpacerItem(20, 384, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);

        tabWidget2->addTab(memoTab, QString());

        verticalLayout_3->addWidget(tabWidget2);

        spacer1 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(spacer1);

        QWidget::setTabOrder(m_payeeRegExpEdit, m_memoRegExpEdit);

        retranslateUi(KBAccountSettings);
        QObject::connect(m_extractPayeeButton, &QRadioButton::toggled, m_memoFrame, &QFrame::setEnabled);

        tabWidget2->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KBAccountSettings);
    } // setupUi

    void retranslateUi(QWidget *KBAccountSettings)
    {
        groupBox->setTitle(QString());
        m_usePayeeAsIsButton->setText(tr2i18n("Use payee field as provided by institution", nullptr));
        m_extractPayeeButton->setText(tr2i18n("Extract payee from memo field using regular expressions", nullptr));
        textLabel1->setText(tr2i18n("Payee", nullptr));
        textLabel2->setText(tr2i18n("Memo", nullptr));
        textLabel3->setText(tr2i18n("Regular Expression for", nullptr));
        tabWidget2->setTabText(tabWidget2->indexOf(payeeTab), tr2i18n("Payee", nullptr));
#if QT_CONFIG(whatsthis)
        m_transactionDownload->setWhatsThis(tr2i18n("The plugin usually downloads the account balance and transactions. In some circumstances, the download of transaction data is not supported for certain accounts. The following option allows to turn off the request for transaction data.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_transactionDownload->setText(tr2i18n("Download transactions from institution", nullptr));
        m_preferredStatementDate->setItemText(0, tr2i18n("Ask user", nullptr));
        m_preferredStatementDate->setItemText(1, tr2i18n("No date", nullptr));
        m_preferredStatementDate->setItemText(2, tr2i18n("Last download", nullptr));
        m_preferredStatementDate->setItemText(3, tr2i18n("First possible", nullptr));

        textLabel2_2->setText(tr2i18n("Default start date for statement download", nullptr));
        tabWidget2->setTabText(tabWidget2->indexOf(downloadTab), tr2i18n("Download", nullptr));
#if QT_CONFIG(tooltip)
        m_removeLineBreaksFromMemo->setToolTip(tr2i18n("<html><head/><body><p>Remove the line breaks in the memo field that have been inserted between multiple purpose fields of the transaction.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_removeLineBreaksFromMemo->setWhatsThis(tr2i18n("<html><head/><body><p>Some institutions take multiple purpose fields of a transaction and treat it as a single one. In this case the line break characters inserted after each purpose field are distracting and are removed when this option is enabled.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_removeLineBreaksFromMemo->setText(tr2i18n("Remove line breaks from memo field", nullptr));
#if QT_CONFIG(tooltip)
        m_includePayeeDetails->setToolTip(tr2i18n("<html><head/><body><p>Add additional payee details found in the transaction to the memo.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_includePayeeDetails->setText(tr2i18n("Include the payee details in memo", nullptr));
        tabWidget2->setTabText(tabWidget2->indexOf(memoTab), tr2i18n("Memo", nullptr));
        (void)KBAccountSettings;
    } // retranslateUi

};

namespace Ui {
    class KBAccountSettings: public Ui_KBAccountSettings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KBACCOUNTSETTINGS_H

