/*
    SPDX-FileCopyrightText: 2002 Thomas Baumgart <tbaumgart@kde.org>
    SPDX-FileCopyrightText: 2002 Kevin Tambascio <ktambascio@users.sourceforge.net>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "imymoneystorageformat.h"

IMyMoneyOperationsFormat::IMyMoneyOperationsFormat()
{
}

IMyMoneyOperationsFormat::~IMyMoneyOperationsFormat()
{
}
