/********************************************************************************
** Form generated from reading UI file 'kaccounttypepage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KACCOUNTTYPEPAGE_H
#define UI_KACCOUNTTYPEPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "amountedit.h"
#include "kcombobox.h"
#include "kdatecombobox.h"
#include "klineedit.h"
#include "kmymoneycurrencyselector.h"
#include "kmymoneydateedit.h"
#include "kmymoneygeneralcombo.h"

QT_BEGIN_NAMESPACE

class Ui_KAccountTypePage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel4;
    QGridLayout *gridLayout;
    QLabel *textLabel1;
    KLineEdit *m_accountName;
    QLabel *textLabel2_2;
    KMyMoneyGeneralCombo *m_typeSelection;
    QLabel *textLabel3;
    KMyMoneyCurrencySelector *m_currencyComboBox;
    QSpacerItem *spacer6;
    QLabel *textLabel2;
    KMyMoneyDateEdit *m_openingDate;
    QLabel *m_openingBalanceLabel;
    AmountEdit *m_openingBalance;
    QLabel *m_conversionLabel;
    AmountEdit *m_conversionRate;
    QPushButton *m_onlineQuote;
    QSpacerItem *spacer3;
    QLabel *m_conversionExample;
    QCheckBox *m_preferredAccount;
    QSpacerItem *spacer18;

    void setupUi(QWidget *KAccountTypePage)
    {
        if (KAccountTypePage->objectName().isEmpty())
            KAccountTypePage->setObjectName("KAccountTypePage");
        KAccountTypePage->resize(555, 404);
        vboxLayout = new QVBoxLayout(KAccountTypePage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(10, 10, 10, 10);
        qLabel4 = new QLabel(KAccountTypePage);
        qLabel4->setObjectName("qLabel4");
        qLabel4->setFocusPolicy(Qt::NoFocus);
        qLabel4->setWordWrap(true);

        vboxLayout->addWidget(qLabel4);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        textLabel1 = new QLabel(KAccountTypePage);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        m_accountName = new KLineEdit(KAccountTypePage);
        m_accountName->setObjectName("m_accountName");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_accountName->sizePolicy().hasHeightForWidth());
        m_accountName->setSizePolicy(sizePolicy);

        gridLayout->addWidget(m_accountName, 0, 1, 1, 1);

        textLabel2_2 = new QLabel(KAccountTypePage);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setWordWrap(false);

        gridLayout->addWidget(textLabel2_2, 1, 0, 1, 1);

        m_typeSelection = new KMyMoneyGeneralCombo(KAccountTypePage);
        m_typeSelection->setObjectName("m_typeSelection");
        sizePolicy.setHeightForWidth(m_typeSelection->sizePolicy().hasHeightForWidth());
        m_typeSelection->setSizePolicy(sizePolicy);

        gridLayout->addWidget(m_typeSelection, 1, 1, 1, 1);

        textLabel3 = new QLabel(KAccountTypePage);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout->addWidget(textLabel3, 2, 0, 1, 1);

        m_currencyComboBox = new KMyMoneyCurrencySelector(KAccountTypePage);
        m_currencyComboBox->setObjectName("m_currencyComboBox");
        sizePolicy.setHeightForWidth(m_currencyComboBox->sizePolicy().hasHeightForWidth());
        m_currencyComboBox->setSizePolicy(sizePolicy);

        gridLayout->addWidget(m_currencyComboBox, 2, 1, 1, 1);

        spacer6 = new QSpacerItem(231, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacer6, 2, 2, 1, 2);

        textLabel2 = new QLabel(KAccountTypePage);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 3, 0, 1, 1);

        m_openingDate = new KMyMoneyDateEdit(KAccountTypePage);
        m_openingDate->setObjectName("m_openingDate");
        m_openingDate->setFocusPolicy(Qt::StrongFocus);

        gridLayout->addWidget(m_openingDate, 3, 1, 1, 1);

        m_openingBalanceLabel = new QLabel(KAccountTypePage);
        m_openingBalanceLabel->setObjectName("m_openingBalanceLabel");
        m_openingBalanceLabel->setWordWrap(false);

        gridLayout->addWidget(m_openingBalanceLabel, 4, 0, 1, 1);

        m_openingBalance = new AmountEdit(KAccountTypePage);
        m_openingBalance->setObjectName("m_openingBalance");
        m_openingBalance->setMinimumSize(QSize(150, 0));
        m_openingBalance->setFocusPolicy(Qt::StrongFocus);
        m_openingBalance->setProperty("resetButtonVisibility", QVariant(false));

        gridLayout->addWidget(m_openingBalance, 4, 1, 1, 1);

        m_conversionLabel = new QLabel(KAccountTypePage);
        m_conversionLabel->setObjectName("m_conversionLabel");
        m_conversionLabel->setWordWrap(false);

        gridLayout->addWidget(m_conversionLabel, 5, 0, 1, 1);

        m_conversionRate = new AmountEdit(KAccountTypePage);
        m_conversionRate->setObjectName("m_conversionRate");
        m_conversionRate->setFocusPolicy(Qt::StrongFocus);
        m_conversionRate->setProperty("resetButtonVisibility", QVariant(false));

        gridLayout->addWidget(m_conversionRate, 5, 1, 1, 1);

        m_onlineQuote = new QPushButton(KAccountTypePage);
        m_onlineQuote->setObjectName("m_onlineQuote");

        gridLayout->addWidget(m_onlineQuote, 5, 2, 1, 1);

        spacer3 = new QSpacerItem(61, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacer3, 5, 3, 1, 1);

        m_conversionExample = new QLabel(KAccountTypePage);
        m_conversionExample->setObjectName("m_conversionExample");
        m_conversionExample->setWordWrap(false);

        gridLayout->addWidget(m_conversionExample, 6, 1, 1, 1);

        m_preferredAccount = new QCheckBox(KAccountTypePage);
        m_preferredAccount->setObjectName("m_preferredAccount");

        gridLayout->addWidget(m_preferredAccount, 7, 0, 1, 2);


        vboxLayout->addLayout(gridLayout);

        spacer18 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer18);


        retranslateUi(KAccountTypePage);

        QMetaObject::connectSlotsByName(KAccountTypePage);
    } // setupUi

    void retranslateUi(QWidget *KAccountTypePage)
    {
        qLabel4->setText(tr2i18n("Enter the name of the account under which it is known within KMyMoney. Select the type for the new account. Use the <b>What's this?</b> feature to see more details about the various account types. Enter the date the account was opened and its currency.", nullptr));
        textLabel1->setText(tr2i18n("Account name", nullptr));
#if QT_CONFIG(whatsthis)
        m_accountName->setWhatsThis(tr2i18n("Enter a name under which this account is known within KMyMoney.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel2_2->setText(tr2i18n("Account type", nullptr));
#if QT_CONFIG(whatsthis)
        m_typeSelection->setWhatsThis(tr2i18n("<p><b>Checking</b>\n"
"Use the checking account type to manage activities on your checking account e.g. payments, checks and cash card purchases.</p>\n"
"\n"
"<p><b>Savings</b>\n"
"Use the savings account type to manage activities on your savings account.</p>\n"
"\n"
"<p><b>Credit Card</b>\n"
"Use the credit card account type to manage activities on your credit card.</p>\n"
"\n"
"<p><b>Cash</b>\n"
"\n"
"Use the cash account type to manage activities in your wallet.</p>\n"
"\n"
"<p><b>Loan</b>\n"
"Use the loan account type to manage amortization loans (e.g. mortgages, car loan, money you lend, private loans etc.).</p>\n"
"\n"
"<p><b>Investment</b>\n"
"Use the investment account to manage your stock, mutual fund and other investments.</p>\n"
"\n"
"<p><b>Asset</b>\n"
"Use the asset account type to manage assets (e.g. your house, car or art collection).</p>\n"
"\n"
"<p><b>Liability</b>\n"
"Use the liability account type to manage any type of liability except amortization loans. Use it for taxes you owe or money yo"
                        "u borrowed from friends. For amortization loans like mortgages you should create a loan account.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel3->setText(tr2i18n("Currency", nullptr));
#if QT_CONFIG(whatsthis)
        m_currencyComboBox->setWhatsThis(tr2i18n("Select the currency in which this account is denominated.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel2->setText(tr2i18n("Opening date", nullptr));
#if QT_CONFIG(whatsthis)
        m_openingDate->setWhatsThis(tr2i18n("<p>The opening date would be the date of the last statement. In case you want to keep track of transactions prior to this date, enter the ending balance and statement date of a prior statement and add all following transactions into KMyMoney. KMyMoney will not allow you to enter or modify transactions prior to this date.</p>\n"
"<p><i>Note</i>: If you do not know the exact amount of the opening balance, enter an estimate. You can change this value later before you reconcile this account for the first time.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_openingBalanceLabel->setText(tr2i18n("Opening balance", nullptr));
#if QT_CONFIG(whatsthis)
        m_openingBalance->setWhatsThis(tr2i18n("<p>The opening balance is the balance of the account when you start using it with KMyMoney. For new accounts this is usually 0 but for existing accounts this may well differ. Please consult the account statements to figure out this value. The opening balance is to be provided in the currency of the account as selected with the currency button.</p>\n"
"<p><i>Note</i>: If you do not know the exact amount of the opening balance, enter an estimate. You can change this value later before you reconcile this account for the first time.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_conversionLabel->setText(tr2i18n("Conversion rate", nullptr));
#if QT_CONFIG(whatsthis)
        m_conversionRate->setWhatsThis(tr2i18n("For foreign currencies an initial conversion rate needs to be provided. This should be the price of the foreign currency in the base currency on the opening date of the account.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_onlineQuote->setText(tr2i18n("Online quote", nullptr));
        m_conversionExample->setText(tr2i18n("xxx", nullptr));
#if QT_CONFIG(tooltip)
        m_preferredAccount->setToolTip(tr2i18n("Mark this checkbox to include the account in the group of favorite accounts", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_preferredAccount->setWhatsThis(tr2i18n("Selecting the <b>Favorite Account</b> checkbox will allow preferred access to this account in some dialogs and views of KMyMoney.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_preferredAccount->setText(tr2i18n("Favorite account", nullptr));
        (void)KAccountTypePage;
    } // retranslateUi

};

namespace Ui {
    class KAccountTypePage: public Ui_KAccountTypePage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KACCOUNTTYPEPAGE_H

