/********************************************************************************
** Form generated from reading UI file 'paymentfrequencywizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAYMENTFREQUENCYWIZARDPAGE_H
#define UI_PAYMENTFREQUENCYWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "kmymoneyfrequencycombo.h"

QT_BEGIN_NAMESPACE

class Ui_PaymentFrequencyWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel11_2;
    QSpacerItem *Spacer54;
    QHBoxLayout *hboxLayout;
    QSpacerItem *Spacer57;
    KMyMoneyFrequencyCombo *m_paymentFrequencyUnitEdit;
    QSpacerItem *Spacer56;
    QSpacerItem *Spacer55;

    void setupUi(QWizardPage *PaymentFrequencyWizardPage)
    {
        if (PaymentFrequencyWizardPage->objectName().isEmpty())
            PaymentFrequencyWizardPage->setObjectName("PaymentFrequencyWizardPage");
        vboxLayout = new QVBoxLayout(PaymentFrequencyWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel11_2 = new QLabel(PaymentFrequencyWizardPage);
        TextLabel11_2->setObjectName("TextLabel11_2");
        TextLabel11_2->setAlignment(Qt::AlignTop);
        TextLabel11_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel11_2);

        Spacer54 = new QSpacerItem(0, 49, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer54);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        Spacer57 = new QSpacerItem(39, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer57);

        m_paymentFrequencyUnitEdit = new KMyMoneyFrequencyCombo(PaymentFrequencyWizardPage);
        m_paymentFrequencyUnitEdit->setObjectName("m_paymentFrequencyUnitEdit");

        hboxLayout->addWidget(m_paymentFrequencyUnitEdit);

        Spacer56 = new QSpacerItem(56, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer56);


        vboxLayout->addLayout(hboxLayout);

        Spacer55 = new QSpacerItem(0, 49, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer55);


        retranslateUi(PaymentFrequencyWizardPage);

        QMetaObject::connectSlotsByName(PaymentFrequencyWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *PaymentFrequencyWizardPage)
    {
        TextLabel11_2->setText(tr2i18n("\n"
"How often will there be payments made to this loan?", nullptr));
        (void)PaymentFrequencyWizardPage;
    } // retranslateUi

};

namespace Ui {
    class PaymentFrequencyWizardPage: public Ui_PaymentFrequencyWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PAYMENTFREQUENCYWIZARDPAGE_H

