/********************************************************************************
** Form generated from reading UI file 'checkprintingsettingsdecl.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CHECKPRINTINGSETTINGSDECL_H
#define UI_CHECKPRINTINGSETTINGSDECL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "klineedit.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_CheckPrintingSettingsDecl
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QCheckBox *kcfg_useCustomCheckTemplate;
    KUrlRequester *kcfg_checkTemplateFile;
    QFrame *m_previewFrame;

    void setupUi(QWidget *CheckPrintingSettingsDecl)
    {
        if (CheckPrintingSettingsDecl->objectName().isEmpty())
            CheckPrintingSettingsDecl->setObjectName("CheckPrintingSettingsDecl");
        CheckPrintingSettingsDecl->resize(810, 442);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(CheckPrintingSettingsDecl->sizePolicy().hasHeightForWidth());
        CheckPrintingSettingsDecl->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(CheckPrintingSettingsDecl);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        kcfg_useCustomCheckTemplate = new QCheckBox(CheckPrintingSettingsDecl);
        kcfg_useCustomCheckTemplate->setObjectName("kcfg_useCustomCheckTemplate");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(kcfg_useCustomCheckTemplate->sizePolicy().hasHeightForWidth());
        kcfg_useCustomCheckTemplate->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(kcfg_useCustomCheckTemplate);

        kcfg_checkTemplateFile = new KUrlRequester(CheckPrintingSettingsDecl);
        kcfg_checkTemplateFile->setObjectName("kcfg_checkTemplateFile");
        kcfg_checkTemplateFile->setProperty("kcfg_property", QVariant(QByteArray("text")));
        kcfg_checkTemplateFile->setEnabled(false);
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(kcfg_checkTemplateFile->sizePolicy().hasHeightForWidth());
        kcfg_checkTemplateFile->setSizePolicy(sizePolicy2);

        horizontalLayout->addWidget(kcfg_checkTemplateFile);


        verticalLayout->addLayout(horizontalLayout);

        m_previewFrame = new QFrame(CheckPrintingSettingsDecl);
        m_previewFrame->setObjectName("m_previewFrame");
        m_previewFrame->setMinimumSize(QSize(800, 400));
        m_previewFrame->setFrameShape(QFrame::StyledPanel);
        m_previewFrame->setFrameShadow(QFrame::Raised);

        verticalLayout->addWidget(m_previewFrame);


        verticalLayout_2->addLayout(verticalLayout);


        retranslateUi(CheckPrintingSettingsDecl);

        QMetaObject::connectSlotsByName(CheckPrintingSettingsDecl);
    } // setupUi

    void retranslateUi(QWidget *CheckPrintingSettingsDecl)
    {
        CheckPrintingSettingsDecl->setWindowTitle(tr2i18n("Checking settings", nullptr));
        kcfg_useCustomCheckTemplate->setText(tr2i18n("Use custom check template", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CheckPrintingSettingsDecl: public Ui_CheckPrintingSettingsDecl {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CHECKPRINTINGSETTINGSDECL_H

