/********************************************************************************
** Form generated from reading UI file 'ksearchtransactiondlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KSEARCHTRANSACTIONDLG_H
#define UI_KSEARCHTRANSACTIONDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "ledgerview.h"

QT_BEGIN_NAMESPACE

class Ui_KSearchTransactionDlg
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *m_tabWidget;
    QWidget *m_resultPage;
    QVBoxLayout *vboxLayout;
    LedgerView *m_ledgerView;
    QLabel *m_foundText;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KSearchTransactionDlg)
    {
        if (KSearchTransactionDlg->objectName().isEmpty())
            KSearchTransactionDlg->setObjectName("KSearchTransactionDlg");
        KSearchTransactionDlg->resize(771, 481);
        KSearchTransactionDlg->setSizeGripEnabled(true);
        verticalLayout = new QVBoxLayout(KSearchTransactionDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        m_tabWidget = new QTabWidget(KSearchTransactionDlg);
        m_tabWidget->setObjectName("m_tabWidget");
        m_resultPage = new QWidget();
        m_resultPage->setObjectName("m_resultPage");
        vboxLayout = new QVBoxLayout(m_resultPage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        m_ledgerView = new LedgerView(m_resultPage);
        m_ledgerView->setObjectName("m_ledgerView");

        vboxLayout->addWidget(m_ledgerView);

        m_foundText = new QLabel(m_resultPage);
        m_foundText->setObjectName("m_foundText");
        m_foundText->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        m_foundText->setWordWrap(false);

        vboxLayout->addWidget(m_foundText);

        m_tabWidget->addTab(m_resultPage, QString());

        verticalLayout->addWidget(m_tabWidget);

        buttonBox = new QDialogButtonBox(KSearchTransactionDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Close|QDialogButtonBox::Help|QDialogButtonBox::Reset);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(KSearchTransactionDlg);

        QMetaObject::connectSlotsByName(KSearchTransactionDlg);
    } // setupUi

    void retranslateUi(QDialog *KSearchTransactionDlg)
    {
        KSearchTransactionDlg->setWindowTitle(tr2i18n("Search transactions", nullptr));
        m_foundText->setText(tr2i18n("F", nullptr));
        m_tabWidget->setTabText(m_tabWidget->indexOf(m_resultPage), tr2i18n("Result", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KSearchTransactionDlg: public Ui_KSearchTransactionDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KSEARCHTRANSACTIONDLG_H

