/********************************************************************************
** Form generated from reading UI file 'checkingstartwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CHECKINGSTARTWIZARDPAGE_H
#define UI_CHECKINGSTARTWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_CheckingStartWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel6;
    QSpacerItem *verticalSpacer;
    QCheckBox *m_skipIntroPage;

    void setupUi(QWizardPage *CheckingStartWizardPage)
    {
        if (CheckingStartWizardPage->objectName().isEmpty())
            CheckingStartWizardPage->setObjectName("CheckingStartWizardPage");
        CheckingStartWizardPage->resize(577, 388);
        vboxLayout = new QVBoxLayout(CheckingStartWizardPage);
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        TextLabel6 = new QLabel(CheckingStartWizardPage);
        TextLabel6->setObjectName("TextLabel6");
        TextLabel6->setAlignment(Qt::AlignTop);
        TextLabel6->setWordWrap(true);

        vboxLayout->addWidget(TextLabel6);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(verticalSpacer);

        m_skipIntroPage = new QCheckBox(CheckingStartWizardPage);
        m_skipIntroPage->setObjectName("m_skipIntroPage");

        vboxLayout->addWidget(m_skipIntroPage);


        retranslateUi(CheckingStartWizardPage);

        QMetaObject::connectSlotsByName(CheckingStartWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *CheckingStartWizardPage)
    {
        CheckingStartWizardPage->setTitle(tr2i18n("General Information", nullptr));
        TextLabel6->setText(tr2i18n("Reconciling your account ensures your records match those of your institution, and that there are no mistakes on either side. You should reconcile your account whenever you receive a statement from your institution.\n"
"\n"
"All relevant information necessary for this process is usually printed on your statement.\n"
"\n"
"On the next page you will verify that the ending balance matches that on your statement. If not, please modify it.\n"
"\n"
"The starting balance is the sum of your cleared and reconciled transactions.", nullptr));
        m_skipIntroPage->setText(tr2i18n("Don't show this page again when starting the reconciliation next time.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CheckingStartWizardPage: public Ui_CheckingStartWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CHECKINGSTARTWIZARDPAGE_H

