/*
    SPDX-FileCopyrightText: 2013 Christian Dávid <christian-david@web.de>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "ionlinejobedit.h"

IonlineJobEdit::IonlineJobEdit(QWidget* parent, QVariantList args)
    : QWidget(parent)
{
    Q_UNUSED(args);
}

void IonlineJobEdit::showAllErrorMessages(const bool)
{
}
