/********************************************************************************
** Form generated from reading UI file 'kcreditcardschedulepage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KCREDITCARDSCHEDULEPAGE_H
#define UI_KCREDITCARDSCHEDULEPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klineedit.h>
#include <klocalizedstring.h>
#include <kmymoneycategory.h>
#include "amountedit.h"
#include "kmymoneydateedit.h"
#include "kmymoneygeneralcombo.h"
#include "kmymoneypayeecombo.h"

QT_BEGIN_NAMESPACE

class Ui_KCreditCardSchedulePage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel16;
    QCheckBox *m_reminderCheckBox;
    QFrame *m_paymentFrame;
    QGridLayout *gridLayout;
    QLabel *textLabel2;
    KLineEdit *m_name;
    QLabel *textLabel3;
    KMyMoneyPayeeCombo *m_payee;
    QLabel *TextLabel2_2;
    AmountEdit *m_amount;
    QLabel *TextLabel4_3;
    KMyMoneyDateEdit *m_date;
    QLabel *TextLabel3_3;
    KMyMoneyCategory *m_paymentAccount;
    QLabel *textLabel4;
    KMyMoneyGeneralCombo *m_method;
    QSpacerItem *spacer23;

    void setupUi(QWidget *KCreditCardSchedulePage)
    {
        if (KCreditCardSchedulePage->objectName().isEmpty())
            KCreditCardSchedulePage->setObjectName("KCreditCardSchedulePage");
        KCreditCardSchedulePage->resize(544, 391);
        vboxLayout = new QVBoxLayout(KCreditCardSchedulePage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        qLabel16 = new QLabel(KCreditCardSchedulePage);
        qLabel16->setObjectName("qLabel16");
        qLabel16->setWordWrap(true);

        vboxLayout->addWidget(qLabel16);

        m_reminderCheckBox = new QCheckBox(KCreditCardSchedulePage);
        m_reminderCheckBox->setObjectName("m_reminderCheckBox");
        m_reminderCheckBox->setFocusPolicy(Qt::StrongFocus);

        vboxLayout->addWidget(m_reminderCheckBox);

        m_paymentFrame = new QFrame(KCreditCardSchedulePage);
        m_paymentFrame->setObjectName("m_paymentFrame");
        m_paymentFrame->setFrameShape(QFrame::NoFrame);
        m_paymentFrame->setFrameShadow(QFrame::Plain);
        gridLayout = new QGridLayout(m_paymentFrame);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName("gridLayout");
        textLabel2 = new QLabel(m_paymentFrame);
        textLabel2->setObjectName("textLabel2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textLabel2->sizePolicy().hasHeightForWidth());
        textLabel2->setSizePolicy(sizePolicy);
        textLabel2->setMinimumSize(QSize(180, 0));
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 0, 0, 1, 1);

        m_name = new KLineEdit(m_paymentFrame);
        m_name->setObjectName("m_name");

        gridLayout->addWidget(m_name, 0, 1, 1, 1);

        textLabel3 = new QLabel(m_paymentFrame);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout->addWidget(textLabel3, 1, 0, 1, 1);

        m_payee = new KMyMoneyPayeeCombo(m_paymentFrame);
        m_payee->setObjectName("m_payee");

        gridLayout->addWidget(m_payee, 1, 1, 1, 1);

        TextLabel2_2 = new QLabel(m_paymentFrame);
        TextLabel2_2->setObjectName("TextLabel2_2");
        TextLabel2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel2_2, 2, 0, 1, 1);

        m_amount = new AmountEdit(m_paymentFrame);
        m_amount->setObjectName("m_amount");
        m_amount->setFocusPolicy(Qt::StrongFocus);

        gridLayout->addWidget(m_amount, 2, 1, 1, 1);

        TextLabel4_3 = new QLabel(m_paymentFrame);
        TextLabel4_3->setObjectName("TextLabel4_3");
        TextLabel4_3->setWordWrap(false);

        gridLayout->addWidget(TextLabel4_3, 3, 0, 1, 1);

        m_date = new KMyMoneyDateEdit(m_paymentFrame);
        m_date->setObjectName("m_date");
        m_date->setFocusPolicy(Qt::StrongFocus);

        gridLayout->addWidget(m_date, 3, 1, 1, 1);

        TextLabel3_3 = new QLabel(m_paymentFrame);
        TextLabel3_3->setObjectName("TextLabel3_3");
        TextLabel3_3->setAlignment(Qt::AlignTop);
        TextLabel3_3->setWordWrap(false);

        gridLayout->addWidget(TextLabel3_3, 4, 0, 1, 1);

        m_paymentAccount = new KMyMoneyCategory(m_paymentFrame);
        m_paymentAccount->setObjectName("m_paymentAccount");

        gridLayout->addWidget(m_paymentAccount, 4, 1, 1, 1);

        textLabel4 = new QLabel(m_paymentFrame);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 5, 0, 1, 1);

        m_method = new KMyMoneyGeneralCombo(m_paymentFrame);
        m_method->setObjectName("m_method");

        gridLayout->addWidget(m_method, 5, 1, 1, 1);


        vboxLayout->addWidget(m_paymentFrame);

        spacer23 = new QSpacerItem(20, 31, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer23);


        retranslateUi(KCreditCardSchedulePage);
        QObject::connect(m_reminderCheckBox, &QCheckBox::toggled, m_paymentFrame, &QFrame::setEnabled);

        QMetaObject::connectSlotsByName(KCreditCardSchedulePage);
    } // setupUi

    void retranslateUi(QWidget *KCreditCardSchedulePage)
    {
        qLabel16->setText(tr2i18n("For credit card accounts you can create a monthly schedule which will automatically remind you about the payment. Please fill in the details about this payment here.", nullptr));
        m_reminderCheckBox->setText(tr2i18n("Yes, remind me when the payment is due", nullptr));
        textLabel2->setText(tr2i18n("Name of schedule", nullptr));
        textLabel3->setText(tr2i18n("Payee", nullptr));
        TextLabel2_2->setText(tr2i18n("Estimated monthly payment", nullptr));
        TextLabel4_3->setText(tr2i18n("Due date of next payment", nullptr));
        TextLabel3_3->setText(tr2i18n("Payment should be made\n"
"from account", nullptr));
        textLabel4->setText(tr2i18n("Payment method", nullptr));
        (void)KCreditCardSchedulePage;
    } // retranslateUi

};

namespace Ui {
    class KCreditCardSchedulePage: public Ui_KCreditCardSchedulePage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KCREDITCARDSCHEDULEPAGE_H

