/********************************************************************************
** Form generated from reading UI file 'reporttabcapitalgain.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REPORTTABCAPITALGAIN_H
#define UI_REPORTTABCAPITALGAIN_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_ReportTabCapitalGain
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *lblSettlementPeriod;
    QLabel *lblTermsSeparator;
    QCheckBox *m_showSTLTCapitalGains;
    KMyMoneyDateEdit *m_termSeparator;
    QLabel *label;
    QLabel *lblinvestmentSum;
    QSpinBox *m_settlementPeriod;
    QComboBox *m_investmentSum;
    QLabel *label_2;
    QSpacerItem *horizontalSpacer;
    QCheckBox *m_checkHideTotals;
    QSpacerItem *spacer15;

    void setupUi(QWidget *ReportTabCapitalGain)
    {
        if (ReportTabCapitalGain->objectName().isEmpty())
            ReportTabCapitalGain->setObjectName("ReportTabCapitalGain");
        ReportTabCapitalGain->resize(441, 166);
        verticalLayout = new QVBoxLayout(ReportTabCapitalGain);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        lblSettlementPeriod = new QLabel(ReportTabCapitalGain);
        lblSettlementPeriod->setObjectName("lblSettlementPeriod");

        gridLayout->addWidget(lblSettlementPeriod, 1, 0, 1, 1);

        lblTermsSeparator = new QLabel(ReportTabCapitalGain);
        lblTermsSeparator->setObjectName("lblTermsSeparator");

        gridLayout->addWidget(lblTermsSeparator, 3, 0, 1, 1);

        m_showSTLTCapitalGains = new QCheckBox(ReportTabCapitalGain);
        m_showSTLTCapitalGains->setObjectName("m_showSTLTCapitalGains");

        gridLayout->addWidget(m_showSTLTCapitalGains, 2, 1, 1, 1, Qt::AlignRight);

        m_termSeparator = new KMyMoneyDateEdit(ReportTabCapitalGain);
        m_termSeparator->setObjectName("m_termSeparator");

        gridLayout->addWidget(m_termSeparator, 3, 1, 1, 1);

        label = new QLabel(ReportTabCapitalGain);
        label->setObjectName("label");

        gridLayout->addWidget(label, 2, 0, 1, 1);

        lblinvestmentSum = new QLabel(ReportTabCapitalGain);
        lblinvestmentSum->setObjectName("lblinvestmentSum");

        gridLayout->addWidget(lblinvestmentSum, 0, 0, 1, 1);

        m_settlementPeriod = new QSpinBox(ReportTabCapitalGain);
        m_settlementPeriod->setObjectName("m_settlementPeriod");
        m_settlementPeriod->setMaximum(30);
        m_settlementPeriod->setValue(3);

        gridLayout->addWidget(m_settlementPeriod, 1, 1, 1, 1);

        m_investmentSum = new QComboBox(ReportTabCapitalGain);
        m_investmentSum->setObjectName("m_investmentSum");

        gridLayout->addWidget(m_investmentSum, 0, 1, 1, 1);

        label_2 = new QLabel(ReportTabCapitalGain);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 4, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 2, 5, 1);

        m_checkHideTotals = new QCheckBox(ReportTabCapitalGain);
        m_checkHideTotals->setObjectName("m_checkHideTotals");

        gridLayout->addWidget(m_checkHideTotals, 4, 1, 1, 1, Qt::AlignRight);


        verticalLayout->addLayout(gridLayout);

        spacer15 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacer15);


        retranslateUi(ReportTabCapitalGain);

        QMetaObject::connectSlotsByName(ReportTabCapitalGain);
    } // setupUi

    void retranslateUi(QWidget *ReportTabCapitalGain)
    {
        ReportTabCapitalGain->setWindowTitle(tr2i18n("Report Tab", nullptr));
#if QT_CONFIG(tooltip)
        ReportTabCapitalGain->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        ReportTabCapitalGain->setWhatsThis(tr2i18n("<p>On this tab, you set the basic properties of this report.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        lblSettlementPeriod->setText(tr2i18n("Settlement period", nullptr));
        lblTermsSeparator->setText(tr2i18n("Terms separator", nullptr));
#if QT_CONFIG(tooltip)
        m_termSeparator->setToolTip(tr2i18n("Before this date investments are counted as long-term investments.", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(tr2i18n("Show short-term and long-term capital gains", nullptr));
        lblinvestmentSum->setText(tr2i18n("Sum investments", nullptr));
#if QT_CONFIG(tooltip)
        m_settlementPeriod->setToolTip(tr2i18n("Time in days between the settlement date and the transaction date.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(tr2i18n("Hide Totals", nullptr));
        m_checkHideTotals->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class ReportTabCapitalGain: public Ui_ReportTabCapitalGain {};
} // namespace Ui

QT_END_NAMESPACE

#endif // REPORTTABCAPITALGAIN_H

