/********************************************************************************
** Form generated from reading UI file 'formatswizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FORMATSWIZARDPAGE_H
#define UI_FORMATSWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_FormatsPage
{
public:
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *leftSpacer;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QSpacerItem *verticalSpacer_2;
    QFrame *line;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QGridLayout *gridLayout;
    QComboBox *m_thousandsDelimiter;
    QLabel *labelSet_decimal;
    QComboBox *m_decimalSymbol;
    QLabel *labelSet_thousands;
    QLabel *labelSet_dateFormat;
    QComboBox *m_dateFormat;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer_low;
    QSpacerItem *rightSpacer;

    void setupUi(QWidget *FormatsPage)
    {
        if (FormatsPage->objectName().isEmpty())
            FormatsPage->setObjectName("FormatsPage");
        FormatsPage->resize(477, 239);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(FormatsPage->sizePolicy().hasHeightForWidth());
        FormatsPage->setSizePolicy(sizePolicy);
        horizontalLayout_2 = new QHBoxLayout(FormatsPage);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        leftSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(leftSpacer);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(FormatsPage);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignCenter);
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        verticalSpacer_2 = new QSpacerItem(371, 3, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);

        line = new QFrame(FormatsPage);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        verticalSpacer_3 = new QSpacerItem(371, 3, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_3);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(13, 13, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        m_thousandsDelimiter = new QComboBox(FormatsPage);
        m_thousandsDelimiter->addItem(QString());
        m_thousandsDelimiter->addItem(QString());
        m_thousandsDelimiter->addItem(QString());
        m_thousandsDelimiter->setObjectName("m_thousandsDelimiter");
        m_thousandsDelimiter->setEnabled(false);
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_thousandsDelimiter->sizePolicy().hasHeightForWidth());
        m_thousandsDelimiter->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(m_thousandsDelimiter, 1, 1, 1, 1);

        labelSet_decimal = new QLabel(FormatsPage);
        labelSet_decimal->setObjectName("labelSet_decimal");
        labelSet_decimal->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelSet_decimal, 0, 0, 1, 1);

        m_decimalSymbol = new QComboBox(FormatsPage);
        m_decimalSymbol->addItem(QString());
        m_decimalSymbol->addItem(QString());
        m_decimalSymbol->addItem(QString());
        m_decimalSymbol->setObjectName("m_decimalSymbol");
        sizePolicy1.setHeightForWidth(m_decimalSymbol->sizePolicy().hasHeightForWidth());
        m_decimalSymbol->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(m_decimalSymbol, 0, 1, 1, 1);

        labelSet_thousands = new QLabel(FormatsPage);
        labelSet_thousands->setObjectName("labelSet_thousands");
        labelSet_thousands->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelSet_thousands, 1, 0, 1, 1);

        labelSet_dateFormat = new QLabel(FormatsPage);
        labelSet_dateFormat->setObjectName("labelSet_dateFormat");
        labelSet_dateFormat->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelSet_dateFormat, 2, 0, 1, 1);

        m_dateFormat = new QComboBox(FormatsPage);
        m_dateFormat->addItem(QString());
        m_dateFormat->addItem(QString());
        m_dateFormat->addItem(QString());
        m_dateFormat->setObjectName("m_dateFormat");
        sizePolicy1.setHeightForWidth(m_dateFormat->sizePolicy().hasHeightForWidth());
        m_dateFormat->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(m_dateFormat, 2, 1, 1, 1);

        gridLayout->setColumnStretch(0, 10);
        gridLayout->setColumnStretch(1, 10);

        horizontalLayout->addLayout(gridLayout);

        horizontalSpacer_2 = new QSpacerItem(13, 13, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        horizontalLayout->setStretch(0, 20);
        horizontalLayout->setStretch(1, 50);
        horizontalLayout->setStretch(2, 35);

        verticalLayout->addLayout(horizontalLayout);

        verticalSpacer_low = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_low);

        verticalLayout->setStretch(0, 30);
        verticalLayout->setStretch(1, 1);
        verticalLayout->setStretch(2, 1);
        verticalLayout->setStretch(3, 1);
        verticalLayout->setStretch(4, 1);
        verticalLayout->setStretch(5, 2);

        horizontalLayout_2->addLayout(verticalLayout);

        rightSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(rightSpacer);

        horizontalLayout_2->setStretch(0, 1);
        horizontalLayout_2->setStretch(1, 10);
        horizontalLayout_2->setStretch(2, 1);
        QWidget::setTabOrder(m_decimalSymbol, m_thousandsDelimiter);
        QWidget::setTabOrder(m_thousandsDelimiter, m_dateFormat);

        retranslateUi(FormatsPage);

        m_thousandsDelimiter->setCurrentIndex(-1);
        m_decimalSymbol->setCurrentIndex(-1);
        m_dateFormat->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(FormatsPage);
    } // setupUi

    void retranslateUi(QWidget *FormatsPage)
    {
        label->setText(tr2i18n("Please select the decimal symbol used in your file.\n"
"The thousands separator changes automatically.\n"
"If all is OK, you will be able to click 'Import'.\n"
"If required, a QIF file may then be produced.", nullptr));
        m_thousandsDelimiter->setItemText(0, tr2i18n("comma (,)", nullptr));
        m_thousandsDelimiter->setItemText(1, tr2i18n("dot (.)", nullptr));
        m_thousandsDelimiter->setItemText(2, tr2i18n("auto", nullptr));

        labelSet_decimal->setText(tr2i18n("Decimal Symbol", nullptr));
        m_decimalSymbol->setItemText(0, tr2i18n("dot (.)", nullptr));
        m_decimalSymbol->setItemText(1, tr2i18n("comma (,)", nullptr));
        m_decimalSymbol->setItemText(2, tr2i18n("auto", nullptr));

#if QT_CONFIG(tooltip)
        m_decimalSymbol->setToolTip(tr2i18n("Click to select the required decimal symbol, and to\n"
"check that the selected symbol matches the data.", nullptr));
#endif // QT_CONFIG(tooltip)
        labelSet_thousands->setText(tr2i18n("Thousands Symbol", nullptr));
        labelSet_dateFormat->setText(tr2i18n("Date format", "date format to use"));
        m_dateFormat->setItemText(0, tr2i18n("y m d", "abbreviation of 'year', 'month', 'day'."));
        m_dateFormat->setItemText(1, tr2i18n("m d y", "abbreviation of 'month', 'day', 'year',"));
        m_dateFormat->setItemText(2, tr2i18n("d m y", "abbreviation of  'day', 'month', 'year',"));

        (void)FormatsPage;
    } // retranslateUi

};

namespace Ui {
    class FormatsPage: public Ui_FormatsPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // FORMATSWIZARDPAGE_H

