/****************************************************************************
** Meta object code from reading C++ file 'kmymoneyaccounttreeview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/widgets/kmymoneyaccounttreeview.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kmymoneyaccounttreeview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN23KMyMoneyAccountTreeViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KMyMoneyAccountTreeView::qt_create_metaobjectdata<qt_meta_tag_ZN23KMyMoneyAccountTreeViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KMyMoneyAccountTreeView",
        "requestSelectionChange",
        "",
        "SelectedObjects",
        "selections",
        "requestCustomContextMenu",
        "eMenu::Menu",
        "contextMenu",
        "QPoint",
        "pos",
        "requestActionTrigger",
        "eMenu::Action",
        "action",
        "customContextMenuRequested",
        "selectionChanged",
        "QItemSelection",
        "selected",
        "deselected"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestSelectionChange'
        QtMocHelpers::SignalData<void(const SelectedObjects &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'requestCustomContextMenu'
        QtMocHelpers::SignalData<void(eMenu::Menu, const QPoint &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'requestActionTrigger'
        QtMocHelpers::SignalData<void(eMenu::Action)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'customContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Slot 'selectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 15, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KMyMoneyAccountTreeView, qt_meta_tag_ZN23KMyMoneyAccountTreeViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KMyMoneyAccountTreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<KMyMoneyTreeView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KMyMoneyAccountTreeViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KMyMoneyAccountTreeViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23KMyMoneyAccountTreeViewE_t>.metaTypes,
    nullptr
} };

void KMyMoneyAccountTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KMyMoneyAccountTreeView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestSelectionChange((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 1: _t->requestCustomContextMenu((*reinterpret_cast<std::add_pointer_t<eMenu::Menu>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 2: _t->requestActionTrigger((*reinterpret_cast<std::add_pointer_t<eMenu::Action>>(_a[1]))); break;
        case 3: _t->customContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 4: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyAccountTreeView::*)(const SelectedObjects & )>(_a, &KMyMoneyAccountTreeView::requestSelectionChange, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyAccountTreeView::*)(eMenu::Menu , const QPoint & )>(_a, &KMyMoneyAccountTreeView::requestCustomContextMenu, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyAccountTreeView::*)(eMenu::Action )>(_a, &KMyMoneyAccountTreeView::requestActionTrigger, 2))
            return;
    }
}

const QMetaObject *KMyMoneyAccountTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KMyMoneyAccountTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KMyMoneyAccountTreeViewE_t>.strings))
        return static_cast<void*>(this);
    return KMyMoneyTreeView::qt_metacast(_clname);
}

int KMyMoneyAccountTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KMyMoneyTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void KMyMoneyAccountTreeView::requestSelectionChange(const SelectedObjects & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KMyMoneyAccountTreeView::requestCustomContextMenu(eMenu::Menu _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KMyMoneyAccountTreeView::requestActionTrigger(eMenu::Action _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
