/********************************************************************************
** Form generated from reading UI file 'recordpaymentwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RECORDPAYMENTWIZARDPAGE_H
#define UI_RECORDPAYMENTWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_RecordPaymentWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel7_2;
    QSpacerItem *Spacer33;
    QHBoxLayout *hboxLayout;
    QFrame *frame_5;
    QVBoxLayout *vboxLayout1;
    QRadioButton *m_allPaymentsButton;
    QRadioButton *m_thisYearPaymentButton;
    QSpacerItem *Spacer14_2_2_2;
    QSpacerItem *Spacer34;
    QButtonGroup *ButtonGroup4;

    void setupUi(QWizardPage *RecordPaymentWizardPage)
    {
        if (RecordPaymentWizardPage->objectName().isEmpty())
            RecordPaymentWizardPage->setObjectName("RecordPaymentWizardPage");
        vboxLayout = new QVBoxLayout(RecordPaymentWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel7_2 = new QLabel(RecordPaymentWizardPage);
        TextLabel7_2->setObjectName("TextLabel7_2");
        TextLabel7_2->setAlignment(Qt::AlignTop);
        TextLabel7_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel7_2);

        Spacer33 = new QSpacerItem(0, 69, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer33);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        frame_5 = new QFrame(RecordPaymentWizardPage);
        frame_5->setObjectName("frame_5");
        frame_5->setFrameShape(QFrame::NoFrame);
        vboxLayout1 = new QVBoxLayout(frame_5);
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        m_allPaymentsButton = new QRadioButton(frame_5);
        ButtonGroup4 = new QButtonGroup(RecordPaymentWizardPage);
        ButtonGroup4->setObjectName("ButtonGroup4");
        ButtonGroup4->addButton(m_allPaymentsButton);
        m_allPaymentsButton->setObjectName("m_allPaymentsButton");

        vboxLayout1->addWidget(m_allPaymentsButton);

        m_thisYearPaymentButton = new QRadioButton(frame_5);
        ButtonGroup4->addButton(m_thisYearPaymentButton);
        m_thisYearPaymentButton->setObjectName("m_thisYearPaymentButton");

        vboxLayout1->addWidget(m_thisYearPaymentButton);


        hboxLayout->addWidget(frame_5);

        Spacer14_2_2_2 = new QSpacerItem(137, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer14_2_2_2);


        vboxLayout->addLayout(hboxLayout);

        Spacer34 = new QSpacerItem(0, 69, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer34);


        retranslateUi(RecordPaymentWizardPage);

        QMetaObject::connectSlotsByName(RecordPaymentWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *RecordPaymentWizardPage)
    {
        TextLabel7_2->setText(tr2i18n("\n"
"Do you want to record all payments of this loan with KMyMoney?", nullptr));
        m_allPaymentsButton->setText(tr2i18n("Yes, record all payments.", nullptr));
        m_thisYearPaymentButton->setText(tr2i18n("No, only record payments since the beginning of this year.", nullptr));
        (void)RecordPaymentWizardPage;
    } // retranslateUi

};

namespace Ui {
    class RecordPaymentWizardPage: public Ui_RecordPaymentWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // RECORDPAYMENTWIZARDPAGE_H

