/********************************************************************************
** Form generated from reading UI file 'kmymoneyfileinfodlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KMYMONEYFILEINFODLG_H
#define UI_KMYMONEYFILEINFODLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KMyMoneyFileInfoDlg
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *textLabel3;
    QLabel *m_creationDate;
    QLabel *textLabel4;
    QLabel *m_lastModificationDate;
    QLabel *textLabel5;
    QLabel *m_baseCurrency;
    QLabel *textLabel10;
    QLabel *m_payeeCount;
    QLabel *textLabel6;
    QLabel *m_institutionCount;
    QLabel *textLabel7;
    QLabel *m_accountCount;
    QLabel *textLabel8;
    QLabel *m_transactionCount;
    QLabel *textLabel1;
    QLabel *m_splitCount;
    QLabel *textLabel9;
    QLabel *m_scheduleCount;
    QLabel *textLabel19;
    QLabel *m_priceCount;
    QTreeWidget *m_accountView;
    QFrame *line1;
    QHBoxLayout *horizontalLayout;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KMyMoneyFileInfoDlg)
    {
        if (KMyMoneyFileInfoDlg->objectName().isEmpty())
            KMyMoneyFileInfoDlg->setObjectName("KMyMoneyFileInfoDlg");
        KMyMoneyFileInfoDlg->resize(398, 640);
        KMyMoneyFileInfoDlg->setSizeGripEnabled(true);
        KMyMoneyFileInfoDlg->setModal(true);
        verticalLayout = new QVBoxLayout(KMyMoneyFileInfoDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        textLabel3 = new QLabel(KMyMoneyFileInfoDlg);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout->addWidget(textLabel3, 0, 0, 1, 1);

        m_creationDate = new QLabel(KMyMoneyFileInfoDlg);
        m_creationDate->setObjectName("m_creationDate");
        m_creationDate->setWordWrap(false);

        gridLayout->addWidget(m_creationDate, 0, 1, 1, 1);

        textLabel4 = new QLabel(KMyMoneyFileInfoDlg);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 1, 0, 1, 1);

        m_lastModificationDate = new QLabel(KMyMoneyFileInfoDlg);
        m_lastModificationDate->setObjectName("m_lastModificationDate");
        m_lastModificationDate->setWordWrap(false);

        gridLayout->addWidget(m_lastModificationDate, 1, 1, 1, 1);

        textLabel5 = new QLabel(KMyMoneyFileInfoDlg);
        textLabel5->setObjectName("textLabel5");
        textLabel5->setWordWrap(false);

        gridLayout->addWidget(textLabel5, 2, 0, 1, 1);

        m_baseCurrency = new QLabel(KMyMoneyFileInfoDlg);
        m_baseCurrency->setObjectName("m_baseCurrency");
        m_baseCurrency->setWordWrap(false);

        gridLayout->addWidget(m_baseCurrency, 2, 1, 1, 1);

        textLabel10 = new QLabel(KMyMoneyFileInfoDlg);
        textLabel10->setObjectName("textLabel10");
        textLabel10->setWordWrap(false);

        gridLayout->addWidget(textLabel10, 3, 0, 1, 1);

        m_payeeCount = new QLabel(KMyMoneyFileInfoDlg);
        m_payeeCount->setObjectName("m_payeeCount");
        m_payeeCount->setWordWrap(false);

        gridLayout->addWidget(m_payeeCount, 3, 1, 1, 1);

        textLabel6 = new QLabel(KMyMoneyFileInfoDlg);
        textLabel6->setObjectName("textLabel6");
        textLabel6->setWordWrap(false);

        gridLayout->addWidget(textLabel6, 4, 0, 1, 1);

        m_institutionCount = new QLabel(KMyMoneyFileInfoDlg);
        m_institutionCount->setObjectName("m_institutionCount");
        m_institutionCount->setWordWrap(false);

        gridLayout->addWidget(m_institutionCount, 4, 1, 1, 1);

        textLabel7 = new QLabel(KMyMoneyFileInfoDlg);
        textLabel7->setObjectName("textLabel7");
        textLabel7->setWordWrap(false);

        gridLayout->addWidget(textLabel7, 5, 0, 1, 1);

        m_accountCount = new QLabel(KMyMoneyFileInfoDlg);
        m_accountCount->setObjectName("m_accountCount");
        m_accountCount->setWordWrap(false);

        gridLayout->addWidget(m_accountCount, 5, 1, 1, 1);

        textLabel8 = new QLabel(KMyMoneyFileInfoDlg);
        textLabel8->setObjectName("textLabel8");
        textLabel8->setWordWrap(false);

        gridLayout->addWidget(textLabel8, 8, 0, 1, 1);

        m_transactionCount = new QLabel(KMyMoneyFileInfoDlg);
        m_transactionCount->setObjectName("m_transactionCount");
        m_transactionCount->setWordWrap(false);

        gridLayout->addWidget(m_transactionCount, 8, 1, 1, 1);

        textLabel1 = new QLabel(KMyMoneyFileInfoDlg);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 9, 0, 1, 1);

        m_splitCount = new QLabel(KMyMoneyFileInfoDlg);
        m_splitCount->setObjectName("m_splitCount");
        m_splitCount->setWordWrap(false);

        gridLayout->addWidget(m_splitCount, 9, 1, 1, 1);

        textLabel9 = new QLabel(KMyMoneyFileInfoDlg);
        textLabel9->setObjectName("textLabel9");
        textLabel9->setWordWrap(false);

        gridLayout->addWidget(textLabel9, 10, 0, 1, 1);

        m_scheduleCount = new QLabel(KMyMoneyFileInfoDlg);
        m_scheduleCount->setObjectName("m_scheduleCount");
        m_scheduleCount->setWordWrap(false);

        gridLayout->addWidget(m_scheduleCount, 10, 1, 1, 1);

        textLabel19 = new QLabel(KMyMoneyFileInfoDlg);
        textLabel19->setObjectName("textLabel19");
        textLabel19->setWordWrap(false);

        gridLayout->addWidget(textLabel19, 11, 0, 1, 1);

        m_priceCount = new QLabel(KMyMoneyFileInfoDlg);
        m_priceCount->setObjectName("m_priceCount");
        m_priceCount->setWordWrap(false);

        gridLayout->addWidget(m_priceCount, 11, 1, 1, 1);

        m_accountView = new QTreeWidget(KMyMoneyFileInfoDlg);
        m_accountView->setObjectName("m_accountView");
        m_accountView->setRootIsDecorated(false);

        gridLayout->addWidget(m_accountView, 6, 0, 1, 2);


        verticalLayout->addLayout(gridLayout);

        line1 = new QFrame(KMyMoneyFileInfoDlg);
        line1->setObjectName("line1");
        line1->setFrameShape(QFrame::HLine);
        line1->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        buttonBox = new QDialogButtonBox(KMyMoneyFileInfoDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        horizontalLayout->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(KMyMoneyFileInfoDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KMyMoneyFileInfoDlg, qOverload<>(&QDialog::reject));
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KMyMoneyFileInfoDlg, qOverload<>(&QDialog::accept));

        QMetaObject::connectSlotsByName(KMyMoneyFileInfoDlg);
    } // setupUi

    void retranslateUi(QDialog *KMyMoneyFileInfoDlg)
    {
        KMyMoneyFileInfoDlg->setWindowTitle(tr2i18n("File Information", nullptr));
        textLabel3->setText(tr2i18n("Created on", nullptr));
        m_creationDate->setText(QString());
        textLabel4->setText(tr2i18n("Last modified on", nullptr));
        m_lastModificationDate->setText(QString());
        textLabel5->setText(tr2i18n("Base currency", nullptr));
        m_baseCurrency->setText(QString());
        textLabel10->setText(tr2i18n("Payees", nullptr));
        m_payeeCount->setText(QString());
        textLabel6->setText(tr2i18n("Institutions", nullptr));
        m_institutionCount->setText(QString());
        textLabel7->setText(tr2i18n("Accounts/Categories", nullptr));
        m_accountCount->setText(QString());
        textLabel8->setText(tr2i18n("Transactions", nullptr));
        m_transactionCount->setText(QString());
        textLabel1->setText(tr2i18n("Splits", nullptr));
        m_splitCount->setText(QString());
        textLabel9->setText(tr2i18n("Schedules", nullptr));
        m_scheduleCount->setText(QString());
        textLabel19->setText(tr2i18n("Prices", nullptr));
        m_priceCount->setText(QString());
        QTreeWidgetItem *___qtreewidgetitem = m_accountView->headerItem();
        ___qtreewidgetitem->setText(2, tr2i18n("Closed", nullptr));
        ___qtreewidgetitem->setText(1, tr2i18n("Total", nullptr));
        ___qtreewidgetitem->setText(0, tr2i18n("Type", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KMyMoneyFileInfoDlg: public Ui_KMyMoneyFileInfoDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KMYMONEYFILEINFODLG_H

