/********************************************************************************
** Form generated from reading UI file 'userinfo.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_USERINFO_H
#define UI_USERINFO_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include <klineedit.h>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_UserInfo
{
public:
    QGridLayout *gridLayout;
    QLabel *qLabel1;
    KLineEdit *m_userNameEdit;
    QLabel *TextLabel3;
    QLabel *TextLabel4;
    QLabel *TextLabel2;
    KLineEdit *m_streetEdit;
    KLineEdit *m_emailEdit;
    KLineEdit *m_townEdit;
    KLineEdit *m_telephoneEdit;
    QLabel *TextLabel7;
    QLabel *TextLabel6;
    QLabel *TextLabel1;
    QHBoxLayout *hboxLayout;
    QPushButton *m_loadAddressButton;
    QSpacerItem *spacer11;
    QSpacerItem *spacer5;
    KLineEdit *m_countyEdit;
    QLabel *TextLabel5;
    KLineEdit *m_postcodeEdit;

    void setupUi(QWidget *UserInfo)
    {
        if (UserInfo->objectName().isEmpty())
            UserInfo->setObjectName("UserInfo");
        UserInfo->resize(603, 374);
        gridLayout = new QGridLayout(UserInfo);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        qLabel1 = new QLabel(UserInfo);
        qLabel1->setObjectName("qLabel1");
        qLabel1->setFocusPolicy(Qt::NoFocus);
        qLabel1->setWordWrap(true);

        gridLayout->addWidget(qLabel1, 0, 0, 1, 4);

        m_userNameEdit = new KLineEdit(UserInfo);
        m_userNameEdit->setObjectName("m_userNameEdit");

        gridLayout->addWidget(m_userNameEdit, 1, 1, 1, 3);

        TextLabel3 = new QLabel(UserInfo);
        TextLabel3->setObjectName("TextLabel3");
        TextLabel3->setMinimumSize(QSize(100, 0));
        TextLabel3->setWordWrap(false);

        gridLayout->addWidget(TextLabel3, 3, 0, 1, 1);

        TextLabel4 = new QLabel(UserInfo);
        TextLabel4->setObjectName("TextLabel4");
        TextLabel4->setMinimumSize(QSize(100, 0));
        TextLabel4->setWordWrap(false);

        gridLayout->addWidget(TextLabel4, 4, 0, 1, 1);

        TextLabel2 = new QLabel(UserInfo);
        TextLabel2->setObjectName("TextLabel2");
        TextLabel2->setMinimumSize(QSize(100, 0));
        TextLabel2->setWordWrap(false);

        gridLayout->addWidget(TextLabel2, 2, 0, 1, 1);

        m_streetEdit = new KLineEdit(UserInfo);
        m_streetEdit->setObjectName("m_streetEdit");

        gridLayout->addWidget(m_streetEdit, 2, 1, 1, 3);

        m_emailEdit = new KLineEdit(UserInfo);
        m_emailEdit->setObjectName("m_emailEdit");

        gridLayout->addWidget(m_emailEdit, 6, 1, 1, 3);

        m_townEdit = new KLineEdit(UserInfo);
        m_townEdit->setObjectName("m_townEdit");

        gridLayout->addWidget(m_townEdit, 3, 1, 1, 3);

        m_telephoneEdit = new KLineEdit(UserInfo);
        m_telephoneEdit->setObjectName("m_telephoneEdit");

        gridLayout->addWidget(m_telephoneEdit, 5, 1, 1, 3);

        TextLabel7 = new QLabel(UserInfo);
        TextLabel7->setObjectName("TextLabel7");
        TextLabel7->setMinimumSize(QSize(100, 0));
        TextLabel7->setWordWrap(false);

        gridLayout->addWidget(TextLabel7, 6, 0, 1, 1);

        TextLabel6 = new QLabel(UserInfo);
        TextLabel6->setObjectName("TextLabel6");
        TextLabel6->setMinimumSize(QSize(100, 0));
        TextLabel6->setWordWrap(false);

        gridLayout->addWidget(TextLabel6, 5, 0, 1, 1);

        TextLabel1 = new QLabel(UserInfo);
        TextLabel1->setObjectName("TextLabel1");
        TextLabel1->setWordWrap(false);

        gridLayout->addWidget(TextLabel1, 1, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_loadAddressButton = new QPushButton(UserInfo);
        m_loadAddressButton->setObjectName("m_loadAddressButton");

        hboxLayout->addWidget(m_loadAddressButton);

        spacer11 = new QSpacerItem(305, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer11);


        gridLayout->addLayout(hboxLayout, 7, 0, 1, 4);

        spacer5 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacer5, 8, 2, 1, 1);

        m_countyEdit = new KLineEdit(UserInfo);
        m_countyEdit->setObjectName("m_countyEdit");

        gridLayout->addWidget(m_countyEdit, 4, 1, 1, 1);

        TextLabel5 = new QLabel(UserInfo);
        TextLabel5->setObjectName("TextLabel5");
        TextLabel5->setMinimumSize(QSize(100, 0));
        TextLabel5->setWordWrap(false);

        gridLayout->addWidget(TextLabel5, 4, 2, 1, 1);

        m_postcodeEdit = new KLineEdit(UserInfo);
        m_postcodeEdit->setObjectName("m_postcodeEdit");

        gridLayout->addWidget(m_postcodeEdit, 4, 3, 1, 1);

        QWidget::setTabOrder(m_userNameEdit, m_streetEdit);
        QWidget::setTabOrder(m_streetEdit, m_townEdit);
        QWidget::setTabOrder(m_townEdit, m_countyEdit);
        QWidget::setTabOrder(m_countyEdit, m_postcodeEdit);
        QWidget::setTabOrder(m_postcodeEdit, m_telephoneEdit);
        QWidget::setTabOrder(m_telephoneEdit, m_emailEdit);
        QWidget::setTabOrder(m_emailEdit, m_loadAddressButton);

        retranslateUi(UserInfo);

        QMetaObject::connectSlotsByName(UserInfo);
    } // setupUi

    void retranslateUi(QWidget *UserInfo)
    {
        qLabel1->setText(tr2i18n("<p>Use the fields below to enter some information about yourself.</p>\n"
"\n"
"<p>All information is optional and only used to personalize your KMyMoney file.</p>", nullptr));
        TextLabel3->setText(tr2i18n("Town:", nullptr));
        TextLabel4->setText(tr2i18n("County/State:", nullptr));
        TextLabel2->setText(tr2i18n("Street:", nullptr));
        TextLabel7->setText(tr2i18n("Email:", "@label email address"));
        TextLabel6->setText(tr2i18n("Telephone:", nullptr));
        TextLabel1->setText(tr2i18n("Your name:", nullptr));
        m_loadAddressButton->setText(tr2i18n("Load from Addressbook", nullptr));
        TextLabel5->setText(tr2i18n("Postal Code:", nullptr));
        (void)UserInfo;
    } // retranslateUi

};

namespace Ui {
    class UserInfo: public Ui_UserInfo {};
} // namespace Ui

QT_END_NAMESPACE

#endif // USERINFO_H

