/********************************************************************************
** Form generated from reading UI file 'ktagsview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KTAGSVIEW_H
#define UI_KTAGSVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "../widgets/kmymoneyaccountcombo.h"
#include "kcolorbutton.h"
#include "kmmemptyview.h"
#include "ktextedit.h"
#include "ledgerview.h"

QT_BEGIN_NAMESPACE

class Ui_KTagsView
{
public:
    QVBoxLayout *verticalLayout_3;
    QVBoxLayout *verticalLayout;
    QSplitter *m_splitter;
    QGroupBox *groupBox1;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QToolButton *m_newButton;
    QToolButton *m_renameButton;
    QToolButton *m_deleteButton;
    QHBoxLayout *m_listTopHLayout;
    QLineEdit *m_searchWidget;
    QComboBox *m_filterBox;
    KMMEmptyListView *m_tagsList;
    QTabWidget *m_tabWidget;
    QWidget *tab;
    QVBoxLayout *vboxLayout1;
    QGroupBox *transactionGroup;
    QVBoxLayout *vboxLayout2;
    QVBoxLayout *vboxLayout3;
    LedgerView *m_register;
    QLabel *m_balanceLabel;
    QWidget *tab1;
    QVBoxLayout *vboxLayout4;
    QGroupBox *detailsGroup;
    QGridLayout *gridLayout;
    KTextEdit *m_notes;
    QCheckBox *m_closed;
    QLabel *label_3;
    QLabel *TextLabel4;
    QSpacerItem *horizontalSpacer;
    KColorButton *m_colorbutton;
    QSpacerItem *spacer6;
    QHBoxLayout *hboxLayout1;
    QPushButton *m_helpButton;
    QSpacerItem *spacer14;
    QToolButton *m_updateButton;

    void setupUi(QWidget *KTagsView)
    {
        if (KTagsView->objectName().isEmpty())
            KTagsView->setObjectName("KTagsView");
        KTagsView->resize(643, 404);
        verticalLayout_3 = new QVBoxLayout(KTagsView);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        m_splitter = new QSplitter(KTagsView);
        m_splitter->setObjectName("m_splitter");
        m_splitter->setOrientation(Qt::Horizontal);
        m_splitter->setChildrenCollapsible(false);
        groupBox1 = new QGroupBox(m_splitter);
        groupBox1->setObjectName("groupBox1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(groupBox1->sizePolicy().hasHeightForWidth());
        groupBox1->setSizePolicy(sizePolicy);
        vboxLayout = new QVBoxLayout(groupBox1);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_newButton = new QToolButton(groupBox1);
        m_newButton->setObjectName("m_newButton");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_newButton->sizePolicy().hasHeightForWidth());
        m_newButton->setSizePolicy(sizePolicy1);

        hboxLayout->addWidget(m_newButton);

        m_renameButton = new QToolButton(groupBox1);
        m_renameButton->setObjectName("m_renameButton");
        sizePolicy1.setHeightForWidth(m_renameButton->sizePolicy().hasHeightForWidth());
        m_renameButton->setSizePolicy(sizePolicy1);

        hboxLayout->addWidget(m_renameButton);

        m_deleteButton = new QToolButton(groupBox1);
        m_deleteButton->setObjectName("m_deleteButton");
        sizePolicy1.setHeightForWidth(m_deleteButton->sizePolicy().hasHeightForWidth());
        m_deleteButton->setSizePolicy(sizePolicy1);

        hboxLayout->addWidget(m_deleteButton);


        vboxLayout->addLayout(hboxLayout);

        m_listTopHLayout = new QHBoxLayout();
        m_listTopHLayout->setSpacing(6);
        m_listTopHLayout->setObjectName("m_listTopHLayout");
        m_searchWidget = new QLineEdit(groupBox1);
        m_searchWidget->setObjectName("m_searchWidget");
        m_searchWidget->setClearButtonEnabled(true);

        m_listTopHLayout->addWidget(m_searchWidget);

        m_filterBox = new QComboBox(groupBox1);
        m_filterBox->setObjectName("m_filterBox");

        m_listTopHLayout->addWidget(m_filterBox);


        vboxLayout->addLayout(m_listTopHLayout);

        m_tagsList = new KMMEmptyListView(groupBox1);
        m_tagsList->setObjectName("m_tagsList");

        vboxLayout->addWidget(m_tagsList);

        m_splitter->addWidget(groupBox1);
        m_tabWidget = new QTabWidget(m_splitter);
        m_tabWidget->setObjectName("m_tabWidget");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy2.setHorizontalStretch(2);
        sizePolicy2.setVerticalStretch(1);
        sizePolicy2.setHeightForWidth(m_tabWidget->sizePolicy().hasHeightForWidth());
        m_tabWidget->setSizePolicy(sizePolicy2);
        tab = new QWidget();
        tab->setObjectName("tab");
        vboxLayout1 = new QVBoxLayout(tab);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        transactionGroup = new QGroupBox(tab);
        transactionGroup->setObjectName("transactionGroup");
        vboxLayout2 = new QVBoxLayout(transactionGroup);
        vboxLayout2->setSpacing(6);
        vboxLayout2->setContentsMargins(11, 11, 11, 11);
        vboxLayout2->setObjectName("vboxLayout2");
        vboxLayout2->setContentsMargins(11, 11, 11, 11);
        vboxLayout3 = new QVBoxLayout();
        vboxLayout3->setSpacing(6);
        vboxLayout3->setObjectName("vboxLayout3");
        vboxLayout3->setContentsMargins(0, 0, 0, 0);
        m_register = new LedgerView(transactionGroup);
        m_register->setObjectName("m_register");

        vboxLayout3->addWidget(m_register);

        m_balanceLabel = new QLabel(transactionGroup);
        m_balanceLabel->setObjectName("m_balanceLabel");
        m_balanceLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        m_balanceLabel->setWordWrap(false);

        vboxLayout3->addWidget(m_balanceLabel);


        vboxLayout2->addLayout(vboxLayout3);


        vboxLayout1->addWidget(transactionGroup);

        m_tabWidget->addTab(tab, QString());
        tab1 = new QWidget();
        tab1->setObjectName("tab1");
        vboxLayout4 = new QVBoxLayout(tab1);
        vboxLayout4->setSpacing(6);
        vboxLayout4->setContentsMargins(11, 11, 11, 11);
        vboxLayout4->setObjectName("vboxLayout4");
        detailsGroup = new QGroupBox(tab1);
        detailsGroup->setObjectName("detailsGroup");
        gridLayout = new QGridLayout(detailsGroup);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(11, 11, 11, 11);
        m_notes = new KTextEdit(detailsGroup);
        m_notes->setObjectName("m_notes");

        gridLayout->addWidget(m_notes, 2, 1, 1, 1);

        m_closed = new QCheckBox(detailsGroup);
        m_closed->setObjectName("m_closed");

        gridLayout->addWidget(m_closed, 1, 0, 1, 1);

        label_3 = new QLabel(detailsGroup);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        TextLabel4 = new QLabel(detailsGroup);
        TextLabel4->setObjectName("TextLabel4");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(TextLabel4->sizePolicy().hasHeightForWidth());
        TextLabel4->setSizePolicy(sizePolicy3);
        TextLabel4->setMinimumSize(QSize(90, 0));
        TextLabel4->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        TextLabel4->setWordWrap(false);

        gridLayout->addWidget(TextLabel4, 0, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 2, 1, 1);

        m_colorbutton = new KColorButton(detailsGroup);
        m_colorbutton->setObjectName("m_colorbutton");

        gridLayout->addWidget(m_colorbutton, 0, 1, 1, 1);


        vboxLayout4->addWidget(detailsGroup);

        spacer6 = new QSpacerItem(21, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout4->addItem(spacer6);

        m_tabWidget->addTab(tab1, QString());
        m_splitter->addWidget(m_tabWidget);

        verticalLayout->addWidget(m_splitter);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        m_helpButton = new QPushButton(KTagsView);
        m_helpButton->setObjectName("m_helpButton");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(m_helpButton->sizePolicy().hasHeightForWidth());
        m_helpButton->setSizePolicy(sizePolicy4);

        hboxLayout1->addWidget(m_helpButton);

        spacer14 = new QSpacerItem(260, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacer14);

        m_updateButton = new QToolButton(KTagsView);
        m_updateButton->setObjectName("m_updateButton");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(m_updateButton->sizePolicy().hasHeightForWidth());
        m_updateButton->setSizePolicy(sizePolicy5);
        m_updateButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        hboxLayout1->addWidget(m_updateButton);


        verticalLayout->addLayout(hboxLayout1);


        verticalLayout_3->addLayout(verticalLayout);


        retranslateUi(KTagsView);

        m_tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KTagsView);
    } // setupUi

    void retranslateUi(QWidget *KTagsView)
    {
        groupBox1->setTitle(tr2i18n("Your tags", nullptr));
        m_newButton->setText(tr2i18n("New", nullptr));
        m_renameButton->setText(tr2i18n("Ren", nullptr));
        m_deleteButton->setText(tr2i18n("Del", nullptr));
        transactionGroup->setTitle(tr2i18n("Transactions", nullptr));
        m_balanceLabel->setText(tr2i18n("Balance: ", nullptr));
        m_tabWidget->setTabText(m_tabWidget->indexOf(tab), tr2i18n("Transactions", nullptr));
        detailsGroup->setTitle(tr2i18n("Tag configuration", nullptr));
        m_closed->setText(tr2i18n("Closed", nullptr));
        label_3->setText(tr2i18n("Notes", nullptr));
        TextLabel4->setText(tr2i18n("Tag color:", nullptr));
        m_tabWidget->setTabText(m_tabWidget->indexOf(tab1), tr2i18n("Details", nullptr));
        m_helpButton->setText(tr2i18n("Help", nullptr));
        m_updateButton->setText(tr2i18n("Update", nullptr));
        (void)KTagsView;
    } // retranslateUi

};

namespace Ui {
    class KTagsView: public Ui_KTagsView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KTAGSVIEW_H

