/********************************************************************************
** Form generated from reading UI file 'csvwizard.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CSVWIZARD_H
#define UI_CSVWIZARD_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_CSVWizard
{
public:
    QVBoxLayout *verticalLayout;
    QTableView *tableView;
    QHBoxLayout *horizontalLayout;
    QGroupBox *wizardBox;
    QVBoxLayout *verticalLayout_3;
    QSpacerItem *verticalSpacer_2;
    QLabel *label_intro;
    QLabel *label_separators;
    QLabel *label_rows;
    QLabel *label_columns;
    QLabel *label_formats;
    QSpacerItem *verticalSpacer_3;

    void setupUi(QDialog *CSVWizard)
    {
        if (CSVWizard->objectName().isEmpty())
            CSVWizard->setObjectName("CSVWizard");
        CSVWizard->resize(800, 400);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(CSVWizard->sizePolicy().hasHeightForWidth());
        CSVWizard->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(CSVWizard);
        verticalLayout->setObjectName("verticalLayout");
        tableView = new QTableView(CSVWizard);
        tableView->setObjectName("tableView");
        tableView->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContentsOnFirstShow);
        tableView->setTabKeyNavigation(false);

        verticalLayout->addWidget(tableView);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        wizardBox = new QGroupBox(CSVWizard);
        wizardBox->setObjectName("wizardBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(wizardBox->sizePolicy().hasHeightForWidth());
        wizardBox->setSizePolicy(sizePolicy1);
        verticalLayout_3 = new QVBoxLayout(wizardBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);

        label_intro = new QLabel(wizardBox);
        label_intro->setObjectName("label_intro");
        label_intro->setMinimumSize(QSize(90, 0));

        verticalLayout_3->addWidget(label_intro);

        label_separators = new QLabel(wizardBox);
        label_separators->setObjectName("label_separators");
        label_separators->setMinimumSize(QSize(90, 0));

        verticalLayout_3->addWidget(label_separators);

        label_rows = new QLabel(wizardBox);
        label_rows->setObjectName("label_rows");
        label_rows->setMinimumSize(QSize(90, 0));

        verticalLayout_3->addWidget(label_rows);

        label_columns = new QLabel(wizardBox);
        label_columns->setObjectName("label_columns");

        verticalLayout_3->addWidget(label_columns);

        label_formats = new QLabel(wizardBox);
        label_formats->setObjectName("label_formats");

        verticalLayout_3->addWidget(label_formats);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        verticalLayout_3->setStretch(0, 1);
        verticalLayout_3->setStretch(1, 1);
        verticalLayout_3->setStretch(2, 1);
        verticalLayout_3->setStretch(3, 1);
        verticalLayout_3->setStretch(4, 1);
        verticalLayout_3->setStretch(5, 1);
        verticalLayout_3->setStretch(6, 400);

        horizontalLayout->addWidget(wizardBox);


        verticalLayout->addLayout(horizontalLayout);

        verticalLayout->setStretch(0, 100);
        verticalLayout->setStretch(1, 1);

        retranslateUi(CSVWizard);

        QMetaObject::connectSlotsByName(CSVWizard);
    } // setupUi

    void retranslateUi(QDialog *CSVWizard)
    {
        label_intro->setText(tr2i18n("1. File", "Noun, the introduction."));
        label_separators->setText(tr2i18n("2. Separators", nullptr));
        label_rows->setText(tr2i18n("3. Rows", nullptr));
        label_columns->setText(tr2i18n("4. Columns", nullptr));
        label_formats->setText(tr2i18n("5. Formats", nullptr));
        (void)CSVWizard;
    } // retranslateUi

};

namespace Ui {
    class CSVWizard: public Ui_CSVWizard {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CSVWIZARD_H

