/********************************************************************************
** Form generated from reading UI file 'konlinebankingstatusdecl.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KONLINEBANKINGSTATUSDECL_H
#define UI_KONLINEBANKINGSTATUSDECL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTimeEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "kdatecombobox.h"
#include "kled.h"
#include "klineedit.h"
#include "kmymoneydateedit.h"
#include "kpasswordlineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KOnlineBankingStatusDecl
{
public:
    QVBoxLayout *verticalLayout_5;
    QTabWidget *tabWidget;
    QWidget *accountTab;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout_2;
    QLabel *textLabel1;
    QHBoxLayout *hboxLayout;
    KLed *m_ledOnlineStatus;
    QLabel *m_textOnlineStatus;
    QSpacerItem *spacer4;
    QLabel *textLabel2;
    QLabel *m_textBank;
    QLabel *textLabel3;
    QLabel *m_textOnlineAccount;
    QCheckBox *m_storePassword;
    KPasswordLineEdit *m_password;
    QSpacerItem *verticalSpacer_3;
    QWidget *ofxTab;
    QVBoxLayout *verticalLayout_3;
    QGridLayout *gridLayout_1;
    KComboBox *m_headerVersionCombo;
    QLabel *textLabel1_2;
    KLineEdit *m_applicationEdit;
    QSpacerItem *spacer4_2;
    QLabel *textLabel1_3;
    KLineEdit *m_headerVersionEdit;
    KComboBox *m_applicationCombo;
    QLabel *m_clientUidLabel;
    QLabel *label_5;
    QLineEdit *m_clientUidEdit;
    QLineEdit *m_userAgentEdit;
    QSpacerItem *verticalSpacer_2;
    QWidget *importTab;
    QVBoxLayout *verticalLayout_4;
    QGroupBox *buttonGroupBox2;
    QVBoxLayout *verticalLayout;
    QFrame *frame;
    QGridLayout *gridLayout;
    QRadioButton *m_lastUpdateRB;
    QLabel *m_lastUpdateTXT;
    QRadioButton *m_todayRB;
    QSpinBox *m_numdaysSpin;
    QLabel *textLabel2_2;
    QRadioButton *m_pickDateRB;
    KMyMoneyDateEdit *m_specificDate;
    QSpacerItem *horizontalSpacer;
    QGroupBox *buttonGroupBox1;
    QGridLayout *gridLayout_3;
    QComboBox *m_preferredPrice;
    QLabel *textLabel3_2;
    QHBoxLayout *hl2;
    KComboBox *m_timestampOffsetSign;
    QTimeEdit *m_timestampOffset;
    QSpacerItem *horizontalSpacer_2;
    QComboBox *m_uniqueTransactionId;
    QLabel *label_3;
    QCheckBox *m_invertAmount;
    QLabel *label_4;
    QCheckBox *m_fixBuySellSignage;
    QLabel *label_6;
    QComboBox *m_preferredPayee;
    QLabel *label;
    QLabel *label_2;
    QSpacerItem *verticalSpacer;
    QButtonGroup *buttonGroup2;

    void setupUi(QWidget *KOnlineBankingStatusDecl)
    {
        if (KOnlineBankingStatusDecl->objectName().isEmpty())
            KOnlineBankingStatusDecl->setObjectName("KOnlineBankingStatusDecl");
        KOnlineBankingStatusDecl->resize(638, 468);
        verticalLayout_5 = new QVBoxLayout(KOnlineBankingStatusDecl);
        verticalLayout_5->setSpacing(6);
        verticalLayout_5->setContentsMargins(11, 11, 11, 11);
        verticalLayout_5->setObjectName("verticalLayout_5");
        tabWidget = new QTabWidget(KOnlineBankingStatusDecl);
        tabWidget->setObjectName("tabWidget");
        accountTab = new QWidget();
        accountTab->setObjectName("accountTab");
        verticalLayout_2 = new QVBoxLayout(accountTab);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setSpacing(6);
        gridLayout_2->setObjectName("gridLayout_2");
        textLabel1 = new QLabel(accountTab);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout_2->addWidget(textLabel1, 0, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_ledOnlineStatus = new KLed(accountTab);
        m_ledOnlineStatus->setObjectName("m_ledOnlineStatus");
        m_ledOnlineStatus->setState(KLed::Off);

        hboxLayout->addWidget(m_ledOnlineStatus);

        m_textOnlineStatus = new QLabel(accountTab);
        m_textOnlineStatus->setObjectName("m_textOnlineStatus");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(10);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_textOnlineStatus->sizePolicy().hasHeightForWidth());
        m_textOnlineStatus->setSizePolicy(sizePolicy);
        m_textOnlineStatus->setWordWrap(false);

        hboxLayout->addWidget(m_textOnlineStatus);

        spacer4 = new QSpacerItem(41, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer4);


        gridLayout_2->addLayout(hboxLayout, 0, 1, 1, 1);

        textLabel2 = new QLabel(accountTab);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout_2->addWidget(textLabel2, 1, 0, 1, 1);

        m_textBank = new QLabel(accountTab);
        m_textBank->setObjectName("m_textBank");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_textBank->sizePolicy().hasHeightForWidth());
        m_textBank->setSizePolicy(sizePolicy1);
        QFont font;
        font.setBold(true);
        m_textBank->setFont(font);
        m_textBank->setWordWrap(false);

        gridLayout_2->addWidget(m_textBank, 1, 1, 1, 1);

        textLabel3 = new QLabel(accountTab);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout_2->addWidget(textLabel3, 2, 0, 1, 1);

        m_textOnlineAccount = new QLabel(accountTab);
        m_textOnlineAccount->setObjectName("m_textOnlineAccount");
        sizePolicy1.setHeightForWidth(m_textOnlineAccount->sizePolicy().hasHeightForWidth());
        m_textOnlineAccount->setSizePolicy(sizePolicy1);
        m_textOnlineAccount->setFont(font);
        m_textOnlineAccount->setWordWrap(false);

        gridLayout_2->addWidget(m_textOnlineAccount, 2, 1, 1, 1);

        m_storePassword = new QCheckBox(accountTab);
        m_storePassword->setObjectName("m_storePassword");

        gridLayout_2->addWidget(m_storePassword, 3, 0, 1, 1);

        m_password = new KPasswordLineEdit(accountTab);
        m_password->setObjectName("m_password");
        m_password->setEnabled(false);
        m_password->setProperty("showClearButton", QVariant(true));

        gridLayout_2->addWidget(m_password, 3, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout_2);

        verticalSpacer_3 = new QSpacerItem(20, 143, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_3);

        tabWidget->addTab(accountTab, QString());
        ofxTab = new QWidget();
        ofxTab->setObjectName("ofxTab");
        verticalLayout_3 = new QVBoxLayout(ofxTab);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        gridLayout_1 = new QGridLayout();
        gridLayout_1->setSpacing(6);
        gridLayout_1->setObjectName("gridLayout_1");
        m_headerVersionCombo = new KComboBox(ofxTab);
        m_headerVersionCombo->setObjectName("m_headerVersionCombo");

        gridLayout_1->addWidget(m_headerVersionCombo, 1, 1, 1, 1);

        textLabel1_2 = new QLabel(ofxTab);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        gridLayout_1->addWidget(textLabel1_2, 0, 0, 1, 1);

        m_applicationEdit = new KLineEdit(ofxTab);
        m_applicationEdit->setObjectName("m_applicationEdit");

        gridLayout_1->addWidget(m_applicationEdit, 0, 2, 1, 1);

        spacer4_2 = new QSpacerItem(150, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_1->addItem(spacer4_2, 0, 4, 1, 1);

        textLabel1_3 = new QLabel(ofxTab);
        textLabel1_3->setObjectName("textLabel1_3");
        textLabel1_3->setWordWrap(false);

        gridLayout_1->addWidget(textLabel1_3, 1, 0, 1, 1);

        m_headerVersionEdit = new KLineEdit(ofxTab);
        m_headerVersionEdit->setObjectName("m_headerVersionEdit");

        gridLayout_1->addWidget(m_headerVersionEdit, 1, 2, 1, 1);

        m_applicationCombo = new KComboBox(ofxTab);
        m_applicationCombo->setObjectName("m_applicationCombo");

        gridLayout_1->addWidget(m_applicationCombo, 0, 1, 1, 1);

        m_clientUidLabel = new QLabel(ofxTab);
        m_clientUidLabel->setObjectName("m_clientUidLabel");

        gridLayout_1->addWidget(m_clientUidLabel, 2, 0, 1, 1);

        label_5 = new QLabel(ofxTab);
        label_5->setObjectName("label_5");

        gridLayout_1->addWidget(label_5, 3, 0, 1, 1);

        m_clientUidEdit = new QLineEdit(ofxTab);
        m_clientUidEdit->setObjectName("m_clientUidEdit");

        gridLayout_1->addWidget(m_clientUidEdit, 2, 1, 1, 4);

        m_userAgentEdit = new QLineEdit(ofxTab);
        m_userAgentEdit->setObjectName("m_userAgentEdit");

        gridLayout_1->addWidget(m_userAgentEdit, 3, 1, 1, 4);

        gridLayout_1->setColumnStretch(1, 2);
        gridLayout_1->setColumnStretch(2, 2);
        gridLayout_1->setColumnStretch(4, 1);

        verticalLayout_3->addLayout(gridLayout_1);

        verticalSpacer_2 = new QSpacerItem(20, 195, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);

        tabWidget->addTab(ofxTab, QString());
        importTab = new QWidget();
        importTab->setObjectName("importTab");
        verticalLayout_4 = new QVBoxLayout(importTab);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName("verticalLayout_4");
        buttonGroupBox2 = new QGroupBox(importTab);
        buttonGroupBox2->setObjectName("buttonGroupBox2");
        verticalLayout = new QVBoxLayout(buttonGroupBox2);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        frame = new QFrame(buttonGroupBox2);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::NoFrame);
        frame->setFrameShadow(QFrame::Plain);
        frame->setLineWidth(0);
        gridLayout = new QGridLayout(frame);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setSizeConstraint(QLayout::SetMinimumSize);
        m_lastUpdateRB = new QRadioButton(frame);
        buttonGroup2 = new QButtonGroup(KOnlineBankingStatusDecl);
        buttonGroup2->setObjectName("buttonGroup2");
        buttonGroup2->addButton(m_lastUpdateRB);
        m_lastUpdateRB->setObjectName("m_lastUpdateRB");

        gridLayout->addWidget(m_lastUpdateRB, 0, 0, 1, 2);

        m_lastUpdateTXT = new QLabel(frame);
        m_lastUpdateTXT->setObjectName("m_lastUpdateTXT");
        m_lastUpdateTXT->setWordWrap(false);

        gridLayout->addWidget(m_lastUpdateTXT, 0, 2, 1, 3);

        m_todayRB = new QRadioButton(frame);
        buttonGroup2->addButton(m_todayRB);
        m_todayRB->setObjectName("m_todayRB");
        m_todayRB->setChecked(true);

        gridLayout->addWidget(m_todayRB, 1, 0, 1, 2);

        m_numdaysSpin = new QSpinBox(frame);
        m_numdaysSpin->setObjectName("m_numdaysSpin");
        m_numdaysSpin->setMaximum(180);

        gridLayout->addWidget(m_numdaysSpin, 1, 2, 1, 1);

        textLabel2_2 = new QLabel(frame);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setWordWrap(false);

        gridLayout->addWidget(textLabel2_2, 1, 3, 1, 2);

        m_pickDateRB = new QRadioButton(frame);
        buttonGroup2->addButton(m_pickDateRB);
        m_pickDateRB->setObjectName("m_pickDateRB");

        gridLayout->addWidget(m_pickDateRB, 2, 0, 1, 1);

        m_specificDate = new KMyMoneyDateEdit(frame);
        m_specificDate->setObjectName("m_specificDate");
        m_specificDate->setEnabled(false);

        gridLayout->addWidget(m_specificDate, 2, 1, 1, 3);

        horizontalSpacer = new QSpacerItem(214, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 2, 4, 1, 1);


        verticalLayout->addWidget(frame);


        verticalLayout_4->addWidget(buttonGroupBox2);

        buttonGroupBox1 = new QGroupBox(importTab);
        buttonGroupBox1->setObjectName("buttonGroupBox1");
        gridLayout_3 = new QGridLayout(buttonGroupBox1);
        gridLayout_3->setSpacing(6);
        gridLayout_3->setContentsMargins(11, 11, 11, 11);
        gridLayout_3->setObjectName("gridLayout_3");
        m_preferredPrice = new QComboBox(buttonGroupBox1);
        m_preferredPrice->addItem(QString());
        m_preferredPrice->addItem(QString());
        m_preferredPrice->setObjectName("m_preferredPrice");

        gridLayout_3->addWidget(m_preferredPrice, 2, 1, 1, 1);

        textLabel3_2 = new QLabel(buttonGroupBox1);
        textLabel3_2->setObjectName("textLabel3_2");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(textLabel3_2->sizePolicy().hasHeightForWidth());
        textLabel3_2->setSizePolicy(sizePolicy2);
        textLabel3_2->setWordWrap(false);

        gridLayout_3->addWidget(textLabel3_2, 0, 0, 1, 1);

        hl2 = new QHBoxLayout();
        hl2->setSpacing(6);
        hl2->setObjectName("hl2");
        m_timestampOffsetSign = new KComboBox(buttonGroupBox1);
        m_timestampOffsetSign->addItem(QString());
        m_timestampOffsetSign->addItem(QString());
        m_timestampOffsetSign->setObjectName("m_timestampOffsetSign");

        hl2->addWidget(m_timestampOffsetSign);

        m_timestampOffset = new QTimeEdit(buttonGroupBox1);
        m_timestampOffset->setObjectName("m_timestampOffset");

        hl2->addWidget(m_timestampOffset);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hl2->addItem(horizontalSpacer_2);

        hl2->setStretch(1, 1);

        gridLayout_3->addLayout(hl2, 3, 1, 1, 1);

        m_uniqueTransactionId = new QComboBox(buttonGroupBox1);
        m_uniqueTransactionId->addItem(QString());
        m_uniqueTransactionId->addItem(QString());
        m_uniqueTransactionId->setObjectName("m_uniqueTransactionId");

        gridLayout_3->addWidget(m_uniqueTransactionId, 1, 1, 1, 1);

        label_3 = new QLabel(buttonGroupBox1);
        label_3->setObjectName("label_3");

        gridLayout_3->addWidget(label_3, 4, 0, 1, 1);

        m_invertAmount = new QCheckBox(buttonGroupBox1);
        m_invertAmount->setObjectName("m_invertAmount");

        gridLayout_3->addWidget(m_invertAmount, 4, 1, 1, 1);

        label_4 = new QLabel(buttonGroupBox1);
        label_4->setObjectName("label_4");

        gridLayout_3->addWidget(label_4, 5, 0, 1, 1);

        m_fixBuySellSignage = new QCheckBox(buttonGroupBox1);
        m_fixBuySellSignage->setObjectName("m_fixBuySellSignage");

        gridLayout_3->addWidget(m_fixBuySellSignage, 5, 1, 1, 1);

        label_6 = new QLabel(buttonGroupBox1);
        label_6->setObjectName("label_6");

        gridLayout_3->addWidget(label_6, 2, 0, 1, 1);

        m_preferredPayee = new QComboBox(buttonGroupBox1);
        m_preferredPayee->addItem(QString::fromUtf8("PAYEEID"));
        m_preferredPayee->addItem(QString::fromUtf8("NAME"));
        m_preferredPayee->addItem(QString::fromUtf8("MEMO"));
        m_preferredPayee->setObjectName("m_preferredPayee");

        gridLayout_3->addWidget(m_preferredPayee, 0, 1, 1, 1);

        label = new QLabel(buttonGroupBox1);
        label->setObjectName("label");

        gridLayout_3->addWidget(label, 3, 0, 1, 1);

        label_2 = new QLabel(buttonGroupBox1);
        label_2->setObjectName("label_2");

        gridLayout_3->addWidget(label_2, 1, 0, 1, 1);

        gridLayout_3->setRowStretch(0, 2);

        verticalLayout_4->addWidget(buttonGroupBox1);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);

        tabWidget->addTab(importTab, QString());

        verticalLayout_5->addWidget(tabWidget);

        QWidget::setTabOrder(tabWidget, m_storePassword);
        QWidget::setTabOrder(m_storePassword, m_password);
        QWidget::setTabOrder(m_password, m_applicationCombo);
        QWidget::setTabOrder(m_applicationCombo, m_applicationEdit);
        QWidget::setTabOrder(m_applicationEdit, m_headerVersionCombo);
        QWidget::setTabOrder(m_headerVersionCombo, m_headerVersionEdit);
        QWidget::setTabOrder(m_headerVersionEdit, m_clientUidEdit);
        QWidget::setTabOrder(m_clientUidEdit, m_userAgentEdit);
        QWidget::setTabOrder(m_userAgentEdit, m_lastUpdateRB);
        QWidget::setTabOrder(m_lastUpdateRB, m_todayRB);
        QWidget::setTabOrder(m_todayRB, m_numdaysSpin);
        QWidget::setTabOrder(m_numdaysSpin, m_pickDateRB);
        QWidget::setTabOrder(m_pickDateRB, m_preferredPayee);
        QWidget::setTabOrder(m_preferredPayee, m_uniqueTransactionId);
        QWidget::setTabOrder(m_uniqueTransactionId, m_timestampOffsetSign);
        QWidget::setTabOrder(m_timestampOffsetSign, m_timestampOffset);
        QWidget::setTabOrder(m_timestampOffset, m_invertAmount);
        QWidget::setTabOrder(m_invertAmount, m_fixBuySellSignage);

        retranslateUi(KOnlineBankingStatusDecl);
        QObject::connect(m_pickDateRB, SIGNAL(toggled(bool)), m_specificDate, SLOT(setEnabled(bool)));
        QObject::connect(m_todayRB, &QRadioButton::toggled, m_numdaysSpin, &QSpinBox::setEnabled);
        QObject::connect(m_storePassword, SIGNAL(toggled(bool)), m_password, SLOT(setEnabled(bool)));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KOnlineBankingStatusDecl);
    } // setupUi

    void retranslateUi(QWidget *KOnlineBankingStatusDecl)
    {
        textLabel1->setText(tr2i18n("STATUS:", nullptr));
        m_textOnlineStatus->setText(tr2i18n("Unavailable", nullptr));
        textLabel2->setText(tr2i18n("BANK/BROKER:", nullptr));
        m_textBank->setText(tr2i18n("&lt;Not configured&gt;", nullptr));
        textLabel3->setText(tr2i18n("ACCOUNT:", nullptr));
        m_textOnlineAccount->setText(tr2i18n("&lt;Not configured&gt;", nullptr));
        m_storePassword->setText(tr2i18n("Store password", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(accountTab), tr2i18n("Account Details", nullptr));
        textLabel1_2->setText(tr2i18n("Identify as", nullptr));
        textLabel1_3->setText(tr2i18n("Header Version", nullptr));
        m_clientUidLabel->setText(tr2i18n("Client UID", nullptr));
        label_5->setText(tr2i18n("User Agent", nullptr));
#if QT_CONFIG(tooltip)
        m_clientUidEdit->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_clientUidEdit->setWhatsThis(tr2i18n("OFX <CLIENTUID> for institutions that require it", nullptr));
#endif // QT_CONFIG(whatsthis)
        tabWidget->setTabText(tabWidget->indexOf(ofxTab), tr2i18n("OFX Details", nullptr));
        buttonGroupBox2->setTitle(tr2i18n("Start date of import", nullptr));
        m_lastUpdateRB->setText(tr2i18n("Last &update", nullptr));
        m_lastUpdateTXT->setText(QString());
        m_todayRB->setText(tr2i18n("To&day minus", nullptr));
        textLabel2_2->setText(tr2i18n("days", "@action number of days"));
        m_pickDateRB->setText(tr2i18n("Pi&ck date", nullptr));
        buttonGroupBox1->setTitle(tr2i18n("Import options", nullptr));
        m_preferredPrice->setItemText(0, tr2i18n("Price information", nullptr));
        m_preferredPrice->setItemText(1, tr2i18n("Value and number of shares", nullptr));

        textLabel3_2->setText(tr2i18n("Payee's name is based on contents of the OFX tag", nullptr));
        m_timestampOffsetSign->setItemText(0, tr2i18n("+", nullptr));
        m_timestampOffsetSign->setItemText(1, tr2i18n("-", nullptr));

        m_uniqueTransactionId->setItemText(0, tr2i18n("OFX FITID", "@item:inlistbox Method to identify duplicate transactions"));
        m_uniqueTransactionId->setItemText(1, tr2i18n("KMyMoney ID", "@item:inlistbox Method to identify duplicate transactions"));

        label_3->setText(tr2i18n("Invert sign of transaction amount", nullptr));
#if QT_CONFIG(tooltip)
        m_invertAmount->setToolTip(tr2i18n("<html><head/><body><p>Check this if you need to invert the sign of imported transaction amounts, e.g. if you need to reverse charges and payments because your institution provides them inverted.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_invertAmount->setText(QString());
        label_4->setText(tr2i18n("Fix sign of the investment transaction amount and quantity", nullptr));
#if QT_CONFIG(tooltip)
        m_fixBuySellSignage->setToolTip(tr2i18n("<html><head/><body><p>Check this if the investment transactions after importing have incorrect Buy/Sell type assigned, which may be caused by your institution not applying the correct negative/positive sign to the share amount or quantity value.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_fixBuySellSignage->setText(QString());
        label_6->setText(tr2i18n("Price is derived from", nullptr));

        label->setText(tr2i18n("Timezone offset", nullptr));
        label_2->setText(tr2i18n("Method to detect duplicate transactions during import", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(importTab), tr2i18n("Import Details", nullptr));
        (void)KOnlineBankingStatusDecl;
    } // retranslateUi

};

namespace Ui {
    class KOnlineBankingStatusDecl: public Ui_KOnlineBankingStatusDecl {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KONLINEBANKINGSTATUSDECL_H

