/********************************************************************************
** Form generated from reading UI file 'knewbudgetdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KNEWBUDGETDLG_H
#define UI_KNEWBUDGETDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KNewBudgetDlg
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout1;
    QLabel *textLabel2;
    QLabel *textLabel1;
    QVBoxLayout *vboxLayout2;
    KLineEdit *m_leBudgetName;
    KComboBox *m_cbYear;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KNewBudgetDlg)
    {
        if (KNewBudgetDlg->objectName().isEmpty())
            KNewBudgetDlg->setObjectName("KNewBudgetDlg");
        KNewBudgetDlg->resize(283, 116);
        KNewBudgetDlg->setModal(true);
        gridLayout = new QGridLayout(KNewBudgetDlg);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        vboxLayout = new QVBoxLayout();
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName("vboxLayout1");
        textLabel2 = new QLabel(KNewBudgetDlg);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        vboxLayout1->addWidget(textLabel2);

        textLabel1 = new QLabel(KNewBudgetDlg);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        vboxLayout1->addWidget(textLabel1);


        hboxLayout->addLayout(vboxLayout1);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(6);
        vboxLayout2->setObjectName("vboxLayout2");
        m_leBudgetName = new KLineEdit(KNewBudgetDlg);
        m_leBudgetName->setObjectName("m_leBudgetName");

        vboxLayout2->addWidget(m_leBudgetName);

        m_cbYear = new KComboBox(KNewBudgetDlg);
        m_cbYear->setObjectName("m_cbYear");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_cbYear->sizePolicy().hasHeightForWidth());
        m_cbYear->setSizePolicy(sizePolicy);

        vboxLayout2->addWidget(m_cbYear);


        hboxLayout->addLayout(vboxLayout2);


        vboxLayout->addLayout(hboxLayout);

        buttonBox = new QDialogButtonBox(KNewBudgetDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        gridLayout->addLayout(vboxLayout, 0, 0, 1, 1);


        retranslateUi(KNewBudgetDlg);

        m_cbYear->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(KNewBudgetDlg);
    } // setupUi

    void retranslateUi(QDialog *KNewBudgetDlg)
    {
        KNewBudgetDlg->setWindowTitle(tr2i18n("New Budget", nullptr));
        textLabel2->setText(tr2i18n("Name", "header of the budget name column"));
        textLabel1->setText(tr2i18n("Year", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KNewBudgetDlg: public Ui_KNewBudgetDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KNEWBUDGETDLG_H

