/********************************************************************************
** Form generated from reading UI file 'ibanbicitemedit.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IBANBICITEMEDIT_H
#define UI_IBANBICITEMEDIT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kmymoneyvalidationfeedback.h"
#include "widgets/payeeidentifier/ibanbic/kbicedit.h"
#include "widgets/payeeidentifier/ibanbic/kibanlineedit.h"

QT_BEGIN_NAMESPACE

class Ui_ibanBicItemEdit
{
public:
    QVBoxLayout *verticalLayout;
    KIbanLineEdit *ibanEdit;
    KMyMoneyValidationFeedback *ibanFeedback;
    KBicEdit *bicEdit;
    QWidget *bicFeedback;

    void setupUi(QWidget *ibanBicItemEdit)
    {
        if (ibanBicItemEdit->objectName().isEmpty())
            ibanBicItemEdit->setObjectName("ibanBicItemEdit");
        ibanBicItemEdit->resize(86, 84);
        verticalLayout = new QVBoxLayout(ibanBicItemEdit);
        verticalLayout->setObjectName("verticalLayout");
        ibanEdit = new KIbanLineEdit(ibanBicItemEdit);
        ibanEdit->setObjectName("ibanEdit");

        verticalLayout->addWidget(ibanEdit);

        ibanFeedback = new KMyMoneyValidationFeedback(ibanBicItemEdit);
        ibanFeedback->setObjectName("ibanFeedback");

        verticalLayout->addWidget(ibanFeedback);

        bicEdit = new KBicEdit(ibanBicItemEdit);
        bicEdit->setObjectName("bicEdit");

        verticalLayout->addWidget(bicEdit);

        bicFeedback = new QWidget(ibanBicItemEdit);
        bicFeedback->setObjectName("bicFeedback");

        verticalLayout->addWidget(bicFeedback);


        retranslateUi(ibanBicItemEdit);

        QMetaObject::connectSlotsByName(ibanBicItemEdit);
    } // setupUi

    void retranslateUi(QWidget *ibanBicItemEdit)
    {
        ibanEdit->setPlaceholderText(tr2i18n("IBAN", nullptr));
        bicEdit->setPlaceholderText(tr2i18n("BIC", nullptr));
        (void)ibanBicItemEdit;
    } // retranslateUi

};

namespace Ui {
    class ibanBicItemEdit: public Ui_ibanBicItemEdit {};
} // namespace Ui

QT_END_NAMESPACE

#endif // IBANBICITEMEDIT_H

