/****************************************************************************
** Meta object code from reading C++ file 'kmymoneyviewbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/widgets/kmymoneyviewbase.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kmymoneyviewbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16KMyMoneyViewBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto KMyMoneyViewBase::qt_create_metaobjectdata<qt_meta_tag_ZN16KMyMoneyViewBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KMyMoneyViewBase",
        "requestSelectionChange",
        "",
        "SelectedObjects",
        "selection",
        "requestCustomContextMenu",
        "eMenu::Menu",
        "type",
        "QPoint",
        "pos",
        "requestActionTrigger",
        "eMenu::Action",
        "action",
        "viewStateChanged",
        "enabled",
        "customActionRequested",
        "View",
        "eView::Action",
        "requestView",
        "QWidget*",
        "viewWidget",
        "accountId",
        "journalEntryId",
        "editTransaction",
        "id",
        "isEditing",
        "updateActions",
        "selections",
        "slotSettingsChanged",
        "setOnlinePlugins",
        "QMap<QString,KMyMoneyPlugin::OnlinePlugin*>*",
        "plugins",
        "setDefaultFocus"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestSelectionChange'
        QtMocHelpers::SignalData<void(const SelectedObjects &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'requestCustomContextMenu'
        QtMocHelpers::SignalData<void(eMenu::Menu, const QPoint &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'requestActionTrigger'
        QtMocHelpers::SignalData<void(eMenu::Action)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'viewStateChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Signal 'customActionRequested'
        QtMocHelpers::SignalData<void(View, eView::Action)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 2 }, { 0x80000000 | 17, 2 },
        }}),
        // Signal 'requestView'
        QtMocHelpers::SignalData<void(QWidget *, const QString &, const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::QString, 21 }, { QMetaType::QString, 22 },
        }}),
        // Signal 'editTransaction'
        QtMocHelpers::SignalData<void(void *, bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::VoidStar, 24 }, { QMetaType::Bool, 25 },
        }}),
        // Slot 'updateActions'
        QtMocHelpers::SlotData<void(const SelectedObjects &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 27 },
        }}),
        // Slot 'slotSettingsChanged'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setOnlinePlugins'
        QtMocHelpers::SlotData<void(QMap<QString,KMyMoneyPlugin::OnlinePlugin*> *)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'setDefaultFocus'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KMyMoneyViewBase, qt_meta_tag_ZN16KMyMoneyViewBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KMyMoneyViewBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KMyMoneyViewBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KMyMoneyViewBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16KMyMoneyViewBaseE_t>.metaTypes,
    nullptr
} };

void KMyMoneyViewBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KMyMoneyViewBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestSelectionChange((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 1: _t->requestCustomContextMenu((*reinterpret_cast<std::add_pointer_t<eMenu::Menu>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 2: _t->requestActionTrigger((*reinterpret_cast<std::add_pointer_t<eMenu::Action>>(_a[1]))); break;
        case 3: _t->viewStateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->customActionRequested((*reinterpret_cast<std::add_pointer_t<View>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<eView::Action>>(_a[2]))); break;
        case 5: _t->requestView((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 6: _t->editTransaction((*reinterpret_cast<std::add_pointer_t<void*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->updateActions((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 8: _t->slotSettingsChanged(); break;
        case 9: _t->setOnlinePlugins((*reinterpret_cast<std::add_pointer_t<QMap<QString,KMyMoneyPlugin::OnlinePlugin*>*>>(_a[1]))); break;
        case 10: _t->setDefaultFocus(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyViewBase::*)(const SelectedObjects & )>(_a, &KMyMoneyViewBase::requestSelectionChange, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyViewBase::*)(eMenu::Menu , const QPoint & )>(_a, &KMyMoneyViewBase::requestCustomContextMenu, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyViewBase::*)(eMenu::Action )>(_a, &KMyMoneyViewBase::requestActionTrigger, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyViewBase::*)(bool )>(_a, &KMyMoneyViewBase::viewStateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyViewBase::*)(View , eView::Action )>(_a, &KMyMoneyViewBase::customActionRequested, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyViewBase::*)(QWidget * , const QString & , const QString & )>(_a, &KMyMoneyViewBase::requestView, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyViewBase::*)(void * , bool )>(_a, &KMyMoneyViewBase::editTransaction, 6))
            return;
    }
}

const QMetaObject *KMyMoneyViewBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KMyMoneyViewBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KMyMoneyViewBaseE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int KMyMoneyViewBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void KMyMoneyViewBase::requestSelectionChange(const SelectedObjects & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KMyMoneyViewBase::requestCustomContextMenu(eMenu::Menu _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KMyMoneyViewBase::requestActionTrigger(eMenu::Action _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KMyMoneyViewBase::viewStateChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KMyMoneyViewBase::customActionRequested(View _t1, eView::Action _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void KMyMoneyViewBase::requestView(QWidget * _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void KMyMoneyViewBase::editTransaction(void * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}
QT_WARNING_POP
