/********************************************************************************
** Form generated from reading UI file 'kselectdatabasedlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KSELECTDATABASEDLG_H
#define UI_KSELECTDATABASEDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "klineedit.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_KSelectDatabaseDlg
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox6;
    QVBoxLayout *verticalLayout;
    QPlainTextEdit *plainTextEdit;
    QFormLayout *formLayout_2;
    QLabel *label_6;
    QComboBox *databaseTypeCombo;
    QLabel *label;
    KUrlRequester *urlSqlite;
    QWidget *databaseFields;
    QFormLayout *formLayout;
    QLabel *label_2;
    KLineEdit *textDbName;
    QLabel *label_3;
    KLineEdit *textHostName;
    QLabel *label_4;
    KLineEdit *textUserName;
    QLabel *label_5;
    KLineEdit *textPassword;
    QCheckBox *checkPreLoad;
    QHBoxLayout *horizontalLayout;
    QDialogButtonBox *buttonBox;

    void setupUi(QWidget *KSelectDatabaseDlg)
    {
        if (KSelectDatabaseDlg->objectName().isEmpty())
            KSelectDatabaseDlg->setObjectName("KSelectDatabaseDlg");
        KSelectDatabaseDlg->resize(551, 451);
        KSelectDatabaseDlg->setMinimumSize(QSize(0, 451));
        KSelectDatabaseDlg->setBaseSize(QSize(518, 679));
        verticalLayout_2 = new QVBoxLayout(KSelectDatabaseDlg);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox6 = new QGroupBox(KSelectDatabaseDlg);
        groupBox6->setObjectName("groupBox6");
        groupBox6->setMinimumSize(QSize(0, 111));
        QPalette palette;
        QBrush brush(QColor(255, 0, 0, 255));
        brush.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::WindowText, brush);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::WindowText, brush);
        QBrush brush1(QColor(126, 125, 124, 255));
        brush1.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::WindowText, brush1);
        groupBox6->setPalette(palette);
        verticalLayout = new QVBoxLayout(groupBox6);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        plainTextEdit = new QPlainTextEdit(groupBox6);
        plainTextEdit->setObjectName("plainTextEdit");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(plainTextEdit->sizePolicy().hasHeightForWidth());
        plainTextEdit->setSizePolicy(sizePolicy);
        QPalette palette1;
        palette1.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::WindowText, brush);
        palette1.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Text, brush);
        palette1.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::WindowText, brush);
        palette1.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Text, brush);
        palette1.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::WindowText, brush1);
        palette1.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Text, brush1);
        plainTextEdit->setPalette(palette1);
        QFont font;
        font.setPointSize(9);
        plainTextEdit->setFont(font);
        plainTextEdit->setAcceptDrops(false);
        plainTextEdit->setUndoRedoEnabled(false);
        plainTextEdit->setReadOnly(true);

        verticalLayout->addWidget(plainTextEdit);


        verticalLayout_2->addWidget(groupBox6);

        formLayout_2 = new QFormLayout();
        formLayout_2->setSpacing(6);
        formLayout_2->setObjectName("formLayout_2");
        label_6 = new QLabel(KSelectDatabaseDlg);
        label_6->setObjectName("label_6");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, label_6);

        databaseTypeCombo = new QComboBox(KSelectDatabaseDlg);
        databaseTypeCombo->setObjectName("databaseTypeCombo");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, databaseTypeCombo);

        label = new QLabel(KSelectDatabaseDlg);
        label->setObjectName("label");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, label);

        urlSqlite = new KUrlRequester(KSelectDatabaseDlg);
        urlSqlite->setObjectName("urlSqlite");
        urlSqlite->setEnabled(false);
        urlSqlite->setMinimumSize(QSize(131, 0));

        formLayout_2->setWidget(1, QFormLayout::ItemRole::FieldRole, urlSqlite);

        databaseFields = new QWidget(KSelectDatabaseDlg);
        databaseFields->setObjectName("databaseFields");
        formLayout = new QFormLayout(databaseFields);
        formLayout->setSpacing(6);
        formLayout->setContentsMargins(0, 0, 0, 0);
        formLayout->setObjectName("formLayout");
        label_2 = new QLabel(databaseFields);
        label_2->setObjectName("label_2");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label_2);

        textDbName = new KLineEdit(databaseFields);
        textDbName->setObjectName("textDbName");
        textDbName->setEnabled(false);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, textDbName);

        label_3 = new QLabel(databaseFields);
        label_3->setObjectName("label_3");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_3);

        textHostName = new KLineEdit(databaseFields);
        textHostName->setObjectName("textHostName");
        textHostName->setEnabled(false);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, textHostName);

        label_4 = new QLabel(databaseFields);
        label_4->setObjectName("label_4");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label_4);

        textUserName = new KLineEdit(databaseFields);
        textUserName->setObjectName("textUserName");
        textUserName->setEnabled(false);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, textUserName);

        label_5 = new QLabel(databaseFields);
        label_5->setObjectName("label_5");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, label_5);

        textPassword = new KLineEdit(databaseFields);
        textPassword->setObjectName("textPassword");
        textPassword->setEnabled(false);
        textPassword->setEchoMode(QLineEdit::Password);

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, textPassword);


        formLayout_2->setWidget(2, QFormLayout::ItemRole::SpanningRole, databaseFields);


        verticalLayout_2->addLayout(formLayout_2);

        checkPreLoad = new QCheckBox(KSelectDatabaseDlg);
        checkPreLoad->setObjectName("checkPreLoad");

        verticalLayout_2->addWidget(checkPreLoad);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        buttonBox = new QDialogButtonBox(KSelectDatabaseDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Ok);

        horizontalLayout->addWidget(buttonBox);


        verticalLayout_2->addLayout(horizontalLayout);

#if QT_CONFIG(shortcut)
        label->setBuddy(urlSqlite);
        label_2->setBuddy(textDbName);
        label_3->setBuddy(textHostName);
        label_4->setBuddy(textUserName);
        label_5->setBuddy(textPassword);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KSelectDatabaseDlg);

        QMetaObject::connectSlotsByName(KSelectDatabaseDlg);
    } // setupUi

    void retranslateUi(QWidget *KSelectDatabaseDlg)
    {
        KSelectDatabaseDlg->setWindowTitle(tr2i18n("KMyMoney - Select Database", nullptr));
        groupBox6->setTitle(tr2i18n("PLEASE NOTE", nullptr));
        plainTextEdit->setPlainText(tr2i18n("This software is still in an experimental stage, and there are still some known response time problems. Please be patient; we are working on them.\n"
"As always, please make sure you have adequate backups of your data.", nullptr));
        label_6->setText(tr2i18n("Database Type", nullptr));
        label->setText(tr2i18n("Fi&le", nullptr));
        label_2->setText(tr2i18n("Database &Name", nullptr));
        label_3->setText(tr2i18n("Host &Name", nullptr));
        label_4->setText(tr2i18n("&Username", nullptr));
        label_5->setText(tr2i18n("&Password", nullptr));
        checkPreLoad->setText(tr2i18n("Preload &all data", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KSelectDatabaseDlg: public Ui_KSelectDatabaseDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KSELECTDATABASEDLG_H

