/********************************************************************************
** Form generated from reading UI file 'kpayeeidentifierview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPAYEEIDENTIFIERVIEW_H
#define UI_KPAYEEIDENTIFIERVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KPayeeIdentifierView
{
public:
    QHBoxLayout *horizontalLayout;
    QListView *view;
    QVBoxLayout *verticalLayout;
    QPushButton *buttonRemove;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KPayeeIdentifierView)
    {
        if (KPayeeIdentifierView->objectName().isEmpty())
            KPayeeIdentifierView->setObjectName("KPayeeIdentifierView");
        KPayeeIdentifierView->resize(647, 583);
        horizontalLayout = new QHBoxLayout(KPayeeIdentifierView);
        horizontalLayout->setObjectName("horizontalLayout");
        view = new QListView(KPayeeIdentifierView);
        view->setObjectName("view");
        view->setSelectionMode(QAbstractItemView::ExtendedSelection);
        view->setSelectionBehavior(QAbstractItemView::SelectRows);
        view->setUniformItemSizes(false);
        view->setSelectionRectVisible(true);

        horizontalLayout->addWidget(view);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        buttonRemove = new QPushButton(KPayeeIdentifierView);
        buttonRemove->setObjectName("buttonRemove");

        verticalLayout->addWidget(buttonRemove);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout);


        retranslateUi(KPayeeIdentifierView);
        QObject::connect(buttonRemove, SIGNAL(clicked()), KPayeeIdentifierView, SLOT(removeSelected()));

        QMetaObject::connectSlotsByName(KPayeeIdentifierView);
    } // setupUi

    void retranslateUi(QWidget *KPayeeIdentifierView)
    {
        buttonRemove->setText(tr2i18n("Remove", nullptr));
        (void)KPayeeIdentifierView;
    } // retranslateUi

};

namespace Ui {
    class KPayeeIdentifierView: public Ui_KPayeeIdentifierView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KPAYEEIDENTIFIERVIEW_H

