/********************************************************************************
** Form generated from reading UI file 'reporttabrowcolpivot.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REPORTTABROWCOLPIVOT_H
#define UI_REPORTTABROWCOLPIVOT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "kmymoneygeneralcombo.h"

QT_BEGIN_NAMESPACE

class Ui_ReportTabRowColPivot
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QGridLayout *m_gridLayout;
    QLabel *m_movingAverageLabel;
    QSpinBox *m_movingAverageDays;
    QLabel *m_rowsLabel;
    QCheckBox *m_checkTotalRow;
    KComboBox *m_comboRows;
    QCheckBox *m_checkTotalColumn;
    QFrame *m_budgetFrame;
    QHBoxLayout *hboxLayout1;
    QLabel *textLabel1;
    KMyMoneyGeneralCombo *m_comboBudget;
    QLabel *textLabel6;
    KComboBox *m_comboDetail;
    QCheckBox *m_propagateRemainder;
    QSpacerItem *spacer12_2;
    QCheckBox *m_checkScheduled;
    QCheckBox *m_checkTransfers;
    QCheckBox *m_checkUnused;
    QSpacerItem *spacer13;

    void setupUi(QWidget *ReportTabRowColPivot)
    {
        if (ReportTabRowColPivot->objectName().isEmpty())
            ReportTabRowColPivot->setObjectName("ReportTabRowColPivot");
        ReportTabRowColPivot->resize(599, 342);
        vboxLayout = new QVBoxLayout(ReportTabRowColPivot);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_gridLayout = new QGridLayout();
        m_gridLayout->setSpacing(6);
        m_gridLayout->setObjectName("m_gridLayout");
        m_movingAverageLabel = new QLabel(ReportTabRowColPivot);
        m_movingAverageLabel->setObjectName("m_movingAverageLabel");
        m_movingAverageLabel->setWordWrap(false);

        m_gridLayout->addWidget(m_movingAverageLabel, 7, 0, 1, 1);

        m_movingAverageDays = new QSpinBox(ReportTabRowColPivot);
        m_movingAverageDays->setObjectName("m_movingAverageDays");
        m_movingAverageDays->setMinimum(1);
        m_movingAverageDays->setMaximum(366);

        m_gridLayout->addWidget(m_movingAverageDays, 7, 1, 1, 1);

        m_rowsLabel = new QLabel(ReportTabRowColPivot);
        m_rowsLabel->setObjectName("m_rowsLabel");
        m_rowsLabel->setWordWrap(false);

        m_gridLayout->addWidget(m_rowsLabel, 5, 0, 1, 1);

        m_checkTotalRow = new QCheckBox(ReportTabRowColPivot);
        m_checkTotalRow->setObjectName("m_checkTotalRow");

        m_gridLayout->addWidget(m_checkTotalRow, 4, 1, 1, 1);

        m_comboRows = new KComboBox(ReportTabRowColPivot);
        m_comboRows->addItem(QString());
        m_comboRows->addItem(QString());
        m_comboRows->setObjectName("m_comboRows");

        m_gridLayout->addWidget(m_comboRows, 5, 1, 1, 1);

        m_checkTotalColumn = new QCheckBox(ReportTabRowColPivot);
        m_checkTotalColumn->setObjectName("m_checkTotalColumn");
        m_checkTotalColumn->setEnabled(false);

        m_gridLayout->addWidget(m_checkTotalColumn, 3, 1, 1, 1);

        m_budgetFrame = new QFrame(ReportTabRowColPivot);
        m_budgetFrame->setObjectName("m_budgetFrame");
        m_budgetFrame->setEnabled(false);
        m_budgetFrame->setFrameShape(QFrame::NoFrame);
        m_budgetFrame->setFrameShadow(QFrame::Raised);
        hboxLayout1 = new QHBoxLayout(m_budgetFrame);
        hboxLayout1->setSpacing(6);
        hboxLayout1->setContentsMargins(11, 11, 11, 11);
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);

        m_gridLayout->addWidget(m_budgetFrame, 0, 0, 1, 2);

        textLabel1 = new QLabel(ReportTabRowColPivot);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        m_gridLayout->addWidget(textLabel1, 1, 0, 1, 1);

        m_comboBudget = new KMyMoneyGeneralCombo(ReportTabRowColPivot);
        m_comboBudget->setObjectName("m_comboBudget");

        m_gridLayout->addWidget(m_comboBudget, 1, 1, 1, 1);

        textLabel6 = new QLabel(ReportTabRowColPivot);
        textLabel6->setObjectName("textLabel6");
        textLabel6->setWordWrap(false);

        m_gridLayout->addWidget(textLabel6, 6, 0, 1, 1);

        m_comboDetail = new KComboBox(ReportTabRowColPivot);
        m_comboDetail->addItem(QString());
        m_comboDetail->addItem(QString());
        m_comboDetail->addItem(QString());
        m_comboDetail->addItem(QString());
        m_comboDetail->setObjectName("m_comboDetail");

        m_gridLayout->addWidget(m_comboDetail, 6, 1, 1, 1);

        m_propagateRemainder = new QCheckBox(ReportTabRowColPivot);
        m_propagateRemainder->setObjectName("m_propagateRemainder");
        m_propagateRemainder->setEnabled(false);

        m_gridLayout->addWidget(m_propagateRemainder, 2, 1, 1, 1);


        hboxLayout->addLayout(m_gridLayout);

        spacer12_2 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer12_2);


        vboxLayout->addLayout(hboxLayout);

        m_checkScheduled = new QCheckBox(ReportTabRowColPivot);
        m_checkScheduled->setObjectName("m_checkScheduled");

        vboxLayout->addWidget(m_checkScheduled);

        m_checkTransfers = new QCheckBox(ReportTabRowColPivot);
        m_checkTransfers->setObjectName("m_checkTransfers");

        vboxLayout->addWidget(m_checkTransfers);

        m_checkUnused = new QCheckBox(ReportTabRowColPivot);
        m_checkUnused->setObjectName("m_checkUnused");

        vboxLayout->addWidget(m_checkUnused);

        spacer13 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer13);

        QWidget::setTabOrder(m_checkTotalColumn, m_checkTotalRow);
        QWidget::setTabOrder(m_checkTotalRow, m_comboRows);
        QWidget::setTabOrder(m_comboRows, m_comboDetail);
        QWidget::setTabOrder(m_comboDetail, m_movingAverageDays);
        QWidget::setTabOrder(m_movingAverageDays, m_checkScheduled);
        QWidget::setTabOrder(m_checkScheduled, m_checkTransfers);
        QWidget::setTabOrder(m_checkTransfers, m_checkUnused);

        retranslateUi(ReportTabRowColPivot);

        QMetaObject::connectSlotsByName(ReportTabRowColPivot);
    } // setupUi

    void retranslateUi(QWidget *ReportTabRowColPivot)
    {
        ReportTabRowColPivot->setWindowTitle(tr2i18n("Rows/Columns Tab", nullptr));
#if QT_CONFIG(tooltip)
        ReportTabRowColPivot->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        ReportTabRowColPivot->setWhatsThis(tr2i18n("<p>On this tab, you configure how you would like the rows and columns to be selected and organized.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_movingAverageLabel->setText(tr2i18n("Average days", nullptr));
        m_rowsLabel->setText(tr2i18n("Rows", nullptr));
        m_checkTotalRow->setText(tr2i18n("Show totals row", nullptr));
        m_comboRows->setItemText(0, tr2i18n("Income & Expenses", nullptr));
        m_comboRows->setItemText(1, tr2i18n("Assets & Liabilities", nullptr));

#if QT_CONFIG(tooltip)
        m_comboRows->setToolTip(tr2i18n("<p>Choose what kind of accounts to display as the rows of this report.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkTotalColumn->setText(tr2i18n("Show totals column", nullptr));
        textLabel1->setText(tr2i18n("Budget", nullptr));
        textLabel6->setText(tr2i18n("Detail", nullptr));
        m_comboDetail->setItemText(0, tr2i18n("All", "@item all accounts"));
        m_comboDetail->setItemText(1, tr2i18n("Top-Level", nullptr));
        m_comboDetail->setItemText(2, tr2i18n("Groups", nullptr));
        m_comboDetail->setItemText(3, tr2i18n("Totals", nullptr));

#if QT_CONFIG(tooltip)
        m_comboDetail->setToolTip(tr2i18n("<p>Choose what kind of accounts to display as the rows of this report.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_propagateRemainder->setToolTip(tr2i18n("Check this to propagate the difference of one period to the budget of the next period.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_propagateRemainder->setText(tr2i18n("Propagate difference of budget to next period", nullptr));
        m_checkScheduled->setText(tr2i18n("Include scheduled transactions", nullptr));
        m_checkTransfers->setText(tr2i18n("Include transfers", nullptr));
        m_checkUnused->setText(tr2i18n("Include unused accounts/categories", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ReportTabRowColPivot: public Ui_ReportTabRowColPivot {};
} // namespace Ui

QT_END_NAMESPACE

#endif // REPORTTABROWCOLPIVOT_H

