/********************************************************************************
** Form generated from reading UI file 'ksettingsschedules.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KSETTINGSSCHEDULES_H
#define UI_KSETTINGSSCHEDULES_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KSettingsSchedules
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox6;
    QVBoxLayout *vboxLayout1;
    QCheckBox *kcfg_CheckSchedule;
    QFrame *m_dayCountFrame;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel3;
    QSpinBox *kcfg_CheckSchedulePreview;
    QGroupBox *processingDaysBox;
    QHBoxLayout *processingDaysLayout;
    QLabel *calendarLabel;
    KLineEdit *kcfg_HolidayRegion;
    KComboBox *m_holidayRegion;
    QSpacerItem *processingDaysSpacer;
    QHBoxLayout *hboxLayout1;
    QLabel *textLabel1;
    QSpinBox *kcfg_SchedulePreview;
    QSpacerItem *spacer2;
    QSpacerItem *spacer10;

    void setupUi(QWidget *KSettingsSchedules)
    {
        if (KSettingsSchedules->objectName().isEmpty())
            KSettingsSchedules->setObjectName("KSettingsSchedules");
        KSettingsSchedules->resize(482, 236);
        vboxLayout = new QVBoxLayout(KSettingsSchedules);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        groupBox6 = new QGroupBox(KSettingsSchedules);
        groupBox6->setObjectName("groupBox6");
        vboxLayout1 = new QVBoxLayout(groupBox6);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        kcfg_CheckSchedule = new QCheckBox(groupBox6);
        kcfg_CheckSchedule->setObjectName("kcfg_CheckSchedule");

        vboxLayout1->addWidget(kcfg_CheckSchedule);

        m_dayCountFrame = new QFrame(groupBox6);
        m_dayCountFrame->setObjectName("m_dayCountFrame");
        m_dayCountFrame->setEnabled(false);
        m_dayCountFrame->setFrameShape(QFrame::StyledPanel);
        m_dayCountFrame->setFrameShadow(QFrame::Raised);
        hboxLayout = new QHBoxLayout(m_dayCountFrame);
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName("hboxLayout");
        textLabel3 = new QLabel(m_dayCountFrame);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        hboxLayout->addWidget(textLabel3);

        kcfg_CheckSchedulePreview = new QSpinBox(m_dayCountFrame);
        kcfg_CheckSchedulePreview->setObjectName("kcfg_CheckSchedulePreview");
        kcfg_CheckSchedulePreview->setMaximum(999);

        hboxLayout->addWidget(kcfg_CheckSchedulePreview);


        vboxLayout1->addWidget(m_dayCountFrame);


        vboxLayout->addWidget(groupBox6);

        processingDaysBox = new QGroupBox(KSettingsSchedules);
        processingDaysBox->setObjectName("processingDaysBox");
        processingDaysLayout = new QHBoxLayout(processingDaysBox);
        processingDaysLayout->setSpacing(6);
        processingDaysLayout->setContentsMargins(11, 11, 11, 11);
        processingDaysLayout->setObjectName("processingDaysLayout");
        calendarLabel = new QLabel(processingDaysBox);
        calendarLabel->setObjectName("calendarLabel");
        calendarLabel->setWordWrap(false);

        processingDaysLayout->addWidget(calendarLabel);

        kcfg_HolidayRegion = new KLineEdit(processingDaysBox);
        kcfg_HolidayRegion->setObjectName("kcfg_HolidayRegion");

        processingDaysLayout->addWidget(kcfg_HolidayRegion);

        m_holidayRegion = new KComboBox(processingDaysBox);
        m_holidayRegion->addItem(QString());
        m_holidayRegion->setObjectName("m_holidayRegion");

        processingDaysLayout->addWidget(m_holidayRegion);

        processingDaysSpacer = new QSpacerItem(282, 17, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        processingDaysLayout->addItem(processingDaysSpacer);


        vboxLayout->addWidget(processingDaysBox);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        textLabel1 = new QLabel(KSettingsSchedules);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        hboxLayout1->addWidget(textLabel1);

        kcfg_SchedulePreview = new QSpinBox(KSettingsSchedules);
        kcfg_SchedulePreview->setObjectName("kcfg_SchedulePreview");
        kcfg_SchedulePreview->setMaximum(999);

        hboxLayout1->addWidget(kcfg_SchedulePreview);

        spacer2 = new QSpacerItem(41, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacer2);


        vboxLayout->addLayout(hboxLayout1);

        spacer10 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer10);


        retranslateUi(KSettingsSchedules);
        QObject::connect(kcfg_CheckSchedule, &QCheckBox::toggled, m_dayCountFrame, &QFrame::setEnabled);

        QMetaObject::connectSlotsByName(KSettingsSchedules);
    } // setupUi

    void retranslateUi(QWidget *KSettingsSchedules)
    {
        KSettingsSchedules->setWindowTitle(tr2i18n("Schedule Settings", nullptr));
        groupBox6->setTitle(tr2i18n("Startup options", nullptr));
        kcfg_CheckSchedule->setText(tr2i18n("Check schedules on startup", nullptr));
        textLabel3->setText(tr2i18n("Enter transactions this number of days in advance", nullptr));
        processingDaysBox->setTitle(tr2i18n("Processing Days", nullptr));
        calendarLabel->setText(tr2i18n("Use holiday calendar for region", nullptr));
        m_holidayRegion->setItemText(0, tr2i18n("(None)", "@item no holiday region selected"));

        textLabel1->setText(tr2i18n("Number of days to preview schedules in ledger", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KSettingsSchedules: public Ui_KSettingsSchedules {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KSETTINGSSCHEDULES_H

