/****************************************************************************
** Meta object code from reading C++ file 'sepacredittransferedit.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../kmymoney-5.2.2/kmymoney/plugins/onlinetasks/sepa/ui/sepacredittransferedit.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'sepacredittransferedit.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22sepaCreditTransferEditE_t {};
} // unnamed namespace

template <> constexpr inline auto sepaCreditTransferEdit::qt_create_metaobjectdata<qt_meta_tag_ZN22sepaCreditTransferEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "sepaCreditTransferEdit",
        "readOnlyChanged",
        "",
        "setOnlineJob",
        "onlineJobTyped<sepaOnlineTransfer>",
        "job",
        "onlineJob",
        "setOriginAccount",
        "accountId",
        "setReadOnly",
        "updateSettings",
        "updateEveryStatus",
        "purposeChanged",
        "beneficiaryIbanChanged",
        "iban",
        "beneficiaryBicChanged",
        "bic",
        "beneficiaryNameChanged",
        "name",
        "valueChanged",
        "endToEndReferenceChanged",
        "reference",
        "requiredFieldsCompleted",
        "status",
        "readOnly"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'readOnlyChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setOnlineJob'
        QtMocHelpers::SlotData<void(const onlineJobTyped<sepaOnlineTransfer> &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'setOnlineJob'
        QtMocHelpers::SlotData<bool(const onlineJob &)>(3, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 6, 5 },
        }}),
        // Slot 'setOriginAccount'
        QtMocHelpers::SlotData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'setReadOnly'
        QtMocHelpers::SlotData<void(const bool &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'updateSettings'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateEveryStatus'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'purposeChanged'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'beneficiaryIbanChanged'
        QtMocHelpers::SlotData<void(const QString &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'beneficiaryBicChanged'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'beneficiaryNameChanged'
        QtMocHelpers::SlotData<void(const QString &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'valueChanged'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'endToEndReferenceChanged'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'requiredFieldsCompleted'
        QtMocHelpers::SlotData<void(const bool &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Slot 'requiredFieldsCompleted'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'readOnly'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'job'
        QtMocHelpers::PropertyData<onlineJob>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<sepaCreditTransferEdit, qt_meta_tag_ZN22sepaCreditTransferEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject sepaCreditTransferEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<IonlineJobEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22sepaCreditTransferEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22sepaCreditTransferEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22sepaCreditTransferEditE_t>.metaTypes,
    nullptr
} };

void sepaCreditTransferEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<sepaCreditTransferEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->readOnlyChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->setOnlineJob((*reinterpret_cast<std::add_pointer_t<onlineJobTyped<sepaOnlineTransfer>>>(_a[1]))); break;
        case 2: { bool _r = _t->setOnlineJob((*reinterpret_cast<std::add_pointer_t<onlineJob>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->setOriginAccount((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->setReadOnly((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->updateSettings(); break;
        case 6: _t->updateEveryStatus(); break;
        case 7: _t->purposeChanged(); break;
        case 8: _t->beneficiaryIbanChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->beneficiaryBicChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->beneficiaryNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->valueChanged(); break;
        case 12: _t->endToEndReferenceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->requiredFieldsCompleted((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->requiredFieldsCompleted(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< onlineJob >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (sepaCreditTransferEdit::*)(bool )>(_a, &sepaCreditTransferEdit::readOnlyChanged, 0))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< onlineJob >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isReadOnly(); break;
        case 1: *reinterpret_cast<onlineJob*>(_v) = _t->getOnlineJob(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setReadOnly(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setOnlineJob(*reinterpret_cast<onlineJob*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *sepaCreditTransferEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *sepaCreditTransferEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22sepaCreditTransferEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "org.kmymoney.plugin.ionlinejobedit"))
        return static_cast< IonlineJobEdit*>(this);
    return IonlineJobEdit::qt_metacast(_clname);
}

int sepaCreditTransferEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = IonlineJobEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void sepaCreditTransferEdit::readOnlyChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
