/********************************************************************************
** Form generated from reading UI file 'kgncpricesourcedlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KGNCPRICESOURCEDLG_H
#define UI_KGNCPRICESOURCEDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KGncPriceSourceDlg
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *textStockName;
    QLabel *textGncSource;
    QLabel *textLabel4;
    QGroupBox *buttonsSourceGroupBox;
    QVBoxLayout *vboxLayout;
    QRadioButton *buttonNoSource;
    QRadioButton *buttonSelectSource;
    QListWidget *listKnownSource;
    QRadioButton *buttonUserSource;
    KLineEdit *lineUserSource;
    QCheckBox *checkAlwaysUse;
    QDialogButtonBox *buttonBox;
    QButtonGroup *buttonsSource;

    void setupUi(QDialog *KGncPriceSourceDlg)
    {
        if (KGncPriceSourceDlg->objectName().isEmpty())
            KGncPriceSourceDlg->setObjectName("KGncPriceSourceDlg");
        KGncPriceSourceDlg->resize(619, 480);
        KGncPriceSourceDlg->setMinimumSize(QSize(619, 480));
        verticalLayout = new QVBoxLayout(KGncPriceSourceDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        textStockName = new QLabel(KGncPriceSourceDlg);
        textStockName->setObjectName("textStockName");
        textStockName->setWordWrap(false);

        verticalLayout->addWidget(textStockName);

        textGncSource = new QLabel(KGncPriceSourceDlg);
        textGncSource->setObjectName("textGncSource");
        textGncSource->setWordWrap(false);

        verticalLayout->addWidget(textGncSource);

        textLabel4 = new QLabel(KGncPriceSourceDlg);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        verticalLayout->addWidget(textLabel4);

        buttonsSourceGroupBox = new QGroupBox(KGncPriceSourceDlg);
        buttonsSourceGroupBox->setObjectName("buttonsSourceGroupBox");
        vboxLayout = new QVBoxLayout(buttonsSourceGroupBox);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        buttonNoSource = new QRadioButton(buttonsSourceGroupBox);
        buttonsSource = new QButtonGroup(KGncPriceSourceDlg);
        buttonsSource->setObjectName("buttonsSource");
        buttonsSource->addButton(buttonNoSource);
        buttonNoSource->setObjectName("buttonNoSource");

        vboxLayout->addWidget(buttonNoSource);

        buttonSelectSource = new QRadioButton(buttonsSourceGroupBox);
        buttonsSource->addButton(buttonSelectSource);
        buttonSelectSource->setObjectName("buttonSelectSource");

        vboxLayout->addWidget(buttonSelectSource);

        listKnownSource = new QListWidget(buttonsSourceGroupBox);
        listKnownSource->setObjectName("listKnownSource");

        vboxLayout->addWidget(listKnownSource);

        buttonUserSource = new QRadioButton(buttonsSourceGroupBox);
        buttonsSource->addButton(buttonUserSource);
        buttonUserSource->setObjectName("buttonUserSource");

        vboxLayout->addWidget(buttonUserSource);

        lineUserSource = new KLineEdit(buttonsSourceGroupBox);
        lineUserSource->setObjectName("lineUserSource");

        vboxLayout->addWidget(lineUserSource);


        verticalLayout->addWidget(buttonsSourceGroupBox);

        checkAlwaysUse = new QCheckBox(KGncPriceSourceDlg);
        checkAlwaysUse->setObjectName("checkAlwaysUse");

        verticalLayout->addWidget(checkAlwaysUse);

        buttonBox = new QDialogButtonBox(KGncPriceSourceDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(KGncPriceSourceDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KGncPriceSourceDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KGncPriceSourceDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KGncPriceSourceDlg);
    } // setupUi

    void retranslateUi(QDialog *KGncPriceSourceDlg)
    {
        KGncPriceSourceDlg->setWindowTitle(tr2i18n("Online Quotes - Select price source", nullptr));
        textStockName->setText(QString());
        textGncSource->setText(QString());
        textLabel4->setText(tr2i18n("This price source is not known to KMyMoney. Please select an option below.", nullptr));
        buttonsSourceGroupBox->setTitle(QString());
        buttonNoSource->setText(tr2i18n("Do &not perform online quotes for this investment", nullptr));
        buttonSelectSource->setText(tr2i18n("Select a &known KMyMoney source from the list below", nullptr));
        buttonUserSource->setText(tr2i18n("Use the fo&llowing name for the price source.\n"
"(Click Help for further information.)", nullptr));
        checkAlwaysUse->setText(tr2i18n("Always use this selection for this price source.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KGncPriceSourceDlg: public Ui_KGncPriceSourceDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KGNCPRICESOURCEDLG_H

