/********************************************************************************
** Form generated from reading UI file 'kintropage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KINTROPAGE_H
#define UI_KINTROPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KIntroPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *textLabel1;
    QLabel *qLabel1;
    QSpacerItem *spacer2;

    void setupUi(QWidget *KIntroPage)
    {
        if (KIntroPage->objectName().isEmpty())
            KIntroPage->setObjectName("KIntroPage");
        KIntroPage->resize(600, 415);
        vboxLayout = new QVBoxLayout(KIntroPage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        textLabel1 = new QLabel(KIntroPage);
        textLabel1->setObjectName("textLabel1");
        QFont font;
        font.setPointSize(16);
        font.setBold(true);
        textLabel1->setFont(font);
        textLabel1->setAlignment(Qt::AlignCenter);
        textLabel1->setWordWrap(false);

        vboxLayout->addWidget(textLabel1);

        qLabel1 = new QLabel(KIntroPage);
        qLabel1->setObjectName("qLabel1");
        qLabel1->setWordWrap(true);

        vboxLayout->addWidget(qLabel1);

        spacer2 = new QSpacerItem(20, 41, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer2);


        retranslateUi(KIntroPage);

        QMetaObject::connectSlotsByName(KIntroPage);
    } // setupUi

    void retranslateUi(QWidget *KIntroPage)
    {
        textLabel1->setText(tr2i18n("Welcome to KMyMoney!", nullptr));
        qLabel1->setText(tr2i18n("You have started KMyMoney for the first time on this computer. Before this program can help you to manage your finances, it needs to collect some data from you.<p>\n"
"\n"
"On the next few pages you will be guided through the steps necessary to setup the program. This will happen only once.", nullptr));
        (void)KIntroPage;
    } // retranslateUi

};

namespace Ui {
    class KIntroPage: public Ui_KIntroPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KINTROPAGE_H

