/********************************************************************************
** Form generated from reading UI file 'kimportdlgdecl.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KIMPORTDLGDECL_H
#define UI_KIMPORTDLGDECL_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KImportDlgDecl
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *qLabel1;
    QHBoxLayout *horizontalLayout;
    QLabel *lblImportFile;
    KLineEdit *m_qlineeditFile;
    QPushButton *m_qbuttonBrowse;
    QGroupBox *m_optionGroupBox;
    QGridLayout *gridLayout;
    QLabel *textLabel1;
    QLabel *TextLabel3_2;
    KComboBox *m_typeComboBox;
    KComboBox *m_profileComboBox;
    QSpacerItem *verticalSpacer;
    QFrame *Line1;
    QDialogButtonBox *m_buttonBox;

    void setupUi(QDialog *KImportDlgDecl)
    {
        if (KImportDlgDecl->objectName().isEmpty())
            KImportDlgDecl->setObjectName("KImportDlgDecl");
        KImportDlgDecl->resize(595, 464);
        KImportDlgDecl->setSizeGripEnabled(true);
        KImportDlgDecl->setModal(true);
        verticalLayout = new QVBoxLayout(KImportDlgDecl);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        qLabel1 = new QLabel(KImportDlgDecl);
        qLabel1->setObjectName("qLabel1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(qLabel1->sizePolicy().hasHeightForWidth());
        qLabel1->setSizePolicy(sizePolicy);
        qLabel1->setFocusPolicy(Qt::NoFocus);
        qLabel1->setWordWrap(true);

        verticalLayout->addWidget(qLabel1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        lblImportFile = new QLabel(KImportDlgDecl);
        lblImportFile->setObjectName("lblImportFile");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(lblImportFile->sizePolicy().hasHeightForWidth());
        lblImportFile->setSizePolicy(sizePolicy1);
        lblImportFile->setWordWrap(false);

        horizontalLayout->addWidget(lblImportFile);

        m_qlineeditFile = new KLineEdit(KImportDlgDecl);
        m_qlineeditFile->setObjectName("m_qlineeditFile");

        horizontalLayout->addWidget(m_qlineeditFile);

        m_qbuttonBrowse = new QPushButton(KImportDlgDecl);
        m_qbuttonBrowse->setObjectName("m_qbuttonBrowse");

        horizontalLayout->addWidget(m_qbuttonBrowse);


        verticalLayout->addLayout(horizontalLayout);

        m_optionGroupBox = new QGroupBox(KImportDlgDecl);
        m_optionGroupBox->setObjectName("m_optionGroupBox");
        gridLayout = new QGridLayout(m_optionGroupBox);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        textLabel1 = new QLabel(m_optionGroupBox);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 1, 0, 1, 1);

        TextLabel3_2 = new QLabel(m_optionGroupBox);
        TextLabel3_2->setObjectName("TextLabel3_2");
        TextLabel3_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel3_2, 0, 0, 1, 1);

        m_typeComboBox = new KComboBox(m_optionGroupBox);
        m_typeComboBox->addItem(QString());
        m_typeComboBox->addItem(QString());
        m_typeComboBox->setObjectName("m_typeComboBox");

        gridLayout->addWidget(m_typeComboBox, 1, 1, 1, 1);

        m_profileComboBox = new KComboBox(m_optionGroupBox);
        m_profileComboBox->setObjectName("m_profileComboBox");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(m_profileComboBox->sizePolicy().hasHeightForWidth());
        m_profileComboBox->setSizePolicy(sizePolicy2);
        m_profileComboBox->setMinimumSize(QSize(200, 0));
        m_profileComboBox->setDuplicatesEnabled(false);

        gridLayout->addWidget(m_profileComboBox, 0, 1, 1, 1);


        verticalLayout->addWidget(m_optionGroupBox);

        verticalSpacer = new QSpacerItem(20, 47, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        Line1 = new QFrame(KImportDlgDecl);
        Line1->setObjectName("Line1");
        Line1->setFrameShape(QFrame::HLine);
        Line1->setFrameShadow(QFrame::Raised);

        verticalLayout->addWidget(Line1);

        m_buttonBox = new QDialogButtonBox(KImportDlgDecl);
        m_buttonBox->setObjectName("m_buttonBox");
        m_buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(m_buttonBox);

        m_optionGroupBox->raise();
        qLabel1->raise();
        Line1->raise();
        m_buttonBox->raise();

        retranslateUi(KImportDlgDecl);

        QMetaObject::connectSlotsByName(KImportDlgDecl);
    } // setupUi

    void retranslateUi(QDialog *KImportDlgDecl)
    {
        KImportDlgDecl->setWindowTitle(tr2i18n("QIF Import", nullptr));
        qLabel1->setText(tr2i18n("<p>From this dialog you are able to import transactions from a Quicken<b>&trade;</b> compatible file, (known as a QIF file, because of the extension).</p>\n"
"<p>Please enter the path to the QIF file or select it by clicking on the Browse button. Once you have the file's path press the Import button and KMyMoney will import all the transactions, categories and payees it finds.</p>", nullptr));
        lblImportFile->setText(tr2i18n("QIF File to Import:", nullptr));
        m_qbuttonBrowse->setText(tr2i18n("&Browse...", nullptr));
        m_optionGroupBox->setTitle(tr2i18n("Import options", nullptr));
        textLabel1->setText(tr2i18n("Source of QIF", nullptr));
        TextLabel3_2->setText(tr2i18n("QIF Profile", nullptr));
        m_typeComboBox->setItemText(0, tr2i18n("Bank statement", nullptr));
        m_typeComboBox->setItemText(1, tr2i18n("Other application", nullptr));

#if QT_CONFIG(whatsthis)
        m_typeComboBox->setWhatsThis(tr2i18n("Select <b>Bank statement</b> turns on automatic category matching which is turned off in case of <b>Other application</b>. Use the latter if you import files from other Personal Finance Management software.", nullptr));
#endif // QT_CONFIG(whatsthis)
    } // retranslateUi

};

namespace Ui {
    class KImportDlgDecl: public Ui_KImportDlgDecl {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KIMPORTDLGDECL_H

