/********************************************************************************
** Form generated from reading UI file 'currency.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CURRENCY_H
#define UI_CURRENCY_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_Currency
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel1;
    QTreeWidget *m_currencyList;

    void setupUi(QWidget *Currency)
    {
        if (Currency->objectName().isEmpty())
            Currency->setObjectName("Currency");
        Currency->resize(383, 494);
        vboxLayout = new QVBoxLayout(Currency);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        qLabel1 = new QLabel(Currency);
        qLabel1->setObjectName("qLabel1");
        qLabel1->setWordWrap(true);

        vboxLayout->addWidget(qLabel1);

        m_currencyList = new QTreeWidget(Currency);
        m_currencyList->setObjectName("m_currencyList");
        m_currencyList->setRootIsDecorated(false);

        vboxLayout->addWidget(m_currencyList);


        retranslateUi(Currency);

        QMetaObject::connectSlotsByName(Currency);
    } // setupUi

    void retranslateUi(QWidget *Currency)
    {
        qLabel1->setText(tr2i18n("KMyMoney supports multiple currencies, nevertheless one of them will be used as your base currency. The base currency is used as default for new accounts and reports. Please select the base currency from the following list.", "@label"));
        QTreeWidgetItem *___qtreewidgetitem = m_currencyList->headerItem();
        ___qtreewidgetitem->setText(2, tr2i18n("Symbol", "@title"));
        ___qtreewidgetitem->setText(1, tr2i18n("ISO Code", "@title"));
        ___qtreewidgetitem->setText(0, tr2i18n("Name", "@title header of the currency name column"));
        (void)Currency;
    } // retranslateUi

};

namespace Ui {
    class Currency: public Ui_Currency {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CURRENCY_H

