// This file is generated by kconfig_compiler_kf6 from reportsviewsettings.kcfg.
// All changes you do to this file will be lost.

#include "reportsviewsettings.h"

#include <qglobal.h>
#include <QFile>

class ReportsViewSettingsHelper
{
  public:
    ReportsViewSettingsHelper() : q(nullptr) {}
    ~ReportsViewSettingsHelper() { delete q; q = nullptr; }
    ReportsViewSettingsHelper(const ReportsViewSettingsHelper&) = delete;
    ReportsViewSettingsHelper& operator=(const ReportsViewSettingsHelper&) = delete;
    ReportsViewSettings *q;
};
Q_GLOBAL_STATIC(ReportsViewSettingsHelper, s_globalReportsViewSettings)
ReportsViewSettings *ReportsViewSettings::self()
{
  if (!s_globalReportsViewSettings()->q) {
    new ReportsViewSettings;
    s_globalReportsViewSettings()->q->read();
  }

  return s_globalReportsViewSettings()->q;
}

ReportsViewSettings::ReportsViewSettings( )
  : KConfigSkeleton( QStringLiteral( "kmymoney/kmymoneyrc" ) )
{
  Q_ASSERT(!s_globalReportsViewSettings()->q);
  s_globalReportsViewSettings()->q = this;
}

ReportsViewSettings::~ReportsViewSettings()
{
  if (s_globalReportsViewSettings.exists() && !s_globalReportsViewSettings.isDestroyed()) {
    s_globalReportsViewSettings()->q = nullptr;
  }
}

