/********************************************************************************
** Form generated from reading UI file 'knewloanwizard.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KNEWLOANWIZARD_H
#define UI_KNEWLOANWIZARD_H

#include <../widgets/kmymoneyaccountselector.h>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QWizard>
#include <kcombobox.h>
#include <klocalizedstring.h>
#include "../widgets/kmymoneylineedit.h"
#include "additionalfeeswizardpage.h"
#include "assetaccountwizardpage.h"
#include "calculationoverviewwizardpage.h"
#include "durationwizardpage.h"
#include "editintrowizardpage.h"
#include "editselectionwizardpage.h"
#include "effectivedatewizardpage.h"
#include "finalpaymentwizardpage.h"
#include "firstpaymentwizardpage.h"
#include "interestcalculationwizardpage.h"
#include "interestcategorywizardpage.h"
#include "interesteditwizardpage.h"
#include "interesttypewizardpage.h"
#include "interestwizardpage.h"
#include "kmymoneygeneralcombo.h"
#include "lendborrowwizardpage.h"
#include "loanamountwizardpage.h"
#include "loanattributeswizardpage.h"
#include "namewizardpage.h"
#include "newcalculateloanwizardpage.h"
#include "newgeneralinfowizardpage.h"
#include "newintrowizardpage.h"
#include "newpaymentswizardpage.h"
#include "paymenteditwizardpage.h"
#include "paymentfrequencywizardpage.h"
#include "paymentwizardpage.h"
#include "previouspaymentswizardpage.h"
#include "recordpaymentwizardpage.h"
#include "schedulewizardpage.h"
#include "summaryeditwizardpage.h"
#include "summarywizardpage.h"
#include "variableinterestdatewizardpage.h"

QT_BEGIN_NAMESPACE

class Ui_KNewLoanWizard
{
public:
    NewIntroWizardPage *m_newIntroPage;
    EditIntroWizardPage *m_editIntroPage;
    NewGeneralInfoWizardPage *m_newGeneralInfoPage;
    EditSelectionWizardPage *m_editSelectionPage;
    LoanAttributesWizardPage *m_loanAttributesPage;
    EffectiveDateWizardPage *m_effectiveDatePage;
    LendBorrowWizardPage *m_lendBorrowPage;
    NameWizardPage *m_namePage;
    InterestTypeWizardPage *m_interestTypePage;
    PreviousPaymentsWizardPage *m_previousPaymentsPage;
    RecordPaymentWizardPage *m_recordPaymentPage;
    VariableInterestDateWizardPage *m_variableInterestDatePage;
    PaymentEditWizardPage *m_paymentEditPage;
    InterestEditWizardPage *m_interestEditPage;
    FirstPaymentWizardPage *m_firstPaymentPage;
    NewCalculateLoanWizardPage *m_newCalculateLoanPage;
    PaymentFrequencyWizardPage *m_paymentFrequencyPage;
    InterestCalculationWizardPage *m_interestCalculationPage;
    LoanAmountWizardPage *m_loanAmountPage;
    InterestWizardPage *m_interestPage;
    DurationWizardPage *m_durationPage;
    PaymentWizardPage *m_paymentPage;
    FinalPaymentWizardPage *m_finalPaymentPage;
    CalculationOverviewWizardPage *m_calculationOverviewPage;
    NewPaymentsWizardPage *m_newPaymentsPage;
    InterestCategoryWizardPage *m_interestCategoryPage;
    AdditionalFeesWizardPage *m_additionalFeesPage;
    ScheduleWizardPage *m_schedulePage;
    SummaryEditWizardPage *m_summaryEditPage;
    AssetAccountWizardPage *m_assetAccountPage;
    SummaryWizardPage *m_summaryPage;

    void setupUi(QWizard *KNewLoanWizard)
    {
        if (KNewLoanWizard->objectName().isEmpty())
            KNewLoanWizard->setObjectName("KNewLoanWizard");
        KNewLoanWizard->resize(589, 458);
        KNewLoanWizard->setMaximumSize(QSize(800, 32767));
        KNewLoanWizard->setWizardStyle(QWizard::ModernStyle);
        KNewLoanWizard->setSizeGripEnabled(true);
        m_newIntroPage = new NewIntroWizardPage();
        m_newIntroPage->setObjectName("m_newIntroPage");
        KNewLoanWizard->addPage(m_newIntroPage);
        m_editIntroPage = new EditIntroWizardPage();
        m_editIntroPage->setObjectName("m_editIntroPage");
        KNewLoanWizard->addPage(m_editIntroPage);
        m_newGeneralInfoPage = new NewGeneralInfoWizardPage();
        m_newGeneralInfoPage->setObjectName("m_newGeneralInfoPage");
        KNewLoanWizard->addPage(m_newGeneralInfoPage);
        m_editSelectionPage = new EditSelectionWizardPage();
        m_editSelectionPage->setObjectName("m_editSelectionPage");
        KNewLoanWizard->addPage(m_editSelectionPage);
        m_loanAttributesPage = new LoanAttributesWizardPage();
        m_loanAttributesPage->setObjectName("m_loanAttributesPage");
        KNewLoanWizard->addPage(m_loanAttributesPage);
        m_effectiveDatePage = new EffectiveDateWizardPage();
        m_effectiveDatePage->setObjectName("m_effectiveDatePage");
        KNewLoanWizard->addPage(m_effectiveDatePage);
        m_lendBorrowPage = new LendBorrowWizardPage();
        m_lendBorrowPage->setObjectName("m_lendBorrowPage");
        KNewLoanWizard->addPage(m_lendBorrowPage);
        m_namePage = new NameWizardPage();
        m_namePage->setObjectName("m_namePage");
        KNewLoanWizard->addPage(m_namePage);
        m_interestTypePage = new InterestTypeWizardPage();
        m_interestTypePage->setObjectName("m_interestTypePage");
        KNewLoanWizard->addPage(m_interestTypePage);
        m_previousPaymentsPage = new PreviousPaymentsWizardPage();
        m_previousPaymentsPage->setObjectName("m_previousPaymentsPage");
        KNewLoanWizard->addPage(m_previousPaymentsPage);
        m_recordPaymentPage = new RecordPaymentWizardPage();
        m_recordPaymentPage->setObjectName("m_recordPaymentPage");
        KNewLoanWizard->addPage(m_recordPaymentPage);
        m_variableInterestDatePage = new VariableInterestDateWizardPage();
        m_variableInterestDatePage->setObjectName("m_variableInterestDatePage");
        KNewLoanWizard->addPage(m_variableInterestDatePage);
        m_paymentEditPage = new PaymentEditWizardPage();
        m_paymentEditPage->setObjectName("m_paymentEditPage");
        KNewLoanWizard->addPage(m_paymentEditPage);
        m_interestEditPage = new InterestEditWizardPage();
        m_interestEditPage->setObjectName("m_interestEditPage");
        KNewLoanWizard->addPage(m_interestEditPage);
        m_firstPaymentPage = new FirstPaymentWizardPage();
        m_firstPaymentPage->setObjectName("m_firstPaymentPage");
        KNewLoanWizard->addPage(m_firstPaymentPage);
        m_newCalculateLoanPage = new NewCalculateLoanWizardPage();
        m_newCalculateLoanPage->setObjectName("m_newCalculateLoanPage");
        KNewLoanWizard->addPage(m_newCalculateLoanPage);
        m_paymentFrequencyPage = new PaymentFrequencyWizardPage();
        m_paymentFrequencyPage->setObjectName("m_paymentFrequencyPage");
        KNewLoanWizard->addPage(m_paymentFrequencyPage);
        m_interestCalculationPage = new InterestCalculationWizardPage();
        m_interestCalculationPage->setObjectName("m_interestCalculationPage");
        KNewLoanWizard->addPage(m_interestCalculationPage);
        m_loanAmountPage = new LoanAmountWizardPage();
        m_loanAmountPage->setObjectName("m_loanAmountPage");
        KNewLoanWizard->addPage(m_loanAmountPage);
        m_interestPage = new InterestWizardPage();
        m_interestPage->setObjectName("m_interestPage");
        KNewLoanWizard->addPage(m_interestPage);
        m_durationPage = new DurationWizardPage();
        m_durationPage->setObjectName("m_durationPage");
        KNewLoanWizard->addPage(m_durationPage);
        m_paymentPage = new PaymentWizardPage();
        m_paymentPage->setObjectName("m_paymentPage");
        KNewLoanWizard->addPage(m_paymentPage);
        m_finalPaymentPage = new FinalPaymentWizardPage();
        m_finalPaymentPage->setObjectName("m_finalPaymentPage");
        KNewLoanWizard->addPage(m_finalPaymentPage);
        m_calculationOverviewPage = new CalculationOverviewWizardPage();
        m_calculationOverviewPage->setObjectName("m_calculationOverviewPage");
        KNewLoanWizard->addPage(m_calculationOverviewPage);
        m_newPaymentsPage = new NewPaymentsWizardPage();
        m_newPaymentsPage->setObjectName("m_newPaymentsPage");
        KNewLoanWizard->addPage(m_newPaymentsPage);
        m_interestCategoryPage = new InterestCategoryWizardPage();
        m_interestCategoryPage->setObjectName("m_interestCategoryPage");
        KNewLoanWizard->addPage(m_interestCategoryPage);
        m_additionalFeesPage = new AdditionalFeesWizardPage();
        m_additionalFeesPage->setObjectName("m_additionalFeesPage");
        KNewLoanWizard->addPage(m_additionalFeesPage);
        m_schedulePage = new ScheduleWizardPage();
        m_schedulePage->setObjectName("m_schedulePage");
        KNewLoanWizard->addPage(m_schedulePage);
        m_summaryEditPage = new SummaryEditWizardPage();
        m_summaryEditPage->setObjectName("m_summaryEditPage");
        KNewLoanWizard->addPage(m_summaryEditPage);
        m_assetAccountPage = new AssetAccountWizardPage();
        m_assetAccountPage->setObjectName("m_assetAccountPage");
        KNewLoanWizard->addPage(m_assetAccountPage);
        m_summaryPage = new SummaryWizardPage();
        m_summaryPage->setObjectName("m_summaryPage");
        KNewLoanWizard->addPage(m_summaryPage);

        retranslateUi(KNewLoanWizard);

        QMetaObject::connectSlotsByName(KNewLoanWizard);
    } // setupUi

    void retranslateUi(QWizard *KNewLoanWizard)
    {
        KNewLoanWizard->setWindowTitle(tr2i18n("New Loan Wizard", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KNewLoanWizard: public Ui_KNewLoanWizard {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KNEWLOANWIZARD_H

