/********************************************************************************
** Form generated from reading UI file 'reporttabperformance.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REPORTTABPERFORMANCE_H
#define UI_REPORTTABPERFORMANCE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ReportTabPerformance
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QComboBox *m_investmentSum;
    QLabel *lblinvestmentSum;
    QSpacerItem *horizontalSpacer;
    QCheckBox *m_checkHideTotals;
    QLabel *lblHideTotals;
    QSpacerItem *spacer15;

    void setupUi(QWidget *ReportTabPerformance)
    {
        if (ReportTabPerformance->objectName().isEmpty())
            ReportTabPerformance->setObjectName("ReportTabPerformance");
        ReportTabPerformance->resize(441, 84);
        verticalLayout = new QVBoxLayout(ReportTabPerformance);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        m_investmentSum = new QComboBox(ReportTabPerformance);
        m_investmentSum->setObjectName("m_investmentSum");

        gridLayout->addWidget(m_investmentSum, 0, 1, 1, 1);

        lblinvestmentSum = new QLabel(ReportTabPerformance);
        lblinvestmentSum->setObjectName("lblinvestmentSum");

        gridLayout->addWidget(lblinvestmentSum, 0, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 2, 3, 1);

        m_checkHideTotals = new QCheckBox(ReportTabPerformance);
        m_checkHideTotals->setObjectName("m_checkHideTotals");

        gridLayout->addWidget(m_checkHideTotals, 1, 1, 1, 1, Qt::AlignRight);

        lblHideTotals = new QLabel(ReportTabPerformance);
        lblHideTotals->setObjectName("lblHideTotals");

        gridLayout->addWidget(lblHideTotals, 1, 0, 1, 1);


        verticalLayout->addLayout(gridLayout);

        spacer15 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacer15);


        retranslateUi(ReportTabPerformance);

        QMetaObject::connectSlotsByName(ReportTabPerformance);
    } // setupUi

    void retranslateUi(QWidget *ReportTabPerformance)
    {
        ReportTabPerformance->setWindowTitle(tr2i18n("Report Tab", nullptr));
#if QT_CONFIG(tooltip)
        ReportTabPerformance->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        ReportTabPerformance->setWhatsThis(tr2i18n("<p>On this tab, you set the basic properties of this report.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        lblinvestmentSum->setText(tr2i18n("Sum investments", nullptr));
        m_checkHideTotals->setText(QString());
        lblHideTotals->setText(tr2i18n("Hide Totals", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ReportTabPerformance: public Ui_ReportTabPerformance {};
} // namespace Ui

QT_END_NAMESPACE

#endif // REPORTTABPERFORMANCE_H

