/********************************************************************************
** Form generated from reading UI file 'securitiesdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SECURITIESDLG_H
#define UI_SECURITIESDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SecuritiesDlg
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label_title;
    QLabel *label_info;
    QTableWidget *tableWidget;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *SecuritiesDlg)
    {
        if (SecuritiesDlg->objectName().isEmpty())
            SecuritiesDlg->setObjectName("SecuritiesDlg");
        SecuritiesDlg->resize(606, 306);
        verticalLayout = new QVBoxLayout(SecuritiesDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        label_title = new QLabel(SecuritiesDlg);
        label_title->setObjectName("label_title");
        QFont font;
        font.setBold(true);
        label_title->setFont(font);
        label_title->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(label_title);

        label_info = new QLabel(SecuritiesDlg);
        label_info->setObjectName("label_info");
        label_info->setAlignment(Qt::AlignCenter);
        label_info->setWordWrap(true);

        verticalLayout->addWidget(label_info);

        tableWidget = new QTableWidget(SecuritiesDlg);
        if (tableWidget->columnCount() < 3)
            tableWidget->setColumnCount(3);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        tableWidget->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        tableWidget->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        tableWidget->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        tableWidget->setObjectName("tableWidget");
        QFont font1;
        tableWidget->setFont(font1);
        tableWidget->setSelectionMode(QAbstractItemView::NoSelection);
        tableWidget->setWordWrap(false);
        tableWidget->setColumnCount(3);
        tableWidget->horizontalHeader()->setCascadingSectionResizes(true);
        tableWidget->horizontalHeader()->setStretchLastSection(true);
        tableWidget->verticalHeader()->setStretchLastSection(false);

        verticalLayout->addWidget(tableWidget);

        buttonBox = new QDialogButtonBox(SecuritiesDlg);
        buttonBox->setObjectName("buttonBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(buttonBox->sizePolicy().hasHeightForWidth());
        buttonBox->setSizePolicy(sizePolicy);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
        buttonBox->setCenterButtons(true);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(SecuritiesDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, SecuritiesDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, SecuritiesDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(SecuritiesDlg);
    } // setupUi

    void retranslateUi(QDialog *SecuritiesDlg)
    {
        SecuritiesDlg->setWindowTitle(tr2i18n("Securities", nullptr));
        label_title->setText(tr2i18n("Missing names and symbols", nullptr));
        label_info->setText(tr2i18n("The following securities' names and symbols are not known to KMyMoney.\n"
"Please complete every missing name or symbol.", nullptr));
        QTableWidgetItem *___qtablewidgetitem = tableWidget->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(tr2i18n("Status", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = tableWidget->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(tr2i18n("Symbol", "@title stock symbol column"));
        QTableWidgetItem *___qtablewidgetitem2 = tableWidget->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(tr2i18n("Name", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SecuritiesDlg: public Ui_SecuritiesDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SECURITIESDLG_H

