/********************************************************************************
** Form generated from reading UI file 'ktransactionsortoptionsdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KTRANSACTIONSORTOPTIONSDLG_H
#define UI_KTRANSACTIONSORTOPTIONSDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "transactionsortoption.h"

QT_BEGIN_NAMESPACE

class Ui_KTransactionSortOptionsDlg
{
public:
    QVBoxLayout *vboxLayout;
    QCheckBox *m_useDefault;
    TransactionSortOption *m_sortOption;
    QDialogButtonBox *m_buttonBox;

    void setupUi(QDialog *KTransactionSortOptionsDlg)
    {
        if (KTransactionSortOptionsDlg->objectName().isEmpty())
            KTransactionSortOptionsDlg->setObjectName("KTransactionSortOptionsDlg");
        KTransactionSortOptionsDlg->resize(511, 348);
        KTransactionSortOptionsDlg->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(KTransactionSortOptionsDlg);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        m_useDefault = new QCheckBox(KTransactionSortOptionsDlg);
        m_useDefault->setObjectName("m_useDefault");

        vboxLayout->addWidget(m_useDefault);

        m_sortOption = new TransactionSortOption(KTransactionSortOptionsDlg);
        m_sortOption->setObjectName("m_sortOption");

        vboxLayout->addWidget(m_sortOption);

        m_buttonBox = new QDialogButtonBox(KTransactionSortOptionsDlg);
        m_buttonBox->setObjectName("m_buttonBox");
        m_buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Ok);

        vboxLayout->addWidget(m_buttonBox);


        retranslateUi(KTransactionSortOptionsDlg);
        QObject::connect(m_useDefault, SIGNAL(toggled(bool)), m_sortOption, SLOT(setDisabled(bool)));
        QObject::connect(m_buttonBox, &QDialogButtonBox::accepted, KTransactionSortOptionsDlg, qOverload<>(&QDialog::accept));
        QObject::connect(m_buttonBox, &QDialogButtonBox::rejected, KTransactionSortOptionsDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KTransactionSortOptionsDlg);
    } // setupUi

    void retranslateUi(QDialog *KTransactionSortOptionsDlg)
    {
        KTransactionSortOptionsDlg->setWindowTitle(tr2i18n("Sort options", nullptr));
        m_useDefault->setText(tr2i18n("Use default", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KTransactionSortOptionsDlg: public Ui_KTransactionSortOptionsDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KTRANSACTIONSORTOPTIONSDLG_H

