/********************************************************************************
** Form generated from reading UI file 'kreportsview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KREPORTSVIEW_H
#define UI_KREPORTSVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KReportsView
{
public:
    QVBoxLayout *verticalLayout_2;
    QTabWidget *m_reportTabWidget;
    QWidget *m_listTab;
    QVBoxLayout *verticalLayout;
    QTreeWidget *m_tocTreeWidget;
    QWidget *m_filterContainer;
    QHBoxLayout *horizontalLayout;
    QToolButton *m_closeButton;
    QLineEdit *m_searchWidget;

    void setupUi(QWidget *KReportsView)
    {
        if (KReportsView->objectName().isEmpty())
            KReportsView->setObjectName("KReportsView");
        KReportsView->resize(621, 417);
        verticalLayout_2 = new QVBoxLayout(KReportsView);
        verticalLayout_2->setObjectName("verticalLayout_2");
        m_reportTabWidget = new QTabWidget(KReportsView);
        m_reportTabWidget->setObjectName("m_reportTabWidget");
        m_reportTabWidget->setTabsClosable(true);
        m_listTab = new QWidget();
        m_listTab->setObjectName("m_listTab");
        verticalLayout = new QVBoxLayout(m_listTab);
        verticalLayout->setObjectName("verticalLayout");
        m_tocTreeWidget = new QTreeWidget(m_listTab);
        m_tocTreeWidget->setObjectName("m_tocTreeWidget");
        m_tocTreeWidget->setContextMenuPolicy(Qt::CustomContextMenu);
        m_tocTreeWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        m_tocTreeWidget->setAlternatingRowColors(true);
        m_tocTreeWidget->setSortingEnabled(true);
        m_tocTreeWidget->setAllColumnsShowFocus(true);

        verticalLayout->addWidget(m_tocTreeWidget);

        m_filterContainer = new QWidget(m_listTab);
        m_filterContainer->setObjectName("m_filterContainer");
        horizontalLayout = new QHBoxLayout(m_filterContainer);
        horizontalLayout->setSpacing(2);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 1, 0, 1);
        m_closeButton = new QToolButton(m_filterContainer);
        m_closeButton->setObjectName("m_closeButton");
        m_closeButton->setAutoRaise(true);

        horizontalLayout->addWidget(m_closeButton);

        m_searchWidget = new QLineEdit(m_filterContainer);
        m_searchWidget->setObjectName("m_searchWidget");

        horizontalLayout->addWidget(m_searchWidget);

        horizontalLayout->setStretch(1, 2);

        verticalLayout->addWidget(m_filterContainer);

        m_reportTabWidget->addTab(m_listTab, QString());

        verticalLayout_2->addWidget(m_reportTabWidget);


        retranslateUi(KReportsView);

        m_reportTabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KReportsView);
    } // setupUi

    void retranslateUi(QWidget *KReportsView)
    {
        KReportsView->setWindowTitle(tr2i18n("KReportsView", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = m_tocTreeWidget->headerItem();
        ___qtreewidgetitem->setText(1, tr2i18n("Comment", nullptr));
        ___qtreewidgetitem->setText(0, tr2i18n("Reports", nullptr));
        m_closeButton->setText(tr2i18n("...", nullptr));
        m_reportTabWidget->setTabText(m_reportTabWidget->indexOf(m_listTab), tr2i18n("Reports", " "));
    } // retranslateUi

};

namespace Ui {
    class KReportsView: public Ui_KReportsView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KREPORTSVIEW_H

