/********************************************************************************
** Form generated from reading UI file 'kaccountselectdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KACCOUNTSELECTDLG_H
#define UI_KACCOUNTSELECTDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kmymoneyaccountcombo.h"
#include "ktextedit.h"

QT_BEGIN_NAMESPACE

class Ui_KAccountSelectDlg
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    KTextEdit *m_qifEntry;
    QVBoxLayout *vboxLayout1;
    QGridLayout *gridLayout;
    QSpacerItem *spacer6;
    QLabel *m_headerLabel;
    QLabel *m_descLabel;
    KMyMoneyAccountCombo *m_accountSelector;
    QPushButton *m_createButton;
    QSpacerItem *spacer2;
    QFrame *Line2_2;
    QHBoxLayout *hboxLayout1;
    QPushButton *m_kButtonAbort;
    QSpacerItem *Spacer1;
    QPushButton *m_qbuttonCancel;
    QPushButton *m_qbuttonOk;

    void setupUi(QDialog *KAccountSelectDlg)
    {
        if (KAccountSelectDlg->objectName().isEmpty())
            KAccountSelectDlg->setObjectName("KAccountSelectDlg");
        KAccountSelectDlg->setWindowModality(Qt::WindowModal);
        KAccountSelectDlg->resize(651, 434);
        KAccountSelectDlg->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(KAccountSelectDlg);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_qifEntry = new KTextEdit(KAccountSelectDlg);
        m_qifEntry->setObjectName("m_qifEntry");
        m_qifEntry->setUndoRedoEnabled(false);
        m_qifEntry->setLineWrapMode(QTextEdit::NoWrap);
        m_qifEntry->setReadOnly(true);
        m_qifEntry->setAcceptRichText(false);

        hboxLayout->addWidget(m_qifEntry);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName("vboxLayout1");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        spacer6 = new QSpacerItem(110, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacer6, 4, 0, 1, 1);

        m_headerLabel = new QLabel(KAccountSelectDlg);
        m_headerLabel->setObjectName("m_headerLabel");
        m_headerLabel->setWordWrap(false);

        gridLayout->addWidget(m_headerLabel, 1, 0, 1, 2);

        m_descLabel = new QLabel(KAccountSelectDlg);
        m_descLabel->setObjectName("m_descLabel");
        m_descLabel->setWordWrap(true);

        gridLayout->addWidget(m_descLabel, 0, 0, 1, 2);

        m_accountSelector = new KMyMoneyAccountCombo(KAccountSelectDlg);
        m_accountSelector->setObjectName("m_accountSelector");

        gridLayout->addWidget(m_accountSelector, 2, 0, 1, 1);

        m_createButton = new QPushButton(KAccountSelectDlg);
        m_createButton->setObjectName("m_createButton");

        gridLayout->addWidget(m_createButton, 2, 1, 1, 1);


        vboxLayout1->addLayout(gridLayout);

        spacer2 = new QSpacerItem(16, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacer2);


        hboxLayout->addLayout(vboxLayout1);


        vboxLayout->addLayout(hboxLayout);

        Line2_2 = new QFrame(KAccountSelectDlg);
        Line2_2->setObjectName("Line2_2");
        Line2_2->setFrameShape(QFrame::HLine);
        Line2_2->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(Line2_2);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        m_kButtonAbort = new QPushButton(KAccountSelectDlg);
        m_kButtonAbort->setObjectName("m_kButtonAbort");

        hboxLayout1->addWidget(m_kButtonAbort);

        Spacer1 = new QSpacerItem(160, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(Spacer1);

        m_qbuttonCancel = new QPushButton(KAccountSelectDlg);
        m_qbuttonCancel->setObjectName("m_qbuttonCancel");

        hboxLayout1->addWidget(m_qbuttonCancel);

        m_qbuttonOk = new QPushButton(KAccountSelectDlg);
        m_qbuttonOk->setObjectName("m_qbuttonOk");

        hboxLayout1->addWidget(m_qbuttonOk);


        vboxLayout->addLayout(hboxLayout1);


        retranslateUi(KAccountSelectDlg);

        QMetaObject::connectSlotsByName(KAccountSelectDlg);
    } // setupUi

    void retranslateUi(QDialog *KAccountSelectDlg)
    {
        KAccountSelectDlg->setWindowTitle(tr2i18n("Account selection", nullptr));
        m_headerLabel->setText(tr2i18n("Account to import to", nullptr));
        m_createButton->setText(tr2i18n("Create", nullptr));
        m_kButtonAbort->setText(tr2i18n("&Abort", nullptr));
        m_qbuttonCancel->setText(tr2i18n("&Skip", nullptr));
        m_qbuttonOk->setText(tr2i18n("OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KAccountSelectDlg: public Ui_KAccountSelectDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KACCOUNTSELECTDLG_H

