/********************************************************************************
** Form generated from reading UI file 'ktemplateexportdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KTEMPLATEEXPORTDLG_H
#define UI_KTEMPLATEEXPORTDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <klocalizedstring.h>
#include "ktextedit.h"

QT_BEGIN_NAMESPACE

class Ui_KTemplateExportDlg
{
public:
    QGridLayout *gridLayout;
    KTextEdit *m_longDescription;
    QLineEdit *m_shortDescription;
    QLabel *label_2;
    QLabel *label_3;
    QLineEdit *m_title;
    QLabel *label;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KTemplateExportDlg)
    {
        if (KTemplateExportDlg->objectName().isEmpty())
            KTemplateExportDlg->setObjectName("KTemplateExportDlg");
        KTemplateExportDlg->resize(397, 362);
        gridLayout = new QGridLayout(KTemplateExportDlg);
        gridLayout->setObjectName("gridLayout");
        m_longDescription = new KTextEdit(KTemplateExportDlg);
        m_longDescription->setObjectName("m_longDescription");

        gridLayout->addWidget(m_longDescription, 6, 0, 1, 1);

        m_shortDescription = new QLineEdit(KTemplateExportDlg);
        m_shortDescription->setObjectName("m_shortDescription");

        gridLayout->addWidget(m_shortDescription, 4, 0, 1, 1);

        label_2 = new QLabel(KTemplateExportDlg);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 3, 0, 1, 1);

        label_3 = new QLabel(KTemplateExportDlg);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 5, 0, 1, 1);

        m_title = new QLineEdit(KTemplateExportDlg);
        m_title->setObjectName("m_title");

        gridLayout->addWidget(m_title, 1, 0, 1, 1);

        label = new QLabel(KTemplateExportDlg);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        buttonBox = new QDialogButtonBox(KTemplateExportDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(buttonBox, 7, 0, 1, 1);

        QWidget::setTabOrder(m_title, m_shortDescription);
        QWidget::setTabOrder(m_shortDescription, m_longDescription);

        retranslateUi(KTemplateExportDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KTemplateExportDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KTemplateExportDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KTemplateExportDlg);
    } // setupUi

    void retranslateUi(QDialog *KTemplateExportDlg)
    {
        KTemplateExportDlg->setWindowTitle(tr2i18n("Template Export Attributes", nullptr));
        label_2->setText(tr2i18n("Short description", nullptr));
        label_3->setText(tr2i18n("Long Description", nullptr));
        label->setText(tr2i18n("Title", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KTemplateExportDlg: public Ui_KTemplateExportDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KTEMPLATEEXPORTDLG_H

