/********************************************************************************
** Form generated from reading UI file 'summaryeditwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SUMMARYEDITWIZARDPAGE_H
#define UI_SUMMARYEDITWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SummaryEditWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel10_4;
    QSpacerItem *Spacer74_3;
    QFrame *Frame3_2_2_2_2_2_2;
    QGridLayout *gridLayout;
    QLabel *TextLabel6_3_2_2_2_2_2;
    QLabel *TextLabel8_3_2_2_2_2_2;
    QLabel *TextLabel9_2_2_2_2_2_2;
    QLabel *TextLabel10_2_2_2_2_2_2;
    QLabel *TextLabel11_3_2_2_2_2_2;
    QLabel *TextLabel11_3_2_2_2_2_2_2;
    QLabel *m_payment7;
    QLabel *m_additionalFees7;
    QLabel *m_totalPayment7;
    QLabel *m_interestRate7;
    QLabel *m_startDateChanges;
    QLabel *m_affectedPayments;

    void setupUi(QWizardPage *SummaryEditWizardPage)
    {
        if (SummaryEditWizardPage->objectName().isEmpty())
            SummaryEditWizardPage->setObjectName("SummaryEditWizardPage");
        vboxLayout = new QVBoxLayout(SummaryEditWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel10_4 = new QLabel(SummaryEditWizardPage);
        TextLabel10_4->setObjectName("TextLabel10_4");
        TextLabel10_4->setAlignment(Qt::AlignTop);
        TextLabel10_4->setWordWrap(true);

        vboxLayout->addWidget(TextLabel10_4);

        Spacer74_3 = new QSpacerItem(0, 23, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer74_3);

        Frame3_2_2_2_2_2_2 = new QFrame(SummaryEditWizardPage);
        Frame3_2_2_2_2_2_2->setObjectName("Frame3_2_2_2_2_2_2");
        Frame3_2_2_2_2_2_2->setFrameShape(QFrame::StyledPanel);
        Frame3_2_2_2_2_2_2->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(Frame3_2_2_2_2_2_2);
#ifndef UI_Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        TextLabel6_3_2_2_2_2_2 = new QLabel(Frame3_2_2_2_2_2_2);
        TextLabel6_3_2_2_2_2_2->setObjectName("TextLabel6_3_2_2_2_2_2");
        TextLabel6_3_2_2_2_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel6_3_2_2_2_2_2->setWordWrap(true);

        gridLayout->addWidget(TextLabel6_3_2_2_2_2_2, 0, 0, 1, 1);

        TextLabel8_3_2_2_2_2_2 = new QLabel(Frame3_2_2_2_2_2_2);
        TextLabel8_3_2_2_2_2_2->setObjectName("TextLabel8_3_2_2_2_2_2");
        TextLabel8_3_2_2_2_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel8_3_2_2_2_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel8_3_2_2_2_2_2, 1, 0, 1, 1);

        TextLabel9_2_2_2_2_2_2 = new QLabel(Frame3_2_2_2_2_2_2);
        TextLabel9_2_2_2_2_2_2->setObjectName("TextLabel9_2_2_2_2_2_2");
        TextLabel9_2_2_2_2_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel9_2_2_2_2_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel9_2_2_2_2_2_2, 2, 0, 1, 1);

        TextLabel10_2_2_2_2_2_2 = new QLabel(Frame3_2_2_2_2_2_2);
        TextLabel10_2_2_2_2_2_2->setObjectName("TextLabel10_2_2_2_2_2_2");
        TextLabel10_2_2_2_2_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel10_2_2_2_2_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel10_2_2_2_2_2_2, 3, 0, 1, 1);

        TextLabel11_3_2_2_2_2_2 = new QLabel(Frame3_2_2_2_2_2_2);
        TextLabel11_3_2_2_2_2_2->setObjectName("TextLabel11_3_2_2_2_2_2");
        TextLabel11_3_2_2_2_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel11_3_2_2_2_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel11_3_2_2_2_2_2, 4, 0, 1, 1);

        TextLabel11_3_2_2_2_2_2_2 = new QLabel(Frame3_2_2_2_2_2_2);
        TextLabel11_3_2_2_2_2_2_2->setObjectName("TextLabel11_3_2_2_2_2_2_2");
        TextLabel11_3_2_2_2_2_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel11_3_2_2_2_2_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel11_3_2_2_2_2_2_2, 5, 0, 1, 1);

        m_payment7 = new QLabel(Frame3_2_2_2_2_2_2);
        m_payment7->setObjectName("m_payment7");
        QFont font;
        font.setBold(true);
        m_payment7->setFont(font);
        m_payment7->setTextFormat(Qt::PlainText);
        m_payment7->setWordWrap(false);

        gridLayout->addWidget(m_payment7, 0, 1, 1, 1);

        m_additionalFees7 = new QLabel(Frame3_2_2_2_2_2_2);
        m_additionalFees7->setObjectName("m_additionalFees7");
        m_additionalFees7->setFont(font);
        m_additionalFees7->setTextFormat(Qt::PlainText);
        m_additionalFees7->setWordWrap(false);

        gridLayout->addWidget(m_additionalFees7, 1, 1, 1, 1);

        m_totalPayment7 = new QLabel(Frame3_2_2_2_2_2_2);
        m_totalPayment7->setObjectName("m_totalPayment7");
        m_totalPayment7->setFont(font);
        m_totalPayment7->setTextFormat(Qt::PlainText);
        m_totalPayment7->setWordWrap(false);

        gridLayout->addWidget(m_totalPayment7, 2, 1, 1, 1);

        m_interestRate7 = new QLabel(Frame3_2_2_2_2_2_2);
        m_interestRate7->setObjectName("m_interestRate7");
        m_interestRate7->setFont(font);
        m_interestRate7->setTextFormat(Qt::PlainText);
        m_interestRate7->setWordWrap(false);

        gridLayout->addWidget(m_interestRate7, 3, 1, 1, 1);

        m_startDateChanges = new QLabel(Frame3_2_2_2_2_2_2);
        m_startDateChanges->setObjectName("m_startDateChanges");
        m_startDateChanges->setFont(font);
        m_startDateChanges->setTextFormat(Qt::PlainText);
        m_startDateChanges->setWordWrap(false);

        gridLayout->addWidget(m_startDateChanges, 4, 1, 1, 1);

        m_affectedPayments = new QLabel(Frame3_2_2_2_2_2_2);
        m_affectedPayments->setObjectName("m_affectedPayments");
        m_affectedPayments->setFont(font);
        m_affectedPayments->setTextFormat(Qt::PlainText);
        m_affectedPayments->setWordWrap(false);

        gridLayout->addWidget(m_affectedPayments, 5, 1, 1, 1);


        vboxLayout->addWidget(Frame3_2_2_2_2_2_2);


        retranslateUi(SummaryEditWizardPage);

        QMetaObject::connectSlotsByName(SummaryEditWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *SummaryEditWizardPage)
    {
        TextLabel10_4->setText(tr2i18n("\n"
"KMyMoney has calculated the loan as shown below. If you want to accept these values use the \"Finish\" button to update your account, otherwise use the \"Back\" button to modify your settings.", nullptr));
        TextLabel6_3_2_2_2_2_2->setText(tr2i18n("Principal + Interest:", nullptr));
        TextLabel8_3_2_2_2_2_2->setText(tr2i18n("Additional Fees:", nullptr));
        TextLabel9_2_2_2_2_2_2->setText(tr2i18n("Total payment:", nullptr));
        TextLabel10_2_2_2_2_2_2->setText(tr2i18n("Interest rate:", nullptr));
        TextLabel11_3_2_2_2_2_2->setText(tr2i18n("Valid from:", nullptr));
        TextLabel11_3_2_2_2_2_2_2->setText(tr2i18n("Affected payments:", nullptr));
        m_payment7->setText(tr2i18n("1", nullptr));
        m_additionalFees7->setText(tr2i18n("1", nullptr));
        m_totalPayment7->setText(tr2i18n("1", nullptr));
        m_interestRate7->setText(tr2i18n("1", nullptr));
        m_startDateChanges->setText(tr2i18n("1", nullptr));
        m_affectedPayments->setText(tr2i18n("1", nullptr));
        (void)SummaryEditWizardPage;
    } // retranslateUi

};

namespace Ui {
    class SummaryEditWizardPage: public Ui_SummaryEditWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SUMMARYEDITWIZARDPAGE_H

