/********************************************************************************
** Form generated from reading UI file 'kcategoryreassigndlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KCATEGORYREASSIGNDLG_H
#define UI_KCATEGORYREASSIGNDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "kmymoneycategory.h"

QT_BEGIN_NAMESPACE

class Ui_KCategoryReassignDlg
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *textLabel1;
    QSpacerItem *spacer8;
    QLabel *textLabel2;
    KMyMoneyCategory *m_category;
    QSpacerItem *spacer9;
    QFrame *line1;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KCategoryReassignDlg)
    {
        if (KCategoryReassignDlg->objectName().isEmpty())
            KCategoryReassignDlg->setObjectName("KCategoryReassignDlg");
        KCategoryReassignDlg->resize(392, 308);
        KCategoryReassignDlg->setSizeGripEnabled(false);
        KCategoryReassignDlg->setModal(true);
        vboxLayout = new QVBoxLayout(KCategoryReassignDlg);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        textLabel1 = new QLabel(KCategoryReassignDlg);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setMinimumSize(QSize(300, 0));
        textLabel1->setAlignment(Qt::AlignJustify|Qt::AlignTop);
        textLabel1->setWordWrap(true);

        vboxLayout->addWidget(textLabel1);

        spacer8 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        vboxLayout->addItem(spacer8);

        textLabel2 = new QLabel(KCategoryReassignDlg);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        vboxLayout->addWidget(textLabel2);

        m_category = new KMyMoneyCategory(KCategoryReassignDlg);
        m_category->setObjectName("m_category");

        vboxLayout->addWidget(m_category);

        spacer9 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer9);

        line1 = new QFrame(KCategoryReassignDlg);
        line1->setObjectName("line1");
        line1->setFrameShape(QFrame::HLine);
        line1->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(line1);

        buttonBox = new QDialogButtonBox(KCategoryReassignDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(KCategoryReassignDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KCategoryReassignDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KCategoryReassignDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KCategoryReassignDlg);
    } // setupUi

    void retranslateUi(QDialog *KCategoryReassignDlg)
    {
        KCategoryReassignDlg->setWindowTitle(tr2i18n("Reassign categories", nullptr));
        textLabel1->setText(tr2i18n("The transactions, schedules and budgets associated with the selected category need to be re-assigned to a different category before the selected category can be deleted. Please select a category from the list below.", nullptr));
        textLabel2->setText(tr2i18n("Available categories:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KCategoryReassignDlg: public Ui_KCategoryReassignDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KCATEGORYREASSIGNDLG_H

