/********************************************************************************
** Form generated from reading UI file 'calculationoverviewwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CALCULATIONOVERVIEWWIZARDPAGE_H
#define UI_CALCULATIONOVERVIEWWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_CalculationOverviewWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel18_2;
    QSpacerItem *Spacer79;
    QFrame *Frame3_2_2_2_2_2;
    QGridLayout *gridLayout;
    QLabel *TextLabel6_3_2_2_2_2;
    QLabel *TextLabel8_3_2_2_2_2;
    QLabel *TextLabel9_2_2_2_2_2;
    QLabel *TextLabel10_2_2_2_2_2;
    QLabel *TextLabel11_3_2_2_2_2;
    QLabel *m_loanAmount6;
    QLabel *m_interestRate6;
    QLabel *m_duration6;
    QLabel *m_payment6;
    QLabel *m_balloon6;

    void setupUi(QWizardPage *CalculationOverviewWizardPage)
    {
        if (CalculationOverviewWizardPage->objectName().isEmpty())
            CalculationOverviewWizardPage->setObjectName("CalculationOverviewWizardPage");
        vboxLayout = new QVBoxLayout(CalculationOverviewWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel18_2 = new QLabel(CalculationOverviewWizardPage);
        TextLabel18_2->setObjectName("TextLabel18_2");
        TextLabel18_2->setAlignment(Qt::AlignTop);
        TextLabel18_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel18_2);

        Spacer79 = new QSpacerItem(0, 22, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer79);

        Frame3_2_2_2_2_2 = new QFrame(CalculationOverviewWizardPage);
        Frame3_2_2_2_2_2->setObjectName("Frame3_2_2_2_2_2");
        Frame3_2_2_2_2_2->setFrameShape(QFrame::StyledPanel);
        Frame3_2_2_2_2_2->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(Frame3_2_2_2_2_2);
#ifndef UI_Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        TextLabel6_3_2_2_2_2 = new QLabel(Frame3_2_2_2_2_2);
        TextLabel6_3_2_2_2_2->setObjectName("TextLabel6_3_2_2_2_2");
        TextLabel6_3_2_2_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel6_3_2_2_2_2->setWordWrap(true);

        gridLayout->addWidget(TextLabel6_3_2_2_2_2, 0, 0, 1, 1);

        TextLabel8_3_2_2_2_2 = new QLabel(Frame3_2_2_2_2_2);
        TextLabel8_3_2_2_2_2->setObjectName("TextLabel8_3_2_2_2_2");
        TextLabel8_3_2_2_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel8_3_2_2_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel8_3_2_2_2_2, 1, 0, 1, 1);

        TextLabel9_2_2_2_2_2 = new QLabel(Frame3_2_2_2_2_2);
        TextLabel9_2_2_2_2_2->setObjectName("TextLabel9_2_2_2_2_2");
        TextLabel9_2_2_2_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel9_2_2_2_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel9_2_2_2_2_2, 2, 0, 1, 1);

        TextLabel10_2_2_2_2_2 = new QLabel(Frame3_2_2_2_2_2);
        TextLabel10_2_2_2_2_2->setObjectName("TextLabel10_2_2_2_2_2");
        TextLabel10_2_2_2_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel10_2_2_2_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel10_2_2_2_2_2, 3, 0, 1, 1);

        TextLabel11_3_2_2_2_2 = new QLabel(Frame3_2_2_2_2_2);
        TextLabel11_3_2_2_2_2->setObjectName("TextLabel11_3_2_2_2_2");
        TextLabel11_3_2_2_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel11_3_2_2_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel11_3_2_2_2_2, 4, 0, 1, 1);

        m_loanAmount6 = new QLabel(Frame3_2_2_2_2_2);
        m_loanAmount6->setObjectName("m_loanAmount6");
        QFont font;
        font.setBold(true);
        m_loanAmount6->setFont(font);
        m_loanAmount6->setTextFormat(Qt::PlainText);
        m_loanAmount6->setWordWrap(false);

        gridLayout->addWidget(m_loanAmount6, 0, 1, 1, 1);

        m_interestRate6 = new QLabel(Frame3_2_2_2_2_2);
        m_interestRate6->setObjectName("m_interestRate6");
        m_interestRate6->setFont(font);
        m_interestRate6->setTextFormat(Qt::PlainText);
        m_interestRate6->setWordWrap(false);

        gridLayout->addWidget(m_interestRate6, 1, 1, 1, 1);

        m_duration6 = new QLabel(Frame3_2_2_2_2_2);
        m_duration6->setObjectName("m_duration6");
        m_duration6->setFont(font);
        m_duration6->setTextFormat(Qt::PlainText);
        m_duration6->setWordWrap(false);

        gridLayout->addWidget(m_duration6, 2, 1, 1, 1);

        m_payment6 = new QLabel(Frame3_2_2_2_2_2);
        m_payment6->setObjectName("m_payment6");
        m_payment6->setFont(font);
        m_payment6->setTextFormat(Qt::PlainText);
        m_payment6->setWordWrap(false);

        gridLayout->addWidget(m_payment6, 3, 1, 1, 1);

        m_balloon6 = new QLabel(Frame3_2_2_2_2_2);
        m_balloon6->setObjectName("m_balloon6");
        m_balloon6->setFont(font);
        m_balloon6->setTextFormat(Qt::PlainText);
        m_balloon6->setWordWrap(false);

        gridLayout->addWidget(m_balloon6, 4, 1, 1, 1);


        vboxLayout->addWidget(Frame3_2_2_2_2_2);


        retranslateUi(CalculationOverviewWizardPage);

        QMetaObject::connectSlotsByName(CalculationOverviewWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *CalculationOverviewWizardPage)
    {
        TextLabel18_2->setText(tr2i18n("\n"
"KMyMoney has calculated the loan as shown in the overview below. You can accept these values by selecting \"Next\" or change them by choosing \"Back\" to return to the input field for the information you want to change.", nullptr));
        TextLabel6_3_2_2_2_2->setText(tr2i18n("Loan amount:", nullptr));
        TextLabel8_3_2_2_2_2->setText(tr2i18n("Interest rate:", nullptr));
        TextLabel9_2_2_2_2_2->setText(tr2i18n("Term:", nullptr));
        TextLabel10_2_2_2_2_2->setText(tr2i18n("Principal + Interest:", nullptr));
        TextLabel11_3_2_2_2_2->setText(tr2i18n("Final amortization payment", nullptr));
        m_loanAmount6->setText(tr2i18n("1", nullptr));
        m_interestRate6->setText(tr2i18n("1", nullptr));
        m_duration6->setText(tr2i18n("1", nullptr));
        m_payment6->setText(tr2i18n("1", nullptr));
        m_balloon6->setText(tr2i18n("1", nullptr));
        (void)CalculationOverviewWizardPage;
    } // retranslateUi

};

namespace Ui {
    class CalculationOverviewWizardPage: public Ui_CalculationOverviewWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CALCULATIONOVERVIEWWIZARDPAGE_H

