/********************************************************************************
** Form generated from reading UI file 'currenciesdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CURRENCIESDLG_H
#define UI_CURRENCIESDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_CurrenciesDlg
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label_title;
    QLabel *label_info;
    QGridLayout *gridLayout;
    QComboBox *cbFrom;
    QLabel *lblFrom;
    QLabel *lblTo;
    QCheckBox *cbDontAsk;
    QComboBox *cbTo;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *CurrenciesDlg)
    {
        if (CurrenciesDlg->objectName().isEmpty())
            CurrenciesDlg->setObjectName("CurrenciesDlg");
        CurrenciesDlg->resize(300, 280);
        verticalLayout = new QVBoxLayout(CurrenciesDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        label_title = new QLabel(CurrenciesDlg);
        label_title->setObjectName("label_title");
        QFont font;
        font.setBold(true);
        label_title->setFont(font);
        label_title->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(label_title);

        label_info = new QLabel(CurrenciesDlg);
        label_info->setObjectName("label_info");
        label_info->setAlignment(Qt::AlignCenter);
        label_info->setWordWrap(true);

        verticalLayout->addWidget(label_info);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        cbFrom = new QComboBox(CurrenciesDlg);
        cbFrom->setObjectName("cbFrom");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(cbFrom->sizePolicy().hasHeightForWidth());
        cbFrom->setSizePolicy(sizePolicy);

        gridLayout->addWidget(cbFrom, 0, 1, 1, 1);

        lblFrom = new QLabel(CurrenciesDlg);
        lblFrom->setObjectName("lblFrom");

        gridLayout->addWidget(lblFrom, 0, 0, 1, 1);

        lblTo = new QLabel(CurrenciesDlg);
        lblTo->setObjectName("lblTo");

        gridLayout->addWidget(lblTo, 1, 0, 1, 1);

        cbDontAsk = new QCheckBox(CurrenciesDlg);
        cbDontAsk->setObjectName("cbDontAsk");
        sizePolicy.setHeightForWidth(cbDontAsk->sizePolicy().hasHeightForWidth());
        cbDontAsk->setSizePolicy(sizePolicy);
        cbDontAsk->setLayoutDirection(Qt::RightToLeft);

        gridLayout->addWidget(cbDontAsk, 2, 1, 1, 1, Qt::AlignLeft);

        cbTo = new QComboBox(CurrenciesDlg);
        cbTo->setObjectName("cbTo");
        sizePolicy.setHeightForWidth(cbTo->sizePolicy().hasHeightForWidth());
        cbTo->setSizePolicy(sizePolicy);

        gridLayout->addWidget(cbTo, 1, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        buttonBox = new QDialogButtonBox(CurrenciesDlg);
        buttonBox->setObjectName("buttonBox");
        sizePolicy.setHeightForWidth(buttonBox->sizePolicy().hasHeightForWidth());
        buttonBox->setSizePolicy(sizePolicy);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
        buttonBox->setCenterButtons(true);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(cbFrom, cbTo);
        QWidget::setTabOrder(cbTo, cbDontAsk);

        retranslateUi(CurrenciesDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, CurrenciesDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, CurrenciesDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(CurrenciesDlg);
    } // setupUi

    void retranslateUi(QDialog *CurrenciesDlg)
    {
        CurrenciesDlg->setWindowTitle(tr2i18n("Currencies", nullptr));
        label_title->setText(tr2i18n("Missing currencies", nullptr));
        label_info->setText(tr2i18n("You need to select from currency and to currency.", nullptr));
        lblFrom->setText(tr2i18n("From currency", nullptr));
        lblTo->setText(tr2i18n("To currency", nullptr));
        cbDontAsk->setText(tr2i18n("Do not ask next time", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CurrenciesDlg: public Ui_CurrenciesDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CURRENCIESDLG_H

