/********************************************************************************
** Form generated from reading UI file 'konlinetransferform.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KONLINETRANSFERFORM_H
#define UI_KONLINETRANSFERFORM_H

#include <KMessageWidget>
#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <amountedit.h>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "kmymoneyaccountcombo.h"

QT_BEGIN_NAMESPACE

class Ui_kOnlineTransferForm
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QFormLayout *formLayout;
    QLabel *label_6;
    AmountEdit *orderAccountBalance;
    KMyMoneyAccountCombo *originAccount;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_2;
    KComboBox *transferTypeSelection;
    QStackedWidget *displayStack;
    QWidget *accountUnsupported;
    QVBoxLayout *verticalLayout_3;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout;
    QLabel *unsupportedIcon;
    QLabel *label;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer_2;
    QWidget *page;
    QVBoxLayout *verticalLayout_4;
    KMessageWidget *headMessage;
    KMessageWidget *convertMessage;
    QScrollArea *creditTransferEdit;
    QWidget *scrollAreaWidgetContents;
    QHBoxLayout *horizontalLayout_4;
    QSpacerItem *horizontalSpacer_4;
    QPushButton *buttonEnque;
    QPushButton *buttonSend;
    QPushButton *buttonAbort;

    void setupUi(QDialog *kOnlineTransferForm)
    {
        if (kOnlineTransferForm->objectName().isEmpty())
            kOnlineTransferForm->setObjectName("kOnlineTransferForm");
        kOnlineTransferForm->resize(624, 617);
        kOnlineTransferForm->setSizeGripEnabled(false);
        verticalLayout = new QVBoxLayout(kOnlineTransferForm);
        verticalLayout->setObjectName("verticalLayout");
        groupBox = new QGroupBox(kOnlineTransferForm);
        groupBox->setObjectName("groupBox");
        formLayout = new QFormLayout(groupBox);
        formLayout->setObjectName("formLayout");
        label_6 = new QLabel(groupBox);
        label_6->setObjectName("label_6");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_6);

        orderAccountBalance = new AmountEdit(groupBox);
        orderAccountBalance->setObjectName("orderAccountBalance");
        orderAccountBalance->setEnabled(false);
        orderAccountBalance->setProperty("calculatorButtonVisibility", QVariant(false));
        orderAccountBalance->setProperty("resetButtonVisibility", QVariant(false));
        orderAccountBalance->setReadOnly(true);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, orderAccountBalance);

        originAccount = new KMyMoneyAccountCombo(groupBox);
        originAccount->setObjectName("originAccount");

        formLayout->setWidget(0, QFormLayout::ItemRole::SpanningRole, originAccount);


        verticalLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(kOnlineTransferForm);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_2 = new QVBoxLayout(groupBox_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        transferTypeSelection = new KComboBox(groupBox_2);
        transferTypeSelection->setObjectName("transferTypeSelection");

        verticalLayout_2->addWidget(transferTypeSelection);

        displayStack = new QStackedWidget(groupBox_2);
        displayStack->setObjectName("displayStack");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(displayStack->sizePolicy().hasHeightForWidth());
        displayStack->setSizePolicy(sizePolicy);
        displayStack->setMinimumSize(QSize(333, 250));
        accountUnsupported = new QWidget();
        accountUnsupported->setObjectName("accountUnsupported");
        verticalLayout_3 = new QVBoxLayout(accountUnsupported);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalSpacer = new QSpacerItem(20, 54, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalSpacer_2 = new QSpacerItem(0, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setSizeConstraint(QLayout::SetMaximumSize);
        unsupportedIcon = new QLabel(accountUnsupported);
        unsupportedIcon->setObjectName("unsupportedIcon");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Maximum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(unsupportedIcon->sizePolicy().hasHeightForWidth());
        unsupportedIcon->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(unsupportedIcon);

        label = new QLabel(accountUnsupported);
        label->setObjectName("label");
        label->setMaximumSize(QSize(300, 16777215));
        label->setWordWrap(true);

        horizontalLayout->addWidget(label);

        horizontalLayout->setStretch(1, 1);

        horizontalLayout_3->addLayout(horizontalLayout);

        horizontalSpacer_3 = new QSpacerItem(0, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);

        horizontalLayout_3->setStretch(1, 1);

        verticalLayout_3->addLayout(horizontalLayout_3);

        verticalSpacer_2 = new QSpacerItem(20, 53, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);

        verticalLayout_3->setStretch(1, 1);
        displayStack->addWidget(accountUnsupported);
        page = new QWidget();
        page->setObjectName("page");
        verticalLayout_4 = new QVBoxLayout(page);
        verticalLayout_4->setObjectName("verticalLayout_4");
        headMessage = new KMessageWidget(page);
        headMessage->setObjectName("headMessage");

        verticalLayout_4->addWidget(headMessage);

        convertMessage = new KMessageWidget(page);
        convertMessage->setObjectName("convertMessage");

        verticalLayout_4->addWidget(convertMessage);

        creditTransferEdit = new QScrollArea(page);
        creditTransferEdit->setObjectName("creditTransferEdit");
        creditTransferEdit->setFrameShape(QFrame::NoFrame);
        creditTransferEdit->setFrameShadow(QFrame::Plain);
        creditTransferEdit->setWidgetResizable(true);
        creditTransferEdit->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 75, 16));
        creditTransferEdit->setWidget(scrollAreaWidgetContents);

        verticalLayout_4->addWidget(creditTransferEdit);

        displayStack->addWidget(page);

        verticalLayout_2->addWidget(displayStack);


        verticalLayout->addWidget(groupBox_2);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_4);

        buttonEnque = new QPushButton(kOnlineTransferForm);
        buttonEnque->setObjectName("buttonEnque");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("document-save");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8(""), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        buttonEnque->setIcon(icon);

        horizontalLayout_4->addWidget(buttonEnque);

        buttonSend = new QPushButton(kOnlineTransferForm);
        buttonSend->setObjectName("buttonSend");
        buttonSend->setEnabled(false);
        QIcon icon1;
        iconThemeName = QString::fromUtf8("mail-send");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8(""), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        buttonSend->setIcon(icon1);

        horizontalLayout_4->addWidget(buttonSend);

        buttonAbort = new QPushButton(kOnlineTransferForm);
        buttonAbort->setObjectName("buttonAbort");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("dialog-cancel");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8(""), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        buttonAbort->setIcon(icon2);

        horizontalLayout_4->addWidget(buttonAbort);


        verticalLayout->addLayout(horizontalLayout_4);

        QWidget::setTabOrder(buttonSend, buttonEnque);
        QWidget::setTabOrder(buttonEnque, buttonAbort);

        retranslateUi(kOnlineTransferForm);

        displayStack->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(kOnlineTransferForm);
    } // setupUi

    void retranslateUi(QDialog *kOnlineTransferForm)
    {
        groupBox->setTitle(tr2i18n("Order Account", nullptr));
        label_6->setText(tr2i18n("Account Balance", nullptr));
        groupBox_2->setTitle(tr2i18n("Credit Transfer", nullptr));
        label->setText(tr2i18n("<html><head/><body><p>This account does not support online banking.</p><p>If you think this is an error, please check if the plugin for this type of credit transfer and your online banking plugins are available.</p></body></html>", nullptr));
        buttonEnque->setText(tr2i18n("Enqueue", nullptr));
        buttonSend->setText(tr2i18n("Send", nullptr));
        buttonAbort->setText(tr2i18n("Abort", nullptr));
        (void)kOnlineTransferForm;
    } // retranslateUi

};

namespace Ui {
    class kOnlineTransferForm: public Ui_kOnlineTransferForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KONLINETRANSFERFORM_H

