/********************************************************************************
** Form generated from reading UI file 'kavailablecurrencydlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KAVAILABLECURRENCYDLG_H
#define UI_KAVAILABLECURRENCYDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KAvailableCurrencyDlg
{
public:
    QVBoxLayout *verticalLayout;
    QTreeWidget *m_currencyList;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KAvailableCurrencyDlg)
    {
        if (KAvailableCurrencyDlg->objectName().isEmpty())
            KAvailableCurrencyDlg->setObjectName("KAvailableCurrencyDlg");
        KAvailableCurrencyDlg->resize(400, 516);
        KAvailableCurrencyDlg->setSizeGripEnabled(true);
        verticalLayout = new QVBoxLayout(KAvailableCurrencyDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        m_currencyList = new QTreeWidget(KAvailableCurrencyDlg);
        m_currencyList->setObjectName("m_currencyList");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(m_currencyList->sizePolicy().hasHeightForWidth());
        m_currencyList->setSizePolicy(sizePolicy);
        m_currencyList->setEditTriggers(QAbstractItemView::NoEditTriggers);
        m_currencyList->setSelectionMode(QAbstractItemView::ExtendedSelection);
        m_currencyList->setRootIsDecorated(false);
        m_currencyList->setAllColumnsShowFocus(true);

        verticalLayout->addWidget(m_currencyList);

        buttonBox = new QDialogButtonBox(KAvailableCurrencyDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(KAvailableCurrencyDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KAvailableCurrencyDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KAvailableCurrencyDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KAvailableCurrencyDlg);
    } // setupUi

    void retranslateUi(QDialog *KAvailableCurrencyDlg)
    {
        KAvailableCurrencyDlg->setWindowTitle(tr2i18n("Select currencies to add", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = m_currencyList->headerItem();
        ___qtreewidgetitem->setText(2, tr2i18n("Symbol", "@title currency symbol column"));
        ___qtreewidgetitem->setText(1, tr2i18n("ID", nullptr));
        ___qtreewidgetitem->setText(0, tr2i18n("Name", "@title header of the currency name column"));
    } // retranslateUi

};

namespace Ui {
    class KAvailableCurrencyDlg: public Ui_KAvailableCurrencyDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KAVAILABLECURRENCYDLG_H

