/********************************************************************************
** Form generated from reading UI file 'kbudgetvalues.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KBUDGETVALUES_H
#define UI_KBUDGETVALUES_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "amountedit.h"

QT_BEGIN_NAMESPACE

class Ui_KBudgetValues
{
public:
    QGridLayout *gridLayout;
    QGroupBox *periodGroupBox;
    QVBoxLayout *vboxLayout;
    QRadioButton *m_monthlyButton;
    QRadioButton *m_yearlyButton;
    QRadioButton *m_individualButton;
    QSpacerItem *spacer10;
    QVBoxLayout *vboxLayout1;
    QPushButton *m_clearButton;
    QSpacerItem *spacer7;
    QLabel *m_label1;
    QStackedWidget *m_firstItemStack;
    QWidget *m_monthlyPage;
    QHBoxLayout *hboxLayout;
    AmountEdit *m_amountMonthly;
    QWidget *m_yearlyPage;
    QHBoxLayout *hboxLayout1;
    AmountEdit *m_amountYearly;
    QWidget *m_individualPage;
    QHBoxLayout *hboxLayout2;
    AmountEdit *m_amount1;
    QLabel *m_label2;
    AmountEdit *m_amount2;
    QLabel *m_label3;
    AmountEdit *m_amount3;
    QLabel *m_label4;
    AmountEdit *m_amount4;
    QLabel *m_label5;
    AmountEdit *m_amount5;
    QLabel *m_label6;
    AmountEdit *m_amount6;
    QLabel *m_label7;
    AmountEdit *m_amount7;
    QLabel *m_label8;
    AmountEdit *m_amount8;
    QLabel *m_label9;
    AmountEdit *m_amount9;
    QLabel *m_label10;
    AmountEdit *m_amount10;
    QLabel *m_label11;
    AmountEdit *m_amount11;
    QLabel *m_label12;
    AmountEdit *m_amount12;
    QButtonGroup *m_periodGroup;

    void setupUi(QWidget *KBudgetValues)
    {
        if (KBudgetValues->objectName().isEmpty())
            KBudgetValues->setObjectName("KBudgetValues");
        KBudgetValues->resize(228, 111);
        gridLayout = new QGridLayout(KBudgetValues);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        periodGroupBox = new QGroupBox(KBudgetValues);
        periodGroupBox->setObjectName("periodGroupBox");
        vboxLayout = new QVBoxLayout(periodGroupBox);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        m_monthlyButton = new QRadioButton(periodGroupBox);
        m_periodGroup = new QButtonGroup(KBudgetValues);
        m_periodGroup->setObjectName("m_periodGroup");
        m_periodGroup->addButton(m_monthlyButton);
        m_monthlyButton->setObjectName("m_monthlyButton");

        vboxLayout->addWidget(m_monthlyButton);

        m_yearlyButton = new QRadioButton(periodGroupBox);
        m_periodGroup->addButton(m_yearlyButton);
        m_yearlyButton->setObjectName("m_yearlyButton");

        vboxLayout->addWidget(m_yearlyButton);

        m_individualButton = new QRadioButton(periodGroupBox);
        m_periodGroup->addButton(m_individualButton);
        m_individualButton->setObjectName("m_individualButton");

        vboxLayout->addWidget(m_individualButton);

        spacer10 = new QSpacerItem(20, 21, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer10);


        gridLayout->addWidget(periodGroupBox, 0, 0, 6, 1);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName("vboxLayout1");
        m_clearButton = new QPushButton(KBudgetValues);
        m_clearButton->setObjectName("m_clearButton");

        vboxLayout1->addWidget(m_clearButton);

        spacer7 = new QSpacerItem(20, 21, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacer7);


        gridLayout->addLayout(vboxLayout1, 0, 1, 6, 1);

        m_label1 = new QLabel(KBudgetValues);
        m_label1->setObjectName("m_label1");
        m_label1->setWordWrap(false);

        gridLayout->addWidget(m_label1, 0, 2, 1, 1);

        m_firstItemStack = new QStackedWidget(KBudgetValues);
        m_firstItemStack->setObjectName("m_firstItemStack");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_firstItemStack->sizePolicy().hasHeightForWidth());
        m_firstItemStack->setSizePolicy(sizePolicy);
        m_monthlyPage = new QWidget();
        m_monthlyPage->setObjectName("m_monthlyPage");
        hboxLayout = new QHBoxLayout(m_monthlyPage);
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        m_amountMonthly = new AmountEdit(m_monthlyPage);
        m_amountMonthly->setObjectName("m_amountMonthly");

        hboxLayout->addWidget(m_amountMonthly);

        m_firstItemStack->addWidget(m_monthlyPage);
        m_yearlyPage = new QWidget();
        m_yearlyPage->setObjectName("m_yearlyPage");
        hboxLayout1 = new QHBoxLayout(m_yearlyPage);
        hboxLayout1->setSpacing(6);
        hboxLayout1->setContentsMargins(11, 11, 11, 11);
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        m_amountYearly = new AmountEdit(m_yearlyPage);
        m_amountYearly->setObjectName("m_amountYearly");

        hboxLayout1->addWidget(m_amountYearly);

        m_firstItemStack->addWidget(m_yearlyPage);
        m_individualPage = new QWidget();
        m_individualPage->setObjectName("m_individualPage");
        hboxLayout2 = new QHBoxLayout(m_individualPage);
        hboxLayout2->setSpacing(6);
        hboxLayout2->setContentsMargins(11, 11, 11, 11);
        hboxLayout2->setObjectName("hboxLayout2");
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        m_amount1 = new AmountEdit(m_individualPage);
        m_amount1->setObjectName("m_amount1");

        hboxLayout2->addWidget(m_amount1);

        m_firstItemStack->addWidget(m_individualPage);

        gridLayout->addWidget(m_firstItemStack, 0, 3, 1, 1);

        m_label2 = new QLabel(KBudgetValues);
        m_label2->setObjectName("m_label2");
        m_label2->setWordWrap(false);

        gridLayout->addWidget(m_label2, 1, 2, 1, 1);

        m_amount2 = new AmountEdit(KBudgetValues);
        m_amount2->setObjectName("m_amount2");

        gridLayout->addWidget(m_amount2, 1, 3, 1, 1);

        m_label3 = new QLabel(KBudgetValues);
        m_label3->setObjectName("m_label3");
        m_label3->setWordWrap(false);

        gridLayout->addWidget(m_label3, 2, 2, 1, 1);

        m_amount3 = new AmountEdit(KBudgetValues);
        m_amount3->setObjectName("m_amount3");

        gridLayout->addWidget(m_amount3, 2, 3, 1, 1);

        m_label4 = new QLabel(KBudgetValues);
        m_label4->setObjectName("m_label4");
        m_label4->setWordWrap(false);

        gridLayout->addWidget(m_label4, 3, 2, 1, 1);

        m_amount4 = new AmountEdit(KBudgetValues);
        m_amount4->setObjectName("m_amount4");

        gridLayout->addWidget(m_amount4, 3, 3, 1, 1);

        m_label5 = new QLabel(KBudgetValues);
        m_label5->setObjectName("m_label5");
        m_label5->setWordWrap(false);

        gridLayout->addWidget(m_label5, 4, 2, 1, 1);

        m_amount5 = new AmountEdit(KBudgetValues);
        m_amount5->setObjectName("m_amount5");

        gridLayout->addWidget(m_amount5, 4, 3, 1, 1);

        m_label6 = new QLabel(KBudgetValues);
        m_label6->setObjectName("m_label6");
        m_label6->setWordWrap(false);

        gridLayout->addWidget(m_label6, 5, 2, 1, 1);

        m_amount6 = new AmountEdit(KBudgetValues);
        m_amount6->setObjectName("m_amount6");

        gridLayout->addWidget(m_amount6, 5, 3, 1, 1);

        m_label7 = new QLabel(KBudgetValues);
        m_label7->setObjectName("m_label7");
        m_label7->setWordWrap(false);

        gridLayout->addWidget(m_label7, 0, 4, 1, 1);

        m_amount7 = new AmountEdit(KBudgetValues);
        m_amount7->setObjectName("m_amount7");

        gridLayout->addWidget(m_amount7, 0, 5, 1, 1);

        m_label8 = new QLabel(KBudgetValues);
        m_label8->setObjectName("m_label8");
        m_label8->setWordWrap(false);

        gridLayout->addWidget(m_label8, 1, 4, 1, 1);

        m_amount8 = new AmountEdit(KBudgetValues);
        m_amount8->setObjectName("m_amount8");

        gridLayout->addWidget(m_amount8, 1, 5, 1, 1);

        m_label9 = new QLabel(KBudgetValues);
        m_label9->setObjectName("m_label9");
        m_label9->setWordWrap(false);

        gridLayout->addWidget(m_label9, 2, 4, 1, 1);

        m_amount9 = new AmountEdit(KBudgetValues);
        m_amount9->setObjectName("m_amount9");

        gridLayout->addWidget(m_amount9, 2, 5, 1, 1);

        m_label10 = new QLabel(KBudgetValues);
        m_label10->setObjectName("m_label10");
        m_label10->setWordWrap(false);

        gridLayout->addWidget(m_label10, 3, 4, 1, 1);

        m_amount10 = new AmountEdit(KBudgetValues);
        m_amount10->setObjectName("m_amount10");

        gridLayout->addWidget(m_amount10, 3, 5, 1, 1);

        m_label11 = new QLabel(KBudgetValues);
        m_label11->setObjectName("m_label11");
        m_label11->setWordWrap(false);

        gridLayout->addWidget(m_label11, 4, 4, 1, 1);

        m_amount11 = new AmountEdit(KBudgetValues);
        m_amount11->setObjectName("m_amount11");

        gridLayout->addWidget(m_amount11, 4, 5, 1, 1);

        m_label12 = new QLabel(KBudgetValues);
        m_label12->setObjectName("m_label12");
        m_label12->setWordWrap(false);

        gridLayout->addWidget(m_label12, 5, 4, 1, 1);

        m_amount12 = new AmountEdit(KBudgetValues);
        m_amount12->setObjectName("m_amount12");

        gridLayout->addWidget(m_amount12, 5, 5, 1, 1);

        QWidget::setTabOrder(m_monthlyButton, m_yearlyButton);
        QWidget::setTabOrder(m_yearlyButton, m_individualButton);
        QWidget::setTabOrder(m_individualButton, m_clearButton);
        QWidget::setTabOrder(m_clearButton, m_amount1);
        QWidget::setTabOrder(m_amount1, m_amountMonthly);
        QWidget::setTabOrder(m_amountMonthly, m_amountYearly);
        QWidget::setTabOrder(m_amountYearly, m_amount2);
        QWidget::setTabOrder(m_amount2, m_amount3);
        QWidget::setTabOrder(m_amount3, m_amount4);
        QWidget::setTabOrder(m_amount4, m_amount5);
        QWidget::setTabOrder(m_amount5, m_amount6);
        QWidget::setTabOrder(m_amount6, m_amount7);
        QWidget::setTabOrder(m_amount7, m_amount8);
        QWidget::setTabOrder(m_amount8, m_amount9);
        QWidget::setTabOrder(m_amount9, m_amount10);
        QWidget::setTabOrder(m_amount10, m_amount11);
        QWidget::setTabOrder(m_amount11, m_amount12);

        retranslateUi(KBudgetValues);

        QMetaObject::connectSlotsByName(KBudgetValues);
    } // setupUi

    void retranslateUi(QWidget *KBudgetValues)
    {
        periodGroupBox->setTitle(tr2i18n("Period", nullptr));
        m_monthlyButton->setText(tr2i18n("Monthly", "@action timeframe of the budget"));
        m_yearlyButton->setText(tr2i18n("Yearly", "@action timeframe of the budget"));
        m_individualButton->setText(tr2i18n("Individual", "@action timeframe of the budget. month-by-month budget"));
        m_clearButton->setText(QString());
        m_label1->setText(tr2i18n("xxx", nullptr));
        m_label2->setText(tr2i18n("xxx", nullptr));
        m_label3->setText(tr2i18n("xxx", nullptr));
        m_label4->setText(tr2i18n("xxx", nullptr));
        m_label5->setText(tr2i18n("xxx", nullptr));
        m_label6->setText(tr2i18n("xxx", nullptr));
        m_label7->setText(tr2i18n("xxx", nullptr));
        m_label8->setText(tr2i18n("xxx", nullptr));
        m_label9->setText(tr2i18n("xxx", nullptr));
        m_label10->setText(tr2i18n("xxx", nullptr));
        m_label11->setText(tr2i18n("xxx", nullptr));
        m_label12->setText(tr2i18n("xxx", nullptr));
        (void)KBudgetValues;
    } // retranslateUi

};

namespace Ui {
    class KBudgetValues: public Ui_KBudgetValues {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KBUDGETVALUES_H

