// This file is generated by kconfig_compiler_kf6 from reportsviewsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef REPORTSVIEWSETTINGS_H
#define REPORTSVIEWSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class ReportsViewSettings : public KConfigSkeleton
{
  public:
    static ReportsViewSettings *self();
    ~ReportsViewSettings() override;

  protected:
    ReportsViewSettings();
    friend class ReportsViewSettingsHelper;

};

#endif

