/********************************************************************************
** Form generated from reading UI file 'kaccountpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KACCOUNTPAGE_H
#define UI_KACCOUNTPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "amountedit.h"
#include "kdatecombobox.h"
#include "klineedit.h"
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_KAccountPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel2;
    QCheckBox *m_haveCheckingAccountButton;
    QFrame *m_accountDataFrame;
    QGridLayout *gridLayout;
    KLineEdit *m_institutionNameEdit;
    KLineEdit *m_institutionNumberEdit;
    QLabel *textLabel3;
    QLabel *textLabel4;
    KLineEdit *m_accountNumberEdit;
    KLineEdit *m_accountNameEdit;
    QLabel *textLabel1_2;
    QHBoxLayout *hboxLayout;
    KMyMoneyDateEdit *m_openingDateEdit;
    QSpacerItem *spacer5;
    QLabel *textLabel2;
    QLabel *textLabel1;
    QLabel *textLabel2_2;
    QHBoxLayout *hboxLayout1;
    QLabel *m_accountCurrencyLabel;
    AmountEdit *m_openingBalanceEdit;
    QSpacerItem *spacer6;
    QLabel *qLabel1;
    QSpacerItem *spacer1;

    void setupUi(QWidget *KAccountPage)
    {
        if (KAccountPage->objectName().isEmpty())
            KAccountPage->setObjectName("KAccountPage");
        KAccountPage->resize(600, 478);
        vboxLayout = new QVBoxLayout(KAccountPage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        qLabel2 = new QLabel(KAccountPage);
        qLabel2->setObjectName("qLabel2");
        qLabel2->setFocusPolicy(Qt::NoFocus);
        qLabel2->setWordWrap(true);

        vboxLayout->addWidget(qLabel2);

        m_haveCheckingAccountButton = new QCheckBox(KAccountPage);
        m_haveCheckingAccountButton->setObjectName("m_haveCheckingAccountButton");

        vboxLayout->addWidget(m_haveCheckingAccountButton);

        m_accountDataFrame = new QFrame(KAccountPage);
        m_accountDataFrame->setObjectName("m_accountDataFrame");
        m_accountDataFrame->setFrameShape(QFrame::NoFrame);
        m_accountDataFrame->setFrameShadow(QFrame::Plain);
        gridLayout = new QGridLayout(m_accountDataFrame);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        m_institutionNameEdit = new KLineEdit(m_accountDataFrame);
        m_institutionNameEdit->setObjectName("m_institutionNameEdit");

        gridLayout->addWidget(m_institutionNameEdit, 5, 1, 1, 1);

        m_institutionNumberEdit = new KLineEdit(m_accountDataFrame);
        m_institutionNumberEdit->setObjectName("m_institutionNumberEdit");

        gridLayout->addWidget(m_institutionNumberEdit, 6, 1, 1, 1);

        textLabel3 = new QLabel(m_accountDataFrame);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout->addWidget(textLabel3, 5, 0, 1, 1);

        textLabel4 = new QLabel(m_accountDataFrame);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 6, 0, 1, 1);

        m_accountNumberEdit = new KLineEdit(m_accountDataFrame);
        m_accountNumberEdit->setObjectName("m_accountNumberEdit");

        gridLayout->addWidget(m_accountNumberEdit, 2, 1, 1, 1);

        m_accountNameEdit = new KLineEdit(m_accountDataFrame);
        m_accountNameEdit->setObjectName("m_accountNameEdit");

        gridLayout->addWidget(m_accountNameEdit, 1, 1, 1, 1);

        textLabel1_2 = new QLabel(m_accountDataFrame);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        gridLayout->addWidget(textLabel1_2, 3, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_openingDateEdit = new KMyMoneyDateEdit(m_accountDataFrame);
        m_openingDateEdit->setObjectName("m_openingDateEdit");

        hboxLayout->addWidget(m_openingDateEdit);

        spacer5 = new QSpacerItem(81, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer5);


        gridLayout->addLayout(hboxLayout, 3, 1, 1, 1);

        textLabel2 = new QLabel(m_accountDataFrame);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 2, 0, 1, 1);

        textLabel1 = new QLabel(m_accountDataFrame);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 1, 0, 1, 1);

        textLabel2_2 = new QLabel(m_accountDataFrame);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setWordWrap(false);

        gridLayout->addWidget(textLabel2_2, 4, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        m_accountCurrencyLabel = new QLabel(m_accountDataFrame);
        m_accountCurrencyLabel->setObjectName("m_accountCurrencyLabel");
        m_accountCurrencyLabel->setWordWrap(false);

        hboxLayout1->addWidget(m_accountCurrencyLabel);

        m_openingBalanceEdit = new AmountEdit(m_accountDataFrame);
        m_openingBalanceEdit->setObjectName("m_openingBalanceEdit");
        m_openingBalanceEdit->setProperty("resetButtonVisibility", QVariant(false));

        hboxLayout1->addWidget(m_openingBalanceEdit);

        spacer6 = new QSpacerItem(200, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacer6);


        gridLayout->addLayout(hboxLayout1, 4, 1, 1, 1);

        qLabel1 = new QLabel(m_accountDataFrame);
        qLabel1->setObjectName("qLabel1");
        qLabel1->setFocusPolicy(Qt::NoFocus);
        qLabel1->setWordWrap(true);

        gridLayout->addWidget(qLabel1, 0, 0, 1, 2);


        vboxLayout->addWidget(m_accountDataFrame);

        spacer1 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer1);

        QWidget::setTabOrder(m_accountNameEdit, m_accountNumberEdit);
        QWidget::setTabOrder(m_accountNumberEdit, m_openingDateEdit);
        QWidget::setTabOrder(m_openingDateEdit, m_openingBalanceEdit);
        QWidget::setTabOrder(m_openingBalanceEdit, m_institutionNameEdit);
        QWidget::setTabOrder(m_institutionNameEdit, m_institutionNumberEdit);
        QWidget::setTabOrder(m_institutionNumberEdit, m_haveCheckingAccountButton);

        retranslateUi(KAccountPage);
        QObject::connect(m_haveCheckingAccountButton, &QCheckBox::toggled, m_accountDataFrame, &QFrame::setEnabled);

        QMetaObject::connectSlotsByName(KAccountPage);
    } // setupUi

    void retranslateUi(QWidget *KAccountPage)
    {
        qLabel2->setText(tr2i18n("A typical KMyMoney user maintains a checking account with some institution and uses it to receive money or pay bills. If this is not the case, please deselect the checkbox below and continue on the next page.", nullptr));
        m_haveCheckingAccountButton->setText(tr2i18n("I have a checking account that I want to manage with KMyMoney", nullptr));
#if QT_CONFIG(whatsthis)
        m_institutionNameEdit->setWhatsThis(tr2i18n("The name of the institution that issued the account.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        m_institutionNumberEdit->setWhatsThis(tr2i18n("Each institution has a bank code, known as routing number in the US or a sort code in the UK. Enter it here. If uncertain, leave the field blank. You can modify it later.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel3->setText(tr2i18n("Name of the institution", nullptr));
        textLabel4->setText(tr2i18n("Bank Code", nullptr));
#if QT_CONFIG(whatsthis)
        m_accountNumberEdit->setWhatsThis(tr2i18n("In general the institution that issued the account also issued a number for it. Enter it here. If uncertain, you can modify this entry later.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        m_accountNameEdit->setWhatsThis(tr2i18n("Enter a descriptive name that will be used by KMyMoney to identify your checking account.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1_2->setText(tr2i18n("Opening date", nullptr));
#if QT_CONFIG(whatsthis)
        m_openingDateEdit->setWhatsThis(tr2i18n("Enter the date from when on you plan to keep track of the transactions in that account. This is usually the date of the last statement. If uncertain, leave as is.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel2->setText(tr2i18n("Number of the account", nullptr));
        textLabel1->setText(tr2i18n("Name of the account", nullptr));
        textLabel2_2->setText(tr2i18n("Opening balance", nullptr));
        m_accountCurrencyLabel->setText(tr2i18n("xxx", nullptr));
#if QT_CONFIG(whatsthis)
        m_openingBalanceEdit->setWhatsThis(tr2i18n("Enter the opening balance of the account. This is usually the ending balance of the last statement. If uncertain, leave as is.", nullptr));
#endif // QT_CONFIG(whatsthis)
        qLabel1->setText(tr2i18n("The fields below allow you to enter some information about your checking account. Once this wizard finishes, the account will be created for you within KMyMoney.", nullptr));
        (void)KAccountPage;
    } // retranslateUi

};

namespace Ui {
    class KAccountPage: public Ui_KAccountPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KACCOUNTPAGE_H

