/********************************************************************************
** Form generated from reading UI file 'ksettingsfonts.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KSETTINGSFONTS_H
#define UI_KSETTINGSFONTS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kfontrequester.h"

QT_BEGIN_NAMESPACE

class Ui_KSettingsFonts
{
public:
    QVBoxLayout *vboxLayout;
    QCheckBox *kcfg_useSystemFont;
    QGroupBox *m_individualFontsFrame;
    QGridLayout *gridLayout;
    KFontRequester *kcfg_listCellFont;
    KFontRequester *kcfg_listHeaderFont;
    QSpacerItem *spacer2;
    QSpacerItem *spacer3;
    QLabel *textLabel1;
    QLabel *textLabel2;
    QSpacerItem *spacer1;

    void setupUi(QWidget *KSettingsFonts)
    {
        if (KSettingsFonts->objectName().isEmpty())
            KSettingsFonts->setObjectName("KSettingsFonts");
        KSettingsFonts->resize(398, 222);
        vboxLayout = new QVBoxLayout(KSettingsFonts);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        kcfg_useSystemFont = new QCheckBox(KSettingsFonts);
        kcfg_useSystemFont->setObjectName("kcfg_useSystemFont");

        vboxLayout->addWidget(kcfg_useSystemFont);

        m_individualFontsFrame = new QGroupBox(KSettingsFonts);
        m_individualFontsFrame->setObjectName("m_individualFontsFrame");
        gridLayout = new QGridLayout(m_individualFontsFrame);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        kcfg_listCellFont = new KFontRequester(m_individualFontsFrame);
        kcfg_listCellFont->setObjectName("kcfg_listCellFont");

        gridLayout->addWidget(kcfg_listCellFont, 0, 2, 1, 1);

        kcfg_listHeaderFont = new KFontRequester(m_individualFontsFrame);
        kcfg_listHeaderFont->setObjectName("kcfg_listHeaderFont");

        gridLayout->addWidget(kcfg_listHeaderFont, 1, 2, 1, 1);

        spacer2 = new QSpacerItem(110, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacer2, 0, 1, 1, 1);

        spacer3 = new QSpacerItem(100, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacer3, 1, 1, 1, 1);

        textLabel1 = new QLabel(m_individualFontsFrame);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        textLabel2 = new QLabel(m_individualFontsFrame);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 1, 0, 1, 1);


        vboxLayout->addWidget(m_individualFontsFrame);

        spacer1 = new QSpacerItem(20, 30, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer1);


        retranslateUi(KSettingsFonts);
        QObject::connect(kcfg_useSystemFont, &QCheckBox::toggled, m_individualFontsFrame, &QGroupBox::setDisabled);

        QMetaObject::connectSlotsByName(KSettingsFonts);
    } // setupUi

    void retranslateUi(QWidget *KSettingsFonts)
    {
        KSettingsFonts->setWindowTitle(tr2i18n("Font settings", nullptr));
        kcfg_useSystemFont->setText(tr2i18n("Use system fonts", nullptr));
        m_individualFontsFrame->setTitle(tr2i18n("Individual font settings for ledgers", nullptr));
        textLabel1->setText(tr2i18n("Cell font", nullptr));
        textLabel2->setText(tr2i18n("Header font", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KSettingsFonts: public Ui_KSettingsFonts {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KSETTINGSFONTS_H

