// This file is generated by kconfig_compiler_kf6 from checkprintingsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef CHECKPRINTINGSETTINGS_H
#define CHECKPRINTINGSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class CheckPrintingSettings : public KConfigSkeleton
{
  public:

    static CheckPrintingSettings *self();
    ~CheckPrintingSettings() override;

    /**
      Set Use custom check template
    */
    static
    void setUseCustomCheckTemplate( bool v )
    {
      if (!self()->isUseCustomCheckTemplateImmutable())
        self()->mUseCustomCheckTemplate = v;
    }

    /**
      Get Use custom check template
    */
    static
    bool useCustomCheckTemplate()
    {
      return self()->mUseCustomCheckTemplate;
    }

    /**
      Is Use custom check template Immutable
    */
    static
    bool isUseCustomCheckTemplateImmutable()
    {
      return self()->isImmutable( QStringLiteral( "useCustomCheckTemplate" ) );
    }

    /**
      Get Use custom check template default value
    */
    static
    bool defaultUseCustomCheckTemplateValue()
    {
        return defaultUseCustomCheckTemplateValue_helper();
    }

    /**
      Set The check template file which defines the way the printed check will look.
    */
    static
    void setCheckTemplateFile( const QString & v )
    {
      if (!self()->isCheckTemplateFileImmutable())
        self()->mCheckTemplateFile = v;
    }

    /**
      Get The check template file which defines the way the printed check will look.
    */
    static
    QString checkTemplateFile()
    {
      return self()->mCheckTemplateFile;
    }

    /**
      Is The check template file which defines the way the printed check will look. Immutable
    */
    static
    bool isCheckTemplateFileImmutable()
    {
      return self()->isImmutable( QStringLiteral( "checkTemplateFile" ) );
    }

    /**
      Get The check template file which defines the way the printed check will look. default value
    */
    static
    QString defaultCheckTemplateFileValue()
    {
        return defaultCheckTemplateFileValue_helper();
    }

    /**
      Set A list containing the id's of already printed checks.
    */
    static
    void setPrintedChecks( const QStringList & v )
    {
      if (!self()->isPrintedChecksImmutable())
        self()->mPrintedChecks = v;
    }

    /**
      Get A list containing the id's of already printed checks.
    */
    static
    QStringList printedChecks()
    {
      return self()->mPrintedChecks;
    }

    /**
      Is A list containing the id's of already printed checks. Immutable
    */
    static
    bool isPrintedChecksImmutable()
    {
      return self()->isImmutable( QStringLiteral( "printedChecks" ) );
    }

  protected:
    CheckPrintingSettings();
    friend class CheckPrintingSettingsHelper;


    // General
    bool mUseCustomCheckTemplate;
    static bool defaultUseCustomCheckTemplateValue_helper();
    QString mCheckTemplateFile;
    static QString defaultCheckTemplateFileValue_helper();
    QStringList mPrintedChecks;
    static QStringList defaultPrintedChecksValue_helper();
};

#endif

