/****************************************************************************
** Meta object code from reading C++ file 'ledgerviewpage.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/views/ledgerviewpage.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ledgerviewpage.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14LedgerViewPageE_t {};
} // unnamed namespace

template <> constexpr inline auto LedgerViewPage::qt_create_metaobjectdata<qt_meta_tag_ZN14LedgerViewPageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LedgerViewPage",
        "requestSelectionChanged",
        "",
        "SelectedObjects",
        "selection",
        "requestCustomContextMenu",
        "eMenu::Menu",
        "type",
        "QPoint",
        "pos",
        "transactionSelected",
        "QModelIndex",
        "idx",
        "aboutToStartEdit",
        "aboutToFinishEdit",
        "resizeSection",
        "QWidget*",
        "view",
        "configGroupName",
        "section",
        "oldSize",
        "newSize",
        "sectionResized",
        "moveSection",
        "oldIndex",
        "newIndex",
        "sectionMoved",
        "requestView",
        "viewWidget",
        "accountId",
        "journalEntryId",
        "editTransaction",
        "id",
        "isEditing",
        "searchWidgetPinned",
        "pinned",
        "showTransactionForm",
        "show",
        "splitterChanged",
        "index",
        "slotSettingsChanged",
        "updateSummaryInformation",
        "QHash<QString,AccountBalances>",
        "balances",
        "pinSearchWidget",
        "startEdit",
        "finishEdit",
        "keepSelection",
        "reloadFilter",
        "slotRequestSelectionChanged",
        "selections"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestSelectionChanged'
        QtMocHelpers::SignalData<void(const SelectedObjects &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'requestCustomContextMenu'
        QtMocHelpers::SignalData<void(eMenu::Menu, const QPoint &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'transactionSelected'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'aboutToStartEdit'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aboutToFinishEdit'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resizeSection'
        QtMocHelpers::SignalData<void(QWidget *, const QString &, int, int, int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::QString, 18 }, { QMetaType::Int, 19 }, { QMetaType::Int, 20 },
            { QMetaType::Int, 21 },
        }}),
        // Signal 'sectionResized'
        QtMocHelpers::SignalData<void(QWidget *, const QString &, int, int, int)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::QString, 18 }, { QMetaType::Int, 19 }, { QMetaType::Int, 20 },
            { QMetaType::Int, 21 },
        }}),
        // Signal 'moveSection'
        QtMocHelpers::SignalData<void(QWidget *, int, int, int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::Int, 19 }, { QMetaType::Int, 24 }, { QMetaType::Int, 25 },
        }}),
        // Signal 'sectionMoved'
        QtMocHelpers::SignalData<void(QWidget *, int, int, int)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::Int, 19 }, { QMetaType::Int, 24 }, { QMetaType::Int, 25 },
        }}),
        // Signal 'requestView'
        QtMocHelpers::SignalData<void(QWidget *, const QString &, const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 28 }, { QMetaType::QString, 29 }, { QMetaType::QString, 30 },
        }}),
        // Signal 'editTransaction'
        QtMocHelpers::SignalData<void(void *, bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::VoidStar, 32 }, { QMetaType::Bool, 33 },
        }}),
        // Signal 'searchWidgetPinned'
        QtMocHelpers::SignalData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'showTransactionForm'
        QtMocHelpers::SlotData<void(bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 37 },
        }}),
        // Slot 'splitterChanged'
        QtMocHelpers::SlotData<void(int, int)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { QMetaType::Int, 39 },
        }}),
        // Slot 'slotSettingsChanged'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateSummaryInformation'
        QtMocHelpers::SlotData<void(const QHash<QString,AccountBalances> &)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 42, 43 },
        }}),
        // Slot 'pinSearchWidget'
        QtMocHelpers::SlotData<void(bool)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'startEdit'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'finishEdit'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'keepSelection'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'reloadFilter'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRequestSelectionChanged'
        QtMocHelpers::SlotData<void(const SelectedObjects &) const>(49, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 50 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LedgerViewPage, qt_meta_tag_ZN14LedgerViewPageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LedgerViewPage::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LedgerViewPageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LedgerViewPageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14LedgerViewPageE_t>.metaTypes,
    nullptr
} };

void LedgerViewPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LedgerViewPage *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestSelectionChanged((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 1: _t->requestCustomContextMenu((*reinterpret_cast<std::add_pointer_t<eMenu::Menu>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 2: _t->transactionSelected((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 3: _t->aboutToStartEdit(); break;
        case 4: _t->aboutToFinishEdit(); break;
        case 5: _t->resizeSection((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 6: _t->sectionResized((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 7: _t->moveSection((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 8: _t->sectionMoved((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 9: _t->requestView((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 10: _t->editTransaction((*reinterpret_cast<std::add_pointer_t<void*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->searchWidgetPinned((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->showTransactionForm((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->splitterChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 14: _t->slotSettingsChanged(); break;
        case 15: _t->updateSummaryInformation((*reinterpret_cast<std::add_pointer_t<QHash<QString,AccountBalances>>>(_a[1]))); break;
        case 16: _t->pinSearchWidget((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->startEdit(); break;
        case 18: _t->finishEdit(); break;
        case 19: _t->keepSelection(); break;
        case 20: _t->reloadFilter(); break;
        case 21: _t->slotRequestSelectionChanged((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LedgerViewPage::*)(const SelectedObjects & )>(_a, &LedgerViewPage::requestSelectionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerViewPage::*)(eMenu::Menu , const QPoint & )>(_a, &LedgerViewPage::requestCustomContextMenu, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerViewPage::*)(const QModelIndex & )>(_a, &LedgerViewPage::transactionSelected, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerViewPage::*)()>(_a, &LedgerViewPage::aboutToStartEdit, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerViewPage::*)()>(_a, &LedgerViewPage::aboutToFinishEdit, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerViewPage::*)(QWidget * , const QString & , int , int , int )>(_a, &LedgerViewPage::resizeSection, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerViewPage::*)(QWidget * , const QString & , int , int , int )>(_a, &LedgerViewPage::sectionResized, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerViewPage::*)(QWidget * , int , int , int )>(_a, &LedgerViewPage::moveSection, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerViewPage::*)(QWidget * , int , int , int )>(_a, &LedgerViewPage::sectionMoved, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerViewPage::*)(QWidget * , const QString & , const QString & )>(_a, &LedgerViewPage::requestView, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerViewPage::*)(void * , bool )>(_a, &LedgerViewPage::editTransaction, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerViewPage::*)(bool )>(_a, &LedgerViewPage::searchWidgetPinned, 11))
            return;
    }
}

const QMetaObject *LedgerViewPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LedgerViewPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LedgerViewPageE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int LedgerViewPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void LedgerViewPage::requestSelectionChanged(const SelectedObjects & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void LedgerViewPage::requestCustomContextMenu(eMenu::Menu _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void LedgerViewPage::transactionSelected(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void LedgerViewPage::aboutToStartEdit()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void LedgerViewPage::aboutToFinishEdit()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void LedgerViewPage::resizeSection(QWidget * _t1, const QString & _t2, int _t3, int _t4, int _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 6
void LedgerViewPage::sectionResized(QWidget * _t1, const QString & _t2, int _t3, int _t4, int _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 7
void LedgerViewPage::moveSection(QWidget * _t1, int _t2, int _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 8
void LedgerViewPage::sectionMoved(QWidget * _t1, int _t2, int _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 9
void LedgerViewPage::requestView(QWidget * _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2, _t3);
}

// SIGNAL 10
void LedgerViewPage::editTransaction(void * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void LedgerViewPage::searchWidgetPinned(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}
QT_WARNING_POP
