/********************************************************************************
** Form generated from reading UI file 'reportsviewsettings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REPORTSVIEWSETTINGS_H
#define UI_REPORTSVIEWSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_ReportsViewSettings
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox6;
    QGridLayout *gridLayout;
    QLabel *label_1;
    QHBoxLayout *horizontalLayout_6;
    QSpinBox *kcfg_MaximumLegendItems;
    QSpacerItem *horizontalSpacer;
    QLabel *label_3;
    QHBoxLayout *horizontalLayout_7;
    QSpinBox *kcfg_LineWidth;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    KComboBox *kcfg_ChartsPalette;
    QSpacerItem *horizontalSpacer_3;
    QLabel *label_4;
    KUrlRequester *kcfg_CssFileDefault;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *ReportsViewSettings)
    {
        if (ReportsViewSettings->objectName().isEmpty())
            ReportsViewSettings->setObjectName("ReportsViewSettings");
        ReportsViewSettings->resize(614, 468);
        vboxLayout = new QVBoxLayout(ReportsViewSettings);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        groupBox6 = new QGroupBox(ReportsViewSettings);
        groupBox6->setObjectName("groupBox6");
        gridLayout = new QGridLayout(groupBox6);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        label_1 = new QLabel(groupBox6);
        label_1->setObjectName("label_1");

        gridLayout->addWidget(label_1, 0, 0, 1, 1);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setSpacing(6);
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        kcfg_MaximumLegendItems = new QSpinBox(groupBox6);
        kcfg_MaximumLegendItems->setObjectName("kcfg_MaximumLegendItems");

        horizontalLayout_6->addWidget(kcfg_MaximumLegendItems);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer);


        gridLayout->addLayout(horizontalLayout_6, 0, 1, 1, 1);

        label_3 = new QLabel(groupBox6);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 1, 0, 1, 1);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setSpacing(6);
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        kcfg_LineWidth = new QSpinBox(groupBox6);
        kcfg_LineWidth->setObjectName("kcfg_LineWidth");

        horizontalLayout_7->addWidget(kcfg_LineWidth);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_2);


        gridLayout->addLayout(horizontalLayout_7, 1, 1, 1, 1);

        label = new QLabel(groupBox6);
        label->setObjectName("label");

        gridLayout->addWidget(label, 2, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        kcfg_ChartsPalette = new KComboBox(groupBox6);
        kcfg_ChartsPalette->addItem(QString());
        kcfg_ChartsPalette->addItem(QString());
        kcfg_ChartsPalette->addItem(QString());
        kcfg_ChartsPalette->setObjectName("kcfg_ChartsPalette");

        horizontalLayout->addWidget(kcfg_ChartsPalette);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);


        gridLayout->addLayout(horizontalLayout, 2, 1, 1, 1);

        label_4 = new QLabel(groupBox6);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 3, 0, 1, 1);

        kcfg_CssFileDefault = new KUrlRequester(groupBox6);
        kcfg_CssFileDefault->setObjectName("kcfg_CssFileDefault");
        kcfg_CssFileDefault->setProperty("kcfg_property", QVariant(QByteArray("text")));

        gridLayout->addWidget(kcfg_CssFileDefault, 3, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 4, 1, 1, 1);


        vboxLayout->addWidget(groupBox6);

        QWidget::setTabOrder(kcfg_MaximumLegendItems, kcfg_LineWidth);

        retranslateUi(ReportsViewSettings);

        kcfg_ChartsPalette->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ReportsViewSettings);
    } // setupUi

    void retranslateUi(QWidget *ReportsViewSettings)
    {
        ReportsViewSettings->setWindowTitle(tr2i18n("Report Settings", nullptr));
        groupBox6->setTitle(QString());
        label_1->setText(tr2i18n("Maximum number of legend items to display", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_MaximumLegendItems->setToolTip(tr2i18n("maximum number of items in diagram legends", nullptr));
#endif // QT_CONFIG(tooltip)
        label_3->setText(tr2i18n("Charts line width", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_LineWidth->setToolTip(tr2i18n("Line width for diagrams, in pixels", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(tr2i18n("Charts palette", nullptr));
        kcfg_ChartsPalette->setItemText(0, tr2i18n("Default", "@item KDChart color palette"));
        kcfg_ChartsPalette->setItemText(1, tr2i18n("Rainbow", "@item KDChart color palette"));
        kcfg_ChartsPalette->setItemText(2, tr2i18n("Subdued", "@item KDChart color palette"));

        label_4->setText(tr2i18n("Default CSS file", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_CssFileDefault->setToolTip(tr2i18n("default style sheet", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class ReportsViewSettings: public Ui_ReportsViewSettings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // REPORTSVIEWSETTINGS_H

