/********************************************************************************
** Form generated from reading UI file 'kbrokeragepage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KBROKERAGEPAGE_H
#define UI_KBROKERAGEPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "klineedit.h"
#include "kmymoneycurrencyselector.h"

QT_BEGIN_NAMESPACE

class Ui_KBrokeragePage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel14;
    QCheckBox *m_createBrokerageButton;
    QFrame *m_currencyFrame;
    QHBoxLayout *hboxLayout;
    QGridLayout *gridLayout;
    QLabel *textLabel1;
    KMyMoneySecuritySelector *m_brokerageCurrency;
    QLabel *m_accountNumberLabel;
    KLineEdit *m_accountNumber;
    QLabel *m_ibanLabel;
    KLineEdit *m_iban;
    QSpacerItem *spacer2;
    QSpacerItem *spacer20;

    void setupUi(QWidget *KBrokeragePage)
    {
        if (KBrokeragePage->objectName().isEmpty())
            KBrokeragePage->setObjectName("KBrokeragePage");
        KBrokeragePage->resize(594, 398);
        vboxLayout = new QVBoxLayout(KBrokeragePage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(10, 10, 10, 10);
        vboxLayout->setObjectName("vboxLayout");
        qLabel14 = new QLabel(KBrokeragePage);
        qLabel14->setObjectName("qLabel14");
        qLabel14->setFocusPolicy(Qt::NoFocus);
        qLabel14->setWordWrap(true);

        vboxLayout->addWidget(qLabel14);

        m_createBrokerageButton = new QCheckBox(KBrokeragePage);
        m_createBrokerageButton->setObjectName("m_createBrokerageButton");

        vboxLayout->addWidget(m_createBrokerageButton);

        m_currencyFrame = new QFrame(KBrokeragePage);
        m_currencyFrame->setObjectName("m_currencyFrame");
        m_currencyFrame->setEnabled(false);
        m_currencyFrame->setFrameShape(QFrame::NoFrame);
        m_currencyFrame->setFrameShadow(QFrame::Raised);
        hboxLayout = new QHBoxLayout(m_currencyFrame);
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        textLabel1 = new QLabel(m_currencyFrame);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        m_brokerageCurrency = new KMyMoneySecuritySelector(m_currencyFrame);
        m_brokerageCurrency->setObjectName("m_brokerageCurrency");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_brokerageCurrency->sizePolicy().hasHeightForWidth());
        m_brokerageCurrency->setSizePolicy(sizePolicy);

        gridLayout->addWidget(m_brokerageCurrency, 0, 1, 1, 1);

        m_accountNumberLabel = new QLabel(m_currencyFrame);
        m_accountNumberLabel->setObjectName("m_accountNumberLabel");
        m_accountNumberLabel->setWordWrap(false);

        gridLayout->addWidget(m_accountNumberLabel, 1, 0, 1, 1);

        m_accountNumber = new KLineEdit(m_currencyFrame);
        m_accountNumber->setObjectName("m_accountNumber");

        gridLayout->addWidget(m_accountNumber, 1, 1, 1, 1);

        m_ibanLabel = new QLabel(m_currencyFrame);
        m_ibanLabel->setObjectName("m_ibanLabel");
        m_ibanLabel->setWordWrap(false);

        gridLayout->addWidget(m_ibanLabel, 2, 0, 1, 1);

        m_iban = new KLineEdit(m_currencyFrame);
        m_iban->setObjectName("m_iban");

        gridLayout->addWidget(m_iban, 2, 1, 1, 1);


        hboxLayout->addLayout(gridLayout);

        spacer2 = new QSpacerItem(30, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer2);


        vboxLayout->addWidget(m_currencyFrame);

        spacer20 = new QSpacerItem(21, 30, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer20);

        QWidget::setTabOrder(m_createBrokerageButton, m_brokerageCurrency);
        QWidget::setTabOrder(m_brokerageCurrency, m_accountNumber);
        QWidget::setTabOrder(m_accountNumber, m_iban);

        retranslateUi(KBrokeragePage);
        QObject::connect(m_createBrokerageButton, &QCheckBox::toggled, m_currencyFrame, &QFrame::setEnabled);

        QMetaObject::connectSlotsByName(KBrokeragePage);
    } // setupUi

    void retranslateUi(QWidget *KBrokeragePage)
    {
        qLabel14->setText(tr2i18n("Does this investment account have a brokerage account?<p>\n"
"\n"
"<b>Check</b> the mark if this investment account has an associated account for money that is not invested.<br>\n"
"\n"
"<b>Uncheck</b> it if you do not use this account for active stock brokerage. You should also uncheck it if the account is not maintained by a bank or broker.", nullptr));
        m_createBrokerageButton->setText(tr2i18n("Create brokerage account", nullptr));
        textLabel1->setText(tr2i18n("Currency", nullptr));
        m_accountNumberLabel->setText(tr2i18n("Account number", nullptr));
        m_ibanLabel->setText(tr2i18n("IBAN", nullptr));
        (void)KBrokeragePage;
    } // retranslateUi

};

namespace Ui {
    class KBrokeragePage: public Ui_KBrokeragePage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KBROKERAGEPAGE_H

