/********************************************************************************
** Form generated from reading UI file 'interesttypewizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INTERESTTYPEWIZARDPAGE_H
#define UI_INTERESTTYPEWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_InterestTypeWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel5_2;
    QSpacerItem *Spacer23;
    QHBoxLayout *hboxLayout;
    QFrame *frame_3;
    QVBoxLayout *vboxLayout1;
    QRadioButton *m_fixedInterestButton;
    QRadioButton *m_variableInterestButton;
    QSpacerItem *Spacer14_2;
    QSpacerItem *Spacer24;
    QButtonGroup *ButtonGroup2;

    void setupUi(QWizardPage *InterestTypeWizardPage)
    {
        if (InterestTypeWizardPage->objectName().isEmpty())
            InterestTypeWizardPage->setObjectName("InterestTypeWizardPage");
        vboxLayout = new QVBoxLayout(InterestTypeWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel5_2 = new QLabel(InterestTypeWizardPage);
        TextLabel5_2->setObjectName("TextLabel5_2");
        TextLabel5_2->setAlignment(Qt::AlignTop);
        TextLabel5_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel5_2);

        Spacer23 = new QSpacerItem(0, 53, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer23);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        frame_3 = new QFrame(InterestTypeWizardPage);
        frame_3->setObjectName("frame_3");
        frame_3->setFrameShape(QFrame::NoFrame);
        vboxLayout1 = new QVBoxLayout(frame_3);
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        m_fixedInterestButton = new QRadioButton(frame_3);
        ButtonGroup2 = new QButtonGroup(InterestTypeWizardPage);
        ButtonGroup2->setObjectName("ButtonGroup2");
        ButtonGroup2->addButton(m_fixedInterestButton);
        m_fixedInterestButton->setObjectName("m_fixedInterestButton");

        vboxLayout1->addWidget(m_fixedInterestButton);

        m_variableInterestButton = new QRadioButton(frame_3);
        ButtonGroup2->addButton(m_variableInterestButton);
        m_variableInterestButton->setObjectName("m_variableInterestButton");

        vboxLayout1->addWidget(m_variableInterestButton);


        hboxLayout->addWidget(frame_3);

        Spacer14_2 = new QSpacerItem(286, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer14_2);


        vboxLayout->addLayout(hboxLayout);

        Spacer24 = new QSpacerItem(0, 53, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer24);


        retranslateUi(InterestTypeWizardPage);

        QMetaObject::connectSlotsByName(InterestTypeWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *InterestTypeWizardPage)
    {
        TextLabel5_2->setText(tr2i18n("\n"
"Is the interest of this loan fixed over a period of time or is it adapted from time to time? If the interest rate changes during the amortization phase of the loan you should choose the option 'variable interest rate'.", nullptr));
        m_fixedInterestButton->setText(tr2i18n("Fixed interest rate", nullptr));
        m_variableInterestButton->setText(tr2i18n("Variable interest rate", nullptr));
        (void)InterestTypeWizardPage;
    } // retranslateUi

};

namespace Ui {
    class InterestTypeWizardPage: public Ui_InterestTypeWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // INTERESTTYPEWIZARDPAGE_H

