/****************************************************************************
** Meta object code from reading C++ file 'knewloanwizard.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../kmymoney-5.2.2/kmymoney/wizards/newloanwizard/knewloanwizard.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'knewloanwizard.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14KNewLoanWizardE_t {};
} // unnamed namespace

template <> constexpr inline auto KNewLoanWizard::qt_create_metaobjectdata<qt_meta_tag_ZN14KNewLoanWizardE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KNewLoanWizard",
        "newCategory",
        "",
        "MyMoneyAccount&",
        "acc",
        "createPayee",
        "txt",
        "QString&",
        "id",
        "slotReloadEditWidgets",
        "slotNewCategory",
        "slotNewPayee",
        "newnameBase"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newCategory'
        QtMocHelpers::SignalData<void(MyMoneyAccount &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'createPayee'
        QtMocHelpers::SignalData<void(const QString &, QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotReloadEditWidgets'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotNewCategory'
        QtMocHelpers::SlotData<void(MyMoneyAccount &)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotNewPayee'
        QtMocHelpers::SlotData<void(const QString &, QString &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KNewLoanWizard, qt_meta_tag_ZN14KNewLoanWizardE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KNewLoanWizard::staticMetaObject = { {
    QMetaObject::SuperData::link<QWizard::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KNewLoanWizardE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KNewLoanWizardE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14KNewLoanWizardE_t>.metaTypes,
    nullptr
} };

void KNewLoanWizard::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KNewLoanWizard *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newCategory((*reinterpret_cast<std::add_pointer_t<MyMoneyAccount&>>(_a[1]))); break;
        case 1: _t->createPayee((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[2]))); break;
        case 2: _t->slotReloadEditWidgets(); break;
        case 3: _t->slotNewCategory((*reinterpret_cast<std::add_pointer_t<MyMoneyAccount&>>(_a[1]))); break;
        case 4: _t->slotNewPayee((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KNewLoanWizard::*)(MyMoneyAccount & )>(_a, &KNewLoanWizard::newCategory, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KNewLoanWizard::*)(const QString & , QString & )>(_a, &KNewLoanWizard::createPayee, 1))
            return;
    }
}

const QMetaObject *KNewLoanWizard::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KNewLoanWizard::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KNewLoanWizardE_t>.strings))
        return static_cast<void*>(this);
    return QWizard::qt_metacast(_clname);
}

int KNewLoanWizard::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWizard::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void KNewLoanWizard::newCategory(MyMoneyAccount & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KNewLoanWizard::createPayee(const QString & _t1, QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
