/********************************************************************************
** Form generated from reading UI file 'checkingstatementinfowizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CHECKINGSTATEMENTINFOWIZARDPAGE_H
#define UI_CHECKINGSTATEMENTINFOWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>
#include <kmymoneydateedit.h>
#include "amountedit.h"

QT_BEGIN_NAMESPACE

class Ui_CheckingStatementInfoWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *m_enterInformationLabel;
    QSpacerItem *Spacer4;
    QHBoxLayout *hboxLayout;
    QSpacerItem *Spacer3;
    QGridLayout *gridLayout;
    QLabel *TextLabel3;
    KMyMoneyDateEdit *m_statementDate;
    QLabel *TextLabel2;
    AmountEdit *m_previousBalance;
    QLabel *TextLabel4;
    AmountEdit *m_endingBalance;
    QSpacerItem *Spacer2;
    QSpacerItem *Spacer5;
    QLabel *m_oldestTransactionDate;
    QLabel *m_lastStatementDate;

    void setupUi(QWizardPage *CheckingStatementInfoWizardPage)
    {
        if (CheckingStatementInfoWizardPage->objectName().isEmpty())
            CheckingStatementInfoWizardPage->setObjectName("CheckingStatementInfoWizardPage");
        CheckingStatementInfoWizardPage->resize(577, 388);
        vboxLayout = new QVBoxLayout(CheckingStatementInfoWizardPage);
        vboxLayout->setObjectName("vboxLayout");
        m_enterInformationLabel = new QLabel(CheckingStatementInfoWizardPage);
        m_enterInformationLabel->setObjectName("m_enterInformationLabel");
        m_enterInformationLabel->setWordWrap(true);

        vboxLayout->addWidget(m_enterInformationLabel);

        Spacer4 = new QSpacerItem(20, 50, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer4);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        Spacer3 = new QSpacerItem(42, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer3);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(0);
#ifndef UI_Q_OS_MAC
        gridLayout->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout->setObjectName("gridLayout");
        TextLabel3 = new QLabel(CheckingStatementInfoWizardPage);
        TextLabel3->setObjectName("TextLabel3");
        TextLabel3->setWordWrap(false);

        gridLayout->addWidget(TextLabel3, 0, 0, 1, 1);

        m_statementDate = new KMyMoneyDateEdit(CheckingStatementInfoWizardPage);
        m_statementDate->setObjectName("m_statementDate");

        gridLayout->addWidget(m_statementDate, 0, 1, 1, 1);

        TextLabel2 = new QLabel(CheckingStatementInfoWizardPage);
        TextLabel2->setObjectName("TextLabel2");
        TextLabel2->setWordWrap(false);

        gridLayout->addWidget(TextLabel2, 1, 0, 1, 1);

        m_previousBalance = new AmountEdit(CheckingStatementInfoWizardPage);
        m_previousBalance->setObjectName("m_previousBalance");
        m_previousBalance->setEnabled(false);

        gridLayout->addWidget(m_previousBalance, 1, 1, 1, 1);

        TextLabel4 = new QLabel(CheckingStatementInfoWizardPage);
        TextLabel4->setObjectName("TextLabel4");
        TextLabel4->setWordWrap(false);

        gridLayout->addWidget(TextLabel4, 3, 0, 1, 1);

        m_endingBalance = new AmountEdit(CheckingStatementInfoWizardPage);
        m_endingBalance->setObjectName("m_endingBalance");

        gridLayout->addWidget(m_endingBalance, 3, 1, 1, 1);


        hboxLayout->addLayout(gridLayout);

        Spacer2 = new QSpacerItem(46, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer2);


        vboxLayout->addLayout(hboxLayout);

        Spacer5 = new QSpacerItem(16, 87, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer5);

        m_oldestTransactionDate = new QLabel(CheckingStatementInfoWizardPage);
        m_oldestTransactionDate->setObjectName("m_oldestTransactionDate");

        vboxLayout->addWidget(m_oldestTransactionDate);

        m_lastStatementDate = new QLabel(CheckingStatementInfoWizardPage);
        m_lastStatementDate->setObjectName("m_lastStatementDate");
        m_lastStatementDate->setWordWrap(false);

        vboxLayout->addWidget(m_lastStatementDate);


        retranslateUi(CheckingStatementInfoWizardPage);

        QMetaObject::connectSlotsByName(CheckingStatementInfoWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *CheckingStatementInfoWizardPage)
    {
        m_enterInformationLabel->setProperty("text", QVariant(QString()));
        TextLabel3->setText(tr2i18n("Statement date:", nullptr));
        TextLabel2->setText(tr2i18n("Starting balance of this statement:", nullptr));
        TextLabel4->setText(tr2i18n("Ending balance of this statement:", nullptr));
        m_lastStatementDate->setText(tr2i18n("1", nullptr));
        (void)CheckingStatementInfoWizardPage;
    } // retranslateUi

};

namespace Ui {
    class CheckingStatementInfoWizardPage: public Ui_CheckingStatementInfoWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CHECKINGSTATEMENTINFOWIZARDPAGE_H

