/********************************************************************************
** Form generated from reading UI file 'newtransactionform.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NEWTRANSACTIONFORM_H
#define UI_NEWTRANSACTIONFORM_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_NewTransactionForm
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *payeeLabel;
    QLineEdit *payeeEdit;
    QLabel *numberLabel;
    QLineEdit *numberEdit;
    QLabel *categoryLabel;
    QLineEdit *accountEdit;
    QLabel *costCenterLabel;
    QLineEdit *costCenterEdit;
    QLabel *tagsLabel;
    QLineEdit *tagEdit;
    QLabel *dateLabel;
    QDateEdit *dateEdit;
    QLabel *memoLabel;
    QPlainTextEdit *memoEdit;
    QLineEdit *amountEdit;
    QLabel *amountLabel;
    QLabel *statusLabel;
    QLineEdit *statusEdit;
    QSpacerItem *verticalSpacer_2;
    QLabel *linkLabel;

    void setupUi(QFrame *NewTransactionForm)
    {
        if (NewTransactionForm->objectName().isEmpty())
            NewTransactionForm->setObjectName("NewTransactionForm");
        NewTransactionForm->resize(642, 198);
        NewTransactionForm->setFrameShape(QFrame::StyledPanel);
        NewTransactionForm->setFrameShadow(QFrame::Raised);
        verticalLayout = new QVBoxLayout(NewTransactionForm);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(4, 0, 4, 0);
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        gridLayout->setVerticalSpacing(3);
        payeeLabel = new QLabel(NewTransactionForm);
        payeeLabel->setObjectName("payeeLabel");

        gridLayout->addWidget(payeeLabel, 0, 0, 1, 1);

        payeeEdit = new QLineEdit(NewTransactionForm);
        payeeEdit->setObjectName("payeeEdit");
        payeeEdit->setReadOnly(true);

        gridLayout->addWidget(payeeEdit, 0, 1, 1, 1);

        numberLabel = new QLabel(NewTransactionForm);
        numberLabel->setObjectName("numberLabel");

        gridLayout->addWidget(numberLabel, 0, 2, 1, 1);

        numberEdit = new QLineEdit(NewTransactionForm);
        numberEdit->setObjectName("numberEdit");
        numberEdit->setReadOnly(true);

        gridLayout->addWidget(numberEdit, 0, 3, 1, 1);

        categoryLabel = new QLabel(NewTransactionForm);
        categoryLabel->setObjectName("categoryLabel");

        gridLayout->addWidget(categoryLabel, 1, 0, 1, 1);

        accountEdit = new QLineEdit(NewTransactionForm);
        accountEdit->setObjectName("accountEdit");
        accountEdit->setReadOnly(true);

        gridLayout->addWidget(accountEdit, 1, 1, 1, 1);

        costCenterLabel = new QLabel(NewTransactionForm);
        costCenterLabel->setObjectName("costCenterLabel");

        gridLayout->addWidget(costCenterLabel, 1, 2, 1, 1);

        costCenterEdit = new QLineEdit(NewTransactionForm);
        costCenterEdit->setObjectName("costCenterEdit");
        costCenterEdit->setReadOnly(true);

        gridLayout->addWidget(costCenterEdit, 1, 3, 1, 1);

        tagsLabel = new QLabel(NewTransactionForm);
        tagsLabel->setObjectName("tagsLabel");

        gridLayout->addWidget(tagsLabel, 2, 0, 1, 1);

        tagEdit = new QLineEdit(NewTransactionForm);
        tagEdit->setObjectName("tagEdit");
        tagEdit->setReadOnly(true);

        gridLayout->addWidget(tagEdit, 2, 1, 1, 1);

        dateLabel = new QLabel(NewTransactionForm);
        dateLabel->setObjectName("dateLabel");

        gridLayout->addWidget(dateLabel, 2, 2, 1, 1);

        dateEdit = new QDateEdit(NewTransactionForm);
        dateEdit->setObjectName("dateEdit");
        dateEdit->setReadOnly(true);

        gridLayout->addWidget(dateEdit, 2, 3, 1, 1);

        memoLabel = new QLabel(NewTransactionForm);
        memoLabel->setObjectName("memoLabel");

        gridLayout->addWidget(memoLabel, 3, 0, 1, 1);

        memoEdit = new QPlainTextEdit(NewTransactionForm);
        memoEdit->setObjectName("memoEdit");
        memoEdit->setReadOnly(true);

        gridLayout->addWidget(memoEdit, 3, 1, 3, 1);

        amountEdit = new QLineEdit(NewTransactionForm);
        amountEdit->setObjectName("amountEdit");
        amountEdit->setReadOnly(true);

        gridLayout->addWidget(amountEdit, 3, 3, 1, 1);

        amountLabel = new QLabel(NewTransactionForm);
        amountLabel->setObjectName("amountLabel");

        gridLayout->addWidget(amountLabel, 3, 2, 1, 1);

        statusLabel = new QLabel(NewTransactionForm);
        statusLabel->setObjectName("statusLabel");

        gridLayout->addWidget(statusLabel, 4, 2, 1, 1);

        statusEdit = new QLineEdit(NewTransactionForm);
        statusEdit->setObjectName("statusEdit");
        statusEdit->setReadOnly(true);

        gridLayout->addWidget(statusEdit, 4, 3, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 18, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer_2, 5, 3, 1, 1);

        linkLabel = new QLabel(NewTransactionForm);
        linkLabel->setObjectName("linkLabel");

        gridLayout->addWidget(linkLabel, 4, 0, 1, 1);

        gridLayout->setColumnStretch(1, 4);
        gridLayout->setColumnStretch(3, 1);

        verticalLayout->addLayout(gridLayout);


        retranslateUi(NewTransactionForm);

        QMetaObject::connectSlotsByName(NewTransactionForm);
    } // setupUi

    void retranslateUi(QFrame *NewTransactionForm)
    {
        payeeLabel->setText(tr2i18n("Payer/Payee", nullptr));
        numberLabel->setText(tr2i18n("Number", nullptr));
        categoryLabel->setText(tr2i18n("Category", nullptr));
        costCenterLabel->setText(tr2i18n("Costcenter", nullptr));
        tagsLabel->setText(tr2i18n("Tags", nullptr));
        dateLabel->setText(tr2i18n("Date", nullptr));
        memoLabel->setText(tr2i18n("Memo", nullptr));
        amountLabel->setText(tr2i18n("Amount", nullptr));
        statusLabel->setText(tr2i18n("Status", nullptr));
        linkLabel->setText(tr2i18n("Link", nullptr));
        (void)NewTransactionForm;
    } // retranslateUi

};

namespace Ui {
    class NewTransactionForm: public Ui_NewTransactionForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // NEWTRANSACTIONFORM_H

