/********************************************************************************
** Form generated from reading UI file 'kbmapaccount.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KBMAPACCOUNT_H
#define UI_KBMAPACCOUNT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "../widgets/kbaccountlist.h"

QT_BEGIN_NAMESPACE

class Ui_KBMapAccount
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *textLabel1;
    QGroupBox *groupBox1;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel2;
    QLineEdit *bankCodeEdit;
    QLabel *textLabel2_2;
    QLineEdit *accountIdEdit;
    QGroupBox *accountBox;
    QHBoxLayout *hboxLayout1;
    KBAccountListView *accountList;
    QHBoxLayout *hboxLayout2;
    QPushButton *helpButton;
    QSpacerItem *spacer1;
    QPushButton *assignButton;
    QPushButton *abortButton;

    void setupUi(QDialog *KBMapAccount)
    {
        if (KBMapAccount->objectName().isEmpty())
            KBMapAccount->setObjectName("KBMapAccount");
        KBMapAccount->resize(743, 534);
        vboxLayout = new QVBoxLayout(KBMapAccount);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        textLabel1 = new QLabel(KBMapAccount);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        vboxLayout->addWidget(textLabel1);

        groupBox1 = new QGroupBox(KBMapAccount);
        groupBox1->setObjectName("groupBox1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(groupBox1->sizePolicy().hasHeightForWidth());
        groupBox1->setSizePolicy(sizePolicy);
        hboxLayout = new QHBoxLayout(groupBox1);
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName("hboxLayout");
        textLabel2 = new QLabel(groupBox1);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        hboxLayout->addWidget(textLabel2);

        bankCodeEdit = new QLineEdit(groupBox1);
        bankCodeEdit->setObjectName("bankCodeEdit");
        bankCodeEdit->setReadOnly(true);

        hboxLayout->addWidget(bankCodeEdit);

        textLabel2_2 = new QLabel(groupBox1);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setWordWrap(false);

        hboxLayout->addWidget(textLabel2_2);

        accountIdEdit = new QLineEdit(groupBox1);
        accountIdEdit->setObjectName("accountIdEdit");
        accountIdEdit->setReadOnly(true);

        hboxLayout->addWidget(accountIdEdit);


        vboxLayout->addWidget(groupBox1);

        accountBox = new QGroupBox(KBMapAccount);
        accountBox->setObjectName("accountBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(accountBox->sizePolicy().hasHeightForWidth());
        accountBox->setSizePolicy(sizePolicy1);
        hboxLayout1 = new QHBoxLayout(accountBox);
        hboxLayout1->setSpacing(6);
        hboxLayout1->setContentsMargins(11, 11, 11, 11);
        hboxLayout1->setObjectName("hboxLayout1");
        accountList = new KBAccountListView(accountBox);
        accountList->setObjectName("accountList");

        hboxLayout1->addWidget(accountList);


        vboxLayout->addWidget(accountBox);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        helpButton = new QPushButton(KBMapAccount);
        helpButton->setObjectName("helpButton");

        hboxLayout2->addWidget(helpButton);

        spacer1 = new QSpacerItem(326, 29, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout2->addItem(spacer1);

        assignButton = new QPushButton(KBMapAccount);
        assignButton->setObjectName("assignButton");

        hboxLayout2->addWidget(assignButton);

        abortButton = new QPushButton(KBMapAccount);
        abortButton->setObjectName("abortButton");

        hboxLayout2->addWidget(abortButton);


        vboxLayout->addLayout(hboxLayout2);


        retranslateUi(KBMapAccount);
        QObject::connect(abortButton, &QPushButton::clicked, KBMapAccount, qOverload<>(&QDialog::reject));
        QObject::connect(assignButton, &QPushButton::clicked, KBMapAccount, qOverload<>(&QDialog::accept));

        QMetaObject::connectSlotsByName(KBMapAccount);
    } // setupUi

    void retranslateUi(QDialog *KBMapAccount)
    {
        KBMapAccount->setWindowTitle(tr2i18n("Account Mapping", nullptr));
        textLabel1->setText(tr2i18n("<qt>\n"
"Please choose the <b>online</b> account you want the \n"
"<b>application</b> account to be mapped to from the list below.\n"
"</qt>", nullptr));
        groupBox1->setTitle(tr2i18n("Application Account", nullptr));
        textLabel2->setText(tr2i18n("Bank Code", nullptr));
        textLabel2_2->setText(tr2i18n("Account Number", nullptr));
        accountBox->setTitle(tr2i18n("Online Accounts", nullptr));
        helpButton->setText(tr2i18n("Help", nullptr));
        assignButton->setText(tr2i18n("Assign", nullptr));
        abortButton->setText(tr2i18n("Abort", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KBMapAccount: public Ui_KBMapAccount {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KBMAPACCOUNT_H

