/****************************************************************************
** Meta object code from reading C++ file 'kpayeesview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/views/kpayeesview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kpayeesview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11KPayeesViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KPayeesView::qt_create_metaobjectdata<qt_meta_tag_ZN11KPayeesViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPayeesView",
        "transactionSelected",
        "",
        "accountId",
        "transactionId",
        "slotHelp",
        "slotSettingsChanged",
        "slotPayeeSelectionChanged",
        "QItemSelection",
        "selected",
        "deselected",
        "slotPayeeDataChanged",
        "slotPayeeMatchingCheck",
        "slotKeyListChanged",
        "slotRenameSinglePayee",
        "QModelIndex",
        "idx",
        "QVariant",
        "value",
        "slotUpdatePayee",
        "slotChooseDefaultAccount",
        "slotSyncAddressBook",
        "slotContactFetched",
        "ContactData",
        "identity",
        "slotSendMail",
        "slotNewPayee",
        "slotRenamePayee",
        "slotDeletePayee",
        "slotMergePayee",
        "slotModelDataChanged",
        "topLeft",
        "bottomRight"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'transactionSelected'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Slot 'slotHelp'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSettingsChanged'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPayeeSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 8, 10 },
        }}),
        // Slot 'slotPayeeDataChanged'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPayeeMatchingCheck'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotKeyListChanged'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRenameSinglePayee'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QVariant &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotUpdatePayee'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotChooseDefaultAccount'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSyncAddressBook'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotContactFetched'
        QtMocHelpers::SlotData<void(const ContactData &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'slotSendMail'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotNewPayee'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRenamePayee'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDeletePayee'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMergePayee'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotModelDataChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 31 }, { 0x80000000 | 15, 32 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KPayeesView, qt_meta_tag_ZN11KPayeesViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPayeesView::staticMetaObject = { {
    QMetaObject::SuperData::link<KMyMoneyViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KPayeesViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KPayeesViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11KPayeesViewE_t>.metaTypes,
    nullptr
} };

void KPayeesView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KPayeesView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->transactionSelected((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->slotHelp(); break;
        case 2: _t->slotSettingsChanged(); break;
        case 3: _t->slotPayeeSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 4: _t->slotPayeeDataChanged(); break;
        case 5: _t->slotPayeeMatchingCheck(); break;
        case 6: _t->slotKeyListChanged(); break;
        case 7: _t->slotRenameSinglePayee((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 8: _t->slotUpdatePayee(); break;
        case 9: _t->slotChooseDefaultAccount(); break;
        case 10: _t->slotSyncAddressBook(); break;
        case 11: _t->slotContactFetched((*reinterpret_cast<std::add_pointer_t<ContactData>>(_a[1]))); break;
        case 12: _t->slotSendMail(); break;
        case 13: _t->slotNewPayee(); break;
        case 14: _t->slotRenamePayee(); break;
        case 15: _t->slotDeletePayee(); break;
        case 16: _t->slotMergePayee(); break;
        case 17: _t->slotModelDataChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KPayeesView::*)(const QString & , const QString & )>(_a, &KPayeesView::transactionSelected, 0))
            return;
    }
}

const QMetaObject *KPayeesView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPayeesView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KPayeesViewE_t>.strings))
        return static_cast<void*>(this);
    return KMyMoneyViewBase::qt_metacast(_clname);
}

int KPayeesView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KMyMoneyViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void KPayeesView::transactionSelected(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
