/********************************************************************************
** Form generated from reading UI file 'ktagreassigndlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KTAGREASSIGNDLG_H
#define UI_KTAGREASSIGNDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KTagReassignDlg
{
public:
    QVBoxLayout *vboxLayout;
    QWidget *tagSelectionGroup;
    QVBoxLayout *verticalLayout;
    QLabel *textLabel1;
    QSpacerItem *verticalSpacer;
    QLabel *textLabel2;
    QComboBox *tagCombo;
    QSpacerItem *spacer8;
    QCheckBox *removeCheckBox;
    QSpacerItem *spacer9;
    QFrame *line1;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KTagReassignDlg)
    {
        if (KTagReassignDlg->objectName().isEmpty())
            KTagReassignDlg->setObjectName("KTagReassignDlg");
        KTagReassignDlg->resize(577, 306);
        KTagReassignDlg->setSizeGripEnabled(false);
        KTagReassignDlg->setModal(true);
        vboxLayout = new QVBoxLayout(KTagReassignDlg);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        tagSelectionGroup = new QWidget(KTagReassignDlg);
        tagSelectionGroup->setObjectName("tagSelectionGroup");
        tagSelectionGroup->setMinimumSize(QSize(0, 0));
        verticalLayout = new QVBoxLayout(tagSelectionGroup);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        textLabel1 = new QLabel(tagSelectionGroup);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setMinimumSize(QSize(300, 0));
        textLabel1->setAlignment(Qt::AlignJustify|Qt::AlignTop);
        textLabel1->setWordWrap(true);

        verticalLayout->addWidget(textLabel1);

        verticalSpacer = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        textLabel2 = new QLabel(tagSelectionGroup);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        verticalLayout->addWidget(textLabel2);

        tagCombo = new QComboBox(tagSelectionGroup);
        tagCombo->setObjectName("tagCombo");

        verticalLayout->addWidget(tagCombo);


        vboxLayout->addWidget(tagSelectionGroup);

        spacer8 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        vboxLayout->addItem(spacer8);

        removeCheckBox = new QCheckBox(KTagReassignDlg);
        removeCheckBox->setObjectName("removeCheckBox");

        vboxLayout->addWidget(removeCheckBox);

        spacer9 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer9);

        line1 = new QFrame(KTagReassignDlg);
        line1->setObjectName("line1");
        line1->setFrameShape(QFrame::HLine);
        line1->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(line1);

        buttonBox = new QDialogButtonBox(KTagReassignDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(KTagReassignDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KTagReassignDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KTagReassignDlg, qOverload<>(&QDialog::reject));
        QObject::connect(removeCheckBox, &QCheckBox::toggled, tagSelectionGroup, &QWidget::setDisabled);

        QMetaObject::connectSlotsByName(KTagReassignDlg);
    } // setupUi

    void retranslateUi(QDialog *KTagReassignDlg)
    {
        KTagReassignDlg->setWindowTitle(tr2i18n("Delete or reassign tags", nullptr));
        textLabel1->setText(tr2i18n("To complete the deletion operation, either select a tag from the available tags that will replace the selected tags before they are deleted or mark the checkbox below depending on your choice. The OK button becomes accessible once you have selected a tag or marked the checkbox.", nullptr));
        textLabel2->setText(tr2i18n("Available tags:", nullptr));
        removeCheckBox->setText(tr2i18n("Remove selected tags from all transactions, schedules and reports before deleting them.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KTagReassignDlg: public Ui_KTagReassignDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KTAGREASSIGNDLG_H

