/********************************************************************************
** Form generated from reading UI file 'chiptandialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CHIPTANDIALOG_H
#define UI_CHIPTANDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtQuickWidgets/QQuickWidget>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_chipTanDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_2;
    QToolButton *reduceButton;
    QToolButton *enlargeButton;
    QSpacerItem *horizontalSpacer_2;
    QToolButton *decelerateButton;
    QToolButton *accelerateButton;
    QSpacerItem *horizontalSpacer_3;
    QTextBrowser *infoText;
    QQuickWidget *declarativeView;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QSpacerItem *horizontalSpacer;
    QLineEdit *tanInput;
    QDialogButtonBox *buttonBox;

    void setupUi(QWidget *chipTanDialog)
    {
        if (chipTanDialog->objectName().isEmpty())
            chipTanDialog->setObjectName("chipTanDialog");
        chipTanDialog->resize(656, 405);
        verticalLayout = new QVBoxLayout(chipTanDialog);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        reduceButton = new QToolButton(chipTanDialog);
        reduceButton->setObjectName("reduceButton");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("zoom-out");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8(""), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        reduceButton->setIcon(icon);

        horizontalLayout_2->addWidget(reduceButton);

        enlargeButton = new QToolButton(chipTanDialog);
        enlargeButton->setObjectName("enlargeButton");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("zoom-in");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8(""), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        enlargeButton->setIcon(icon1);

        horizontalLayout_2->addWidget(enlargeButton);

        horizontalSpacer_2 = new QSpacerItem(5, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        decelerateButton = new QToolButton(chipTanDialog);
        decelerateButton->setObjectName("decelerateButton");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("media-seek-backward");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8(""), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        decelerateButton->setIcon(icon2);

        horizontalLayout_2->addWidget(decelerateButton);

        accelerateButton = new QToolButton(chipTanDialog);
        accelerateButton->setObjectName("accelerateButton");
        QIcon icon3;
        iconThemeName = QString::fromUtf8("media-seek-forward");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8(""), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        accelerateButton->setIcon(icon3);

        horizontalLayout_2->addWidget(accelerateButton);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_3);


        gridLayout->addLayout(horizontalLayout_2, 0, 0, 1, 1);

        infoText = new QTextBrowser(chipTanDialog);
        infoText->setObjectName("infoText");
        infoText->setStyleSheet(QString::fromUtf8("background:transparent"));
        infoText->setFrameShape(QFrame::NoFrame);
        infoText->setFrameShadow(QFrame::Plain);
        infoText->setReadOnly(true);
        infoText->setAcceptRichText(false);

        gridLayout->addWidget(infoText, 0, 1, 2, 1);

        declarativeView = new QQuickWidget(chipTanDialog);
        declarativeView->setObjectName("declarativeView");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(declarativeView->sizePolicy().hasHeightForWidth());
        declarativeView->setSizePolicy(sizePolicy);
        declarativeView->setMinimumSize(QSize(360, 300));
        declarativeView->setStyleSheet(QString::fromUtf8("background:transparent"));

        gridLayout->addWidget(declarativeView, 1, 0, 1, 1);

        gridLayout->setColumnStretch(0, 10);

        verticalLayout->addLayout(gridLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(chipTanDialog);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        tanInput = new QLineEdit(chipTanDialog);
        tanInput->setObjectName("tanInput");

        horizontalLayout->addWidget(tanInput);


        verticalLayout->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(chipTanDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label->setBuddy(tanInput);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(tanInput, buttonBox);
        QWidget::setTabOrder(buttonBox, reduceButton);
        QWidget::setTabOrder(reduceButton, enlargeButton);
        QWidget::setTabOrder(enlargeButton, decelerateButton);
        QWidget::setTabOrder(decelerateButton, accelerateButton);
        QWidget::setTabOrder(accelerateButton, infoText);

        retranslateUi(chipTanDialog);

        QMetaObject::connectSlotsByName(chipTanDialog);
    } // setupUi

    void retranslateUi(QWidget *chipTanDialog)
    {
        chipTanDialog->setWindowTitle(tr2i18n("Order confirmation", nullptr));
#if QT_CONFIG(tooltip)
        reduceButton->setToolTip(tr2i18n("Reduce optical tan area in size", nullptr));
#endif // QT_CONFIG(tooltip)
        reduceButton->setText(tr2i18n("Reduce", nullptr));
#if QT_CONFIG(tooltip)
        enlargeButton->setToolTip(tr2i18n("Enlarge optical tan field", nullptr));
#endif // QT_CONFIG(tooltip)
        enlargeButton->setText(tr2i18n("Enlarge", nullptr));
#if QT_CONFIG(tooltip)
        decelerateButton->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        decelerateButton->setText(tr2i18n("Decelerate", nullptr));
        accelerateButton->setText(tr2i18n("Accelerate", nullptr));
        label->setText(tr2i18n("To confirm this order enter the tan displayed by your generator", nullptr));
    } // retranslateUi

};

namespace Ui {
    class chipTanDialog: public Ui_chipTanDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CHIPTANDIALOG_H

