/********************************************************************************
** Form generated from reading UI file 'pluginsettingsdecl.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PLUGINSETTINGSDECL_H
#define UI_PLUGINSETTINGSDECL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_PluginSettingsDecl
{
public:
    QVBoxLayout *verticalLayout_3;
    QGroupBox *autodetect;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *kcfg_AutoFieldDelimiter;
    QCheckBox *kcfg_AutoDecimalSymbol;
    QCheckBox *kcfg_AutoDateFormat;
    QCheckBox *kcfg_AutoAccountInvest;
    QCheckBox *kcfg_AutoAccountBank;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QLineEdit *kcfg_QifExportProfile;
    QComboBox *qifProfileCombo;

    void setupUi(QWidget *PluginSettingsDecl)
    {
        if (PluginSettingsDecl->objectName().isEmpty())
            PluginSettingsDecl->setObjectName("PluginSettingsDecl");
        PluginSettingsDecl->resize(254, 306);
        verticalLayout_3 = new QVBoxLayout(PluginSettingsDecl);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        autodetect = new QGroupBox(PluginSettingsDecl);
        autodetect->setObjectName("autodetect");
        verticalLayout_2 = new QVBoxLayout(autodetect);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        kcfg_AutoFieldDelimiter = new QCheckBox(autodetect);
        kcfg_AutoFieldDelimiter->setObjectName("kcfg_AutoFieldDelimiter");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(kcfg_AutoFieldDelimiter->sizePolicy().hasHeightForWidth());
        kcfg_AutoFieldDelimiter->setSizePolicy(sizePolicy);

        verticalLayout_2->addWidget(kcfg_AutoFieldDelimiter);

        kcfg_AutoDecimalSymbol = new QCheckBox(autodetect);
        kcfg_AutoDecimalSymbol->setObjectName("kcfg_AutoDecimalSymbol");
        sizePolicy.setHeightForWidth(kcfg_AutoDecimalSymbol->sizePolicy().hasHeightForWidth());
        kcfg_AutoDecimalSymbol->setSizePolicy(sizePolicy);

        verticalLayout_2->addWidget(kcfg_AutoDecimalSymbol);

        kcfg_AutoDateFormat = new QCheckBox(autodetect);
        kcfg_AutoDateFormat->setObjectName("kcfg_AutoDateFormat");
        sizePolicy.setHeightForWidth(kcfg_AutoDateFormat->sizePolicy().hasHeightForWidth());
        kcfg_AutoDateFormat->setSizePolicy(sizePolicy);

        verticalLayout_2->addWidget(kcfg_AutoDateFormat);

        kcfg_AutoAccountInvest = new QCheckBox(autodetect);
        kcfg_AutoAccountInvest->setObjectName("kcfg_AutoAccountInvest");
        sizePolicy.setHeightForWidth(kcfg_AutoAccountInvest->sizePolicy().hasHeightForWidth());
        kcfg_AutoAccountInvest->setSizePolicy(sizePolicy);

        verticalLayout_2->addWidget(kcfg_AutoAccountInvest);

        kcfg_AutoAccountBank = new QCheckBox(autodetect);
        kcfg_AutoAccountBank->setObjectName("kcfg_AutoAccountBank");
        sizePolicy.setHeightForWidth(kcfg_AutoAccountBank->sizePolicy().hasHeightForWidth());
        kcfg_AutoAccountBank->setSizePolicy(sizePolicy);

        verticalLayout_2->addWidget(kcfg_AutoAccountBank);


        verticalLayout_3->addWidget(autodetect);

        groupBox = new QGroupBox(PluginSettingsDecl);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        kcfg_QifExportProfile = new QLineEdit(groupBox);
        kcfg_QifExportProfile->setObjectName("kcfg_QifExportProfile");

        verticalLayout->addWidget(kcfg_QifExportProfile);

        qifProfileCombo = new QComboBox(groupBox);
        qifProfileCombo->setObjectName("qifProfileCombo");

        verticalLayout->addWidget(qifProfileCombo);


        verticalLayout_3->addWidget(groupBox);

        verticalLayout_3->setStretch(0, 1);

        retranslateUi(PluginSettingsDecl);

        QMetaObject::connectSlotsByName(PluginSettingsDecl);
    } // setupUi

    void retranslateUi(QWidget *PluginSettingsDecl)
    {
        PluginSettingsDecl->setWindowTitle(tr2i18n("CSV Importer settings", nullptr));
        autodetect->setTitle(tr2i18n("Autodetecting", nullptr));
        kcfg_AutoFieldDelimiter->setText(tr2i18n("Field delimiter", nullptr));
        kcfg_AutoDecimalSymbol->setText(tr2i18n("Decimal symbol", nullptr));
        kcfg_AutoDateFormat->setText(tr2i18n("Date format", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_AutoAccountInvest->setToolTip(tr2i18n("Statement header will be scanned for names and numbers of accounts you have already setup.", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_AutoAccountInvest->setText(tr2i18n("Account in investment statement", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_AutoAccountBank->setToolTip(tr2i18n("Statement header will be scanned for names and numbers of accounts you have already setup.", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_AutoAccountBank->setText(tr2i18n("Account in bank statement", nullptr));
        groupBox->setTitle(tr2i18n("QIF export profile", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PluginSettingsDecl: public Ui_PluginSettingsDecl {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PLUGINSETTINGSDECL_H

