/********************************************************************************
** Form generated from reading UI file 'lendborrowwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LENDBORROWWIZARDPAGE_H
#define UI_LENDBORROWWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_LendBorrowWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel3_2;
    QSpacerItem *Spacer15;
    QHBoxLayout *hboxLayout;
    QFrame *frame_2;
    QVBoxLayout *vboxLayout1;
    QRadioButton *m_borrowButton;
    QRadioButton *m_lendButton;
    QSpacerItem *Spacer14;
    QSpacerItem *Spacer16;
    QButtonGroup *ButtonGroup1;

    void setupUi(QWizardPage *LendBorrowWizardPage)
    {
        if (LendBorrowWizardPage->objectName().isEmpty())
            LendBorrowWizardPage->setObjectName("LendBorrowWizardPage");
        vboxLayout = new QVBoxLayout(LendBorrowWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel3_2 = new QLabel(LendBorrowWizardPage);
        TextLabel3_2->setObjectName("TextLabel3_2");
        TextLabel3_2->setAlignment(Qt::AlignTop);
        TextLabel3_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel3_2);

        Spacer15 = new QSpacerItem(0, 61, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer15);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        frame_2 = new QFrame(LendBorrowWizardPage);
        frame_2->setObjectName("frame_2");
        frame_2->setFrameShape(QFrame::NoFrame);
        vboxLayout1 = new QVBoxLayout(frame_2);
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        m_borrowButton = new QRadioButton(frame_2);
        ButtonGroup1 = new QButtonGroup(LendBorrowWizardPage);
        ButtonGroup1->setObjectName("ButtonGroup1");
        ButtonGroup1->addButton(m_borrowButton);
        m_borrowButton->setObjectName("m_borrowButton");

        vboxLayout1->addWidget(m_borrowButton);

        m_lendButton = new QRadioButton(frame_2);
        ButtonGroup1->addButton(m_lendButton);
        m_lendButton->setObjectName("m_lendButton");

        vboxLayout1->addWidget(m_lendButton);


        hboxLayout->addWidget(frame_2);

        Spacer14 = new QSpacerItem(299, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer14);


        vboxLayout->addLayout(hboxLayout);

        Spacer16 = new QSpacerItem(0, 60, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer16);


        retranslateUi(LendBorrowWizardPage);

        QMetaObject::connectSlotsByName(LendBorrowWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *LendBorrowWizardPage)
    {
        TextLabel3_2->setText(tr2i18n("\n"
"Do you borrow or lend money?", nullptr));
        m_borrowButton->setText(tr2i18n("I am borrowing money", nullptr));
        m_lendButton->setText(tr2i18n("I am lending money", nullptr));
        (void)LendBorrowWizardPage;
    } // retranslateUi

};

namespace Ui {
    class LendBorrowWizardPage: public Ui_LendBorrowWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // LENDBORROWWIZARDPAGE_H

