/********************************************************************************
** Form generated from reading UI file 'kbpickstartdate.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KBPICKSTARTDATE_H
#define UI_KBPICKSTARTDATE_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_KBPickStartDate
{
public:
    QVBoxLayout *verticalLayout;
    QTextBrowser *label;
    QGroupBox *buttonGroup;
    QGridLayout *gridLayout;
    QRadioButton *noDateButton;
    QRadioButton *lastUpdateButton;
    QLabel *lastUpdateLabel;
    QRadioButton *firstDateButton;
    QLabel *firstDateLabel;
    QRadioButton *pickDateButton;
    KMyMoneyDateEdit *pickDateEdit;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KBPickStartDate)
    {
        if (KBPickStartDate->objectName().isEmpty())
            KBPickStartDate->setObjectName("KBPickStartDate");
        KBPickStartDate->resize(511, 404);
        KBPickStartDate->setSizeGripEnabled(true);
        verticalLayout = new QVBoxLayout(KBPickStartDate);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        label = new QTextBrowser(KBPickStartDate);
        label->setObjectName("label");
        label->setFrameShape(QFrame::Panel);
        label->setFrameShadow(QFrame::Raised);
        label->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        label->setAutoFormatting(QTextEdit::AutoAll);

        verticalLayout->addWidget(label);

        buttonGroup = new QGroupBox(KBPickStartDate);
        buttonGroup->setObjectName("buttonGroup");
        gridLayout = new QGridLayout(buttonGroup);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        noDateButton = new QRadioButton(buttonGroup);
        noDateButton->setObjectName("noDateButton");

        gridLayout->addWidget(noDateButton, 0, 0, 1, 2);

        lastUpdateButton = new QRadioButton(buttonGroup);
        lastUpdateButton->setObjectName("lastUpdateButton");

        gridLayout->addWidget(lastUpdateButton, 1, 0, 1, 1);

        lastUpdateLabel = new QLabel(buttonGroup);
        lastUpdateLabel->setObjectName("lastUpdateLabel");
        lastUpdateLabel->setFrameShape(QFrame::Panel);
        lastUpdateLabel->setFrameShadow(QFrame::Sunken);
        lastUpdateLabel->setWordWrap(false);

        gridLayout->addWidget(lastUpdateLabel, 1, 1, 1, 1);

        firstDateButton = new QRadioButton(buttonGroup);
        firstDateButton->setObjectName("firstDateButton");

        gridLayout->addWidget(firstDateButton, 2, 0, 1, 1);

        firstDateLabel = new QLabel(buttonGroup);
        firstDateLabel->setObjectName("firstDateLabel");
        firstDateLabel->setFrameShape(QFrame::Panel);
        firstDateLabel->setFrameShadow(QFrame::Sunken);
        firstDateLabel->setWordWrap(false);

        gridLayout->addWidget(firstDateLabel, 2, 1, 1, 1);

        pickDateButton = new QRadioButton(buttonGroup);
        pickDateButton->setObjectName("pickDateButton");
        pickDateButton->setChecked(true);

        gridLayout->addWidget(pickDateButton, 3, 0, 1, 1);

        pickDateEdit = new KMyMoneyDateEdit(buttonGroup);
        pickDateEdit->setObjectName("pickDateEdit");

        gridLayout->addWidget(pickDateEdit, 3, 1, 1, 1);


        verticalLayout->addWidget(buttonGroup);

        buttonBox = new QDialogButtonBox(KBPickStartDate);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(noDateButton, lastUpdateButton);
        QWidget::setTabOrder(lastUpdateButton, firstDateButton);
        QWidget::setTabOrder(firstDateButton, pickDateButton);

        retranslateUi(KBPickStartDate);
        QObject::connect(pickDateButton, SIGNAL(toggled(bool)), pickDateEdit, SLOT(setEnabled(bool)));
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KBPickStartDate, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KBPickStartDate, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KBPickStartDate);
    } // setupUi

    void retranslateUi(QDialog *KBPickStartDate)
    {
        KBPickStartDate->setWindowTitle(tr2i18n("Pick Start Date", nullptr));
        buttonGroup->setTitle(tr2i18n("Start Date", nullptr));
        noDateButton->setText(tr2i18n("&No date (let the bank determine the start date)", nullptr));
        lastUpdateButton->setText(tr2i18n("&Last Update", nullptr));
        lastUpdateLabel->setText(QString());
        firstDateButton->setText(tr2i18n("F&irst possible date", nullptr));
        firstDateLabel->setText(QString());
        pickDateButton->setText(tr2i18n("Pick &date", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KBPickStartDate: public Ui_KBPickStartDate {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KBPICKSTARTDATE_H

