/********************************************************************************
** Form generated from reading UI file 'splitdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SPLITDIALOG_H
#define UI_SPLITDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "splitview.h"

QT_BEGIN_NAMESPACE

class Ui_SplitDialog
{
public:
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *splitsLayout;
    SplitView *splitView;
    QTableWidget *summaryView;
    QFrame *buttonContainer;
    QVBoxLayout *verticalLayout;
    QPushButton *newSplitButton;
    QSpacerItem *verticalSpacer_2;
    QPushButton *mergeButton;
    QPushButton *adjustUnassigned;
    QPushButton *deleteZeroButton;
    QPushButton *deleteButton;
    QPushButton *deleteAllButton;
    QSpacerItem *verticalSpacer;
    QPushButton *okButton;
    QPushButton *cancelButton;

    void setupUi(QDialog *SplitDialog)
    {
        if (SplitDialog->objectName().isEmpty())
            SplitDialog->setObjectName("SplitDialog");
        SplitDialog->resize(771, 487);
        horizontalLayout = new QHBoxLayout(SplitDialog);
        horizontalLayout->setObjectName("horizontalLayout");
        splitsLayout = new QVBoxLayout();
        splitsLayout->setObjectName("splitsLayout");
        splitView = new SplitView(SplitDialog);
        splitView->setObjectName("splitView");

        splitsLayout->addWidget(splitView);

        summaryView = new QTableWidget(SplitDialog);
        if (summaryView->columnCount() < 2)
            summaryView->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        __qtablewidgetitem->setText(QString::fromUtf8("col"));
        __qtablewidgetitem->setTextAlignment(Qt::AlignLeading|Qt::AlignVCenter);
        summaryView->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        __qtablewidgetitem1->setText(QString::fromUtf8("col"));
        summaryView->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        if (summaryView->rowCount() < 3)
            summaryView->setRowCount(3);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        __qtablewidgetitem2->setText(QString::fromUtf8("row"));
        summaryView->setVerticalHeaderItem(0, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        __qtablewidgetitem3->setText(QString::fromUtf8("row"));
        summaryView->setVerticalHeaderItem(1, __qtablewidgetitem3);
        QTableWidgetItem *__qtablewidgetitem4 = new QTableWidgetItem();
        __qtablewidgetitem4->setText(QString::fromUtf8("row"));
        summaryView->setVerticalHeaderItem(2, __qtablewidgetitem4);
        QTableWidgetItem *__qtablewidgetitem5 = new QTableWidgetItem();
        __qtablewidgetitem5->setTextAlignment(Qt::AlignTrailing|Qt::AlignVCenter);
        __qtablewidgetitem5->setFlags(Qt::NoItemFlags);
        summaryView->setItem(0, 0, __qtablewidgetitem5);
        QTableWidgetItem *__qtablewidgetitem6 = new QTableWidgetItem();
        __qtablewidgetitem6->setTextAlignment(Qt::AlignTrailing|Qt::AlignVCenter);
        summaryView->setItem(0, 1, __qtablewidgetitem6);
        QTableWidgetItem *__qtablewidgetitem7 = new QTableWidgetItem();
        __qtablewidgetitem7->setText(QString::fromUtf8("Unassigned"));
        __qtablewidgetitem7->setTextAlignment(Qt::AlignTrailing|Qt::AlignVCenter);
        __qtablewidgetitem7->setFlags(Qt::ItemIsEnabled);
        summaryView->setItem(1, 0, __qtablewidgetitem7);
        QTableWidgetItem *__qtablewidgetitem8 = new QTableWidgetItem();
        __qtablewidgetitem8->setTextAlignment(Qt::AlignTrailing|Qt::AlignVCenter);
        summaryView->setItem(1, 1, __qtablewidgetitem8);
        QTableWidgetItem *__qtablewidgetitem9 = new QTableWidgetItem();
        __qtablewidgetitem9->setTextAlignment(Qt::AlignTrailing|Qt::AlignVCenter);
        __qtablewidgetitem9->setFlags(Qt::ItemIsEnabled);
        summaryView->setItem(2, 0, __qtablewidgetitem9);
        QTableWidgetItem *__qtablewidgetitem10 = new QTableWidgetItem();
        __qtablewidgetitem10->setTextAlignment(Qt::AlignTrailing|Qt::AlignVCenter);
        summaryView->setItem(2, 1, __qtablewidgetitem10);
        summaryView->setObjectName("summaryView");
        summaryView->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(summaryView->sizePolicy().hasHeightForWidth());
        summaryView->setSizePolicy(sizePolicy);
        summaryView->setMinimumSize(QSize(0, 60));
        QPalette palette;
        QBrush brush(QColor(0, 0, 0, 255));
        brush.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::WindowText, brush);
        QBrush brush1(QColor(207, 204, 201, 255));
        brush1.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Button, brush1);
        QBrush brush2(QColor(255, 255, 255, 255));
        brush2.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Light, brush2);
        QBrush brush3(QColor(231, 229, 228, 255));
        brush3.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Midlight, brush3);
        QBrush brush4(QColor(103, 102, 100, 255));
        brush4.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Dark, brush4);
        QBrush brush5(QColor(138, 136, 134, 255));
        brush5.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Mid, brush5);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Text, brush);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::BrightText, brush2);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::ButtonText, brush);
        QBrush brush6(QColor(255, 255, 255, 0));
        brush6.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Base, brush6);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Window, brush1);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Shadow, brush);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::AlternateBase, brush3);
        QBrush brush7(QColor(255, 255, 220, 255));
        brush7.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::ToolTipBase, brush7);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::ToolTipText, brush);
        QBrush brush8(QColor(0, 0, 0, 128));
        brush8.setStyle(Qt::BrushStyle::NoBrush);
#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::PlaceholderText, brush8);
#endif
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::WindowText, brush);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Button, brush1);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Light, brush2);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Midlight, brush3);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Dark, brush4);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Mid, brush5);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Text, brush);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::BrightText, brush2);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::ButtonText, brush);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Base, brush6);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Window, brush1);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Shadow, brush);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::AlternateBase, brush3);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::ToolTipBase, brush7);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::ToolTipText, brush);
        QBrush brush9(QColor(0, 0, 0, 128));
        brush9.setStyle(Qt::BrushStyle::NoBrush);
#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::PlaceholderText, brush9);
#endif
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::WindowText, brush4);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Button, brush1);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Light, brush2);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Midlight, brush3);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Dark, brush4);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Mid, brush5);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Text, brush);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::BrightText, brush2);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::ButtonText, brush4);
        QBrush brush10(QColor(207, 204, 201, 0));
        brush10.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Base, brush10);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Window, brush1);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Shadow, brush);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::AlternateBase, brush1);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::ToolTipBase, brush7);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::ToolTipText, brush);
        QBrush brush11(QColor(0, 0, 0, 128));
        brush11.setStyle(Qt::BrushStyle::NoBrush);
#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::PlaceholderText, brush11);
#endif
        summaryView->setPalette(palette);
        QFont font;
        font.setBold(true);
        summaryView->setFont(font);
        summaryView->setFrameShape(QFrame::NoFrame);
        summaryView->setFrameShadow(QFrame::Plain);
        summaryView->setLineWidth(0);
        summaryView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        summaryView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        summaryView->setSelectionMode(QAbstractItemView::NoSelection);
        summaryView->setShowGrid(false);
        summaryView->setWordWrap(false);
        summaryView->horizontalHeader()->setVisible(false);
        summaryView->verticalHeader()->setVisible(false);
        summaryView->verticalHeader()->setMinimumSectionSize(20);
        summaryView->verticalHeader()->setDefaultSectionSize(20);

        splitsLayout->addWidget(summaryView);

        splitsLayout->setStretch(0, 10);
        splitsLayout->setStretch(1, 1);

        horizontalLayout->addLayout(splitsLayout);

        buttonContainer = new QFrame(SplitDialog);
        buttonContainer->setObjectName("buttonContainer");
        buttonContainer->setFrameShape(QFrame::NoFrame);
        buttonContainer->setFrameShadow(QFrame::Plain);
        verticalLayout = new QVBoxLayout(buttonContainer);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        newSplitButton = new QPushButton(buttonContainer);
        newSplitButton->setObjectName("newSplitButton");

        verticalLayout->addWidget(newSplitButton);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);

        mergeButton = new QPushButton(buttonContainer);
        mergeButton->setObjectName("mergeButton");

        verticalLayout->addWidget(mergeButton);

        adjustUnassigned = new QPushButton(buttonContainer);
        adjustUnassigned->setObjectName("adjustUnassigned");

        verticalLayout->addWidget(adjustUnassigned);

        deleteZeroButton = new QPushButton(buttonContainer);
        deleteZeroButton->setObjectName("deleteZeroButton");

        verticalLayout->addWidget(deleteZeroButton);

        deleteButton = new QPushButton(buttonContainer);
        deleteButton->setObjectName("deleteButton");

        verticalLayout->addWidget(deleteButton);

        deleteAllButton = new QPushButton(buttonContainer);
        deleteAllButton->setObjectName("deleteAllButton");

        verticalLayout->addWidget(deleteAllButton);

        verticalSpacer = new QSpacerItem(20, 280, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        okButton = new QPushButton(buttonContainer);
        okButton->setObjectName("okButton");
        okButton->setAutoDefault(true);

        verticalLayout->addWidget(okButton);

        cancelButton = new QPushButton(buttonContainer);
        cancelButton->setObjectName("cancelButton");

        verticalLayout->addWidget(cancelButton);

        verticalLayout->setStretch(0, 1);
        verticalLayout->setStretch(2, 1);
        verticalLayout->setStretch(3, 1);
        verticalLayout->setStretch(4, 1);
        verticalLayout->setStretch(5, 1);
        verticalLayout->setStretch(6, 3);
        verticalLayout->setStretch(7, 1);
        verticalLayout->setStretch(8, 1);

        horizontalLayout->addWidget(buttonContainer);


        retranslateUi(SplitDialog);
        QObject::connect(okButton, &QPushButton::clicked, SplitDialog, qOverload<>(&QDialog::accept));
        QObject::connect(cancelButton, &QPushButton::clicked, SplitDialog, qOverload<>(&QDialog::reject));

        okButton->setDefault(true);


        QMetaObject::connectSlotsByName(SplitDialog);
    } // setupUi

    void retranslateUi(QDialog *SplitDialog)
    {
        SplitDialog->setWindowTitle(tr2i18n("Split editor", nullptr));

        const bool __sortingEnabled = summaryView->isSortingEnabled();
        summaryView->setSortingEnabled(false);
        QTableWidgetItem *___qtablewidgetitem = summaryView->item(0, 0);
        ___qtablewidgetitem->setText(tr2i18n("Sum of splits", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = summaryView->item(2, 0);
        ___qtablewidgetitem1->setText(tr2i18n("Transaction amount", nullptr));
        summaryView->setSortingEnabled(__sortingEnabled);

#if QT_CONFIG(tooltip)
        newSplitButton->setToolTip(tr2i18n("Create a new split", nullptr));
#endif // QT_CONFIG(tooltip)
        newSplitButton->setText(tr2i18n("New", nullptr));
#if QT_CONFIG(tooltip)
        mergeButton->setToolTip(tr2i18n("Merges splits with the same category to one split", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mergeButton->setWhatsThis(tr2i18n("In case you have multiple split entries to the same category and you like to keep them as a single split, press this button. The amount for identical categories will be added and stored in a single split for that category.", nullptr));
#endif // QT_CONFIG(whatsthis)
        mergeButton->setText(tr2i18n("&Merge", nullptr));
#if QT_CONFIG(tooltip)
        adjustUnassigned->setToolTip(tr2i18n("Apply 'Unassigned' or 'Overassigned' amount to the current split.\n"
"'Unassigned' amount will be added or 'Overassigned' amount\n"
"will be subtracted from the amount of the selected split.", nullptr));
#endif // QT_CONFIG(tooltip)
        adjustUnassigned->setText(tr2i18n("Apply Difference", nullptr));
#if QT_CONFIG(tooltip)
        deleteZeroButton->setToolTip(tr2i18n("Removes all splits that have a value of zero", nullptr));
#endif // QT_CONFIG(tooltip)
        deleteZeroButton->setText(tr2i18n("Delete &Zero", nullptr));
        deleteButton->setText(tr2i18n("Delete", nullptr));
#if QT_CONFIG(tooltip)
        deleteAllButton->setToolTip(tr2i18n("Clear all splits", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        deleteAllButton->setWhatsThis(tr2i18n("Use this to delete all splits of this transaction.", nullptr));
#endif // QT_CONFIG(whatsthis)
        deleteAllButton->setText(tr2i18n("Delete &All", nullptr));
        okButton->setText(tr2i18n("&OK", nullptr));
        cancelButton->setText(tr2i18n("&Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SplitDialog: public Ui_SplitDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SPLITDIALOG_H

