/********************************************************************************
** Form generated from reading UI file 'kaccountsview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KACCOUNTSVIEW_H
#define UI_KACCOUNTSVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "klineedit.h"
#include "kmymoneyaccounttreeview.h"

QT_BEGIN_NAMESPACE

class Ui_KAccountsView
{
public:
    QVBoxLayout *vboxLayout;
    QVBoxLayout *verticalLayout;
    KMyMoneyAccountTreeView *m_accountTree;
    QHBoxLayout *searchLayout;
    KLineEdit *m_searchWidget;
    QComboBox *m_filterBox;
    QPushButton *m_collapseButton;
    QPushButton *m_expandButton;
    QLabel *m_hiddenCategories;
    QHBoxLayout *hboxLayout;
    QSpacerItem *Spacer1;
    QLabel *m_totalProfitsLabel;

    void setupUi(QWidget *KAccountsView)
    {
        if (KAccountsView->objectName().isEmpty())
            KAccountsView->setObjectName("KAccountsView");
        KAccountsView->resize(405, 185);
        vboxLayout = new QVBoxLayout(KAccountsView);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        m_accountTree = new KMyMoneyAccountTreeView(KAccountsView);
        m_accountTree->setObjectName("m_accountTree");

        verticalLayout->addWidget(m_accountTree);

        searchLayout = new QHBoxLayout();
        searchLayout->setSpacing(6);
        searchLayout->setObjectName("searchLayout");
        m_searchWidget = new KLineEdit(KAccountsView);
        m_searchWidget->setObjectName("m_searchWidget");
        m_searchWidget->setProperty("showClearButton", QVariant(true));

        searchLayout->addWidget(m_searchWidget);

        m_filterBox = new QComboBox(KAccountsView);
        m_filterBox->setObjectName("m_filterBox");

        searchLayout->addWidget(m_filterBox);

        m_collapseButton = new QPushButton(KAccountsView);
        m_collapseButton->setObjectName("m_collapseButton");

        searchLayout->addWidget(m_collapseButton);

        m_expandButton = new QPushButton(KAccountsView);
        m_expandButton->setObjectName("m_expandButton");

        searchLayout->addWidget(m_expandButton);


        verticalLayout->addLayout(searchLayout);


        vboxLayout->addLayout(verticalLayout);

        m_hiddenCategories = new QLabel(KAccountsView);
        m_hiddenCategories->setObjectName("m_hiddenCategories");
        m_hiddenCategories->setWordWrap(false);

        vboxLayout->addWidget(m_hiddenCategories);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        Spacer1 = new QSpacerItem(367, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer1);

        m_totalProfitsLabel = new QLabel(KAccountsView);
        m_totalProfitsLabel->setObjectName("m_totalProfitsLabel");
        m_totalProfitsLabel->setMinimumSize(QSize(150, 0));
        m_totalProfitsLabel->setTextFormat(Qt::RichText);
        m_totalProfitsLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        m_totalProfitsLabel->setWordWrap(false);

        hboxLayout->addWidget(m_totalProfitsLabel);


        vboxLayout->addLayout(hboxLayout);


        retranslateUi(KAccountsView);
        QObject::connect(m_collapseButton, &QPushButton::clicked, m_accountTree, qOverload<>(&KMyMoneyAccountTreeView::collapseAll));
        QObject::connect(m_expandButton, &QPushButton::clicked, m_accountTree, qOverload<>(&KMyMoneyAccountTreeView::expandAll));

        QMetaObject::connectSlotsByName(KAccountsView);
    } // setupUi

    void retranslateUi(QWidget *KAccountsView)
    {
#if QT_CONFIG(tooltip)
        m_collapseButton->setToolTip(tr2i18n("Collapse all accounts in the list", nullptr));
#endif // QT_CONFIG(tooltip)
        m_collapseButton->setText(QString());
#if QT_CONFIG(tooltip)
        m_expandButton->setToolTip(tr2i18n("Expand all accounts in the list", nullptr));
#endif // QT_CONFIG(tooltip)
        m_expandButton->setText(QString());
        m_hiddenCategories->setText(tr2i18n("Note: Unused categories are not shown as selected by settings.", nullptr));
        m_totalProfitsLabel->setText(tr2i18n("Total Profits:", nullptr));
        (void)KAccountsView;
    } // retranslateUi

};

namespace Ui {
    class KAccountsView: public Ui_KAccountsView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KACCOUNTSVIEW_H

