/********************************************************************************
** Form generated from reading UI file 'konlineupdatewizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KONLINEUPDATEWIZARDPAGE_H
#define UI_KONLINEUPDATEWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <amountedit.h>
#include <klocalizedstring.h>
#include <konlineupdatewizardpage.h>
#include "kcombobox.h"

QT_BEGIN_NAMESPACE

class Ui_KOnlineUpdateWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *textLabel6;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout1;
    QLabel *textLabel2_4;
    QCheckBox *m_useFinanceQuote;
    QHBoxLayout *hboxLayout2;
    QLabel *textLabel2_3;
    KComboBox *m_onlineSourceCombo;
    QHBoxLayout *hboxLayout3;
    QLabel *textLabel1_6;
    AmountEdit *m_onlineFactor;
    QSpacerItem *spacer29;
    QSpacerItem *spacer30;

    void setupUi(KOnlineUpdateWizardPage *KOnlineUpdateWizardPage)
    {
        if (KOnlineUpdateWizardPage->objectName().isEmpty())
            KOnlineUpdateWizardPage->setObjectName("KOnlineUpdateWizardPage");
        KOnlineUpdateWizardPage->resize(661, 347);
        vboxLayout = new QVBoxLayout(KOnlineUpdateWizardPage);
        vboxLayout->setObjectName("vboxLayout");
        textLabel6 = new QLabel(KOnlineUpdateWizardPage);
        textLabel6->setObjectName("textLabel6");
        textLabel6->setAlignment(Qt::AlignTop);
        textLabel6->setWordWrap(true);

        vboxLayout->addWidget(textLabel6);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(0);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setObjectName("vboxLayout1");
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        textLabel2_4 = new QLabel(KOnlineUpdateWizardPage);
        textLabel2_4->setObjectName("textLabel2_4");
        textLabel2_4->setWordWrap(false);

        hboxLayout1->addWidget(textLabel2_4);

        m_useFinanceQuote = new QCheckBox(KOnlineUpdateWizardPage);
        m_useFinanceQuote->setObjectName("m_useFinanceQuote");

        hboxLayout1->addWidget(m_useFinanceQuote);


        vboxLayout1->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName("hboxLayout2");
        textLabel2_3 = new QLabel(KOnlineUpdateWizardPage);
        textLabel2_3->setObjectName("textLabel2_3");
        textLabel2_3->setWordWrap(false);

        hboxLayout2->addWidget(textLabel2_3);

        m_onlineSourceCombo = new KComboBox(KOnlineUpdateWizardPage);
        m_onlineSourceCombo->addItem(QString());
        m_onlineSourceCombo->setObjectName("m_onlineSourceCombo");
        m_onlineSourceCombo->setMinimumSize(QSize(180, 0));

        hboxLayout2->addWidget(m_onlineSourceCombo);


        vboxLayout1->addLayout(hboxLayout2);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setObjectName("hboxLayout3");
        textLabel1_6 = new QLabel(KOnlineUpdateWizardPage);
        textLabel1_6->setObjectName("textLabel1_6");
        textLabel1_6->setWordWrap(false);

        hboxLayout3->addWidget(textLabel1_6);

        m_onlineFactor = new AmountEdit(KOnlineUpdateWizardPage);
        m_onlineFactor->setObjectName("m_onlineFactor");

        hboxLayout3->addWidget(m_onlineFactor);


        vboxLayout1->addLayout(hboxLayout3);


        hboxLayout->addLayout(vboxLayout1);

        spacer29 = new QSpacerItem(371, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer29);


        vboxLayout->addLayout(hboxLayout);

        spacer30 = new QSpacerItem(20, 181, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer30);

        QWidget::setTabOrder(m_useFinanceQuote, m_onlineSourceCombo);
        QWidget::setTabOrder(m_onlineSourceCombo, m_onlineFactor);

        retranslateUi(KOnlineUpdateWizardPage);

        QMetaObject::connectSlotsByName(KOnlineUpdateWizardPage);
    } // setupUi

    void retranslateUi(KOnlineUpdateWizardPage *KOnlineUpdateWizardPage)
    {
        KOnlineUpdateWizardPage->setTitle(tr2i18n("Online Update", nullptr));
        textLabel6->setText(tr2i18n("Select an online source and click <b>Finish</b> to store the investment data. If you do not want to use online updates, just leave the data as is.", nullptr));
        textLabel2_4->setText(tr2i18n("Use Finance::Quote", nullptr));
        m_useFinanceQuote->setText(QString());
        textLabel2_3->setText(tr2i18n("Online Source", nullptr));
        m_onlineSourceCombo->setItemText(0, QString());

        textLabel1_6->setText(tr2i18n("Factor", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KOnlineUpdateWizardPage: public Ui_KOnlineUpdateWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KONLINEUPDATEWIZARDPAGE_H

