/********************************************************************************
** Form generated from reading UI file 'separatorwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SEPARATORWIZARDPAGE_H
#define UI_SEPARATORWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SeparatorPage
{
public:
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *leftSpacer;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QSpacerItem *verticalSpacer_2;
    QFrame *line;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QGridLayout *gridLayout;
    QLabel *labelSet_fieldDelim;
    QComboBox *m_fieldDelimiter;
    QLabel *labelSet_textDelim;
    QComboBox *m_textDelimiter;
    QComboBox *m_encoding;
    QLabel *lblEncoding;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer_low;
    QSpacerItem *rightSpacer;

    void setupUi(QWidget *SeparatorPage)
    {
        if (SeparatorPage->objectName().isEmpty())
            SeparatorPage->setObjectName("SeparatorPage");
        SeparatorPage->resize(477, 239);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SeparatorPage->sizePolicy().hasHeightForWidth());
        SeparatorPage->setSizePolicy(sizePolicy);
        horizontalLayout_2 = new QHBoxLayout(SeparatorPage);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        leftSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(leftSpacer);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(SeparatorPage);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignCenter);
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        verticalSpacer_2 = new QSpacerItem(371, 3, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);

        line = new QFrame(SeparatorPage);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        verticalSpacer_3 = new QSpacerItem(371, 3, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_3);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(13, 13, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        labelSet_fieldDelim = new QLabel(SeparatorPage);
        labelSet_fieldDelim->setObjectName("labelSet_fieldDelim");
        labelSet_fieldDelim->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelSet_fieldDelim, 1, 0, 1, 1);

        m_fieldDelimiter = new QComboBox(SeparatorPage);
        m_fieldDelimiter->addItem(QString());
        m_fieldDelimiter->addItem(QString());
        m_fieldDelimiter->addItem(QString());
        m_fieldDelimiter->addItem(QString());
        m_fieldDelimiter->setObjectName("m_fieldDelimiter");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_fieldDelimiter->sizePolicy().hasHeightForWidth());
        m_fieldDelimiter->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(m_fieldDelimiter, 1, 1, 1, 1);

        labelSet_textDelim = new QLabel(SeparatorPage);
        labelSet_textDelim->setObjectName("labelSet_textDelim");
        labelSet_textDelim->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelSet_textDelim, 2, 0, 1, 1);

        m_textDelimiter = new QComboBox(SeparatorPage);
        m_textDelimiter->addItem(QString());
        m_textDelimiter->addItem(QString());
        m_textDelimiter->setObjectName("m_textDelimiter");
        sizePolicy1.setHeightForWidth(m_textDelimiter->sizePolicy().hasHeightForWidth());
        m_textDelimiter->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(m_textDelimiter, 2, 1, 1, 1);

        m_encoding = new QComboBox(SeparatorPage);
        m_encoding->setObjectName("m_encoding");

        gridLayout->addWidget(m_encoding, 0, 1, 1, 1);

        lblEncoding = new QLabel(SeparatorPage);
        lblEncoding->setObjectName("lblEncoding");
        lblEncoding->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblEncoding, 0, 0, 1, 1);

        gridLayout->setColumnStretch(0, 10);
        gridLayout->setColumnStretch(1, 10);

        horizontalLayout->addLayout(gridLayout);

        horizontalSpacer_2 = new QSpacerItem(13, 13, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        horizontalLayout->setStretch(0, 20);
        horizontalLayout->setStretch(1, 50);
        horizontalLayout->setStretch(2, 35);

        verticalLayout->addLayout(horizontalLayout);

        verticalSpacer_low = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_low);

        verticalLayout->setStretch(0, 30);
        verticalLayout->setStretch(1, 1);
        verticalLayout->setStretch(2, 1);
        verticalLayout->setStretch(3, 1);
        verticalLayout->setStretch(4, 1);
        verticalLayout->setStretch(5, 2);

        horizontalLayout_2->addLayout(verticalLayout);

        rightSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(rightSpacer);

        horizontalLayout_2->setStretch(0, 1);
        horizontalLayout_2->setStretch(1, 10);
        horizontalLayout_2->setStretch(2, 1);
        QWidget::setTabOrder(m_encoding, m_fieldDelimiter);
        QWidget::setTabOrder(m_fieldDelimiter, m_textDelimiter);

        retranslateUi(SeparatorPage);

        QMetaObject::connectSlotsByName(SeparatorPage);
    } // setupUi

    void retranslateUi(QWidget *SeparatorPage)
    {
        SeparatorPage->setWindowTitle(tr2i18n("Separator Wizard Page", nullptr));
        label->setText(tr2i18n("The field delimiter should be detected automatically.\n"
"Attempting to change it will reset any field choices.\n"
"The text delimiter does not normally need to be\n"
"changed, except possibly when within a field.", nullptr));
        labelSet_fieldDelim->setText(tr2i18n("Field Delimiter", nullptr));
        m_fieldDelimiter->setItemText(0, tr2i18n("comma (,)", nullptr));
        m_fieldDelimiter->setItemText(1, tr2i18n("semiColon (;)", nullptr));
        m_fieldDelimiter->setItemText(2, tr2i18n("colon (:)", nullptr));
        m_fieldDelimiter->setItemText(3, tr2i18n("tab (\\t)", nullptr));

#if QT_CONFIG(accessibility)
        labelSet_textDelim->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelSet_textDelim->setText(tr2i18n("Text Delimiter", "punctuation mark used as field delimiter"));
        m_textDelimiter->setItemText(0, tr2i18n("quote (\")", nullptr));
        m_textDelimiter->setItemText(1, tr2i18n("apostrophe (')", nullptr));

#if QT_CONFIG(tooltip)
        m_textDelimiter->setToolTip(tr2i18n("Select field delimiter/separator character.\n"
"This will reset any existing field choices.", nullptr));
#endif // QT_CONFIG(tooltip)
        lblEncoding->setText(tr2i18n("Encoding", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SeparatorPage: public Ui_SeparatorPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SEPARATORWIZARDPAGE_H

