/********************************************************************************
** Form generated from reading UI file 'kaccountsummarypage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KACCOUNTSUMMARYPAGE_H
#define UI_KACCOUNTSUMMARYPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include <ktextedit.h>

QT_BEGIN_NAMESPACE

class Ui_KAccountSummaryPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel1;
    KTextEdit *m_dataList;

    void setupUi(QWidget *KAccountSummaryPage)
    {
        if (KAccountSummaryPage->objectName().isEmpty())
            KAccountSummaryPage->setObjectName("KAccountSummaryPage");
        KAccountSummaryPage->resize(600, 480);
        vboxLayout = new QVBoxLayout(KAccountSummaryPage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        qLabel1 = new QLabel(KAccountSummaryPage);
        qLabel1->setObjectName("qLabel1");
        qLabel1->setFocusPolicy(Qt::NoFocus);
        qLabel1->setWordWrap(true);

        vboxLayout->addWidget(qLabel1);

        m_dataList = new KTextEdit(KAccountSummaryPage);
        m_dataList->setObjectName("m_dataList");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(2);
        sizePolicy.setHeightForWidth(m_dataList->sizePolicy().hasHeightForWidth());
        m_dataList->setSizePolicy(sizePolicy);
        m_dataList->setReadOnly(true);

        vboxLayout->addWidget(m_dataList);


        retranslateUi(KAccountSummaryPage);

        QMetaObject::connectSlotsByName(KAccountSummaryPage);
    } // setupUi

    void retranslateUi(QWidget *KAccountSummaryPage)
    {
        qLabel1->setText(tr2i18n("This page summarizes the data you have entered. Press <b>Finish</b> to create the account, schedules, etc. or use <b>Back</b> to modify your entries.", nullptr));
        (void)KAccountSummaryPage;
    } // retranslateUi

};

namespace Ui {
    class KAccountSummaryPage: public Ui_KAccountSummaryPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KACCOUNTSUMMARYPAGE_H

