/********************************************************************************
** Form generated from reading UI file 'kbackupdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KBACKUPDLG_H
#define UI_KBACKUPDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KBackupDlg
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QLabel *TextLabel1;
    QLabel *txtAutoMount;
    QFrame *Line1;
    QGroupBox *GroupBox1;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *lblMountPoint;
    KLineEdit *txtMountPoint;
    QPushButton *chooseButton;
    QCheckBox *mountCheckBox;
    QSpacerItem *Spacer17;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KBackupDlg)
    {
        if (KBackupDlg->objectName().isEmpty())
            KBackupDlg->setObjectName("KBackupDlg");
        KBackupDlg->resize(641, 369);
        verticalLayout_2 = new QVBoxLayout(KBackupDlg);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        TextLabel1 = new QLabel(KBackupDlg);
        TextLabel1->setObjectName("TextLabel1");
        TextLabel1->setAlignment(Qt::AlignTop);
        TextLabel1->setWordWrap(true);

        verticalLayout->addWidget(TextLabel1);

        txtAutoMount = new QLabel(KBackupDlg);
        txtAutoMount->setObjectName("txtAutoMount");
        txtAutoMount->setWordWrap(true);

        verticalLayout->addWidget(txtAutoMount);


        verticalLayout_2->addLayout(verticalLayout);

        Line1 = new QFrame(KBackupDlg);
        Line1->setObjectName("Line1");
        Line1->setFrameShape(QFrame::HLine);
        Line1->setFrameShadow(QFrame::Raised);

        verticalLayout_2->addWidget(Line1);

        GroupBox1 = new QGroupBox(KBackupDlg);
        GroupBox1->setObjectName("GroupBox1");
        vboxLayout = new QVBoxLayout(GroupBox1);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        lblMountPoint = new QLabel(GroupBox1);
        lblMountPoint->setObjectName("lblMountPoint");
        lblMountPoint->setWordWrap(false);

        hboxLayout->addWidget(lblMountPoint);

        txtMountPoint = new KLineEdit(GroupBox1);
        txtMountPoint->setObjectName("txtMountPoint");

        hboxLayout->addWidget(txtMountPoint);

        chooseButton = new QPushButton(GroupBox1);
        chooseButton->setObjectName("chooseButton");

        hboxLayout->addWidget(chooseButton);


        vboxLayout->addLayout(hboxLayout);

        mountCheckBox = new QCheckBox(GroupBox1);
        mountCheckBox->setObjectName("mountCheckBox");

        vboxLayout->addWidget(mountCheckBox);

        Spacer17 = new QSpacerItem(0, 24, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer17);


        verticalLayout_2->addWidget(GroupBox1);

        buttonBox = new QDialogButtonBox(KBackupDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);


        retranslateUi(KBackupDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KBackupDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KBackupDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KBackupDlg);
    } // setupUi

    void retranslateUi(QDialog *KBackupDlg)
    {
        KBackupDlg->setWindowTitle(tr2i18n("Backup", nullptr));
        TextLabel1->setText(tr2i18n("Use this dialog to backup your data.\n"
"\n"
"Please make sure you have a disk inserted and that the drive is ready. Then choose the mount point from either the Choose button or by entering the path in the available box.\n"
"\n"
"Click OK to perform the backup.", nullptr));
        txtAutoMount->setText(tr2i18n("If your system does not use an automounter, make sure you mark the checkbox below to \"mount this directory before backing up.\"", nullptr));
        GroupBox1->setTitle(tr2i18n("Device options", nullptr));
        lblMountPoint->setText(tr2i18n("Mount Point:", "This is the mount point"));
#if QT_CONFIG(tooltip)
        chooseButton->setToolTip(tr2i18n("Select mount point", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        chooseButton->setWhatsThis(tr2i18n("Use this to browse to the mount point.", nullptr));
#endif // QT_CONFIG(whatsthis)
        chooseButton->setText(tr2i18n("C&hoose...", nullptr));
        mountCheckBox->setText(tr2i18n("Mount this directory before backing up.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KBackupDlg: public Ui_KBackupDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KBACKUPDLG_H

