/********************************************************************************
** Form generated from reading UI file 'onlinejobmessagesview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ONLINEJOBMESSAGESVIEW_H
#define UI_ONLINEJOBMESSAGESVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_onlineJobMessageView
{
public:
    QVBoxLayout *verticalLayout;
    QTableView *tableView;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *closeButton;

    void setupUi(QWidget *onlineJobMessageView)
    {
        if (onlineJobMessageView->objectName().isEmpty())
            onlineJobMessageView->setObjectName("onlineJobMessageView");
        onlineJobMessageView->resize(668, 262);
        verticalLayout = new QVBoxLayout(onlineJobMessageView);
        verticalLayout->setObjectName("verticalLayout");
        tableView = new QTableView(onlineJobMessageView);
        tableView->setObjectName("tableView");
        tableView->setSelectionMode(QAbstractItemView::NoSelection);
        tableView->verticalHeader()->setVisible(false);

        verticalLayout->addWidget(tableView);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        closeButton = new QPushButton(onlineJobMessageView);
        closeButton->setObjectName("closeButton");

        horizontalLayout->addWidget(closeButton);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(onlineJobMessageView);

        QMetaObject::connectSlotsByName(onlineJobMessageView);
    } // setupUi

    void retranslateUi(QWidget *onlineJobMessageView)
    {
        closeButton->setText(tr2i18n("Close", nullptr));
        (void)onlineJobMessageView;
    } // retranslateUi

};

namespace Ui {
    class onlineJobMessageView: public Ui_onlineJobMessageView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // ONLINEJOBMESSAGESVIEW_H

