/****************************************************************************
** Meta object code from reading C++ file 'ktagsview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/views/ktagsview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ktagsview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9KTagsViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KTagsView::qt_create_metaobjectdata<qt_meta_tag_ZN9KTagsViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KTagsView",
        "slotSelectTag",
        "",
        "tagId",
        "slotHelp",
        "slotSettingsChanged",
        "updateActions",
        "SelectedObjects",
        "selections",
        "slotTagSelectionChanged",
        "QItemSelection",
        "selected",
        "deselected",
        "slotTransactionSelectionChanged",
        "slotTagDataChanged",
        "slotRenameSingleTag",
        "QModelIndex",
        "idx",
        "QVariant",
        "value",
        "slotUpdateTag",
        "slotNewTag",
        "slotRenameTag",
        "slotDeleteTag",
        "slotModelDataChanged",
        "topLeft",
        "bottomRight"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotSelectTag'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'slotHelp'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSettingsChanged'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateActions'
        QtMocHelpers::SlotData<void(const SelectedObjects &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotTagSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 10, 12 },
        }}),
        // Slot 'slotTransactionSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 10, 12 },
        }}),
        // Slot 'slotTagDataChanged'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRenameSingleTag'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QVariant &)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'slotUpdateTag'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotNewTag'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRenameTag'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDeleteTag'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotModelDataChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 25 }, { 0x80000000 | 16, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KTagsView, qt_meta_tag_ZN9KTagsViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KTagsView::staticMetaObject = { {
    QMetaObject::SuperData::link<KMyMoneyViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KTagsViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KTagsViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9KTagsViewE_t>.metaTypes,
    nullptr
} };

void KTagsView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KTagsView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotSelectTag((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->slotHelp(); break;
        case 2: _t->slotSettingsChanged(); break;
        case 3: _t->updateActions((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 4: _t->slotTagSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 5: _t->slotTransactionSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 6: _t->slotTagDataChanged(); break;
        case 7: _t->slotRenameSingleTag((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 8: _t->slotUpdateTag(); break;
        case 9: _t->slotNewTag(); break;
        case 10: _t->slotRenameTag(); break;
        case 11: _t->slotDeleteTag(); break;
        case 12: _t->slotModelDataChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *KTagsView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KTagsView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KTagsViewE_t>.strings))
        return static_cast<void*>(this);
    return KMyMoneyViewBase::qt_metacast(_clname);
}

int KTagsView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KMyMoneyViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
