/********************************************************************************
** Form generated from reading UI file 'newspliteditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NEWSPLITEDITOR_H
#define UI_NEWSPLITEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "creditdebitedit.h"
#include "kmymoneyaccountcombo.h"
#include "ktagcontainer.h"

QT_BEGIN_NAMESPACE

class Ui_NewSplitEditor
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *hl1;
    QSpacerItem *hs2;
    QLabel *amountLabel;
    CreditDebitEdit *creditDebitEdit;
    QPlainTextEdit *memoEdit;
    QHBoxLayout *hl2;
    QSpacerItem *hs3;
    QToolButton *enterButton;
    QToolButton *cancelButton;
    QLabel *tagsLabel;
    QComboBox *payeeEdit;
    QLabel *numberLabel;
    QLabel *payToLabel;
    KMyMoneyAccountCombo *accountCombo;
    QLabel *categoryLabel;
    QLabel *costCenterLabel;
    QLineEdit *numberEdit;
    QComboBox *costCenterCombo;
    QLabel *memoLabel;
    KTagContainer *tagContainer;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *NewSplitEditor)
    {
        if (NewSplitEditor->objectName().isEmpty())
            NewSplitEditor->setObjectName("NewSplitEditor");
        NewSplitEditor->resize(651, 295);
        gridLayout = new QGridLayout(NewSplitEditor);
        gridLayout->setObjectName("gridLayout");
        hl1 = new QHBoxLayout();
        hl1->setObjectName("hl1");
        hs2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hl1->addItem(hs2);

        amountLabel = new QLabel(NewSplitEditor);
        amountLabel->setObjectName("amountLabel");
        amountLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        hl1->addWidget(amountLabel);

        creditDebitEdit = new CreditDebitEdit(NewSplitEditor);
        creditDebitEdit->setObjectName("creditDebitEdit");
        creditDebitEdit->setMinimumSize(QSize(220, 0));
        creditDebitEdit->setProperty("kmm_taborder", QVariant(true));

        hl1->addWidget(creditDebitEdit);

        hl1->setStretch(1, 1);

        gridLayout->addLayout(hl1, 0, 0, 1, 4);

        memoEdit = new QPlainTextEdit(NewSplitEditor);
        memoEdit->setObjectName("memoEdit");
        memoEdit->setTabChangesFocus(true);
        memoEdit->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(memoEdit, 5, 1, 4, 1);

        hl2 = new QHBoxLayout();
        hl2->setObjectName("hl2");
        hs3 = new QSpacerItem(17, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hl2->addItem(hs3);

        enterButton = new QToolButton(NewSplitEditor);
        enterButton->setObjectName("enterButton");
        enterButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        enterButton->setProperty("kmm_taborder", QVariant(true));

        hl2->addWidget(enterButton);

        cancelButton = new QToolButton(NewSplitEditor);
        cancelButton->setObjectName("cancelButton");
        cancelButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        cancelButton->setProperty("kmm_taborder", QVariant(true));

        hl2->addWidget(cancelButton);


        gridLayout->addLayout(hl2, 8, 2, 1, 2);

        tagsLabel = new QLabel(NewSplitEditor);
        tagsLabel->setObjectName("tagsLabel");

        gridLayout->addWidget(tagsLabel, 3, 0, 1, 1);

        payeeEdit = new QComboBox(NewSplitEditor);
        payeeEdit->setObjectName("payeeEdit");
        payeeEdit->setEditable(true);
        payeeEdit->setInsertPolicy(QComboBox::InsertAfterCurrent);
        payeeEdit->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(payeeEdit, 1, 1, 1, 1);

        numberLabel = new QLabel(NewSplitEditor);
        numberLabel->setObjectName("numberLabel");

        gridLayout->addWidget(numberLabel, 1, 2, 1, 1);

        payToLabel = new QLabel(NewSplitEditor);
        payToLabel->setObjectName("payToLabel");

        gridLayout->addWidget(payToLabel, 1, 0, 1, 1);

        accountCombo = new KMyMoneyAccountCombo(NewSplitEditor);
        accountCombo->setObjectName("accountCombo");
        accountCombo->setEditable(true);
        accountCombo->setIconSize(QSize(0, 0));
        accountCombo->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(accountCombo, 2, 1, 1, 1);

        categoryLabel = new QLabel(NewSplitEditor);
        categoryLabel->setObjectName("categoryLabel");

        gridLayout->addWidget(categoryLabel, 2, 0, 1, 1);

        costCenterLabel = new QLabel(NewSplitEditor);
        costCenterLabel->setObjectName("costCenterLabel");

        gridLayout->addWidget(costCenterLabel, 2, 2, 1, 1);

        numberEdit = new QLineEdit(NewSplitEditor);
        numberEdit->setObjectName("numberEdit");
        numberEdit->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(numberEdit, 1, 3, 1, 1);

        costCenterCombo = new QComboBox(NewSplitEditor);
        costCenterCombo->setObjectName("costCenterCombo");
        costCenterCombo->setEditable(true);
        costCenterCombo->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(costCenterCombo, 2, 3, 1, 1);

        memoLabel = new QLabel(NewSplitEditor);
        memoLabel->setObjectName("memoLabel");
        memoLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout->addWidget(memoLabel, 5, 0, 1, 1);

        tagContainer = new KTagContainer(NewSplitEditor);
        tagContainer->setObjectName("tagContainer");
        tagContainer->setFocusPolicy(Qt::WheelFocus);
        tagContainer->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(tagContainer, 3, 1, 2, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 7, 2, 1, 1);

        gridLayout->setColumnStretch(1, 3);
        gridLayout->setColumnStretch(3, 1);
        QWidget::setTabOrder(payeeEdit, numberEdit);
        QWidget::setTabOrder(numberEdit, accountCombo);
        QWidget::setTabOrder(accountCombo, costCenterCombo);
        QWidget::setTabOrder(costCenterCombo, memoEdit);
        QWidget::setTabOrder(memoEdit, enterButton);
        QWidget::setTabOrder(enterButton, cancelButton);

        retranslateUi(NewSplitEditor);

        QMetaObject::connectSlotsByName(NewSplitEditor);
    } // setupUi

    void retranslateUi(QWidget *NewSplitEditor)
    {
        amountLabel->setText(tr2i18n("Amount", nullptr));
        enterButton->setText(tr2i18n("Enter", nullptr));
        cancelButton->setText(tr2i18n("Cancel", nullptr));
        tagsLabel->setText(tr2i18n("Tags", nullptr));
        numberLabel->setText(tr2i18n("Number", nullptr));
        payToLabel->setText(tr2i18n("Payer/Payee", nullptr));
        categoryLabel->setText(tr2i18n("Category", nullptr));
        costCenterLabel->setText(tr2i18n("Costcenter", nullptr));
        memoLabel->setText(tr2i18n("Memo", nullptr));
        (void)NewSplitEditor;
    } // retranslateUi

};

namespace Ui {
    class NewSplitEditor: public Ui_NewSplitEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // NEWSPLITEDITOR_H

