/********************************************************************************
** Form generated from reading UI file 'kinvestmentdetailswizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KINVESTMENTDETAILSWIZARDPAGE_H
#define UI_KINVESTMENTDETAILSWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <amountedit.h>
#include <kinvestmentdetailswizardpage.h>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "kmessagewidget.h"
#include "kmymoneycurrencyselector.h"
#include "kmymoneygeneralcombo.h"
#include "kmymoneylineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KInvestmentDetailsWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *textLabel3_2;
    KMessageWidget *m_messageWidget;
    QSpacerItem *spacer6;
    QGridLayout *gridLayout;
    KMyMoneyGeneralCombo *m_priceMode;
    QLabel *m_priceModeLabel;
    KComboBox *m_tradingMarket;
    QSpinBox *m_pricePrecision;
    QLabel *textLabel2_6;
    QLabel *textLabel1_3;
    QComboBox *m_roundingMethod;
    QComboBox *m_investmentSymbol;
    QLabel *textLabel2_2;
    QLabel *textLabel4;
    QLabel *textLabel1_4;
    KMyMoneySecuritySelector *m_tradingCurrencyEdit;
    QLabel *textLabel2_2_2;
    QLabel *textLabel3;
    KMyMoneyLineEdit *m_investmentName;
    QLabel *textLabel1_2;
    KMyMoneyLineEdit *m_investmentIdentification;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel1_5;
    AmountEdit *m_fraction;
    QCheckBox *m_createSymbolDuplicate;
    QSpacerItem *spacer5;

    void setupUi(KInvestmentDetailsWizardPage *KInvestmentDetailsWizardPage)
    {
        if (KInvestmentDetailsWizardPage->objectName().isEmpty())
            KInvestmentDetailsWizardPage->setObjectName("KInvestmentDetailsWizardPage");
        KInvestmentDetailsWizardPage->resize(395, 424);
        vboxLayout = new QVBoxLayout(KInvestmentDetailsWizardPage);
        vboxLayout->setObjectName("vboxLayout");
        textLabel3_2 = new QLabel(KInvestmentDetailsWizardPage);
        textLabel3_2->setObjectName("textLabel3_2");
        textLabel3_2->setFrameShape(QFrame::NoFrame);
        textLabel3_2->setAlignment(Qt::AlignTop);
        textLabel3_2->setWordWrap(true);

        vboxLayout->addWidget(textLabel3_2);

        m_messageWidget = new KMessageWidget(KInvestmentDetailsWizardPage);
        m_messageWidget->setObjectName("m_messageWidget");
        m_messageWidget->setWordWrap(true);
        m_messageWidget->setMessageType(KMessageWidget::Warning);

        vboxLayout->addWidget(m_messageWidget);

        spacer6 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer6);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        m_priceMode = new KMyMoneyGeneralCombo(KInvestmentDetailsWizardPage);
        m_priceMode->setObjectName("m_priceMode");

        gridLayout->addWidget(m_priceMode, 9, 1, 1, 1);

        m_priceModeLabel = new QLabel(KInvestmentDetailsWizardPage);
        m_priceModeLabel->setObjectName("m_priceModeLabel");
        m_priceModeLabel->setWordWrap(false);

        gridLayout->addWidget(m_priceModeLabel, 9, 0, 1, 1);

        m_tradingMarket = new KComboBox(KInvestmentDetailsWizardPage);
        m_tradingMarket->addItem(QString());
        m_tradingMarket->addItem(QString());
        m_tradingMarket->addItem(QString());
        m_tradingMarket->addItem(QString());
        m_tradingMarket->addItem(QString());
        m_tradingMarket->addItem(QString());
        m_tradingMarket->setObjectName("m_tradingMarket");
        m_tradingMarket->setEditable(true);

        gridLayout->addWidget(m_tradingMarket, 4, 1, 1, 1);

        m_pricePrecision = new QSpinBox(KInvestmentDetailsWizardPage);
        m_pricePrecision->setObjectName("m_pricePrecision");
        m_pricePrecision->setMinimum(2);
        m_pricePrecision->setMaximum(20);

        gridLayout->addWidget(m_pricePrecision, 7, 1, 1, 1);

        textLabel2_6 = new QLabel(KInvestmentDetailsWizardPage);
        textLabel2_6->setObjectName("textLabel2_6");

        gridLayout->addWidget(textLabel2_6, 7, 0, 1, 1);

        textLabel1_3 = new QLabel(KInvestmentDetailsWizardPage);
        textLabel1_3->setObjectName("textLabel1_3");
        textLabel1_3->setWordWrap(false);

        gridLayout->addWidget(textLabel1_3, 4, 0, 1, 1);

        m_roundingMethod = new QComboBox(KInvestmentDetailsWizardPage);
        m_roundingMethod->setObjectName("m_roundingMethod");

        gridLayout->addWidget(m_roundingMethod, 8, 1, 1, 1);

        m_investmentSymbol = new QComboBox(KInvestmentDetailsWizardPage);
        m_investmentSymbol->setObjectName("m_investmentSymbol");
        m_investmentSymbol->setEditable(true);

        gridLayout->addWidget(m_investmentSymbol, 0, 1, 1, 1);

        textLabel2_2 = new QLabel(KInvestmentDetailsWizardPage);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setWordWrap(false);

        gridLayout->addWidget(textLabel2_2, 0, 0, 1, 1);

        textLabel4 = new QLabel(KInvestmentDetailsWizardPage);
        textLabel4->setObjectName("textLabel4");

        gridLayout->addWidget(textLabel4, 8, 0, 1, 1);

        textLabel1_4 = new QLabel(KInvestmentDetailsWizardPage);
        textLabel1_4->setObjectName("textLabel1_4");
        textLabel1_4->setWordWrap(false);

        gridLayout->addWidget(textLabel1_4, 6, 0, 1, 1);

        m_tradingCurrencyEdit = new KMyMoneySecuritySelector(KInvestmentDetailsWizardPage);
        m_tradingCurrencyEdit->setObjectName("m_tradingCurrencyEdit");

        gridLayout->addWidget(m_tradingCurrencyEdit, 6, 1, 1, 1);

        textLabel2_2_2 = new QLabel(KInvestmentDetailsWizardPage);
        textLabel2_2_2->setObjectName("textLabel2_2_2");
        textLabel2_2_2->setWordWrap(false);

        gridLayout->addWidget(textLabel2_2_2, 3, 0, 1, 1);

        textLabel3 = new QLabel(KInvestmentDetailsWizardPage);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout->addWidget(textLabel3, 2, 0, 1, 1);

        m_investmentName = new KMyMoneyLineEdit(KInvestmentDetailsWizardPage);
        m_investmentName->setObjectName("m_investmentName");
        m_investmentName->setClearButtonEnabled(true);

        gridLayout->addWidget(m_investmentName, 2, 1, 1, 1);

        textLabel1_2 = new QLabel(KInvestmentDetailsWizardPage);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        gridLayout->addWidget(textLabel1_2, 5, 0, 1, 1);

        m_investmentIdentification = new KMyMoneyLineEdit(KInvestmentDetailsWizardPage);
        m_investmentIdentification->setObjectName("m_investmentIdentification");
        m_investmentIdentification->setClearButtonEnabled(true);

        gridLayout->addWidget(m_investmentIdentification, 5, 1, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        textLabel1_5 = new QLabel(KInvestmentDetailsWizardPage);
        textLabel1_5->setObjectName("textLabel1_5");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textLabel1_5->sizePolicy().hasHeightForWidth());
        textLabel1_5->setSizePolicy(sizePolicy);
        textLabel1_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel1_5->setWordWrap(false);

        hboxLayout->addWidget(textLabel1_5);

        m_fraction = new AmountEdit(KInvestmentDetailsWizardPage);
        m_fraction->setObjectName("m_fraction");

        hboxLayout->addWidget(m_fraction);


        gridLayout->addLayout(hboxLayout, 3, 1, 1, 1);

        m_createSymbolDuplicate = new QCheckBox(KInvestmentDetailsWizardPage);
        m_createSymbolDuplicate->setObjectName("m_createSymbolDuplicate");

        gridLayout->addWidget(m_createSymbolDuplicate, 1, 0, 1, 2);


        vboxLayout->addLayout(gridLayout);

        spacer5 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer5);

        QWidget::setTabOrder(m_investmentSymbol, m_investmentName);
        QWidget::setTabOrder(m_investmentName, m_fraction);
        QWidget::setTabOrder(m_fraction, m_tradingMarket);
        QWidget::setTabOrder(m_tradingMarket, m_investmentIdentification);
        QWidget::setTabOrder(m_investmentIdentification, m_tradingCurrencyEdit);
        QWidget::setTabOrder(m_tradingCurrencyEdit, m_pricePrecision);
        QWidget::setTabOrder(m_pricePrecision, m_roundingMethod);
        QWidget::setTabOrder(m_roundingMethod, m_priceMode);

        retranslateUi(KInvestmentDetailsWizardPage);

        QMetaObject::connectSlotsByName(KInvestmentDetailsWizardPage);
    } // setupUi

    void retranslateUi(KInvestmentDetailsWizardPage *KInvestmentDetailsWizardPage)
    {
        KInvestmentDetailsWizardPage->setTitle(tr2i18n("Security details", nullptr));
        textLabel3_2->setText(tr2i18n("Enter the details below and click <b>Next</b> to continue entering the online update details.", nullptr));
        m_priceModeLabel->setText(tr2i18n("Price entry", nullptr));
        m_tradingMarket->setItemText(0, tr2i18n("AMEX", nullptr));
        m_tradingMarket->setItemText(1, tr2i18n("EUREX", nullptr));
        m_tradingMarket->setItemText(2, tr2i18n("FUND", nullptr));
        m_tradingMarket->setItemText(3, tr2i18n("NASDAQ", nullptr));
        m_tradingMarket->setItemText(4, tr2i18n("NYSE", nullptr));
        m_tradingMarket->setItemText(5, tr2i18n("XETRA", nullptr));

#if QT_CONFIG(tooltip)
        m_pricePrecision->setToolTip(tr2i18n("This determines how many digits after decimal symbol price can have.", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel2_6->setText(tr2i18n("Price precision", nullptr));
        textLabel1_3->setText(tr2i18n("Trading market", nullptr));
#if QT_CONFIG(tooltip)
        m_roundingMethod->setToolTip(tr2i18n("This controls what to do with digit situated after precision digits in amount values.", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel2_2->setText(tr2i18n("Trading symbol", nullptr));
        textLabel4->setText(tr2i18n("Remainder", nullptr));
        textLabel1_4->setText(tr2i18n("Trading Currency", nullptr));
        textLabel2_2_2->setText(tr2i18n("Fraction", nullptr));
        textLabel3->setText(tr2i18n("Full name", nullptr));
        textLabel1_2->setText(tr2i18n("Identification\n"
"number", nullptr));
        textLabel1_5->setText(tr2i18n("1 /", nullptr));
        m_createSymbolDuplicate->setText(tr2i18n("Create duplicate of the symbol", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KInvestmentDetailsWizardPage: public Ui_KInvestmentDetailsWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KINVESTMENTDETAILSWIZARDPAGE_H

