/********************************************************************************
** Form generated from reading UI file 'interestchargecheckingswizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INTERESTCHARGECHECKINGSWIZARDPAGE_H
#define UI_INTERESTCHARGECHECKINGSWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>
#include <kmymoneycategory.h>
#include <kmymoneydateedit.h>
#include "amountedit.h"
#include "kmymoneypayeecombo.h"

QT_BEGIN_NAMESPACE

class Ui_InterestChargeCheckingsWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel1;
    QFrame *Line2;
    QSpacerItem *Spacer8;
    QHBoxLayout *hboxLayout;
    QSpacerItem *Spacer7;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout1;
    QLabel *textLabel1;
    KMyMoneyPayeeCombo *m_payeeEdit;
    QGroupBox *groupBox1;
    QVBoxLayout *vboxLayout2;
    KMyMoneyCategory *m_interestCategoryEdit;
    QHBoxLayout *hboxLayout2;
    AmountEdit *m_interestEdit;
    KMyMoneyDateEdit *m_interestDateEdit;
    QGroupBox *groupBox2;
    QVBoxLayout *vboxLayout3;
    KMyMoneyCategory *m_chargesCategoryEdit;
    QHBoxLayout *hboxLayout3;
    AmountEdit *m_chargesEdit;
    KMyMoneyDateEdit *m_chargesDateEdit;
    QSpacerItem *Spacer6;
    QSpacerItem *Spacer9;

    void setupUi(QWizardPage *InterestChargeCheckingsWizardPage)
    {
        if (InterestChargeCheckingsWizardPage->objectName().isEmpty())
            InterestChargeCheckingsWizardPage->setObjectName("InterestChargeCheckingsWizardPage");
        vboxLayout = new QVBoxLayout(InterestChargeCheckingsWizardPage);
        vboxLayout->setObjectName("vboxLayout");
        qLabel1 = new QLabel(InterestChargeCheckingsWizardPage);
        qLabel1->setObjectName("qLabel1");
        qLabel1->setWordWrap(true);

        vboxLayout->addWidget(qLabel1);

        Line2 = new QFrame(InterestChargeCheckingsWizardPage);
        Line2->setObjectName("Line2");
        Line2->setFrameShape(QFrame::HLine);
        Line2->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(Line2);

        Spacer8 = new QSpacerItem(16, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer8);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        Spacer7 = new QSpacerItem(60, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer7);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setObjectName("vboxLayout1");
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        textLabel1 = new QLabel(InterestChargeCheckingsWizardPage);
        textLabel1->setObjectName("textLabel1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textLabel1->sizePolicy().hasHeightForWidth());
        textLabel1->setSizePolicy(sizePolicy);
        textLabel1->setWordWrap(false);

        hboxLayout1->addWidget(textLabel1);

        m_payeeEdit = new KMyMoneyPayeeCombo(InterestChargeCheckingsWizardPage);
        m_payeeEdit->setObjectName("m_payeeEdit");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_payeeEdit->sizePolicy().hasHeightForWidth());
        m_payeeEdit->setSizePolicy(sizePolicy1);

        hboxLayout1->addWidget(m_payeeEdit);


        vboxLayout1->addLayout(hboxLayout1);

        groupBox1 = new QGroupBox(InterestChargeCheckingsWizardPage);
        groupBox1->setObjectName("groupBox1");
        vboxLayout2 = new QVBoxLayout(groupBox1);
        vboxLayout2->setContentsMargins(2, 2, 2, 2);
        vboxLayout2->setObjectName("vboxLayout2");
        m_interestCategoryEdit = new KMyMoneyCategory(groupBox1);
        m_interestCategoryEdit->setObjectName("m_interestCategoryEdit");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(m_interestCategoryEdit->sizePolicy().hasHeightForWidth());
        m_interestCategoryEdit->setSizePolicy(sizePolicy2);

        vboxLayout2->addWidget(m_interestCategoryEdit);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName("hboxLayout2");
        m_interestEdit = new AmountEdit(groupBox1);
        m_interestEdit->setObjectName("m_interestEdit");
        m_interestEdit->setProperty("resetButtonVisibility", QVariant(false));

        hboxLayout2->addWidget(m_interestEdit);

        m_interestDateEdit = new KMyMoneyDateEdit(groupBox1);
        m_interestDateEdit->setObjectName("m_interestDateEdit");

        hboxLayout2->addWidget(m_interestDateEdit);


        vboxLayout2->addLayout(hboxLayout2);


        vboxLayout1->addWidget(groupBox1);

        groupBox2 = new QGroupBox(InterestChargeCheckingsWizardPage);
        groupBox2->setObjectName("groupBox2");
        vboxLayout3 = new QVBoxLayout(groupBox2);
        vboxLayout3->setContentsMargins(2, 2, 2, 2);
        vboxLayout3->setObjectName("vboxLayout3");
        m_chargesCategoryEdit = new KMyMoneyCategory(groupBox2);
        m_chargesCategoryEdit->setObjectName("m_chargesCategoryEdit");
        sizePolicy2.setHeightForWidth(m_chargesCategoryEdit->sizePolicy().hasHeightForWidth());
        m_chargesCategoryEdit->setSizePolicy(sizePolicy2);

        vboxLayout3->addWidget(m_chargesCategoryEdit);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setObjectName("hboxLayout3");
        m_chargesEdit = new AmountEdit(groupBox2);
        m_chargesEdit->setObjectName("m_chargesEdit");
        m_chargesEdit->setProperty("resetButtonVisibility", QVariant(false));

        hboxLayout3->addWidget(m_chargesEdit);

        m_chargesDateEdit = new KMyMoneyDateEdit(groupBox2);
        m_chargesDateEdit->setObjectName("m_chargesDateEdit");

        hboxLayout3->addWidget(m_chargesDateEdit);


        vboxLayout3->addLayout(hboxLayout3);


        vboxLayout1->addWidget(groupBox2);


        hboxLayout->addLayout(vboxLayout1);

        Spacer6 = new QSpacerItem(60, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer6);


        vboxLayout->addLayout(hboxLayout);

        Spacer9 = new QSpacerItem(16, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer9);


        retranslateUi(InterestChargeCheckingsWizardPage);

        QMetaObject::connectSlotsByName(InterestChargeCheckingsWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *InterestChargeCheckingsWizardPage)
    {
        qLabel1->setProperty("text", QVariant(tr2i18n("If necessary, enter information about interest or charges here. KMyMoney will create transactions and mark them as cleared.", nullptr)));
        textLabel1->setText(tr2i18n("Payee", nullptr));
        groupBox1->setTitle(tr2i18n("Interest", nullptr));
        groupBox2->setTitle(tr2i18n("Charges", nullptr));
        (void)InterestChargeCheckingsWizardPage;
    } // retranslateUi

};

namespace Ui {
    class InterestChargeCheckingsWizardPage: public Ui_InterestChargeCheckingsWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // INTERESTCHARGECHECKINGSWIZARDPAGE_H

