/*
    SPDX-FileCopyrightText: 2013 Christian Dávid <christian-david@web.de>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "credittransfer.h"

creditTransfer::~creditTransfer()
{
}

QString creditTransfer::jobTypeName() const
{
    return "Credit Transfer";
}
