// This file is generated by kconfig_compiler_kf6 from pluginsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef PLUGINSETTINGS_H
#define PLUGINSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class PluginSettings : public KConfigSkeleton
{
  public:

    static PluginSettings *self();
    ~PluginSettings() override;

    /**
      Set Determines whether field delimiter will be autodetected.
    */
    static
    void setAutoFieldDelimiter( bool v )
    {
      if (!self()->isAutoFieldDelimiterImmutable())
        self()->mAutoFieldDelimiter = v;
    }

    /**
      Get Determines whether field delimiter will be autodetected.
    */
    static
    bool autoFieldDelimiter()
    {
      return self()->mAutoFieldDelimiter;
    }

    /**
      Is Determines whether field delimiter will be autodetected. Immutable
    */
    static
    bool isAutoFieldDelimiterImmutable()
    {
      return self()->isImmutable( QStringLiteral( "AutoFieldDelimiter" ) );
    }

    /**
      Set Determines whether decimal symbol will be autodetected.
    */
    static
    void setAutoDecimalSymbol( bool v )
    {
      if (!self()->isAutoDecimalSymbolImmutable())
        self()->mAutoDecimalSymbol = v;
    }

    /**
      Get Determines whether decimal symbol will be autodetected.
    */
    static
    bool autoDecimalSymbol()
    {
      return self()->mAutoDecimalSymbol;
    }

    /**
      Is Determines whether decimal symbol will be autodetected. Immutable
    */
    static
    bool isAutoDecimalSymbolImmutable()
    {
      return self()->isImmutable( QStringLiteral( "AutoDecimalSymbol" ) );
    }

    /**
      Set Determines whether date format will be autodetected.
    */
    static
    void setAutoDateFormat( bool v )
    {
      if (!self()->isAutoDateFormatImmutable())
        self()->mAutoDateFormat = v;
    }

    /**
      Get Determines whether date format will be autodetected.
    */
    static
    bool autoDateFormat()
    {
      return self()->mAutoDateFormat;
    }

    /**
      Is Determines whether date format will be autodetected. Immutable
    */
    static
    bool isAutoDateFormatImmutable()
    {
      return self()->isImmutable( QStringLiteral( "AutoDateFormat" ) );
    }

    /**
      Set The profile to be used for QIF exports.
    */
    static
    void setQifExportProfile( const QString & v )
    {
      if (!self()->isQifExportProfileImmutable())
        self()->mQifExportProfile = v;
    }

    /**
      Get The profile to be used for QIF exports.
    */
    static
    QString qifExportProfile()
    {
      return self()->mQifExportProfile;
    }

    /**
      Is The profile to be used for QIF exports. Immutable
    */
    static
    bool isQifExportProfileImmutable()
    {
      return self()->isImmutable( QStringLiteral( "QifExportProfile" ) );
    }

  protected:
    PluginSettings();
    friend class PluginSettingsHelper;


    // Misc
    bool mAutoFieldDelimiter;
    bool mAutoDecimalSymbol;
    bool mAutoDateFormat;
    QString mQifExportProfile;
};

#endif

