/********************************************************************************
** Form generated from reading UI file 'kenterscheduledlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KENTERSCHEDULEDLG_H
#define UI_KENTERSCHEDULEDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "newtransactioneditor.h"

QT_BEGIN_NAMESPACE

class Ui_KEnterScheduleDlg
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox1;
    QHBoxLayout *hboxLayout;
    QLabel *nameLabel;
    QLabel *m_scheduleName;
    QSpacerItem *hs2;
    QLabel *typeLabel;
    QLabel *m_type;
    NewTransactionEditor *m_editor;
    QFrame *line2;
    QHBoxLayout *hboxLayout1;
    QPushButton *buttonHelp;
    QSpacerItem *hs1;
    QPushButton *buttonOk;
    QPushButton *buttonSkip;
    QPushButton *buttonIgnore;
    QPushButton *buttonCancel;

    void setupUi(QDialog *KEnterScheduleDlg)
    {
        if (KEnterScheduleDlg->objectName().isEmpty())
            KEnterScheduleDlg->setObjectName("KEnterScheduleDlg");
        KEnterScheduleDlg->resize(780, 364);
        KEnterScheduleDlg->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(KEnterScheduleDlg);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        groupBox1 = new QGroupBox(KEnterScheduleDlg);
        groupBox1->setObjectName("groupBox1");
        hboxLayout = new QHBoxLayout(groupBox1);
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName("hboxLayout");
        nameLabel = new QLabel(groupBox1);
        nameLabel->setObjectName("nameLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(nameLabel->sizePolicy().hasHeightForWidth());
        nameLabel->setSizePolicy(sizePolicy);
        nameLabel->setMinimumSize(QSize(70, 0));
        QFont font;
        nameLabel->setFont(font);
        nameLabel->setWordWrap(false);

        hboxLayout->addWidget(nameLabel);

        m_scheduleName = new QLabel(groupBox1);
        m_scheduleName->setObjectName("m_scheduleName");
        m_scheduleName->setWordWrap(false);

        hboxLayout->addWidget(m_scheduleName);

        hs2 = new QSpacerItem(20, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(hs2);

        typeLabel = new QLabel(groupBox1);
        typeLabel->setObjectName("typeLabel");
        sizePolicy.setHeightForWidth(typeLabel->sizePolicy().hasHeightForWidth());
        typeLabel->setSizePolicy(sizePolicy);
        typeLabel->setMinimumSize(QSize(70, 0));
        typeLabel->setFont(font);
        typeLabel->setWordWrap(false);

        hboxLayout->addWidget(typeLabel);

        m_type = new QLabel(groupBox1);
        m_type->setObjectName("m_type");
        sizePolicy.setHeightForWidth(m_type->sizePolicy().hasHeightForWidth());
        m_type->setSizePolicy(sizePolicy);
        m_type->setWordWrap(false);

        hboxLayout->addWidget(m_type);


        vboxLayout->addWidget(groupBox1);

        m_editor = new NewTransactionEditor(KEnterScheduleDlg);
        m_editor->setObjectName("m_editor");

        vboxLayout->addWidget(m_editor);

        line2 = new QFrame(KEnterScheduleDlg);
        line2->setObjectName("line2");
        line2->setFrameShape(QFrame::HLine);
        line2->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(line2);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        buttonHelp = new QPushButton(KEnterScheduleDlg);
        buttonHelp->setObjectName("buttonHelp");

        hboxLayout1->addWidget(buttonHelp);

        hs1 = new QSpacerItem(190, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(hs1);

        buttonOk = new QPushButton(KEnterScheduleDlg);
        buttonOk->setObjectName("buttonOk");
        buttonOk->setAutoDefault(true);

        hboxLayout1->addWidget(buttonOk);

        buttonSkip = new QPushButton(KEnterScheduleDlg);
        buttonSkip->setObjectName("buttonSkip");
        buttonSkip->setProperty("isDragEnabled", QVariant(true));

        hboxLayout1->addWidget(buttonSkip);

        buttonIgnore = new QPushButton(KEnterScheduleDlg);
        buttonIgnore->setObjectName("buttonIgnore");

        hboxLayout1->addWidget(buttonIgnore);

        buttonCancel = new QPushButton(KEnterScheduleDlg);
        buttonCancel->setObjectName("buttonCancel");

        hboxLayout1->addWidget(buttonCancel);


        vboxLayout->addLayout(hboxLayout1);

        vboxLayout->setStretch(1, 2);

        retranslateUi(KEnterScheduleDlg);
        QObject::connect(buttonOk, &QPushButton::clicked, KEnterScheduleDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonCancel, &QPushButton::clicked, KEnterScheduleDlg, qOverload<>(&QDialog::reject));

        buttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(KEnterScheduleDlg);
    } // setupUi

    void retranslateUi(QDialog *KEnterScheduleDlg)
    {
        KEnterScheduleDlg->setWindowTitle(tr2i18n("Enter Schedule", nullptr));
        groupBox1->setTitle(tr2i18n("Schedule Details", nullptr));
        nameLabel->setText(tr2i18n("Name:", "@label schedule name"));
        m_scheduleName->setText(QString());
        typeLabel->setText(tr2i18n("Type:", nullptr));
        m_type->setText(QString());
        buttonHelp->setText(tr2i18n("Help", nullptr));
#if QT_CONFIG(tooltip)
        buttonOk->setToolTip(tr2i18n("Enter the transaction and advance the next due date of this schedule to the next payment date.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonOk->setText(tr2i18n("Enter", nullptr));
#if QT_CONFIG(shortcut)
        buttonOk->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        buttonSkip->setToolTip(tr2i18n("Do not enter the transaction but advance the next due date of this schedule to the next payment date.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonSkip->setText(tr2i18n("&Skip", nullptr));
#if QT_CONFIG(tooltip)
        buttonIgnore->setToolTip(tr2i18n("Do not enter or skip payments for this schedule but continue with the next schedule.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonIgnore->setText(tr2i18n("Ignore", nullptr));
#if QT_CONFIG(tooltip)
        buttonCancel->setToolTip(tr2i18n("Cancel processing of schedule entry.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonCancel->setText(tr2i18n("Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KEnterScheduleDlg: public Ui_KEnterScheduleDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KENTERSCHEDULEDLG_H

