/********************************************************************************
** Form generated from reading UI file 'keditscheduledlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KEDITSCHEDULEDLG_H
#define UI_KEDITSCHEDULEDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "kdatecombobox.h"
#include "klineedit.h"
#include "kmymoneydateedit.h"
#include "newtransactioneditor.h"

QT_BEGIN_NAMESPACE

class Ui_KEditScheduleDlg
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QHBoxLayout *hboxLayout1;
    QLabel *textLabel1;
    KLineEdit *nameEdit;
    QHBoxLayout *hboxLayout2;
    QLabel *textLabel3;
    QSpinBox *frequencyNoEdit;
    QComboBox *frequencyEdit;
    QGroupBox *paymentInformation;
    QVBoxLayout *paymentInformationLayout;
    QHBoxLayout *hboxLayout3;
    QLabel *textLabel1_3;
    QComboBox *paymentMethodCombo;
    QSpacerItem *spacer4;
    QFrame *line1;
    NewTransactionEditor *transactionEditor;
    QGroupBox *groupBox2;
    QVBoxLayout *vboxLayout1;
    QGridLayout *gridLayout;
    KComboBox *weekendOptionCombo;
    QLabel *textLabel1_2;
    QHBoxLayout *hboxLayout4;
    QCheckBox *estimateOption;
    QSpinBox *variationEdit;
    QSpacerItem *spacer5;
    QCheckBox *keepMultiCurrencyAmount;
    QCheckBox *lastDayInMonthOption;
    QCheckBox *autoEnterOption;
    QCheckBox *endSeriesOption;
    QFrame *endOptionsFrame;
    QHBoxLayout *hboxLayout5;
    QSpacerItem *Spacer3_2_2;
    QLabel *endLabel1;
    QSpinBox *remainingEdit;
    QLabel *endLabel2;
    KMyMoneyDateEdit *finalPaymentDateEdit;
    QSpacerItem *spacer12;
    QDialogButtonBox *buttonBox;

    void setupUi(QWidget *KEditScheduleDlg)
    {
        if (KEditScheduleDlg->objectName().isEmpty())
            KEditScheduleDlg->setObjectName("KEditScheduleDlg");
        KEditScheduleDlg->resize(798, 662);
        vboxLayout = new QVBoxLayout(KEditScheduleDlg);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        textLabel1 = new QLabel(KEditScheduleDlg);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        hboxLayout1->addWidget(textLabel1);

        nameEdit = new KLineEdit(KEditScheduleDlg);
        nameEdit->setObjectName("nameEdit");
        nameEdit->setFocusPolicy(Qt::StrongFocus);
        nameEdit->setProperty("kmm_taborder", QVariant(true));

        hboxLayout1->addWidget(nameEdit);


        hboxLayout->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        textLabel3 = new QLabel(KEditScheduleDlg);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        hboxLayout2->addWidget(textLabel3);

        frequencyNoEdit = new QSpinBox(KEditScheduleDlg);
        frequencyNoEdit->setObjectName("frequencyNoEdit");
        frequencyNoEdit->setFocusPolicy(Qt::StrongFocus);
        frequencyNoEdit->setMinimum(1);
        frequencyNoEdit->setMaximum(999);
        frequencyNoEdit->setProperty("kmm_taborder", QVariant(true));

        hboxLayout2->addWidget(frequencyNoEdit);

        frequencyEdit = new QComboBox(KEditScheduleDlg);
        frequencyEdit->setObjectName("frequencyEdit");
        frequencyEdit->setProperty("kmm_taborder", QVariant(true));

        hboxLayout2->addWidget(frequencyEdit);


        hboxLayout->addLayout(hboxLayout2);


        vboxLayout->addLayout(hboxLayout);

        paymentInformation = new QGroupBox(KEditScheduleDlg);
        paymentInformation->setObjectName("paymentInformation");
        paymentInformationLayout = new QVBoxLayout(paymentInformation);
        paymentInformationLayout->setSpacing(6);
        paymentInformationLayout->setContentsMargins(11, 11, 11, 11);
        paymentInformationLayout->setObjectName("paymentInformationLayout");
        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setSpacing(6);
        hboxLayout3->setObjectName("hboxLayout3");
        textLabel1_3 = new QLabel(paymentInformation);
        textLabel1_3->setObjectName("textLabel1_3");
        textLabel1_3->setWordWrap(false);

        hboxLayout3->addWidget(textLabel1_3);

        paymentMethodCombo = new QComboBox(paymentInformation);
        paymentMethodCombo->setObjectName("paymentMethodCombo");
        paymentMethodCombo->setProperty("kmm_taborder", QVariant(true));

        hboxLayout3->addWidget(paymentMethodCombo);

        spacer4 = new QSpacerItem(61, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout3->addItem(spacer4);


        paymentInformationLayout->addLayout(hboxLayout3);

        line1 = new QFrame(paymentInformation);
        line1->setObjectName("line1");
        line1->setFrameShape(QFrame::HLine);
        line1->setFrameShadow(QFrame::Sunken);

        paymentInformationLayout->addWidget(line1);

        transactionEditor = new NewTransactionEditor(paymentInformation);
        transactionEditor->setObjectName("transactionEditor");

        paymentInformationLayout->addWidget(transactionEditor);


        vboxLayout->addWidget(paymentInformation);

        groupBox2 = new QGroupBox(KEditScheduleDlg);
        groupBox2->setObjectName("groupBox2");
        vboxLayout1 = new QVBoxLayout(groupBox2);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        weekendOptionCombo = new KComboBox(groupBox2);
        weekendOptionCombo->addItem(QString());
        weekendOptionCombo->addItem(QString());
        weekendOptionCombo->addItem(QString());
        weekendOptionCombo->setObjectName("weekendOptionCombo");
        weekendOptionCombo->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(weekendOptionCombo, 1, 1, 1, 1);

        textLabel1_2 = new QLabel(groupBox2);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        gridLayout->addWidget(textLabel1_2, 1, 0, 1, 1);


        vboxLayout1->addLayout(gridLayout);

        hboxLayout4 = new QHBoxLayout();
        hboxLayout4->setSpacing(6);
        hboxLayout4->setObjectName("hboxLayout4");
        estimateOption = new QCheckBox(groupBox2);
        estimateOption->setObjectName("estimateOption");
        estimateOption->setProperty("kmm_taborder", QVariant(true));

        hboxLayout4->addWidget(estimateOption);

        variationEdit = new QSpinBox(groupBox2);
        variationEdit->setObjectName("variationEdit");
        variationEdit->setEnabled(false);
        variationEdit->setMaximum(32767);
        variationEdit->setValue(10);
        variationEdit->setProperty("kmm_taborder", QVariant(true));

        hboxLayout4->addWidget(variationEdit);

        spacer5 = new QSpacerItem(21, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout4->addItem(spacer5);


        vboxLayout1->addLayout(hboxLayout4);

        keepMultiCurrencyAmount = new QCheckBox(groupBox2);
        keepMultiCurrencyAmount->setObjectName("keepMultiCurrencyAmount");

        vboxLayout1->addWidget(keepMultiCurrencyAmount);

        lastDayInMonthOption = new QCheckBox(groupBox2);
        lastDayInMonthOption->setObjectName("lastDayInMonthOption");
        lastDayInMonthOption->setProperty("kmm_taborder", QVariant(true));

        vboxLayout1->addWidget(lastDayInMonthOption);

        autoEnterOption = new QCheckBox(groupBox2);
        autoEnterOption->setObjectName("autoEnterOption");
        autoEnterOption->setProperty("kmm_taborder", QVariant(true));

        vboxLayout1->addWidget(autoEnterOption);

        endSeriesOption = new QCheckBox(groupBox2);
        endSeriesOption->setObjectName("endSeriesOption");
        endSeriesOption->setProperty("kmm_taborder", QVariant(true));

        vboxLayout1->addWidget(endSeriesOption);

        endOptionsFrame = new QFrame(groupBox2);
        endOptionsFrame->setObjectName("endOptionsFrame");
        endOptionsFrame->setFrameShape(QFrame::NoFrame);
        endOptionsFrame->setFrameShadow(QFrame::Raised);
        hboxLayout5 = new QHBoxLayout(endOptionsFrame);
        hboxLayout5->setSpacing(6);
        hboxLayout5->setContentsMargins(11, 11, 11, 11);
        hboxLayout5->setObjectName("hboxLayout5");
        hboxLayout5->setContentsMargins(0, 0, 0, 0);
        Spacer3_2_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        hboxLayout5->addItem(Spacer3_2_2);

        endLabel1 = new QLabel(endOptionsFrame);
        endLabel1->setObjectName("endLabel1");
        endLabel1->setWordWrap(false);

        hboxLayout5->addWidget(endLabel1);

        remainingEdit = new QSpinBox(endOptionsFrame);
        remainingEdit->setObjectName("remainingEdit");
        remainingEdit->setMinimum(0);
        remainingEdit->setMaximum(9999);
        remainingEdit->setProperty("kmm_taborder", QVariant(true));

        hboxLayout5->addWidget(remainingEdit);

        endLabel2 = new QLabel(endOptionsFrame);
        endLabel2->setObjectName("endLabel2");
        endLabel2->setWordWrap(false);

        hboxLayout5->addWidget(endLabel2);

        finalPaymentDateEdit = new KMyMoneyDateEdit(endOptionsFrame);
        finalPaymentDateEdit->setObjectName("finalPaymentDateEdit");
        finalPaymentDateEdit->setFocusPolicy(Qt::StrongFocus);
        finalPaymentDateEdit->setProperty("kmm_taborder", QVariant(true));

        hboxLayout5->addWidget(finalPaymentDateEdit);


        vboxLayout1->addWidget(endOptionsFrame);


        vboxLayout->addWidget(groupBox2);

        spacer12 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer12);

        buttonBox = new QDialogButtonBox(KEditScheduleDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Ok);
        buttonBox->setProperty("kmm_taborder", QVariant(true));

        vboxLayout->addWidget(buttonBox);


        retranslateUi(KEditScheduleDlg);
        QObject::connect(estimateOption, &QCheckBox::toggled, variationEdit, &QSpinBox::setEnabled);
        QObject::connect(endSeriesOption, &QCheckBox::toggled, endOptionsFrame, &QFrame::setEnabled);

        QMetaObject::connectSlotsByName(KEditScheduleDlg);
    } // setupUi

    void retranslateUi(QWidget *KEditScheduleDlg)
    {
        KEditScheduleDlg->setWindowTitle(tr2i18n("Edit Scheduled transaction", nullptr));
        textLabel1->setText(tr2i18n("Schedule name:", nullptr));
        textLabel3->setText(tr2i18n("Frequency:", nullptr));
#if QT_CONFIG(tooltip)
        frequencyNoEdit->setToolTip(tr2i18n("Number of selected periods between entries", nullptr));
#endif // QT_CONFIG(tooltip)
        paymentInformation->setTitle(tr2i18n("Payment information", nullptr));
        textLabel1_3->setText(tr2i18n("Payment method", nullptr));
        groupBox2->setTitle(tr2i18n("Options", nullptr));
        weekendOptionCombo->setItemText(0, tr2i18n("Do not change the date", nullptr));
        weekendOptionCombo->setItemText(1, tr2i18n("Change the date to the previous processing day", nullptr));
        weekendOptionCombo->setItemText(2, tr2i18n("Change the date to the next processing day", nullptr));

        textLabel1_2->setText(tr2i18n("If this schedule occurs on a non-processing day:", nullptr));
        estimateOption->setText(tr2i18n("The amount is an estimate because it varies for each payment", nullptr));
#if QT_CONFIG(tooltip)
        keepMultiCurrencyAmount->setToolTip(tr2i18n("<html><head/><body><p>Select this if you don't want to modify the amount of the category side of a multi-currency transaction. The exchange rate of the transaction will be adjusted instead when the schedule is entered with a modified amount.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        keepMultiCurrencyAmount->setText(tr2i18n("Do not change amount of category but modify exchange rate of transaction", nullptr));
        lastDayInMonthOption->setText(tr2i18n("Process this schedule always at the last day of a month", nullptr));
        autoEnterOption->setText(tr2i18n("Enter this schedule into the register automatically when it is due", nullptr));
        endSeriesOption->setText(tr2i18n("This schedule will end at some time", nullptr));
        endLabel1->setText(tr2i18n("Number of transactions remaining:", nullptr));
        endLabel2->setText(tr2i18n("Date of final transaction:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KEditScheduleDlg: public Ui_KEditScheduleDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KEDITSCHEDULEDLG_H

