/********************************************************************************
** Form generated from reading UI file 'reporttabchart.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REPORTTABCHART_H
#define UI_REPORTTABCHART_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kmymoneygeneralcombo.h"

QT_BEGIN_NAMESPACE

class Ui_ReportTabChart
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel1;
    KMyMoneyGeneralCombo *m_comboType;
    QSpacerItem *spacer1;
    QHBoxLayout *_2;
    QLabel *textLabel1_2;
    KMyMoneyGeneralCombo *m_comboPalette;
    QSpacerItem *spacer1_2;
    QGridLayout *gridLayout;
    QCheckBox *m_checkValues;
    QCheckBox *m_checkShowChart;
    QCheckBox *m_checkSVGridLines;
    QCheckBox *m_checkCHGridLines;
    QCheckBox *m_logYaxis;
    QCheckBox *m_negExpenses;
    QHBoxLayout *hboxLayout1;
    QLabel *textLabel6;
    QSpinBox *m_lineWidth;
    QSpacerItem *spacer2;
    QSpacerItem *spacer15;

    void setupUi(QWidget *ReportTabChart)
    {
        if (ReportTabChart->objectName().isEmpty())
            ReportTabChart->setObjectName("ReportTabChart");
        ReportTabChart->resize(600, 234);
        vboxLayout = new QVBoxLayout(ReportTabChart);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        textLabel1 = new QLabel(ReportTabChart);
        textLabel1->setObjectName("textLabel1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textLabel1->sizePolicy().hasHeightForWidth());
        textLabel1->setSizePolicy(sizePolicy);
        textLabel1->setWordWrap(false);

        hboxLayout->addWidget(textLabel1);

        m_comboType = new KMyMoneyGeneralCombo(ReportTabChart);
        m_comboType->setObjectName("m_comboType");

        hboxLayout->addWidget(m_comboType);

        spacer1 = new QSpacerItem(121, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer1);


        vboxLayout->addLayout(hboxLayout);

        _2 = new QHBoxLayout();
        _2->setSpacing(6);
        _2->setObjectName("_2");
        textLabel1_2 = new QLabel(ReportTabChart);
        textLabel1_2->setObjectName("textLabel1_2");
        sizePolicy.setHeightForWidth(textLabel1_2->sizePolicy().hasHeightForWidth());
        textLabel1_2->setSizePolicy(sizePolicy);
        textLabel1_2->setWordWrap(false);

        _2->addWidget(textLabel1_2);

        m_comboPalette = new KMyMoneyGeneralCombo(ReportTabChart);
        m_comboPalette->setObjectName("m_comboPalette");

        _2->addWidget(m_comboPalette);

        spacer1_2 = new QSpacerItem(121, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _2->addItem(spacer1_2);


        vboxLayout->addLayout(_2);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        m_checkValues = new QCheckBox(ReportTabChart);
        m_checkValues->setObjectName("m_checkValues");

        gridLayout->addWidget(m_checkValues, 2, 0, 1, 1);

        m_checkShowChart = new QCheckBox(ReportTabChart);
        m_checkShowChart->setObjectName("m_checkShowChart");

        gridLayout->addWidget(m_checkShowChart, 3, 0, 1, 1);

        m_checkSVGridLines = new QCheckBox(ReportTabChart);
        m_checkSVGridLines->setObjectName("m_checkSVGridLines");
        m_checkSVGridLines->setText(QString::fromUtf8("SV grid lines"));

        gridLayout->addWidget(m_checkSVGridLines, 1, 0, 1, 1);

        m_checkCHGridLines = new QCheckBox(ReportTabChart);
        m_checkCHGridLines->setObjectName("m_checkCHGridLines");
        m_checkCHGridLines->setText(QString::fromUtf8("CH grid lines"));

        gridLayout->addWidget(m_checkCHGridLines, 0, 0, 1, 1);

        m_logYaxis = new QCheckBox(ReportTabChart);
        m_logYaxis->setObjectName("m_logYaxis");

        gridLayout->addWidget(m_logYaxis, 4, 0, 1, 1);

        m_negExpenses = new QCheckBox(ReportTabChart);
        m_negExpenses->setObjectName("m_negExpenses");

        gridLayout->addWidget(m_negExpenses, 5, 0, 1, 1);


        vboxLayout->addLayout(gridLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        textLabel6 = new QLabel(ReportTabChart);
        textLabel6->setObjectName("textLabel6");
        sizePolicy.setHeightForWidth(textLabel6->sizePolicy().hasHeightForWidth());
        textLabel6->setSizePolicy(sizePolicy);
        textLabel6->setWordWrap(false);

        hboxLayout1->addWidget(textLabel6);

        m_lineWidth = new QSpinBox(ReportTabChart);
        m_lineWidth->setObjectName("m_lineWidth");
        m_lineWidth->setMinimum(1);
        m_lineWidth->setMaximum(10);

        hboxLayout1->addWidget(m_lineWidth);

        spacer2 = new QSpacerItem(121, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacer2);


        vboxLayout->addLayout(hboxLayout1);

        spacer15 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer15);

        QWidget::setTabOrder(m_checkCHGridLines, m_checkSVGridLines);
        QWidget::setTabOrder(m_checkSVGridLines, m_checkValues);
        QWidget::setTabOrder(m_checkValues, m_checkShowChart);
        QWidget::setTabOrder(m_checkShowChart, m_logYaxis);
        QWidget::setTabOrder(m_logYaxis, m_negExpenses);
        QWidget::setTabOrder(m_negExpenses, m_lineWidth);

        retranslateUi(ReportTabChart);

        QMetaObject::connectSlotsByName(ReportTabChart);
    } // setupUi

    void retranslateUi(QWidget *ReportTabChart)
    {
        ReportTabChart->setWindowTitle(tr2i18n("Chart Tab", nullptr));
#if QT_CONFIG(tooltip)
        ReportTabChart->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        ReportTabChart->setWhatsThis(tr2i18n("<p>On this tab, you configure the chart drawn for this report.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        textLabel1->setToolTip(tr2i18n("<p>Select what form you would like the chart to be drawn as.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel1->setText(tr2i18n("Chart Type", nullptr));
#if QT_CONFIG(tooltip)
        textLabel1_2->setToolTip(tr2i18n("<p>Select what form you would like the chart to be drawn as.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel1_2->setText(tr2i18n("Chart Palette", nullptr));
#if QT_CONFIG(tooltip)
        m_checkValues->setToolTip(tr2i18n("<p>Select this option to draw the numeric values for data points next to their plot location.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkValues->setText(tr2i18n("Draw values on chart", nullptr));
#if QT_CONFIG(tooltip)
        m_checkShowChart->setToolTip(tr2i18n("<p>Select this option to cause the report to be shown as a chart when you first open the report.  Otherwise, it will come up as a text report.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkShowChart->setText(tr2i18n("Show as chart by default", nullptr));
#if QT_CONFIG(tooltip)
        m_checkCHGridLines->setToolTip(tr2i18n("<p>Select this option to show horizontal and vertical grid lines on the chart.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_logYaxis->setText(tr2i18n("Logarithmic vertical axis", nullptr));
        m_negExpenses->setText(tr2i18n("Plot expenses downwards", nullptr));
#if QT_CONFIG(tooltip)
        textLabel6->setToolTip(tr2i18n("<p>Select what width should be used to draw the line on the chart</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel6->setText(tr2i18n("Line width", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ReportTabChart: public Ui_ReportTabChart {};
} // namespace Ui

QT_END_NAMESPACE

#endif // REPORTTABCHART_H

