/********************************************************************************
** Form generated from reading UI file 'kgeneratesqldlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KGENERATESQLDLG_H
#define UI_KGENERATESQLDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "klineedit.h"
#include "ktextedit.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_KGenerateSqlDlg
{
public:
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QGroupBox *groupBox1;
    QVBoxLayout *verticalLayout;
    QListWidget *listDrivers;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout;
    QLabel *label;
    KUrlRequester *urlSqlite;
    QLabel *label_2;
    KLineEdit *textDbName;
    QLabel *label_3;
    KLineEdit *textHostName;
    QLabel *label_4;
    KLineEdit *textUserName;
    QLabel *label_5;
    KLineEdit *textPassword;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_7;
    KTextEdit *textSQL;
    QDialogButtonBox *buttonBox;

    void setupUi(QWidget *KGenerateSqlDlg)
    {
        if (KGenerateSqlDlg->objectName().isEmpty())
            KGenerateSqlDlg->setObjectName("KGenerateSqlDlg");
        KGenerateSqlDlg->resize(800, 528);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(KGenerateSqlDlg->sizePolicy().hasHeightForWidth());
        KGenerateSqlDlg->setSizePolicy(sizePolicy);
        KGenerateSqlDlg->setMinimumSize(QSize(0, 0));
        KGenerateSqlDlg->setBaseSize(QSize(518, 679));
        verticalLayout_3 = new QVBoxLayout(KGenerateSqlDlg);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        groupBox1 = new QGroupBox(KGenerateSqlDlg);
        groupBox1->setObjectName("groupBox1");
        verticalLayout = new QVBoxLayout(groupBox1);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        listDrivers = new QListWidget(groupBox1);
        listDrivers->setObjectName("listDrivers");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(listDrivers->sizePolicy().hasHeightForWidth());
        listDrivers->setSizePolicy(sizePolicy1);

        verticalLayout->addWidget(listDrivers);


        horizontalLayout->addWidget(groupBox1);

        groupBox = new QGroupBox(KGenerateSqlDlg);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(groupBox);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        urlSqlite = new KUrlRequester(groupBox);
        urlSqlite->setObjectName("urlSqlite");

        gridLayout->addWidget(urlSqlite, 0, 1, 1, 1);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        textDbName = new KLineEdit(groupBox);
        textDbName->setObjectName("textDbName");

        gridLayout->addWidget(textDbName, 1, 1, 1, 1);

        label_3 = new QLabel(groupBox);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        textHostName = new KLineEdit(groupBox);
        textHostName->setObjectName("textHostName");

        gridLayout->addWidget(textHostName, 2, 1, 1, 1);

        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 3, 0, 1, 1);

        textUserName = new KLineEdit(groupBox);
        textUserName->setObjectName("textUserName");

        gridLayout->addWidget(textUserName, 3, 1, 1, 1);

        label_5 = new QLabel(groupBox);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 4, 0, 1, 1);

        textPassword = new KLineEdit(groupBox);
        textPassword->setObjectName("textPassword");
        textPassword->setEchoMode(QLineEdit::Password);

        gridLayout->addWidget(textPassword, 4, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout);


        horizontalLayout->addWidget(groupBox);


        verticalLayout_3->addLayout(horizontalLayout);

        groupBox_2 = new QGroupBox(KGenerateSqlDlg);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_7 = new QVBoxLayout(groupBox_2);
        verticalLayout_7->setSpacing(6);
        verticalLayout_7->setContentsMargins(11, 11, 11, 11);
        verticalLayout_7->setObjectName("verticalLayout_7");
        textSQL = new KTextEdit(groupBox_2);
        textSQL->setObjectName("textSQL");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(1);
        sizePolicy2.setHeightForWidth(textSQL->sizePolicy().hasHeightForWidth());
        textSQL->setSizePolicy(sizePolicy2);

        verticalLayout_7->addWidget(textSQL);


        verticalLayout_3->addWidget(groupBox_2);

        buttonBox = new QDialogButtonBox(KGenerateSqlDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Ok);
        buttonBox->setCenterButtons(false);

        verticalLayout_3->addWidget(buttonBox);


        retranslateUi(KGenerateSqlDlg);

        QMetaObject::connectSlotsByName(KGenerateSqlDlg);
    } // setupUi

    void retranslateUi(QWidget *KGenerateSqlDlg)
    {
        KGenerateSqlDlg->setWindowTitle(tr2i18n("KMyMoney - Generate SQL", nullptr));
        groupBox1->setTitle(tr2i18n("Database Type", nullptr));
        groupBox->setTitle(tr2i18n("Database Details", nullptr));
        label->setText(tr2i18n("Database File (SQLite only)", nullptr));
        label_2->setText(tr2i18n("Database Name (Others)", nullptr));
        label_3->setText(tr2i18n("Host Name", nullptr));
        label_4->setText(tr2i18n("Username", nullptr));
        label_5->setText(tr2i18n("Password", nullptr));
        groupBox_2->setTitle(tr2i18n("SQL for creation", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KGenerateSqlDlg: public Ui_KGenerateSqlDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KGENERATESQLDLG_H

