/********************************************************************************
** Form generated from reading UI file 'ksettingshome.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KSETTINGSHOME_H
#define UI_KSETTINGSHOME_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KSettingsHome
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout;
    QVBoxLayout *vboxLayout1;
    QSpacerItem *spacer5;
    QPushButton *m_upButton;
    QPushButton *m_downButton;
    QSpacerItem *spacer15;
    QSpacerItem *spacer16;
    QLabel *textLabel4;
    KLineEdit *kcfg_ItemList;
    QListWidget *m_homePageList;
    QGroupBox *groupbox;
    QGridLayout *gridLayout1;
    QLabel *percentageLabel;
    QCheckBox *kcfg_RememberZoomFactor;
    QDoubleSpinBox *kcfg_ZoomFactor;
    QSpacerItem *spacer4;
    QGroupBox *groupbox2;
    QGridLayout *gridLayout2;
    QCheckBox *kcfg_ShowCountOfUnmarkedTransactions;
    QCheckBox *kcfg_ShowDateOfLastReconciliation;
    QCheckBox *kcfg_ShowBalanceStatusOfOnlineAccounts;
    QCheckBox *kcfg_ShowCountOfClearedTransactions;
    QCheckBox *kcfg_ShowCountOfNotReconciledTransactions;
    QCheckBox *kcfg_ShowLimitInfo;
    QCheckBox *kcfg_HideZeroBalanceAccountsHome;

    void setupUi(QWidget *KSettingsHome)
    {
        if (KSettingsHome->objectName().isEmpty())
            KSettingsHome->setObjectName("KSettingsHome");
        KSettingsHome->resize(659, 520);
        vboxLayout = new QVBoxLayout(KSettingsHome);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName("vboxLayout1");
        spacer5 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        vboxLayout1->addItem(spacer5);

        m_upButton = new QPushButton(KSettingsHome);
        m_upButton->setObjectName("m_upButton");

        vboxLayout1->addWidget(m_upButton);

        m_downButton = new QPushButton(KSettingsHome);
        m_downButton->setObjectName("m_downButton");

        vboxLayout1->addWidget(m_downButton);

        spacer15 = new QSpacerItem(21, 70, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacer15);


        gridLayout->addLayout(vboxLayout1, 0, 2, 1, 1);

        spacer16 = new QSpacerItem(80, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacer16, 0, 3, 1, 1);

        textLabel4 = new QLabel(KSettingsHome);
        textLabel4->setObjectName("textLabel4");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textLabel4->sizePolicy().hasHeightForWidth());
        textLabel4->setSizePolicy(sizePolicy);
        textLabel4->setAlignment(Qt::AlignTop);
        textLabel4->setWordWrap(true);

        gridLayout->addWidget(textLabel4, 1, 2, 1, 2);

        kcfg_ItemList = new KLineEdit(KSettingsHome);
        kcfg_ItemList->setObjectName("kcfg_ItemList");

        gridLayout->addWidget(kcfg_ItemList, 0, 0, 2, 1);

        m_homePageList = new QListWidget(KSettingsHome);
        m_homePageList->setObjectName("m_homePageList");

        gridLayout->addWidget(m_homePageList, 0, 1, 2, 1);


        vboxLayout->addLayout(gridLayout);

        groupbox = new QGroupBox(KSettingsHome);
        groupbox->setObjectName("groupbox");
        gridLayout1 = new QGridLayout(groupbox);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName("gridLayout1");
        percentageLabel = new QLabel(groupbox);
        percentageLabel->setObjectName("percentageLabel");
        percentageLabel->setWordWrap(false);

        gridLayout1->addWidget(percentageLabel, 0, 0, 1, 1);

        kcfg_RememberZoomFactor = new QCheckBox(groupbox);
        kcfg_RememberZoomFactor->setObjectName("kcfg_RememberZoomFactor");

        gridLayout1->addWidget(kcfg_RememberZoomFactor, 1, 0, 1, 3);

        kcfg_ZoomFactor = new QDoubleSpinBox(groupbox);
        kcfg_ZoomFactor->setObjectName("kcfg_ZoomFactor");
        kcfg_ZoomFactor->setMinimum(0.250000000000000);
        kcfg_ZoomFactor->setMaximum(5.000000000000000);
        kcfg_ZoomFactor->setSingleStep(0.050000000000000);
        kcfg_ZoomFactor->setValue(1.000000000000000);

        gridLayout1->addWidget(kcfg_ZoomFactor, 0, 1, 1, 1);

        spacer4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout1->addItem(spacer4, 0, 2, 1, 1);


        vboxLayout->addWidget(groupbox);

        groupbox2 = new QGroupBox(KSettingsHome);
        groupbox2->setObjectName("groupbox2");
        gridLayout2 = new QGridLayout(groupbox2);
        gridLayout2->setSpacing(6);
        gridLayout2->setContentsMargins(11, 11, 11, 11);
        gridLayout2->setObjectName("gridLayout2");
        kcfg_ShowCountOfUnmarkedTransactions = new QCheckBox(groupbox2);
        kcfg_ShowCountOfUnmarkedTransactions->setObjectName("kcfg_ShowCountOfUnmarkedTransactions");

        gridLayout2->addWidget(kcfg_ShowCountOfUnmarkedTransactions, 2, 0, 1, 2);

        kcfg_ShowDateOfLastReconciliation = new QCheckBox(groupbox2);
        kcfg_ShowDateOfLastReconciliation->setObjectName("kcfg_ShowDateOfLastReconciliation");

        gridLayout2->addWidget(kcfg_ShowDateOfLastReconciliation, 5, 0, 1, 2);

        kcfg_ShowBalanceStatusOfOnlineAccounts = new QCheckBox(groupbox2);
        kcfg_ShowBalanceStatusOfOnlineAccounts->setObjectName("kcfg_ShowBalanceStatusOfOnlineAccounts");

        gridLayout2->addWidget(kcfg_ShowBalanceStatusOfOnlineAccounts, 1, 0, 1, 2);

        kcfg_ShowCountOfClearedTransactions = new QCheckBox(groupbox2);
        kcfg_ShowCountOfClearedTransactions->setObjectName("kcfg_ShowCountOfClearedTransactions");

        gridLayout2->addWidget(kcfg_ShowCountOfClearedTransactions, 3, 0, 1, 2);

        kcfg_ShowCountOfNotReconciledTransactions = new QCheckBox(groupbox2);
        kcfg_ShowCountOfNotReconciledTransactions->setObjectName("kcfg_ShowCountOfNotReconciledTransactions");

        gridLayout2->addWidget(kcfg_ShowCountOfNotReconciledTransactions, 4, 0, 1, 2);

        kcfg_ShowLimitInfo = new QCheckBox(groupbox2);
        kcfg_ShowLimitInfo->setObjectName("kcfg_ShowLimitInfo");

        gridLayout2->addWidget(kcfg_ShowLimitInfo, 0, 0, 1, 2);

        kcfg_HideZeroBalanceAccountsHome = new QCheckBox(groupbox2);
        kcfg_HideZeroBalanceAccountsHome->setObjectName("kcfg_HideZeroBalanceAccountsHome");

        gridLayout2->addWidget(kcfg_HideZeroBalanceAccountsHome, 6, 0, 1, 2);


        vboxLayout->addWidget(groupbox2);


        retranslateUi(KSettingsHome);

        QMetaObject::connectSlotsByName(KSettingsHome);
    } // setupUi

    void retranslateUi(QWidget *KSettingsHome)
    {
        KSettingsHome->setWindowTitle(tr2i18n("Homepage Settings", nullptr));
#if QT_CONFIG(tooltip)
        m_upButton->setToolTip(tr2i18n("Move selected item up", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_upButton->setWhatsThis(tr2i18n("Use this to move the selected item up by one position in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_upButton->setText(tr2i18n("Up", "@action move item up"));
#if QT_CONFIG(tooltip)
        m_downButton->setToolTip(tr2i18n("Move selected item down", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_downButton->setWhatsThis(tr2i18n("Use this to move the selected item down by one position in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_downButton->setText(tr2i18n("Down", "@action move item down"));
        textLabel4->setText(tr2i18n("Selected entries are shown on the home page of the application.<p>\n"
"Use the buttons and checkboxes to customize the layout of the home page.", nullptr));
        kcfg_ItemList->setText(QString());
        groupbox->setTitle(tr2i18n("Homepage/Summary page scaling", nullptr));
        percentageLabel->setText(tr2i18n("Zoom factor:", nullptr));
        kcfg_RememberZoomFactor->setText(tr2i18n("Remember zoom factor when leaving the program if manually changed with mouse-wheel", nullptr));
        groupbox2->setTitle(tr2i18n("Account information display", nullptr));
        kcfg_ShowCountOfUnmarkedTransactions->setText(tr2i18n("Show number of not marked transactions per account [!M]", nullptr));
        kcfg_ShowDateOfLastReconciliation->setText(tr2i18n("Show date of last reconciliation", nullptr));
        kcfg_ShowBalanceStatusOfOnlineAccounts->setText(tr2i18n("Show balance-status of mapped online accounts", nullptr));
        kcfg_ShowCountOfClearedTransactions->setText(tr2i18n("Show number of cleared transactions per account [C]", nullptr));
        kcfg_ShowCountOfNotReconciledTransactions->setText(tr2i18n("Show number of not reconciled (not marked + cleared) transactions per account [!R]", nullptr));
        kcfg_ShowLimitInfo->setText(tr2i18n("Show Account Limit Information", nullptr));
        kcfg_HideZeroBalanceAccountsHome->setText(tr2i18n("Hide accounts with a zero balance from Home", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KSettingsHome: public Ui_KSettingsHome {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KSETTINGSHOME_H

