/****************************************************************************
** Meta object code from reading C++ file 'journalmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/mymoney/storage/journalmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'journalmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12JournalModelE_t {};
} // unnamed namespace

template <> constexpr inline auto JournalModel::qt_create_metaobjectdata<qt_meta_tag_ZN12JournalModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JournalModel",
        "balancesChanged",
        "",
        "QHash<QString,AccountBalances>",
        "balances",
        "balanceChanged",
        "accountId",
        "idChanged",
        "current",
        "previous",
        "updateBalances",
        "resetRowHeightInformation",
        "Column",
        "Invisible",
        "Number",
        "EntryDate",
        "Date",
        "Account",
        "Payee",
        "Security",
        "CostCenter",
        "Detail",
        "Reconciliation",
        "Payment",
        "Deposit",
        "Quantity",
        "Price",
        "Amount",
        "Value",
        "Balance",
        "MaxColumns"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'balancesChanged'
        QtMocHelpers::SignalData<void(const QHash<QString,AccountBalances> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'balanceChanged'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'idChanged'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 },
        }}),
        // Slot 'updateBalances'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetRowHeightInformation'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Column'
        QtMocHelpers::EnumData<enum Column>(12, 12, QMC::EnumFlags{}).add({
            {   13, Column::Invisible },
            {   14, Column::Number },
            {   15, Column::EntryDate },
            {   16, Column::Date },
            {   17, Column::Account },
            {   18, Column::Payee },
            {   19, Column::Security },
            {   20, Column::CostCenter },
            {   21, Column::Detail },
            {   22, Column::Reconciliation },
            {   23, Column::Payment },
            {   24, Column::Deposit },
            {   25, Column::Quantity },
            {   26, Column::Price },
            {   27, Column::Amount },
            {   28, Column::Value },
            {   29, Column::Balance },
            {   30, Column::MaxColumns },
        }),
    };
    return QtMocHelpers::metaObjectData<JournalModel, qt_meta_tag_ZN12JournalModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JournalModel::staticMetaObject = { {
    QMetaObject::SuperData::link<MyMoneyModel<JournalEntry>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JournalModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JournalModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12JournalModelE_t>.metaTypes,
    nullptr
} };

void JournalModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JournalModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->balancesChanged((*reinterpret_cast<std::add_pointer_t<QHash<QString,AccountBalances>>>(_a[1]))); break;
        case 1: _t->balanceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->idChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->updateBalances(); break;
        case 4: _t->resetRowHeightInformation(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (JournalModel::*)(const QHash<QString,AccountBalances> & )>(_a, &JournalModel::balancesChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (JournalModel::*)(const QString & )>(_a, &JournalModel::balanceChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (JournalModel::*)(const QString & , const QString & )>(_a, &JournalModel::idChanged, 2))
            return;
    }
}

const QMetaObject *JournalModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JournalModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JournalModelE_t>.strings))
        return static_cast<void*>(this);
    return MyMoneyModel<JournalEntry>::qt_metacast(_clname);
}

int JournalModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MyMoneyModel<JournalEntry>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void JournalModel::balancesChanged(const QHash<QString,AccountBalances> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void JournalModel::balanceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void JournalModel::idChanged(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN26JournalModelNewTransactionE_t {};
} // unnamed namespace

template <> constexpr inline auto JournalModelNewTransaction::qt_create_metaobjectdata<qt_meta_tag_ZN26JournalModelNewTransactionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JournalModelNewTransaction"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JournalModelNewTransaction, qt_meta_tag_ZN26JournalModelNewTransactionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JournalModelNewTransaction::staticMetaObject = { {
    QMetaObject::SuperData::link<JournalModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JournalModelNewTransactionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JournalModelNewTransactionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26JournalModelNewTransactionE_t>.metaTypes,
    nullptr
} };

void JournalModelNewTransaction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JournalModelNewTransaction *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JournalModelNewTransaction::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JournalModelNewTransaction::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JournalModelNewTransactionE_t>.strings))
        return static_cast<void*>(this);
    return JournalModel::qt_metacast(_clname);
}

int JournalModelNewTransaction::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JournalModel::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
