/********************************************************************************
** Form generated from reading UI file 'kpayeereassigndlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPAYEEREASSIGNDLG_H
#define UI_KPAYEEREASSIGNDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kmymoneypayeecombo.h"

QT_BEGIN_NAMESPACE

class Ui_KPayeeReassignDlg
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *textLabel1;
    QSpacerItem *spacer8;
    QLabel *textLabel2;
    KMyMoneyPayeeCombo *payeeCombo;
    QSpacerItem *spacer9;
    QCheckBox *m_copyToMatchList;
    QFrame *line1;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KPayeeReassignDlg)
    {
        if (KPayeeReassignDlg->objectName().isEmpty())
            KPayeeReassignDlg->setObjectName("KPayeeReassignDlg");
        KPayeeReassignDlg->resize(558, 312);
        KPayeeReassignDlg->setSizeGripEnabled(false);
        KPayeeReassignDlg->setModal(true);
        vboxLayout = new QVBoxLayout(KPayeeReassignDlg);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        textLabel1 = new QLabel(KPayeeReassignDlg);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setMinimumSize(QSize(300, 0));
        textLabel1->setAlignment(Qt::AlignJustify|Qt::AlignTop);
        textLabel1->setWordWrap(true);

        vboxLayout->addWidget(textLabel1);

        spacer8 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        vboxLayout->addItem(spacer8);

        textLabel2 = new QLabel(KPayeeReassignDlg);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        vboxLayout->addWidget(textLabel2);

        payeeCombo = new KMyMoneyPayeeCombo(KPayeeReassignDlg);
        payeeCombo->setObjectName("payeeCombo");

        vboxLayout->addWidget(payeeCombo);

        spacer9 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer9);

        m_copyToMatchList = new QCheckBox(KPayeeReassignDlg);
        m_copyToMatchList->setObjectName("m_copyToMatchList");
        m_copyToMatchList->setChecked(true);

        vboxLayout->addWidget(m_copyToMatchList);

        line1 = new QFrame(KPayeeReassignDlg);
        line1->setObjectName("line1");
        line1->setFrameShape(QFrame::HLine);
        line1->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(line1);

        buttonBox = new QDialogButtonBox(KPayeeReassignDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(KPayeeReassignDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KPayeeReassignDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KPayeeReassignDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KPayeeReassignDlg);
    } // setupUi

    void retranslateUi(QDialog *KPayeeReassignDlg)
    {
        KPayeeReassignDlg->setWindowTitle(tr2i18n("Reassign payees", nullptr));
        textLabel2->setText(tr2i18n("Available payees:", nullptr));
        m_copyToMatchList->setText(tr2i18n("Assign deleted names to the above selected payee's matching list", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KPayeeReassignDlg: public Ui_KPayeeReassignDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KPAYEEREASSIGNDLG_H

