/********************************************************************************
** Form generated from reading UI file 'priceswizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PRICESWIZARDPAGE_H
#define UI_PRICESWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_PricesPage
{
public:
    QHBoxLayout *horizontalLayout;
    QSpacerItem *leftSpacer;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QGridLayout *gridLayout;
    QFrame *line_2;
    QFrame *line_3;
    QComboBox *m_priceFraction;
    QLabel *lbldateCol;
    QLabel *lblpriceCol;
    QComboBox *m_dateCol;
    QComboBox *m_priceCol;
    QLabel *lblpriceFraction;
    QPushButton *button_clear;
    QSpacerItem *rightSpacer;

    void setupUi(QWidget *PricesPage)
    {
        if (PricesPage->objectName().isEmpty())
            PricesPage->setObjectName("PricesPage");
        PricesPage->resize(846, 284);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(PricesPage->sizePolicy().hasHeightForWidth());
        PricesPage->setSizePolicy(sizePolicy);
        horizontalLayout = new QHBoxLayout(PricesPage);
        horizontalLayout->setObjectName("horizontalLayout");
        leftSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(leftSpacer);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(PricesPage);
        label->setObjectName("label");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);
        QFont font;
        font.setBold(false);
        label->setFont(font);
        label->setAlignment(Qt::AlignCenter);
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        line_2 = new QFrame(PricesPage);
        line_2->setObjectName("line_2");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(line_2->sizePolicy().hasHeightForWidth());
        line_2->setSizePolicy(sizePolicy2);
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line_2, 5, 0, 1, 7);

        line_3 = new QFrame(PricesPage);
        line_3->setObjectName("line_3");
        sizePolicy2.setHeightForWidth(line_3->sizePolicy().hasHeightForWidth());
        line_3->setSizePolicy(sizePolicy2);
        line_3->setFrameShape(QFrame::Shape::HLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line_3, 0, 0, 1, 7);

        m_priceFraction = new QComboBox(PricesPage);
        m_priceFraction->setObjectName("m_priceFraction");
        sizePolicy1.setHeightForWidth(m_priceFraction->sizePolicy().hasHeightForWidth());
        m_priceFraction->setSizePolicy(sizePolicy1);
        m_priceFraction->setMaxVisibleItems(5);

        gridLayout->addWidget(m_priceFraction, 1, 5, 1, 1);

        lbldateCol = new QLabel(PricesPage);
        lbldateCol->setObjectName("lbldateCol");
        lbldateCol->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lbldateCol, 1, 0, 1, 1);

        lblpriceCol = new QLabel(PricesPage);
        lblpriceCol->setObjectName("lblpriceCol");
        lblpriceCol->setEnabled(true);
        lblpriceCol->setMaximumSize(QSize(16777215, 16777215));
        lblpriceCol->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblpriceCol, 1, 2, 1, 1);

        m_dateCol = new QComboBox(PricesPage);
        m_dateCol->setObjectName("m_dateCol");
        sizePolicy1.setHeightForWidth(m_dateCol->sizePolicy().hasHeightForWidth());
        m_dateCol->setSizePolicy(sizePolicy1);
        m_dateCol->setMaxVisibleItems(12);

        gridLayout->addWidget(m_dateCol, 1, 1, 1, 1);

        m_priceCol = new QComboBox(PricesPage);
        m_priceCol->setObjectName("m_priceCol");
        sizePolicy1.setHeightForWidth(m_priceCol->sizePolicy().hasHeightForWidth());
        m_priceCol->setSizePolicy(sizePolicy1);
        m_priceCol->setMaxVisibleItems(12);

        gridLayout->addWidget(m_priceCol, 1, 3, 1, 1);

        lblpriceFraction = new QLabel(PricesPage);
        lblpriceFraction->setObjectName("lblpriceFraction");
        lblpriceFraction->setMaximumSize(QSize(16777215, 16777215));
        lblpriceFraction->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblpriceFraction, 1, 4, 1, 1);

        button_clear = new QPushButton(PricesPage);
        button_clear->setObjectName("button_clear");
        sizePolicy1.setHeightForWidth(button_clear->sizePolicy().hasHeightForWidth());
        button_clear->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(button_clear, 1, 6, 1, 1);


        verticalLayout->addLayout(gridLayout);


        horizontalLayout->addLayout(verticalLayout);

        rightSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(rightSpacer);

        horizontalLayout->setStretch(0, 1);
        horizontalLayout->setStretch(1, 50);
        horizontalLayout->setStretch(2, 1);

        retranslateUi(PricesPage);

        m_priceFraction->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(PricesPage);
    } // setupUi

    void retranslateUi(QWidget *PricesPage)
    {
        PricesPage->setWindowTitle(tr2i18n("Prices Wizard Page", nullptr));
        label->setText(tr2i18n("Please select the appropriate columns to use, corresponding to your data.\n"
"", nullptr));
#if QT_CONFIG(tooltip)
        m_priceFraction->setToolTip(tr2i18n("If necessary, select a fraction/multiplier for compatibility\n"
"between imported and stored prices. For instance, if the\n"
"import price is in cents but the file is priced in dollars, select 0.01.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        lbldateCol->setToolTip(tr2i18n("Select column containing date field", "verb, Select column containing date field"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        lbldateCol->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        lbldateCol->setText(tr2i18n("Date", nullptr));
#if QT_CONFIG(accessibility)
        lblpriceCol->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        lblpriceCol->setText(tr2i18n("Price", nullptr));
#if QT_CONFIG(tooltip)
        m_dateCol->setToolTip(tr2i18n("Select column containing date field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_priceCol->setToolTip(tr2i18n("Select column containing price field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        lblpriceFraction->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        lblpriceFraction->setText(tr2i18n("Price Fraction", "noun, conversion fraction/multiplier to use to make compatible the csv pricing with KMM pricing."));
#if QT_CONFIG(tooltip)
        button_clear->setToolTip(tr2i18n("Clear selected column entries.", nullptr));
#endif // QT_CONFIG(tooltip)
        button_clear->setText(tr2i18n("Clear", "verb, to clear selected columns"));
    } // retranslateUi

};

namespace Ui {
    class PricesPage: public Ui_PricesPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PRICESWIZARDPAGE_H

