/********************************************************************************
** Form generated from reading UI file 'kinstitutionsview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KINSTITUTIONSVIEW_H
#define UI_KINSTITUTIONSVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kmymoneyaccounttreeview.h"

QT_BEGIN_NAMESPACE

class Ui_KInstitutionsView
{
public:
    QVBoxLayout *verticalLayout;
    KMyMoneyAccountTreeView *m_accountTree;
    QWidget *m_filterContainer;
    QHBoxLayout *horizontalLayout;
    QToolButton *m_closeButton;
    QLineEdit *m_searchWidget;
    QHBoxLayout *h1;
    QToolButton *m_collapseButton;
    QToolButton *m_expandButton;
    QSpacerItem *Spacer1_2;
    QLabel *m_totalProfitsLabel;

    void setupUi(QWidget *KInstitutionsView)
    {
        if (KInstitutionsView->objectName().isEmpty())
            KInstitutionsView->setObjectName("KInstitutionsView");
        KInstitutionsView->resize(237, 211);
        verticalLayout = new QVBoxLayout(KInstitutionsView);
        verticalLayout->setSpacing(0);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        m_accountTree = new KMyMoneyAccountTreeView(KInstitutionsView);
        m_accountTree->setObjectName("m_accountTree");

        verticalLayout->addWidget(m_accountTree);

        m_filterContainer = new QWidget(KInstitutionsView);
        m_filterContainer->setObjectName("m_filterContainer");
        horizontalLayout = new QHBoxLayout(m_filterContainer);
        horizontalLayout->setSpacing(2);
        horizontalLayout->setContentsMargins(11, 11, 11, 11);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 1, 0, 1);
        m_closeButton = new QToolButton(m_filterContainer);
        m_closeButton->setObjectName("m_closeButton");
        m_closeButton->setAutoRaise(true);

        horizontalLayout->addWidget(m_closeButton);

        m_searchWidget = new QLineEdit(m_filterContainer);
        m_searchWidget->setObjectName("m_searchWidget");

        horizontalLayout->addWidget(m_searchWidget);


        verticalLayout->addWidget(m_filterContainer);

        h1 = new QHBoxLayout();
        h1->setSpacing(6);
        h1->setObjectName("h1");
        m_collapseButton = new QToolButton(KInstitutionsView);
        m_collapseButton->setObjectName("m_collapseButton");
        m_collapseButton->setAutoRaise(true);

        h1->addWidget(m_collapseButton);

        m_expandButton = new QToolButton(KInstitutionsView);
        m_expandButton->setObjectName("m_expandButton");
        m_expandButton->setAutoRaise(true);

        h1->addWidget(m_expandButton);

        Spacer1_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        h1->addItem(Spacer1_2);

        m_totalProfitsLabel = new QLabel(KInstitutionsView);
        m_totalProfitsLabel->setObjectName("m_totalProfitsLabel");
        m_totalProfitsLabel->setMinimumSize(QSize(150, 0));
        m_totalProfitsLabel->setTextFormat(Qt::RichText);
        m_totalProfitsLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        m_totalProfitsLabel->setWordWrap(false);

        h1->addWidget(m_totalProfitsLabel);


        verticalLayout->addLayout(h1);


        retranslateUi(KInstitutionsView);
        QObject::connect(m_expandButton, &QToolButton::clicked, m_accountTree, qOverload<>(&KMyMoneyAccountTreeView::expandAll));
        QObject::connect(m_collapseButton, &QToolButton::clicked, m_accountTree, qOverload<>(&KMyMoneyAccountTreeView::collapseAll));

        QMetaObject::connectSlotsByName(KInstitutionsView);
    } // setupUi

    void retranslateUi(QWidget *KInstitutionsView)
    {
        m_closeButton->setText(tr2i18n("...", nullptr));
        m_collapseButton->setText(tr2i18n("...", nullptr));
        m_expandButton->setText(tr2i18n("...", nullptr));
        m_totalProfitsLabel->setText(tr2i18n("Total Profits:", nullptr));
        (void)KInstitutionsView;
    } // retranslateUi

};

namespace Ui {
    class KInstitutionsView: public Ui_KInstitutionsView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KINSTITUTIONSVIEW_H

