/********************************************************************************
** Form generated from reading UI file 'kmymoneypricedlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KMYMONEYPRICEDLG_H
#define UI_KMYMONEYPRICEDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KMyMoneyPriceDlg
{
public:
    QVBoxLayout *verticalLayout;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *m_listLayout;
    KLineEdit *m_searchWidget;
    QTreeView *m_priceList;
    QCheckBox *m_showAllPrices;
    QVBoxLayout *vboxLayout1;
    QPushButton *m_newButton;
    QPushButton *m_editButton;
    QPushButton *m_onlineQuoteButton;
    QPushButton *m_deleteButton;
    QSpacerItem *spacer7;
    QDialogButtonBox *buttonBox;
    QFrame *line2;

    void setupUi(QDialog *KMyMoneyPriceDlg)
    {
        if (KMyMoneyPriceDlg->objectName().isEmpty())
            KMyMoneyPriceDlg->setObjectName("KMyMoneyPriceDlg");
        KMyMoneyPriceDlg->resize(806, 440);
        verticalLayout = new QVBoxLayout(KMyMoneyPriceDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        vboxLayout = new QVBoxLayout();
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_listLayout = new QVBoxLayout();
        m_listLayout->setSpacing(6);
        m_listLayout->setObjectName("m_listLayout");
        m_searchWidget = new KLineEdit(KMyMoneyPriceDlg);
        m_searchWidget->setObjectName("m_searchWidget");
        m_searchWidget->setProperty("showClearButton", QVariant(true));

        m_listLayout->addWidget(m_searchWidget);

        m_priceList = new QTreeView(KMyMoneyPriceDlg);
        m_priceList->setObjectName("m_priceList");
        m_priceList->setProperty("showDropIndicator", QVariant(false));
        m_priceList->setSelectionMode(QAbstractItemView::ExtendedSelection);
        m_priceList->setRootIsDecorated(false);
        m_priceList->setAllColumnsShowFocus(true);

        m_listLayout->addWidget(m_priceList);

        m_showAllPrices = new QCheckBox(KMyMoneyPriceDlg);
        m_showAllPrices->setObjectName("m_showAllPrices");

        m_listLayout->addWidget(m_showAllPrices);


        hboxLayout->addLayout(m_listLayout);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName("vboxLayout1");
        m_newButton = new QPushButton(KMyMoneyPriceDlg);
        m_newButton->setObjectName("m_newButton");

        vboxLayout1->addWidget(m_newButton);

        m_editButton = new QPushButton(KMyMoneyPriceDlg);
        m_editButton->setObjectName("m_editButton");

        vboxLayout1->addWidget(m_editButton);

        m_onlineQuoteButton = new QPushButton(KMyMoneyPriceDlg);
        m_onlineQuoteButton->setObjectName("m_onlineQuoteButton");

        vboxLayout1->addWidget(m_onlineQuoteButton);

        m_deleteButton = new QPushButton(KMyMoneyPriceDlg);
        m_deleteButton->setObjectName("m_deleteButton");

        vboxLayout1->addWidget(m_deleteButton);

        spacer7 = new QSpacerItem(21, 150, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacer7);


        hboxLayout->addLayout(vboxLayout1);


        vboxLayout->addLayout(hboxLayout);

        buttonBox = new QDialogButtonBox(KMyMoneyPriceDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Close|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);

        line2 = new QFrame(KMyMoneyPriceDlg);
        line2->setObjectName("line2");
        line2->setFrameShape(QFrame::HLine);
        line2->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(line2);


        verticalLayout->addLayout(vboxLayout);

        QWidget::setTabOrder(m_newButton, m_editButton);
        QWidget::setTabOrder(m_editButton, m_deleteButton);
        QWidget::setTabOrder(m_deleteButton, m_onlineQuoteButton);

        retranslateUi(KMyMoneyPriceDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KMyMoneyPriceDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KMyMoneyPriceDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KMyMoneyPriceDlg);
    } // setupUi

    void retranslateUi(QDialog *KMyMoneyPriceDlg)
    {
        KMyMoneyPriceDlg->setWindowTitle(tr2i18n("Price Editor", nullptr));
        m_searchWidget->setPlaceholderText(tr2i18n("Search...", "Placeholder text for search bar"));
        m_showAllPrices->setText(tr2i18n("Show all stored prices", nullptr));
#if QT_CONFIG(tooltip)
        m_newButton->setToolTip(tr2i18n("Add a new entry", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_newButton->setWhatsThis(tr2i18n("Create a new price entry.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_newButton->setText(tr2i18n("New", "@action add new price"));
#if QT_CONFIG(tooltip)
        m_editButton->setToolTip(tr2i18n("Modify the selected entry", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_editButton->setWhatsThis(tr2i18n("Change the details of selected price information.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_editButton->setText(tr2i18n("Modify", nullptr));
        m_onlineQuoteButton->setText(tr2i18n("Online Quotes", nullptr));
#if QT_CONFIG(tooltip)
        m_deleteButton->setToolTip(tr2i18n("Delete this entry", nullptr));
#endif // QT_CONFIG(tooltip)
        m_deleteButton->setText(tr2i18n("Delete", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KMyMoneyPriceDlg: public Ui_KMyMoneyPriceDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KMYMONEYPRICEDLG_H

