/********************************************************************************
** Form generated from reading UI file 'editselectionwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITSELECTIONWIZARDPAGE_H
#define UI_EDITSELECTIONWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_EditSelectionWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel2_2_1;
    QSpacerItem *Spacer64;
    QHBoxLayout *hboxLayout;
    QSpacerItem *Spacer67_4;
    QFrame *frame;
    QVBoxLayout *vboxLayout1;
    QRadioButton *m_editInterestRateButton;
    QRadioButton *m_editOtherCostButton;
    QRadioButton *m_editOtherInfoButton;
    QRadioButton *m_editAttributesButton;
    QSpacerItem *Spacer66;
    QSpacerItem *Spacer65;
    QButtonGroup *m_selectionButtonGroup;

    void setupUi(QWizardPage *EditSelectionWizardPage)
    {
        if (EditSelectionWizardPage->objectName().isEmpty())
            EditSelectionWizardPage->setObjectName("EditSelectionWizardPage");
        EditSelectionWizardPage->resize(467, 262);
        vboxLayout = new QVBoxLayout(EditSelectionWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel2_2_1 = new QLabel(EditSelectionWizardPage);
        TextLabel2_2_1->setObjectName("TextLabel2_2_1");
        TextLabel2_2_1->setAlignment(Qt::AlignTop);
        TextLabel2_2_1->setWordWrap(true);

        vboxLayout->addWidget(TextLabel2_2_1);

        Spacer64 = new QSpacerItem(0, 48, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer64);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        Spacer67_4 = new QSpacerItem(93, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer67_4);

        frame = new QFrame(EditSelectionWizardPage);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::NoFrame);
        frame->setFrameShadow(QFrame::Plain);
        vboxLayout1 = new QVBoxLayout(frame);
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        m_editInterestRateButton = new QRadioButton(frame);
        m_selectionButtonGroup = new QButtonGroup(EditSelectionWizardPage);
        m_selectionButtonGroup->setObjectName("m_selectionButtonGroup");
        m_selectionButtonGroup->addButton(m_editInterestRateButton);
        m_editInterestRateButton->setObjectName("m_editInterestRateButton");

        vboxLayout1->addWidget(m_editInterestRateButton);

        m_editOtherCostButton = new QRadioButton(frame);
        m_selectionButtonGroup->addButton(m_editOtherCostButton);
        m_editOtherCostButton->setObjectName("m_editOtherCostButton");

        vboxLayout1->addWidget(m_editOtherCostButton);

        m_editOtherInfoButton = new QRadioButton(frame);
        m_selectionButtonGroup->addButton(m_editOtherInfoButton);
        m_editOtherInfoButton->setObjectName("m_editOtherInfoButton");

        vboxLayout1->addWidget(m_editOtherInfoButton);

        m_editAttributesButton = new QRadioButton(frame);
        m_selectionButtonGroup->addButton(m_editAttributesButton);
        m_editAttributesButton->setObjectName("m_editAttributesButton");

        vboxLayout1->addWidget(m_editAttributesButton);


        hboxLayout->addWidget(frame);

        Spacer66 = new QSpacerItem(108, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer66);


        vboxLayout->addLayout(hboxLayout);

        Spacer65 = new QSpacerItem(0, 47, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer65);


        retranslateUi(EditSelectionWizardPage);

        QMetaObject::connectSlotsByName(EditSelectionWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *EditSelectionWizardPage)
    {
        EditSelectionWizardPage->setTitle(tr2i18n("Edit selection", nullptr));
        TextLabel2_2_1->setText(tr2i18n("\n"
"Please select, which data of the loan you want to modify.", nullptr));
        m_editInterestRateButton->setText(tr2i18n("Modify the interest rate", nullptr));
        m_editOtherCostButton->setText(tr2i18n("Modify additional fees", nullptr));
        m_editOtherInfoButton->setText(tr2i18n("Modify other loan information", nullptr));
        m_editAttributesButton->setText(tr2i18n("Modify loan attributes", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EditSelectionWizardPage: public Ui_EditSelectionWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // EDITSELECTIONWIZARDPAGE_H

