/********************************************************************************
** Form generated from reading UI file 'kreconciliationreportdlgdecl.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KRECONCILIATIONREPORTDLGDECL_H
#define UI_KRECONCILIATIONREPORTDLGDECL_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KReconciliationReportDlgDecl
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *m_tabWidget;
    QWidget *m_summaryTab;
    QVBoxLayout *m_summaryLayout;
    QWidget *m_detailsTab;
    QVBoxLayout *m_detailsLayout;
    QDialogButtonBox *m_buttonBox;

    void setupUi(QDialog *KReconciliationReportDlgDecl)
    {
        if (KReconciliationReportDlgDecl->objectName().isEmpty())
            KReconciliationReportDlgDecl->setObjectName("KReconciliationReportDlgDecl");
        KReconciliationReportDlgDecl->resize(980, 480);
        KReconciliationReportDlgDecl->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(KReconciliationReportDlgDecl);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        m_tabWidget = new QTabWidget(KReconciliationReportDlgDecl);
        m_tabWidget->setObjectName("m_tabWidget");
        m_summaryTab = new QWidget();
        m_summaryTab->setObjectName("m_summaryTab");
        m_summaryLayout = new QVBoxLayout(m_summaryTab);
        m_summaryLayout->setSpacing(6);
        m_summaryLayout->setContentsMargins(11, 11, 11, 11);
        m_summaryLayout->setObjectName("m_summaryLayout");
        m_tabWidget->addTab(m_summaryTab, QString());
        m_detailsTab = new QWidget();
        m_detailsTab->setObjectName("m_detailsTab");
        m_detailsLayout = new QVBoxLayout(m_detailsTab);
        m_detailsLayout->setSpacing(6);
        m_detailsLayout->setContentsMargins(11, 11, 11, 11);
        m_detailsLayout->setObjectName("m_detailsLayout");
        m_tabWidget->addTab(m_detailsTab, QString());

        vboxLayout->addWidget(m_tabWidget);

        m_buttonBox = new QDialogButtonBox(KReconciliationReportDlgDecl);
        m_buttonBox->setObjectName("m_buttonBox");
        m_buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        vboxLayout->addWidget(m_buttonBox);


        retranslateUi(KReconciliationReportDlgDecl);
        QObject::connect(m_buttonBox, &QDialogButtonBox::accepted, KReconciliationReportDlgDecl, qOverload<>(&QDialog::accept));

        m_tabWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(KReconciliationReportDlgDecl);
    } // setupUi

    void retranslateUi(QDialog *KReconciliationReportDlgDecl)
    {
        KReconciliationReportDlgDecl->setWindowTitle(tr2i18n("Reconciliation report", nullptr));
        m_tabWidget->setTabText(m_tabWidget->indexOf(m_summaryTab), tr2i18n("Summar&y", nullptr));
        m_tabWidget->setTabText(m_tabWidget->indexOf(m_detailsTab), tr2i18n("&Details", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KReconciliationReportDlgDecl: public Ui_KReconciliationReportDlgDecl {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KRECONCILIATIONREPORTDLGDECL_H

