/********************************************************************************
** Form generated from reading UI file 'ksettingsregister.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KSETTINGSREGISTER_H
#define UI_KSETTINGSREGISTER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "klineedit.h"
#include "transactionsortoption.h"

QT_BEGIN_NAMESPACE

class Ui_KSettingsRegister
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *tab;
    QWidget *tab1;
    QVBoxLayout *vboxLayout1;
    QCheckBox *kcfg_ShowGrid;
    QCheckBox *kcfg_TransactionForm;
    QCheckBox *kcfg_ShowPlannedScheduleDates;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *kcfg_ShowRegisterDetailed;
    QCheckBox *kcfg_LedgerLens;
    QCheckBox *kcfg_ShowAllSplits;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *kcfg_ShowFancyMarker;
    QCheckBox *kcfg_ShowFiscalMarker;
    QComboBox *kcfg_ShowReconciliationMarker;
    QSpacerItem *spacer6;
    QWidget *sorting;
    QVBoxLayout *vboxLayout2;
    QTabWidget *tabWidget3;
    QWidget *tab3;
    QHBoxLayout *hboxLayout;
    KLineEdit *kcfg_sortNormalView;
    TransactionSortOption *m_sortNormalView;
    QWidget *tab4;
    QHBoxLayout *hboxLayout1;
    KLineEdit *kcfg_sortReconcileView;
    TransactionSortOption *m_sortReconcileView;
    QWidget *TabPage;
    QHBoxLayout *hboxLayout2;
    KLineEdit *kcfg_sortSearchView;
    TransactionSortOption *m_sortSearchView;
    QLabel *qLabel2;
    QWidget *TabPage1;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *kcfg_AutoIncCheckNumber;
    QCheckBox *kcfg_FocusChangeIsEnter;
    QCheckBox *kcfg_EnterMovesBetweenFields;
    QCheckBox *kcfg_stringMatchFromStart;
    QCheckBox *kcfg_AutoReconciliation;
    QGridLayout *gridLayout;
    QLabel *textLabel2_1;
    QComboBox *kcfg_initialDateFieldCursorPosition;
    QSpacerItem *spacer16;
    QLabel *textLabel2_2;
    KComboBox *kcfg_defaultReconciliationState;
    QSpacerItem *spacer15;
    QGroupBox *kcfg_AutoFillTransaction;
    QVBoxLayout *verticalLayout;
    QRadioButton *radioButton1;
    QRadioButton *radioButton2;
    QRadioButton *radioButton3;
    QWidget *percentageWidget;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QSpinBox *kcfg_AutoFillDifference;
    QSpacerItem *horizontalSpacer;
    QCheckBox *kcfg_AutoFillUseMemos;
    QSpacerItem *spacer4;
    QWidget *TabPage2;
    QVBoxLayout *vboxLayout3;
    QHBoxLayout *hboxLayout3;
    QLabel *textLabel1_2;
    QSpinBox *kcfg_matchInterval;
    QSpacerItem *spacer13;
    QCheckBox *kcfg_askForPayeeCategory;
    QSpacerItem *spacer14;

    void setupUi(QWidget *KSettingsRegister)
    {
        if (KSettingsRegister->objectName().isEmpty())
            KSettingsRegister->setObjectName("KSettingsRegister");
        KSettingsRegister->resize(632, 512);
        vboxLayout = new QVBoxLayout(KSettingsRegister);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        tab = new QTabWidget(KSettingsRegister);
        tab->setObjectName("tab");
        tab1 = new QWidget();
        tab1->setObjectName("tab1");
        vboxLayout1 = new QVBoxLayout(tab1);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        kcfg_ShowGrid = new QCheckBox(tab1);
        kcfg_ShowGrid->setObjectName("kcfg_ShowGrid");

        vboxLayout1->addWidget(kcfg_ShowGrid);

        kcfg_TransactionForm = new QCheckBox(tab1);
        kcfg_TransactionForm->setObjectName("kcfg_TransactionForm");

        vboxLayout1->addWidget(kcfg_TransactionForm);

        kcfg_ShowPlannedScheduleDates = new QCheckBox(tab1);
        kcfg_ShowPlannedScheduleDates->setObjectName("kcfg_ShowPlannedScheduleDates");

        vboxLayout1->addWidget(kcfg_ShowPlannedScheduleDates);

        groupBox_2 = new QGroupBox(tab1);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_4 = new QVBoxLayout(groupBox_2);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName("verticalLayout_4");
        kcfg_ShowRegisterDetailed = new QCheckBox(groupBox_2);
        kcfg_ShowRegisterDetailed->setObjectName("kcfg_ShowRegisterDetailed");

        verticalLayout_4->addWidget(kcfg_ShowRegisterDetailed);

        kcfg_LedgerLens = new QCheckBox(groupBox_2);
        kcfg_LedgerLens->setObjectName("kcfg_LedgerLens");

        verticalLayout_4->addWidget(kcfg_LedgerLens);

        kcfg_ShowAllSplits = new QCheckBox(groupBox_2);
        kcfg_ShowAllSplits->setObjectName("kcfg_ShowAllSplits");
        kcfg_ShowAllSplits->setEnabled(false);

        verticalLayout_4->addWidget(kcfg_ShowAllSplits);


        vboxLayout1->addWidget(groupBox_2);

        groupBox = new QGroupBox(tab1);
        groupBox->setObjectName("groupBox");
        verticalLayout_3 = new QVBoxLayout(groupBox);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        kcfg_ShowFancyMarker = new QCheckBox(groupBox);
        kcfg_ShowFancyMarker->setObjectName("kcfg_ShowFancyMarker");

        verticalLayout_3->addWidget(kcfg_ShowFancyMarker);

        kcfg_ShowFiscalMarker = new QCheckBox(groupBox);
        kcfg_ShowFiscalMarker->setObjectName("kcfg_ShowFiscalMarker");

        verticalLayout_3->addWidget(kcfg_ShowFiscalMarker);

        kcfg_ShowReconciliationMarker = new QComboBox(groupBox);
        kcfg_ShowReconciliationMarker->addItem(QString());
        kcfg_ShowReconciliationMarker->addItem(QString());
        kcfg_ShowReconciliationMarker->addItem(QString());
        kcfg_ShowReconciliationMarker->setObjectName("kcfg_ShowReconciliationMarker");

        verticalLayout_3->addWidget(kcfg_ShowReconciliationMarker);


        vboxLayout1->addWidget(groupBox);

        spacer6 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacer6);

        tab->addTab(tab1, QString());
        sorting = new QWidget();
        sorting->setObjectName("sorting");
        vboxLayout2 = new QVBoxLayout(sorting);
        vboxLayout2->setSpacing(6);
        vboxLayout2->setContentsMargins(11, 11, 11, 11);
        vboxLayout2->setObjectName("vboxLayout2");
        tabWidget3 = new QTabWidget(sorting);
        tabWidget3->setObjectName("tabWidget3");
        tab3 = new QWidget();
        tab3->setObjectName("tab3");
        hboxLayout = new QHBoxLayout(tab3);
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName("hboxLayout");
        kcfg_sortNormalView = new KLineEdit(tab3);
        kcfg_sortNormalView->setObjectName("kcfg_sortNormalView");

        hboxLayout->addWidget(kcfg_sortNormalView);

        m_sortNormalView = new TransactionSortOption(tab3);
        m_sortNormalView->setObjectName("m_sortNormalView");

        hboxLayout->addWidget(m_sortNormalView);

        tabWidget3->addTab(tab3, QString());
        tab4 = new QWidget();
        tab4->setObjectName("tab4");
        hboxLayout1 = new QHBoxLayout(tab4);
        hboxLayout1->setSpacing(6);
        hboxLayout1->setContentsMargins(11, 11, 11, 11);
        hboxLayout1->setObjectName("hboxLayout1");
        kcfg_sortReconcileView = new KLineEdit(tab4);
        kcfg_sortReconcileView->setObjectName("kcfg_sortReconcileView");

        hboxLayout1->addWidget(kcfg_sortReconcileView);

        m_sortReconcileView = new TransactionSortOption(tab4);
        m_sortReconcileView->setObjectName("m_sortReconcileView");

        hboxLayout1->addWidget(m_sortReconcileView);

        tabWidget3->addTab(tab4, QString());
        TabPage = new QWidget();
        TabPage->setObjectName("TabPage");
        hboxLayout2 = new QHBoxLayout(TabPage);
        hboxLayout2->setSpacing(6);
        hboxLayout2->setContentsMargins(11, 11, 11, 11);
        hboxLayout2->setObjectName("hboxLayout2");
        kcfg_sortSearchView = new KLineEdit(TabPage);
        kcfg_sortSearchView->setObjectName("kcfg_sortSearchView");

        hboxLayout2->addWidget(kcfg_sortSearchView);

        m_sortSearchView = new TransactionSortOption(TabPage);
        m_sortSearchView->setObjectName("m_sortSearchView");

        hboxLayout2->addWidget(m_sortSearchView);

        tabWidget3->addTab(TabPage, QString());

        vboxLayout2->addWidget(tabWidget3);

        qLabel2 = new QLabel(sorting);
        qLabel2->setObjectName("qLabel2");
        qLabel2->setWordWrap(true);

        vboxLayout2->addWidget(qLabel2);

        tab->addTab(sorting, QString());
        TabPage1 = new QWidget();
        TabPage1->setObjectName("TabPage1");
        verticalLayout_2 = new QVBoxLayout(TabPage1);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        kcfg_AutoIncCheckNumber = new QCheckBox(TabPage1);
        kcfg_AutoIncCheckNumber->setObjectName("kcfg_AutoIncCheckNumber");

        verticalLayout_2->addWidget(kcfg_AutoIncCheckNumber);

        kcfg_FocusChangeIsEnter = new QCheckBox(TabPage1);
        kcfg_FocusChangeIsEnter->setObjectName("kcfg_FocusChangeIsEnter");

        verticalLayout_2->addWidget(kcfg_FocusChangeIsEnter);

        kcfg_EnterMovesBetweenFields = new QCheckBox(TabPage1);
        kcfg_EnterMovesBetweenFields->setObjectName("kcfg_EnterMovesBetweenFields");

        verticalLayout_2->addWidget(kcfg_EnterMovesBetweenFields);

        kcfg_stringMatchFromStart = new QCheckBox(TabPage1);
        kcfg_stringMatchFromStart->setObjectName("kcfg_stringMatchFromStart");

        verticalLayout_2->addWidget(kcfg_stringMatchFromStart);

        kcfg_AutoReconciliation = new QCheckBox(TabPage1);
        kcfg_AutoReconciliation->setObjectName("kcfg_AutoReconciliation");

        verticalLayout_2->addWidget(kcfg_AutoReconciliation);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        textLabel2_1 = new QLabel(TabPage1);
        textLabel2_1->setObjectName("textLabel2_1");

        gridLayout->addWidget(textLabel2_1, 0, 0, 1, 1);

        kcfg_initialDateFieldCursorPosition = new QComboBox(TabPage1);
        kcfg_initialDateFieldCursorPosition->addItem(QString());
        kcfg_initialDateFieldCursorPosition->addItem(QString());
        kcfg_initialDateFieldCursorPosition->addItem(QString());
        kcfg_initialDateFieldCursorPosition->setObjectName("kcfg_initialDateFieldCursorPosition");

        gridLayout->addWidget(kcfg_initialDateFieldCursorPosition, 0, 1, 1, 1);

        spacer16 = new QSpacerItem(31, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacer16, 0, 2, 1, 1);

        textLabel2_2 = new QLabel(TabPage1);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setWordWrap(false);

        gridLayout->addWidget(textLabel2_2, 1, 0, 1, 1);

        kcfg_defaultReconciliationState = new KComboBox(TabPage1);
        kcfg_defaultReconciliationState->addItem(QString());
        kcfg_defaultReconciliationState->addItem(QString());
        kcfg_defaultReconciliationState->addItem(QString());
        kcfg_defaultReconciliationState->setObjectName("kcfg_defaultReconciliationState");

        gridLayout->addWidget(kcfg_defaultReconciliationState, 1, 1, 1, 1);

        spacer15 = new QSpacerItem(31, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacer15, 1, 2, 1, 1);


        verticalLayout_2->addLayout(gridLayout);

        kcfg_AutoFillTransaction = new QGroupBox(TabPage1);
        kcfg_AutoFillTransaction->setObjectName("kcfg_AutoFillTransaction");
        kcfg_AutoFillTransaction->setProperty("selectedId", QVariant(-1));
        verticalLayout = new QVBoxLayout(kcfg_AutoFillTransaction);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        radioButton1 = new QRadioButton(kcfg_AutoFillTransaction);
        radioButton1->setObjectName("radioButton1");

        verticalLayout->addWidget(radioButton1);

        radioButton2 = new QRadioButton(kcfg_AutoFillTransaction);
        radioButton2->setObjectName("radioButton2");
        radioButton2->setChecked(true);

        verticalLayout->addWidget(radioButton2);

        radioButton3 = new QRadioButton(kcfg_AutoFillTransaction);
        radioButton3->setObjectName("radioButton3");

        verticalLayout->addWidget(radioButton3);


        verticalLayout_2->addWidget(kcfg_AutoFillTransaction);

        percentageWidget = new QWidget(TabPage1);
        percentageWidget->setObjectName("percentageWidget");
        horizontalLayout = new QHBoxLayout(percentageWidget);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setContentsMargins(11, 11, 11, 11);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, -1, 0, -1);
        label = new QLabel(percentageWidget);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        kcfg_AutoFillDifference = new QSpinBox(percentageWidget);
        kcfg_AutoFillDifference->setObjectName("kcfg_AutoFillDifference");
        kcfg_AutoFillDifference->setMaximum(100);
        kcfg_AutoFillDifference->setValue(10);

        horizontalLayout->addWidget(kcfg_AutoFillDifference);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_2->addWidget(percentageWidget);

        kcfg_AutoFillUseMemos = new QCheckBox(TabPage1);
        kcfg_AutoFillUseMemos->setObjectName("kcfg_AutoFillUseMemos");

        verticalLayout_2->addWidget(kcfg_AutoFillUseMemos);

        spacer4 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(spacer4);

        tab->addTab(TabPage1, QString());
        TabPage2 = new QWidget();
        TabPage2->setObjectName("TabPage2");
        vboxLayout3 = new QVBoxLayout(TabPage2);
        vboxLayout3->setSpacing(6);
        vboxLayout3->setContentsMargins(11, 11, 11, 11);
        vboxLayout3->setObjectName("vboxLayout3");
        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setSpacing(6);
        hboxLayout3->setObjectName("hboxLayout3");
        textLabel1_2 = new QLabel(TabPage2);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        hboxLayout3->addWidget(textLabel1_2);

        kcfg_matchInterval = new QSpinBox(TabPage2);
        kcfg_matchInterval->setObjectName("kcfg_matchInterval");
        kcfg_matchInterval->setButtonSymbols(QAbstractSpinBox::UpDownArrows);
        kcfg_matchInterval->setMinimum(0);
        kcfg_matchInterval->setMaximum(99);
        kcfg_matchInterval->setValue(4);

        hboxLayout3->addWidget(kcfg_matchInterval);

        spacer13 = new QSpacerItem(61, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout3->addItem(spacer13);


        vboxLayout3->addLayout(hboxLayout3);

        kcfg_askForPayeeCategory = new QCheckBox(TabPage2);
        kcfg_askForPayeeCategory->setObjectName("kcfg_askForPayeeCategory");

        vboxLayout3->addWidget(kcfg_askForPayeeCategory);

        spacer14 = new QSpacerItem(20, 60, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout3->addItem(spacer14);

        tab->addTab(TabPage2, QString());

        vboxLayout->addWidget(tab);


        retranslateUi(KSettingsRegister);
        QObject::connect(radioButton2, &QRadioButton::toggled, percentageWidget, &QWidget::setEnabled);
        QObject::connect(kcfg_LedgerLens, &QCheckBox::toggled, kcfg_ShowAllSplits, &QCheckBox::setEnabled);

        tab->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KSettingsRegister);
    } // setupUi

    void retranslateUi(QWidget *KSettingsRegister)
    {
        KSettingsRegister->setWindowTitle(tr2i18n("Register settings", nullptr));
        kcfg_ShowGrid->setText(tr2i18n("Show a grid in the ledger", nullptr));
#if QT_CONFIG(shortcut)
        kcfg_ShowGrid->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        kcfg_TransactionForm->setText(tr2i18n("Show transaction form", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_ShowPlannedScheduleDates->setWhatsThis(tr2i18n("If a planned transaction is overdue, today's date is used by default as the posting date. With this option, the originally planned date is used instead.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_ShowPlannedScheduleDates->setText(tr2i18n("Display overdue schedules with planned date", nullptr));
        groupBox_2->setTitle(tr2i18n("Ledger entry details", nullptr));
        kcfg_ShowRegisterDetailed->setText(tr2i18n("Show all ledger entries in full detail", nullptr));
#if QT_CONFIG(shortcut)
        kcfg_ShowRegisterDetailed->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(whatsthis)
        kcfg_LedgerLens->setWhatsThis(tr2i18n("Using the ledger lens shows the details for the transaction that has focus in the ledger. Usually, when using the transaction form, only a one line summary is displayed for each transaction as the details are shown in the form.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_LedgerLens->setText(tr2i18n("Use the ledger lens", nullptr));
#if QT_CONFIG(shortcut)
        kcfg_LedgerLens->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        kcfg_ShowAllSplits->setText(tr2i18n("Show all split details in ledger", nullptr));
        groupBox->setTitle(tr2i18n("Information headers", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_ShowFancyMarker->setWhatsThis(tr2i18n("Draws a larger header above each group of transactions. The grouping depends on the current sort order.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_ShowFancyMarker->setText(tr2i18n("Show group headers between transactions", nullptr));
#if QT_CONFIG(shortcut)
        kcfg_ShowFancyMarker->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        kcfg_ShowFiscalMarker->setText(tr2i18n("Show header for the previous and current fiscal year", nullptr));
        kcfg_ShowReconciliationMarker->setItemText(0, tr2i18n("Don't show reconciliation entries", nullptr));
        kcfg_ShowReconciliationMarker->setItemText(1, tr2i18n("Show last reconciliation entry with date and balance in ledger", nullptr));
        kcfg_ShowReconciliationMarker->setItemText(2, tr2i18n("Show all reconciliation entries with date and balance in ledger", nullptr));

        tab->setTabText(tab->indexOf(tab1), tr2i18n("Display", "@title The way in which items will be displayed"));
        tabWidget3->setTabText(tabWidget3->indexOf(tab3), tr2i18n("Normal view", nullptr));
        tabWidget3->setTabText(tabWidget3->indexOf(tab4), tr2i18n("Reconciliation view", nullptr));
        tabWidget3->setTabText(tabWidget3->indexOf(TabPage), tr2i18n("Search view", nullptr));
        qLabel2->setText(tr2i18n("Use the <i>left</i> and <i>right</i> buttons to add and remove sort options. Use the <i>up</i> and <i>down</i> buttons to modify the sort order. Double-Click a selected entry to toggle the sort order between <i>ascending</i> and <i>descending</i>.", nullptr));
        tab->setTabText(tab->indexOf(sorting), tr2i18n("Sorting", nullptr));
        kcfg_AutoIncCheckNumber->setText(tr2i18n("Auto increment check number", nullptr));
#if QT_CONFIG(shortcut)
        kcfg_AutoIncCheckNumber->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        kcfg_FocusChangeIsEnter->setText(tr2i18n("Keep changes when selecting a different transaction/split", nullptr));
        kcfg_EnterMovesBetweenFields->setText(tr2i18n("Use Enter to move between fields", nullptr));
#if QT_CONFIG(shortcut)
        kcfg_EnterMovesBetweenFields->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(whatsthis)
        kcfg_stringMatchFromStart->setWhatsThis(tr2i18n("Mark this option, if you always want to match names e.g. for payees from the start. If unset, any substring is matched.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_stringMatchFromStart->setText(tr2i18n("Match names from start", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_AutoReconciliation->setWhatsThis(tr2i18n("After entering the reconciliation data automatically detect the transactions that match that data (in some cases it might be not possible to do that).", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_AutoReconciliation->setText(tr2i18n("Automatic reconciliation", nullptr));
#if QT_CONFIG(shortcut)
        kcfg_AutoReconciliation->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        textLabel2_1->setText(tr2i18n("Initial cursor position within the date field", nullptr));
        kcfg_initialDateFieldCursorPosition->setItemText(0, tr2i18n("Day", nullptr));
        kcfg_initialDateFieldCursorPosition->setItemText(1, tr2i18n("Month", nullptr));
        kcfg_initialDateFieldCursorPosition->setItemText(2, tr2i18n("Year", nullptr));

        textLabel2_2->setText(tr2i18n("Default reconciliation state", nullptr));
        kcfg_defaultReconciliationState->setItemText(0, tr2i18n("Not marked", nullptr));
        kcfg_defaultReconciliationState->setItemText(1, tr2i18n("Cleared", "@item reconciliation status"));
        kcfg_defaultReconciliationState->setItemText(2, tr2i18n("Reconciled", nullptr));

#if QT_CONFIG(tooltip)
        kcfg_defaultReconciliationState->setToolTip(tr2i18n("Default reconciliation state for transactions entered during reconciliation of an account", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_AutoFillTransaction->setTitle(tr2i18n("Autofill", nullptr));
#if QT_CONFIG(whatsthis)
        radioButton1->setWhatsThis(tr2i18n("Do not auto-fill transaction data at all.", nullptr));
#endif // QT_CONFIG(whatsthis)
        radioButton1->setText(tr2i18n("No Autofill", nullptr));
#if QT_CONFIG(whatsthis)
        radioButton2->setWhatsThis(tr2i18n("Collect all transactions for the given payee. Treat all transactions that refer to the same category and have an amount with +/- X % as identical. If more than one transaction is found, a list of them is presented to the user.\n"
"\n"
"Selecting 0% will list all transactions.", nullptr));
#endif // QT_CONFIG(whatsthis)
        radioButton2->setText(tr2i18n("Same transaction if amount differs less than percentage set below", nullptr));
#if QT_CONFIG(shortcut)
        radioButton2->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(whatsthis)
        radioButton3->setWhatsThis(tr2i18n("The data of the last transaction assigned to the category used most often for this payee is autofilled into the transaction editor.", nullptr));
#endif // QT_CONFIG(whatsthis)
        radioButton3->setText(tr2i18n("With previously most often used transaction for the payee", nullptr));
        label->setText(tr2i18n("Same transaction if amount differs less than ", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_AutoFillDifference->setWhatsThis(tr2i18n("Two transactions are usually treated identical for autofill, if they refer the same accounts. They are treated as different transactions though, when their amount varies by more than the percentage given here.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_AutoFillDifference->setSuffix(tr2i18n("%", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_AutoFillUseMemos->setWhatsThis(tr2i18n("If this option is checked the memos from the previous transaction will be used otherwise the memos will not be considered when the transaction is autofilled.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_AutoFillUseMemos->setText(tr2i18n("Use memos from previous transaction", nullptr));
#if QT_CONFIG(shortcut)
        kcfg_AutoFillUseMemos->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        tab->setTabText(tab->indexOf(TabPage1), tr2i18n("Data entry", nullptr));
        textLabel1_2->setText(tr2i18n("Match transactions within days", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_matchInterval->setWhatsThis(tr2i18n("Search for matching transactions within the range of the posting date of the imported transaction +/- the number of given days.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        kcfg_askForPayeeCategory->setWhatsThis(tr2i18n("Whenever a new payee is detected during import of a statement, the user will be asked to assign a default category for this user when this option is selected.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_askForPayeeCategory->setText(tr2i18n("Ask for a new payee's default category", nullptr));
#if QT_CONFIG(shortcut)
        kcfg_askForPayeeCategory->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        tab->setTabText(tab->indexOf(TabPage2), tr2i18n("Import", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KSettingsRegister: public Ui_KSettingsRegister {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KSETTINGSREGISTER_H

