/********************************************************************************
** Form generated from reading UI file 'interestwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INTERESTWIZARDPAGE_H
#define UI_INTERESTWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <amountedit.h>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_InterestWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel14_2;
    QSpacerItem *Spacer67;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel1_4;
    AmountEdit *m_interestRateEdit;
    QFrame *Frame3_2;
    QGridLayout *gridLayout;
    QLabel *TextLabel6_3;
    QLabel *TextLabel8_3;
    QLabel *TextLabel9_2;
    QLabel *TextLabel10_2;
    QLabel *TextLabel11_3;
    QLabel *m_loanAmount2;
    QLabel *m_interestRate2;
    QLabel *m_duration2;
    QLabel *m_payment2;
    QLabel *m_balloon2;

    void setupUi(QWizardPage *InterestWizardPage)
    {
        if (InterestWizardPage->objectName().isEmpty())
            InterestWizardPage->setObjectName("InterestWizardPage");
        vboxLayout = new QVBoxLayout(InterestWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel14_2 = new QLabel(InterestWizardPage);
        TextLabel14_2->setObjectName("TextLabel14_2");
        TextLabel14_2->setAlignment(Qt::AlignTop);
        TextLabel14_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel14_2);

        Spacer67 = new QSpacerItem(0, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer67);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        TextLabel1_4 = new QLabel(InterestWizardPage);
        TextLabel1_4->setObjectName("TextLabel1_4");
        TextLabel1_4->setWordWrap(false);

        hboxLayout->addWidget(TextLabel1_4);

        m_interestRateEdit = new AmountEdit(InterestWizardPage);
        m_interestRateEdit->setObjectName("m_interestRateEdit");

        hboxLayout->addWidget(m_interestRateEdit);


        vboxLayout->addLayout(hboxLayout);

        Frame3_2 = new QFrame(InterestWizardPage);
        Frame3_2->setObjectName("Frame3_2");
        Frame3_2->setFrameShape(QFrame::StyledPanel);
        Frame3_2->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(Frame3_2);
#ifndef UI_Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        TextLabel6_3 = new QLabel(Frame3_2);
        TextLabel6_3->setObjectName("TextLabel6_3");
        TextLabel6_3->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel6_3->setWordWrap(true);

        gridLayout->addWidget(TextLabel6_3, 0, 0, 1, 1);

        TextLabel8_3 = new QLabel(Frame3_2);
        TextLabel8_3->setObjectName("TextLabel8_3");
        TextLabel8_3->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel8_3->setWordWrap(false);

        gridLayout->addWidget(TextLabel8_3, 1, 0, 1, 1);

        TextLabel9_2 = new QLabel(Frame3_2);
        TextLabel9_2->setObjectName("TextLabel9_2");
        TextLabel9_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel9_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel9_2, 2, 0, 1, 1);

        TextLabel10_2 = new QLabel(Frame3_2);
        TextLabel10_2->setObjectName("TextLabel10_2");
        TextLabel10_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel10_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel10_2, 3, 0, 1, 1);

        TextLabel11_3 = new QLabel(Frame3_2);
        TextLabel11_3->setObjectName("TextLabel11_3");
        TextLabel11_3->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel11_3->setWordWrap(false);

        gridLayout->addWidget(TextLabel11_3, 4, 0, 1, 1);

        m_loanAmount2 = new QLabel(Frame3_2);
        m_loanAmount2->setObjectName("m_loanAmount2");
        QFont font;
        font.setBold(true);
        m_loanAmount2->setFont(font);
        m_loanAmount2->setTextFormat(Qt::PlainText);
        m_loanAmount2->setWordWrap(false);

        gridLayout->addWidget(m_loanAmount2, 0, 1, 1, 1);

        m_interestRate2 = new QLabel(Frame3_2);
        m_interestRate2->setObjectName("m_interestRate2");
        m_interestRate2->setFont(font);
        m_interestRate2->setTextFormat(Qt::PlainText);
        m_interestRate2->setWordWrap(false);

        gridLayout->addWidget(m_interestRate2, 1, 1, 1, 1);

        m_duration2 = new QLabel(Frame3_2);
        m_duration2->setObjectName("m_duration2");
        m_duration2->setFont(font);
        m_duration2->setTextFormat(Qt::PlainText);
        m_duration2->setWordWrap(false);

        gridLayout->addWidget(m_duration2, 2, 1, 1, 1);

        m_payment2 = new QLabel(Frame3_2);
        m_payment2->setObjectName("m_payment2");
        m_payment2->setFont(font);
        m_payment2->setTextFormat(Qt::PlainText);
        m_payment2->setWordWrap(false);

        gridLayout->addWidget(m_payment2, 3, 1, 1, 1);

        m_balloon2 = new QLabel(Frame3_2);
        m_balloon2->setObjectName("m_balloon2");
        m_balloon2->setFont(font);
        m_balloon2->setTextFormat(Qt::PlainText);
        m_balloon2->setWordWrap(false);

        gridLayout->addWidget(m_balloon2, 4, 1, 1, 1);


        vboxLayout->addWidget(Frame3_2);


        retranslateUi(InterestWizardPage);

        QMetaObject::connectSlotsByName(InterestWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *InterestWizardPage)
    {
        TextLabel14_2->setText(tr2i18n("\n"
"Please enter the interest rate or leave the field empty to calculate it.", nullptr));
        TextLabel1_4->setText(tr2i18n("Interest rate", nullptr));
        TextLabel6_3->setText(tr2i18n("Loan amount:", nullptr));
        TextLabel8_3->setText(tr2i18n("Interest rate:", nullptr));
        TextLabel9_2->setText(tr2i18n("Term:", nullptr));
        TextLabel10_2->setText(tr2i18n("Principal + Interest:", nullptr));
        TextLabel11_3->setText(tr2i18n("Final amortization payment", nullptr));
        m_loanAmount2->setText(tr2i18n("1", nullptr));
        m_interestRate2->setText(tr2i18n("1", nullptr));
        m_duration2->setText(tr2i18n("1", nullptr));
        m_payment2->setText(tr2i18n("1", nullptr));
        m_balloon2->setText(tr2i18n("1", nullptr));
        (void)InterestWizardPage;
    } // retranslateUi

};

namespace Ui {
    class InterestWizardPage: public Ui_InterestWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // INTERESTWIZARDPAGE_H

