/****************************************************************************
** Meta object code from reading C++ file 'kbudgetview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../kmymoney-5.2.2/kmymoney/plugins/views/budget/kbudgetview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kbudgetview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11KBudgetViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KBudgetView::qt_create_metaobjectdata<qt_meta_tag_ZN11KBudgetViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KBudgetView",
        "slotSettingsChanged",
        "",
        "updateActions",
        "SelectedObjects",
        "selections",
        "slotNewBudget",
        "slotDeleteBudget",
        "slotCopyBudget",
        "slotStartRename",
        "slotBudgetForecast",
        "slotResetBudget",
        "slotUpdateBudget",
        "slotSelectAccount",
        "MyMoneyObject",
        "obj",
        "slotBudgetedAmountChanged",
        "cb_includesSubaccounts_clicked",
        "slotBudgetBalanceChanged",
        "MyMoneyMoney",
        "balance",
        "slotSelectBudget",
        "slotHideUnused",
        "toggled",
        "slotAccountSelectionChanged",
        "slotOpenAccountContextMenu",
        "eMenu::Menu",
        "type",
        "QPoint",
        "p",
        "slotTreatAsIncome",
        "slotTreatAsExpense"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotSettingsChanged'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateActions'
        QtMocHelpers::SlotData<void(const SelectedObjects &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotNewBudget'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDeleteBudget'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCopyBudget'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotStartRename'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotBudgetForecast'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotResetBudget'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUpdateBudget'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSelectAccount'
        QtMocHelpers::SlotData<void(const MyMoneyObject &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotBudgetedAmountChanged'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cb_includesSubaccounts_clicked'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotBudgetBalanceChanged'
        QtMocHelpers::SlotData<void(const MyMoneyMoney &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotSelectBudget'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotHideUnused'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Slot 'slotAccountSelectionChanged'
        QtMocHelpers::SlotData<void(const SelectedObjects &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotOpenAccountContextMenu'
        QtMocHelpers::SlotData<void(eMenu::Menu, const QPoint &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 }, { 0x80000000 | 28, 29 },
        }}),
        // Slot 'slotTreatAsIncome'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTreatAsExpense'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KBudgetView, qt_meta_tag_ZN11KBudgetViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KBudgetView::staticMetaObject = { {
    QMetaObject::SuperData::link<KMyMoneyViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KBudgetViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KBudgetViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11KBudgetViewE_t>.metaTypes,
    nullptr
} };

void KBudgetView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KBudgetView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotSettingsChanged(); break;
        case 1: _t->updateActions((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 2: _t->slotNewBudget(); break;
        case 3: _t->slotDeleteBudget(); break;
        case 4: _t->slotCopyBudget(); break;
        case 5: _t->slotStartRename(); break;
        case 6: _t->slotBudgetForecast(); break;
        case 7: _t->slotResetBudget(); break;
        case 8: _t->slotUpdateBudget(); break;
        case 9: _t->slotSelectAccount((*reinterpret_cast<std::add_pointer_t<MyMoneyObject>>(_a[1]))); break;
        case 10: _t->slotBudgetedAmountChanged(); break;
        case 11: _t->cb_includesSubaccounts_clicked(); break;
        case 12: _t->slotBudgetBalanceChanged((*reinterpret_cast<std::add_pointer_t<MyMoneyMoney>>(_a[1]))); break;
        case 13: _t->slotSelectBudget(); break;
        case 14: _t->slotHideUnused((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->slotAccountSelectionChanged((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 16: _t->slotOpenAccountContextMenu((*reinterpret_cast<std::add_pointer_t<eMenu::Menu>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 17: _t->slotTreatAsIncome(); break;
        case 18: _t->slotTreatAsExpense(); break;
        default: ;
        }
    }
}

const QMetaObject *KBudgetView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KBudgetView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KBudgetViewE_t>.strings))
        return static_cast<void*>(this);
    return KMyMoneyViewBase::qt_metacast(_clname);
}

int KBudgetView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KMyMoneyViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
