/********************************************************************************
** Form generated from reading UI file 'assetaccountwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ASSETACCOUNTWIZARDPAGE_H
#define UI_ASSETACCOUNTWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>
#include <kmymoneyaccountselector.h>
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_AssetAccountWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel10_4_2;
    QSpacerItem *spacer80;
    QCheckBox *m_dontCreatePayoutCheckBox;
    QVBoxLayout *vboxLayout1;
    KMyMoneyAccountSelector *m_assetAccountEdit;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacer244;
    QPushButton *m_createNewAssetButton;
    QHBoxLayout *hboxLayout1;
    QLabel *textLabel2;
    KMyMoneyDateEdit *m_paymentDate;
    QSpacerItem *spacer328;

    void setupUi(QWizardPage *AssetAccountWizardPage)
    {
        if (AssetAccountWizardPage->objectName().isEmpty())
            AssetAccountWizardPage->setObjectName("AssetAccountWizardPage");
        vboxLayout = new QVBoxLayout(AssetAccountWizardPage);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel10_4_2 = new QLabel(AssetAccountWizardPage);
        TextLabel10_4_2->setObjectName("TextLabel10_4_2");
        TextLabel10_4_2->setAlignment(Qt::AlignTop);
        TextLabel10_4_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel10_4_2);

        spacer80 = new QSpacerItem(20, 21, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer80);

        m_dontCreatePayoutCheckBox = new QCheckBox(AssetAccountWizardPage);
        m_dontCreatePayoutCheckBox->setObjectName("m_dontCreatePayoutCheckBox");

        vboxLayout->addWidget(m_dontCreatePayoutCheckBox);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setObjectName("vboxLayout1");
        m_assetAccountEdit = new KMyMoneyAccountSelector(AssetAccountWizardPage);
        m_assetAccountEdit->setObjectName("m_assetAccountEdit");

        vboxLayout1->addWidget(m_assetAccountEdit);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        spacer244 = new QSpacerItem(466, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer244);

        m_createNewAssetButton = new QPushButton(AssetAccountWizardPage);
        m_createNewAssetButton->setObjectName("m_createNewAssetButton");

        hboxLayout->addWidget(m_createNewAssetButton);


        vboxLayout1->addLayout(hboxLayout);


        vboxLayout->addLayout(vboxLayout1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        textLabel2 = new QLabel(AssetAccountWizardPage);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        hboxLayout1->addWidget(textLabel2);

        m_paymentDate = new KMyMoneyDateEdit(AssetAccountWizardPage);
        m_paymentDate->setObjectName("m_paymentDate");

        hboxLayout1->addWidget(m_paymentDate);


        vboxLayout->addLayout(hboxLayout1);

        spacer328 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer328);


        retranslateUi(AssetAccountWizardPage);

        QMetaObject::connectSlotsByName(AssetAccountWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *AssetAccountWizardPage)
    {
        TextLabel10_4_2->setText(tr2i18n("If this loan is for an asset, such as a car or a house, you can create the asset account now. An asset account represents the total value of an asset. The money from this loan will be transferred into the asset account you create or select.\n"
"If this loan is a 'consumer loan' (money to use however you want), you can use a checking account instead.", nullptr));
        m_dontCreatePayoutCheckBox->setText(tr2i18n("Do not create payout transaction", nullptr));
        m_createNewAssetButton->setText(QString());
#if QT_CONFIG(shortcut)
        m_createNewAssetButton->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        m_createNewAssetButton->setProperty("toolTip", QVariant(tr2i18n("Press this to create a new asset account", nullptr)));
#endif // QT_CONFIG(tooltip)
        textLabel2->setText(tr2i18n("Date of payment", nullptr));
        (void)AssetAccountWizardPage;
    } // retranslateUi

};

namespace Ui {
    class AssetAccountWizardPage: public Ui_AssetAccountWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // ASSETACCOUNTWIZARDPAGE_H

