/********************************************************************************
** Form generated from reading UI file 'kloadtemplatedlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KLOADTEMPLATEDLG_H
#define UI_KLOADTEMPLATEDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kaccounttemplateselector.h"

QT_BEGIN_NAMESPACE

class Ui_KLoadTemplateDlg
{
public:
    QVBoxLayout *vboxLayout;
    KAccountTemplateSelector *m_templateSelector;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KLoadTemplateDlg)
    {
        if (KLoadTemplateDlg->objectName().isEmpty())
            KLoadTemplateDlg->setObjectName("KLoadTemplateDlg");
        KLoadTemplateDlg->resize(636, 509);
        KLoadTemplateDlg->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(KLoadTemplateDlg);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        m_templateSelector = new KAccountTemplateSelector(KLoadTemplateDlg);
        m_templateSelector->setObjectName("m_templateSelector");

        vboxLayout->addWidget(m_templateSelector);

        buttonBox = new QDialogButtonBox(KLoadTemplateDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(KLoadTemplateDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KLoadTemplateDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KLoadTemplateDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KLoadTemplateDlg);
    } // setupUi

    void retranslateUi(QDialog *KLoadTemplateDlg)
    {
        KLoadTemplateDlg->setWindowTitle(tr2i18n("Load Account Hierarchy", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KLoadTemplateDlg: public Ui_KLoadTemplateDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KLOADTEMPLATEDLG_H

