/********************************************************************************
** Form generated from reading UI file 'interestcategorywizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INTERESTCATEGORYWIZARDPAGE_H
#define UI_INTERESTCATEGORYWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>
#include <kmymoneyaccountselector.h>

QT_BEGIN_NAMESPACE

class Ui_InterestCategoryWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel20_2;
    QSpacerItem *Spacer90;
    KMyMoneyAccountSelector *m_interestAccountEdit;
    QHBoxLayout *hboxLayout;
    QSpacerItem *Spacer54_2;
    QPushButton *m_createCategoryButton;
    QSpacerItem *Spacer91;

    void setupUi(QWizardPage *InterestCategoryWizardPage)
    {
        if (InterestCategoryWizardPage->objectName().isEmpty())
            InterestCategoryWizardPage->setObjectName("InterestCategoryWizardPage");
        vboxLayout = new QVBoxLayout(InterestCategoryWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel20_2 = new QLabel(InterestCategoryWizardPage);
        TextLabel20_2->setObjectName("TextLabel20_2");
        TextLabel20_2->setAlignment(Qt::AlignTop);
        TextLabel20_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel20_2);

        Spacer90 = new QSpacerItem(0, 38, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer90);

        m_interestAccountEdit = new KMyMoneyAccountSelector(InterestCategoryWizardPage);
        m_interestAccountEdit->setObjectName("m_interestAccountEdit");

        vboxLayout->addWidget(m_interestAccountEdit);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        Spacer54_2 = new QSpacerItem(264, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer54_2);

        m_createCategoryButton = new QPushButton(InterestCategoryWizardPage);
        m_createCategoryButton->setObjectName("m_createCategoryButton");

        hboxLayout->addWidget(m_createCategoryButton);


        vboxLayout->addLayout(hboxLayout);

        Spacer91 = new QSpacerItem(0, 22, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer91);


        retranslateUi(InterestCategoryWizardPage);

        QMetaObject::connectSlotsByName(InterestCategoryWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *InterestCategoryWizardPage)
    {
        TextLabel20_2->setText(tr2i18n("\n"
"Please select the category you want to assign the interest payments to or create a new category.", nullptr));
        m_createCategoryButton->setText(QString());
        (void)InterestCategoryWizardPage;
    } // retranslateUi

};

namespace Ui {
    class InterestCategoryWizardPage: public Ui_InterestCategoryWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // INTERESTCATEGORYWIZARDPAGE_H

