/********************************************************************************
** Form generated from reading UI file 'rowswizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ROWSWIZARDPAGE_H
#define UI_ROWSWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_RowsPage
{
public:
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *leftSpacer;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QSpacerItem *verticalSpacer_2;
    QFrame *line;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_4;
    QCheckBox *m_headerHasName;
    QSpacerItem *horizontalSpacer_3;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QGridLayout *gridLayout;
    QLabel *lblStartLine;
    QLabel *lblEndLine;
    QSpinBox *m_startLine;
    QSpinBox *m_endLine;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer_low;
    QSpacerItem *rightSpacer;

    void setupUi(QWidget *RowsPage)
    {
        if (RowsPage->objectName().isEmpty())
            RowsPage->setObjectName("RowsPage");
        RowsPage->resize(477, 239);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(RowsPage->sizePolicy().hasHeightForWidth());
        RowsPage->setSizePolicy(sizePolicy);
        horizontalLayout_2 = new QHBoxLayout(RowsPage);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        leftSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(leftSpacer);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(RowsPage);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignCenter);
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        verticalSpacer_2 = new QSpacerItem(371, 3, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);

        line = new QFrame(RowsPage);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        verticalSpacer_3 = new QSpacerItem(371, 3, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_3);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_3->setContentsMargins(-1, -1, -1, 0);
        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_4);

        m_headerHasName = new QCheckBox(RowsPage);
        m_headerHasName->setObjectName("m_headerHasName");

        horizontalLayout_3->addWidget(m_headerHasName);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);


        verticalLayout->addLayout(horizontalLayout_3);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(13, 13, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        lblStartLine = new QLabel(RowsPage);
        lblStartLine->setObjectName("lblStartLine");
        lblStartLine->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblStartLine, 0, 0, 1, 1);

        lblEndLine = new QLabel(RowsPage);
        lblEndLine->setObjectName("lblEndLine");
        lblEndLine->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblEndLine, 1, 0, 1, 1);

        m_startLine = new QSpinBox(RowsPage);
        m_startLine->setObjectName("m_startLine");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_startLine->sizePolicy().hasHeightForWidth());
        m_startLine->setSizePolicy(sizePolicy1);
        m_startLine->setMinimum(1);

        gridLayout->addWidget(m_startLine, 0, 1, 1, 1);

        m_endLine = new QSpinBox(RowsPage);
        m_endLine->setObjectName("m_endLine");
        sizePolicy1.setHeightForWidth(m_endLine->sizePolicy().hasHeightForWidth());
        m_endLine->setSizePolicy(sizePolicy1);
        m_endLine->setMinimum(1);

        gridLayout->addWidget(m_endLine, 1, 1, 1, 1);

        gridLayout->setColumnStretch(0, 10);

        horizontalLayout->addLayout(gridLayout);

        horizontalSpacer_2 = new QSpacerItem(13, 13, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        horizontalLayout->setStretch(0, 20);
        horizontalLayout->setStretch(1, 50);
        horizontalLayout->setStretch(2, 35);

        verticalLayout->addLayout(horizontalLayout);

        verticalSpacer_low = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_low);

        verticalLayout->setStretch(0, 30);
        verticalLayout->setStretch(1, 1);
        verticalLayout->setStretch(2, 1);
        verticalLayout->setStretch(3, 1);
        verticalLayout->setStretch(5, 1);
        verticalLayout->setStretch(6, 2);

        horizontalLayout_2->addLayout(verticalLayout);

        rightSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(rightSpacer);

        horizontalLayout_2->setStretch(0, 1);
        horizontalLayout_2->setStretch(1, 10);
        horizontalLayout_2->setStretch(2, 1);

        retranslateUi(RowsPage);

        QMetaObject::connectSlotsByName(RowsPage);
    } // setupUi

    void retranslateUi(QWidget *RowsPage)
    {
        RowsPage->setWindowTitle(tr2i18n("Rows Wizard Page", nullptr));
        label->setText(tr2i18n("Please set your data range.\n"
"Ensure that statement header and footer is excluded.", nullptr));
#if QT_CONFIG(tooltip)
        m_headerHasName->setToolTip(tr2i18n("Statement header will be scanned for names and numbers of accounts you have already setup.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_headerHasName->setText(tr2i18n("Header contains account name or number", nullptr));
        lblStartLine->setText(tr2i18n("Start line", nullptr));
        lblEndLine->setText(tr2i18n("End line", nullptr));
#if QT_CONFIG(tooltip)
        m_startLine->setToolTip(tr2i18n("Enter the number of the first transaction line.\n"
"Will not accept value greater than the End Line value.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_endLine->setToolTip(tr2i18n("Enter the number of the last transaction line.\n"
"Will not accept value less than Start Line value.", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class RowsPage: public Ui_RowsPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // ROWSWIZARDPAGE_H

