/****************************************************************************
** Meta object code from reading C++ file 'kmymoneybriefschedule.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/widgets/kmymoneybriefschedule.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kmymoneybriefschedule.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21KMyMoneyBriefScheduleE_t {};
} // unnamed namespace

template <> constexpr inline auto KMyMoneyBriefSchedule::qt_create_metaobjectdata<qt_meta_tag_ZN21KMyMoneyBriefScheduleE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KMyMoneyBriefSchedule",
        "enterClicked",
        "",
        "MyMoneySchedule",
        "skipClicked",
        "slotPrevClicked",
        "slotNextClicked",
        "slotEnterClicked",
        "slotSkipClicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'enterClicked'
        QtMocHelpers::SignalData<void(const MyMoneySchedule &, const QDate &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::QDate, 2 },
        }}),
        // Signal 'skipClicked'
        QtMocHelpers::SignalData<void(const MyMoneySchedule &, const QDate &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::QDate, 2 },
        }}),
        // Slot 'slotPrevClicked'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotNextClicked'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEnterClicked'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSkipClicked'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KMyMoneyBriefSchedule, qt_meta_tag_ZN21KMyMoneyBriefScheduleE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KMyMoneyBriefSchedule::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KMyMoneyBriefScheduleE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KMyMoneyBriefScheduleE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21KMyMoneyBriefScheduleE_t>.metaTypes,
    nullptr
} };

void KMyMoneyBriefSchedule::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KMyMoneyBriefSchedule *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->enterClicked((*reinterpret_cast<std::add_pointer_t<MyMoneySchedule>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDate>>(_a[2]))); break;
        case 1: _t->skipClicked((*reinterpret_cast<std::add_pointer_t<MyMoneySchedule>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDate>>(_a[2]))); break;
        case 2: _t->slotPrevClicked(); break;
        case 3: _t->slotNextClicked(); break;
        case 4: _t->slotEnterClicked(); break;
        case 5: _t->slotSkipClicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyBriefSchedule::*)(const MyMoneySchedule & , const QDate & )>(_a, &KMyMoneyBriefSchedule::enterClicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyBriefSchedule::*)(const MyMoneySchedule & , const QDate & )>(_a, &KMyMoneyBriefSchedule::skipClicked, 1))
            return;
    }
}

const QMetaObject *KMyMoneyBriefSchedule::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KMyMoneyBriefSchedule::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KMyMoneyBriefScheduleE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int KMyMoneyBriefSchedule::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void KMyMoneyBriefSchedule::enterClicked(const MyMoneySchedule & _t1, const QDate & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KMyMoneyBriefSchedule::skipClicked(const MyMoneySchedule & _t1, const QDate & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
