/********************************************************************************
** Form generated from reading UI file 'kconfirmmanualenterdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KCONFIRMMANUALENTERDLG_H
#define UI_KCONFIRMMANUALENTERDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "ktextedit.h"

QT_BEGIN_NAMESPACE

class Ui_KConfirmManualEnterDlg
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *m_message;
    KTextEdit *m_details;
    QGroupBox *buttonGroupBox1;
    QVBoxLayout *vboxLayout1;
    QRadioButton *m_discardRadio;
    QRadioButton *m_onceRadio;
    QRadioButton *m_setRadio;
    QDialogButtonBox *buttonBox;
    QButtonGroup *buttonGroup1;

    void setupUi(QDialog *KConfirmManualEnterDlg)
    {
        if (KConfirmManualEnterDlg->objectName().isEmpty())
            KConfirmManualEnterDlg->setObjectName("KConfirmManualEnterDlg");
        KConfirmManualEnterDlg->resize(578, 384);
        KConfirmManualEnterDlg->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(KConfirmManualEnterDlg);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        m_message = new QLabel(KConfirmManualEnterDlg);
        m_message->setObjectName("m_message");
        m_message->setTextFormat(Qt::RichText);
        m_message->setWordWrap(false);

        vboxLayout->addWidget(m_message);

        m_details = new KTextEdit(KConfirmManualEnterDlg);
        m_details->setObjectName("m_details");
        m_details->setReadOnly(true);

        vboxLayout->addWidget(m_details);

        buttonGroupBox1 = new QGroupBox(KConfirmManualEnterDlg);
        buttonGroupBox1->setObjectName("buttonGroupBox1");
        QFont font;
        buttonGroupBox1->setFont(font);
        vboxLayout1 = new QVBoxLayout(buttonGroupBox1);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        m_discardRadio = new QRadioButton(buttonGroupBox1);
        buttonGroup1 = new QButtonGroup(KConfirmManualEnterDlg);
        buttonGroup1->setObjectName("buttonGroup1");
        buttonGroup1->addButton(m_discardRadio);
        m_discardRadio->setObjectName("m_discardRadio");

        vboxLayout1->addWidget(m_discardRadio);

        m_onceRadio = new QRadioButton(buttonGroupBox1);
        buttonGroup1->addButton(m_onceRadio);
        m_onceRadio->setObjectName("m_onceRadio");
        m_onceRadio->setChecked(true);

        vboxLayout1->addWidget(m_onceRadio);

        m_setRadio = new QRadioButton(buttonGroupBox1);
        buttonGroup1->addButton(m_setRadio);
        m_setRadio->setObjectName("m_setRadio");

        vboxLayout1->addWidget(m_setRadio);


        vboxLayout->addWidget(buttonGroupBox1);

        buttonBox = new QDialogButtonBox(KConfirmManualEnterDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(KConfirmManualEnterDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KConfirmManualEnterDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KConfirmManualEnterDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KConfirmManualEnterDlg);
    } // setupUi

    void retranslateUi(QDialog *KConfirmManualEnterDlg)
    {
        KConfirmManualEnterDlg->setWindowTitle(tr2i18n("Confirm Manual Enter", nullptr));
        m_message->setText(tr2i18n("The following changes have been made to the transaction data:", nullptr));
        buttonGroupBox1->setTitle(tr2i18n("Please choose what you wish to do with the above changes", nullptr));
        m_discardRadio->setText(tr2i18n("&Discard the changes and enter the original transaction into the register.", nullptr));
        m_onceRadio->setText(tr2i18n("Enter &these new values this one time, for this occurrence only.", nullptr));
        m_setRadio->setText(tr2i18n("Set all further occurrences &in this schedule to be these values.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KConfirmManualEnterDlg: public Ui_KConfirmManualEnterDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KCONFIRMMANUALENTERDLG_H

