/********************************************************************************
** Form generated from reading UI file 'kcurrencycalculator.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KCURRENCYCALCULATOR_H
#define UI_KCURRENCYCALCULATOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "amountedit.h"
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_KCurrencyCalculator
{
public:
    QVBoxLayout *verticalLayout;
    QVBoxLayout *vboxLayout;
    QFrame *m_amountDateFrame;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel3;
    QLabel *m_fromAmount;
    QSpacerItem *spacer10;
    QLabel *textLabel2;
    QLabel *m_dateText;
    QSpacerItem *spacer4;
    QHBoxLayout *hboxLayout1;
    QGroupBox *groupBox2_2;
    QHBoxLayout *hboxLayout2;
    QLabel *m_fromCurrencyText;
    QLabel *label100;
    QLabel *m_toCurrencyText;
    QGridLayout *gridLayout;
    QGroupBox *buttonGroupBox1;
    QVBoxLayout *vboxLayout2;
    QFrame *frame;
    QVBoxLayout *vboxLayout3;
    QRadioButton *m_amountButton;
    QRadioButton *m_rateButton;
    QFrame *m_dateFrame;
    QHBoxLayout *hboxLayout3;
    QLabel *textLabel1;
    KMyMoneyDateEdit *m_dateEdit;
    AmountEdit *m_toAmount;
    AmountEdit *m_conversionRate;
    QLabel *m_conversionExample;
    QCheckBox *m_updateButton;
    QSpacerItem *spacer3;
    QDialogButtonBox *buttonBox;
    QButtonGroup *buttonGroup1;

    void setupUi(QDialog *KCurrencyCalculator)
    {
        if (KCurrencyCalculator->objectName().isEmpty())
            KCurrencyCalculator->setObjectName("KCurrencyCalculator");
        KCurrencyCalculator->setEnabled(true);
        KCurrencyCalculator->resize(387, 336);
        KCurrencyCalculator->setSizeGripEnabled(true);
        verticalLayout = new QVBoxLayout(KCurrencyCalculator);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        vboxLayout = new QVBoxLayout();
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        m_amountDateFrame = new QFrame(KCurrencyCalculator);
        m_amountDateFrame->setObjectName("m_amountDateFrame");
        m_amountDateFrame->setFrameShape(QFrame::NoFrame);
        m_amountDateFrame->setFrameShadow(QFrame::Plain);
        vboxLayout1 = new QVBoxLayout(m_amountDateFrame);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        textLabel3 = new QLabel(m_amountDateFrame);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        hboxLayout->addWidget(textLabel3);

        m_fromAmount = new QLabel(m_amountDateFrame);
        m_fromAmount->setObjectName("m_fromAmount");
        m_fromAmount->setWordWrap(false);

        hboxLayout->addWidget(m_fromAmount);

        spacer10 = new QSpacerItem(80, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer10);

        textLabel2 = new QLabel(m_amountDateFrame);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        hboxLayout->addWidget(textLabel2);

        m_dateText = new QLabel(m_amountDateFrame);
        m_dateText->setObjectName("m_dateText");
        m_dateText->setWordWrap(false);

        hboxLayout->addWidget(m_dateText);

        spacer4 = new QSpacerItem(145, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer4);


        vboxLayout1->addLayout(hboxLayout);


        vboxLayout->addWidget(m_amountDateFrame);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        groupBox2_2 = new QGroupBox(KCurrencyCalculator);
        groupBox2_2->setObjectName("groupBox2_2");
        hboxLayout2 = new QHBoxLayout(groupBox2_2);
        hboxLayout2->setSpacing(0);
        hboxLayout2->setContentsMargins(11, 11, 11, 11);
        hboxLayout2->setObjectName("hboxLayout2");
        m_fromCurrencyText = new QLabel(groupBox2_2);
        m_fromCurrencyText->setObjectName("m_fromCurrencyText");
        m_fromCurrencyText->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        m_fromCurrencyText->setWordWrap(false);

        hboxLayout2->addWidget(m_fromCurrencyText);

        label100 = new QLabel(groupBox2_2);
        label100->setObjectName("label100");
        label100->setAlignment(Qt::AlignCenter);
        label100->setWordWrap(false);

        hboxLayout2->addWidget(label100);

        m_toCurrencyText = new QLabel(groupBox2_2);
        m_toCurrencyText->setObjectName("m_toCurrencyText");
        m_toCurrencyText->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        m_toCurrencyText->setWordWrap(false);

        hboxLayout2->addWidget(m_toCurrencyText);


        hboxLayout1->addWidget(groupBox2_2);


        vboxLayout->addLayout(hboxLayout1);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        buttonGroupBox1 = new QGroupBox(KCurrencyCalculator);
        buttonGroupBox1->setObjectName("buttonGroupBox1");
        buttonGroupBox1->setFlat(true);
        vboxLayout2 = new QVBoxLayout(buttonGroupBox1);
        vboxLayout2->setSpacing(6);
        vboxLayout2->setContentsMargins(11, 11, 11, 11);
        vboxLayout2->setObjectName("vboxLayout2");
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        frame = new QFrame(buttonGroupBox1);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::NoFrame);
        frame->setFrameShadow(QFrame::Plain);
        vboxLayout3 = new QVBoxLayout(frame);
        vboxLayout3->setSpacing(6);
        vboxLayout3->setContentsMargins(11, 11, 11, 11);
        vboxLayout3->setObjectName("vboxLayout3");
        vboxLayout3->setContentsMargins(0, 0, 0, 0);
        m_amountButton = new QRadioButton(frame);
        buttonGroup1 = new QButtonGroup(KCurrencyCalculator);
        buttonGroup1->setObjectName("buttonGroup1");
        buttonGroup1->addButton(m_amountButton);
        m_amountButton->setObjectName("m_amountButton");

        vboxLayout3->addWidget(m_amountButton);

        m_rateButton = new QRadioButton(frame);
        buttonGroup1->addButton(m_rateButton);
        m_rateButton->setObjectName("m_rateButton");

        vboxLayout3->addWidget(m_rateButton);


        vboxLayout2->addWidget(frame);


        gridLayout->addWidget(buttonGroupBox1, 0, 0, 3, 1);

        m_dateFrame = new QFrame(KCurrencyCalculator);
        m_dateFrame->setObjectName("m_dateFrame");
        m_dateFrame->setFrameShape(QFrame::NoFrame);
        m_dateFrame->setFrameShadow(QFrame::Plain);
        hboxLayout3 = new QHBoxLayout(m_dateFrame);
        hboxLayout3->setSpacing(6);
        hboxLayout3->setContentsMargins(11, 11, 11, 11);
        hboxLayout3->setObjectName("hboxLayout3");
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        textLabel1 = new QLabel(m_dateFrame);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        hboxLayout3->addWidget(textLabel1);

        m_dateEdit = new KMyMoneyDateEdit(m_dateFrame);
        m_dateEdit->setObjectName("m_dateEdit");

        hboxLayout3->addWidget(m_dateEdit);


        gridLayout->addWidget(m_dateFrame, 4, 0, 1, 2);

        m_toAmount = new AmountEdit(KCurrencyCalculator);
        m_toAmount->setObjectName("m_toAmount");

        gridLayout->addWidget(m_toAmount, 0, 1, 1, 1);

        m_conversionRate = new AmountEdit(KCurrencyCalculator);
        m_conversionRate->setObjectName("m_conversionRate");

        gridLayout->addWidget(m_conversionRate, 1, 1, 1, 1);

        m_conversionExample = new QLabel(KCurrencyCalculator);
        m_conversionExample->setObjectName("m_conversionExample");
        m_conversionExample->setWordWrap(false);

        gridLayout->addWidget(m_conversionExample, 3, 1, 1, 1);


        vboxLayout->addLayout(gridLayout);

        m_updateButton = new QCheckBox(KCurrencyCalculator);
        m_updateButton->setObjectName("m_updateButton");
        m_updateButton->setChecked(true);

        vboxLayout->addWidget(m_updateButton);

        spacer3 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer3);

        buttonBox = new QDialogButtonBox(KCurrencyCalculator);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        verticalLayout->addLayout(vboxLayout);

        QWidget::setTabOrder(m_amountButton, m_rateButton);
        QWidget::setTabOrder(m_rateButton, m_toAmount);
        QWidget::setTabOrder(m_toAmount, m_conversionRate);
        QWidget::setTabOrder(m_conversionRate, m_dateEdit);
        QWidget::setTabOrder(m_dateEdit, m_updateButton);

        retranslateUi(KCurrencyCalculator);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KCurrencyCalculator, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KCurrencyCalculator, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KCurrencyCalculator);
    } // setupUi

    void retranslateUi(QDialog *KCurrencyCalculator)
    {
        KCurrencyCalculator->setWindowTitle(tr2i18n("Exchange Rate/Price Editor", nullptr));
        textLabel3->setText(tr2i18n("Amount", nullptr));
        m_fromAmount->setText(tr2i18n("xxx", nullptr));
        textLabel2->setText(tr2i18n("Date", nullptr));
        m_dateText->setText(tr2i18n("xxx", nullptr));
        groupBox2_2->setTitle(tr2i18n("Convert", nullptr));
        m_fromCurrencyText->setText(QString());
        label100->setText(tr2i18n("to", "@label convert from stock to currency"));
        m_toCurrencyText->setText(QString());
        buttonGroupBox1->setTitle(QString());
        m_amountButton->setText(tr2i18n("To a&mount", nullptr));
        m_rateButton->setText(tr2i18n("Exchange &rate / Price", nullptr));
        textLabel1->setText(tr2i18n("Date", nullptr));
        m_conversionExample->setText(tr2i18n("xx\n"
"xx", nullptr));
        m_updateButton->setText(tr2i18n("Update price history", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KCurrencyCalculator: public Ui_KCurrencyCalculator {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KCURRENCYCALCULATOR_H

