/********************************************************************************
** Form generated from reading UI file 'kaccounttemplateselector.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KACCOUNTTEMPLATESELECTOR_H
#define UI_KACCOUNTTEMPLATESELECTOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "ktextedit.h"

QT_BEGIN_NAMESPACE

class Ui_KAccountTemplateSelector
{
public:
    QGridLayout *gridLayout_3;
    QHBoxLayout *horizontalLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    KTextEdit *m_description;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_2;
    QTreeWidget *m_accountList;
    QTreeView *m_groupList;

    void setupUi(QWidget *KAccountTemplateSelector)
    {
        if (KAccountTemplateSelector->objectName().isEmpty())
            KAccountTemplateSelector->setObjectName("KAccountTemplateSelector");
        KAccountTemplateSelector->resize(663, 445);
        gridLayout_3 = new QGridLayout(KAccountTemplateSelector);
        gridLayout_3->setSpacing(6);
        gridLayout_3->setContentsMargins(11, 11, 11, 11);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setVerticalSpacing(0);
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(4);
        horizontalLayout->setObjectName("horizontalLayout");
        groupBox = new QGroupBox(KAccountTemplateSelector);
        groupBox->setObjectName("groupBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setSpacing(0);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        m_description = new KTextEdit(groupBox);
        m_description->setObjectName("m_description");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(3);
        sizePolicy1.setHeightForWidth(m_description->sizePolicy().hasHeightForWidth());
        m_description->setSizePolicy(sizePolicy1);
        m_description->setReadOnly(true);

        verticalLayout->addWidget(m_description);


        horizontalLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(KAccountTemplateSelector);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_2 = new QVBoxLayout(groupBox_2);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        m_accountList = new QTreeWidget(groupBox_2);
        m_accountList->setObjectName("m_accountList");

        verticalLayout_2->addWidget(m_accountList);


        horizontalLayout->addWidget(groupBox_2);

        horizontalLayout->setStretch(1, 5);

        gridLayout_3->addLayout(horizontalLayout, 1, 0, 1, 1);

        m_groupList = new QTreeView(KAccountTemplateSelector);
        m_groupList->setObjectName("m_groupList");
        m_groupList->setMinimumSize(QSize(20, 0));

        gridLayout_3->addWidget(m_groupList, 0, 0, 1, 1);

        gridLayout_3->setRowStretch(0, 1);
        gridLayout_3->setRowStretch(1, 1);

        retranslateUi(KAccountTemplateSelector);

        QMetaObject::connectSlotsByName(KAccountTemplateSelector);
    } // setupUi

    void retranslateUi(QWidget *KAccountTemplateSelector)
    {
        groupBox->setTitle(tr2i18n("Detailed description", nullptr));
        m_description->setHtml(tr2i18n("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>", nullptr));
        groupBox_2->setTitle(tr2i18n("Accounts", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = m_accountList->headerItem();
        ___qtreewidgetitem->setText(0, tr2i18n("Name", "@title header of the account name column"));
        (void)KAccountTemplateSelector;
    } // retranslateUi

};

namespace Ui {
    class KAccountTemplateSelector: public Ui_KAccountTemplateSelector {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KACCOUNTTEMPLATESELECTOR_H

