/****************************************************************************
** Meta object code from reading C++ file 'kaccountsview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/views/kaccountsview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kaccountsview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KAccountsViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KAccountsView::qt_create_metaobjectdata<qt_meta_tag_ZN13KAccountsViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KAccountsView",
        "beginImportingStatements",
        "",
        "endImportingStatements",
        "slotNetWorthChanged",
        "MyMoneyMoney",
        "netWorth",
        "isApproximate",
        "slotSettingsChanged",
        "updateActions",
        "SelectedObjects",
        "selections",
        "setOnlinePlugins",
        "QMap<QString,KMyMoneyPlugin::OnlinePlugin*>*",
        "plugins",
        "slotUnusedIncomeExpenseAccountHidden",
        "slotNewAccount",
        "slotEditAccount",
        "slotDeleteAccount",
        "slotNewCategory",
        "slotNewPayee",
        "nameBase",
        "QString&",
        "id",
        "slotAccountMapOnline",
        "slotAccountUnmapOnline",
        "slotAccountUpdateOnline",
        "slotAccountUpdateOnlineAll"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'beginImportingStatements'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'endImportingStatements'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNetWorthChanged'
        QtMocHelpers::SlotData<void(const MyMoneyMoney &, bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Slot 'slotSettingsChanged'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateActions'
        QtMocHelpers::SlotData<void(const SelectedObjects &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setOnlinePlugins'
        QtMocHelpers::SlotData<void(QMap<QString,KMyMoneyPlugin::OnlinePlugin*> *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'slotUnusedIncomeExpenseAccountHidden'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotNewAccount'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEditAccount'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDeleteAccount'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotNewCategory'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotNewPayee'
        QtMocHelpers::SlotData<void(const QString &, QString &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotAccountMapOnline'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAccountUnmapOnline'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAccountUpdateOnline'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAccountUpdateOnlineAll'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KAccountsView, qt_meta_tag_ZN13KAccountsViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KAccountsView::staticMetaObject = { {
    QMetaObject::SuperData::link<KMyMoneyViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KAccountsViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KAccountsViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KAccountsViewE_t>.metaTypes,
    nullptr
} };

void KAccountsView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KAccountsView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->beginImportingStatements(); break;
        case 1: _t->endImportingStatements(); break;
        case 2: _t->slotNetWorthChanged((*reinterpret_cast<std::add_pointer_t<MyMoneyMoney>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->slotSettingsChanged(); break;
        case 4: _t->updateActions((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 5: _t->setOnlinePlugins((*reinterpret_cast<std::add_pointer_t<QMap<QString,KMyMoneyPlugin::OnlinePlugin*>*>>(_a[1]))); break;
        case 6: _t->slotUnusedIncomeExpenseAccountHidden(); break;
        case 7: _t->slotNewAccount(); break;
        case 8: _t->slotEditAccount(); break;
        case 9: _t->slotDeleteAccount(); break;
        case 10: _t->slotNewCategory(); break;
        case 11: _t->slotNewPayee((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[2]))); break;
        case 12: _t->slotAccountMapOnline(); break;
        case 13: _t->slotAccountUnmapOnline(); break;
        case 14: _t->slotAccountUpdateOnline(); break;
        case 15: _t->slotAccountUpdateOnlineAll(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KAccountsView::*)()>(_a, &KAccountsView::beginImportingStatements, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KAccountsView::*)()>(_a, &KAccountsView::endImportingStatements, 1))
            return;
    }
}

const QMetaObject *KAccountsView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KAccountsView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KAccountsViewE_t>.strings))
        return static_cast<void*>(this);
    return KMyMoneyViewBase::qt_metacast(_clname);
}

int KAccountsView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KMyMoneyViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void KAccountsView::beginImportingStatements()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KAccountsView::endImportingStatements()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
