/********************************************************************************
** Form generated from reading UI file 'simpleledgerview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SIMPLELEDGERVIEW_H
#define UI_SIMPLELEDGERVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SimpleLedgerView
{
public:
    QVBoxLayout *verticalLayout_2;
    QTabWidget *ledgerTab;
    QWidget *newLedgerTab;
    QVBoxLayout *verticalLayout;
    QSpacerItem *verticalSpacer_2;
    QLabel *l1;
    QLabel *l2;
    QLabel *l3;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *SimpleLedgerView)
    {
        if (SimpleLedgerView->objectName().isEmpty())
            SimpleLedgerView->setObjectName("SimpleLedgerView");
        SimpleLedgerView->resize(613, 429);
        verticalLayout_2 = new QVBoxLayout(SimpleLedgerView);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        ledgerTab = new QTabWidget(SimpleLedgerView);
        ledgerTab->setObjectName("ledgerTab");
        ledgerTab->setTabsClosable(true);
        ledgerTab->setMovable(true);
        newLedgerTab = new QWidget();
        newLedgerTab->setObjectName("newLedgerTab");
        verticalLayout = new QVBoxLayout(newLedgerTab);
        verticalLayout->setObjectName("verticalLayout");
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);

        l1 = new QLabel(newLedgerTab);
        l1->setObjectName("l1");
        l1->setText(QString::fromUtf8("L1"));
        l1->setAlignment(Qt::AlignCenter);
        l1->setWordWrap(true);

        verticalLayout->addWidget(l1);

        l2 = new QLabel(newLedgerTab);
        l2->setObjectName("l2");
        l2->setAlignment(Qt::AlignCenter);
        l2->setWordWrap(true);

        verticalLayout->addWidget(l2);

        l3 = new QLabel(newLedgerTab);
        l3->setObjectName("l3");
        l3->setAlignment(Qt::AlignCenter);
        l3->setWordWrap(true);

        verticalLayout->addWidget(l3);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        QIcon icon;
        QString iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        ledgerTab->addTab(newLedgerTab, icon, QString());
        ledgerTab->setTabText(ledgerTab->indexOf(newLedgerTab), QString::fromUtf8(""));

        verticalLayout_2->addWidget(ledgerTab);


        retranslateUi(SimpleLedgerView);

        QMetaObject::connectSlotsByName(SimpleLedgerView);
    } // setupUi

    void retranslateUi(QWidget *SimpleLedgerView)
    {
        l2->setText(tr2i18n("<html><head/><body><p>Once an account or category is selected its transactions will be presented here.</p></body></html>", nullptr));
        l3->setText(tr2i18n("<html><head/><body><p>Accounts marked as favorites will be opened automatically after KMyMoney opened the file or database. Use the + button on the tabbar above to select an account.</p></body></html>", nullptr));
        (void)SimpleLedgerView;
    } // retranslateUi

};

namespace Ui {
    class SimpleLedgerView: public Ui_SimpleLedgerView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SIMPLELEDGERVIEW_H

