/********************************************************************************
** Form generated from reading UI file 'editintrowizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITINTROWIZARDPAGE_H
#define UI_EDITINTROWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_EditIntroWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QSpacerItem *Spacer2_2;
    QLabel *TextLabel1_1_2;
    QSpacerItem *Spacer3_2;
    QLabel *TextLabel1_2_2;
    QSpacerItem *spacer79;

    void setupUi(QWizardPage *EditIntroWizardPage)
    {
        if (EditIntroWizardPage->objectName().isEmpty())
            EditIntroWizardPage->setObjectName("EditIntroWizardPage");
        vboxLayout = new QVBoxLayout(EditIntroWizardPage);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName("hboxLayout");
        Spacer2_2 = new QSpacerItem(98, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer2_2);

        TextLabel1_1_2 = new QLabel(EditIntroWizardPage);
        TextLabel1_1_2->setObjectName("TextLabel1_1_2");
        QFont font;
        font.setFamilies({QString::fromUtf8("Helvetica")});
        font.setPointSize(16);
        font.setBold(true);
        TextLabel1_1_2->setFont(font);
        TextLabel1_1_2->setAlignment(Qt::AlignTop|Qt::AlignHCenter);
        TextLabel1_1_2->setWordWrap(false);

        hboxLayout->addWidget(TextLabel1_1_2);

        Spacer3_2 = new QSpacerItem(119, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer3_2);


        vboxLayout->addLayout(hboxLayout);

        TextLabel1_2_2 = new QLabel(EditIntroWizardPage);
        TextLabel1_2_2->setObjectName("TextLabel1_2_2");
        TextLabel1_2_2->setAlignment(Qt::AlignTop);
        TextLabel1_2_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel1_2_2);

        spacer79 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer79);


        retranslateUi(EditIntroWizardPage);

        QMetaObject::connectSlotsByName(EditIntroWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *EditIntroWizardPage)
    {
        TextLabel1_1_2->setText(tr2i18n("Edit Loan Account Wizard", nullptr));
        TextLabel1_2_2->setText(tr2i18n("\n"
"Welcome to the Edit Loan Account Wizard. Please use this wizard to modify information about your loan account.\n"
"\n"
"Please make sure that you have the relevant information handy. You usually get the information out of your contract and the last statement.", nullptr));
        (void)EditIntroWizardPage;
    } // retranslateUi

};

namespace Ui {
    class EditIntroWizardPage: public Ui_EditIntroWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // EDITINTROWIZARDPAGE_H

