/********************************************************************************
** Form generated from reading UI file 'nationalaccountedit.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NATIONALACCOUNTEDIT_H
#define UI_NATIONALACCOUNTEDIT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_nationalAccountEdit
{
public:
    QVBoxLayout *verticalLayout;
    QLineEdit *accountNumberEdit;
    QLineEdit *institutionCodeEdit;

    void setupUi(QWidget *nationalAccountEdit)
    {
        if (nationalAccountEdit->objectName().isEmpty())
            nationalAccountEdit->setObjectName("nationalAccountEdit");
        nationalAccountEdit->resize(86, 84);
        verticalLayout = new QVBoxLayout(nationalAccountEdit);
        verticalLayout->setObjectName("verticalLayout");
        accountNumberEdit = new QLineEdit(nationalAccountEdit);
        accountNumberEdit->setObjectName("accountNumberEdit");

        verticalLayout->addWidget(accountNumberEdit);

        institutionCodeEdit = new QLineEdit(nationalAccountEdit);
        institutionCodeEdit->setObjectName("institutionCodeEdit");

        verticalLayout->addWidget(institutionCodeEdit);


        retranslateUi(nationalAccountEdit);

        QMetaObject::connectSlotsByName(nationalAccountEdit);
    } // setupUi

    void retranslateUi(QWidget *nationalAccountEdit)
    {
        accountNumberEdit->setPlaceholderText(tr2i18n("account number", nullptr));
        institutionCodeEdit->setPlaceholderText(tr2i18n("institution code", nullptr));
        (void)nationalAccountEdit;
    } // retranslateUi

};

namespace Ui {
    class nationalAccountEdit: public Ui_nationalAccountEdit {};
} // namespace Ui

QT_END_NAMESPACE

#endif // NATIONALACCOUNTEDIT_H

