/********************************************************************************
** Form generated from reading UI file 'tabordereditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TABORDEREDITOR_H
#define UI_TABORDEREDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_TabOrderEditor
{
public:
    QVBoxLayout *verticalLayout;
    QFrame *m_targetWidget;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *TabOrderEditor)
    {
        if (TabOrderEditor->objectName().isEmpty())
            TabOrderEditor->setObjectName("TabOrderEditor");
        TabOrderEditor->resize(489, 298);
        verticalLayout = new QVBoxLayout(TabOrderEditor);
        verticalLayout->setObjectName("verticalLayout");
        m_targetWidget = new QFrame(TabOrderEditor);
        m_targetWidget->setObjectName("m_targetWidget");
        m_targetWidget->setFrameShape(QFrame::StyledPanel);
        m_targetWidget->setFrameShadow(QFrame::Raised);

        verticalLayout->addWidget(m_targetWidget);

        buttonBox = new QDialogButtonBox(TabOrderEditor);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(TabOrderEditor);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, TabOrderEditor, qOverload<>(&QDialog::reject));
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, TabOrderEditor, qOverload<>(&QDialog::accept));

        QMetaObject::connectSlotsByName(TabOrderEditor);
    } // setupUi

    void retranslateUi(QDialog *TabOrderEditor)
    {
        TabOrderEditor->setWindowTitle(tr2i18n("Tab Order Editor", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TabOrderEditor: public Ui_TabOrderEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TABORDEREDITOR_H

