/********************************************************************************
** Form generated from reading UI file 'ksettingscolors.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KSETTINGSCOLORS_H
#define UI_KSETTINGSCOLORS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcolorbutton.h"

QT_BEGIN_NAMESPACE

class Ui_KSettingsColors
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *kcfg_useCustomColors;
    QGridLayout *gridLayout;
    QSpacerItem *horizontalSpacer;
    KColorButton *kcfg_fieldRequiredColor;
    KColorButton *kcfg_transactionMatchedText;
    QLabel *textLabel1_2;
    KColorButton *kcfg_transactionImportedColor;
    KColorButton *kcfg_groupMarkerText;
    KColorButton *kcfg_transactionImportedText;
    QLabel *textLabel1_3;
    QLabel *textLabel1_4;
    KColorButton *kcfg_groupMarkerColor;
    QLabel *textLabel4;
    QLabel *textLabel1_5;
    QLabel *textLabel2_2;
    QLabel *label;
    QLabel *label_2;
    KColorButton *kcfg_transactionMatchedColor;
    KColorButton *kcfg_transactionErroneousColor;
    KColorButton *kcfg_missingConversionRateColor;
    QSpacerItem *spacer2;

    void setupUi(QWidget *KSettingsColors)
    {
        if (KSettingsColors->objectName().isEmpty())
            KSettingsColors->setObjectName("KSettingsColors");
        KSettingsColors->resize(453, 448);
        vboxLayout = new QVBoxLayout(KSettingsColors);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        kcfg_useCustomColors = new QGroupBox(KSettingsColors);
        kcfg_useCustomColors->setObjectName("kcfg_useCustomColors");
        kcfg_useCustomColors->setCheckable(true);
        gridLayout = new QGridLayout(kcfg_useCustomColors);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        horizontalSpacer = new QSpacerItem(114, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 3, 1, 1);

        kcfg_fieldRequiredColor = new KColorButton(kcfg_useCustomColors);
        kcfg_fieldRequiredColor->setObjectName("kcfg_fieldRequiredColor");
        kcfg_fieldRequiredColor->setColor(QColor(255, 255, 221));

        gridLayout->addWidget(kcfg_fieldRequiredColor, 4, 1, 1, 1);

        kcfg_transactionMatchedText = new KColorButton(kcfg_useCustomColors);
        kcfg_transactionMatchedText->setObjectName("kcfg_transactionMatchedText");
        kcfg_transactionMatchedText->setColor(QColor(1, 1, 1));

        gridLayout->addWidget(kcfg_transactionMatchedText, 6, 2, 1, 1);

        textLabel1_2 = new QLabel(kcfg_useCustomColors);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        gridLayout->addWidget(textLabel1_2, 4, 0, 1, 1);

        kcfg_transactionImportedColor = new KColorButton(kcfg_useCustomColors);
        kcfg_transactionImportedColor->setObjectName("kcfg_transactionImportedColor");
        kcfg_transactionImportedColor->setColor(QColor(255, 255, 127));

        gridLayout->addWidget(kcfg_transactionImportedColor, 5, 1, 1, 1);

        kcfg_groupMarkerText = new KColorButton(kcfg_useCustomColors);
        kcfg_groupMarkerText->setObjectName("kcfg_groupMarkerText");
        kcfg_groupMarkerText->setColor(QColor(1, 1, 1));

        gridLayout->addWidget(kcfg_groupMarkerText, 3, 2, 1, 1);

        kcfg_transactionImportedText = new KColorButton(kcfg_useCustomColors);
        kcfg_transactionImportedText->setObjectName("kcfg_transactionImportedText");
        kcfg_transactionImportedText->setColor(QColor(1, 1, 1));

        gridLayout->addWidget(kcfg_transactionImportedText, 5, 2, 1, 1);

        textLabel1_3 = new QLabel(kcfg_useCustomColors);
        textLabel1_3->setObjectName("textLabel1_3");
        textLabel1_3->setWordWrap(false);

        gridLayout->addWidget(textLabel1_3, 1, 0, 1, 1);

        textLabel1_4 = new QLabel(kcfg_useCustomColors);
        textLabel1_4->setObjectName("textLabel1_4");
        textLabel1_4->setWordWrap(false);

        gridLayout->addWidget(textLabel1_4, 5, 0, 1, 1);

        kcfg_groupMarkerColor = new KColorButton(kcfg_useCustomColors);
        kcfg_groupMarkerColor->setObjectName("kcfg_groupMarkerColor");
        kcfg_groupMarkerColor->setColor(QColor(255, 170, 0));

        gridLayout->addWidget(kcfg_groupMarkerColor, 3, 1, 1, 1);

        textLabel4 = new QLabel(kcfg_useCustomColors);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 2, 0, 1, 1);

        textLabel1_5 = new QLabel(kcfg_useCustomColors);
        textLabel1_5->setObjectName("textLabel1_5");
        textLabel1_5->setWordWrap(false);

        gridLayout->addWidget(textLabel1_5, 3, 0, 1, 1);

        textLabel2_2 = new QLabel(kcfg_useCustomColors);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setWordWrap(false);

        gridLayout->addWidget(textLabel2_2, 6, 0, 1, 1);

        label = new QLabel(kcfg_useCustomColors);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 1, 1, 1);

        label_2 = new QLabel(kcfg_useCustomColors);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 0, 2, 1, 1);

        kcfg_transactionMatchedColor = new KColorButton(kcfg_useCustomColors);
        kcfg_transactionMatchedColor->setObjectName("kcfg_transactionMatchedColor");
        kcfg_transactionMatchedColor->setColor(QColor(152, 251, 152));

        gridLayout->addWidget(kcfg_transactionMatchedColor, 6, 1, 1, 1);

        kcfg_transactionErroneousColor = new KColorButton(kcfg_useCustomColors);
        kcfg_transactionErroneousColor->setObjectName("kcfg_transactionErroneousColor");
        kcfg_transactionErroneousColor->setColor(QColor(255, 0, 0));

        gridLayout->addWidget(kcfg_transactionErroneousColor, 1, 2, 1, 1);

        kcfg_missingConversionRateColor = new KColorButton(kcfg_useCustomColors);
        kcfg_missingConversionRateColor->setObjectName("kcfg_missingConversionRateColor");
        kcfg_missingConversionRateColor->setColor(QColor(0, 0, 255));

        gridLayout->addWidget(kcfg_missingConversionRateColor, 2, 2, 1, 1);


        vboxLayout->addWidget(kcfg_useCustomColors);

        spacer2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer2);

        QWidget::setTabOrder(kcfg_useCustomColors, kcfg_transactionErroneousColor);
        QWidget::setTabOrder(kcfg_transactionErroneousColor, kcfg_missingConversionRateColor);
        QWidget::setTabOrder(kcfg_missingConversionRateColor, kcfg_groupMarkerColor);
        QWidget::setTabOrder(kcfg_groupMarkerColor, kcfg_groupMarkerText);
        QWidget::setTabOrder(kcfg_groupMarkerText, kcfg_fieldRequiredColor);
        QWidget::setTabOrder(kcfg_fieldRequiredColor, kcfg_transactionImportedColor);
        QWidget::setTabOrder(kcfg_transactionImportedColor, kcfg_transactionImportedText);
        QWidget::setTabOrder(kcfg_transactionImportedText, kcfg_transactionMatchedColor);
        QWidget::setTabOrder(kcfg_transactionMatchedColor, kcfg_transactionMatchedText);

        retranslateUi(KSettingsColors);

        QMetaObject::connectSlotsByName(KSettingsColors);
    } // setupUi

    void retranslateUi(QWidget *KSettingsColors)
    {
        KSettingsColors->setWindowTitle(tr2i18n("Color settings", nullptr));
        kcfg_useCustomColors->setTitle(tr2i18n("Custo&m colors", nullptr));
        textLabel1_2->setText(tr2i18n("Required field", nullptr));
        kcfg_transactionImportedColor->setText(QString());
        kcfg_transactionImportedText->setText(QString());
        textLabel1_3->setText(tr2i18n("Erroneous transaction", nullptr));
        textLabel1_4->setText(tr2i18n("Imported transaction", nullptr));
        kcfg_groupMarkerColor->setText(QString());
        textLabel4->setText(tr2i18n("Missing conversion rate", nullptr));
        textLabel1_5->setText(tr2i18n("Ledger group separator", nullptr));
        textLabel2_2->setText(tr2i18n("Matched transaction", nullptr));
        label->setText(tr2i18n("Background", nullptr));
        label_2->setText(tr2i18n("Foreground", nullptr));
        kcfg_transactionMatchedColor->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class KSettingsColors: public Ui_KSettingsColors {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KSETTINGSCOLORS_H

