/********************************************************************************
** Form generated from reading UI file 'ksettingsicons.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KSETTINGSICONS_H
#define UI_KSETTINGSICONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KSettingsIcons
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *m_individualIconsFrame;
    QGridLayout *gridLayout;
    QComboBox *m_IconsTheme;
    QSpacerItem *spacer2;
    QLabel *textLabel1;
    QLineEdit *kcfg_IconsTheme;
    QLabel *lblTip;
    QSpacerItem *spacer1;

    void setupUi(QWidget *KSettingsIcons)
    {
        if (KSettingsIcons->objectName().isEmpty())
            KSettingsIcons->setObjectName("KSettingsIcons");
        KSettingsIcons->resize(398, 222);
        vboxLayout = new QVBoxLayout(KSettingsIcons);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        m_individualIconsFrame = new QGroupBox(KSettingsIcons);
        m_individualIconsFrame->setObjectName("m_individualIconsFrame");
        gridLayout = new QGridLayout(m_individualIconsFrame);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        m_IconsTheme = new QComboBox(m_individualIconsFrame);
        m_IconsTheme->setObjectName("m_IconsTheme");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_IconsTheme->sizePolicy().hasHeightForWidth());
        m_IconsTheme->setSizePolicy(sizePolicy);

        gridLayout->addWidget(m_IconsTheme, 0, 3, 1, 1);

        spacer2 = new QSpacerItem(110, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacer2, 0, 1, 1, 1);

        textLabel1 = new QLabel(m_individualIconsFrame);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        kcfg_IconsTheme = new QLineEdit(m_individualIconsFrame);
        kcfg_IconsTheme->setObjectName("kcfg_IconsTheme");
        sizePolicy.setHeightForWidth(kcfg_IconsTheme->sizePolicy().hasHeightForWidth());
        kcfg_IconsTheme->setSizePolicy(sizePolicy);

        gridLayout->addWidget(kcfg_IconsTheme, 0, 2, 1, 1);

        lblTip = new QLabel(m_individualIconsFrame);
        lblTip->setObjectName("lblTip");
        lblTip->setWordWrap(true);

        gridLayout->addWidget(lblTip, 1, 0, 1, 4);


        vboxLayout->addWidget(m_individualIconsFrame);

        spacer1 = new QSpacerItem(20, 30, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer1);


        retranslateUi(KSettingsIcons);

        QMetaObject::connectSlotsByName(KSettingsIcons);
    } // setupUi

    void retranslateUi(QWidget *KSettingsIcons)
    {
        KSettingsIcons->setWindowTitle(tr2i18n("Font settings", nullptr));
        m_individualIconsFrame->setTitle(tr2i18n("Individual icon settings", nullptr));
        textLabel1->setText(tr2i18n("Icon theme", nullptr));
        lblTip->setText(tr2i18n("<html><head/><body><p align=\"center\">Tip: Install Oxygen, Tango or Breeze icon theme to get wider selection.</p></body></html>", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KSettingsIcons: public Ui_KSettingsIcons {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KSETTINGSICONS_H

