/********************************************************************************
** Form generated from reading UI file 'forecastviewsettings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FORECASTVIEWSETTINGS_H
#define UI_FORECASTVIEWSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ForecastViewSettings
{
public:
    QGridLayout *gridLayout;
    QGroupBox *groupBox6;
    QGridLayout *gridLayout1;
    QGridLayout *gridLayout2;
    QGridLayout *gridLayout3;
    QSpinBox *kcfg_forecastAccountCycle;
    QLabel *textLabel2;
    QSpinBox *kcfg_forecastDays;
    QLabel *textLabel1;
    QSpinBox *kcfg_beginForecastDay;
    QLabel *textLabel4;
    QSpacerItem *spacer6;
    QGroupBox *kcfg_forecastMethod;
    QVBoxLayout *vboxLayout;
    QRadioButton *radioButton9;
    QRadioButton *radioButton10;
    QGroupBox *groupBox3;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel3;
    QSpinBox *kcfg_forecastCycles;
    QSpacerItem *spacer7;
    QGroupBox *kcfg_historyMethod;
    QVBoxLayout *vboxLayout2;
    QRadioButton *radioButton11;
    QRadioButton *radioButton12;
    QRadioButton *radioButton13;
    QSpacerItem *spacer10;

    void setupUi(QWidget *ForecastViewSettings)
    {
        if (ForecastViewSettings->objectName().isEmpty())
            ForecastViewSettings->setObjectName("ForecastViewSettings");
        ForecastViewSettings->resize(497, 379);
        gridLayout = new QGridLayout(ForecastViewSettings);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        groupBox6 = new QGroupBox(ForecastViewSettings);
        groupBox6->setObjectName("groupBox6");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(groupBox6->sizePolicy().hasHeightForWidth());
        groupBox6->setSizePolicy(sizePolicy);
        gridLayout1 = new QGridLayout(groupBox6);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName("gridLayout1");
        gridLayout2 = new QGridLayout();
        gridLayout2->setSpacing(6);
        gridLayout2->setObjectName("gridLayout2");
        gridLayout3 = new QGridLayout();
        gridLayout3->setSpacing(6);
        gridLayout3->setObjectName("gridLayout3");
        kcfg_forecastAccountCycle = new QSpinBox(groupBox6);
        kcfg_forecastAccountCycle->setObjectName("kcfg_forecastAccountCycle");

        gridLayout3->addWidget(kcfg_forecastAccountCycle, 1, 1, 1, 1);

        textLabel2 = new QLabel(groupBox6);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout3->addWidget(textLabel2, 1, 0, 1, 1);

        kcfg_forecastDays = new QSpinBox(groupBox6);
        kcfg_forecastDays->setObjectName("kcfg_forecastDays");

        gridLayout3->addWidget(kcfg_forecastDays, 0, 1, 1, 1);

        textLabel1 = new QLabel(groupBox6);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout3->addWidget(textLabel1, 2, 0, 1, 1);

        kcfg_beginForecastDay = new QSpinBox(groupBox6);
        kcfg_beginForecastDay->setObjectName("kcfg_beginForecastDay");

        gridLayout3->addWidget(kcfg_beginForecastDay, 2, 1, 1, 1);

        textLabel4 = new QLabel(groupBox6);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout3->addWidget(textLabel4, 0, 0, 1, 1);


        gridLayout2->addLayout(gridLayout3, 0, 0, 1, 1);

        spacer6 = new QSpacerItem(133, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout2->addItem(spacer6, 0, 1, 1, 1);


        gridLayout1->addLayout(gridLayout2, 0, 0, 1, 1);

        kcfg_forecastMethod = new QGroupBox(groupBox6);
        kcfg_forecastMethod->setObjectName("kcfg_forecastMethod");
        vboxLayout = new QVBoxLayout(kcfg_forecastMethod);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        radioButton9 = new QRadioButton(kcfg_forecastMethod);
        radioButton9->setObjectName("radioButton9");

        vboxLayout->addWidget(radioButton9);

        radioButton10 = new QRadioButton(kcfg_forecastMethod);
        radioButton10->setObjectName("radioButton10");

        vboxLayout->addWidget(radioButton10);


        gridLayout1->addWidget(kcfg_forecastMethod, 1, 0, 1, 1);

        groupBox3 = new QGroupBox(groupBox6);
        groupBox3->setObjectName("groupBox3");
        vboxLayout1 = new QVBoxLayout(groupBox3);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        textLabel3 = new QLabel(groupBox3);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        hboxLayout->addWidget(textLabel3);

        kcfg_forecastCycles = new QSpinBox(groupBox3);
        kcfg_forecastCycles->setObjectName("kcfg_forecastCycles");

        hboxLayout->addWidget(kcfg_forecastCycles);

        spacer7 = new QSpacerItem(130, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer7);


        vboxLayout1->addLayout(hboxLayout);

        kcfg_historyMethod = new QGroupBox(groupBox3);
        kcfg_historyMethod->setObjectName("kcfg_historyMethod");
        vboxLayout2 = new QVBoxLayout(kcfg_historyMethod);
        vboxLayout2->setSpacing(6);
        vboxLayout2->setContentsMargins(11, 11, 11, 11);
        vboxLayout2->setObjectName("vboxLayout2");
        radioButton11 = new QRadioButton(kcfg_historyMethod);
        radioButton11->setObjectName("radioButton11");

        vboxLayout2->addWidget(radioButton11);

        radioButton12 = new QRadioButton(kcfg_historyMethod);
        radioButton12->setObjectName("radioButton12");

        vboxLayout2->addWidget(radioButton12);

        radioButton13 = new QRadioButton(kcfg_historyMethod);
        radioButton13->setObjectName("radioButton13");

        vboxLayout2->addWidget(radioButton13);


        vboxLayout1->addWidget(kcfg_historyMethod);


        gridLayout1->addWidget(groupBox3, 2, 0, 1, 1);


        gridLayout->addWidget(groupBox6, 0, 0, 1, 1);

        spacer10 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacer10, 1, 0, 1, 1);


        retranslateUi(ForecastViewSettings);

        QMetaObject::connectSlotsByName(ForecastViewSettings);
    } // setupUi

    void retranslateUi(QWidget *ForecastViewSettings)
    {
        ForecastViewSettings->setWindowTitle(tr2i18n("Forecast Settings", nullptr));
        groupBox6->setTitle(QString());
        textLabel2->setText(tr2i18n("Number of Days of Account Cycle:", nullptr));
        textLabel1->setText(tr2i18n("Day of Month to start Forecast:", nullptr));
        textLabel4->setText(tr2i18n("Number of Days to Forecast:", nullptr));
        kcfg_forecastMethod->setTitle(tr2i18n("Forecast Method", nullptr));
        radioButton9->setText(tr2i18n("Scheduled and Future Transactions", nullptr));
        radioButton10->setText(tr2i18n("History-based", nullptr));
        groupBox3->setTitle(tr2i18n("History-based settings", nullptr));
        textLabel3->setText(tr2i18n("Number of Cycles to use in Forecast:", nullptr));
        kcfg_historyMethod->setTitle(tr2i18n("History Forecast Method", nullptr));
        radioButton11->setText(tr2i18n("Simple Moving Average", nullptr));
        radioButton12->setText(tr2i18n("Weighted Moving Average", nullptr));
        radioButton13->setText(tr2i18n("Linear Regression", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ForecastViewSettings: public Ui_ForecastViewSettings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // FORECASTVIEWSETTINGS_H

