/********************************************************************************
** Form generated from reading UI file 'mapaccountwizard.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAPACCOUNTWIZARD_H
#define UI_MAPACCOUNTWIZARD_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizard>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_MapAccountWizard
{
public:
    QWizardPage *pageBackends;
    QVBoxLayout *verticalLayout;
    QTreeWidget *backendsList;
    QHBoxLayout *horizontalLayout;
    QPushButton *addBackendButton;
    QWizardPage *pageAccounts;
    QVBoxLayout *verticalLayout_2;
    QTreeWidget *accountsList;

    void setupUi(QWizard *MapAccountWizard)
    {
        if (MapAccountWizard->objectName().isEmpty())
            MapAccountWizard->setObjectName("MapAccountWizard");
        MapAccountWizard->resize(543, 360);
        MapAccountWizard->setWizardStyle(QWizard::ModernStyle);
        pageBackends = new QWizardPage();
        pageBackends->setObjectName("pageBackends");
        verticalLayout = new QVBoxLayout(pageBackends);
        verticalLayout->setObjectName("verticalLayout");
        backendsList = new QTreeWidget(pageBackends);
        backendsList->setObjectName("backendsList");

        verticalLayout->addWidget(backendsList);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        addBackendButton = new QPushButton(pageBackends);
        addBackendButton->setObjectName("addBackendButton");

        horizontalLayout->addWidget(addBackendButton);


        verticalLayout->addLayout(horizontalLayout);

        MapAccountWizard->addPage(pageBackends);
        pageAccounts = new QWizardPage();
        pageAccounts->setObjectName("pageAccounts");
        verticalLayout_2 = new QVBoxLayout(pageAccounts);
        verticalLayout_2->setObjectName("verticalLayout_2");
        accountsList = new QTreeWidget(pageAccounts);
        accountsList->setObjectName("accountsList");

        verticalLayout_2->addWidget(accountsList);

        MapAccountWizard->addPage(pageAccounts);

        retranslateUi(MapAccountWizard);

        QMetaObject::connectSlotsByName(MapAccountWizard);
    } // setupUi

    void retranslateUi(QWizard *MapAccountWizard)
    {
        MapAccountWizard->setWindowTitle(tr2i18n("Account Mapping", nullptr));
        pageBackends->setTitle(tr2i18n("Select Backend", nullptr));
        pageBackends->setSubTitle(tr2i18n("Please select a backend from the list below...", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = backendsList->headerItem();
        ___qtreewidgetitem->setText(1, tr2i18n("Module", nullptr));
        ___qtreewidgetitem->setText(0, tr2i18n("Name", nullptr));
        addBackendButton->setText(tr2i18n("Select a backend", nullptr));
        pageAccounts->setTitle(tr2i18n("Select Account", nullptr));
        pageAccounts->setSubTitle(tr2i18n("Select what <b>online</b> account you want to map to the <b>application</b> account...", nullptr));
        QTreeWidgetItem *___qtreewidgetitem1 = accountsList->headerItem();
        ___qtreewidgetitem1->setText(2, tr2i18n("Balance", nullptr));
        ___qtreewidgetitem1->setText(1, tr2i18n("Name", nullptr));
        ___qtreewidgetitem1->setText(0, tr2i18n("ID", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MapAccountWizard: public Ui_MapAccountWizard {};
} // namespace Ui

QT_END_NAMESPACE

#endif // MAPACCOUNTWIZARD_H

