/********************************************************************************
** Form generated from reading UI file 'ksaveasquestion.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KSAVEASQUESTION_H
#define UI_KSAVEASQUESTION_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KSaveAsQuestion
{
public:
    QVBoxLayout *verticalLayout;
    QTextBrowser *textBrowser;
    QFrame *line;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QSpacerItem *horizontalSpacer;
    QComboBox *fileType;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KSaveAsQuestion)
    {
        if (KSaveAsQuestion->objectName().isEmpty())
            KSaveAsQuestion->setObjectName("KSaveAsQuestion");
        KSaveAsQuestion->resize(537, 211);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(KSaveAsQuestion->sizePolicy().hasHeightForWidth());
        KSaveAsQuestion->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(KSaveAsQuestion);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        textBrowser = new QTextBrowser(KSaveAsQuestion);
        textBrowser->setObjectName("textBrowser");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(1);
        sizePolicy1.setHeightForWidth(textBrowser->sizePolicy().hasHeightForWidth());
        textBrowser->setSizePolicy(sizePolicy1);
        textBrowser->setMinimumSize(QSize(0, 0));
        textBrowser->setStyleSheet(QString::fromUtf8("background-color: rgba(255, 255, 255, 0);"));
        textBrowser->setFrameShape(QFrame::NoFrame);

        verticalLayout->addWidget(textBrowser);

        line = new QFrame(KSaveAsQuestion);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        label_2 = new QLabel(KSaveAsQuestion);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        fileType = new QComboBox(KSaveAsQuestion);
        fileType->setObjectName("fileType");

        horizontalLayout->addWidget(fileType);


        verticalLayout->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(KSaveAsQuestion);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(KSaveAsQuestion);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KSaveAsQuestion, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KSaveAsQuestion, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KSaveAsQuestion);
    } // setupUi

    void retranslateUi(QDialog *KSaveAsQuestion)
    {
        KSaveAsQuestion->setWindowTitle(tr2i18n("Save storage as...", nullptr));
#if QT_CONFIG(tooltip)
        KSaveAsQuestion->setToolTip(tr2i18n("Choose XML if unsure.", nullptr));
#endif // QT_CONFIG(tooltip)
        textBrowser->setHtml(tr2i18n("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"justify\" style=\" margin: 0; -qt-block-indent:0; text-indent:0px;\">KMyMoney supports multiple storage types. The availability of storage types depends on the presence of plugins and their activation in the application settings.</p>\n"
"<p align=\"justify\" style=\" margin: 0; -qt-block-indent:0; text-indent:0px;\"><br />If you are uncertain what to choose here, then <span style=\" font-weight:600;\">XML</span> is probably your best choice.</p></body></html>", nullptr));
        label_2->setText(tr2i18n("Storage type to use for your data", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KSaveAsQuestion: public Ui_KSaveAsQuestion {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KSAVEASQUESTION_H

