/********************************************************************************
** Form generated from reading UI file 'kgeneralloaninfopage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KGENERALLOANINFOPAGE_H
#define UI_KGENERALLOANINFOPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "amountedit.h"
#include "kcombobox.h"
#include "kdatecombobox.h"
#include "kmymoneydateedit.h"
#include "kmymoneyfrequencycombo.h"
#include "kmymoneypayeecombo.h"

QT_BEGIN_NAMESPACE

class Ui_KGeneralLoanInfoPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel1;
    QHBoxLayout *hboxLayout;
    QGridLayout *gridLayout;
    QLabel *textLabel1;
    KComboBox *m_loanDirection;
    QLabel *textLabel2;
    KMyMoneyPayeeCombo *m_payee;
    QLabel *textLabel1_2;
    KComboBox *m_anyPayments;
    QLabel *textLabel3;
    KComboBox *m_recordings;
    QLabel *textLabel1_3;
    AmountEdit *m_openingBalance;
    QLabel *textLabel5;
    KMyMoneyFrequencyCombo *m_paymentFrequency;
    QLabel *textLabel1_4;
    KMyMoneyFrequencyCombo *m_compoundFrequency;
    QLabel *textLabel2_2;
    KMyMoneyDateEdit *m_firstPaymentDate;
    QLabel *textLabel4;
    KComboBox *m_interestType;
    QLabel *textLabel3_2;
    QHBoxLayout *hboxLayout1;
    QSpinBox *m_interestFrequencyAmountEdit;
    KComboBox *m_interestFrequencyUnitEdit;
    QLabel *textLabel4_2;
    KMyMoneyDateEdit *m_interestChangeDateEdit;
    QSpacerItem *spacer1;
    QSpacerItem *spacer2;

    void setupUi(QWidget *KGeneralLoanInfoPage)
    {
        if (KGeneralLoanInfoPage->objectName().isEmpty())
            KGeneralLoanInfoPage->setObjectName("KGeneralLoanInfoPage");
        KGeneralLoanInfoPage->resize(675, 493);
        vboxLayout = new QVBoxLayout(KGeneralLoanInfoPage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        qLabel1 = new QLabel(KGeneralLoanInfoPage);
        qLabel1->setObjectName("qLabel1");
        qLabel1->setFocusPolicy(Qt::NoFocus);
        qLabel1->setWordWrap(true);

        vboxLayout->addWidget(qLabel1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        textLabel1 = new QLabel(KGeneralLoanInfoPage);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        m_loanDirection = new KComboBox(KGeneralLoanInfoPage);
        m_loanDirection->addItem(QString());
        m_loanDirection->addItem(QString());
        m_loanDirection->setObjectName("m_loanDirection");

        gridLayout->addWidget(m_loanDirection, 0, 1, 1, 1);

        textLabel2 = new QLabel(KGeneralLoanInfoPage);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 1, 0, 1, 1);

        m_payee = new KMyMoneyPayeeCombo(KGeneralLoanInfoPage);
        m_payee->setObjectName("m_payee");

        gridLayout->addWidget(m_payee, 1, 1, 1, 1);

        textLabel1_2 = new QLabel(KGeneralLoanInfoPage);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        gridLayout->addWidget(textLabel1_2, 2, 0, 1, 1);

        m_anyPayments = new KComboBox(KGeneralLoanInfoPage);
        m_anyPayments->addItem(QString());
        m_anyPayments->addItem(QString());
        m_anyPayments->setObjectName("m_anyPayments");

        gridLayout->addWidget(m_anyPayments, 2, 1, 1, 1);

        textLabel3 = new QLabel(KGeneralLoanInfoPage);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout->addWidget(textLabel3, 3, 0, 1, 1);

        m_recordings = new KComboBox(KGeneralLoanInfoPage);
        m_recordings->addItem(QString());
        m_recordings->addItem(QString());
        m_recordings->setObjectName("m_recordings");

        gridLayout->addWidget(m_recordings, 3, 1, 1, 1);

        textLabel1_3 = new QLabel(KGeneralLoanInfoPage);
        textLabel1_3->setObjectName("textLabel1_3");
        textLabel1_3->setWordWrap(false);

        gridLayout->addWidget(textLabel1_3, 4, 0, 1, 1);

        m_openingBalance = new AmountEdit(KGeneralLoanInfoPage);
        m_openingBalance->setObjectName("m_openingBalance");

        gridLayout->addWidget(m_openingBalance, 4, 1, 1, 1);

        textLabel5 = new QLabel(KGeneralLoanInfoPage);
        textLabel5->setObjectName("textLabel5");
        textLabel5->setWordWrap(false);

        gridLayout->addWidget(textLabel5, 5, 0, 1, 1);

        m_paymentFrequency = new KMyMoneyFrequencyCombo(KGeneralLoanInfoPage);
        m_paymentFrequency->setObjectName("m_paymentFrequency");
        m_paymentFrequency->setFocusPolicy(Qt::TabFocus);

        gridLayout->addWidget(m_paymentFrequency, 5, 1, 1, 1);

        textLabel1_4 = new QLabel(KGeneralLoanInfoPage);
        textLabel1_4->setObjectName("textLabel1_4");
        textLabel1_4->setWordWrap(false);

        gridLayout->addWidget(textLabel1_4, 6, 0, 1, 1);

        m_compoundFrequency = new KMyMoneyFrequencyCombo(KGeneralLoanInfoPage);
        m_compoundFrequency->setObjectName("m_compoundFrequency");

        gridLayout->addWidget(m_compoundFrequency, 6, 1, 1, 1);

        textLabel2_2 = new QLabel(KGeneralLoanInfoPage);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setWordWrap(false);

        gridLayout->addWidget(textLabel2_2, 7, 0, 1, 1);

        m_firstPaymentDate = new KMyMoneyDateEdit(KGeneralLoanInfoPage);
        m_firstPaymentDate->setObjectName("m_firstPaymentDate");
        m_firstPaymentDate->setFocusPolicy(Qt::StrongFocus);

        gridLayout->addWidget(m_firstPaymentDate, 7, 1, 1, 1);

        textLabel4 = new QLabel(KGeneralLoanInfoPage);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 8, 0, 1, 1);

        m_interestType = new KComboBox(KGeneralLoanInfoPage);
        m_interestType->addItem(QString());
        m_interestType->addItem(QString());
        m_interestType->setObjectName("m_interestType");

        gridLayout->addWidget(m_interestType, 8, 1, 1, 1);

        textLabel3_2 = new QLabel(KGeneralLoanInfoPage);
        textLabel3_2->setObjectName("textLabel3_2");
        textLabel3_2->setWordWrap(false);

        gridLayout->addWidget(textLabel3_2, 9, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        m_interestFrequencyAmountEdit = new QSpinBox(KGeneralLoanInfoPage);
        m_interestFrequencyAmountEdit->setObjectName("m_interestFrequencyAmountEdit");
        m_interestFrequencyAmountEdit->setMinimum(1);
        m_interestFrequencyAmountEdit->setMaximum(999);

        hboxLayout1->addWidget(m_interestFrequencyAmountEdit);

        m_interestFrequencyUnitEdit = new KComboBox(KGeneralLoanInfoPage);
        m_interestFrequencyUnitEdit->addItem(QString());
        m_interestFrequencyUnitEdit->addItem(QString());
        m_interestFrequencyUnitEdit->addItem(QString());
        m_interestFrequencyUnitEdit->addItem(QString());
        m_interestFrequencyUnitEdit->setObjectName("m_interestFrequencyUnitEdit");

        hboxLayout1->addWidget(m_interestFrequencyUnitEdit);


        gridLayout->addLayout(hboxLayout1, 9, 1, 1, 1);

        textLabel4_2 = new QLabel(KGeneralLoanInfoPage);
        textLabel4_2->setObjectName("textLabel4_2");
        textLabel4_2->setWordWrap(false);

        gridLayout->addWidget(textLabel4_2, 10, 0, 1, 1);

        m_interestChangeDateEdit = new KMyMoneyDateEdit(KGeneralLoanInfoPage);
        m_interestChangeDateEdit->setObjectName("m_interestChangeDateEdit");
        m_interestChangeDateEdit->setFocusPolicy(Qt::StrongFocus);

        gridLayout->addWidget(m_interestChangeDateEdit, 10, 1, 1, 1);


        hboxLayout->addLayout(gridLayout);

        spacer1 = new QSpacerItem(29, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer1);


        vboxLayout->addLayout(hboxLayout);

        spacer2 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer2);


        retranslateUi(KGeneralLoanInfoPage);

        m_interestFrequencyUnitEdit->setCurrentIndex(3);


        QMetaObject::connectSlotsByName(KGeneralLoanInfoPage);
    } // setupUi

    void retranslateUi(QWidget *KGeneralLoanInfoPage)
    {
        qLabel1->setText(tr2i18n("Please select the required options. Please use the <b>What's this?</b> feature to see more information about the items.", nullptr));
        textLabel1->setText(tr2i18n("What is the type of the loan?", nullptr));
        m_loanDirection->setItemText(0, tr2i18n("I am borrowing money", nullptr));
        m_loanDirection->setItemText(1, tr2i18n("I am lending money", nullptr));

        textLabel2->setText(tr2i18n("Who is the payee/payer of the loan?", nullptr));
#if QT_CONFIG(whatsthis)
        m_payee->setWhatsThis(tr2i18n("Enter the name of the person or bank you owe money or who owes you money.\n"
"\n"
"If the name does not exist within KMyMoney's database, you are asked if you want to create it.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1_2->setText(tr2i18n("Did you make/receive any payments yet?", nullptr));
        m_anyPayments->setItemText(0, tr2i18n("No", "@item no payments were entered for a loan"));
        m_anyPayments->setItemText(1, tr2i18n("Yes", "@item payments were entered for a loan"));

#if QT_CONFIG(whatsthis)
        m_anyPayments->setWhatsThis(tr2i18n("Were there any payments for this loan whether they are entered into KMyMoney or not?\n"
"\n"
"Note: Payments made to obtain the loan (e.g. Disagio) are not considered as payments in this context.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel3->setText(tr2i18n("Which payments do you want to record?", nullptr));
        m_recordings->setItemText(0, tr2i18n("All payments", nullptr));
        m_recordings->setItemText(1, tr2i18n("Start with this year's payments", nullptr));

#if QT_CONFIG(whatsthis)
        m_recordings->setWhatsThis(tr2i18n("<qt>Select which transactions should be recorded.\n"
"\n"
"<b>All payments</b> allows you to enter all payments made for this loan.\n"
"\n"
"The option to <b>start from the beginning of the current year</b> is meant for loans that have already been active for a longer period of time, and for which you do not want to enter all past transactions.\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        textLabel1_3->setWhatsThis(tr2i18n("Enter the ending balance found on the statement that is the last one before you want to start recording this loan in KMyMoney.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1_3->setText(tr2i18n("Balance before start of recording", nullptr));
#if QT_CONFIG(whatsthis)
        m_openingBalance->setWhatsThis(tr2i18n("You have selected to record only payments from the beginning of this year. Since prior transactions will not be recorded, you need to supply the balance of the loan on January 1st of this year.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel5->setText(tr2i18n("Payment frequency", nullptr));
        textLabel1_4->setText(tr2i18n("Interest Compounding frequency", nullptr));
#if QT_CONFIG(whatsthis)
        m_compoundFrequency->setWhatsThis(tr2i18n("Select the frequency with which the interest is compounded. If uncertain, select the same as the payment frequency. Consult your loan contract for details.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel2_2->setText(tr2i18n("Due date of first payment to be recorded", nullptr));
#if QT_CONFIG(whatsthis)
        m_firstPaymentDate->setWhatsThis(tr2i18n("Depending on the answer to <b>What do you want to record?</b> this field means the following:<p>\n"
"\n"
"<b>All payments</b><br>\n"
"Enter the due date of the very first payment<p>\n"
"\n"
"<b>Start with this year's payments</b><br>\n"
"Enter the due date of the first payment in the current year", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel4->setText(tr2i18n("Type of interest rate", nullptr));
        m_interestType->setItemText(0, tr2i18n("Fixed", "@item type of interest"));
        m_interestType->setItemText(1, tr2i18n("Variable", "@item type of interest"));

        textLabel3_2->setText(tr2i18n("Time between interest changes", nullptr));
        m_interestFrequencyUnitEdit->setItemText(0, tr2i18n("Days", "@item frequency of interest calculation"));
        m_interestFrequencyUnitEdit->setItemText(1, tr2i18n("Weeks", "@item frequency of interest calculation"));
        m_interestFrequencyUnitEdit->setItemText(2, tr2i18n("Months", "@item frequency of interest calculation"));
        m_interestFrequencyUnitEdit->setItemText(3, tr2i18n("Years", "@item frequency of interest calculation"));

        textLabel4_2->setText(tr2i18n("Next interest change is due", nullptr));
        (void)KGeneralLoanInfoPage;
    } // retranslateUi

};

namespace Ui {
    class KGeneralLoanInfoPage: public Ui_KGeneralLoanInfoPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KGENERALLOANINFOPAGE_H

