/********************************************************************************
** Form generated from reading UI file 'khierarchypage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KHIERARCHYPAGE_H
#define UI_KHIERARCHYPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kmymoneyaccounttreeview.h"

QT_BEGIN_NAMESPACE

class Ui_KHierarchyPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel1;
    QLabel *m_subAccountLabel;
    KMyMoneyAccountTreeView *m_parentAccounts;
    QSpacerItem *spacer18;

    void setupUi(QWidget *KHierarchyPage)
    {
        if (KHierarchyPage->objectName().isEmpty())
            KHierarchyPage->setObjectName("KHierarchyPage");
        KHierarchyPage->resize(674, 419);
        vboxLayout = new QVBoxLayout(KHierarchyPage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        qLabel1 = new QLabel(KHierarchyPage);
        qLabel1->setObjectName("qLabel1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(qLabel1->sizePolicy().hasHeightForWidth());
        qLabel1->setSizePolicy(sizePolicy);
        qLabel1->setFocusPolicy(Qt::NoFocus);
        qLabel1->setWordWrap(true);

        vboxLayout->addWidget(qLabel1);

        m_subAccountLabel = new QLabel(KHierarchyPage);
        m_subAccountLabel->setObjectName("m_subAccountLabel");
        m_subAccountLabel->setWordWrap(false);

        vboxLayout->addWidget(m_subAccountLabel);

        m_parentAccounts = new KMyMoneyAccountTreeView(KHierarchyPage);
        m_parentAccounts->setObjectName("m_parentAccounts");

        vboxLayout->addWidget(m_parentAccounts);

        spacer18 = new QSpacerItem(20, 30, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);

        vboxLayout->addItem(spacer18);


        retranslateUi(KHierarchyPage);

        QMetaObject::connectSlotsByName(KHierarchyPage);
    } // setupUi

    void retranslateUi(QWidget *KHierarchyPage)
    {
        qLabel1->setText(tr2i18n("This page allows you to select the parent account.", nullptr));
        m_subAccountLabel->setText(tr2i18n("Subaccount of", nullptr));
        (void)KHierarchyPage;
    } // retranslateUi

};

namespace Ui {
    class KHierarchyPage: public Ui_KHierarchyPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KHIERARCHYPAGE_H

