/****************************************************************************
** Meta object code from reading C++ file 'mymoneyfile.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/mymoney/mymoneyfile.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mymoneyfile.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11MyMoneyFileE_t {};
} // unnamed namespace

template <> constexpr inline auto MyMoneyFile::qt_create_metaobjectdata<qt_meta_tag_ZN11MyMoneyFileE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MyMoneyFile",
        "storageTransactionStarted",
        "",
        "journalBlocking",
        "storageTransactionEnded",
        "beginChangeNotification",
        "endChangeNotification",
        "dataChanged",
        "objectAdded",
        "eMyMoney::File::Object",
        "objType",
        "id",
        "objectRemoved",
        "objectModified",
        "balanceChanged",
        "MyMoneyAccount",
        "acc",
        "valueChanged",
        "modelsLoaded",
        "modelsReadyToUse",
        "finalizeFileOpen",
        "reloadSpecialDates"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'storageTransactionStarted'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'storageTransactionEnded'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'beginChangeNotification'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'endChangeNotification'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dataChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'objectAdded'
        QtMocHelpers::SignalData<void(eMyMoney::File::Object, const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'objectRemoved'
        QtMocHelpers::SignalData<void(eMyMoney::File::Object, const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'objectModified'
        QtMocHelpers::SignalData<void(eMyMoney::File::Object, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'balanceChanged'
        QtMocHelpers::SignalData<void(const MyMoneyAccount &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(const MyMoneyAccount &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'modelsLoaded'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'modelsReadyToUse'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'finalizeFileOpen'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'reloadSpecialDates'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MyMoneyFile, qt_meta_tag_ZN11MyMoneyFileE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MyMoneyFile::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MyMoneyFileE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MyMoneyFileE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11MyMoneyFileE_t>.metaTypes,
    nullptr
} };

void MyMoneyFile::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MyMoneyFile *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->storageTransactionStarted((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->storageTransactionEnded((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->beginChangeNotification(); break;
        case 3: _t->endChangeNotification(); break;
        case 4: _t->dataChanged(); break;
        case 5: _t->objectAdded((*reinterpret_cast<std::add_pointer_t<eMyMoney::File::Object>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->objectRemoved((*reinterpret_cast<std::add_pointer_t<eMyMoney::File::Object>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->objectModified((*reinterpret_cast<std::add_pointer_t<eMyMoney::File::Object>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->balanceChanged((*reinterpret_cast<std::add_pointer_t<MyMoneyAccount>>(_a[1]))); break;
        case 9: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<MyMoneyAccount>>(_a[1]))); break;
        case 10: _t->modelsLoaded(); break;
        case 11: _t->modelsReadyToUse(); break;
        case 12: _t->finalizeFileOpen(); break;
        case 13: _t->reloadSpecialDates(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MyMoneyFile::*)(bool )>(_a, &MyMoneyFile::storageTransactionStarted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MyMoneyFile::*)(bool )>(_a, &MyMoneyFile::storageTransactionEnded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MyMoneyFile::*)()>(_a, &MyMoneyFile::beginChangeNotification, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MyMoneyFile::*)()>(_a, &MyMoneyFile::endChangeNotification, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MyMoneyFile::*)()>(_a, &MyMoneyFile::dataChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MyMoneyFile::*)(eMyMoney::File::Object , const QString & )>(_a, &MyMoneyFile::objectAdded, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MyMoneyFile::*)(eMyMoney::File::Object , const QString & )>(_a, &MyMoneyFile::objectRemoved, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MyMoneyFile::*)(eMyMoney::File::Object , const QString & )>(_a, &MyMoneyFile::objectModified, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (MyMoneyFile::*)(const MyMoneyAccount & )>(_a, &MyMoneyFile::balanceChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (MyMoneyFile::*)(const MyMoneyAccount & )>(_a, &MyMoneyFile::valueChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (MyMoneyFile::*)()>(_a, &MyMoneyFile::modelsLoaded, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (MyMoneyFile::*)()>(_a, &MyMoneyFile::modelsReadyToUse, 11))
            return;
    }
}

const QMetaObject *MyMoneyFile::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MyMoneyFile::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MyMoneyFileE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MyMoneyFile::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void MyMoneyFile::storageTransactionStarted(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MyMoneyFile::storageTransactionEnded(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MyMoneyFile::beginChangeNotification()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MyMoneyFile::endChangeNotification()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MyMoneyFile::dataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MyMoneyFile::objectAdded(eMyMoney::File::Object _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void MyMoneyFile::objectRemoved(eMyMoney::File::Object _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void MyMoneyFile::objectModified(eMyMoney::File::Object _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void MyMoneyFile::balanceChanged(const MyMoneyAccount & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void MyMoneyFile::valueChanged(const MyMoneyAccount & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void MyMoneyFile::modelsLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void MyMoneyFile::modelsReadyToUse()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
