/********************************************************************************
** Form generated from reading UI file 'newcalculateloanwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NEWCALCULATELOANWIZARDPAGE_H
#define UI_NEWCALCULATELOANWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_NewCalculateLoanWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel2_2_2;
    QSpacerItem *Spacer10_2;
    QHBoxLayout *hboxLayout;
    QSpacerItem *Spacer8_2;
    QVBoxLayout *vboxLayout1;
    QLabel *TextLabel3_4;
    QLabel *TextLabel4_3_2;
    QLabel *TextLabel4_3;
    QSpacerItem *Spacer7_2;
    QSpacerItem *Spacer9_2;
    QSpacerItem *Spacer51;

    void setupUi(QWizardPage *NewCalculateLoanWizardPage)
    {
        if (NewCalculateLoanWizardPage->objectName().isEmpty())
            NewCalculateLoanWizardPage->setObjectName("NewCalculateLoanWizardPage");
        vboxLayout = new QVBoxLayout(NewCalculateLoanWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel2_2_2 = new QLabel(NewCalculateLoanWizardPage);
        TextLabel2_2_2->setObjectName("TextLabel2_2_2");
        TextLabel2_2_2->setAlignment(Qt::AlignTop);
        TextLabel2_2_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel2_2_2);

        Spacer10_2 = new QSpacerItem(0, 19, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer10_2);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        Spacer8_2 = new QSpacerItem(136, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer8_2);

        vboxLayout1 = new QVBoxLayout();
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout1->setObjectName("vboxLayout1");
        TextLabel3_4 = new QLabel(NewCalculateLoanWizardPage);
        TextLabel3_4->setObjectName("TextLabel3_4");
        TextLabel3_4->setEnabled(false);
        TextLabel3_4->setAlignment(Qt::AlignTop);
        TextLabel3_4->setWordWrap(false);

        vboxLayout1->addWidget(TextLabel3_4);

        TextLabel4_3_2 = new QLabel(NewCalculateLoanWizardPage);
        TextLabel4_3_2->setObjectName("TextLabel4_3_2");
        TextLabel4_3_2->setWordWrap(false);

        vboxLayout1->addWidget(TextLabel4_3_2);

        TextLabel4_3 = new QLabel(NewCalculateLoanWizardPage);
        TextLabel4_3->setObjectName("TextLabel4_3");
        TextLabel4_3->setEnabled(false);
        TextLabel4_3->setWordWrap(false);

        vboxLayout1->addWidget(TextLabel4_3);

        Spacer7_2 = new QSpacerItem(0, 18, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(Spacer7_2);


        hboxLayout->addLayout(vboxLayout1);

        Spacer9_2 = new QSpacerItem(135, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer9_2);


        vboxLayout->addLayout(hboxLayout);

        Spacer51 = new QSpacerItem(0, 23, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer51);


        retranslateUi(NewCalculateLoanWizardPage);

        QMetaObject::connectSlotsByName(NewCalculateLoanWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *NewCalculateLoanWizardPage)
    {
        TextLabel2_2_2->setText(tr2i18n("\n"
"You have successfully entered the general information about your loan. Next, KMyMoney needs some information about the calculation of the loan.", nullptr));
        TextLabel3_4->setText(tr2i18n("1. General Information", nullptr));
        TextLabel4_3_2->setText(tr2i18n("\n"
"2. Calculate Loan", nullptr));
        TextLabel4_3->setText(tr2i18n("\n"
"3. Payments", nullptr));
        (void)NewCalculateLoanWizardPage;
    } // retranslateUi

};

namespace Ui {
    class NewCalculateLoanWizardPage: public Ui_NewCalculateLoanWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // NEWCALCULATELOANWIZARDPAGE_H

