/********************************************************************************
** Form generated from reading UI file 'mymoneyqifprofileeditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MYMONEYQIFPROFILEEDITOR_H
#define UI_MYMONEYQIFPROFILEEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "klineedit.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_MyMoneyQifProfileEditor
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QListWidget *m_profileListBox;
    QTabWidget *profileTabs;
    QWidget *tabGeneral;
    QVBoxLayout *vboxLayout1;
    QGridLayout *gridLayout;
    QLabel *TextLabel2_2_3;
    QLabel *TextLabel2_2;
    KLineEdit *m_editOpeningBalance;
    QLabel *TextLabel2;
    KLineEdit *m_editAccountDelimiter;
    KLineEdit *m_editVoidMark;
    QLabel *TextLabel2_2_2;
    KLineEdit *m_editType;
    KLineEdit *m_editDescription;
    QLabel *TextLabel1;
    QCheckBox *m_attemptMatch;
    QSpacerItem *Spacer12;
    QWidget *tabFilter;
    QVBoxLayout *vboxLayout2;
    QGridLayout *gridLayout1;
    QLabel *TextLabel1_4_2;
    KUrlRequester *m_editInputFilterLocation;
    KUrlRequester *m_editOutputFilterLocation;
    QLabel *TextLabel1_4_3;
    KLineEdit *m_editInputFilterFileType;
    QLabel *TextLabel1_4;
    QSpacerItem *Spacer9_2;
    QWidget *tabDate;
    QVBoxLayout *vboxLayout3;
    QHBoxLayout *hboxLayout1;
    QGridLayout *gridLayout2;
    KComboBox *m_editDateFormat;
    QLabel *TextLabel3;
    KComboBox *m_editApostrophe;
    QLabel *TextLabel4;
    QSpacerItem *Spacer9;
    QSpacerItem *Spacer8;
    QWidget *tabMoney;
    QVBoxLayout *vboxLayout4;
    QTreeWidget *m_editAmounts;
    QHBoxLayout *hboxLayout2;
    QGridLayout *gridLayout3;
    KComboBox *m_thousandsBox;
    QLabel *TextLabel1_2;
    QLabel *TextLabel1_3;
    KComboBox *m_decimalBox;
    QSpacerItem *Spacer7;
    QSpacerItem *Spacer8_2;
    QHBoxLayout *hboxLayout3;
    QPushButton *m_helpButton;
    QPushButton *m_newButton;
    QPushButton *m_deleteButton;
    QPushButton *m_renameButton;
    QSpacerItem *Horizontal_Spacing2;
    QPushButton *m_resetButton;

    void setupUi(QWidget *MyMoneyQifProfileEditor)
    {
        if (MyMoneyQifProfileEditor->objectName().isEmpty())
            MyMoneyQifProfileEditor->setObjectName("MyMoneyQifProfileEditor");
        MyMoneyQifProfileEditor->resize(736, 334);
        vboxLayout = new QVBoxLayout(MyMoneyQifProfileEditor);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_profileListBox = new QListWidget(MyMoneyQifProfileEditor);
        new QListWidgetItem(m_profileListBox);
        m_profileListBox->setObjectName("m_profileListBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_profileListBox->sizePolicy().hasHeightForWidth());
        m_profileListBox->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(m_profileListBox);

        profileTabs = new QTabWidget(MyMoneyQifProfileEditor);
        profileTabs->setObjectName("profileTabs");
        tabGeneral = new QWidget();
        tabGeneral->setObjectName("tabGeneral");
        vboxLayout1 = new QVBoxLayout(tabGeneral);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        TextLabel2_2_3 = new QLabel(tabGeneral);
        TextLabel2_2_3->setObjectName("TextLabel2_2_3");
        TextLabel2_2_3->setMinimumSize(QSize(140, 0));
        TextLabel2_2_3->setWordWrap(false);

        gridLayout->addWidget(TextLabel2_2_3, 4, 0, 1, 1);

        TextLabel2_2 = new QLabel(tabGeneral);
        TextLabel2_2->setObjectName("TextLabel2_2");
        TextLabel2_2->setMinimumSize(QSize(140, 0));
        TextLabel2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel2_2, 2, 0, 1, 1);

        m_editOpeningBalance = new KLineEdit(tabGeneral);
        m_editOpeningBalance->setObjectName("m_editOpeningBalance");

        gridLayout->addWidget(m_editOpeningBalance, 2, 1, 1, 1);

        TextLabel2 = new QLabel(tabGeneral);
        TextLabel2->setObjectName("TextLabel2");
        TextLabel2->setMinimumSize(QSize(140, 0));
        TextLabel2->setWordWrap(false);

        gridLayout->addWidget(TextLabel2, 1, 0, 1, 1);

        m_editAccountDelimiter = new KLineEdit(tabGeneral);
        m_editAccountDelimiter->setObjectName("m_editAccountDelimiter");

        gridLayout->addWidget(m_editAccountDelimiter, 3, 1, 1, 1);

        m_editVoidMark = new KLineEdit(tabGeneral);
        m_editVoidMark->setObjectName("m_editVoidMark");

        gridLayout->addWidget(m_editVoidMark, 4, 1, 1, 1);

        TextLabel2_2_2 = new QLabel(tabGeneral);
        TextLabel2_2_2->setObjectName("TextLabel2_2_2");
        TextLabel2_2_2->setMinimumSize(QSize(140, 0));
        TextLabel2_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel2_2_2, 3, 0, 1, 1);

        m_editType = new KLineEdit(tabGeneral);
        m_editType->setObjectName("m_editType");

        gridLayout->addWidget(m_editType, 1, 1, 1, 1);

        m_editDescription = new KLineEdit(tabGeneral);
        m_editDescription->setObjectName("m_editDescription");

        gridLayout->addWidget(m_editDescription, 0, 1, 1, 1);

        TextLabel1 = new QLabel(tabGeneral);
        TextLabel1->setObjectName("TextLabel1");
        TextLabel1->setMinimumSize(QSize(140, 0));
        TextLabel1->setWordWrap(false);

        gridLayout->addWidget(TextLabel1, 0, 0, 1, 1);


        vboxLayout1->addLayout(gridLayout);

        m_attemptMatch = new QCheckBox(tabGeneral);
        m_attemptMatch->setObjectName("m_attemptMatch");

        vboxLayout1->addWidget(m_attemptMatch);

        Spacer12 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(Spacer12);

        profileTabs->addTab(tabGeneral, QString());
        tabFilter = new QWidget();
        tabFilter->setObjectName("tabFilter");
        vboxLayout2 = new QVBoxLayout(tabFilter);
        vboxLayout2->setSpacing(6);
        vboxLayout2->setContentsMargins(11, 11, 11, 11);
        vboxLayout2->setObjectName("vboxLayout2");
        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setObjectName("gridLayout1");
        TextLabel1_4_2 = new QLabel(tabFilter);
        TextLabel1_4_2->setObjectName("TextLabel1_4_2");
        TextLabel1_4_2->setMinimumSize(QSize(140, 0));
        TextLabel1_4_2->setWordWrap(false);

        gridLayout1->addWidget(TextLabel1_4_2, 2, 0, 1, 1);

        m_editInputFilterLocation = new KUrlRequester(tabFilter);
        m_editInputFilterLocation->setObjectName("m_editInputFilterLocation");

        gridLayout1->addWidget(m_editInputFilterLocation, 0, 1, 1, 1);

        m_editOutputFilterLocation = new KUrlRequester(tabFilter);
        m_editOutputFilterLocation->setObjectName("m_editOutputFilterLocation");
        m_editOutputFilterLocation->setEnabled(false);

        gridLayout1->addWidget(m_editOutputFilterLocation, 2, 1, 1, 1);

        TextLabel1_4_3 = new QLabel(tabFilter);
        TextLabel1_4_3->setObjectName("TextLabel1_4_3");
        TextLabel1_4_3->setMinimumSize(QSize(140, 0));
        TextLabel1_4_3->setWordWrap(false);

        gridLayout1->addWidget(TextLabel1_4_3, 1, 0, 1, 1);

        m_editInputFilterFileType = new KLineEdit(tabFilter);
        m_editInputFilterFileType->setObjectName("m_editInputFilterFileType");

        gridLayout1->addWidget(m_editInputFilterFileType, 1, 1, 1, 1);

        TextLabel1_4 = new QLabel(tabFilter);
        TextLabel1_4->setObjectName("TextLabel1_4");
        TextLabel1_4->setMinimumSize(QSize(140, 0));
        TextLabel1_4->setWordWrap(false);

        gridLayout1->addWidget(TextLabel1_4, 0, 0, 1, 1);


        vboxLayout2->addLayout(gridLayout1);

        Spacer9_2 = new QSpacerItem(20, 90, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout2->addItem(Spacer9_2);

        profileTabs->addTab(tabFilter, QString());
        tabDate = new QWidget();
        tabDate->setObjectName("tabDate");
        vboxLayout3 = new QVBoxLayout(tabDate);
        vboxLayout3->setSpacing(6);
        vboxLayout3->setContentsMargins(11, 11, 11, 11);
        vboxLayout3->setObjectName("vboxLayout3");
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        gridLayout2 = new QGridLayout();
        gridLayout2->setSpacing(6);
        gridLayout2->setObjectName("gridLayout2");
        m_editDateFormat = new KComboBox(tabDate);
        m_editDateFormat->setObjectName("m_editDateFormat");

        gridLayout2->addWidget(m_editDateFormat, 0, 1, 1, 1);

        TextLabel3 = new QLabel(tabDate);
        TextLabel3->setObjectName("TextLabel3");
        TextLabel3->setMinimumSize(QSize(140, 0));
        TextLabel3->setWordWrap(false);

        gridLayout2->addWidget(TextLabel3, 0, 0, 1, 1);

        m_editApostrophe = new KComboBox(tabDate);
        m_editApostrophe->setObjectName("m_editApostrophe");

        gridLayout2->addWidget(m_editApostrophe, 1, 1, 1, 1);

        TextLabel4 = new QLabel(tabDate);
        TextLabel4->setObjectName("TextLabel4");
        TextLabel4->setMinimumSize(QSize(140, 0));
        TextLabel4->setWordWrap(false);

        gridLayout2->addWidget(TextLabel4, 1, 0, 1, 1);


        hboxLayout1->addLayout(gridLayout2);

        Spacer9 = new QSpacerItem(90, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(Spacer9);


        vboxLayout3->addLayout(hboxLayout1);

        Spacer8 = new QSpacerItem(20, 80, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout3->addItem(Spacer8);

        profileTabs->addTab(tabDate, QString());
        tabMoney = new QWidget();
        tabMoney->setObjectName("tabMoney");
        vboxLayout4 = new QVBoxLayout(tabMoney);
        vboxLayout4->setSpacing(6);
        vboxLayout4->setContentsMargins(11, 11, 11, 11);
        vboxLayout4->setObjectName("vboxLayout4");
        m_editAmounts = new QTreeWidget(tabMoney);
        new QTreeWidgetItem(m_editAmounts);
        new QTreeWidgetItem(m_editAmounts);
        new QTreeWidgetItem(m_editAmounts);
        new QTreeWidgetItem(m_editAmounts);
        new QTreeWidgetItem(m_editAmounts);
        m_editAmounts->setObjectName("m_editAmounts");
        m_editAmounts->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        m_editAmounts->setSelectionMode(QAbstractItemView::SingleSelection);
        m_editAmounts->setAllColumnsShowFocus(true);

        vboxLayout4->addWidget(m_editAmounts);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        gridLayout3 = new QGridLayout();
        gridLayout3->setSpacing(6);
        gridLayout3->setObjectName("gridLayout3");
        m_thousandsBox = new KComboBox(tabMoney);
        m_thousandsBox->setObjectName("m_thousandsBox");

        gridLayout3->addWidget(m_thousandsBox, 1, 1, 1, 1);

        TextLabel1_2 = new QLabel(tabMoney);
        TextLabel1_2->setObjectName("TextLabel1_2");
        TextLabel1_2->setMinimumSize(QSize(150, 0));
        TextLabel1_2->setWordWrap(false);

        gridLayout3->addWidget(TextLabel1_2, 0, 0, 1, 1);

        TextLabel1_3 = new QLabel(tabMoney);
        TextLabel1_3->setObjectName("TextLabel1_3");
        TextLabel1_3->setMinimumSize(QSize(150, 0));
        TextLabel1_3->setWordWrap(false);

        gridLayout3->addWidget(TextLabel1_3, 1, 0, 1, 1);

        m_decimalBox = new KComboBox(tabMoney);
        m_decimalBox->setObjectName("m_decimalBox");

        gridLayout3->addWidget(m_decimalBox, 0, 1, 1, 1);


        hboxLayout2->addLayout(gridLayout3);

        Spacer7 = new QSpacerItem(100, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout2->addItem(Spacer7);


        vboxLayout4->addLayout(hboxLayout2);

        Spacer8_2 = new QSpacerItem(16, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout4->addItem(Spacer8_2);

        profileTabs->addTab(tabMoney, QString());

        hboxLayout->addWidget(profileTabs);


        vboxLayout->addLayout(hboxLayout);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setSpacing(6);
        hboxLayout3->setObjectName("hboxLayout3");
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        m_helpButton = new QPushButton(MyMoneyQifProfileEditor);
        m_helpButton->setObjectName("m_helpButton");

        hboxLayout3->addWidget(m_helpButton);

        m_newButton = new QPushButton(MyMoneyQifProfileEditor);
        m_newButton->setObjectName("m_newButton");

        hboxLayout3->addWidget(m_newButton);

        m_deleteButton = new QPushButton(MyMoneyQifProfileEditor);
        m_deleteButton->setObjectName("m_deleteButton");

        hboxLayout3->addWidget(m_deleteButton);

        m_renameButton = new QPushButton(MyMoneyQifProfileEditor);
        m_renameButton->setObjectName("m_renameButton");

        hboxLayout3->addWidget(m_renameButton);

        Horizontal_Spacing2 = new QSpacerItem(104, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout3->addItem(Horizontal_Spacing2);

        m_resetButton = new QPushButton(MyMoneyQifProfileEditor);
        m_resetButton->setObjectName("m_resetButton");

        hboxLayout3->addWidget(m_resetButton);


        vboxLayout->addLayout(hboxLayout3);

        QWidget::setTabOrder(m_profileListBox, profileTabs);
        QWidget::setTabOrder(profileTabs, m_editDescription);
        QWidget::setTabOrder(m_editDescription, m_editType);
        QWidget::setTabOrder(m_editType, m_editOpeningBalance);
        QWidget::setTabOrder(m_editOpeningBalance, m_editAccountDelimiter);
        QWidget::setTabOrder(m_editAccountDelimiter, m_editVoidMark);
        QWidget::setTabOrder(m_editVoidMark, m_helpButton);
        QWidget::setTabOrder(m_helpButton, m_newButton);
        QWidget::setTabOrder(m_newButton, m_deleteButton);
        QWidget::setTabOrder(m_deleteButton, m_renameButton);
        QWidget::setTabOrder(m_renameButton, m_resetButton);
        QWidget::setTabOrder(m_resetButton, m_editDateFormat);
        QWidget::setTabOrder(m_editDateFormat, m_editApostrophe);
        QWidget::setTabOrder(m_editApostrophe, m_decimalBox);
        QWidget::setTabOrder(m_decimalBox, m_thousandsBox);
        QWidget::setTabOrder(m_thousandsBox, m_editAmounts);
        QWidget::setTabOrder(m_editAmounts, m_editInputFilterLocation);
        QWidget::setTabOrder(m_editInputFilterLocation, m_editOutputFilterLocation);

        retranslateUi(MyMoneyQifProfileEditor);

        QMetaObject::connectSlotsByName(MyMoneyQifProfileEditor);
    } // setupUi

    void retranslateUi(QWidget *MyMoneyQifProfileEditor)
    {
        MyMoneyQifProfileEditor->setWindowTitle(tr2i18n("QIF Profile Editor", nullptr));

        const bool __sortingEnabled = m_profileListBox->isSortingEnabled();
        m_profileListBox->setSortingEnabled(false);
        m_profileListBox->setSortingEnabled(__sortingEnabled);

        TextLabel2_2_3->setText(tr2i18n("Void mark", nullptr));
        TextLabel2_2->setText(tr2i18n("Opening Balance text", nullptr));
        TextLabel2->setText(tr2i18n("Type field text", nullptr));
        TextLabel2_2_2->setText(tr2i18n("Account delimiter", nullptr));
        TextLabel1->setText(tr2i18n("Description", nullptr));
        m_attemptMatch->setText(tr2i18n("Attempt to match similar transactions", nullptr));
        profileTabs->setTabText(profileTabs->indexOf(tabGeneral), tr2i18n("General", "@title QIF profile general information"));
        TextLabel1_4_2->setText(tr2i18n("Output filter location", nullptr));
        TextLabel1_4_3->setText(tr2i18n("Input filter file type", nullptr));
        m_editInputFilterFileType->setText(tr2i18n("*.qif", nullptr));
        TextLabel1_4->setText(tr2i18n("Input filter location", nullptr));
        profileTabs->setTabText(profileTabs->indexOf(tabFilter), tr2i18n("Filter", nullptr));
#if QT_CONFIG(whatsthis)
        m_editDateFormat->setWhatsThis(tr2i18n("The format of the dates in the QIF file.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        TextLabel3->setWhatsThis(tr2i18n("The format of the dates in the QIF file.", nullptr));
#endif // QT_CONFIG(whatsthis)
        TextLabel3->setText(tr2i18n("Date Format", nullptr));
#if QT_CONFIG(whatsthis)
        m_editApostrophe->setWhatsThis(tr2i18n("If a QIF file contains date entries with years represented with two digits then either an apostrophe or a slash may be used to delimit the year in the dates for certain centuries.  This enables 1905 to be distinguished from 2005.  Specify here which range of years will have year delimited by an apostrophe (eg. for Quicken this is usually 1900-1949).", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        TextLabel4->setWhatsThis(tr2i18n("If a QIF file contains date entries with years represented with two digits then either an apostrophe or a slash may be used to delimit the year in the dates for certain centuries.  This enables 1905 to be distinguished from 2005.  Specify here which range of years will have year delimited by an apostrophe (eg. for Quicken this is usually 1900-1949).", nullptr));
#endif // QT_CONFIG(whatsthis)
        TextLabel4->setText(tr2i18n("Apostrophe Handling", nullptr));
        profileTabs->setTabText(profileTabs->indexOf(tabDate), tr2i18n("Date", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = m_editAmounts->headerItem();
        ___qtreewidgetitem->setText(4, tr2i18n("SortColumnInvisible", nullptr));
        ___qtreewidgetitem->setText(3, tr2i18n("Thousand Delimiter", nullptr));
        ___qtreewidgetitem->setText(2, tr2i18n("Decimal Symbol", nullptr));
        ___qtreewidgetitem->setText(1, tr2i18n("QIF-Record", nullptr));
        ___qtreewidgetitem->setText(0, tr2i18n("Field", "@title header of the QIF field name column"));

        const bool __sortingEnabled1 = m_editAmounts->isSortingEnabled();
        m_editAmounts->setSortingEnabled(false);
        QTreeWidgetItem *___qtreewidgetitem1 = m_editAmounts->topLevelItem(0);
        ___qtreewidgetitem1->setText(4, tr2i18n("0", nullptr));
        ___qtreewidgetitem1->setText(1, tr2i18n("T", nullptr));
        ___qtreewidgetitem1->setText(0, tr2i18n("Value", nullptr));
        QTreeWidgetItem *___qtreewidgetitem2 = m_editAmounts->topLevelItem(1);
        ___qtreewidgetitem2->setText(4, tr2i18n("1", nullptr));
        ___qtreewidgetitem2->setText(1, tr2i18n("$", nullptr));
        ___qtreewidgetitem2->setText(0, tr2i18n("Split-Value", nullptr));
        QTreeWidgetItem *___qtreewidgetitem3 = m_editAmounts->topLevelItem(2);
        ___qtreewidgetitem3->setText(4, tr2i18n("2", nullptr));
        ___qtreewidgetitem3->setText(1, tr2i18n("O", nullptr));
        ___qtreewidgetitem3->setText(0, tr2i18n("Commission", nullptr));
        QTreeWidgetItem *___qtreewidgetitem4 = m_editAmounts->topLevelItem(3);
        ___qtreewidgetitem4->setText(4, tr2i18n("3", nullptr));
        ___qtreewidgetitem4->setText(1, tr2i18n("I", nullptr));
        ___qtreewidgetitem4->setText(0, tr2i18n("Price", nullptr));
        QTreeWidgetItem *___qtreewidgetitem5 = m_editAmounts->topLevelItem(4);
        ___qtreewidgetitem5->setText(4, tr2i18n("4", nullptr));
        ___qtreewidgetitem5->setText(1, tr2i18n("Q", nullptr));
        ___qtreewidgetitem5->setText(0, tr2i18n("Quantity", nullptr));
        m_editAmounts->setSortingEnabled(__sortingEnabled1);

        TextLabel1_2->setText(tr2i18n("Decimal Symbol", nullptr));
        TextLabel1_3->setText(tr2i18n("Thousands Separator", nullptr));
        profileTabs->setTabText(profileTabs->indexOf(tabMoney), tr2i18n("Amounts", nullptr));
        m_helpButton->setText(tr2i18n("Help", nullptr));
        m_newButton->setText(tr2i18n("New", "@action create new profile"));
        m_deleteButton->setText(tr2i18n("Delete", nullptr));
        m_renameButton->setText(tr2i18n("Rename", nullptr));
        m_resetButton->setText(tr2i18n("Reset", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MyMoneyQifProfileEditor: public Ui_MyMoneyQifProfileEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // MYMONEYQIFPROFILEEDITOR_H

