/********************************************************************************
** Form generated from reading UI file 'xmlstoragesettings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_XMLSTORAGESETTINGS_H
#define UI_XMLSTORAGESETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "keditlistwidget.h"
#include "kled.h"
#include "klineedit.h"
#include "kmessagewidget.h"

QT_BEGIN_NAMESPACE

class Ui_XMLStorageSettings
{
public:
    QVBoxLayout *verticalLayout_2;
    QCheckBox *kcfg_WriteDataEncrypted;
    KMessageWidget *m_messageWidget;
    QGroupBox *m_idGroup;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel1_2;
    KComboBox *m_masterKeyCombo;
    KLineEdit *kcfg_GpgRecipient;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_3;
    KEditListWidget *kcfg_GpgRecipientList;
    QHBoxLayout *hboxLayout1;
    KLed *m_userKeysFound;
    QLabel *textLabel1;
    QSpacerItem *spacer3;
    QHBoxLayout *hboxLayout2;
    KLed *m_recoverKeyFound;
    QLabel *textLabel2;
    QSpacerItem *spacer6;
    QCheckBox *kcfg_EncryptRecover;
    QSpacerItem *spacer5;

    void setupUi(QWidget *XMLStorageSettings)
    {
        if (XMLStorageSettings->objectName().isEmpty())
            XMLStorageSettings->setObjectName("XMLStorageSettings");
        XMLStorageSettings->resize(409, 486);
        verticalLayout_2 = new QVBoxLayout(XMLStorageSettings);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        kcfg_WriteDataEncrypted = new QCheckBox(XMLStorageSettings);
        kcfg_WriteDataEncrypted->setObjectName("kcfg_WriteDataEncrypted");

        verticalLayout_2->addWidget(kcfg_WriteDataEncrypted);

        m_messageWidget = new KMessageWidget(XMLStorageSettings);
        m_messageWidget->setObjectName("m_messageWidget");

        verticalLayout_2->addWidget(m_messageWidget);

        m_idGroup = new QGroupBox(XMLStorageSettings);
        m_idGroup->setObjectName("m_idGroup");
        verticalLayout = new QVBoxLayout(m_idGroup);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        textLabel1_2 = new QLabel(m_idGroup);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        hboxLayout->addWidget(textLabel1_2);

        m_masterKeyCombo = new KComboBox(m_idGroup);
        m_masterKeyCombo->setObjectName("m_masterKeyCombo");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_masterKeyCombo->sizePolicy().hasHeightForWidth());
        m_masterKeyCombo->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(m_masterKeyCombo);


        verticalLayout->addLayout(hboxLayout);

        kcfg_GpgRecipient = new KLineEdit(m_idGroup);
        kcfg_GpgRecipient->setObjectName("kcfg_GpgRecipient");

        verticalLayout->addWidget(kcfg_GpgRecipient);

        groupBox = new QGroupBox(m_idGroup);
        groupBox->setObjectName("groupBox");
        verticalLayout_3 = new QVBoxLayout(groupBox);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        kcfg_GpgRecipientList = new KEditListWidget(groupBox);
        kcfg_GpgRecipientList->setObjectName("kcfg_GpgRecipientList");
        kcfg_GpgRecipientList->setButtons(KEditListWidget::Add|KEditListWidget::Remove);

        verticalLayout_3->addWidget(kcfg_GpgRecipientList);


        verticalLayout->addWidget(groupBox);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        m_userKeysFound = new KLed(m_idGroup);
        m_userKeysFound->setObjectName("m_userKeysFound");
        m_userKeysFound->setShape(KLed::Circular);
        m_userKeysFound->setLook(KLed::Sunken);

        hboxLayout1->addWidget(m_userKeysFound);

        textLabel1 = new QLabel(m_idGroup);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        hboxLayout1->addWidget(textLabel1);

        spacer3 = new QSpacerItem(50, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacer3);


        verticalLayout->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        m_recoverKeyFound = new KLed(m_idGroup);
        m_recoverKeyFound->setObjectName("m_recoverKeyFound");
        m_recoverKeyFound->setShape(KLed::Circular);
        m_recoverKeyFound->setLook(KLed::Sunken);

        hboxLayout2->addWidget(m_recoverKeyFound);

        textLabel2 = new QLabel(m_idGroup);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        hboxLayout2->addWidget(textLabel2);

        spacer6 = new QSpacerItem(16, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout2->addItem(spacer6);


        verticalLayout->addLayout(hboxLayout2);

        kcfg_EncryptRecover = new QCheckBox(m_idGroup);
        kcfg_EncryptRecover->setObjectName("kcfg_EncryptRecover");

        verticalLayout->addWidget(kcfg_EncryptRecover);


        verticalLayout_2->addWidget(m_idGroup);

        spacer5 = new QSpacerItem(21, 30, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(spacer5);


        retranslateUi(XMLStorageSettings);

        QMetaObject::connectSlotsByName(XMLStorageSettings);
    } // setupUi

    void retranslateUi(QWidget *XMLStorageSettings)
    {
        XMLStorageSettings->setWindowTitle(tr2i18n("GPG encryption settings", nullptr));
#if QT_CONFIG(whatsthis)
        XMLStorageSettings->setWhatsThis(tr2i18n("This page allows you to set the parameters for encrypted file storage of your <b>KMyMoney</b> data based on <b>GPG</b>.<p>\n"
"Access to the settings is disabled if <b>GPG</b> could not be detected on your system. In this case, please make sure that <b>GPG</b> is working properly for the current user.<p>\n"
"The <i>additional recovery encryption</i> is only accessible, if the necessary key for <b>kmymoney-recover@users.sourceforge.net</b> with id 0x8AFDDC8E is found in your keyring.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_WriteDataEncrypted->setText(tr2i18n("Use GPG encryption", nullptr));
        m_idGroup->setTitle(tr2i18n("GPG encryption", nullptr));
        textLabel1_2->setText(tr2i18n("Your key", nullptr));
#if QT_CONFIG(whatsthis)
        m_masterKeyCombo->setWhatsThis(tr2i18n("This combo box lists all the secret keys you have in your keyring. Select the one you want to use for encryption when saving to a file.", nullptr));
#endif // QT_CONFIG(whatsthis)
        groupBox->setTitle(tr2i18n("Additional keys", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_GpgRecipientList->setWhatsThis(tr2i18n("Enter the id of the key you want to use for data encryption. This can either be an e-mail address or the hexadecimal key id. In case of the key id do not forget the leading 0x.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_GpgRecipientList->setProperty("title", QVariant(tr2i18n("Additional keys", nullptr)));
#if QT_CONFIG(whatsthis)
        m_userKeysFound->setWhatsThis(tr2i18n("This symbol denotes, if the key for the given user id has been found in your keyring. It is green when found, dark otherwise.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1->setText(tr2i18n("Keys for all of the above user ids found", nullptr));
#if QT_CONFIG(whatsthis)
        m_recoverKeyFound->setWhatsThis(tr2i18n("This symbol denotes, if the KMyMoney recovery key has been found in your keyring. It is green when found, dark otherwise.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel2->setText(tr2i18n("Recover Key available in keyring", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_EncryptRecover->setWhatsThis(tr2i18n("You can specify to encrypt the data also with the KMyMoney recover key. Only the core KMyMoney developers are in possession of the respective private key required to read back such encrypted data.<p>\n"
"\n"
"This mechanism is provided for the case that you have lost your key and cannot access your data anymore. With this option activated, the KMyMoney developers can decrypt the data and supply you with it in a readable form. Please be prepared, that you have to answer a few detailed questions about the contents of your data before we will send it out.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_EncryptRecover->setText(tr2i18n("Also encrypt with KMyMoney's recover key", nullptr));
    } // retranslateUi

};

namespace Ui {
    class XMLStorageSettings: public Ui_XMLStorageSettings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // XMLSTORAGESETTINGS_H

