/********************************************************************************
** Form generated from reading UI file 'effectivedatewizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EFFECTIVEDATEWIZARDPAGE_H
#define UI_EFFECTIVEDATEWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>
#include "kdatecombobox.h"
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_EffectiveDateWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *m_effectiveDateLabel;
    QLabel *m_effectiveDateNoteLabel;
    QSpacerItem *Spacer68_3;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel1_7;
    KMyMoneyDateEdit *m_effectiveChangeDateEdit;
    QSpacerItem *Spacer69_2;

    void setupUi(QWizardPage *EffectiveDateWizardPage)
    {
        if (EffectiveDateWizardPage->objectName().isEmpty())
            EffectiveDateWizardPage->setObjectName("EffectiveDateWizardPage");
        EffectiveDateWizardPage->resize(385, 232);
        vboxLayout = new QVBoxLayout(EffectiveDateWizardPage);
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        m_effectiveDateLabel = new QLabel(EffectiveDateWizardPage);
        m_effectiveDateLabel->setObjectName("m_effectiveDateLabel");
        m_effectiveDateLabel->setAlignment(Qt::AlignTop);
        m_effectiveDateLabel->setWordWrap(true);

        vboxLayout->addWidget(m_effectiveDateLabel);

        m_effectiveDateNoteLabel = new QLabel(EffectiveDateWizardPage);
        m_effectiveDateNoteLabel->setObjectName("m_effectiveDateNoteLabel");
        m_effectiveDateNoteLabel->setAlignment(Qt::AlignTop);
        m_effectiveDateNoteLabel->setWordWrap(true);

        vboxLayout->addWidget(m_effectiveDateNoteLabel);

        Spacer68_3 = new QSpacerItem(0, 30, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer68_3);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        TextLabel1_7 = new QLabel(EffectiveDateWizardPage);
        TextLabel1_7->setObjectName("TextLabel1_7");
        TextLabel1_7->setWordWrap(false);

        hboxLayout->addWidget(TextLabel1_7);

        m_effectiveChangeDateEdit = new KMyMoneyDateEdit(EffectiveDateWizardPage);
        m_effectiveChangeDateEdit->setObjectName("m_effectiveChangeDateEdit");

        hboxLayout->addWidget(m_effectiveChangeDateEdit);


        vboxLayout->addLayout(hboxLayout);

        Spacer69_2 = new QSpacerItem(0, 87, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer69_2);


        retranslateUi(EffectiveDateWizardPage);

        QMetaObject::connectSlotsByName(EffectiveDateWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *EffectiveDateWizardPage)
    {
        EffectiveDateWizardPage->setTitle(tr2i18n("Effective date", nullptr));
        m_effectiveDateLabel->setText(tr2i18n("1", nullptr));
        m_effectiveDateNoteLabel->setText(tr2i18n("1", nullptr));
        TextLabel1_7->setText(tr2i18n("When should the changes become active?", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EffectiveDateWizardPage: public Ui_EffectiveDateWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // EFFECTIVEDATEWIZARDPAGE_H

