/********************************************************************************
** Form generated from reading UI file 'paymentwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAYMENTWIZARDPAGE_H
#define UI_PAYMENTWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <amountedit.h>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_PaymentWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel16_2;
    QSpacerItem *Spacer74;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel1_4_2;
    AmountEdit *m_paymentEdit;
    QFrame *Frame3_2_2_2;
    QGridLayout *gridLayout;
    QLabel *TextLabel6_3_2_2;
    QLabel *TextLabel8_3_2_2;
    QLabel *TextLabel9_2_2_2;
    QLabel *TextLabel10_2_2_2;
    QLabel *TextLabel11_3_2_2;
    QLabel *m_loanAmount4;
    QLabel *m_interestRate4;
    QLabel *m_duration4;
    QLabel *m_payment4;
    QLabel *m_balloon4;

    void setupUi(QWizardPage *PaymentWizardPage)
    {
        if (PaymentWizardPage->objectName().isEmpty())
            PaymentWizardPage->setObjectName("PaymentWizardPage");
        vboxLayout = new QVBoxLayout(PaymentWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel16_2 = new QLabel(PaymentWizardPage);
        TextLabel16_2->setObjectName("TextLabel16_2");
        TextLabel16_2->setAlignment(Qt::AlignTop);
        TextLabel16_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel16_2);

        Spacer74 = new QSpacerItem(0, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer74);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        TextLabel1_4_2 = new QLabel(PaymentWizardPage);
        TextLabel1_4_2->setObjectName("TextLabel1_4_2");
        TextLabel1_4_2->setWordWrap(false);

        hboxLayout->addWidget(TextLabel1_4_2);

        m_paymentEdit = new AmountEdit(PaymentWizardPage);
        m_paymentEdit->setObjectName("m_paymentEdit");

        hboxLayout->addWidget(m_paymentEdit);


        vboxLayout->addLayout(hboxLayout);

        Frame3_2_2_2 = new QFrame(PaymentWizardPage);
        Frame3_2_2_2->setObjectName("Frame3_2_2_2");
        Frame3_2_2_2->setFrameShape(QFrame::StyledPanel);
        Frame3_2_2_2->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(Frame3_2_2_2);
#ifndef UI_Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        TextLabel6_3_2_2 = new QLabel(Frame3_2_2_2);
        TextLabel6_3_2_2->setObjectName("TextLabel6_3_2_2");
        TextLabel6_3_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel6_3_2_2->setWordWrap(true);

        gridLayout->addWidget(TextLabel6_3_2_2, 0, 0, 1, 1);

        TextLabel8_3_2_2 = new QLabel(Frame3_2_2_2);
        TextLabel8_3_2_2->setObjectName("TextLabel8_3_2_2");
        TextLabel8_3_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel8_3_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel8_3_2_2, 1, 0, 1, 1);

        TextLabel9_2_2_2 = new QLabel(Frame3_2_2_2);
        TextLabel9_2_2_2->setObjectName("TextLabel9_2_2_2");
        TextLabel9_2_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel9_2_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel9_2_2_2, 2, 0, 1, 1);

        TextLabel10_2_2_2 = new QLabel(Frame3_2_2_2);
        TextLabel10_2_2_2->setObjectName("TextLabel10_2_2_2");
        TextLabel10_2_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel10_2_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel10_2_2_2, 3, 0, 1, 1);

        TextLabel11_3_2_2 = new QLabel(Frame3_2_2_2);
        TextLabel11_3_2_2->setObjectName("TextLabel11_3_2_2");
        TextLabel11_3_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel11_3_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel11_3_2_2, 4, 0, 1, 1);

        m_loanAmount4 = new QLabel(Frame3_2_2_2);
        m_loanAmount4->setObjectName("m_loanAmount4");
        QFont font;
        font.setBold(true);
        m_loanAmount4->setFont(font);
        m_loanAmount4->setTextFormat(Qt::PlainText);
        m_loanAmount4->setWordWrap(false);

        gridLayout->addWidget(m_loanAmount4, 0, 1, 1, 1);

        m_interestRate4 = new QLabel(Frame3_2_2_2);
        m_interestRate4->setObjectName("m_interestRate4");
        m_interestRate4->setFont(font);
        m_interestRate4->setTextFormat(Qt::PlainText);
        m_interestRate4->setWordWrap(false);

        gridLayout->addWidget(m_interestRate4, 1, 1, 1, 1);

        m_duration4 = new QLabel(Frame3_2_2_2);
        m_duration4->setObjectName("m_duration4");
        m_duration4->setFont(font);
        m_duration4->setTextFormat(Qt::PlainText);
        m_duration4->setWordWrap(false);

        gridLayout->addWidget(m_duration4, 2, 1, 1, 1);

        m_payment4 = new QLabel(Frame3_2_2_2);
        m_payment4->setObjectName("m_payment4");
        m_payment4->setFont(font);
        m_payment4->setTextFormat(Qt::PlainText);
        m_payment4->setWordWrap(false);

        gridLayout->addWidget(m_payment4, 3, 1, 1, 1);

        m_balloon4 = new QLabel(Frame3_2_2_2);
        m_balloon4->setObjectName("m_balloon4");
        m_balloon4->setFont(font);
        m_balloon4->setTextFormat(Qt::PlainText);
        m_balloon4->setWordWrap(false);

        gridLayout->addWidget(m_balloon4, 4, 1, 1, 1);


        vboxLayout->addWidget(Frame3_2_2_2);


        retranslateUi(PaymentWizardPage);

        QMetaObject::connectSlotsByName(PaymentWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *PaymentWizardPage)
    {
        TextLabel16_2->setText(tr2i18n("\n"
"Please enter the amount you pay for principal and interest or leave the field empty to calculate it.", nullptr));
        TextLabel1_4_2->setText(tr2i18n("Principal and interest", nullptr));
        TextLabel6_3_2_2->setText(tr2i18n("Loan amount:", nullptr));
        TextLabel8_3_2_2->setText(tr2i18n("Interest rate:", nullptr));
        TextLabel9_2_2_2->setText(tr2i18n("Term:", nullptr));
        TextLabel10_2_2_2->setText(tr2i18n("Principal + Interest:", nullptr));
        TextLabel11_3_2_2->setText(tr2i18n("Final amortization payment", nullptr));
        m_loanAmount4->setText(tr2i18n("1", nullptr));
        m_interestRate4->setText(tr2i18n("1", nullptr));
        m_duration4->setText(tr2i18n("1", nullptr));
        m_payment4->setText(tr2i18n("1", nullptr));
        m_balloon4->setText(tr2i18n("1", nullptr));
        (void)PaymentWizardPage;
    } // retranslateUi

};

namespace Ui {
    class PaymentWizardPage: public Ui_PaymentWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PAYMENTWIZARDPAGE_H

