/********************************************************************************
** Form generated from reading UI file 'kforecastview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KFORECASTVIEW_H
#define UI_KFORECASTVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "ktextedit.h"

QT_BEGIN_NAMESPACE

class Ui_KForecastView
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox3;
    QHBoxLayout *horizontalLayout_3;
    QFormLayout *formLayout;
    QLabel *textLabel1;
    QSpinBox *m_forecastDays;
    QLabel *textLabel3;
    QSpinBox *m_accountsCycle;
    QLabel *textLabel5;
    QSpinBox *m_beginDay;
    QLabel *textLabel4;
    QSpinBox *m_forecastCycles;
    QLabel *textLabel6;
    KComboBox *m_comboDetail;
    QSpacerItem *horizontalSpacer;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *textLabel2;
    QLabel *m_forecastMethod;
    QSpacerItem *spacer22;
    QPushButton *m_forecastButton;
    QGroupBox *m_historyMethodGroupBox;
    QVBoxLayout *vboxLayout;
    QRadioButton *radioButton11;
    QRadioButton *radioButton12;
    QRadioButton *radioButton13;
    QTabWidget *m_tab;
    QWidget *summary;
    QHBoxLayout *horizontalLayout;
    QSplitter *splitter;
    QTreeWidget *m_summaryList;
    KTextEdit *m_adviceText;
    QWidget *detail;
    QVBoxLayout *vboxLayout1;
    QTreeWidget *m_forecastList;
    QWidget *advanced;
    QVBoxLayout *vboxLayout2;
    QTreeWidget *m_advancedList;
    QWidget *budget;
    QVBoxLayout *vboxLayout3;
    QTreeWidget *m_budgetList;
    QLabel *m_budgetHistoryRange;
    QWidget *m_tabChart;
    QVBoxLayout *vboxLayout4;
    QButtonGroup *m_historyMethod;

    void setupUi(QWidget *KForecastView)
    {
        if (KForecastView->objectName().isEmpty())
            KForecastView->setObjectName("KForecastView");
        KForecastView->resize(631, 286);
        verticalLayout = new QVBoxLayout(KForecastView);
        verticalLayout->setSpacing(0);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        groupBox3 = new QGroupBox(KForecastView);
        groupBox3->setObjectName("groupBox3");
        horizontalLayout_3 = new QHBoxLayout(groupBox3);
        horizontalLayout_3->setSpacing(0);
        horizontalLayout_3->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        formLayout = new QFormLayout();
        formLayout->setSpacing(0);
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout->setLabelAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        textLabel1 = new QLabel(groupBox3);
        textLabel1->setObjectName("textLabel1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textLabel1->sizePolicy().hasHeightForWidth());
        textLabel1->setSizePolicy(sizePolicy);
        textLabel1->setWordWrap(false);

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, textLabel1);

        m_forecastDays = new QSpinBox(groupBox3);
        m_forecastDays->setObjectName("m_forecastDays");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_forecastDays->sizePolicy().hasHeightForWidth());
        m_forecastDays->setSizePolicy(sizePolicy1);
        m_forecastDays->setMinimum(1);
        m_forecastDays->setMaximum(999);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, m_forecastDays);

        textLabel3 = new QLabel(groupBox3);
        textLabel3->setObjectName("textLabel3");
        sizePolicy.setHeightForWidth(textLabel3->sizePolicy().hasHeightForWidth());
        textLabel3->setSizePolicy(sizePolicy);
        textLabel3->setWordWrap(false);

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, textLabel3);

        m_accountsCycle = new QSpinBox(groupBox3);
        m_accountsCycle->setObjectName("m_accountsCycle");
        sizePolicy1.setHeightForWidth(m_accountsCycle->sizePolicy().hasHeightForWidth());
        m_accountsCycle->setSizePolicy(sizePolicy1);
        m_accountsCycle->setMinimum(1);
        m_accountsCycle->setMaximum(999);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, m_accountsCycle);

        textLabel5 = new QLabel(groupBox3);
        textLabel5->setObjectName("textLabel5");
        textLabel5->setWordWrap(false);

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, textLabel5);

        m_beginDay = new QSpinBox(groupBox3);
        m_beginDay->setObjectName("m_beginDay");
        sizePolicy1.setHeightForWidth(m_beginDay->sizePolicy().hasHeightForWidth());
        m_beginDay->setSizePolicy(sizePolicy1);
        m_beginDay->setMinimum(0);
        m_beginDay->setMaximum(31);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, m_beginDay);

        textLabel4 = new QLabel(groupBox3);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, textLabel4);

        m_forecastCycles = new QSpinBox(groupBox3);
        m_forecastCycles->setObjectName("m_forecastCycles");
        sizePolicy1.setHeightForWidth(m_forecastCycles->sizePolicy().hasHeightForWidth());
        m_forecastCycles->setSizePolicy(sizePolicy1);
        m_forecastCycles->setMinimum(1);
        m_forecastCycles->setMaximum(999);

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, m_forecastCycles);

        textLabel6 = new QLabel(groupBox3);
        textLabel6->setObjectName("textLabel6");
        textLabel6->setWordWrap(false);

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, textLabel6);

        m_comboDetail = new KComboBox(groupBox3);
        m_comboDetail->addItem(QString());
        m_comboDetail->addItem(QString());
        m_comboDetail->addItem(QString());
        m_comboDetail->addItem(QString());
        m_comboDetail->setObjectName("m_comboDetail");

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, m_comboDetail);


        horizontalLayout_3->addLayout(formLayout);

        horizontalSpacer = new QSpacerItem(62, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(0);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        textLabel2 = new QLabel(groupBox3);
        textLabel2->setObjectName("textLabel2");
        sizePolicy.setHeightForWidth(textLabel2->sizePolicy().hasHeightForWidth());
        textLabel2->setSizePolicy(sizePolicy);
        textLabel2->setWordWrap(false);

        horizontalLayout_2->addWidget(textLabel2);

        m_forecastMethod = new QLabel(groupBox3);
        m_forecastMethod->setObjectName("m_forecastMethod");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(2);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(m_forecastMethod->sizePolicy().hasHeightForWidth());
        m_forecastMethod->setSizePolicy(sizePolicy2);
        QFont font;
        font.setBold(true);
        m_forecastMethod->setFont(font);
        m_forecastMethod->setWordWrap(false);

        horizontalLayout_2->addWidget(m_forecastMethod);

        spacer22 = new QSpacerItem(100, 21, QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(spacer22);

        m_forecastButton = new QPushButton(groupBox3);
        m_forecastButton->setObjectName("m_forecastButton");

        horizontalLayout_2->addWidget(m_forecastButton);


        verticalLayout_2->addLayout(horizontalLayout_2);

        m_historyMethodGroupBox = new QGroupBox(groupBox3);
        m_historyMethodGroupBox->setObjectName("m_historyMethodGroupBox");
        vboxLayout = new QVBoxLayout(m_historyMethodGroupBox);
        vboxLayout->setSpacing(0);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName("vboxLayout");
        radioButton11 = new QRadioButton(m_historyMethodGroupBox);
        m_historyMethod = new QButtonGroup(KForecastView);
        m_historyMethod->setObjectName("m_historyMethod");
        m_historyMethod->addButton(radioButton11);
        radioButton11->setObjectName("radioButton11");

        vboxLayout->addWidget(radioButton11);

        radioButton12 = new QRadioButton(m_historyMethodGroupBox);
        m_historyMethod->addButton(radioButton12);
        radioButton12->setObjectName("radioButton12");

        vboxLayout->addWidget(radioButton12);

        radioButton13 = new QRadioButton(m_historyMethodGroupBox);
        m_historyMethod->addButton(radioButton13);
        radioButton13->setObjectName("radioButton13");

        vboxLayout->addWidget(radioButton13);


        verticalLayout_2->addWidget(m_historyMethodGroupBox);


        horizontalLayout_3->addLayout(verticalLayout_2);


        verticalLayout->addWidget(groupBox3);

        m_tab = new QTabWidget(KForecastView);
        m_tab->setObjectName("m_tab");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(1);
        sizePolicy3.setHeightForWidth(m_tab->sizePolicy().hasHeightForWidth());
        m_tab->setSizePolicy(sizePolicy3);
        summary = new QWidget();
        summary->setObjectName("summary");
        horizontalLayout = new QHBoxLayout(summary);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout->setObjectName("horizontalLayout");
        splitter = new QSplitter(summary);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Horizontal);
        m_summaryList = new QTreeWidget(splitter);
        QTreeWidgetItem *__qtreewidgetitem = new QTreeWidgetItem();
        __qtreewidgetitem->setText(0, QString::fromUtf8("1"));
        m_summaryList->setHeaderItem(__qtreewidgetitem);
        m_summaryList->setObjectName("m_summaryList");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy4.setHorizontalStretch(1);
        sizePolicy4.setVerticalStretch(1);
        sizePolicy4.setHeightForWidth(m_summaryList->sizePolicy().hasHeightForWidth());
        m_summaryList->setSizePolicy(sizePolicy4);
        m_summaryList->setAlternatingRowColors(true);
        splitter->addWidget(m_summaryList);
        m_adviceText = new KTextEdit(splitter);
        m_adviceText->setObjectName("m_adviceText");
        m_adviceText->setReadOnly(true);
        splitter->addWidget(m_adviceText);

        horizontalLayout->addWidget(splitter);

        m_tab->addTab(summary, QString());
        detail = new QWidget();
        detail->setObjectName("detail");
        vboxLayout1 = new QVBoxLayout(detail);
        vboxLayout1->setSpacing(0);
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        vboxLayout1->setObjectName("vboxLayout1");
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        m_forecastList = new QTreeWidget(detail);
        QTreeWidgetItem *__qtreewidgetitem1 = new QTreeWidgetItem();
        __qtreewidgetitem1->setText(0, QString::fromUtf8("1"));
        m_forecastList->setHeaderItem(__qtreewidgetitem1);
        m_forecastList->setObjectName("m_forecastList");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(m_forecastList->sizePolicy().hasHeightForWidth());
        m_forecastList->setSizePolicy(sizePolicy5);
        m_forecastList->setAlternatingRowColors(true);

        vboxLayout1->addWidget(m_forecastList);

        m_tab->addTab(detail, QString());
        advanced = new QWidget();
        advanced->setObjectName("advanced");
        vboxLayout2 = new QVBoxLayout(advanced);
        vboxLayout2->setSpacing(0);
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        vboxLayout2->setObjectName("vboxLayout2");
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        m_advancedList = new QTreeWidget(advanced);
        QTreeWidgetItem *__qtreewidgetitem2 = new QTreeWidgetItem();
        __qtreewidgetitem2->setText(0, QString::fromUtf8("1"));
        m_advancedList->setHeaderItem(__qtreewidgetitem2);
        m_advancedList->setObjectName("m_advancedList");
        m_advancedList->setAlternatingRowColors(true);
        m_advancedList->setRootIsDecorated(false);

        vboxLayout2->addWidget(m_advancedList);

        m_tab->addTab(advanced, QString());
        budget = new QWidget();
        budget->setObjectName("budget");
        vboxLayout3 = new QVBoxLayout(budget);
        vboxLayout3->setSpacing(0);
        vboxLayout3->setContentsMargins(0, 0, 0, 0);
        vboxLayout3->setObjectName("vboxLayout3");
        vboxLayout3->setContentsMargins(0, 0, 0, 0);
        m_budgetList = new QTreeWidget(budget);
        QTreeWidgetItem *__qtreewidgetitem3 = new QTreeWidgetItem();
        __qtreewidgetitem3->setText(0, QString::fromUtf8("1"));
        m_budgetList->setHeaderItem(__qtreewidgetitem3);
        m_budgetList->setObjectName("m_budgetList");
        m_budgetList->setAlternatingRowColors(true);

        vboxLayout3->addWidget(m_budgetList);

        m_budgetHistoryRange = new QLabel(budget);
        m_budgetHistoryRange->setObjectName("m_budgetHistoryRange");
        m_budgetHistoryRange->setText(QString::fromUtf8("Historic data range"));

        vboxLayout3->addWidget(m_budgetHistoryRange);

        m_tab->addTab(budget, QString());
        m_tabChart = new QWidget();
        m_tabChart->setObjectName("m_tabChart");
        vboxLayout4 = new QVBoxLayout(m_tabChart);
        vboxLayout4->setSpacing(0);
        vboxLayout4->setContentsMargins(0, 0, 0, 0);
        vboxLayout4->setObjectName("vboxLayout4");
        vboxLayout4->setContentsMargins(0, 0, 0, 0);
        m_tab->addTab(m_tabChart, QString());

        verticalLayout->addWidget(m_tab);


        retranslateUi(KForecastView);

        m_comboDetail->setCurrentIndex(2);
        m_tab->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KForecastView);
    } // setupUi

    void retranslateUi(QWidget *KForecastView)
    {
        groupBox3->setTitle(tr2i18n("Forecast Settings", nullptr));
        textLabel1->setText(tr2i18n("Days to Forecast:", nullptr));
        textLabel3->setText(tr2i18n("Days of Accounts Cycle:", nullptr));
        textLabel5->setText(tr2i18n("Day of Month to start Forecast:", nullptr));
        textLabel4->setText(tr2i18n("Historic Cycles:", nullptr));
        textLabel6->setText(tr2i18n("Chart Detail:", nullptr));
        m_comboDetail->setItemText(0, tr2i18n("All", "Show all details"));
        m_comboDetail->setItemText(1, tr2i18n("Top-Level", nullptr));
        m_comboDetail->setItemText(2, tr2i18n("Groups", nullptr));
        m_comboDetail->setItemText(3, tr2i18n("Totals", nullptr));

#if QT_CONFIG(tooltip)
        m_comboDetail->setToolTip(tr2i18n("<p>Choose what level of detail to show on the chart.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel2->setText(tr2i18n("Current Forecast Method:", nullptr));
        m_forecastMethod->setText(tr2i18n("method", nullptr));
        m_forecastButton->setText(tr2i18n("Forecast", nullptr));
        m_historyMethodGroupBox->setTitle(tr2i18n("History Forecast Method", nullptr));
        radioButton11->setText(tr2i18n("Simple Moving Average", nullptr));
        radioButton12->setText(tr2i18n("Weighted Moving Average", nullptr));
        radioButton13->setText(tr2i18n("Linear Regression", nullptr));
        m_tab->setTabText(m_tab->indexOf(summary), tr2i18n("Summary", nullptr));
        m_tab->setTabText(m_tab->indexOf(detail), tr2i18n("Detail", nullptr));
        m_tab->setTabText(m_tab->indexOf(advanced), tr2i18n("Advanced", "Advanced forecast view"));
        m_tab->setTabText(m_tab->indexOf(budget), tr2i18n("Budget Forecast", nullptr));
        m_tab->setTabText(m_tab->indexOf(m_tabChart), tr2i18n("Chart", nullptr));
        (void)KForecastView;
    } // retranslateUi

};

namespace Ui {
    class KForecastView: public Ui_KForecastView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KFORECASTVIEW_H

