/********************************************************************************
** Form generated from reading UI file 'csvexportdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CSVEXPORTDLG_H
#define UI_CSVEXPORTDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kmessagewidget.h"
#include "kmymoneyaccountcombo.h"
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_CsvExportDlg
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *TextLabel2;
    QFrame *Line2;
    QGridLayout *gridLayout;
    QLabel *lblFileExport;
    QLineEdit *m_qlineeditFile;
    QPushButton *m_qbuttonBrowse;
    QLabel *label;
    KMyMoneyAccountCombo *m_accountComboBox;
    KMessageWidget *infoMessage;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_3;
    QGroupBox *m_qgroupboxContents;
    QVBoxLayout *vboxLayout;
    QRadioButton *m_radioButtonAccount;
    QRadioButton *m_radioButtonCategories;
    QGroupBox *m_qgroupboxDates;
    QGridLayout *gridLayout_3;
    QGridLayout *gridLayout_2;
    QLabel *lblStartDate;
    KMyMoneyDateEdit *m_kmymoneydateStart;
    QLabel *lblEndDate;
    KMyMoneyDateEdit *m_kmymoneydateEnd;
    QGroupBox *m_qgroupboxSeparator;
    QVBoxLayout *verticalLayout_2;
    QComboBox *m_separatorComboBox;
    QSpacerItem *Spacer7;
    QFrame *Line2_2;
    QHBoxLayout *horizontalLayout_4;
    QSpacerItem *horizontalSpacer;
    QPushButton *m_qbuttonOk;
    QPushButton *m_qbuttonCancel;

    void setupUi(QDialog *CsvExportDlg)
    {
        if (CsvExportDlg->objectName().isEmpty())
            CsvExportDlg->setObjectName("CsvExportDlg");
        CsvExportDlg->resize(631, 394);
        CsvExportDlg->setSizeGripEnabled(true);
        verticalLayout = new QVBoxLayout(CsvExportDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        TextLabel2 = new QLabel(CsvExportDlg);
        TextLabel2->setObjectName("TextLabel2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(TextLabel2->sizePolicy().hasHeightForWidth());
        TextLabel2->setSizePolicy(sizePolicy);
        TextLabel2->setMaximumSize(QSize(16777215, 16777215));
        TextLabel2->setScaledContents(false);
        TextLabel2->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        TextLabel2->setWordWrap(true);
        TextLabel2->setMargin(2);

        verticalLayout->addWidget(TextLabel2);

        Line2 = new QFrame(CsvExportDlg);
        Line2->setObjectName("Line2");
        Line2->setFrameShape(QFrame::HLine);
        Line2->setFrameShadow(QFrame::Raised);

        verticalLayout->addWidget(Line2);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        lblFileExport = new QLabel(CsvExportDlg);
        lblFileExport->setObjectName("lblFileExport");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(lblFileExport->sizePolicy().hasHeightForWidth());
        lblFileExport->setSizePolicy(sizePolicy1);
        lblFileExport->setWordWrap(false);

        gridLayout->addWidget(lblFileExport, 0, 0, 1, 1);

        m_qlineeditFile = new QLineEdit(CsvExportDlg);
        m_qlineeditFile->setObjectName("m_qlineeditFile");

        gridLayout->addWidget(m_qlineeditFile, 0, 1, 1, 1);

        m_qbuttonBrowse = new QPushButton(CsvExportDlg);
        m_qbuttonBrowse->setObjectName("m_qbuttonBrowse");

        gridLayout->addWidget(m_qbuttonBrowse, 0, 2, 1, 1);

        label = new QLabel(CsvExportDlg);
        label->setObjectName("label");
        label->setMaximumSize(QSize(140, 16777215));

        gridLayout->addWidget(label, 1, 0, 1, 1);

        m_accountComboBox = new KMyMoneyAccountCombo(CsvExportDlg);
        m_accountComboBox->setObjectName("m_accountComboBox");
        m_accountComboBox->setEditable(true);
        m_accountComboBox->setIconSize(QSize(0, 0));
        m_accountComboBox->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(m_accountComboBox, 1, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        infoMessage = new KMessageWidget(CsvExportDlg);
        infoMessage->setObjectName("infoMessage");

        verticalLayout->addWidget(infoMessage);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);

        m_qgroupboxContents = new QGroupBox(CsvExportDlg);
        m_qgroupboxContents->setObjectName("m_qgroupboxContents");
        m_qgroupboxContents->setMaximumSize(QSize(16777215, 100));
        vboxLayout = new QVBoxLayout(m_qgroupboxContents);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        m_radioButtonAccount = new QRadioButton(m_qgroupboxContents);
        m_radioButtonAccount->setObjectName("m_radioButtonAccount");

        vboxLayout->addWidget(m_radioButtonAccount);

        m_radioButtonCategories = new QRadioButton(m_qgroupboxContents);
        m_radioButtonCategories->setObjectName("m_radioButtonCategories");

        vboxLayout->addWidget(m_radioButtonCategories);


        horizontalLayout_3->addWidget(m_qgroupboxContents);

        m_qgroupboxDates = new QGroupBox(CsvExportDlg);
        m_qgroupboxDates->setObjectName("m_qgroupboxDates");
        m_qgroupboxDates->setMaximumSize(QSize(16777215, 100));
        gridLayout_3 = new QGridLayout(m_qgroupboxDates);
        gridLayout_3->setSpacing(6);
        gridLayout_3->setContentsMargins(11, 11, 11, 11);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setSpacing(6);
        gridLayout_2->setObjectName("gridLayout_2");
        lblStartDate = new QLabel(m_qgroupboxDates);
        lblStartDate->setObjectName("lblStartDate");
        sizePolicy1.setHeightForWidth(lblStartDate->sizePolicy().hasHeightForWidth());
        lblStartDate->setSizePolicy(sizePolicy1);
        lblStartDate->setMinimumSize(QSize(50, 0));
        lblStartDate->setWordWrap(false);

        gridLayout_2->addWidget(lblStartDate, 0, 0, 1, 1);

        m_kmymoneydateStart = new KMyMoneyDateEdit(m_qgroupboxDates);
        m_kmymoneydateStart->setObjectName("m_kmymoneydateStart");

        gridLayout_2->addWidget(m_kmymoneydateStart, 0, 1, 1, 1);

        lblEndDate = new QLabel(m_qgroupboxDates);
        lblEndDate->setObjectName("lblEndDate");
        sizePolicy1.setHeightForWidth(lblEndDate->sizePolicy().hasHeightForWidth());
        lblEndDate->setSizePolicy(sizePolicy1);
        lblEndDate->setMinimumSize(QSize(50, 0));
        lblEndDate->setWordWrap(false);

        gridLayout_2->addWidget(lblEndDate, 1, 0, 1, 1);

        m_kmymoneydateEnd = new KMyMoneyDateEdit(m_qgroupboxDates);
        m_kmymoneydateEnd->setObjectName("m_kmymoneydateEnd");

        gridLayout_2->addWidget(m_kmymoneydateEnd, 1, 1, 1, 1);


        gridLayout_3->addLayout(gridLayout_2, 0, 0, 1, 1);


        horizontalLayout_3->addWidget(m_qgroupboxDates);

        m_qgroupboxSeparator = new QGroupBox(CsvExportDlg);
        m_qgroupboxSeparator->setObjectName("m_qgroupboxSeparator");
        m_qgroupboxSeparator->setMaximumSize(QSize(16777215, 100));
        verticalLayout_2 = new QVBoxLayout(m_qgroupboxSeparator);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        m_separatorComboBox = new QComboBox(m_qgroupboxSeparator);
        m_separatorComboBox->setObjectName("m_separatorComboBox");

        verticalLayout_2->addWidget(m_separatorComboBox);


        horizontalLayout_3->addWidget(m_qgroupboxSeparator);

        Spacer7 = new QSpacerItem(230, 16, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(Spacer7);

        horizontalLayout_3->setStretch(0, 3);
        horizontalLayout_3->setStretch(1, 12);
        horizontalLayout_3->setStretch(2, 12);
        horizontalLayout_3->setStretch(3, 8);
        horizontalLayout_3->setStretch(4, 3);

        verticalLayout->addLayout(horizontalLayout_3);

        Line2_2 = new QFrame(CsvExportDlg);
        Line2_2->setObjectName("Line2_2");
        Line2_2->setFrameShape(QFrame::HLine);
        Line2_2->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(Line2_2);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(6);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);

        m_qbuttonOk = new QPushButton(CsvExportDlg);
        m_qbuttonOk->setObjectName("m_qbuttonOk");
        m_qbuttonOk->setAutoDefault(false);

        horizontalLayout_4->addWidget(m_qbuttonOk);

        m_qbuttonCancel = new QPushButton(CsvExportDlg);
        m_qbuttonCancel->setObjectName("m_qbuttonCancel");
        m_qbuttonCancel->setAutoDefault(false);

        horizontalLayout_4->addWidget(m_qbuttonCancel);


        verticalLayout->addLayout(horizontalLayout_4);


        retranslateUi(CsvExportDlg);

        m_qbuttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(CsvExportDlg);
    } // setupUi

    void retranslateUi(QDialog *CsvExportDlg)
    {
        CsvExportDlg->setWindowTitle(tr2i18n("Csv Export", nullptr));
        TextLabel2->setText(tr2i18n("From this dialog you are able to export transactions to a comma separated file, (known as a CSV file, because of the extension).  However, you are able to specify an alternative separator below.  Please enter the path to the CSV file or select it by clicking on the Browse button.\n"
"\n"
"You can choose the file's path, and the account. Choose Account to export all the transactions between the specified dates,  or just choose to export Categories.", nullptr));
        lblFileExport->setText(tr2i18n("File to export to:", nullptr));
        m_qbuttonBrowse->setText(tr2i18n("&Browse...", nullptr));
        label->setText(tr2i18n("Account to export", nullptr));
        m_qgroupboxContents->setTitle(tr2i18n("Contents to Export", nullptr));
        m_radioButtonAccount->setText(tr2i18n("Acco&unt", nullptr));
        m_radioButtonCategories->setText(tr2i18n("Ca&tegories", nullptr));
        m_qgroupboxDates->setTitle(tr2i18n("Date Range", nullptr));
        lblStartDate->setText(tr2i18n("Start on:", nullptr));
        lblEndDate->setText(tr2i18n("End on:", nullptr));
        m_qgroupboxSeparator->setTitle(tr2i18n("Field Separator", nullptr));
        m_qbuttonOk->setText(tr2i18n("&Export", nullptr));
        m_qbuttonCancel->setText(tr2i18n("Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CsvExportDlg: public Ui_CsvExportDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CSVEXPORTDLG_H

