/********************************************************************************
** Form generated from reading UI file 'accountsettings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ACCOUNTSETTINGS_H
#define UI_ACCOUNTSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_AccountSettings
{
public:
    QVBoxLayout *verticalLayout_3;
    QGroupBox *WoobMain;
    QGridLayout *gridLayout;
    QLineEdit *id;
    QLabel *label;
    QLineEdit *backend;
    QLineEdit *max_history;
    QLabel *label_2;
    QLabel *label_3;

    void setupUi(QWidget *AccountSettings)
    {
        if (AccountSettings->objectName().isEmpty())
            AccountSettings->setObjectName("AccountSettings");
        AccountSettings->resize(561, 154);
        verticalLayout_3 = new QVBoxLayout(AccountSettings);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        WoobMain = new QGroupBox(AccountSettings);
        WoobMain->setObjectName("WoobMain");
        WoobMain->setFlat(false);
        gridLayout = new QGridLayout(WoobMain);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        id = new QLineEdit(WoobMain);
        id->setObjectName("id");

        gridLayout->addWidget(id, 0, 1, 1, 1);

        label = new QLabel(WoobMain);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        backend = new QLineEdit(WoobMain);
        backend->setObjectName("backend");

        gridLayout->addWidget(backend, 1, 1, 1, 1);

        max_history = new QLineEdit(WoobMain);
        max_history->setObjectName("max_history");

        gridLayout->addWidget(max_history, 2, 1, 1, 1);

        label_2 = new QLabel(WoobMain);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        label_3 = new QLabel(WoobMain);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 2, 0, 1, 1);


        verticalLayout_3->addWidget(WoobMain);


        retranslateUi(AccountSettings);

        QMetaObject::connectSlotsByName(AccountSettings);
    } // setupUi

    void retranslateUi(QWidget *AccountSettings)
    {
        label->setText(tr2i18n("ID", nullptr));
        label_2->setText(tr2i18n("Backend", nullptr));
        label_3->setText(tr2i18n("History", nullptr));
        (void)AccountSettings;
    } // retranslateUi

};

namespace Ui {
    class AccountSettings: public Ui_AccountSettings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // ACCOUNTSETTINGS_H

