/********************************************************************************
** Form generated from reading UI file 'kloandetailspage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KLOANDETAILSPAGE_H
#define UI_KLOANDETAILSPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <amountedit.h>
#include <kcombobox.h>
#include <klocalizedstring.h>
#include "kmymoneyfrequencycombo.h"

QT_BEGIN_NAMESPACE

class Ui_KLoanDetailsPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel2;
    QSpacerItem *spacer89;
    QHBoxLayout *hboxLayout;
    QGridLayout *gridLayout;
    QLabel *textLabel7;
    KComboBox *m_paymentDue;
    QLabel *textLabel8;
    AmountEdit *m_loanAmount;
    QLabel *textLabel9;
    AmountEdit *m_interestRate;
    QLabel *textLabel10;
    QHBoxLayout *hboxLayout1;
    QSpinBox *m_termAmount;
    KMyMoneyFrequencyCombo *m_termUnit;
    QLabel *textLabel11;
    AmountEdit *m_paymentAmount;
    QLabel *textLabel12;
    AmountEdit *m_balloonAmount;
    QHBoxLayout *hboxLayout2;
    QSpacerItem *spacer86;
    QPushButton *m_calculateButton;
    QSpacerItem *spacer88_2;
    QSpacerItem *spacer87;
    QSpacerItem *spacer88;

    void setupUi(QWidget *KLoanDetailsPage)
    {
        if (KLoanDetailsPage->objectName().isEmpty())
            KLoanDetailsPage->setObjectName("KLoanDetailsPage");
        KLoanDetailsPage->resize(592, 434);
        vboxLayout = new QVBoxLayout(KLoanDetailsPage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        qLabel2 = new QLabel(KLoanDetailsPage);
        qLabel2->setObjectName("qLabel2");
        qLabel2->setFocusPolicy(Qt::NoFocus);
        qLabel2->setWordWrap(true);

        vboxLayout->addWidget(qLabel2);

        spacer89 = new QSpacerItem(20, 70, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer89);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        textLabel7 = new QLabel(KLoanDetailsPage);
        textLabel7->setObjectName("textLabel7");
        textLabel7->setWordWrap(false);

        gridLayout->addWidget(textLabel7, 0, 0, 1, 1);

        m_paymentDue = new KComboBox(KLoanDetailsPage);
        m_paymentDue->addItem(QString());
        m_paymentDue->addItem(QString());
        m_paymentDue->setObjectName("m_paymentDue");

        gridLayout->addWidget(m_paymentDue, 0, 1, 1, 3);

        textLabel8 = new QLabel(KLoanDetailsPage);
        textLabel8->setObjectName("textLabel8");
        textLabel8->setWordWrap(false);

        gridLayout->addWidget(textLabel8, 1, 0, 1, 1);

        m_loanAmount = new AmountEdit(KLoanDetailsPage);
        m_loanAmount->setObjectName("m_loanAmount");
        m_loanAmount->setProperty("resetButtonVisibility", QVariant(true));

        gridLayout->addWidget(m_loanAmount, 1, 1, 1, 3);

        textLabel9 = new QLabel(KLoanDetailsPage);
        textLabel9->setObjectName("textLabel9");
        textLabel9->setWordWrap(false);

        gridLayout->addWidget(textLabel9, 2, 0, 1, 1);

        m_interestRate = new AmountEdit(KLoanDetailsPage);
        m_interestRate->setObjectName("m_interestRate");

        gridLayout->addWidget(m_interestRate, 2, 1, 1, 3);

        textLabel10 = new QLabel(KLoanDetailsPage);
        textLabel10->setObjectName("textLabel10");
        textLabel10->setWordWrap(false);

        gridLayout->addWidget(textLabel10, 3, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        m_termAmount = new QSpinBox(KLoanDetailsPage);
        m_termAmount->setObjectName("m_termAmount");
        m_termAmount->setMaximum(999);

        hboxLayout1->addWidget(m_termAmount);

        m_termUnit = new KMyMoneyFrequencyCombo(KLoanDetailsPage);
        m_termUnit->setObjectName("m_termUnit");

        hboxLayout1->addWidget(m_termUnit);


        gridLayout->addLayout(hboxLayout1, 3, 1, 1, 3);

        textLabel11 = new QLabel(KLoanDetailsPage);
        textLabel11->setObjectName("textLabel11");
        textLabel11->setWordWrap(false);

        gridLayout->addWidget(textLabel11, 4, 0, 1, 1);

        m_paymentAmount = new AmountEdit(KLoanDetailsPage);
        m_paymentAmount->setObjectName("m_paymentAmount");

        gridLayout->addWidget(m_paymentAmount, 4, 1, 1, 3);

        textLabel12 = new QLabel(KLoanDetailsPage);
        textLabel12->setObjectName("textLabel12");
        textLabel12->setWordWrap(false);

        gridLayout->addWidget(textLabel12, 5, 0, 1, 1);

        m_balloonAmount = new AmountEdit(KLoanDetailsPage);
        m_balloonAmount->setObjectName("m_balloonAmount");

        gridLayout->addWidget(m_balloonAmount, 5, 1, 1, 3);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        spacer86 = new QSpacerItem(71, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout2->addItem(spacer86);

        m_calculateButton = new QPushButton(KLoanDetailsPage);
        m_calculateButton->setObjectName("m_calculateButton");

        hboxLayout2->addWidget(m_calculateButton);


        gridLayout->addLayout(hboxLayout2, 6, 1, 1, 1);

        spacer88_2 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacer88_2, 6, 2, 1, 1);


        hboxLayout->addLayout(gridLayout);

        spacer87 = new QSpacerItem(27, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer87);


        vboxLayout->addLayout(hboxLayout);

        spacer88 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer88);


        retranslateUi(KLoanDetailsPage);

        QMetaObject::connectSlotsByName(KLoanDetailsPage);
    } // setupUi

    void retranslateUi(QWidget *KLoanDetailsPage)
    {
        qLabel2->setText(tr2i18n("Now enter the details for your loan. You can leave one of the fields empty and KMyMoney will calculate it when you press the <b>Calculate</b> button. Before you can continue with the next page you also need to press <b>Calculate</b> so that KMyMoney can check the logical correctness of the values you have supplied.", nullptr));
        textLabel7->setText(tr2i18n("The interest rate gets calculated", nullptr));
        m_paymentDue->setItemText(0, tr2i18n("when the payment is received", nullptr));
        m_paymentDue->setItemText(1, tr2i18n("when the payment is due", nullptr));

        textLabel8->setText(tr2i18n("Loan amount", nullptr));
#if QT_CONFIG(whatsthis)
        m_loanAmount->setProperty("whatsThis", QVariant(tr2i18n("Enter the amount of the loan in this field. If you want to calculate this field out of the other parameters, please leave it empty. If the field is marked as required (colored background) you have informed KMyMoney about the fact that there were already some payments towards the loan. In this case, please enter the ending balance of your last statement.", nullptr)));
#endif // QT_CONFIG(whatsthis)
        textLabel9->setText(tr2i18n("Interest rate", nullptr));
#if QT_CONFIG(whatsthis)
        m_interestRate->setProperty("whatsThis", QVariant(tr2i18n("Please enter the interest rate (as a percentage) or leave the field empty to calculate it.", nullptr)));
#endif // QT_CONFIG(whatsthis)
        textLabel10->setText(tr2i18n("Term", nullptr));
#if QT_CONFIG(whatsthis)
        m_termAmount->setProperty("whatsThis", QVariant(tr2i18n("Please enter the term of this loan or enter 0 to calculate it. The term is the time that is required to fully repay the loan. This time might be different from the time your loan contract is signed for.", nullptr)));
#endif // QT_CONFIG(whatsthis)
        textLabel11->setText(tr2i18n("Payment (principal and interest)", nullptr));
#if QT_CONFIG(whatsthis)
        m_paymentAmount->setProperty("whatsThis", QVariant(tr2i18n("Please enter the amount you pay for principal and interest or leave the field empty to calculate it.", nullptr)));
#endif // QT_CONFIG(whatsthis)
        textLabel12->setText(tr2i18n("Balloon payment", nullptr));
#if QT_CONFIG(whatsthis)
        m_balloonAmount->setProperty("whatsThis", QVariant(tr2i18n("Please enter the amount of a final amortization payment or leave the field empty to calculate it.", nullptr)));
#endif // QT_CONFIG(whatsthis)
        m_calculateButton->setText(tr2i18n("Calculate", nullptr));
#if QT_CONFIG(tooltip)
        m_calculateButton->setProperty("toolTip", QVariant(tr2i18n("Press this button to calculate/verify your loan details.", nullptr)));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_calculateButton->setProperty("whatsThis", QVariant(tr2i18n("Pressing this button calculates a possibly missing parameter for your loan or verifies that the values entered match. If something is not correct you will receive information about it.", nullptr)));
#endif // QT_CONFIG(whatsthis)
        (void)KLoanDetailsPage;
    } // retranslateUi

};

namespace Ui {
    class KLoanDetailsPage: public Ui_KLoanDetailsPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KLOANDETAILSPAGE_H

