/********************************************************************************
** Form generated from reading UI file 'newintrowizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NEWINTROWIZARDPAGE_H
#define UI_NEWINTROWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_NewIntroWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QSpacerItem *Spacer2;
    QLabel *TextLabel1_1;
    QSpacerItem *Spacer3;
    QLabel *TextLabel1_2;
    QSpacerItem *Spacer4;

    void setupUi(QWizardPage *NewIntroWizardPage)
    {
        if (NewIntroWizardPage->objectName().isEmpty())
            NewIntroWizardPage->setObjectName("NewIntroWizardPage");
        vboxLayout = new QVBoxLayout(NewIntroWizardPage);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName("hboxLayout");
        Spacer2 = new QSpacerItem(98, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer2);

        TextLabel1_1 = new QLabel(NewIntroWizardPage);
        TextLabel1_1->setObjectName("TextLabel1_1");
        QFont font;
        font.setFamilies({QString::fromUtf8("Helvetica")});
        font.setPointSize(16);
        font.setBold(true);
        TextLabel1_1->setFont(font);
        TextLabel1_1->setAlignment(Qt::AlignTop|Qt::AlignHCenter);
        TextLabel1_1->setWordWrap(false);

        hboxLayout->addWidget(TextLabel1_1);

        Spacer3 = new QSpacerItem(119, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer3);


        vboxLayout->addLayout(hboxLayout);

        TextLabel1_2 = new QLabel(NewIntroWizardPage);
        TextLabel1_2->setObjectName("TextLabel1_2");
        TextLabel1_2->setAlignment(Qt::AlignTop);
        TextLabel1_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel1_2);

        Spacer4 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer4);


        retranslateUi(NewIntroWizardPage);

        QMetaObject::connectSlotsByName(NewIntroWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *NewIntroWizardPage)
    {
        TextLabel1_1->setText(tr2i18n("New Loan Account Wizard", nullptr));
        TextLabel1_2->setText(tr2i18n("\n"
"Welcome to the New Loan Account Wizard which will guide you through the creation of a new loan e.g. for your home, your car or any other loan that you pay or gain interest for.\n"
"\n"
"Please make sure that you have the relevant information handy. You usually get the information out of your contract and the last statement.", nullptr));
        (void)NewIntroWizardPage;
    } // retranslateUi

};

namespace Ui {
    class NewIntroWizardPage: public Ui_NewIntroWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // NEWINTROWIZARDPAGE_H

