// This file is generated by kconfig_compiler_kf6 from xmlstoragesettings.kcfg.
// All changes you do to this file will be lost.
#ifndef XMLSTORAGESETTINGS_H
#define XMLSTORAGESETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class XMLStorageSettings : public KConfigSkeleton
{
  public:
    static XMLStorageSettings *self();
    ~XMLStorageSettings() override;

  protected:
    XMLStorageSettings();
    friend class XMLStorageSettingsHelper;

};

#endif

