/********************************************************************************
** Form generated from reading UI file 'reporttabrowcolquery.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REPORTTABROWCOLQUERY_H
#define UI_REPORTTABROWCOLQUERY_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"

QT_BEGIN_NAMESPACE

class Ui_ReportTabRowColQuery
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel3;
    KComboBox *m_comboOrganizeBy;
    QSpacerItem *spacer10_2;
    QHBoxLayout *hboxLayout1;
    QGroupBox *buttonGroupBox1;
    QGridLayout *gridLayout;
    QCheckBox *m_checkMemo;
    QCheckBox *m_checkShares;
    QCheckBox *m_checkPrice;
    QCheckBox *m_checkReconciled;
    QCheckBox *m_checkAccount;
    QCheckBox *m_checkNumber;
    QCheckBox *m_checkTag;
    QCheckBox *m_checkPayee;
    QCheckBox *m_checkCategory;
    QCheckBox *m_checkAction;
    QCheckBox *m_checkBalance;
    QSpacerItem *spacer10;
    QGridLayout *gridLayout1;
    QCheckBox *m_checkLoans;
    QCheckBox *m_checkInvestments;
    QCheckBox *m_checkHideSplitDetails;
    QCheckBox *m_checkTax;
    QCheckBox *m_checkHideTransactions;
    QCheckBox *m_checkHideTotals;
    QCheckBox *m_checkTransfers;
    QSpacerItem *spacer4;
    QButtonGroup *buttonGroup1;

    void setupUi(QWidget *ReportTabRowColQuery)
    {
        if (ReportTabRowColQuery->objectName().isEmpty())
            ReportTabRowColQuery->setObjectName("ReportTabRowColQuery");
        ReportTabRowColQuery->resize(730, 267);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ReportTabRowColQuery->sizePolicy().hasHeightForWidth());
        ReportTabRowColQuery->setSizePolicy(sizePolicy);
        vboxLayout = new QVBoxLayout(ReportTabRowColQuery);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        textLabel3 = new QLabel(ReportTabRowColQuery);
        textLabel3->setObjectName("textLabel3");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textLabel3->sizePolicy().hasHeightForWidth());
        textLabel3->setSizePolicy(sizePolicy1);
        QFont font;
        textLabel3->setFont(font);
        textLabel3->setWordWrap(false);

        hboxLayout->addWidget(textLabel3);

        m_comboOrganizeBy = new KComboBox(ReportTabRowColQuery);
        m_comboOrganizeBy->addItem(QString());
        m_comboOrganizeBy->addItem(QString());
        m_comboOrganizeBy->addItem(QString());
        m_comboOrganizeBy->addItem(QString());
        m_comboOrganizeBy->addItem(QString());
        m_comboOrganizeBy->addItem(QString());
        m_comboOrganizeBy->addItem(QString());
        m_comboOrganizeBy->addItem(QString());
        m_comboOrganizeBy->setObjectName("m_comboOrganizeBy");
        m_comboOrganizeBy->setEnabled(true);
        sizePolicy1.setHeightForWidth(m_comboOrganizeBy->sizePolicy().hasHeightForWidth());
        m_comboOrganizeBy->setSizePolicy(sizePolicy1);

        hboxLayout->addWidget(m_comboOrganizeBy);

        spacer10_2 = new QSpacerItem(358, 16, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer10_2);


        vboxLayout->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        buttonGroupBox1 = new QGroupBox(ReportTabRowColQuery);
        buttonGroupBox1->setObjectName("buttonGroupBox1");
        buttonGroupBox1->setEnabled(true);
        sizePolicy1.setHeightForWidth(buttonGroupBox1->sizePolicy().hasHeightForWidth());
        buttonGroupBox1->setSizePolicy(sizePolicy1);
        gridLayout = new QGridLayout(buttonGroupBox1);
        gridLayout->setSpacing(2);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        m_checkMemo = new QCheckBox(buttonGroupBox1);
        buttonGroup1 = new QButtonGroup(ReportTabRowColQuery);
        buttonGroup1->setObjectName("buttonGroup1");
        buttonGroup1->addButton(m_checkMemo);
        m_checkMemo->setObjectName("m_checkMemo");
        m_checkMemo->setMinimumSize(QSize(0, 0));

        gridLayout->addWidget(m_checkMemo, 1, 1, 1, 1);

        m_checkShares = new QCheckBox(buttonGroupBox1);
        buttonGroup1->addButton(m_checkShares);
        m_checkShares->setObjectName("m_checkShares");

        gridLayout->addWidget(m_checkShares, 0, 4, 1, 1);

        m_checkPrice = new QCheckBox(buttonGroupBox1);
        buttonGroup1->addButton(m_checkPrice);
        m_checkPrice->setObjectName("m_checkPrice");

        gridLayout->addWidget(m_checkPrice, 1, 3, 1, 1);

        m_checkReconciled = new QCheckBox(buttonGroupBox1);
        buttonGroup1->addButton(m_checkReconciled);
        m_checkReconciled->setObjectName("m_checkReconciled");

        gridLayout->addWidget(m_checkReconciled, 1, 2, 1, 1);

        m_checkAccount = new QCheckBox(buttonGroupBox1);
        buttonGroup1->addButton(m_checkAccount);
        m_checkAccount->setObjectName("m_checkAccount");

        gridLayout->addWidget(m_checkAccount, 0, 3, 1, 1);

        m_checkNumber = new QCheckBox(buttonGroupBox1);
        buttonGroup1->addButton(m_checkNumber);
        m_checkNumber->setObjectName("m_checkNumber");
        m_checkNumber->setMinimumSize(QSize(0, 0));

        gridLayout->addWidget(m_checkNumber, 0, 0, 1, 1);

        m_checkTag = new QCheckBox(buttonGroupBox1);
        buttonGroup1->addButton(m_checkTag);
        m_checkTag->setObjectName("m_checkTag");
        m_checkTag->setMinimumSize(QSize(0, 0));

        gridLayout->addWidget(m_checkTag, 0, 2, 1, 1);

        m_checkPayee = new QCheckBox(buttonGroupBox1);
        buttonGroup1->addButton(m_checkPayee);
        m_checkPayee->setObjectName("m_checkPayee");
        m_checkPayee->setMinimumSize(QSize(0, 0));

        gridLayout->addWidget(m_checkPayee, 1, 0, 1, 1);

        m_checkCategory = new QCheckBox(buttonGroupBox1);
        buttonGroup1->addButton(m_checkCategory);
        m_checkCategory->setObjectName("m_checkCategory");
        m_checkCategory->setMinimumSize(QSize(0, 0));

        gridLayout->addWidget(m_checkCategory, 0, 1, 1, 1);

        m_checkAction = new QCheckBox(buttonGroupBox1);
        buttonGroup1->addButton(m_checkAction);
        m_checkAction->setObjectName("m_checkAction");

        gridLayout->addWidget(m_checkAction, 0, 5, 1, 1);

        m_checkBalance = new QCheckBox(buttonGroupBox1);
        buttonGroup1->addButton(m_checkBalance);
        m_checkBalance->setObjectName("m_checkBalance");

        gridLayout->addWidget(m_checkBalance, 1, 4, 1, 1);


        hboxLayout1->addWidget(buttonGroupBox1);

        spacer10 = new QSpacerItem(205, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacer10);


        vboxLayout->addLayout(hboxLayout1);

        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setObjectName("gridLayout1");
        m_checkLoans = new QCheckBox(ReportTabRowColQuery);
        m_checkLoans->setObjectName("m_checkLoans");

        gridLayout1->addWidget(m_checkLoans, 2, 0, 1, 1);

        m_checkInvestments = new QCheckBox(ReportTabRowColQuery);
        m_checkInvestments->setObjectName("m_checkInvestments");

        gridLayout1->addWidget(m_checkInvestments, 1, 0, 1, 1);

        m_checkHideSplitDetails = new QCheckBox(ReportTabRowColQuery);
        m_checkHideSplitDetails->setObjectName("m_checkHideSplitDetails");

        gridLayout1->addWidget(m_checkHideSplitDetails, 0, 1, 1, 1);

        m_checkTax = new QCheckBox(ReportTabRowColQuery);
        m_checkTax->setObjectName("m_checkTax");

        gridLayout1->addWidget(m_checkTax, 0, 0, 1, 1);

        m_checkHideTransactions = new QCheckBox(ReportTabRowColQuery);
        m_checkHideTransactions->setObjectName("m_checkHideTransactions");

        gridLayout1->addWidget(m_checkHideTransactions, 1, 1, 1, 1);

        m_checkHideTotals = new QCheckBox(ReportTabRowColQuery);
        m_checkHideTotals->setObjectName("m_checkHideTotals");

        gridLayout1->addWidget(m_checkHideTotals, 2, 1, 1, 1);

        m_checkTransfers = new QCheckBox(ReportTabRowColQuery);
        m_checkTransfers->setObjectName("m_checkTransfers");

        gridLayout1->addWidget(m_checkTransfers, 3, 0, 1, 1);


        vboxLayout->addLayout(gridLayout1);

        spacer4 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer4);

        QWidget::setTabOrder(m_comboOrganizeBy, m_checkNumber);
        QWidget::setTabOrder(m_checkNumber, m_checkCategory);
        QWidget::setTabOrder(m_checkCategory, m_checkTag);
        QWidget::setTabOrder(m_checkTag, m_checkAccount);
        QWidget::setTabOrder(m_checkAccount, m_checkShares);
        QWidget::setTabOrder(m_checkShares, m_checkAction);
        QWidget::setTabOrder(m_checkAction, m_checkPayee);
        QWidget::setTabOrder(m_checkPayee, m_checkMemo);
        QWidget::setTabOrder(m_checkMemo, m_checkReconciled);
        QWidget::setTabOrder(m_checkReconciled, m_checkPrice);
        QWidget::setTabOrder(m_checkPrice, m_checkBalance);
        QWidget::setTabOrder(m_checkBalance, m_checkTax);
        QWidget::setTabOrder(m_checkTax, m_checkInvestments);
        QWidget::setTabOrder(m_checkInvestments, m_checkLoans);
        QWidget::setTabOrder(m_checkLoans, m_checkHideSplitDetails);
        QWidget::setTabOrder(m_checkHideSplitDetails, m_checkHideTransactions);
        QWidget::setTabOrder(m_checkHideTransactions, m_checkHideTotals);

        retranslateUi(ReportTabRowColQuery);
        QObject::connect(m_checkTax, &QCheckBox::toggled, m_checkInvestments, &QCheckBox::setDisabled);
        QObject::connect(m_checkTax, &QCheckBox::toggled, m_checkLoans, &QCheckBox::setDisabled);
        QObject::connect(m_checkInvestments, &QCheckBox::toggled, m_checkTax, &QCheckBox::setDisabled);
        QObject::connect(m_checkInvestments, &QCheckBox::toggled, m_checkLoans, &QCheckBox::setDisabled);
        QObject::connect(m_checkLoans, &QCheckBox::toggled, m_checkTax, &QCheckBox::setDisabled);
        QObject::connect(m_checkLoans, &QCheckBox::toggled, m_checkInvestments, &QCheckBox::setDisabled);

        QMetaObject::connectSlotsByName(ReportTabRowColQuery);
    } // setupUi

    void retranslateUi(QWidget *ReportTabRowColQuery)
    {
        ReportTabRowColQuery->setWindowTitle(tr2i18n("Rows/Columns Tab", nullptr));
#if QT_CONFIG(tooltip)
        ReportTabRowColQuery->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        ReportTabRowColQuery->setWhatsThis(tr2i18n("<p>On this tab, you configure how you would like the rows and columns to be selected and organized.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel3->setText(tr2i18n("Organize by:", nullptr));
        m_comboOrganizeBy->setItemText(0, tr2i18n("Categories", nullptr));
        m_comboOrganizeBy->setItemText(1, tr2i18n("Top Categories", nullptr));
        m_comboOrganizeBy->setItemText(2, tr2i18n("Tags", nullptr));
        m_comboOrganizeBy->setItemText(3, tr2i18n("Payees", nullptr));
        m_comboOrganizeBy->setItemText(4, tr2i18n("Accounts", nullptr));
        m_comboOrganizeBy->setItemText(5, tr2i18n("Top Accounts", nullptr));
        m_comboOrganizeBy->setItemText(6, tr2i18n("Month", nullptr));
        m_comboOrganizeBy->setItemText(7, tr2i18n("Week", nullptr));

#if QT_CONFIG(tooltip)
        m_comboOrganizeBy->setToolTip(tr2i18n("<p>Choose how to group the transactions in this report</p>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonGroupBox1->setToolTip(tr2i18n("<p>Choose which columns should be shown in the report.</p><p>The date and transaction amount are always shown.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonGroupBox1->setTitle(tr2i18n("Show Columns", nullptr));
#if QT_CONFIG(tooltip)
        m_checkMemo->setToolTip(tr2i18n("<p>Select this option to show the Memo column</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkMemo->setText(tr2i18n("Memo", nullptr));
#if QT_CONFIG(tooltip)
        m_checkShares->setToolTip(tr2i18n("<p>Select this option to show the Shares column for investments</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkShares->setText(tr2i18n("Shares", nullptr));
#if QT_CONFIG(tooltip)
        m_checkPrice->setToolTip(tr2i18n("<p>Select this option to show the Price column for investments</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkPrice->setText(tr2i18n("Price", nullptr));
#if QT_CONFIG(tooltip)
        m_checkReconciled->setToolTip(tr2i18n("<p>Select this option to show the Reconciled column</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkReconciled->setText(tr2i18n("Reconciled", nullptr));
#if QT_CONFIG(tooltip)
        m_checkAccount->setToolTip(tr2i18n("<p>Select this option to show the Account column</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkAccount->setText(tr2i18n("Account", nullptr));
#if QT_CONFIG(tooltip)
        m_checkNumber->setToolTip(tr2i18n("<p>Select this option to show the Number column</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkNumber->setText(tr2i18n("Number", nullptr));
#if QT_CONFIG(tooltip)
        m_checkTag->setToolTip(tr2i18n("<p>Select this option to show the Tag column</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkTag->setText(tr2i18n("Tag", nullptr));
#if QT_CONFIG(tooltip)
        m_checkPayee->setToolTip(tr2i18n("<p>Select this option to show the Payee column</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkPayee->setText(tr2i18n("Payee", nullptr));
#if QT_CONFIG(tooltip)
        m_checkCategory->setToolTip(tr2i18n("<p>Select this option to show the Category column</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkCategory->setText(tr2i18n("Category", nullptr));
#if QT_CONFIG(tooltip)
        m_checkAction->setToolTip(tr2i18n("<p>Select this option to show the Action column</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkAction->setText(tr2i18n("Action", nullptr));
#if QT_CONFIG(tooltip)
        m_checkBalance->setToolTip(tr2i18n("<p>Select this option to show the Running balance column</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkBalance->setText(tr2i18n("Balance", nullptr));
        m_checkLoans->setText(tr2i18n("Include only Loan accounts", nullptr));
#if QT_CONFIG(tooltip)
        m_checkInvestments->setToolTip(tr2i18n("<p>Include only data from Investment accounts in this report</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkInvestments->setText(tr2i18n("Include only Investment accounts", nullptr));
#if QT_CONFIG(tooltip)
        m_checkHideSplitDetails->setToolTip(tr2i18n("Do not display the individual transactions that make up a split transaction", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkHideSplitDetails->setText(tr2i18n("Hide Split Transaction Details", nullptr));
#if QT_CONFIG(tooltip)
        m_checkTax->setToolTip(tr2i18n("<p>Check this box to include only those categories which have been marked to \"Include on Tax Reports\"</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkTax->setText(tr2i18n("Include only Tax categories", nullptr));
#if QT_CONFIG(tooltip)
        m_checkHideTransactions->setToolTip(tr2i18n("Do not display the transactions, leaving only totals displayed.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkHideTransactions->setText(tr2i18n("Hide Transactions", nullptr));
        m_checkHideTotals->setText(tr2i18n("Hide Totals", nullptr));
        m_checkTransfers->setText(tr2i18n("Include transfers", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ReportTabRowColQuery: public Ui_ReportTabRowColQuery {};
} // namespace Ui

QT_END_NAMESPACE

#endif // REPORTTABROWCOLQUERY_H

