/********************************************************************************
** Form generated from reading UI file 'kreportconfigurationfilterdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KREPORTCONFIGURATIONFILTERDLG_H
#define UI_KREPORTCONFIGURATIONFILTERDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KReportConfigurationFilterDlg
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *m_tabWidget;
    QWidget *m_reportPage;
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel1;
    QTabWidget *m_criteriaTab;
    QSpacerItem *spacer16;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KReportConfigurationFilterDlg)
    {
        if (KReportConfigurationFilterDlg->objectName().isEmpty())
            KReportConfigurationFilterDlg->setObjectName("KReportConfigurationFilterDlg");
        KReportConfigurationFilterDlg->resize(771, 481);
        KReportConfigurationFilterDlg->setSizeGripEnabled(true);
        verticalLayout = new QVBoxLayout(KReportConfigurationFilterDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        m_tabWidget = new QTabWidget(KReportConfigurationFilterDlg);
        m_tabWidget->setObjectName("m_tabWidget");
        m_reportPage = new QWidget();
        m_reportPage->setObjectName("m_reportPage");
        vboxLayout = new QVBoxLayout(m_reportPage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel1 = new QLabel(m_reportPage);
        TextLabel1->setObjectName("TextLabel1");
        TextLabel1->setWordWrap(false);

        vboxLayout->addWidget(TextLabel1);

        m_criteriaTab = new QTabWidget(m_reportPage);
        m_criteriaTab->setObjectName("m_criteriaTab");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(3);
        sizePolicy.setHeightForWidth(m_criteriaTab->sizePolicy().hasHeightForWidth());
        m_criteriaTab->setSizePolicy(sizePolicy);

        vboxLayout->addWidget(m_criteriaTab);

        spacer16 = new QSpacerItem(20, 24, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer16);

        m_tabWidget->addTab(m_reportPage, QString());

        verticalLayout->addWidget(m_tabWidget);

        buttonBox = new QDialogButtonBox(KReportConfigurationFilterDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Close|QDialogButtonBox::Help|QDialogButtonBox::Reset);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(m_tabWidget, m_criteriaTab);

        retranslateUi(KReportConfigurationFilterDlg);

        m_tabWidget->setCurrentIndex(0);
        m_criteriaTab->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(KReportConfigurationFilterDlg);
    } // setupUi

    void retranslateUi(QDialog *KReportConfigurationFilterDlg)
    {
        KReportConfigurationFilterDlg->setWindowTitle(tr2i18n("Search transactions", nullptr));
        TextLabel1->setText(tr2i18n("Configure your report", nullptr));
        m_tabWidget->setTabText(m_tabWidget->indexOf(m_reportPage), tr2i18n("Report", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KReportConfigurationFilterDlg: public Ui_KReportConfigurationFilterDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KREPORTCONFIGURATIONFILTERDLG_H

