/********************************************************************************
** Form generated from reading UI file 'summarywizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SUMMARYWIZARDPAGE_H
#define UI_SUMMARYWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SummaryWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel22_2_2;
    QSpacerItem *Spacer69;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout1;
    QGroupBox *GroupBox3;
    QVBoxLayout *vboxLayout2;
    QGridLayout *gridLayout;
    QLabel *TextLabel5_3;
    QLabel *m_summaryLoanType;
    QLabel *m_summaryPayee;
    QLabel *TextLabel4_4;
    QLabel *m_summaryFirstPayment;
    QLabel *TextLabel3_5;
    QGroupBox *GroupBox2;
    QVBoxLayout *vboxLayout3;
    QGridLayout *gridLayout1;
    QLabel *m_summaryTotalPeriodicPayment;
    QLabel *m_summaryPaymentAccount;
    QLabel *m_summaryAdditionalFees;
    QLabel *TextLabel15;
    QLabel *TextLabel14;
    QLabel *TextLabel13;
    QLabel *TextLabel17;
    QLabel *m_summaryInterestCategory;
    QLabel *m_summaryNextPayment;
    QLabel *TextLabel16_3;
    QGroupBox *GroupBox1;
    QVBoxLayout *vboxLayout4;
    QGridLayout *gridLayout2;
    QLabel *m_summaryInterestRate;
    QLabel *TextLabel10_3;
    QLabel *m_summaryPeriodicPayment;
    QLabel *TextLabel9_3;
    QLabel *m_summaryPaymentFrequency;
    QLabel *TextLabel12;
    QLabel *m_summaryTerm;
    QLabel *TextLabel6_4;
    QLabel *m_summaryAmount;
    QLabel *TextLabel11_4;
    QLabel *m_summaryBalloonPayment;
    QLabel *TextLabel8_4;
    QLabel *TextLabel7;
    QLabel *m_summaryInterestDue;
    QSpacerItem *Spacer70;

    void setupUi(QWizardPage *SummaryWizardPage)
    {
        if (SummaryWizardPage->objectName().isEmpty())
            SummaryWizardPage->setObjectName("SummaryWizardPage");
        vboxLayout = new QVBoxLayout(SummaryWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel22_2_2 = new QLabel(SummaryWizardPage);
        TextLabel22_2_2->setObjectName("TextLabel22_2_2");
        TextLabel22_2_2->setAlignment(Qt::AlignTop);
        TextLabel22_2_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel22_2_2);

        Spacer69 = new QSpacerItem(0, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer69);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        vboxLayout1 = new QVBoxLayout();
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout1->setObjectName("vboxLayout1");
        GroupBox3 = new QGroupBox(SummaryWizardPage);
        GroupBox3->setObjectName("GroupBox3");
        vboxLayout2 = new QVBoxLayout(GroupBox3);
        vboxLayout2->setSpacing(1);
        vboxLayout2->setContentsMargins(1, 1, 1, 1);
        vboxLayout2->setObjectName("vboxLayout2");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(1);
        gridLayout->setContentsMargins(1, 1, 1, 1);
        gridLayout->setObjectName("gridLayout");
        TextLabel5_3 = new QLabel(GroupBox3);
        TextLabel5_3->setObjectName("TextLabel5_3");
        TextLabel5_3->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel5_3->setWordWrap(false);

        gridLayout->addWidget(TextLabel5_3, 2, 0, 1, 1);

        m_summaryLoanType = new QLabel(GroupBox3);
        m_summaryLoanType->setObjectName("m_summaryLoanType");
        QFont font;
        font.setBold(true);
        m_summaryLoanType->setFont(font);
        m_summaryLoanType->setTextFormat(Qt::PlainText);
        m_summaryLoanType->setWordWrap(false);

        gridLayout->addWidget(m_summaryLoanType, 0, 1, 1, 1);

        m_summaryPayee = new QLabel(GroupBox3);
        m_summaryPayee->setObjectName("m_summaryPayee");
        m_summaryPayee->setFont(font);
        m_summaryPayee->setTextFormat(Qt::PlainText);
        m_summaryPayee->setWordWrap(false);

        gridLayout->addWidget(m_summaryPayee, 2, 1, 1, 1);

        TextLabel4_4 = new QLabel(GroupBox3);
        TextLabel4_4->setObjectName("TextLabel4_4");
        TextLabel4_4->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel4_4->setWordWrap(false);

        gridLayout->addWidget(TextLabel4_4, 1, 0, 1, 1);

        m_summaryFirstPayment = new QLabel(GroupBox3);
        m_summaryFirstPayment->setObjectName("m_summaryFirstPayment");
        m_summaryFirstPayment->setFont(font);
        m_summaryFirstPayment->setTextFormat(Qt::PlainText);
        m_summaryFirstPayment->setWordWrap(false);

        gridLayout->addWidget(m_summaryFirstPayment, 1, 1, 1, 1);

        TextLabel3_5 = new QLabel(GroupBox3);
        TextLabel3_5->setObjectName("TextLabel3_5");
        TextLabel3_5->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel3_5->setWordWrap(false);

        gridLayout->addWidget(TextLabel3_5, 0, 0, 1, 1);


        vboxLayout2->addLayout(gridLayout);


        vboxLayout1->addWidget(GroupBox3);

        GroupBox2 = new QGroupBox(SummaryWizardPage);
        GroupBox2->setObjectName("GroupBox2");
        vboxLayout3 = new QVBoxLayout(GroupBox2);
        vboxLayout3->setSpacing(1);
        vboxLayout3->setContentsMargins(1, 1, 1, 1);
        vboxLayout3->setObjectName("vboxLayout3");
        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(1);
        gridLayout1->setContentsMargins(1, 1, 1, 1);
        gridLayout1->setObjectName("gridLayout1");
        m_summaryTotalPeriodicPayment = new QLabel(GroupBox2);
        m_summaryTotalPeriodicPayment->setObjectName("m_summaryTotalPeriodicPayment");
        m_summaryTotalPeriodicPayment->setFont(font);
        m_summaryTotalPeriodicPayment->setTextFormat(Qt::PlainText);
        m_summaryTotalPeriodicPayment->setWordWrap(false);

        gridLayout1->addWidget(m_summaryTotalPeriodicPayment, 3, 1, 1, 1);

        m_summaryPaymentAccount = new QLabel(GroupBox2);
        m_summaryPaymentAccount->setObjectName("m_summaryPaymentAccount");
        m_summaryPaymentAccount->setFont(font);
        m_summaryPaymentAccount->setTextFormat(Qt::PlainText);
        m_summaryPaymentAccount->setWordWrap(false);

        gridLayout1->addWidget(m_summaryPaymentAccount, 5, 1, 1, 1);

        m_summaryAdditionalFees = new QLabel(GroupBox2);
        m_summaryAdditionalFees->setObjectName("m_summaryAdditionalFees");
        m_summaryAdditionalFees->setFont(font);
        m_summaryAdditionalFees->setTextFormat(Qt::PlainText);
        m_summaryAdditionalFees->setWordWrap(false);

        gridLayout1->addWidget(m_summaryAdditionalFees, 2, 1, 1, 1);

        TextLabel15 = new QLabel(GroupBox2);
        TextLabel15->setObjectName("TextLabel15");
        TextLabel15->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel15->setWordWrap(false);

        gridLayout1->addWidget(TextLabel15, 3, 0, 1, 1);

        TextLabel14 = new QLabel(GroupBox2);
        TextLabel14->setObjectName("TextLabel14");
        TextLabel14->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel14->setWordWrap(false);

        gridLayout1->addWidget(TextLabel14, 1, 0, 2, 1);

        TextLabel13 = new QLabel(GroupBox2);
        TextLabel13->setObjectName("TextLabel13");
        TextLabel13->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel13->setWordWrap(false);

        gridLayout1->addWidget(TextLabel13, 0, 0, 1, 1);

        TextLabel17 = new QLabel(GroupBox2);
        TextLabel17->setObjectName("TextLabel17");
        TextLabel17->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel17->setWordWrap(false);

        gridLayout1->addWidget(TextLabel17, 5, 0, 1, 1);

        m_summaryInterestCategory = new QLabel(GroupBox2);
        m_summaryInterestCategory->setObjectName("m_summaryInterestCategory");
        m_summaryInterestCategory->setFont(font);
        m_summaryInterestCategory->setTextFormat(Qt::PlainText);
        m_summaryInterestCategory->setWordWrap(false);

        gridLayout1->addWidget(m_summaryInterestCategory, 0, 1, 2, 1);

        m_summaryNextPayment = new QLabel(GroupBox2);
        m_summaryNextPayment->setObjectName("m_summaryNextPayment");
        m_summaryNextPayment->setFont(font);
        m_summaryNextPayment->setTextFormat(Qt::PlainText);
        m_summaryNextPayment->setWordWrap(false);

        gridLayout1->addWidget(m_summaryNextPayment, 4, 1, 1, 1);

        TextLabel16_3 = new QLabel(GroupBox2);
        TextLabel16_3->setObjectName("TextLabel16_3");
        TextLabel16_3->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel16_3->setWordWrap(false);

        gridLayout1->addWidget(TextLabel16_3, 4, 0, 1, 1);


        vboxLayout3->addLayout(gridLayout1);


        vboxLayout1->addWidget(GroupBox2);


        hboxLayout->addLayout(vboxLayout1);

        GroupBox1 = new QGroupBox(SummaryWizardPage);
        GroupBox1->setObjectName("GroupBox1");
        vboxLayout4 = new QVBoxLayout(GroupBox1);
        vboxLayout4->setSpacing(1);
        vboxLayout4->setContentsMargins(1, 1, 1, 1);
        vboxLayout4->setObjectName("vboxLayout4");
        gridLayout2 = new QGridLayout();
        gridLayout2->setSpacing(1);
        gridLayout2->setContentsMargins(1, 1, 1, 1);
        gridLayout2->setObjectName("gridLayout2");
        m_summaryInterestRate = new QLabel(GroupBox1);
        m_summaryInterestRate->setObjectName("m_summaryInterestRate");
        m_summaryInterestRate->setFont(font);
        m_summaryInterestRate->setTextFormat(Qt::PlainText);
        m_summaryInterestRate->setWordWrap(false);

        gridLayout2->addWidget(m_summaryInterestRate, 3, 1, 1, 1);

        TextLabel10_3 = new QLabel(GroupBox1);
        TextLabel10_3->setObjectName("TextLabel10_3");
        TextLabel10_3->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel10_3->setWordWrap(false);

        gridLayout2->addWidget(TextLabel10_3, 4, 0, 1, 1);

        m_summaryPeriodicPayment = new QLabel(GroupBox1);
        m_summaryPeriodicPayment->setObjectName("m_summaryPeriodicPayment");
        m_summaryPeriodicPayment->setFont(font);
        m_summaryPeriodicPayment->setTextFormat(Qt::PlainText);
        m_summaryPeriodicPayment->setWordWrap(false);

        gridLayout2->addWidget(m_summaryPeriodicPayment, 5, 1, 1, 1);

        TextLabel9_3 = new QLabel(GroupBox1);
        TextLabel9_3->setObjectName("TextLabel9_3");
        TextLabel9_3->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel9_3->setWordWrap(false);

        gridLayout2->addWidget(TextLabel9_3, 3, 0, 1, 1);

        m_summaryPaymentFrequency = new QLabel(GroupBox1);
        m_summaryPaymentFrequency->setObjectName("m_summaryPaymentFrequency");
        m_summaryPaymentFrequency->setFont(font);
        m_summaryPaymentFrequency->setTextFormat(Qt::PlainText);
        m_summaryPaymentFrequency->setWordWrap(false);

        gridLayout2->addWidget(m_summaryPaymentFrequency, 1, 1, 1, 1);

        TextLabel12 = new QLabel(GroupBox1);
        TextLabel12->setObjectName("TextLabel12");
        TextLabel12->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel12->setWordWrap(false);

        gridLayout2->addWidget(TextLabel12, 6, 0, 1, 1);

        m_summaryTerm = new QLabel(GroupBox1);
        m_summaryTerm->setObjectName("m_summaryTerm");
        m_summaryTerm->setFont(font);
        m_summaryTerm->setTextFormat(Qt::PlainText);
        m_summaryTerm->setWordWrap(false);

        gridLayout2->addWidget(m_summaryTerm, 4, 1, 1, 1);

        TextLabel6_4 = new QLabel(GroupBox1);
        TextLabel6_4->setObjectName("TextLabel6_4");
        TextLabel6_4->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel6_4->setWordWrap(false);

        gridLayout2->addWidget(TextLabel6_4, 0, 0, 1, 1);

        m_summaryAmount = new QLabel(GroupBox1);
        m_summaryAmount->setObjectName("m_summaryAmount");
        m_summaryAmount->setFont(font);
        m_summaryAmount->setTextFormat(Qt::PlainText);
        m_summaryAmount->setWordWrap(false);

        gridLayout2->addWidget(m_summaryAmount, 2, 1, 1, 1);

        TextLabel11_4 = new QLabel(GroupBox1);
        TextLabel11_4->setObjectName("TextLabel11_4");
        TextLabel11_4->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel11_4->setWordWrap(false);

        gridLayout2->addWidget(TextLabel11_4, 5, 0, 1, 1);

        m_summaryBalloonPayment = new QLabel(GroupBox1);
        m_summaryBalloonPayment->setObjectName("m_summaryBalloonPayment");
        m_summaryBalloonPayment->setFont(font);
        m_summaryBalloonPayment->setTextFormat(Qt::PlainText);
        m_summaryBalloonPayment->setWordWrap(false);

        gridLayout2->addWidget(m_summaryBalloonPayment, 6, 1, 1, 1);

        TextLabel8_4 = new QLabel(GroupBox1);
        TextLabel8_4->setObjectName("TextLabel8_4");
        TextLabel8_4->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel8_4->setWordWrap(false);

        gridLayout2->addWidget(TextLabel8_4, 2, 0, 1, 1);

        TextLabel7 = new QLabel(GroupBox1);
        TextLabel7->setObjectName("TextLabel7");
        TextLabel7->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel7->setWordWrap(false);

        gridLayout2->addWidget(TextLabel7, 1, 0, 1, 1);

        m_summaryInterestDue = new QLabel(GroupBox1);
        m_summaryInterestDue->setObjectName("m_summaryInterestDue");
        m_summaryInterestDue->setFont(font);
        m_summaryInterestDue->setTextFormat(Qt::PlainText);
        m_summaryInterestDue->setWordWrap(false);

        gridLayout2->addWidget(m_summaryInterestDue, 0, 1, 1, 1);


        vboxLayout4->addLayout(gridLayout2);


        hboxLayout->addWidget(GroupBox1);


        vboxLayout->addLayout(hboxLayout);

        Spacer70 = new QSpacerItem(0, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer70);


        retranslateUi(SummaryWizardPage);

        QMetaObject::connectSlotsByName(SummaryWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *SummaryWizardPage)
    {
        TextLabel22_2_2->setText(tr2i18n("\n"
"This page summarizes the data you entered. If you need to modify anything, please use the \"Back\" button to go to respective page. Otherwise use the \"Finish\" button to create the account.", nullptr));
        GroupBox3->setTitle(tr2i18n("General", "@title loan general information"));
        TextLabel5_3->setText(tr2i18n("Payee:", nullptr));
        m_summaryLoanType->setText(tr2i18n("1", nullptr));
        m_summaryPayee->setText(tr2i18n("1", nullptr));
        TextLabel4_4->setText(tr2i18n("First payment:", nullptr));
        m_summaryFirstPayment->setText(tr2i18n("1", nullptr));
        TextLabel3_5->setText(tr2i18n("Amount is:", nullptr));
        GroupBox2->setTitle(tr2i18n("Loan calculation", nullptr));
        m_summaryTotalPeriodicPayment->setText(tr2i18n("1", nullptr));
        m_summaryPaymentAccount->setText(tr2i18n("1", nullptr));
        m_summaryAdditionalFees->setText(tr2i18n("1", nullptr));
        TextLabel15->setText(tr2i18n("Periodic Payment:", nullptr));
        TextLabel14->setText(tr2i18n("Additional Fees:", nullptr));
        TextLabel13->setText(tr2i18n("Interest category:", nullptr));
        TextLabel17->setText(tr2i18n("Payment from:", nullptr));
        m_summaryInterestCategory->setText(tr2i18n("1", nullptr));
        m_summaryNextPayment->setText(tr2i18n("1", nullptr));
        TextLabel16_3->setText(tr2i18n("Next due date:", nullptr));
        GroupBox1->setTitle(tr2i18n("Payment", nullptr));
        m_summaryInterestRate->setText(tr2i18n("1", nullptr));
        TextLabel10_3->setText(tr2i18n("Term:", nullptr));
        m_summaryPeriodicPayment->setText(tr2i18n("1", nullptr));
        TextLabel9_3->setText(tr2i18n("Interest rate:", nullptr));
        m_summaryPaymentFrequency->setText(tr2i18n("1", nullptr));
        TextLabel12->setText(tr2i18n("Final Payment:", nullptr));
        m_summaryTerm->setText(tr2i18n("1", nullptr));
        TextLabel6_4->setText(tr2i18n("Interest is due:", nullptr));
        m_summaryAmount->setText(tr2i18n("1", nullptr));
        TextLabel11_4->setText(tr2i18n("Principal + Interest:", nullptr));
        m_summaryBalloonPayment->setText(tr2i18n("1", nullptr));
        TextLabel8_4->setText(tr2i18n("Loan amount:", nullptr));
        TextLabel7->setText(tr2i18n("Payment frequency:", nullptr));
        m_summaryInterestDue->setText(tr2i18n("1", nullptr));
        (void)SummaryWizardPage;
    } // retranslateUi

};

namespace Ui {
    class SummaryWizardPage: public Ui_SummaryWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SUMMARYWIZARDPAGE_H

