/********************************************************************************
** Form generated from reading UI file 'durationwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DURATIONWIZARDPAGE_H
#define UI_DURATIONWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>
#include "kmymoneygeneralcombo.h"

QT_BEGIN_NAMESPACE

class Ui_DurationWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel15_2;
    QSpacerItem *Spacer67_2;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel2_3;
    QSpinBox *m_durationValueEdit;
    KMyMoneyGeneralCombo *m_durationUnitEdit;
    QFrame *Frame3_2_2;
    QGridLayout *gridLayout;
    QLabel *TextLabel6_3_2;
    QLabel *TextLabel8_3_2;
    QLabel *TextLabel9_2_2;
    QLabel *TextLabel10_2_2;
    QLabel *TextLabel11_3_2;
    QLabel *m_loanAmount3;
    QLabel *m_interestRate3;
    QLabel *m_duration3;
    QLabel *m_payment3;
    QLabel *m_balloon3;

    void setupUi(QWizardPage *DurationWizardPage)
    {
        if (DurationWizardPage->objectName().isEmpty())
            DurationWizardPage->setObjectName("DurationWizardPage");
        vboxLayout = new QVBoxLayout(DurationWizardPage);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel15_2 = new QLabel(DurationWizardPage);
        TextLabel15_2->setObjectName("TextLabel15_2");
        TextLabel15_2->setAlignment(Qt::AlignTop);
        TextLabel15_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel15_2);

        Spacer67_2 = new QSpacerItem(16, 54, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer67_2);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        TextLabel2_3 = new QLabel(DurationWizardPage);
        TextLabel2_3->setObjectName("TextLabel2_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(TextLabel2_3->sizePolicy().hasHeightForWidth());
        TextLabel2_3->setSizePolicy(sizePolicy);
        TextLabel2_3->setWordWrap(false);

        hboxLayout->addWidget(TextLabel2_3);

        m_durationValueEdit = new QSpinBox(DurationWizardPage);
        m_durationValueEdit->setObjectName("m_durationValueEdit");
        m_durationValueEdit->setMinimum(0);
        m_durationValueEdit->setMaximum(1000);

        hboxLayout->addWidget(m_durationValueEdit);

        m_durationUnitEdit = new KMyMoneyGeneralCombo(DurationWizardPage);
        m_durationUnitEdit->setObjectName("m_durationUnitEdit");

        hboxLayout->addWidget(m_durationUnitEdit);


        vboxLayout->addLayout(hboxLayout);

        Frame3_2_2 = new QFrame(DurationWizardPage);
        Frame3_2_2->setObjectName("Frame3_2_2");
        Frame3_2_2->setFrameShape(QFrame::StyledPanel);
        Frame3_2_2->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(Frame3_2_2);
#ifndef UI_Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        TextLabel6_3_2 = new QLabel(Frame3_2_2);
        TextLabel6_3_2->setObjectName("TextLabel6_3_2");
        TextLabel6_3_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel6_3_2->setWordWrap(true);

        gridLayout->addWidget(TextLabel6_3_2, 0, 0, 1, 1);

        TextLabel8_3_2 = new QLabel(Frame3_2_2);
        TextLabel8_3_2->setObjectName("TextLabel8_3_2");
        TextLabel8_3_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel8_3_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel8_3_2, 1, 0, 1, 1);

        TextLabel9_2_2 = new QLabel(Frame3_2_2);
        TextLabel9_2_2->setObjectName("TextLabel9_2_2");
        TextLabel9_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel9_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel9_2_2, 2, 0, 1, 1);

        TextLabel10_2_2 = new QLabel(Frame3_2_2);
        TextLabel10_2_2->setObjectName("TextLabel10_2_2");
        TextLabel10_2_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel10_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel10_2_2, 3, 0, 1, 1);

        TextLabel11_3_2 = new QLabel(Frame3_2_2);
        TextLabel11_3_2->setObjectName("TextLabel11_3_2");
        TextLabel11_3_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel11_3_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel11_3_2, 4, 0, 1, 1);

        m_loanAmount3 = new QLabel(Frame3_2_2);
        m_loanAmount3->setObjectName("m_loanAmount3");
        QFont font;
        font.setBold(true);
        m_loanAmount3->setFont(font);
        m_loanAmount3->setTextFormat(Qt::PlainText);
        m_loanAmount3->setWordWrap(false);

        gridLayout->addWidget(m_loanAmount3, 0, 1, 1, 1);

        m_interestRate3 = new QLabel(Frame3_2_2);
        m_interestRate3->setObjectName("m_interestRate3");
        m_interestRate3->setFont(font);
        m_interestRate3->setTextFormat(Qt::PlainText);
        m_interestRate3->setWordWrap(false);

        gridLayout->addWidget(m_interestRate3, 1, 1, 1, 1);

        m_duration3 = new QLabel(Frame3_2_2);
        m_duration3->setObjectName("m_duration3");
        m_duration3->setFont(font);
        m_duration3->setTextFormat(Qt::PlainText);
        m_duration3->setWordWrap(false);

        gridLayout->addWidget(m_duration3, 2, 1, 1, 1);

        m_payment3 = new QLabel(Frame3_2_2);
        m_payment3->setObjectName("m_payment3");
        m_payment3->setFont(font);
        m_payment3->setTextFormat(Qt::PlainText);
        m_payment3->setWordWrap(false);

        gridLayout->addWidget(m_payment3, 3, 1, 1, 1);

        m_balloon3 = new QLabel(Frame3_2_2);
        m_balloon3->setObjectName("m_balloon3");
        m_balloon3->setFont(font);
        m_balloon3->setTextFormat(Qt::PlainText);
        m_balloon3->setWordWrap(false);

        gridLayout->addWidget(m_balloon3, 4, 1, 1, 1);


        vboxLayout->addWidget(Frame3_2_2);


        retranslateUi(DurationWizardPage);

        QMetaObject::connectSlotsByName(DurationWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *DurationWizardPage)
    {
        TextLabel15_2->setText(tr2i18n("\n"
"Please enter the term of this loan or leave the field empty to calculate it. The term is the time that is required to fully repay the loan. This time might be different from the time your loan contract is signed for.", nullptr));
        TextLabel2_3->setText(tr2i18n("Term", nullptr));
        TextLabel6_3_2->setText(tr2i18n("Loan amount:", nullptr));
        TextLabel8_3_2->setText(tr2i18n("Interest rate:", nullptr));
        TextLabel9_2_2->setText(tr2i18n("Term:", nullptr));
        TextLabel10_2_2->setText(tr2i18n("Principal + Interest:", nullptr));
        TextLabel11_3_2->setText(tr2i18n("Final amortization payment", nullptr));
        m_loanAmount3->setText(tr2i18n("1", nullptr));
        m_interestRate3->setText(tr2i18n("1", nullptr));
        m_duration3->setText(tr2i18n("1", nullptr));
        m_payment3->setText(tr2i18n("1", nullptr));
        m_balloon3->setText(tr2i18n("1", nullptr));
        (void)DurationWizardPage;
    } // retranslateUi

};

namespace Ui {
    class DurationWizardPage: public Ui_DurationWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // DURATIONWIZARDPAGE_H

