/*
    SPDX-FileCopyrightText: 2013 Christian Dávid <christian-david@web.de>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "onlinetask.h"

onlineTask::onlineTask()
{}

onlineTask::onlineTask(const onlineTask &other)
{
    Q_UNUSED(other);
}
