/********************************************************************************
** Form generated from reading UI file 'creditdebitedit.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CREDITDEBITEDIT_H
#define UI_CREDITDEBITEDIT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "amountedit.h"

QT_BEGIN_NAMESPACE

class Ui_CreditDebitEdit
{
public:
    QHBoxLayout *horizontalLayout;
    AmountEdit *creditAmount;
    AmountEdit *debitAmount;

    void setupUi(QWidget *CreditDebitEdit)
    {
        if (CreditDebitEdit->objectName().isEmpty())
            CreditDebitEdit->setObjectName("CreditDebitEdit");
        CreditDebitEdit->resize(219, 32);
        horizontalLayout = new QHBoxLayout(CreditDebitEdit);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        creditAmount = new AmountEdit(CreditDebitEdit);
        creditAmount->setObjectName("creditAmount");

        horizontalLayout->addWidget(creditAmount);

        debitAmount = new AmountEdit(CreditDebitEdit);
        debitAmount->setObjectName("debitAmount");

        horizontalLayout->addWidget(debitAmount);


        retranslateUi(CreditDebitEdit);

        QMetaObject::connectSlotsByName(CreditDebitEdit);
    } // setupUi

    void retranslateUi(QWidget *CreditDebitEdit)
    {
        CreditDebitEdit->setWindowTitle(QString());
        creditAmount->setPlaceholderText(tr2i18n("Payment", nullptr));
        debitAmount->setPlaceholderText(tr2i18n("Deposit", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CreditDebitEdit: public Ui_CreditDebitEdit {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CREDITDEBITEDIT_H

