/********************************************************************************
** Form generated from reading UI file 'knewinstitutiondlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KNEWINSTITUTIONDLG_H
#define UI_KNEWINSTITUTIONDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "klineedit.h"
#include "kmessagewidget.h"

QT_BEGIN_NAMESPACE

class Ui_KNewInstitutionDlg
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *GroupBox1;
    QGridLayout *gridLayout;
    KLineEdit *bicEdit;
    QLabel *TextLabel4;
    KLineEdit *urlEdit;
    QLabel *label_2;
    KLineEdit *telephoneEdit;
    QLabel *TextLabel1;
    QLabel *label;
    QToolButton *iconButton;
    QLabel *TextLabel7;
    KLineEdit *postcodeEdit;
    KLineEdit *bankCodeEdit;
    QLabel *TextLabel3;
    KLineEdit *cityEdit;
    KLineEdit *nameEdit;
    KLineEdit *streetEdit;
    QLabel *TextLabel2;
    KMessageWidget *messageWidget;
    QLabel *TextLabel6;
    QLabel *TextLabel5;
    QLabel *label_3;
    KLineEdit *iconUrlEdit;
    QLabel *label_4;
    QSpacerItem *Spacer30;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KNewInstitutionDlg)
    {
        if (KNewInstitutionDlg->objectName().isEmpty())
            KNewInstitutionDlg->setObjectName("KNewInstitutionDlg");
        KNewInstitutionDlg->resize(631, 450);
        KNewInstitutionDlg->setSizeGripEnabled(true);
        verticalLayout = new QVBoxLayout(KNewInstitutionDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        GroupBox1 = new QGroupBox(KNewInstitutionDlg);
        GroupBox1->setObjectName("GroupBox1");
        GroupBox1->setEnabled(true);
        gridLayout = new QGridLayout(GroupBox1);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        bicEdit = new KLineEdit(GroupBox1);
        bicEdit->setObjectName("bicEdit");

        gridLayout->addWidget(bicEdit, 6, 1, 1, 4);

        TextLabel4 = new QLabel(GroupBox1);
        TextLabel4->setObjectName("TextLabel4");
        TextLabel4->setMinimumSize(QSize(100, 0));
        TextLabel4->setWordWrap(false);

        gridLayout->addWidget(TextLabel4, 3, 0, 1, 1);

        urlEdit = new KLineEdit(GroupBox1);
        urlEdit->setObjectName("urlEdit");

        gridLayout->addWidget(urlEdit, 7, 2, 1, 1);

        label_2 = new QLabel(GroupBox1);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 7, 1, 1, 1);

        telephoneEdit = new KLineEdit(GroupBox1);
        telephoneEdit->setObjectName("telephoneEdit");

        gridLayout->addWidget(telephoneEdit, 4, 1, 1, 4);

        TextLabel1 = new QLabel(GroupBox1);
        TextLabel1->setObjectName("TextLabel1");
        TextLabel1->setWordWrap(false);

        gridLayout->addWidget(TextLabel1, 0, 0, 1, 1);

        label = new QLabel(GroupBox1);
        label->setObjectName("label");

        gridLayout->addWidget(label, 7, 0, 1, 1);

        iconButton = new QToolButton(GroupBox1);
        iconButton->setObjectName("iconButton");
        iconButton->setEnabled(false);

        gridLayout->addWidget(iconButton, 7, 3, 1, 1);

        TextLabel7 = new QLabel(GroupBox1);
        TextLabel7->setObjectName("TextLabel7");
        TextLabel7->setMinimumSize(QSize(100, 0));
        TextLabel7->setWordWrap(false);

        gridLayout->addWidget(TextLabel7, 6, 0, 1, 1);

        postcodeEdit = new KLineEdit(GroupBox1);
        postcodeEdit->setObjectName("postcodeEdit");

        gridLayout->addWidget(postcodeEdit, 3, 1, 1, 4);

        bankCodeEdit = new KLineEdit(GroupBox1);
        bankCodeEdit->setObjectName("bankCodeEdit");

        gridLayout->addWidget(bankCodeEdit, 5, 1, 1, 4);

        TextLabel3 = new QLabel(GroupBox1);
        TextLabel3->setObjectName("TextLabel3");
        TextLabel3->setMinimumSize(QSize(100, 0));
        TextLabel3->setWordWrap(false);

        gridLayout->addWidget(TextLabel3, 2, 0, 1, 1);

        cityEdit = new KLineEdit(GroupBox1);
        cityEdit->setObjectName("cityEdit");

        gridLayout->addWidget(cityEdit, 1, 1, 1, 4);

        nameEdit = new KLineEdit(GroupBox1);
        nameEdit->setObjectName("nameEdit");

        gridLayout->addWidget(nameEdit, 0, 1, 1, 4);

        streetEdit = new KLineEdit(GroupBox1);
        streetEdit->setObjectName("streetEdit");

        gridLayout->addWidget(streetEdit, 2, 1, 1, 4);

        TextLabel2 = new QLabel(GroupBox1);
        TextLabel2->setObjectName("TextLabel2");
        TextLabel2->setMinimumSize(QSize(100, 0));
        TextLabel2->setWordWrap(false);

        gridLayout->addWidget(TextLabel2, 1, 0, 1, 1);

        messageWidget = new KMessageWidget(GroupBox1);
        messageWidget->setObjectName("messageWidget");

        gridLayout->addWidget(messageWidget, 9, 0, 1, 5);

        TextLabel6 = new QLabel(GroupBox1);
        TextLabel6->setObjectName("TextLabel6");
        TextLabel6->setMinimumSize(QSize(100, 0));
        TextLabel6->setWordWrap(false);

        gridLayout->addWidget(TextLabel6, 5, 0, 1, 1);

        TextLabel5 = new QLabel(GroupBox1);
        TextLabel5->setObjectName("TextLabel5");
        TextLabel5->setMinimumSize(QSize(100, 0));
        TextLabel5->setWordWrap(false);

        gridLayout->addWidget(TextLabel5, 4, 0, 1, 1);

        label_3 = new QLabel(GroupBox1);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 8, 0, 1, 1);

        iconUrlEdit = new KLineEdit(GroupBox1);
        iconUrlEdit->setObjectName("iconUrlEdit");
        iconUrlEdit->setProperty("showClearButton", QVariant(true));

        gridLayout->addWidget(iconUrlEdit, 8, 2, 1, 1);

        label_4 = new QLabel(GroupBox1);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 8, 1, 1, 1);


        verticalLayout->addWidget(GroupBox1);

        Spacer30 = new QSpacerItem(16, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(Spacer30);

        buttonBox = new QDialogButtonBox(KNewInstitutionDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(nameEdit, cityEdit);
        QWidget::setTabOrder(cityEdit, streetEdit);
        QWidget::setTabOrder(streetEdit, postcodeEdit);
        QWidget::setTabOrder(postcodeEdit, telephoneEdit);
        QWidget::setTabOrder(telephoneEdit, bankCodeEdit);
        QWidget::setTabOrder(bankCodeEdit, bicEdit);
        QWidget::setTabOrder(bicEdit, urlEdit);
        QWidget::setTabOrder(urlEdit, iconButton);
        QWidget::setTabOrder(iconButton, buttonBox);

        retranslateUi(KNewInstitutionDlg);

        QMetaObject::connectSlotsByName(KNewInstitutionDlg);
    } // setupUi

    void retranslateUi(QDialog *KNewInstitutionDlg)
    {
        KNewInstitutionDlg->setWindowTitle(tr2i18n("New Institution Dialog", nullptr));
        GroupBox1->setTitle(tr2i18n("Institution Details", nullptr));
#if QT_CONFIG(tooltip)
        bicEdit->setToolTip(tr2i18n("A Business Identifier Code, also known as SWIFT.", nullptr));
#endif // QT_CONFIG(tooltip)
        TextLabel4->setText(tr2i18n("Postal Code:", nullptr));
        label_2->setText(tr2i18n("https://", nullptr));
        TextLabel1->setText(tr2i18n("Name:", "institution name column"));
        label->setText(tr2i18n("Website", nullptr));
        iconButton->setText(tr2i18n("...", nullptr));
        TextLabel7->setText(tr2i18n("SWIFT/BIC", nullptr));
#if QT_CONFIG(tooltip)
        bankCodeEdit->setToolTip(tr2i18n("A bank code assigned to Institution by a central bank, a bank supervisory body or a Bankers Association. Known as 'Routing Number' (US accounts) or a 'Sort Code' (UK accounts).", nullptr));
#endif // QT_CONFIG(tooltip)
        TextLabel3->setText(tr2i18n("Street:", nullptr));
        TextLabel2->setText(tr2i18n("City:", nullptr));
        TextLabel6->setText(tr2i18n("Bank Code:", nullptr));
        TextLabel5->setText(tr2i18n("Telephone/Fax:", nullptr));
        label_3->setText(tr2i18n("Favicon URL", nullptr));
#if QT_CONFIG(tooltip)
        iconUrlEdit->setToolTip(tr2i18n("<html><head/><body><p>If left empty, the icon for the institution will be located at the root of the Website URL entered above with the name <span style=\" font-weight:600;\">favicon.ico</span>. You can override the location here.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        label_4->setText(tr2i18n("https://", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KNewInstitutionDlg: public Ui_KNewInstitutionDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KNEWINSTITUTIONDLG_H

