/********************************************************************************
** Form generated from reading UI file 'paymenteditwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAYMENTEDITWIZARDPAGE_H
#define UI_PAYMENTEDITWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <amountedit.h>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_PaymentEditWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel8_2_2;
    QSpacerItem *Spacer70_2;
    QHBoxLayout *hboxLayout;
    QSpacerItem *Spacer76;
    QGridLayout *gridLayout;
    AmountEdit *m_newPaymentEdit;
    QLabel *TextLabel2_4;
    QLabel *m_paymentLabel;
    QLabel *TextLabel1_5;
    QSpacerItem *Spacer75;
    QSpacerItem *Spacer71;
    QLabel *TextLabel4_5;

    void setupUi(QWizardPage *PaymentEditWizardPage)
    {
        if (PaymentEditWizardPage->objectName().isEmpty())
            PaymentEditWizardPage->setObjectName("PaymentEditWizardPage");
        vboxLayout = new QVBoxLayout(PaymentEditWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel8_2_2 = new QLabel(PaymentEditWizardPage);
        TextLabel8_2_2->setObjectName("TextLabel8_2_2");
        TextLabel8_2_2->setAlignment(Qt::AlignTop);
        TextLabel8_2_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel8_2_2);

        Spacer70_2 = new QSpacerItem(0, 21, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer70_2);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        Spacer76 = new QSpacerItem(100, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer76);

        gridLayout = new QGridLayout();
#ifndef UI_Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        gridLayout->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout->setObjectName("gridLayout");
        m_newPaymentEdit = new AmountEdit(PaymentEditWizardPage);
        m_newPaymentEdit->setObjectName("m_newPaymentEdit");

        gridLayout->addWidget(m_newPaymentEdit, 0, 1, 1, 1);

        TextLabel2_4 = new QLabel(PaymentEditWizardPage);
        TextLabel2_4->setObjectName("TextLabel2_4");
        TextLabel2_4->setWordWrap(false);

        gridLayout->addWidget(TextLabel2_4, 1, 0, 1, 1);

        m_paymentLabel = new QLabel(PaymentEditWizardPage);
        m_paymentLabel->setObjectName("m_paymentLabel");
        m_paymentLabel->setWordWrap(false);

        gridLayout->addWidget(m_paymentLabel, 1, 1, 1, 1);

        TextLabel1_5 = new QLabel(PaymentEditWizardPage);
        TextLabel1_5->setObjectName("TextLabel1_5");
        TextLabel1_5->setWordWrap(false);

        gridLayout->addWidget(TextLabel1_5, 0, 0, 1, 1);


        hboxLayout->addLayout(gridLayout);

        Spacer75 = new QSpacerItem(100, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer75);


        vboxLayout->addLayout(hboxLayout);

        Spacer71 = new QSpacerItem(0, 19, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer71);

        TextLabel4_5 = new QLabel(PaymentEditWizardPage);
        TextLabel4_5->setObjectName("TextLabel4_5");
        TextLabel4_5->setAlignment(Qt::AlignVCenter);
        TextLabel4_5->setWordWrap(true);

        vboxLayout->addWidget(TextLabel4_5);


        retranslateUi(PaymentEditWizardPage);

        QMetaObject::connectSlotsByName(PaymentEditWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *PaymentEditWizardPage)
    {
        TextLabel8_2_2->setText(tr2i18n("\n"
"Please enter the amount you pay for principal and interest or leave the field empty to calculate it.", nullptr));
        TextLabel2_4->setText(tr2i18n("Current amount", nullptr));
        m_paymentLabel->setText(tr2i18n("1", nullptr));
        TextLabel1_5->setText(tr2i18n("Principal and interest", nullptr));
        TextLabel4_5->setText(tr2i18n("KMyMoney either calculates the new interest rate or the amount for principal and interest. If you know the amount for principal and interest then enter it here.\n"
"\n"
"If KMyMoney should calculate this value for you, then leave the field blank.", nullptr));
        (void)PaymentEditWizardPage;
    } // retranslateUi

};

namespace Ui {
    class PaymentEditWizardPage: public Ui_PaymentEditWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PAYMENTEDITWIZARDPAGE_H

