// This file is generated by kconfig_compiler_kf6 from kbanking.kcfg.
// All changes you do to this file will be lost.
#ifndef KBANKINGSETTINGS_H
#define KBANKINGSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class KBankingSettings : public KConfigSkeleton
{
  public:

    static KBankingSettings *self();
    ~KBankingSettings() override;

    /**
      Set Width of the flicker field for optical tan generation (chipTan/sm@rtTan).
    */
    static
    void setWidth( int v )
    {
      if (!self()->isWidthImmutable())
        self()->mWidth = v;
    }

    /**
      Get Width of the flicker field for optical tan generation (chipTan/sm@rtTan).
    */
    static
    int width()
    {
      return self()->mWidth;
    }

    /**
      Is Width of the flicker field for optical tan generation (chipTan/sm@rtTan). Immutable
    */
    static
    bool isWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "width" ) );
    }

    /**
      Set Clock length of optical tan flickering in ms.
    */
    static
    void setClocksetting( int v )
    {
      if (!self()->isClocksettingImmutable())
        self()->mClocksetting = v;
    }

    /**
      Get Clock length of optical tan flickering in ms.
    */
    static
    int clocksetting()
    {
      return self()->mClocksetting;
    }

    /**
      Is Clock length of optical tan flickering in ms. Immutable
    */
    static
    bool isClocksettingImmutable()
    {
      return self()->isImmutable( QStringLiteral( "clocksetting" ) );
    }

  protected:
    KBankingSettings();
    friend class KBankingSettingsHelper;


    // opticalTan
    int mWidth;
    int mClocksetting;
};

#endif

