/********************************************************************************
** Form generated from reading UI file 'kloanpayoutpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KLOANPAYOUTPAGE_H
#define UI_KLOANPAYOUTPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kdatecombobox.h"
#include "kmymoneycategory.h"
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_KLoanPayoutPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel1;
    QCheckBox *m_noPayoutTransaction;
    QFrame *m_payoutDetailFrame;
    QGridLayout *gridLayout;
    QCheckBox *m_refinanceLoan;
    QLabel *textLabel1;
    KMyMoneyCategory *m_assetAccount;
    QPushButton *m_createAssetButton;
    QLabel *textLabel1_2;
    KMyMoneyCategory *m_loanAccount;
    QLabel *textLabel2;
    QSpacerItem *spacer88;
    KMyMoneyDateEdit *m_payoutDate;
    QSpacerItem *spacer89;

    void setupUi(QWidget *KLoanPayoutPage)
    {
        if (KLoanPayoutPage->objectName().isEmpty())
            KLoanPayoutPage->setObjectName("KLoanPayoutPage");
        KLoanPayoutPage->resize(600, 380);
        vboxLayout = new QVBoxLayout(KLoanPayoutPage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        qLabel1 = new QLabel(KLoanPayoutPage);
        qLabel1->setObjectName("qLabel1");
        qLabel1->setFocusPolicy(Qt::NoFocus);
        qLabel1->setWordWrap(true);

        vboxLayout->addWidget(qLabel1);

        m_noPayoutTransaction = new QCheckBox(KLoanPayoutPage);
        m_noPayoutTransaction->setObjectName("m_noPayoutTransaction");

        vboxLayout->addWidget(m_noPayoutTransaction);

        m_payoutDetailFrame = new QFrame(KLoanPayoutPage);
        m_payoutDetailFrame->setObjectName("m_payoutDetailFrame");
        m_payoutDetailFrame->setFrameShape(QFrame::NoFrame);
        m_payoutDetailFrame->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(m_payoutDetailFrame);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        m_refinanceLoan = new QCheckBox(m_payoutDetailFrame);
        m_refinanceLoan->setObjectName("m_refinanceLoan");

        gridLayout->addWidget(m_refinanceLoan, 0, 0, 1, 4);

        textLabel1 = new QLabel(m_payoutDetailFrame);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 1, 0, 1, 1);

        m_assetAccount = new KMyMoneyCategory(m_payoutDetailFrame);
        m_assetAccount->setObjectName("m_assetAccount");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_assetAccount->sizePolicy().hasHeightForWidth());
        m_assetAccount->setSizePolicy(sizePolicy);

        gridLayout->addWidget(m_assetAccount, 1, 1, 1, 2);

        m_createAssetButton = new QPushButton(m_payoutDetailFrame);
        m_createAssetButton->setObjectName("m_createAssetButton");

        gridLayout->addWidget(m_createAssetButton, 1, 3, 1, 1);

        textLabel1_2 = new QLabel(m_payoutDetailFrame);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        gridLayout->addWidget(textLabel1_2, 2, 0, 1, 1);

        m_loanAccount = new KMyMoneyCategory(m_payoutDetailFrame);
        m_loanAccount->setObjectName("m_loanAccount");
        m_loanAccount->setEnabled(false);
        sizePolicy.setHeightForWidth(m_loanAccount->sizePolicy().hasHeightForWidth());
        m_loanAccount->setSizePolicy(sizePolicy);

        gridLayout->addWidget(m_loanAccount, 2, 1, 1, 2);

        textLabel2 = new QLabel(m_payoutDetailFrame);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 3, 0, 1, 1);

        spacer88 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacer88, 3, 1, 1, 1);

        m_payoutDate = new KMyMoneyDateEdit(m_payoutDetailFrame);
        m_payoutDate->setObjectName("m_payoutDate");

        gridLayout->addWidget(m_payoutDate, 3, 2, 1, 1);


        vboxLayout->addWidget(m_payoutDetailFrame);

        spacer89 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer89);


        retranslateUi(KLoanPayoutPage);
        QObject::connect(m_noPayoutTransaction, &QCheckBox::toggled, m_payoutDetailFrame, &QFrame::setDisabled);
        QObject::connect(m_refinanceLoan, SIGNAL(toggled(bool)), m_assetAccount, SLOT(setDisabled(bool)));
        QObject::connect(m_refinanceLoan, SIGNAL(toggled(bool)), m_loanAccount, SLOT(setEnabled(bool)));
        QObject::connect(m_refinanceLoan, &QCheckBox::toggled, m_createAssetButton, &QPushButton::setDisabled);

        QMetaObject::connectSlotsByName(KLoanPayoutPage);
    } // setupUi

    void retranslateUi(QWidget *KLoanPayoutPage)
    {
        qLabel1->setText(tr2i18n("If this loan is for an asset, such as a car or a house, you can create the asset account now. An asset account represents the total value of an asset. The money from this loan will be transferred into the asset account you create or select.\n"
"If this loan is a 'consumer loan' (money to use however you want), you can use a checking account instead.", nullptr));
        m_noPayoutTransaction->setText(tr2i18n("Do not create payout transaction", nullptr));
#if QT_CONFIG(whatsthis)
        m_refinanceLoan->setWhatsThis(tr2i18n("Check this if you are refinancing a loan that already exists in KMyMoney", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_refinanceLoan->setText(tr2i18n("Refinance existing loan", nullptr));
        textLabel1->setText(tr2i18n("Asset Account", nullptr));
        m_createAssetButton->setText(tr2i18n("Create", nullptr));
        textLabel1_2->setText(tr2i18n("Loan account", nullptr));
        textLabel2->setText(tr2i18n("Date of payment", nullptr));
        (void)KLoanPayoutPage;
    } // retranslateUi

};

namespace Ui {
    class KLoanPayoutPage: public Ui_KLoanPayoutPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KLOANPAYOUTPAGE_H

