/********************************************************************************
** Form generated from reading UI file 'kgpgkeyselectiondlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KGPGKEYSELECTIONDLG_H
#define UI_KGPGKEYSELECTIONDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "keditlistwidget.h"
#include "kled.h"

QT_BEGIN_NAMESPACE

class Ui_KGpgKeySelectionDlg
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QComboBox *m_secretKey;
    QLabel *label_2;
    KEditListWidget *m_listWidget;
    QHBoxLayout *horizontalLayout;
    KLed *m_keyLed;
    QLabel *ledLabel;
    QSpacerItem *hs1;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KGpgKeySelectionDlg)
    {
        if (KGpgKeySelectionDlg->objectName().isEmpty())
            KGpgKeySelectionDlg->setObjectName("KGpgKeySelectionDlg");
        KGpgKeySelectionDlg->resize(575, 480);
        KGpgKeySelectionDlg->setSizeGripEnabled(true);
        KGpgKeySelectionDlg->setModal(true);
        verticalLayout = new QVBoxLayout(KGpgKeySelectionDlg);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(KGpgKeySelectionDlg);
        label->setObjectName("label");
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        m_secretKey = new QComboBox(KGpgKeySelectionDlg);
        m_secretKey->setObjectName("m_secretKey");

        verticalLayout->addWidget(m_secretKey);

        label_2 = new QLabel(KGpgKeySelectionDlg);
        label_2->setObjectName("label_2");

        verticalLayout->addWidget(label_2);

        m_listWidget = new KEditListWidget(KGpgKeySelectionDlg);
        m_listWidget->setObjectName("m_listWidget");

        verticalLayout->addWidget(m_listWidget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        m_keyLed = new KLed(KGpgKeySelectionDlg);
        m_keyLed->setObjectName("m_keyLed");

        horizontalLayout->addWidget(m_keyLed);

        ledLabel = new QLabel(KGpgKeySelectionDlg);
        ledLabel->setObjectName("ledLabel");

        horizontalLayout->addWidget(ledLabel);

        hs1 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(hs1);


        verticalLayout->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(KGpgKeySelectionDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(KGpgKeySelectionDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KGpgKeySelectionDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KGpgKeySelectionDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KGpgKeySelectionDlg);
    } // setupUi

    void retranslateUi(QDialog *KGpgKeySelectionDlg)
    {
        KGpgKeySelectionDlg->setWindowTitle(tr2i18n("Select additional keys", nullptr));
        label->setText(tr2i18n("You have configured KMyMoney to save your data secured with GPG. Please choose the key you want to use for encryption of your data.", nullptr));
        label_2->setText(tr2i18n("Add additional keys here", nullptr));
#if QT_CONFIG(whatsthis)
        m_listWidget->setWhatsThis(tr2i18n("Enter the id of the key you want to use for data encryption. This can either be an e-mail address or the hexadecimal key id. In case of the key id, do not forget the leading 0x.", nullptr));
#endif // QT_CONFIG(whatsthis)
        ledLabel->setText(tr2i18n("Keys for all of the above user ids found", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KGpgKeySelectionDlg: public Ui_KGpgKeySelectionDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KGPGKEYSELECTIONDLG_H

