/********************************************************************************
** Form generated from reading UI file 'kexportdlgdecl.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KEXPORTDLGDECL_H
#define UI_KEXPORTDLGDECL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "../widgets/kmymoneyaccountcombo.h"
#include "kcombobox.h"
#include "klineedit.h"
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_KExportDlgDecl
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel2;
    QFrame *Line2;
    QHBoxLayout *hboxLayout1;
    QLabel *lblFileExport;
    KLineEdit *m_qlineeditFile;
    QPushButton *m_qbuttonBrowse;
    QHBoxLayout *hboxLayout2;
    QVBoxLayout *vboxLayout1;
    QLabel *TextLabel2_2;
    KMyMoneyAccountCombo *m_accountComboBox;
    QVBoxLayout *vboxLayout2;
    QLabel *TextLabel3;
    QHBoxLayout *hboxLayout3;
    KComboBox *m_profileComboBox;
    QHBoxLayout *hboxLayout4;
    QGroupBox *m_qgroupboxContents;
    QVBoxLayout *vboxLayout3;
    QCheckBox *m_qcheckboxAccount;
    QCheckBox *m_qcheckboxCategories;
    QGroupBox *m_qgroupboxDates;
    QVBoxLayout *vboxLayout4;
    QGridLayout *gridLayout;
    KMyMoneyDateEdit *m_kmymoneydateStart;
    QLabel *lblEndDate;
    QLabel *lblStartDate;
    KMyMoneyDateEdit *m_kmymoneydateEnd;
    QSpacerItem *Spacer7;
    QFrame *Line2_2;
    QHBoxLayout *hboxLayout5;
    QSpacerItem *Spacer5;
    QPushButton *m_qbuttonOk;
    QPushButton *m_qbuttonCancel;

    void setupUi(QDialog *KExportDlgDecl)
    {
        if (KExportDlgDecl->objectName().isEmpty())
            KExportDlgDecl->setObjectName("KExportDlgDecl");
        KExportDlgDecl->resize(655, 468);
        KExportDlgDecl->setSizeGripEnabled(true);
        KExportDlgDecl->setModal(true);
        vboxLayout = new QVBoxLayout(KExportDlgDecl);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        TextLabel2 = new QLabel(KExportDlgDecl);
        TextLabel2->setObjectName("TextLabel2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(TextLabel2->sizePolicy().hasHeightForWidth());
        TextLabel2->setSizePolicy(sizePolicy);
        TextLabel2->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        TextLabel2->setWordWrap(true);

        hboxLayout->addWidget(TextLabel2);


        vboxLayout->addLayout(hboxLayout);

        Line2 = new QFrame(KExportDlgDecl);
        Line2->setObjectName("Line2");
        Line2->setFrameShape(QFrame::HLine);
        Line2->setFrameShadow(QFrame::Raised);

        vboxLayout->addWidget(Line2);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        lblFileExport = new QLabel(KExportDlgDecl);
        lblFileExport->setObjectName("lblFileExport");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(lblFileExport->sizePolicy().hasHeightForWidth());
        lblFileExport->setSizePolicy(sizePolicy1);
        lblFileExport->setWordWrap(false);

        hboxLayout1->addWidget(lblFileExport);

        m_qlineeditFile = new KLineEdit(KExportDlgDecl);
        m_qlineeditFile->setObjectName("m_qlineeditFile");

        hboxLayout1->addWidget(m_qlineeditFile);

        m_qbuttonBrowse = new QPushButton(KExportDlgDecl);
        m_qbuttonBrowse->setObjectName("m_qbuttonBrowse");

        hboxLayout1->addWidget(m_qbuttonBrowse);


        vboxLayout->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName("vboxLayout1");
        TextLabel2_2 = new QLabel(KExportDlgDecl);
        TextLabel2_2->setObjectName("TextLabel2_2");
        TextLabel2_2->setWordWrap(false);

        vboxLayout1->addWidget(TextLabel2_2);

        m_accountComboBox = new KMyMoneyAccountCombo(KExportDlgDecl);
        m_accountComboBox->setObjectName("m_accountComboBox");

        vboxLayout1->addWidget(m_accountComboBox);


        hboxLayout2->addLayout(vboxLayout1);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(6);
        vboxLayout2->setObjectName("vboxLayout2");
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        TextLabel3 = new QLabel(KExportDlgDecl);
        TextLabel3->setObjectName("TextLabel3");
        TextLabel3->setWordWrap(false);

        vboxLayout2->addWidget(TextLabel3);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setSpacing(6);
        hboxLayout3->setObjectName("hboxLayout3");
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        m_profileComboBox = new KComboBox(KExportDlgDecl);
        m_profileComboBox->setObjectName("m_profileComboBox");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(m_profileComboBox->sizePolicy().hasHeightForWidth());
        m_profileComboBox->setSizePolicy(sizePolicy2);
        m_profileComboBox->setMinimumSize(QSize(200, 0));
        m_profileComboBox->setDuplicatesEnabled(false);

        hboxLayout3->addWidget(m_profileComboBox);


        vboxLayout2->addLayout(hboxLayout3);


        hboxLayout2->addLayout(vboxLayout2);


        vboxLayout->addLayout(hboxLayout2);

        hboxLayout4 = new QHBoxLayout();
        hboxLayout4->setSpacing(6);
        hboxLayout4->setObjectName("hboxLayout4");
        m_qgroupboxContents = new QGroupBox(KExportDlgDecl);
        m_qgroupboxContents->setObjectName("m_qgroupboxContents");
        vboxLayout3 = new QVBoxLayout(m_qgroupboxContents);
        vboxLayout3->setSpacing(6);
        vboxLayout3->setContentsMargins(11, 11, 11, 11);
        vboxLayout3->setObjectName("vboxLayout3");
        vboxLayout3->setContentsMargins(11, 11, 11, 11);
        m_qcheckboxAccount = new QCheckBox(m_qgroupboxContents);
        m_qcheckboxAccount->setObjectName("m_qcheckboxAccount");

        vboxLayout3->addWidget(m_qcheckboxAccount);

        m_qcheckboxCategories = new QCheckBox(m_qgroupboxContents);
        m_qcheckboxCategories->setObjectName("m_qcheckboxCategories");

        vboxLayout3->addWidget(m_qcheckboxCategories);


        hboxLayout4->addWidget(m_qgroupboxContents);

        m_qgroupboxDates = new QGroupBox(KExportDlgDecl);
        m_qgroupboxDates->setObjectName("m_qgroupboxDates");
        vboxLayout4 = new QVBoxLayout(m_qgroupboxDates);
        vboxLayout4->setSpacing(6);
        vboxLayout4->setContentsMargins(11, 11, 11, 11);
        vboxLayout4->setObjectName("vboxLayout4");
        vboxLayout4->setContentsMargins(11, 11, 11, 11);
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        m_kmymoneydateStart = new KMyMoneyDateEdit(m_qgroupboxDates);
        m_kmymoneydateStart->setObjectName("m_kmymoneydateStart");

        gridLayout->addWidget(m_kmymoneydateStart, 0, 1, 1, 1);

        lblEndDate = new QLabel(m_qgroupboxDates);
        lblEndDate->setObjectName("lblEndDate");
        sizePolicy1.setHeightForWidth(lblEndDate->sizePolicy().hasHeightForWidth());
        lblEndDate->setSizePolicy(sizePolicy1);
        lblEndDate->setMinimumSize(QSize(50, 0));
        lblEndDate->setWordWrap(false);

        gridLayout->addWidget(lblEndDate, 1, 0, 1, 1);

        lblStartDate = new QLabel(m_qgroupboxDates);
        lblStartDate->setObjectName("lblStartDate");
        sizePolicy1.setHeightForWidth(lblStartDate->sizePolicy().hasHeightForWidth());
        lblStartDate->setSizePolicy(sizePolicy1);
        lblStartDate->setMinimumSize(QSize(50, 0));
        lblStartDate->setWordWrap(false);

        gridLayout->addWidget(lblStartDate, 0, 0, 1, 1);

        m_kmymoneydateEnd = new KMyMoneyDateEdit(m_qgroupboxDates);
        m_kmymoneydateEnd->setObjectName("m_kmymoneydateEnd");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(m_kmymoneydateEnd->sizePolicy().hasHeightForWidth());
        m_kmymoneydateEnd->setSizePolicy(sizePolicy3);

        gridLayout->addWidget(m_kmymoneydateEnd, 1, 1, 1, 1);


        vboxLayout4->addLayout(gridLayout);


        hboxLayout4->addWidget(m_qgroupboxDates);

        Spacer7 = new QSpacerItem(230, 16, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout4->addItem(Spacer7);


        vboxLayout->addLayout(hboxLayout4);

        Line2_2 = new QFrame(KExportDlgDecl);
        Line2_2->setObjectName("Line2_2");
        Line2_2->setFrameShape(QFrame::HLine);
        Line2_2->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(Line2_2);

        hboxLayout5 = new QHBoxLayout();
        hboxLayout5->setSpacing(6);
        hboxLayout5->setObjectName("hboxLayout5");
        hboxLayout5->setContentsMargins(0, 0, 0, 0);
        Spacer5 = new QSpacerItem(451, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout5->addItem(Spacer5);

        m_qbuttonOk = new QPushButton(KExportDlgDecl);
        m_qbuttonOk->setObjectName("m_qbuttonOk");
        m_qbuttonOk->setAutoDefault(true);

        hboxLayout5->addWidget(m_qbuttonOk);

        m_qbuttonCancel = new QPushButton(KExportDlgDecl);
        m_qbuttonCancel->setObjectName("m_qbuttonCancel");
        m_qbuttonCancel->setAutoDefault(true);

        hboxLayout5->addWidget(m_qbuttonCancel);


        vboxLayout->addLayout(hboxLayout5);


        retranslateUi(KExportDlgDecl);

        m_qbuttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(KExportDlgDecl);
    } // setupUi

    void retranslateUi(QDialog *KExportDlgDecl)
    {
        KExportDlgDecl->setWindowTitle(tr2i18n("QIF Export", nullptr));
        TextLabel2->setText(tr2i18n("From this dialog you are able to export transactions to a quicken compatible file, (known as a QIF file, because of the extension).  Please enter the path to the QIF file or select it by clicking on the Browse button.\n"
"\n"
"You can choose the file's path, the account and the format of the QIF file (profile).  Choose Account to export all the transactions between the specified dates or just categories.  You can also limit the transactions that are exported by start and ending date.  Once you have pressed the Export button a message box will appear when the export has completed detailing how many transactions, categories and payees were exported.", nullptr));
        lblFileExport->setText(tr2i18n("File to export to:", nullptr));
        m_qbuttonBrowse->setText(tr2i18n("&Browse...", nullptr));
        TextLabel2_2->setText(tr2i18n("Account to export", nullptr));
        TextLabel3->setText(tr2i18n("QIF Profile", nullptr));
        m_qgroupboxContents->setTitle(tr2i18n("Contents to Export", nullptr));
        m_qcheckboxAccount->setText(tr2i18n("Account", nullptr));
        m_qcheckboxCategories->setText(tr2i18n("Categories", nullptr));
        m_qgroupboxDates->setTitle(tr2i18n("Date Range", nullptr));
        lblEndDate->setText(tr2i18n("End on:", nullptr));
        lblStartDate->setText(tr2i18n("Start on:", nullptr));
        m_qbuttonOk->setText(tr2i18n("&Export", nullptr));
        m_qbuttonCancel->setText(tr2i18n("Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KExportDlgDecl: public Ui_KExportDlgDecl {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KEXPORTDLGDECL_H

