/********************************************************************************
** Form generated from reading UI file 'kmymoneyvalidationfeedback.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KMYMONEYVALIDATIONFEEDBACK_H
#define UI_KMYMONEYVALIDATIONFEEDBACK_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KMyMoneyValidationFeedback
{
public:
    QHBoxLayout *horizontalLayout;
    QLabel *icon;
    QLabel *label;

    void setupUi(QWidget *KMyMoneyValidationFeedback)
    {
        if (KMyMoneyValidationFeedback->objectName().isEmpty())
            KMyMoneyValidationFeedback->setObjectName("KMyMoneyValidationFeedback");
        KMyMoneyValidationFeedback->resize(162, 26);
        horizontalLayout = new QHBoxLayout(KMyMoneyValidationFeedback);
        horizontalLayout->setObjectName("horizontalLayout");
        icon = new QLabel(KMyMoneyValidationFeedback);
        icon->setObjectName("icon");
        icon->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        horizontalLayout->addWidget(icon);

        label = new QLabel(KMyMoneyValidationFeedback);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label->setWordWrap(true);

        horizontalLayout->addWidget(label);


        retranslateUi(KMyMoneyValidationFeedback);

        QMetaObject::connectSlotsByName(KMyMoneyValidationFeedback);
    } // setupUi

    void retranslateUi(QWidget *KMyMoneyValidationFeedback)
    {
        icon->setText(tr2i18n("icon", nullptr));
        label->setText(tr2i18n("Feedback content", nullptr));
        (void)KMyMoneyValidationFeedback;
    } // retranslateUi

};

namespace Ui {
    class KMyMoneyValidationFeedback: public Ui_KMyMoneyValidationFeedback {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KMYMONEYVALIDATIONFEEDBACK_H

