/****************************************************************************
** Meta object code from reading C++ file 'accountsmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/mymoney/storage/accountsmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'accountsmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13AccountsModelE_t {};
} // unnamed namespace

template <> constexpr inline auto AccountsModel::qt_create_metaobjectdata<qt_meta_tag_ZN13AccountsModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AccountsModel",
        "netWorthChanged",
        "",
        "MyMoneyMoney",
        "amount",
        "approximate",
        "profitLossChanged",
        "reconciliationInfoChanged",
        "reparentAccountRequest",
        "accountId",
        "newParentId",
        "aboutToRemoveAccounts",
        "accountsRemoved",
        "setupAccountFractions",
        "updateAccountBalances",
        "QHash<QString,AccountBalances>",
        "balances"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'netWorthChanged'
        QtMocHelpers::SignalData<void(const MyMoneyMoney &, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Signal 'profitLossChanged'
        QtMocHelpers::SignalData<void(const MyMoneyMoney &, bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Signal 'reconciliationInfoChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reparentAccountRequest'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Signal 'aboutToRemoveAccounts'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'accountsRemoved'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setupAccountFractions'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateAccountBalances'
        QtMocHelpers::SlotData<void(const QHash<QString,AccountBalances> &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AccountsModel, qt_meta_tag_ZN13AccountsModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AccountsModel::staticMetaObject = { {
    QMetaObject::SuperData::link<MyMoneyModel<MyMoneyAccount>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13AccountsModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13AccountsModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13AccountsModelE_t>.metaTypes,
    nullptr
} };

void AccountsModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountsModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->netWorthChanged((*reinterpret_cast<std::add_pointer_t<MyMoneyMoney>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->profitLossChanged((*reinterpret_cast<std::add_pointer_t<MyMoneyMoney>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 2: _t->reconciliationInfoChanged(); break;
        case 3: _t->reparentAccountRequest((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->aboutToRemoveAccounts(); break;
        case 5: _t->accountsRemoved(); break;
        case 6: _t->setupAccountFractions(); break;
        case 7: _t->updateAccountBalances((*reinterpret_cast<std::add_pointer_t<QHash<QString,AccountBalances>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MyMoneyMoney >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MyMoneyMoney >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccountsModel::*)(const MyMoneyMoney & , bool )>(_a, &AccountsModel::netWorthChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountsModel::*)(const MyMoneyMoney & , bool )>(_a, &AccountsModel::profitLossChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountsModel::*)()>(_a, &AccountsModel::reconciliationInfoChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountsModel::*)(const QString & , const QString & )>(_a, &AccountsModel::reparentAccountRequest, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountsModel::*)()>(_a, &AccountsModel::aboutToRemoveAccounts, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountsModel::*)()>(_a, &AccountsModel::accountsRemoved, 5))
            return;
    }
}

const QMetaObject *AccountsModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AccountsModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13AccountsModelE_t>.strings))
        return static_cast<void*>(this);
    return MyMoneyModel<MyMoneyAccount>::qt_metacast(_clname);
}

int AccountsModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MyMoneyModel<MyMoneyAccount>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void AccountsModel::netWorthChanged(const MyMoneyMoney & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void AccountsModel::profitLossChanged(const MyMoneyMoney & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void AccountsModel::reconciliationInfoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void AccountsModel::reparentAccountRequest(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void AccountsModel::aboutToRemoveAccounts()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void AccountsModel::accountsRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
