/********************************************************************************
** Form generated from reading UI file 'kgncimportoptionsdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KGNCIMPORTOPTIONSDLG_H
#define UI_KGNCIMPORTOPTIONSDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kcombobox.h"

QT_BEGIN_NAMESPACE

class Ui_KGncImportOptionsDlg
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *textLabel1;
    QGroupBox *buttonInvestGroupBox;
    QVBoxLayout *vboxLayout;
    QRadioButton *radioInvest1;
    QRadioButton *radioInvest2;
    QRadioButton *radioInvest3;
    QGroupBox *buttonGroupBox5;
    QVBoxLayout *vboxLayout1;
    QCheckBox *checkFinanceQuote;
    QGroupBox *buttonGroupBox2;
    QVBoxLayout *vboxLayout2;
    QCheckBox *checkSchedules;
    QGroupBox *buttonGroupBox4;
    QVBoxLayout *vboxLayout3;
    QHBoxLayout *hboxLayout;
    QCheckBox *checkDecode;
    QSpacerItem *spacer4;
    KComboBox *comboDecode;
    QGroupBox *buttonGroupBox18;
    QVBoxLayout *verticalLayout;
    QCheckBox *checkTxNotes;
    QFrame *line1;
    QGroupBox *buttonGroupBox3;
    QVBoxLayout *vboxLayout4;
    QCheckBox *checkDebugGeneral;
    QCheckBox *checkDebugXML;
    QCheckBox *checkAnonymize;
    QDialogButtonBox *buttonBox;
    QSpacerItem *verticalSpacer;
    QButtonGroup *buttonGroup18;
    QButtonGroup *buttonGroup2;
    QButtonGroup *buttonGroup3;
    QButtonGroup *buttonGroup4;
    QButtonGroup *buttonGroup5;
    QButtonGroup *buttonInvestGroup;

    void setupUi(QDialog *KGncImportOptionsDlg)
    {
        if (KGncImportOptionsDlg->objectName().isEmpty())
            KGncImportOptionsDlg->setObjectName("KGncImportOptionsDlg");
        KGncImportOptionsDlg->resize(526, 679);
        KGncImportOptionsDlg->setMinimumSize(QSize(526, 679));
        verticalLayout_2 = new QVBoxLayout(KGncImportOptionsDlg);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        textLabel1 = new QLabel(KGncImportOptionsDlg);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setAlignment(Qt::AlignCenter);
        textLabel1->setWordWrap(false);

        verticalLayout_2->addWidget(textLabel1);

        buttonInvestGroupBox = new QGroupBox(KGncImportOptionsDlg);
        buttonInvestGroupBox->setObjectName("buttonInvestGroupBox");
        vboxLayout = new QVBoxLayout(buttonInvestGroupBox);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        radioInvest1 = new QRadioButton(buttonInvestGroupBox);
        buttonInvestGroup = new QButtonGroup(KGncImportOptionsDlg);
        buttonInvestGroup->setObjectName("buttonInvestGroup");
        buttonInvestGroup->addButton(radioInvest1);
        radioInvest1->setObjectName("radioInvest1");
        radioInvest1->setChecked(true);

        vboxLayout->addWidget(radioInvest1);

        radioInvest2 = new QRadioButton(buttonInvestGroupBox);
        buttonInvestGroup->addButton(radioInvest2);
        radioInvest2->setObjectName("radioInvest2");

        vboxLayout->addWidget(radioInvest2);

        radioInvest3 = new QRadioButton(buttonInvestGroupBox);
        buttonInvestGroup->addButton(radioInvest3);
        radioInvest3->setObjectName("radioInvest3");

        vboxLayout->addWidget(radioInvest3);


        verticalLayout_2->addWidget(buttonInvestGroupBox);

        buttonGroupBox5 = new QGroupBox(KGncImportOptionsDlg);
        buttonGroupBox5->setObjectName("buttonGroupBox5");
        vboxLayout1 = new QVBoxLayout(buttonGroupBox5);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        checkFinanceQuote = new QCheckBox(buttonGroupBox5);
        buttonGroup5 = new QButtonGroup(KGncImportOptionsDlg);
        buttonGroup5->setObjectName("buttonGroup5");
        buttonGroup5->addButton(checkFinanceQuote);
        checkFinanceQuote->setObjectName("checkFinanceQuote");

        vboxLayout1->addWidget(checkFinanceQuote);


        verticalLayout_2->addWidget(buttonGroupBox5);

        buttonGroupBox2 = new QGroupBox(KGncImportOptionsDlg);
        buttonGroupBox2->setObjectName("buttonGroupBox2");
        vboxLayout2 = new QVBoxLayout(buttonGroupBox2);
        vboxLayout2->setSpacing(6);
        vboxLayout2->setContentsMargins(11, 11, 11, 11);
        vboxLayout2->setObjectName("vboxLayout2");
        checkSchedules = new QCheckBox(buttonGroupBox2);
        buttonGroup2 = new QButtonGroup(KGncImportOptionsDlg);
        buttonGroup2->setObjectName("buttonGroup2");
        buttonGroup2->addButton(checkSchedules);
        checkSchedules->setObjectName("checkSchedules");

        vboxLayout2->addWidget(checkSchedules);


        verticalLayout_2->addWidget(buttonGroupBox2);

        buttonGroupBox4 = new QGroupBox(KGncImportOptionsDlg);
        buttonGroupBox4->setObjectName("buttonGroupBox4");
        vboxLayout3 = new QVBoxLayout(buttonGroupBox4);
        vboxLayout3->setSpacing(6);
        vboxLayout3->setContentsMargins(11, 11, 11, 11);
        vboxLayout3->setObjectName("vboxLayout3");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(0);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        checkDecode = new QCheckBox(buttonGroupBox4);
        buttonGroup4 = new QButtonGroup(KGncImportOptionsDlg);
        buttonGroup4->setObjectName("buttonGroup4");
        buttonGroup4->addButton(checkDecode);
        checkDecode->setObjectName("checkDecode");

        hboxLayout->addWidget(checkDecode);

        spacer4 = new QSpacerItem(140, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer4);

        comboDecode = new KComboBox(buttonGroupBox4);
        comboDecode->setObjectName("comboDecode");

        hboxLayout->addWidget(comboDecode);


        vboxLayout3->addLayout(hboxLayout);


        verticalLayout_2->addWidget(buttonGroupBox4);

        buttonGroupBox18 = new QGroupBox(KGncImportOptionsDlg);
        buttonGroupBox18->setObjectName("buttonGroupBox18");
        verticalLayout = new QVBoxLayout(buttonGroupBox18);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        checkTxNotes = new QCheckBox(buttonGroupBox18);
        buttonGroup18 = new QButtonGroup(KGncImportOptionsDlg);
        buttonGroup18->setObjectName("buttonGroup18");
        buttonGroup18->addButton(checkTxNotes);
        checkTxNotes->setObjectName("checkTxNotes");

        verticalLayout->addWidget(checkTxNotes);


        verticalLayout_2->addWidget(buttonGroupBox18);

        line1 = new QFrame(KGncImportOptionsDlg);
        line1->setObjectName("line1");
        line1->setFrameShape(QFrame::HLine);
        line1->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line1);

        buttonGroupBox3 = new QGroupBox(KGncImportOptionsDlg);
        buttonGroupBox3->setObjectName("buttonGroupBox3");
        vboxLayout4 = new QVBoxLayout(buttonGroupBox3);
        vboxLayout4->setSpacing(6);
        vboxLayout4->setContentsMargins(11, 11, 11, 11);
        vboxLayout4->setObjectName("vboxLayout4");
        checkDebugGeneral = new QCheckBox(buttonGroupBox3);
        buttonGroup3 = new QButtonGroup(KGncImportOptionsDlg);
        buttonGroup3->setObjectName("buttonGroup3");
        buttonGroup3->addButton(checkDebugGeneral);
        checkDebugGeneral->setObjectName("checkDebugGeneral");

        vboxLayout4->addWidget(checkDebugGeneral);

        checkDebugXML = new QCheckBox(buttonGroupBox3);
        buttonGroup3->addButton(checkDebugXML);
        checkDebugXML->setObjectName("checkDebugXML");

        vboxLayout4->addWidget(checkDebugXML);

        checkAnonymize = new QCheckBox(buttonGroupBox3);
        buttonGroup3->addButton(checkAnonymize);
        checkAnonymize->setObjectName("checkAnonymize");

        vboxLayout4->addWidget(checkAnonymize);


        verticalLayout_2->addWidget(buttonGroupBox3);

        buttonBox = new QDialogButtonBox(KGncImportOptionsDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);

        verticalSpacer = new QSpacerItem(20, 25, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        retranslateUi(KGncImportOptionsDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KGncImportOptionsDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KGncImportOptionsDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KGncImportOptionsDlg);
    } // setupUi

    void retranslateUi(QDialog *KGncImportOptionsDlg)
    {
        KGncImportOptionsDlg->setWindowTitle(tr2i18n("GnuCash Import Options", nullptr));
        textLabel1->setText(tr2i18n("Use 'Help' for more information on these options", nullptr));
        buttonInvestGroupBox->setTitle(tr2i18n("Investment Handling", nullptr));
        radioInvest1->setText(tr2i18n("O&ne investment account for each stock", nullptr));
        radioInvest2->setText(tr2i18n("One in&vestment account for all stocks", nullptr));
        radioInvest3->setText(tr2i18n("P&rompt for an investment account for each stock", nullptr));
        buttonGroupBox5->setTitle(tr2i18n("Online price quotes", nullptr));
        checkFinanceQuote->setText(tr2i18n("Use Finance::Quote for share price quotes", nullptr));
        buttonGroupBox2->setTitle(tr2i18n("Scheduled Transactions", nullptr));
        checkSchedules->setText(tr2i18n("Drop suspect scheduled transactions", nullptr));
        buttonGroupBox4->setTitle(tr2i18n("Decoding option", nullptr));
        checkDecode->setText(tr2i18n("Decode using", nullptr));
        buttonGroupBox18->setTitle(tr2i18n("Transaction Notes option", nullptr));
        checkTxNotes->setText(tr2i18n("Use transaction notes on non-split transactions", nullptr));
        buttonGroupBox3->setTitle(tr2i18n("Debug Options", nullptr));
        checkDebugGeneral->setText(tr2i18n("General debug data", nullptr));
        checkDebugXML->setText(tr2i18n("Display XML data", nullptr));
        checkAnonymize->setText(tr2i18n("Anonymize data", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KGncImportOptionsDlg: public Ui_KGncImportOptionsDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KGNCIMPORTOPTIONSDLG_H

