/*
    SPDX-FileCopyrightText: 2014 Christian Dávid <christian-david@web.de>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "onlinetaskconverter.h"

onlineTaskConverter::onlineTaskConverter()
{
}

onlineTaskConverter::~onlineTaskConverter()
{
}
