/****************************************************************************
** Meta object code from reading C++ file 'kmymoneyaccountcombo.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/widgets/kmymoneyaccountcombo.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kmymoneyaccountcombo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20KMyMoneyAccountComboE_t {};
} // unnamed namespace

template <> constexpr inline auto KMyMoneyAccountCombo::qt_create_metaobjectdata<qt_meta_tag_ZN20KMyMoneyAccountComboE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KMyMoneyAccountCombo",
        "accountSelected",
        "",
        "splitDialogRequest",
        "expandAll",
        "collapseAll",
        "showPopup",
        "hidePopup",
        "setSelected",
        "id",
        "setSplitActionVisible",
        "show",
        "activated",
        "makeCompletion",
        "txt",
        "selectItem",
        "QModelIndex",
        "index"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'accountSelected'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'splitDialogRequest'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'expandAll'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'collapseAll'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showPopup'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'hidePopup'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSelected'
        QtMocHelpers::SlotData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Slot 'setSplitActionVisible'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'activated'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'makeCompletion'
        QtMocHelpers::SlotData<void(const QString &)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'selectItem'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KMyMoneyAccountCombo, qt_meta_tag_ZN20KMyMoneyAccountComboE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KMyMoneyAccountCombo::staticMetaObject = { {
    QMetaObject::SuperData::link<KComboBox::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KMyMoneyAccountComboE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KMyMoneyAccountComboE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20KMyMoneyAccountComboE_t>.metaTypes,
    nullptr
} };

void KMyMoneyAccountCombo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KMyMoneyAccountCombo *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accountSelected((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->splitDialogRequest(); break;
        case 2: _t->expandAll(); break;
        case 3: _t->collapseAll(); break;
        case 4: _t->showPopup(); break;
        case 5: _t->hidePopup(); break;
        case 6: _t->setSelected((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->setSplitActionVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->activated(); break;
        case 9: _t->makeCompletion((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->selectItem((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyAccountCombo::*)(const QString & )>(_a, &KMyMoneyAccountCombo::accountSelected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyAccountCombo::*)()>(_a, &KMyMoneyAccountCombo::splitDialogRequest, 1))
            return;
    }
}

const QMetaObject *KMyMoneyAccountCombo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KMyMoneyAccountCombo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KMyMoneyAccountComboE_t>.strings))
        return static_cast<void*>(this);
    return KComboBox::qt_metacast(_clname);
}

int KMyMoneyAccountCombo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KComboBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void KMyMoneyAccountCombo::accountSelected(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KMyMoneyAccountCombo::splitDialogRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN31KMyMoneyAccountComboSplitHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto KMyMoneyAccountComboSplitHelper::qt_create_metaobjectdata<qt_meta_tag_ZN31KMyMoneyAccountComboSplitHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KMyMoneyAccountComboSplitHelper",
        "accountComboEnabled",
        "",
        "enabled",
        "accountComboDisabled",
        "disabled",
        "updateWidget",
        "modelDestroyed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'accountComboEnabled'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'accountComboDisabled'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'updateWidget'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'modelDestroyed'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KMyMoneyAccountComboSplitHelper, qt_meta_tag_ZN31KMyMoneyAccountComboSplitHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KMyMoneyAccountComboSplitHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31KMyMoneyAccountComboSplitHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31KMyMoneyAccountComboSplitHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31KMyMoneyAccountComboSplitHelperE_t>.metaTypes,
    nullptr
} };

void KMyMoneyAccountComboSplitHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KMyMoneyAccountComboSplitHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accountComboEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->accountComboDisabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->updateWidget(); break;
        case 3: _t->modelDestroyed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyAccountComboSplitHelper::*)(bool )>(_a, &KMyMoneyAccountComboSplitHelper::accountComboEnabled, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyAccountComboSplitHelper::*)(bool )>(_a, &KMyMoneyAccountComboSplitHelper::accountComboDisabled, 1))
            return;
    }
}

const QMetaObject *KMyMoneyAccountComboSplitHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KMyMoneyAccountComboSplitHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31KMyMoneyAccountComboSplitHelperE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KMyMoneyAccountComboSplitHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void KMyMoneyAccountComboSplitHelper::accountComboEnabled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KMyMoneyAccountComboSplitHelper::accountComboDisabled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
