/********************************************************************************
** Form generated from reading UI file 'newgeneralinfowizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NEWGENERALINFOWIZARDPAGE_H
#define UI_NEWGENERALINFOWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_NewGeneralInfoWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel2_2;
    QSpacerItem *Spacer10;
    QHBoxLayout *hboxLayout;
    QSpacerItem *Spacer8;
    QVBoxLayout *vboxLayout1;
    QLabel *TextLabel3;
    QLabel *TextLabel4;
    QSpacerItem *Spacer7;
    QSpacerItem *Spacer9;
    QSpacerItem *Spacer44;

    void setupUi(QWizardPage *NewGeneralInfoWizardPage)
    {
        if (NewGeneralInfoWizardPage->objectName().isEmpty())
            NewGeneralInfoWizardPage->setObjectName("NewGeneralInfoWizardPage");
        vboxLayout = new QVBoxLayout(NewGeneralInfoWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel2_2 = new QLabel(NewGeneralInfoWizardPage);
        TextLabel2_2->setObjectName("TextLabel2_2");
        TextLabel2_2->setAlignment(Qt::AlignTop);
        TextLabel2_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel2_2);

        Spacer10 = new QSpacerItem(0, 32, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer10);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        Spacer8 = new QSpacerItem(138, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer8);

        vboxLayout1 = new QVBoxLayout();
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout1->setObjectName("vboxLayout1");
        TextLabel3 = new QLabel(NewGeneralInfoWizardPage);
        TextLabel3->setObjectName("TextLabel3");
        TextLabel3->setAlignment(Qt::AlignTop);
        TextLabel3->setWordWrap(false);

        vboxLayout1->addWidget(TextLabel3);

        TextLabel4 = new QLabel(NewGeneralInfoWizardPage);
        TextLabel4->setObjectName("TextLabel4");
        TextLabel4->setEnabled(false);
        TextLabel4->setWordWrap(false);

        vboxLayout1->addWidget(TextLabel4);

        Spacer7 = new QSpacerItem(0, 25, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(Spacer7);


        hboxLayout->addLayout(vboxLayout1);

        Spacer9 = new QSpacerItem(137, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer9);


        vboxLayout->addLayout(hboxLayout);

        Spacer44 = new QSpacerItem(0, 32, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer44);


        retranslateUi(NewGeneralInfoWizardPage);

        QMetaObject::connectSlotsByName(NewGeneralInfoWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *NewGeneralInfoWizardPage)
    {
        TextLabel2_2->setText(tr2i18n("\n"
"In the first step, KMyMoney will ask you some general information about the loan account to be created.", nullptr));
        TextLabel3->setText(tr2i18n("1. General Information", nullptr));
        TextLabel4->setText(tr2i18n("\n"
"2. Calculate Loan\n"
"\n"
"3. Payments", nullptr));
        (void)NewGeneralInfoWizardPage;
    } // retranslateUi

};

namespace Ui {
    class NewGeneralInfoWizardPage: public Ui_NewGeneralInfoWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // NEWGENERALINFOWIZARDPAGE_H

