/********************************************************************************
** Form generated from reading UI file 'previouspaymentswizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREVIOUSPAYMENTSWIZARDPAGE_H
#define UI_PREVIOUSPAYMENTSWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_PreviousPaymentsWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel6_2;
    QSpacerItem *Spacer28;
    QHBoxLayout *hboxLayout;
    QFrame *frame_4;
    QVBoxLayout *vboxLayout1;
    QRadioButton *m_noPreviousPaymentButton;
    QRadioButton *m_previousPaymentButton;
    QSpacerItem *Spacer14_2_2;
    QSpacerItem *Spacer29;
    QLabel *TextLabel3_3;
    QSpacerItem *Spacer41;
    QButtonGroup *ButtonGroup3;

    void setupUi(QWizardPage *PreviousPaymentsWizardPage)
    {
        if (PreviousPaymentsWizardPage->objectName().isEmpty())
            PreviousPaymentsWizardPage->setObjectName("PreviousPaymentsWizardPage");
        vboxLayout = new QVBoxLayout(PreviousPaymentsWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel6_2 = new QLabel(PreviousPaymentsWizardPage);
        TextLabel6_2->setObjectName("TextLabel6_2");
        TextLabel6_2->setAlignment(Qt::AlignTop);
        TextLabel6_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel6_2);

        Spacer28 = new QSpacerItem(0, 32, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer28);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        frame_4 = new QFrame(PreviousPaymentsWizardPage);
        frame_4->setObjectName("frame_4");
        frame_4->setFrameShape(QFrame::NoFrame);
        vboxLayout1 = new QVBoxLayout(frame_4);
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        m_noPreviousPaymentButton = new QRadioButton(frame_4);
        ButtonGroup3 = new QButtonGroup(PreviousPaymentsWizardPage);
        ButtonGroup3->setObjectName("ButtonGroup3");
        ButtonGroup3->addButton(m_noPreviousPaymentButton);
        m_noPreviousPaymentButton->setObjectName("m_noPreviousPaymentButton");

        vboxLayout1->addWidget(m_noPreviousPaymentButton);

        m_previousPaymentButton = new QRadioButton(frame_4);
        ButtonGroup3->addButton(m_previousPaymentButton);
        m_previousPaymentButton->setObjectName("m_previousPaymentButton");

        vboxLayout1->addWidget(m_previousPaymentButton);


        hboxLayout->addWidget(frame_4);

        Spacer14_2_2 = new QSpacerItem(202, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer14_2_2);


        vboxLayout->addLayout(hboxLayout);

        Spacer29 = new QSpacerItem(0, 32, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer29);

        TextLabel3_3 = new QLabel(PreviousPaymentsWizardPage);
        TextLabel3_3->setObjectName("TextLabel3_3");
        TextLabel3_3->setAlignment(Qt::AlignTop);
        TextLabel3_3->setWordWrap(true);

        vboxLayout->addWidget(TextLabel3_3);

        Spacer41 = new QSpacerItem(0, 32, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer41);


        retranslateUi(PreviousPaymentsWizardPage);

        QMetaObject::connectSlotsByName(PreviousPaymentsWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *PreviousPaymentsWizardPage)
    {
        TextLabel6_2->setText(tr2i18n("\n"
"Were there any payments for this loan whether they are entered into KMyMoney or not?", nullptr));
        m_noPreviousPaymentButton->setText(tr2i18n("No, no payments were made yet.", nullptr));
        m_previousPaymentButton->setText(tr2i18n("Yes, payments were made.", nullptr));
        TextLabel3_3->setText(tr2i18n("Note: Payments made to obtain the loan (e.g. Dissagio) are not considered as payments in this context.", nullptr));
        (void)PreviousPaymentsWizardPage;
    } // retranslateUi

};

namespace Ui {
    class PreviousPaymentsWizardPage: public Ui_PreviousPaymentsWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PREVIOUSPAYMENTSWIZARDPAGE_H

