/********************************************************************************
** Form generated from reading UI file 'importsummarydlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMPORTSUMMARYDLG_H
#define UI_IMPORTSUMMARYDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kmymoneytreeview.h"

QT_BEGIN_NAMESPACE

class Ui_ImportSummaryDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *m_resultSummaryLabel;
    KMyMoneyTreeView *m_resultTreeView;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ImportSummaryDialog)
    {
        if (ImportSummaryDialog->objectName().isEmpty())
            ImportSummaryDialog->setObjectName("ImportSummaryDialog");
        ImportSummaryDialog->resize(995, 390);
        ImportSummaryDialog->setModal(true);
        verticalLayout = new QVBoxLayout(ImportSummaryDialog);
        verticalLayout->setObjectName("verticalLayout");
        m_resultSummaryLabel = new QLabel(ImportSummaryDialog);
        m_resultSummaryLabel->setObjectName("m_resultSummaryLabel");
        m_resultSummaryLabel->setText(QString::fromUtf8("TextLabel"));

        verticalLayout->addWidget(m_resultSummaryLabel);

        m_resultTreeView = new KMyMoneyTreeView(ImportSummaryDialog);
        m_resultTreeView->setObjectName("m_resultTreeView");
        m_resultTreeView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        m_resultTreeView->setSelectionMode(QAbstractItemView::NoSelection);
        m_resultTreeView->setAllColumnsShowFocus(true);

        verticalLayout->addWidget(m_resultTreeView);

        buttonBox = new QDialogButtonBox(ImportSummaryDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(ImportSummaryDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::clicked, ImportSummaryDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(ImportSummaryDialog);
    } // setupUi

    void retranslateUi(QDialog *ImportSummaryDialog)
    {
        ImportSummaryDialog->setWindowTitle(tr2i18n("Import summary", "@title:window, this is a compound word (one noun)"));
    } // retranslateUi

};

namespace Ui {
    class ImportSummaryDialog: public Ui_ImportSummaryDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // IMPORTSUMMARYDLG_H

