/****************************************************************************
** Meta object code from reading C++ file 'konlinetransferform.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../kmymoney-5.2.2/kmymoney/plugins/views/onlinejoboutbox/konlinetransferform.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'konlinetransferform.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19kOnlineTransferFormE_t {};
} // unnamed namespace

template <> constexpr inline auto kOnlineTransferForm::qt_create_metaobjectdata<qt_meta_tag_ZN19kOnlineTransferFormE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "kOnlineTransferForm",
        "acceptedForSave",
        "",
        "onlineJob",
        "acceptedForSend",
        "accept",
        "reject",
        "setCurrentAccount",
        "accountId",
        "setOnlineJob",
        "duplicateCurrentJob",
        "accountChanged",
        "convertCurrentJob",
        "index",
        "sendJob",
        "loadOnlineJobEditPlugin",
        "onlineJobAdministration::onlineJobEditOffer",
        "plugin",
        "showEditWidget",
        "onlineTaskName",
        "IonlineJobEdit*",
        "widget",
        "checkNotSupportedWidget",
        "setJobReadOnly",
        "enableSendAndEnqueue",
        "enable"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'acceptedForSave'
        QtMocHelpers::SignalData<void(onlineJob)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'acceptedForSend'
        QtMocHelpers::SignalData<void(onlineJob)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'accept'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reject'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setCurrentAccount'
        QtMocHelpers::SlotData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'setOnlineJob'
        QtMocHelpers::SlotData<bool(const onlineJob)>(9, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'duplicateCurrentJob'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'accountChanged'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'convertCurrentJob'
        QtMocHelpers::SlotData<void(const int &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'sendJob'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'loadOnlineJobEditPlugin'
        QtMocHelpers::SlotData<void(const onlineJobAdministration::onlineJobEditOffer &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'showEditWidget'
        QtMocHelpers::SlotData<bool(const QString &)>(18, 2, QMC::AccessPrivate, QMetaType::Bool, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'showEditWidget'
        QtMocHelpers::SlotData<void(IonlineJobEdit *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'checkNotSupportedWidget'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setJobReadOnly'
        QtMocHelpers::SlotData<void(const bool &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'enableSendAndEnqueue'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<kOnlineTransferForm, qt_meta_tag_ZN19kOnlineTransferFormE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject kOnlineTransferForm::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19kOnlineTransferFormE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19kOnlineTransferFormE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19kOnlineTransferFormE_t>.metaTypes,
    nullptr
} };

void kOnlineTransferForm::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<kOnlineTransferForm *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->acceptedForSave((*reinterpret_cast<std::add_pointer_t<onlineJob>>(_a[1]))); break;
        case 1: _t->acceptedForSend((*reinterpret_cast<std::add_pointer_t<onlineJob>>(_a[1]))); break;
        case 2: _t->accept(); break;
        case 3: _t->reject(); break;
        case 4: _t->setCurrentAccount((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: { bool _r = _t->setOnlineJob((*reinterpret_cast<std::add_pointer_t<onlineJob>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->duplicateCurrentJob(); break;
        case 7: _t->accountChanged(); break;
        case 8: _t->convertCurrentJob((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->sendJob(); break;
        case 10: _t->loadOnlineJobEditPlugin((*reinterpret_cast<std::add_pointer_t<onlineJobAdministration::onlineJobEditOffer>>(_a[1]))); break;
        case 11: { bool _r = _t->showEditWidget((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->showEditWidget((*reinterpret_cast<std::add_pointer_t<IonlineJobEdit*>>(_a[1]))); break;
        case 13: _t->checkNotSupportedWidget(); break;
        case 14: _t->setJobReadOnly((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->enableSendAndEnqueue((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< onlineJob >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< onlineJob >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< onlineJob >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (kOnlineTransferForm::*)(onlineJob )>(_a, &kOnlineTransferForm::acceptedForSave, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (kOnlineTransferForm::*)(onlineJob )>(_a, &kOnlineTransferForm::acceptedForSend, 1))
            return;
    }
}

const QMetaObject *kOnlineTransferForm::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *kOnlineTransferForm::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19kOnlineTransferFormE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int kOnlineTransferForm::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void kOnlineTransferForm::acceptedForSave(onlineJob _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void kOnlineTransferForm::acceptedForSend(onlineJob _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
