/********************************************************************************
** Form generated from reading UI file 'investmentwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INVESTMENTWIZARDPAGE_H
#define UI_INVESTMENTWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_InvestmentPage
{
public:
    QHBoxLayout *horizontalLayout;
    QSpacerItem *leftSpacer;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QGridLayout *gridLayout;
    QFrame *line_3;
    QLabel *labelInv_memo;
    QLabel *labelInv_quantity;
    QComboBox *m_amountCol;
    QComboBox *m_priceFraction;
    QLabel *labelInv_date;
    QLabel *labelInv_price;
    QComboBox *m_dateCol;
    QLabel *labelInv_amount;
    QComboBox *m_quantityCol;
    QComboBox *m_priceCol;
    QComboBox *m_symbolCol;
    QPushButton *m_clear;
    QFrame *line_2;
    QLabel *labelInv_priceFraction;
    QLabel *labelInv_symbolCol;
    QComboBox *m_nameCol;
    QLabel *labelInv_nameCol;
    QLabel *labelInv_feeIsPercentage;
    QLabel *labelInv_feeCol;
    QCheckBox *m_feeIsPercentage;
    QComboBox *m_typeCol;
    QComboBox *m_feeCol;
    QLineEdit *m_feeRate;
    QLabel *labelInv_feeRate;
    QLineEdit *m_minFee;
    QLabel *labelInv_type;
    QComboBox *m_memoCol;
    QLabel *labelInv_feeMinValue;
    QFrame *line;
    QPushButton *m_clearFee;
    QPushButton *m_calculateFee;
    QSpacerItem *rightSpacer;

    void setupUi(QWidget *InvestmentPage)
    {
        if (InvestmentPage->objectName().isEmpty())
            InvestmentPage->setObjectName("InvestmentPage");
        InvestmentPage->resize(846, 284);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(InvestmentPage->sizePolicy().hasHeightForWidth());
        InvestmentPage->setSizePolicy(sizePolicy);
        horizontalLayout = new QHBoxLayout(InvestmentPage);
        horizontalLayout->setObjectName("horizontalLayout");
        leftSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(leftSpacer);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(InvestmentPage);
        label->setObjectName("label");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);
        QFont font;
        font.setBold(false);
        label->setFont(font);
        label->setAlignment(Qt::AlignCenter);
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        line_3 = new QFrame(InvestmentPage);
        line_3->setObjectName("line_3");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(line_3->sizePolicy().hasHeightForWidth());
        line_3->setSizePolicy(sizePolicy2);
        line_3->setFrameShape(QFrame::Shape::HLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line_3, 0, 0, 1, 7);

        labelInv_memo = new QLabel(InvestmentPage);
        labelInv_memo->setObjectName("labelInv_memo");
        labelInv_memo->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelInv_memo, 3, 0, 1, 1);

        labelInv_quantity = new QLabel(InvestmentPage);
        labelInv_quantity->setObjectName("labelInv_quantity");
        labelInv_quantity->setEnabled(true);
        labelInv_quantity->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelInv_quantity, 2, 2, 1, 1);

        m_amountCol = new QComboBox(InvestmentPage);
        m_amountCol->setObjectName("m_amountCol");
        sizePolicy1.setHeightForWidth(m_amountCol->sizePolicy().hasHeightForWidth());
        m_amountCol->setSizePolicy(sizePolicy1);
        m_amountCol->setMaxVisibleItems(12);

        gridLayout->addWidget(m_amountCol, 3, 3, 1, 1);

        m_priceFraction = new QComboBox(InvestmentPage);
        m_priceFraction->setObjectName("m_priceFraction");
        sizePolicy1.setHeightForWidth(m_priceFraction->sizePolicy().hasHeightForWidth());
        m_priceFraction->setSizePolicy(sizePolicy1);
        m_priceFraction->setMaxVisibleItems(5);

        gridLayout->addWidget(m_priceFraction, 1, 5, 1, 1);

        labelInv_date = new QLabel(InvestmentPage);
        labelInv_date->setObjectName("labelInv_date");
        labelInv_date->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelInv_date, 1, 0, 1, 1);

        labelInv_price = new QLabel(InvestmentPage);
        labelInv_price->setObjectName("labelInv_price");
        labelInv_price->setEnabled(true);
        labelInv_price->setMaximumSize(QSize(16777215, 16777215));
        labelInv_price->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelInv_price, 1, 2, 1, 1);

        m_dateCol = new QComboBox(InvestmentPage);
        m_dateCol->setObjectName("m_dateCol");
        sizePolicy1.setHeightForWidth(m_dateCol->sizePolicy().hasHeightForWidth());
        m_dateCol->setSizePolicy(sizePolicy1);
        m_dateCol->setMaxVisibleItems(12);

        gridLayout->addWidget(m_dateCol, 1, 1, 1, 1);

        labelInv_amount = new QLabel(InvestmentPage);
        labelInv_amount->setObjectName("labelInv_amount");
        labelInv_amount->setEnabled(true);
        labelInv_amount->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelInv_amount, 3, 2, 1, 1);

        m_quantityCol = new QComboBox(InvestmentPage);
        m_quantityCol->setObjectName("m_quantityCol");
        sizePolicy1.setHeightForWidth(m_quantityCol->sizePolicy().hasHeightForWidth());
        m_quantityCol->setSizePolicy(sizePolicy1);
        m_quantityCol->setMaxVisibleItems(12);

        gridLayout->addWidget(m_quantityCol, 2, 3, 1, 1);

        m_priceCol = new QComboBox(InvestmentPage);
        m_priceCol->setObjectName("m_priceCol");
        sizePolicy1.setHeightForWidth(m_priceCol->sizePolicy().hasHeightForWidth());
        m_priceCol->setSizePolicy(sizePolicy1);
        m_priceCol->setMaxVisibleItems(12);

        gridLayout->addWidget(m_priceCol, 1, 3, 1, 1);

        m_symbolCol = new QComboBox(InvestmentPage);
        m_symbolCol->setObjectName("m_symbolCol");
        sizePolicy1.setHeightForWidth(m_symbolCol->sizePolicy().hasHeightForWidth());
        m_symbolCol->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(m_symbolCol, 2, 5, 1, 1);

        m_clear = new QPushButton(InvestmentPage);
        m_clear->setObjectName("m_clear");
        sizePolicy1.setHeightForWidth(m_clear->sizePolicy().hasHeightForWidth());
        m_clear->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(m_clear, 3, 6, 1, 1);

        line_2 = new QFrame(InvestmentPage);
        line_2->setObjectName("line_2");
        sizePolicy2.setHeightForWidth(line_2->sizePolicy().hasHeightForWidth());
        line_2->setSizePolicy(sizePolicy2);
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line_2, 9, 0, 1, 7);

        labelInv_priceFraction = new QLabel(InvestmentPage);
        labelInv_priceFraction->setObjectName("labelInv_priceFraction");
        labelInv_priceFraction->setMaximumSize(QSize(16777215, 16777215));
        labelInv_priceFraction->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelInv_priceFraction, 1, 4, 1, 1);

        labelInv_symbolCol = new QLabel(InvestmentPage);
        labelInv_symbolCol->setObjectName("labelInv_symbolCol");
        labelInv_symbolCol->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelInv_symbolCol, 2, 4, 1, 1);

        m_nameCol = new QComboBox(InvestmentPage);
        m_nameCol->setObjectName("m_nameCol");
        sizePolicy1.setHeightForWidth(m_nameCol->sizePolicy().hasHeightForWidth());
        m_nameCol->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(m_nameCol, 3, 5, 1, 1);

        labelInv_nameCol = new QLabel(InvestmentPage);
        labelInv_nameCol->setObjectName("labelInv_nameCol");
        labelInv_nameCol->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelInv_nameCol, 3, 4, 1, 1);

        labelInv_feeIsPercentage = new QLabel(InvestmentPage);
        labelInv_feeIsPercentage->setObjectName("labelInv_feeIsPercentage");
        labelInv_feeIsPercentage->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelInv_feeIsPercentage, 7, 2, 1, 1);

        labelInv_feeCol = new QLabel(InvestmentPage);
        labelInv_feeCol->setObjectName("labelInv_feeCol");
        labelInv_feeCol->setLayoutDirection(Qt::LeftToRight);
        labelInv_feeCol->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelInv_feeCol, 7, 0, 1, 1);

        m_feeIsPercentage = new QCheckBox(InvestmentPage);
        m_feeIsPercentage->setObjectName("m_feeIsPercentage");
        sizePolicy1.setHeightForWidth(m_feeIsPercentage->sizePolicy().hasHeightForWidth());
        m_feeIsPercentage->setSizePolicy(sizePolicy1);
        m_feeIsPercentage->setLayoutDirection(Qt::RightToLeft);

        gridLayout->addWidget(m_feeIsPercentage, 7, 3, 1, 1, Qt::AlignRight);

        m_typeCol = new QComboBox(InvestmentPage);
        m_typeCol->setObjectName("m_typeCol");
        sizePolicy1.setHeightForWidth(m_typeCol->sizePolicy().hasHeightForWidth());
        m_typeCol->setSizePolicy(sizePolicy1);
        m_typeCol->setMaxVisibleItems(12);

        gridLayout->addWidget(m_typeCol, 2, 1, 1, 1);

        m_feeCol = new QComboBox(InvestmentPage);
        m_feeCol->setObjectName("m_feeCol");
        sizePolicy1.setHeightForWidth(m_feeCol->sizePolicy().hasHeightForWidth());
        m_feeCol->setSizePolicy(sizePolicy1);
        m_feeCol->setMaxVisibleItems(12);

        gridLayout->addWidget(m_feeCol, 7, 1, 1, 1);

        m_feeRate = new QLineEdit(InvestmentPage);
        m_feeRate->setObjectName("m_feeRate");
        sizePolicy1.setHeightForWidth(m_feeRate->sizePolicy().hasHeightForWidth());
        m_feeRate->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(m_feeRate, 8, 1, 1, 1);

        labelInv_feeRate = new QLabel(InvestmentPage);
        labelInv_feeRate->setObjectName("labelInv_feeRate");
        labelInv_feeRate->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelInv_feeRate, 8, 0, 1, 1);

        m_minFee = new QLineEdit(InvestmentPage);
        m_minFee->setObjectName("m_minFee");
        sizePolicy1.setHeightForWidth(m_minFee->sizePolicy().hasHeightForWidth());
        m_minFee->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(m_minFee, 8, 3, 1, 1);

        labelInv_type = new QLabel(InvestmentPage);
        labelInv_type->setObjectName("labelInv_type");
        labelInv_type->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelInv_type, 2, 0, 1, 1);

        m_memoCol = new QComboBox(InvestmentPage);
        m_memoCol->setObjectName("m_memoCol");
        sizePolicy1.setHeightForWidth(m_memoCol->sizePolicy().hasHeightForWidth());
        m_memoCol->setSizePolicy(sizePolicy1);
        m_memoCol->setMaxVisibleItems(12);

        gridLayout->addWidget(m_memoCol, 3, 1, 1, 1);

        labelInv_feeMinValue = new QLabel(InvestmentPage);
        labelInv_feeMinValue->setObjectName("labelInv_feeMinValue");
        labelInv_feeMinValue->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelInv_feeMinValue, 8, 2, 1, 1);

        line = new QFrame(InvestmentPage);
        line->setObjectName("line");
        sizePolicy2.setHeightForWidth(line->sizePolicy().hasHeightForWidth());
        line->setSizePolicy(sizePolicy2);
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line, 6, 0, 1, 7);

        m_clearFee = new QPushButton(InvestmentPage);
        m_clearFee->setObjectName("m_clearFee");
        sizePolicy1.setHeightForWidth(m_clearFee->sizePolicy().hasHeightForWidth());
        m_clearFee->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(m_clearFee, 7, 6, 1, 1);

        m_calculateFee = new QPushButton(InvestmentPage);
        m_calculateFee->setObjectName("m_calculateFee");
        sizePolicy1.setHeightForWidth(m_calculateFee->sizePolicy().hasHeightForWidth());
        m_calculateFee->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(m_calculateFee, 8, 6, 1, 1);


        verticalLayout->addLayout(gridLayout);


        horizontalLayout->addLayout(verticalLayout);

        rightSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(rightSpacer);

        horizontalLayout->setStretch(0, 1);
        horizontalLayout->setStretch(1, 50);
        horizontalLayout->setStretch(2, 1);
        QWidget::setTabOrder(m_dateCol, m_typeCol);
        QWidget::setTabOrder(m_typeCol, m_memoCol);
        QWidget::setTabOrder(m_memoCol, m_priceCol);
        QWidget::setTabOrder(m_priceCol, m_quantityCol);
        QWidget::setTabOrder(m_quantityCol, m_amountCol);
        QWidget::setTabOrder(m_amountCol, m_priceFraction);
        QWidget::setTabOrder(m_priceFraction, m_symbolCol);
        QWidget::setTabOrder(m_symbolCol, m_nameCol);
        QWidget::setTabOrder(m_nameCol, m_feeCol);
        QWidget::setTabOrder(m_feeCol, m_feeIsPercentage);
        QWidget::setTabOrder(m_feeIsPercentage, m_feeRate);
        QWidget::setTabOrder(m_feeRate, m_minFee);
        QWidget::setTabOrder(m_minFee, m_calculateFee);
        QWidget::setTabOrder(m_calculateFee, m_clear);
        QWidget::setTabOrder(m_clear, m_clearFee);

        retranslateUi(InvestmentPage);

        QMetaObject::connectSlotsByName(InvestmentPage);
    } // setupUi

    void retranslateUi(QWidget *InvestmentPage)
    {
        InvestmentPage->setWindowTitle(tr2i18n("Investment Wizard Page", nullptr));
        label->setText(tr2i18n("Please select the appropriate columns to use, corresponding to your data.\n"
"If there is just a single security in the file, enter its name below.\n"
"Otherwise, select the column for the symbol, and for the description or name.", nullptr));
#if QT_CONFIG(accessibility)
        labelInv_memo->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelInv_memo->setText(tr2i18n("Memo", nullptr));
#if QT_CONFIG(accessibility)
        labelInv_quantity->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelInv_quantity->setText(tr2i18n("Quantity", nullptr));
#if QT_CONFIG(tooltip)
        m_amountCol->setToolTip(tr2i18n("Select column containing amount field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_priceFraction->setToolTip(tr2i18n("If necessary, select a fraction/multiplier for compatibility\n"
"between imported and stored prices. For instance, if the\n"
"import price is in cents but the file is priced in dollars, select 0.01.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        labelInv_date->setToolTip(tr2i18n("Select column containing date field", "verb, Select column containing date field"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        labelInv_date->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelInv_date->setText(tr2i18n("Date", nullptr));
#if QT_CONFIG(accessibility)
        labelInv_price->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelInv_price->setText(tr2i18n("Price", nullptr));
#if QT_CONFIG(tooltip)
        m_dateCol->setToolTip(tr2i18n("Select column containing date field.", nullptr));
#endif // QT_CONFIG(tooltip)
        labelInv_amount->setText(tr2i18n("Amount", "column for monetary value"));
#if QT_CONFIG(tooltip)
        m_quantityCol->setToolTip(tr2i18n("Select column containing quantity field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_priceCol->setToolTip(tr2i18n("Select column containing price field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_symbolCol->setToolTip(tr2i18n("If more than one stock is present in the file,\n"
"and if the stock symbols are shown in one of the columns,\n"
"select it here.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_clear->setToolTip(tr2i18n("Clear selected column entries.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_clear->setText(tr2i18n("Clear", "verb, to clear selected columns"));
#if QT_CONFIG(accessibility)
        labelInv_priceFraction->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelInv_priceFraction->setText(tr2i18n("Price Fraction", "noun, conversion fraction/multiplier to use to make compatible the csv pricing with KMM pricing."));
        labelInv_symbolCol->setText(tr2i18n("Symbol", "@label:listbox stock symbol"));
#if QT_CONFIG(tooltip)
        m_nameCol->setToolTip(tr2i18n("If a descriptive column also contains an activity type, select it here.\n"
"See also Filter text field.", nullptr));
#endif // QT_CONFIG(tooltip)
        labelInv_nameCol->setText(tr2i18n("Name", nullptr));
        labelInv_feeIsPercentage->setText(tr2i18n("Fee is Percentage", nullptr));
#if QT_CONFIG(accessibility)
        labelInv_feeCol->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelInv_feeCol->setText(tr2i18n("Fee Column", "column containing fee field"));
#if QT_CONFIG(tooltip)
        m_feeIsPercentage->setToolTip(tr2i18n("Check box if any fee is a percentage figure rather than a value.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_typeCol->setToolTip(tr2i18n("Select column containing investment type field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_feeCol->setToolTip(tr2i18n("If necessary, select column containing fee.\n"
"Caution. This might already have been\n"
"incorporated into the price.", nullptr));
#endif // QT_CONFIG(tooltip)
        labelInv_feeRate->setText(tr2i18n("Fee rate", nullptr));
#if QT_CONFIG(accessibility)
        labelInv_type->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelInv_type->setText(tr2i18n("Type/Action", "column containing investment type/action field"));
#if QT_CONFIG(tooltip)
        m_memoCol->setToolTip(tr2i18n("Select column containing memo field.\n"
"More than one column may be entered successively.\n"
"May also be selected to contain a copy of the type or name column.", nullptr));
#endif // QT_CONFIG(tooltip)
        labelInv_feeMinValue->setText(tr2i18n("Minimal Fee", nullptr));
        m_clearFee->setText(tr2i18n("Clear Fee", nullptr));
        m_calculateFee->setText(tr2i18n("Calculate Fee", nullptr));
    } // retranslateUi

};

namespace Ui {
    class InvestmentPage: public Ui_InvestmentPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // INVESTMENTWIZARDPAGE_H

