/********************************************************************************
** Form generated from reading UI file 'kchooseimportexportdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KCHOOSEIMPORTEXPORTDLG_H
#define UI_KCHOOSEIMPORTEXPORTDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kcombobox.h"

QT_BEGIN_NAMESPACE

class Ui_KChooseImportExportDlg
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *topLabel;
    QFrame *Line1;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *Spacer1;
    QLabel *promptLabel;
    KComboBox *typeCombo;
    QLabel *descriptionLabel;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KChooseImportExportDlg)
    {
        if (KChooseImportExportDlg->objectName().isEmpty())
            KChooseImportExportDlg->setObjectName("KChooseImportExportDlg");
        KChooseImportExportDlg->resize(530, 309);
        vboxLayout = new QVBoxLayout(KChooseImportExportDlg);
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        topLabel = new QLabel(KChooseImportExportDlg);
        topLabel->setObjectName("topLabel");
        topLabel->setWordWrap(false);

        hboxLayout->addWidget(topLabel);


        vboxLayout->addLayout(hboxLayout);

        Line1 = new QFrame(KChooseImportExportDlg);
        Line1->setObjectName("Line1");
        Line1->setFrameShadow(QFrame::Raised);

        vboxLayout->addWidget(Line1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        Spacer1 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(Spacer1);

        promptLabel = new QLabel(KChooseImportExportDlg);
        promptLabel->setObjectName("promptLabel");
        promptLabel->setWordWrap(false);

        hboxLayout1->addWidget(promptLabel);

        typeCombo = new KComboBox(KChooseImportExportDlg);
        typeCombo->addItem(QString());
        typeCombo->addItem(QString());
        typeCombo->setObjectName("typeCombo");
        typeCombo->setMinimumSize(QSize(100, 0));

        hboxLayout1->addWidget(typeCombo);


        vboxLayout->addLayout(hboxLayout1);

        descriptionLabel = new QLabel(KChooseImportExportDlg);
        descriptionLabel->setObjectName("descriptionLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(descriptionLabel->sizePolicy().hasHeightForWidth());
        descriptionLabel->setSizePolicy(sizePolicy);
        descriptionLabel->setMinimumSize(QSize(410, 100));
        descriptionLabel->setWordWrap(false);

        vboxLayout->addWidget(descriptionLabel);

        buttonBox = new QDialogButtonBox(KChooseImportExportDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(KChooseImportExportDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KChooseImportExportDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KChooseImportExportDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KChooseImportExportDlg);
    } // setupUi

    void retranslateUi(QDialog *KChooseImportExportDlg)
    {
        KChooseImportExportDlg->setWindowTitle(tr2i18n("Choose Import Type Dialog", nullptr));
        topLabel->setText(tr2i18n("Please choose the type of import you wish to perform.   A simple explanation\n"
"of the import type is available at the bottom of the screen and is updated when\n"
"you select an item from the choice box.\n"
"\n"
"Once you have chosen an import type please press the OK button.", nullptr));
        promptLabel->setText(tr2i18n("Choose import type:", nullptr));
        typeCombo->setItemText(0, tr2i18n("QIF", nullptr));
        typeCombo->setItemText(1, tr2i18n("CSV", nullptr));

        descriptionLabel->setText(tr2i18n("Some description", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KChooseImportExportDlg: public Ui_KChooseImportExportDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KCHOOSEIMPORTEXPORTDLG_H

