/********************************************************************************
** Form generated from reading UI file 'kbudgetview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KBUDGETVIEW_H
#define UI_KBUDGETVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <amountedit.h>
#include <klocalizedstring.h>
#include "kbudgetvalues.h"
#include "klineedit.h"
#include "kmmemptyview.h"
#include "kmymoneyaccounttreeview.h"

QT_BEGIN_NAMESPACE

class Ui_KBudgetView
{
public:
    QVBoxLayout *verticalLayout_2;
    QSplitter *m_splitter;
    QGroupBox *groupBox1;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QToolButton *m_newButton;
    QToolButton *m_renameButton;
    QToolButton *m_deleteButton;
    KMMEmptyTableView *m_budgetList;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    KMyMoneyAccountTreeView *m_accountTree;
    QLabel *m_balanceLabel;
    QHBoxLayout *horizontalLayout;
    QPushButton *m_updateButton;
    QPushButton *m_resetButton;
    QPushButton *m_collapseButton;
    QPushButton *m_expandButton;
    QCheckBox *m_hideUnusedButton;
    KLineEdit *m_searchWidget;
    QGroupBox *m_assignmentBox;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout1;
    QLabel *textLabel3;
    KLineEdit *m_leAccounts;
    QLabel *textLabel1;
    AmountEdit *m_accountTotal;
    QCheckBox *m_cbBudgetSubaccounts;
    QHBoxLayout *hboxLayout2;
    KBudgetValues *m_budgetValue;
    QSpacerItem *spacer2;

    void setupUi(QWidget *KBudgetView)
    {
        if (KBudgetView->objectName().isEmpty())
            KBudgetView->setObjectName("KBudgetView");
        KBudgetView->resize(862, 305);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(KBudgetView->sizePolicy().hasHeightForWidth());
        KBudgetView->setSizePolicy(sizePolicy);
        KBudgetView->setFocusPolicy(Qt::TabFocus);
        verticalLayout_2 = new QVBoxLayout(KBudgetView);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        m_splitter = new QSplitter(KBudgetView);
        m_splitter->setObjectName("m_splitter");
        m_splitter->setOrientation(Qt::Horizontal);
        m_splitter->setChildrenCollapsible(false);
        groupBox1 = new QGroupBox(m_splitter);
        groupBox1->setObjectName("groupBox1");
        sizePolicy.setHeightForWidth(groupBox1->sizePolicy().hasHeightForWidth());
        groupBox1->setSizePolicy(sizePolicy);
        vboxLayout = new QVBoxLayout(groupBox1);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_newButton = new QToolButton(groupBox1);
        m_newButton->setObjectName("m_newButton");

        hboxLayout->addWidget(m_newButton);

        m_renameButton = new QToolButton(groupBox1);
        m_renameButton->setObjectName("m_renameButton");

        hboxLayout->addWidget(m_renameButton);

        m_deleteButton = new QToolButton(groupBox1);
        m_deleteButton->setObjectName("m_deleteButton");

        hboxLayout->addWidget(m_deleteButton);


        vboxLayout->addLayout(hboxLayout);

        m_budgetList = new KMMEmptyTableView(groupBox1);
        m_budgetList->setObjectName("m_budgetList");
        m_budgetList->setAlternatingRowColors(true);
        m_budgetList->setSelectionMode(QAbstractItemView::SingleSelection);
        m_budgetList->setSelectionBehavior(QAbstractItemView::SelectRows);
        m_budgetList->setSortingEnabled(true);
        m_budgetList->horizontalHeader()->setStretchLastSection(true);
        m_budgetList->verticalHeader()->setVisible(false);

        vboxLayout->addWidget(m_budgetList);

        m_splitter->addWidget(groupBox1);
        layoutWidget = new QWidget(m_splitter);
        layoutWidget->setObjectName("layoutWidget");
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        m_accountTree = new KMyMoneyAccountTreeView(layoutWidget);
        m_accountTree->setObjectName("m_accountTree");

        verticalLayout->addWidget(m_accountTree);

        m_balanceLabel = new QLabel(layoutWidget);
        m_balanceLabel->setObjectName("m_balanceLabel");
        m_balanceLabel->setTextFormat(Qt::RichText);
        m_balanceLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        verticalLayout->addWidget(m_balanceLabel);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        m_updateButton = new QPushButton(layoutWidget);
        m_updateButton->setObjectName("m_updateButton");
        m_updateButton->setEnabled(false);

        horizontalLayout->addWidget(m_updateButton);

        m_resetButton = new QPushButton(layoutWidget);
        m_resetButton->setObjectName("m_resetButton");
        m_resetButton->setEnabled(false);

        horizontalLayout->addWidget(m_resetButton);

        m_collapseButton = new QPushButton(layoutWidget);
        m_collapseButton->setObjectName("m_collapseButton");

        horizontalLayout->addWidget(m_collapseButton);

        m_expandButton = new QPushButton(layoutWidget);
        m_expandButton->setObjectName("m_expandButton");

        horizontalLayout->addWidget(m_expandButton);

        m_hideUnusedButton = new QCheckBox(layoutWidget);
        m_hideUnusedButton->setObjectName("m_hideUnusedButton");

        horizontalLayout->addWidget(m_hideUnusedButton);

        m_searchWidget = new KLineEdit(layoutWidget);
        m_searchWidget->setObjectName("m_searchWidget");
        m_searchWidget->setProperty("showClearButton", QVariant(true));

        horizontalLayout->addWidget(m_searchWidget);


        verticalLayout->addLayout(horizontalLayout);

        m_assignmentBox = new QGroupBox(layoutWidget);
        m_assignmentBox->setObjectName("m_assignmentBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(5);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_assignmentBox->sizePolicy().hasHeightForWidth());
        m_assignmentBox->setSizePolicy(sizePolicy1);
        vboxLayout1 = new QVBoxLayout(m_assignmentBox);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        vboxLayout1->setObjectName("vboxLayout1");
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        textLabel3 = new QLabel(m_assignmentBox);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        hboxLayout1->addWidget(textLabel3);

        m_leAccounts = new KLineEdit(m_assignmentBox);
        m_leAccounts->setObjectName("m_leAccounts");
        m_leAccounts->setEnabled(false);
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(m_leAccounts->sizePolicy().hasHeightForWidth());
        m_leAccounts->setSizePolicy(sizePolicy2);
        m_leAccounts->setFocusPolicy(Qt::NoFocus);

        hboxLayout1->addWidget(m_leAccounts);

        textLabel1 = new QLabel(m_assignmentBox);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        hboxLayout1->addWidget(textLabel1);

        m_accountTotal = new AmountEdit(m_assignmentBox);
        m_accountTotal->setObjectName("m_accountTotal");
        m_accountTotal->setFocusPolicy(Qt::NoFocus);
        m_accountTotal->setReadOnly(true);
        m_accountTotal->setProperty("calculatorButtonVisibility", QVariant(false));
        m_accountTotal->setProperty("resetButtonVisibility", QVariant(false));

        hboxLayout1->addWidget(m_accountTotal);


        vboxLayout1->addLayout(hboxLayout1);

        m_cbBudgetSubaccounts = new QCheckBox(m_assignmentBox);
        m_cbBudgetSubaccounts->setObjectName("m_cbBudgetSubaccounts");

        vboxLayout1->addWidget(m_cbBudgetSubaccounts);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        m_budgetValue = new KBudgetValues(m_assignmentBox);
        m_budgetValue->setObjectName("m_budgetValue");

        hboxLayout2->addWidget(m_budgetValue);

        spacer2 = new QSpacerItem(21, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout2->addItem(spacer2);


        vboxLayout1->addLayout(hboxLayout2);


        verticalLayout->addWidget(m_assignmentBox);

        m_splitter->addWidget(layoutWidget);

        verticalLayout_2->addWidget(m_splitter);


        retranslateUi(KBudgetView);
        QObject::connect(m_collapseButton, &QPushButton::clicked, m_accountTree, qOverload<>(&KMyMoneyAccountTreeView::collapseAll));
        QObject::connect(m_expandButton, &QPushButton::clicked, m_accountTree, qOverload<>(&KMyMoneyAccountTreeView::expandAll));

        QMetaObject::connectSlotsByName(KBudgetView);
    } // setupUi

    void retranslateUi(QWidget *KBudgetView)
    {
        groupBox1->setTitle(tr2i18n("Your budgets", nullptr));
        m_newButton->setText(tr2i18n("New", "New budget button"));
        m_renameButton->setText(tr2i18n("Ren", "Rename budget button"));
        m_deleteButton->setText(tr2i18n("Del", "Delete budget button"));
#if QT_CONFIG(tooltip)
        m_updateButton->setToolTip(tr2i18n("Accepts the entered values and stores the budget.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_updateButton->setWhatsThis(tr2i18n("Use this to store the modified data.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_updateButton->setText(tr2i18n("Update", nullptr));
#if QT_CONFIG(shortcut)
        m_updateButton->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        m_resetButton->setToolTip(tr2i18n("Revert budget to last saved state.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_resetButton->setWhatsThis(tr2i18n("Use this to discard the modified data.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_resetButton->setText(tr2i18n("Reset", nullptr));
#if QT_CONFIG(shortcut)
        m_resetButton->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        m_collapseButton->setToolTip(tr2i18n("Collapse the items of the account tree", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_expandButton->setToolTip(tr2i18n("Expand the items of the account tree", nullptr));
#endif // QT_CONFIG(tooltip)
        m_hideUnusedButton->setText(tr2i18n("Hide unused budget categories", nullptr));
        m_assignmentBox->setTitle(tr2i18n("Assignments", nullptr));
        textLabel3->setText(tr2i18n("Account", nullptr));
        textLabel1->setText(tr2i18n("Total", "Budget total"));
        m_cbBudgetSubaccounts->setText(tr2i18n("Include subaccounts", nullptr));
        (void)KBudgetView;
    } // retranslateUi

};

namespace Ui {
    class KBudgetView: public Ui_KBudgetView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KBUDGETVIEW_H

