/********************************************************************************
** Form generated from reading UI file 'interestcalculationwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INTERESTCALCULATIONWIZARDPAGE_H
#define UI_INTERESTCALCULATIONWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_InterestCalculationWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel12_2;
    QSpacerItem *Spacer61;
    QHBoxLayout *hboxLayout;
    QFrame *frame_6;
    QVBoxLayout *vboxLayout1;
    QRadioButton *m_interestOnReceptionButton;
    QRadioButton *m_interestOnPaymentButton;
    QSpacerItem *Spacer14_2_2_2_2;
    QSpacerItem *Spacer62;
    QButtonGroup *ButtonGroup5;

    void setupUi(QWizardPage *InterestCalculationWizardPage)
    {
        if (InterestCalculationWizardPage->objectName().isEmpty())
            InterestCalculationWizardPage->setObjectName("InterestCalculationWizardPage");
        vboxLayout = new QVBoxLayout(InterestCalculationWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel12_2 = new QLabel(InterestCalculationWizardPage);
        TextLabel12_2->setObjectName("TextLabel12_2");
        TextLabel12_2->setAlignment(Qt::AlignTop);
        TextLabel12_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel12_2);

        Spacer61 = new QSpacerItem(0, 69, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer61);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        frame_6 = new QFrame(InterestCalculationWizardPage);
        frame_6->setObjectName("frame_6");
        frame_6->setFrameShape(QFrame::NoFrame);
        vboxLayout1 = new QVBoxLayout(frame_6);
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        m_interestOnReceptionButton = new QRadioButton(frame_6);
        ButtonGroup5 = new QButtonGroup(InterestCalculationWizardPage);
        ButtonGroup5->setObjectName("ButtonGroup5");
        ButtonGroup5->addButton(m_interestOnReceptionButton);
        m_interestOnReceptionButton->setObjectName("m_interestOnReceptionButton");

        vboxLayout1->addWidget(m_interestOnReceptionButton);

        m_interestOnPaymentButton = new QRadioButton(frame_6);
        ButtonGroup5->addButton(m_interestOnPaymentButton);
        m_interestOnPaymentButton->setObjectName("m_interestOnPaymentButton");

        vboxLayout1->addWidget(m_interestOnPaymentButton);


        hboxLayout->addWidget(frame_6);

        Spacer14_2_2_2_2 = new QSpacerItem(296, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer14_2_2_2_2);


        vboxLayout->addLayout(hboxLayout);

        Spacer62 = new QSpacerItem(0, 69, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer62);


        retranslateUi(InterestCalculationWizardPage);

        QMetaObject::connectSlotsByName(InterestCalculationWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *InterestCalculationWizardPage)
    {
        TextLabel12_2->setText(tr2i18n("\n"
"When does the actual interest rate get calculated?", nullptr));
        m_interestOnReceptionButton->setText(tr2i18n("When the payment is received.", nullptr));
        m_interestOnPaymentButton->setText(tr2i18n("When the payment is due.", nullptr));
        (void)InterestCalculationWizardPage;
    } // retranslateUi

};

namespace Ui {
    class InterestCalculationWizardPage: public Ui_InterestCalculationWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // INTERESTCALCULATIONWIZARDPAGE_H

