/****************************************************************************
** Meta object code from reading C++ file 'konlinejoboutboxview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../kmymoney-5.2.2/kmymoney/plugins/views/onlinejoboutbox/konlinejoboutboxview.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'konlinejoboutboxview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20KOnlineJobOutboxViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KOnlineJobOutboxView::qt_create_metaobjectdata<qt_meta_tag_ZN20KOnlineJobOutboxViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOnlineJobOutboxView",
        "sendJobs",
        "",
        "QList<onlineJob>",
        "editJob",
        "newCreditTransfer",
        "updateActions",
        "SelectedObjects",
        "selections",
        "setOnlinePlugins",
        "QMap<QString,KMyMoneyPlugin::OnlinePlugin*>*",
        "plugins",
        "updateSelection",
        "slotRemoveJob",
        "slotSendJobs",
        "slotSendAllSendableJobs",
        "slotSendSelectedJobs",
        "slotEditJob",
        "QModelIndex",
        "slotOnlineJobSave",
        "onlineJob",
        "job",
        "slotOnlineJobSend",
        "jobs",
        "slotOnlineJobLog",
        "onlineJobIds",
        "slotNewCreditTransfer"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sendJobs'
        QtMocHelpers::SignalData<void(QList<onlineJob>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'editJob'
        QtMocHelpers::SignalData<void(QString)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'newCreditTransfer'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateActions'
        QtMocHelpers::SlotData<void(const SelectedObjects &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'setOnlinePlugins'
        QtMocHelpers::SlotData<void(QMap<QString,KMyMoneyPlugin::OnlinePlugin*> *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'updateSelection'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRemoveJob'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSendJobs'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSendAllSendableJobs'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSendSelectedJobs'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEditJob'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEditJob'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 2 },
        }}),
        // Slot 'slotOnlineJobSave'
        QtMocHelpers::SlotData<QString(onlineJob)>(19, 2, QMC::AccessPrivate, QMetaType::QString, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'slotOnlineJobSend'
        QtMocHelpers::SlotData<void(onlineJob)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'slotOnlineJobSend'
        QtMocHelpers::SlotData<void(QList<onlineJob>)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 23 },
        }}),
        // Slot 'slotOnlineJobLog'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOnlineJobLog'
        QtMocHelpers::SlotData<void(const QStringList &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 25 },
        }}),
        // Slot 'slotNewCreditTransfer'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KOnlineJobOutboxView, qt_meta_tag_ZN20KOnlineJobOutboxViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KOnlineJobOutboxView::staticMetaObject = { {
    QMetaObject::SuperData::link<KMyMoneyViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KOnlineJobOutboxViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KOnlineJobOutboxViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20KOnlineJobOutboxViewE_t>.metaTypes,
    nullptr
} };

void KOnlineJobOutboxView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KOnlineJobOutboxView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sendJobs((*reinterpret_cast<std::add_pointer_t<QList<onlineJob>>>(_a[1]))); break;
        case 1: _t->editJob((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->newCreditTransfer(); break;
        case 3: _t->updateActions((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 4: _t->setOnlinePlugins((*reinterpret_cast<std::add_pointer_t<QMap<QString,KMyMoneyPlugin::OnlinePlugin*>*>>(_a[1]))); break;
        case 5: _t->updateSelection(); break;
        case 6: _t->slotRemoveJob(); break;
        case 7: _t->slotSendJobs(); break;
        case 8: _t->slotSendAllSendableJobs(); break;
        case 9: _t->slotSendSelectedJobs(); break;
        case 10: _t->slotEditJob(); break;
        case 11: _t->slotEditJob((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 12: { QString _r = _t->slotOnlineJobSave((*reinterpret_cast<std::add_pointer_t<onlineJob>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->slotOnlineJobSend((*reinterpret_cast<std::add_pointer_t<onlineJob>>(_a[1]))); break;
        case 14: _t->slotOnlineJobSend((*reinterpret_cast<std::add_pointer_t<QList<onlineJob>>>(_a[1]))); break;
        case 15: _t->slotOnlineJobLog(); break;
        case 16: _t->slotOnlineJobLog((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 17: _t->slotNewCreditTransfer(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<onlineJob> >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< onlineJob >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< onlineJob >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<onlineJob> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KOnlineJobOutboxView::*)(QList<onlineJob> )>(_a, &KOnlineJobOutboxView::sendJobs, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KOnlineJobOutboxView::*)(QString )>(_a, &KOnlineJobOutboxView::editJob, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KOnlineJobOutboxView::*)()>(_a, &KOnlineJobOutboxView::newCreditTransfer, 2))
            return;
    }
}

const QMetaObject *KOnlineJobOutboxView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KOnlineJobOutboxView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KOnlineJobOutboxViewE_t>.strings))
        return static_cast<void*>(this);
    return KMyMoneyViewBase::qt_metacast(_clname);
}

int KOnlineJobOutboxView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KMyMoneyViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void KOnlineJobOutboxView::sendJobs(QList<onlineJob> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KOnlineJobOutboxView::editJob(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KOnlineJobOutboxView::newCreditTransfer()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
