/********************************************************************************
** Form generated from reading UI file 'icalendarsettingsdecl.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ICALENDARSETTINGSDECL_H
#define UI_ICALENDARSETTINGSDECL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "klineedit.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_ICalendarSettingsDecl
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *vboxLayout;
    QCheckBox *kcfg_writeEventsVsTodos;
    QCheckBox *kcfg_createAlarm;
    QHBoxLayout *hboxLayout;
    QSpinBox *kcfg_timeUnits;
    KComboBox *kcfg_timeUnitInSeconds;
    KComboBox *kcfg_beforeAfter;
    QHBoxLayout *hboxLayout1;
    QCheckBox *kcfg_repeatingReminders;
    QVBoxLayout *vboxLayout1;
    QLabel *textLabel1;
    QLabel *textLabel2;
    QVBoxLayout *vboxLayout2;
    QSpinBox *kcfg_numberOfReminders;
    QSpinBox *kcfg_intervalBetweenRemindersTimeUnits;
    QVBoxLayout *vboxLayout3;
    QLabel *textLabel4;
    KComboBox *kcfg_intervalBetweenRemindersTimeUnitInSeconds;
    QHBoxLayout *hboxLayout2;
    QLabel *iCalendar_file;
    KUrlRequester *kcfg_icalendarFile;
    QSpacerItem *spacer1;

    void setupUi(QWidget *ICalendarSettingsDecl)
    {
        if (ICalendarSettingsDecl->objectName().isEmpty())
            ICalendarSettingsDecl->setObjectName("ICalendarSettingsDecl");
        ICalendarSettingsDecl->resize(475, 274);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ICalendarSettingsDecl->sizePolicy().hasHeightForWidth());
        ICalendarSettingsDecl->setSizePolicy(sizePolicy);
        ICalendarSettingsDecl->setMinimumSize(QSize(475, 165));
        gridLayout = new QGridLayout(ICalendarSettingsDecl);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        vboxLayout = new QVBoxLayout();
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        kcfg_writeEventsVsTodos = new QCheckBox(ICalendarSettingsDecl);
        kcfg_writeEventsVsTodos->setObjectName("kcfg_writeEventsVsTodos");

        vboxLayout->addWidget(kcfg_writeEventsVsTodos);

        kcfg_createAlarm = new QCheckBox(ICalendarSettingsDecl);
        kcfg_createAlarm->setObjectName("kcfg_createAlarm");

        vboxLayout->addWidget(kcfg_createAlarm);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        kcfg_timeUnits = new QSpinBox(ICalendarSettingsDecl);
        kcfg_timeUnits->setObjectName("kcfg_timeUnits");
        kcfg_timeUnits->setMinimum(1);
        kcfg_timeUnits->setMaximum(360);

        hboxLayout->addWidget(kcfg_timeUnits);

        kcfg_timeUnitInSeconds = new KComboBox(ICalendarSettingsDecl);
        kcfg_timeUnitInSeconds->setObjectName("kcfg_timeUnitInSeconds");
        kcfg_timeUnitInSeconds->setMaxCount(3);
        kcfg_timeUnitInSeconds->setDuplicatesEnabled(false);

        hboxLayout->addWidget(kcfg_timeUnitInSeconds);

        kcfg_beforeAfter = new KComboBox(ICalendarSettingsDecl);
        kcfg_beforeAfter->setObjectName("kcfg_beforeAfter");

        hboxLayout->addWidget(kcfg_beforeAfter);


        vboxLayout->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        kcfg_repeatingReminders = new QCheckBox(ICalendarSettingsDecl);
        kcfg_repeatingReminders->setObjectName("kcfg_repeatingReminders");

        hboxLayout1->addWidget(kcfg_repeatingReminders);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName("vboxLayout1");
        textLabel1 = new QLabel(ICalendarSettingsDecl);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        vboxLayout1->addWidget(textLabel1);

        textLabel2 = new QLabel(ICalendarSettingsDecl);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        vboxLayout1->addWidget(textLabel2);


        hboxLayout1->addLayout(vboxLayout1);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(6);
        vboxLayout2->setObjectName("vboxLayout2");
        kcfg_numberOfReminders = new QSpinBox(ICalendarSettingsDecl);
        kcfg_numberOfReminders->setObjectName("kcfg_numberOfReminders");
        kcfg_numberOfReminders->setMinimum(1);

        vboxLayout2->addWidget(kcfg_numberOfReminders);

        kcfg_intervalBetweenRemindersTimeUnits = new QSpinBox(ICalendarSettingsDecl);
        kcfg_intervalBetweenRemindersTimeUnits->setObjectName("kcfg_intervalBetweenRemindersTimeUnits");
        kcfg_intervalBetweenRemindersTimeUnits->setMinimum(1);
        kcfg_intervalBetweenRemindersTimeUnits->setMaximum(360);

        vboxLayout2->addWidget(kcfg_intervalBetweenRemindersTimeUnits);


        hboxLayout1->addLayout(vboxLayout2);

        vboxLayout3 = new QVBoxLayout();
        vboxLayout3->setSpacing(6);
        vboxLayout3->setObjectName("vboxLayout3");
        textLabel4 = new QLabel(ICalendarSettingsDecl);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        vboxLayout3->addWidget(textLabel4);

        kcfg_intervalBetweenRemindersTimeUnitInSeconds = new KComboBox(ICalendarSettingsDecl);
        kcfg_intervalBetweenRemindersTimeUnitInSeconds->setObjectName("kcfg_intervalBetweenRemindersTimeUnitInSeconds");

        vboxLayout3->addWidget(kcfg_intervalBetweenRemindersTimeUnitInSeconds);


        hboxLayout1->addLayout(vboxLayout3);


        vboxLayout->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        iCalendar_file = new QLabel(ICalendarSettingsDecl);
        iCalendar_file->setObjectName("iCalendar_file");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(iCalendar_file->sizePolicy().hasHeightForWidth());
        iCalendar_file->setSizePolicy(sizePolicy1);
        iCalendar_file->setWordWrap(false);

        hboxLayout2->addWidget(iCalendar_file);

        kcfg_icalendarFile = new KUrlRequester(ICalendarSettingsDecl);
        kcfg_icalendarFile->setObjectName("kcfg_icalendarFile");
        kcfg_icalendarFile->setProperty("kcfg_property", QVariant(QByteArray("text")));

        hboxLayout2->addWidget(kcfg_icalendarFile);


        vboxLayout->addLayout(hboxLayout2);

        spacer1 = new QSpacerItem(20, 183, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer1);


        gridLayout->addLayout(vboxLayout, 0, 0, 1, 1);


        retranslateUi(ICalendarSettingsDecl);

        QMetaObject::connectSlotsByName(ICalendarSettingsDecl);
    } // setupUi

    void retranslateUi(QWidget *ICalendarSettingsDecl)
    {
        ICalendarSettingsDecl->setWindowTitle(tr2i18n("Icalendar export settings", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_writeEventsVsTodos->setToolTip(tr2i18n("Write the iCalendar file so that the entries are recognized by Google Calendar.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_writeEventsVsTodos->setWhatsThis(tr2i18n("<html><head/><body><p>Write the iCalendar file so that the entries are recognized by Google Calendar. The default is to write <a href=\"https://icalendar.org/iCalendar-RFC-5545/3-6-2-to-do-component.html\">VTODO</a> entries into the iCalendar file but for Google Calendar entries of type <a href=\"https://icalendar.org/iCalendar-RFC-5545/3-6-1-event-component.html\">VEVENT</a> are needed.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_writeEventsVsTodos->setText(tr2i18n("Write Google calendar compatible entries", nullptr));
        kcfg_createAlarm->setText(tr2i18n("Create reminders", nullptr));
        kcfg_repeatingReminders->setText(tr2i18n("Repeating:", "@action whether the schedule will have more occurrences over time"));
        textLabel1->setText(tr2i18n("How often:", nullptr));
        textLabel2->setText(tr2i18n("Interval:", nullptr));
        textLabel4->setText(tr2i18n("Time(s)", nullptr));
        iCalendar_file->setText(tr2i18n("iCalendar file", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ICalendarSettingsDecl: public Ui_ICalendarSettingsDecl {};
} // namespace Ui

QT_END_NAMESPACE

#endif // ICALENDARSETTINGSDECL_H

