/********************************************************************************
** Form generated from reading UI file 'newpaymentswizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NEWPAYMENTSWIZARDPAGE_H
#define UI_NEWPAYMENTSWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_NewPaymentsWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel19_2;
    QSpacerItem *Spacer10_2_2;
    QHBoxLayout *hboxLayout;
    QSpacerItem *Spacer8_2_2;
    QVBoxLayout *vboxLayout1;
    QLabel *TextLabel19_3;
    QLabel *TextLabel19_4;
    QLabel *TextLabel19_5;
    QSpacerItem *Spacer7_2_2;
    QSpacerItem *Spacer9_2_2;
    QSpacerItem *Spacer51_2;

    void setupUi(QWizardPage *NewPaymentsWizardPage)
    {
        if (NewPaymentsWizardPage->objectName().isEmpty())
            NewPaymentsWizardPage->setObjectName("NewPaymentsWizardPage");
        vboxLayout = new QVBoxLayout(NewPaymentsWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel19_2 = new QLabel(NewPaymentsWizardPage);
        TextLabel19_2->setObjectName("TextLabel19_2");
        TextLabel19_2->setAlignment(Qt::AlignTop);
        TextLabel19_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel19_2);

        Spacer10_2_2 = new QSpacerItem(0, 14, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer10_2_2);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        Spacer8_2_2 = new QSpacerItem(136, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer8_2_2);

        vboxLayout1 = new QVBoxLayout();
#ifndef UI_Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout1->setObjectName("vboxLayout1");
        TextLabel19_3 = new QLabel(NewPaymentsWizardPage);
        TextLabel19_3->setObjectName("TextLabel19_3");
        TextLabel19_3->setEnabled(false);
        TextLabel19_3->setAlignment(Qt::AlignTop);
        TextLabel19_3->setWordWrap(false);

        vboxLayout1->addWidget(TextLabel19_3);

        TextLabel19_4 = new QLabel(NewPaymentsWizardPage);
        TextLabel19_4->setObjectName("TextLabel19_4");
        TextLabel19_4->setEnabled(false);
        TextLabel19_4->setWordWrap(false);

        vboxLayout1->addWidget(TextLabel19_4);

        TextLabel19_5 = new QLabel(NewPaymentsWizardPage);
        TextLabel19_5->setObjectName("TextLabel19_5");
        TextLabel19_5->setEnabled(true);
        TextLabel19_5->setWordWrap(false);

        vboxLayout1->addWidget(TextLabel19_5);

        Spacer7_2_2 = new QSpacerItem(0, 18, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(Spacer7_2_2);


        hboxLayout->addLayout(vboxLayout1);

        Spacer9_2_2 = new QSpacerItem(135, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(Spacer9_2_2);


        vboxLayout->addLayout(hboxLayout);

        Spacer51_2 = new QSpacerItem(0, 18, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer51_2);


        retranslateUi(NewPaymentsWizardPage);

        QMetaObject::connectSlotsByName(NewPaymentsWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *NewPaymentsWizardPage)
    {
        TextLabel19_2->setText(tr2i18n("\n"
"In the following steps, KMyMoney supports you in setting up categories and schedules for your loan payments.", nullptr));
        TextLabel19_3->setText(tr2i18n("1. General Information", nullptr));
        TextLabel19_4->setText(tr2i18n("\n"
"2. Calculate Loan", nullptr));
        TextLabel19_5->setText(tr2i18n("\n"
"3. Payments", nullptr));
        (void)NewPaymentsWizardPage;
    } // retranslateUi

};

namespace Ui {
    class NewPaymentsWizardPage: public Ui_NewPaymentsWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // NEWPAYMENTSWIZARDPAGE_H

