/********************************************************************************
** Form generated from reading UI file 'ledgerviewpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LEDGERVIEWPAGE_H
#define UI_LEDGERVIEWPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kmmsearchwidget.h"
#include "ledgerview.h"

QT_BEGIN_NAMESPACE

class Ui_LedgerViewPage
{
public:
    QVBoxLayout *verticalLayout_2;
    QSplitter *m_splitter;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    LedgerView *m_ledgerView;
    QWidget *m_reconciliationContainer;
    QHBoxLayout *horizontalLayout_2;
    QLabel *m_leftLabel;
    QLabel *m_centerLabel;
    QLabel *m_rightLabel;
    KMMSearchWidget *m_searchWidget;
    QWidget *m_formWidget;

    void setupUi(QWidget *LedgerViewPage)
    {
        if (LedgerViewPage->objectName().isEmpty())
            LedgerViewPage->setObjectName("LedgerViewPage");
        LedgerViewPage->resize(321, 400);
        verticalLayout_2 = new QVBoxLayout(LedgerViewPage);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        m_splitter = new QSplitter(LedgerViewPage);
        m_splitter->setObjectName("m_splitter");
        m_splitter->setOrientation(Qt::Vertical);
        m_splitter->setHandleWidth(2);
        layoutWidget = new QWidget(m_splitter);
        layoutWidget->setObjectName("layoutWidget");
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setSpacing(1);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        m_ledgerView = new LedgerView(layoutWidget);
        m_ledgerView->setObjectName("m_ledgerView");

        verticalLayout->addWidget(m_ledgerView);

        m_reconciliationContainer = new QWidget(layoutWidget);
        m_reconciliationContainer->setObjectName("m_reconciliationContainer");
        horizontalLayout_2 = new QHBoxLayout(m_reconciliationContainer);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(-1, 3, -1, 3);
        m_leftLabel = new QLabel(m_reconciliationContainer);
        m_leftLabel->setObjectName("m_leftLabel");
        QFont font;
        font.setBold(true);
        m_leftLabel->setFont(font);

        horizontalLayout_2->addWidget(m_leftLabel);

        m_centerLabel = new QLabel(m_reconciliationContainer);
        m_centerLabel->setObjectName("m_centerLabel");
        m_centerLabel->setFont(font);
        m_centerLabel->setAlignment(Qt::AlignCenter);

        horizontalLayout_2->addWidget(m_centerLabel);

        m_rightLabel = new QLabel(m_reconciliationContainer);
        m_rightLabel->setObjectName("m_rightLabel");
        m_rightLabel->setFont(font);
        m_rightLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(m_rightLabel);


        verticalLayout->addWidget(m_reconciliationContainer);

        m_searchWidget = new KMMSearchWidget(layoutWidget);
        m_searchWidget->setObjectName("m_searchWidget");

        verticalLayout->addWidget(m_searchWidget);

        m_splitter->addWidget(layoutWidget);
        m_formWidget = new QWidget(m_splitter);
        m_formWidget->setObjectName("m_formWidget");
        m_splitter->addWidget(m_formWidget);

        verticalLayout_2->addWidget(m_splitter);


        retranslateUi(LedgerViewPage);

        QMetaObject::connectSlotsByName(LedgerViewPage);
    } // setupUi

    void retranslateUi(QWidget *LedgerViewPage)
    {
        m_leftLabel->setText(tr2i18n("TextLabel", nullptr));
        m_centerLabel->setText(tr2i18n("TextLabel", nullptr));
        m_rightLabel->setText(tr2i18n("TextLabel", nullptr));
        (void)LedgerViewPage;
    } // retranslateUi

};

namespace Ui {
    class LedgerViewPage: public Ui_LedgerViewPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // LEDGERVIEWPAGE_H

