/********************************************************************************
** Form generated from reading UI file 'variableinterestdatewizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_VARIABLEINTERESTDATEWIZARDPAGE_H
#define UI_VARIABLEINTERESTDATEWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>
#include "kmymoneydateedit.h"
#include "kmymoneygeneralcombo.h"

QT_BEGIN_NAMESPACE

class Ui_VariableInterestDateWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel8_2;
    QSpacerItem *Spacer39;
    QGridLayout *gridLayout;
    QLabel *TextLabel1;
    QHBoxLayout *hboxLayout;
    QSpinBox *m_interestFrequencyAmountEdit;
    KMyMoneyGeneralCombo *m_interestFrequencyUnitEdit;
    KMyMoneyDateEdit *m_interestChangeDateEdit;
    QLabel *TextLabel2;
    QSpacerItem *Spacer40;

    void setupUi(QWizardPage *VariableInterestDateWizardPage)
    {
        if (VariableInterestDateWizardPage->objectName().isEmpty())
            VariableInterestDateWizardPage->setObjectName("VariableInterestDateWizardPage");
        vboxLayout = new QVBoxLayout(VariableInterestDateWizardPage);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel8_2 = new QLabel(VariableInterestDateWizardPage);
        TextLabel8_2->setObjectName("TextLabel8_2");
        TextLabel8_2->setAlignment(Qt::AlignTop);
        TextLabel8_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel8_2);

        Spacer39 = new QSpacerItem(16, 99, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer39);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        TextLabel1 = new QLabel(VariableInterestDateWizardPage);
        TextLabel1->setObjectName("TextLabel1");
        TextLabel1->setWordWrap(false);

        gridLayout->addWidget(TextLabel1, 0, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        m_interestFrequencyAmountEdit = new QSpinBox(VariableInterestDateWizardPage);
        m_interestFrequencyAmountEdit->setObjectName("m_interestFrequencyAmountEdit");
        m_interestFrequencyAmountEdit->setMinimum(0);
        m_interestFrequencyAmountEdit->setMaximum(1000);

        hboxLayout->addWidget(m_interestFrequencyAmountEdit);

        m_interestFrequencyUnitEdit = new KMyMoneyGeneralCombo(VariableInterestDateWizardPage);
        m_interestFrequencyUnitEdit->setObjectName("m_interestFrequencyUnitEdit");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_interestFrequencyUnitEdit->sizePolicy().hasHeightForWidth());
        m_interestFrequencyUnitEdit->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(m_interestFrequencyUnitEdit);


        gridLayout->addLayout(hboxLayout, 1, 1, 1, 1);

        m_interestChangeDateEdit = new KMyMoneyDateEdit(VariableInterestDateWizardPage);
        m_interestChangeDateEdit->setObjectName("m_interestChangeDateEdit");

        gridLayout->addWidget(m_interestChangeDateEdit, 0, 1, 1, 1);

        TextLabel2 = new QLabel(VariableInterestDateWizardPage);
        TextLabel2->setObjectName("TextLabel2");
        TextLabel2->setWordWrap(false);

        gridLayout->addWidget(TextLabel2, 1, 0, 1, 1);


        vboxLayout->addLayout(gridLayout);

        Spacer40 = new QSpacerItem(20, 80, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer40);


        retranslateUi(VariableInterestDateWizardPage);

        QMetaObject::connectSlotsByName(VariableInterestDateWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *VariableInterestDateWizardPage)
    {
        TextLabel8_2->setText(tr2i18n("\n"
"Select the date when the interest rate for this loan will be modified and the frequency of the future changes.", nullptr));
        TextLabel1->setText(tr2i18n("Next interest change on", nullptr));
        TextLabel2->setText(tr2i18n("Time until next change", nullptr));
        (void)VariableInterestDateWizardPage;
    } // retranslateUi

};

namespace Ui {
    class VariableInterestDateWizardPage: public Ui_VariableInterestDateWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // VARIABLEINTERESTDATEWIZARDPAGE_H

