/********************************************************************************
** Form generated from reading UI file 'kequitypriceupdateconfdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KEQUITYPRICEUPDATECONFDLG_H
#define UI_KEQUITYPRICEUPDATECONFDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_EquityPriceUpdateConfDlg
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QRadioButton *m_updateAll;
    QRadioButton *m_updateMissing;
    QRadioButton *m_updateDownloaded;
    QRadioButton *m_updateSource;
    QRadioButton *m_ask;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *EquityPriceUpdateConfDlg)
    {
        if (EquityPriceUpdateConfDlg->objectName().isEmpty())
            EquityPriceUpdateConfDlg->setObjectName("EquityPriceUpdateConfDlg");
        EquityPriceUpdateConfDlg->resize(238, 228);
        verticalLayout = new QVBoxLayout(EquityPriceUpdateConfDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        groupBox = new QGroupBox(EquityPriceUpdateConfDlg);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        m_updateAll = new QRadioButton(groupBox);
        m_updateAll->setObjectName("m_updateAll");

        verticalLayout_2->addWidget(m_updateAll);

        m_updateMissing = new QRadioButton(groupBox);
        m_updateMissing->setObjectName("m_updateMissing");

        verticalLayout_2->addWidget(m_updateMissing);

        m_updateDownloaded = new QRadioButton(groupBox);
        m_updateDownloaded->setObjectName("m_updateDownloaded");

        verticalLayout_2->addWidget(m_updateDownloaded);

        m_updateSource = new QRadioButton(groupBox);
        m_updateSource->setObjectName("m_updateSource");

        verticalLayout_2->addWidget(m_updateSource);

        m_ask = new QRadioButton(groupBox);
        m_ask->setObjectName("m_ask");

        verticalLayout_2->addWidget(m_ask);


        verticalLayout->addWidget(groupBox);

        buttonBox = new QDialogButtonBox(EquityPriceUpdateConfDlg);
        buttonBox->setObjectName("buttonBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(buttonBox->sizePolicy().hasHeightForWidth());
        buttonBox->setSizePolicy(sizePolicy);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
        buttonBox->setCenterButtons(true);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(EquityPriceUpdateConfDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, EquityPriceUpdateConfDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, EquityPriceUpdateConfDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(EquityPriceUpdateConfDlg);
    } // setupUi

    void retranslateUi(QDialog *EquityPriceUpdateConfDlg)
    {
        EquityPriceUpdateConfDlg->setWindowTitle(tr2i18n("Updating price configuration", nullptr));
        groupBox->setTitle(tr2i18n("Updating price policy", nullptr));
        m_updateAll->setText(tr2i18n("Update all", nullptr));
        m_updateMissing->setText(tr2i18n("Update &missing", nullptr));
        m_updateDownloaded->setText(tr2i18n("Update downloaded", nullptr));
        m_updateSource->setText(tr2i18n("Update from the same source", nullptr));
        m_ask->setText(tr2i18n("Ask", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EquityPriceUpdateConfDlg: public Ui_EquityPriceUpdateConfDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KEQUITYPRICEUPDATECONFDLG_H

