/****************************************************************************
** Meta object code from reading C++ file 'listtable.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../../kmymoney-5.2.2/kmymoney/plugins/views/reports/core/listtable.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'listtable.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7reports9ListTableE_t {};
} // unnamed namespace

template <> constexpr inline auto reports::ListTable::qt_create_metaobjectdata<qt_meta_tag_ZN7reports9ListTableE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "reports::ListTable",
        "Rank",
        "OpeningBalance",
        "FirstSplitOfTransaction",
        "SecondarySplitOfTransaction",
        "ClosingBalance",
        "BaseCurrencyTotals",
        "ForeignCurrencyTotals",
        "cellTypeE",
        "ctValue",
        "ctNetInvValue",
        "ctMarketValue",
        "ctPrice",
        "ctLastPrice",
        "ctBuyPrice",
        "ctBuys",
        "ctSells",
        "ctBuysST",
        "ctSellsST",
        "ctBuysLT",
        "ctSellsLT",
        "ctCapitalGain",
        "ctCapitalGainST",
        "ctCapitalGainLT",
        "ctCashIncome",
        "ctReinvestIncome",
        "ctFees",
        "ctInterest",
        "ctStartingBalance",
        "ctEndingBalance",
        "ctBalance",
        "ctCurrentBalance",
        "ctBalanceWarning",
        "ctMaxBalanceLimit",
        "ctOpeningBalance",
        "ctCreditWarning",
        "ctMaxCreditLimit",
        "ctLoanAmount",
        "ctPeriodicPayment",
        "ctFinalPayment",
        "ctPayment",
        "ctReturn",
        "ctShares",
        "ctAnnualizedReturn",
        "ctExtendedInternalRateOfReturn",
        "ctReturnInvestment",
        "ctInterestRate",
        "ctPercentageGain",
        "ctPostDate",
        "ctEntryDate",
        "ctNextDueDate",
        "ctOpeningDate",
        "ctNextInterestChange",
        "ctMonth",
        "ctWeek",
        "ctReconcileDate",
        "ctCurrency",
        "ctCurrencyName",
        "ctCommodity",
        "ctID",
        "ctRank",
        "ctSplit",
        "ctMemo",
        "ctAccount",
        "ctAccountID",
        "ctTopAccount",
        "ctInvestAccount",
        "ctInstitution",
        "ctCategory",
        "ctTopCategory",
        "ctCategoryType",
        "ctNumber",
        "ctReconcileFlag",
        "ctAction",
        "ctTag",
        "ctPayee",
        "ctEquityType",
        "ctType",
        "ctName",
        "ctDepth",
        "ctRowsCount",
        "ctTax",
        "ctFavorite",
        "ctDescription",
        "ctOccurrence",
        "ctPaymentType",
        "ctSourceLines",
        "ctStartingMarketValue",
        "ctEndingMarketValue",
        "csID"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Rank'
        QtMocHelpers::EnumData<enum Rank>(1, 1, QMC::EnumIsScoped).add({
            {    2, Rank::OpeningBalance },
            {    3, Rank::FirstSplitOfTransaction },
            {    4, Rank::SecondarySplitOfTransaction },
            {    5, Rank::ClosingBalance },
            {    6, Rank::BaseCurrencyTotals },
            {    7, Rank::ForeignCurrencyTotals },
        }),
        // enum 'cellTypeE'
        QtMocHelpers::EnumData<enum cellTypeE>(8, 8, QMC::EnumFlags{}).add({
            {    9, cellTypeE::ctValue },
            {   10, cellTypeE::ctNetInvValue },
            {   11, cellTypeE::ctMarketValue },
            {   12, cellTypeE::ctPrice },
            {   13, cellTypeE::ctLastPrice },
            {   14, cellTypeE::ctBuyPrice },
            {   15, cellTypeE::ctBuys },
            {   16, cellTypeE::ctSells },
            {   17, cellTypeE::ctBuysST },
            {   18, cellTypeE::ctSellsST },
            {   19, cellTypeE::ctBuysLT },
            {   20, cellTypeE::ctSellsLT },
            {   21, cellTypeE::ctCapitalGain },
            {   22, cellTypeE::ctCapitalGainST },
            {   23, cellTypeE::ctCapitalGainLT },
            {   24, cellTypeE::ctCashIncome },
            {   25, cellTypeE::ctReinvestIncome },
            {   26, cellTypeE::ctFees },
            {   27, cellTypeE::ctInterest },
            {   28, cellTypeE::ctStartingBalance },
            {   29, cellTypeE::ctEndingBalance },
            {   30, cellTypeE::ctBalance },
            {   31, cellTypeE::ctCurrentBalance },
            {   32, cellTypeE::ctBalanceWarning },
            {   33, cellTypeE::ctMaxBalanceLimit },
            {   34, cellTypeE::ctOpeningBalance },
            {   35, cellTypeE::ctCreditWarning },
            {   36, cellTypeE::ctMaxCreditLimit },
            {   37, cellTypeE::ctLoanAmount },
            {   38, cellTypeE::ctPeriodicPayment },
            {   39, cellTypeE::ctFinalPayment },
            {   40, cellTypeE::ctPayment },
            {   41, cellTypeE::ctReturn },
            {   42, cellTypeE::ctShares },
            {   43, cellTypeE::ctAnnualizedReturn },
            {   44, cellTypeE::ctExtendedInternalRateOfReturn },
            {   45, cellTypeE::ctReturnInvestment },
            {   46, cellTypeE::ctInterestRate },
            {   47, cellTypeE::ctPercentageGain },
            {   48, cellTypeE::ctPostDate },
            {   49, cellTypeE::ctEntryDate },
            {   50, cellTypeE::ctNextDueDate },
            {   51, cellTypeE::ctOpeningDate },
            {   52, cellTypeE::ctNextInterestChange },
            {   53, cellTypeE::ctMonth },
            {   54, cellTypeE::ctWeek },
            {   55, cellTypeE::ctReconcileDate },
            {   56, cellTypeE::ctCurrency },
            {   57, cellTypeE::ctCurrencyName },
            {   58, cellTypeE::ctCommodity },
            {   59, cellTypeE::ctID },
            {   60, cellTypeE::ctRank },
            {   61, cellTypeE::ctSplit },
            {   62, cellTypeE::ctMemo },
            {   63, cellTypeE::ctAccount },
            {   64, cellTypeE::ctAccountID },
            {   65, cellTypeE::ctTopAccount },
            {   66, cellTypeE::ctInvestAccount },
            {   67, cellTypeE::ctInstitution },
            {   68, cellTypeE::ctCategory },
            {   69, cellTypeE::ctTopCategory },
            {   70, cellTypeE::ctCategoryType },
            {   71, cellTypeE::ctNumber },
            {   72, cellTypeE::ctReconcileFlag },
            {   73, cellTypeE::ctAction },
            {   74, cellTypeE::ctTag },
            {   75, cellTypeE::ctPayee },
            {   76, cellTypeE::ctEquityType },
            {   77, cellTypeE::ctType },
            {   78, cellTypeE::ctName },
            {   79, cellTypeE::ctDepth },
            {   80, cellTypeE::ctRowsCount },
            {   81, cellTypeE::ctTax },
            {   82, cellTypeE::ctFavorite },
            {   83, cellTypeE::ctDescription },
            {   84, cellTypeE::ctOccurrence },
            {   85, cellTypeE::ctPaymentType },
            {   86, cellTypeE::ctSourceLines },
            {   87, cellTypeE::ctStartingMarketValue },
            {   88, cellTypeE::ctEndingMarketValue },
            {   89, cellTypeE::csID },
        }),
    };
    return QtMocHelpers::metaObjectData<ListTable, qt_meta_tag_ZN7reports9ListTableE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject reports::ListTable::staticMetaObject = { {
    QMetaObject::SuperData::link<ReportTable::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7reports9ListTableE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7reports9ListTableE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7reports9ListTableE_t>.metaTypes,
    nullptr
} };

void reports::ListTable::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ListTable *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *reports::ListTable::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *reports::ListTable::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7reports9ListTableE_t>.strings))
        return static_cast<void*>(this);
    return ReportTable::qt_metacast(_clname);
}

int reports::ListTable::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ReportTable::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
