/********************************************************************************
** Form generated from reading UI file 'kequitypriceupdatedlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KEQUITYPRICEUPDATEDLG_H
#define UI_KEQUITYPRICEUPDATEDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kdatecombobox.h"
#include "kmessagewidget.h"
#include "kmmsearchwidget.h"
#include "kmymoneydateedit.h"
#include "ktextedit.h"

QT_BEGIN_NAMESPACE

class Ui_KEquityPriceUpdateDlg
{
public:
    QVBoxLayout *verticalLayout_2;
    KMessageWidget *messageWidget;
    QTreeView *equityView;
    KMMSearchWidget *searchWidget;
    QHBoxLayout *horizontalLayout_3;
    QWidget *m_dateContainer;
    QHBoxLayout *horizontalLayout;
    QLabel *m_fromLabel;
    KMyMoneyDateEdit *m_fromDate;
    QLabel *m_toLabel;
    KMyMoneyDateEdit *m_toDate;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *btnUpdateAll;
    QPushButton *btnUpdateSelected;
    QWidget *statusContainer;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *textLabel1;
    QSpacerItem *horizontalSpacer;
    QToolButton *closeStatusButton;
    KTextEdit *lbStatus;
    QProgressBar *prgOnlineProgress;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KEquityPriceUpdateDlg)
    {
        if (KEquityPriceUpdateDlg->objectName().isEmpty())
            KEquityPriceUpdateDlg->setObjectName("KEquityPriceUpdateDlg");
        KEquityPriceUpdateDlg->resize(696, 526);
        verticalLayout_2 = new QVBoxLayout(KEquityPriceUpdateDlg);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        messageWidget = new KMessageWidget(KEquityPriceUpdateDlg);
        messageWidget->setObjectName("messageWidget");
        messageWidget->setWordWrap(true);
        messageWidget->setMessageType(KMessageWidget::Information);

        verticalLayout_2->addWidget(messageWidget);

        equityView = new QTreeView(KEquityPriceUpdateDlg);
        equityView->setObjectName("equityView");
        equityView->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        equityView->setTabKeyNavigation(false);
        equityView->setAlternatingRowColors(true);
        equityView->setSelectionMode(QAbstractItemView::MultiSelection);
        equityView->setSelectionBehavior(QAbstractItemView::SelectRows);
        equityView->setRootIsDecorated(false);
        equityView->setAllColumnsShowFocus(true);

        verticalLayout_2->addWidget(equityView);

        searchWidget = new KMMSearchWidget(KEquityPriceUpdateDlg);
        searchWidget->setObjectName("searchWidget");

        verticalLayout_2->addWidget(searchWidget);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        m_dateContainer = new QWidget(KEquityPriceUpdateDlg);
        m_dateContainer->setObjectName("m_dateContainer");
        horizontalLayout = new QHBoxLayout(m_dateContainer);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setContentsMargins(11, 11, 11, 11);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        m_fromLabel = new QLabel(m_dateContainer);
        m_fromLabel->setObjectName("m_fromLabel");

        horizontalLayout->addWidget(m_fromLabel);

        m_fromDate = new KMyMoneyDateEdit(m_dateContainer);
        m_fromDate->setObjectName("m_fromDate");

        horizontalLayout->addWidget(m_fromDate);

        m_toLabel = new QLabel(m_dateContainer);
        m_toLabel->setObjectName("m_toLabel");

        horizontalLayout->addWidget(m_toLabel);

        m_toDate = new KMyMoneyDateEdit(m_dateContainer);
        m_toDate->setObjectName("m_toDate");

        horizontalLayout->addWidget(m_toDate);


        horizontalLayout_3->addWidget(m_dateContainer);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);

        btnUpdateAll = new QPushButton(KEquityPriceUpdateDlg);
        btnUpdateAll->setObjectName("btnUpdateAll");

        horizontalLayout_3->addWidget(btnUpdateAll);

        btnUpdateSelected = new QPushButton(KEquityPriceUpdateDlg);
        btnUpdateSelected->setObjectName("btnUpdateSelected");

        horizontalLayout_3->addWidget(btnUpdateSelected);


        verticalLayout_2->addLayout(horizontalLayout_3);

        statusContainer = new QWidget(KEquityPriceUpdateDlg);
        statusContainer->setObjectName("statusContainer");
        verticalLayout = new QVBoxLayout(statusContainer);
        verticalLayout->setSpacing(0);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        textLabel1 = new QLabel(statusContainer);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        horizontalLayout_2->addWidget(textLabel1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        closeStatusButton = new QToolButton(statusContainer);
        closeStatusButton->setObjectName("closeStatusButton");
        closeStatusButton->setAutoRaise(true);

        horizontalLayout_2->addWidget(closeStatusButton);


        verticalLayout->addLayout(horizontalLayout_2);

        lbStatus = new KTextEdit(statusContainer);
        lbStatus->setObjectName("lbStatus");
        lbStatus->setFocusPolicy(Qt::NoFocus);
        lbStatus->setReadOnly(true);

        verticalLayout->addWidget(lbStatus);

        prgOnlineProgress = new QProgressBar(statusContainer);
        prgOnlineProgress->setObjectName("prgOnlineProgress");

        verticalLayout->addWidget(prgOnlineProgress);


        verticalLayout_2->addWidget(statusContainer);

        buttonBox = new QDialogButtonBox(KEquityPriceUpdateDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok|QDialogButtonBox::RestoreDefaults);

        verticalLayout_2->addWidget(buttonBox);

        QWidget::setTabOrder(m_fromDate, m_toDate);
        QWidget::setTabOrder(m_toDate, btnUpdateAll);
        QWidget::setTabOrder(btnUpdateAll, btnUpdateSelected);
        QWidget::setTabOrder(btnUpdateSelected, lbStatus);
        QWidget::setTabOrder(lbStatus, closeStatusButton);

        retranslateUi(KEquityPriceUpdateDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KEquityPriceUpdateDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KEquityPriceUpdateDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KEquityPriceUpdateDlg);
    } // setupUi

    void retranslateUi(QDialog *KEquityPriceUpdateDlg)
    {
        KEquityPriceUpdateDlg->setWindowTitle(tr2i18n("Update Stock and Currency Prices", nullptr));
        m_fromLabel->setText(tr2i18n("From", "@label:chooser Date"));
        m_toLabel->setText(tr2i18n("To", "@label:chooser Date"));
        btnUpdateAll->setText(tr2i18n("Update All", nullptr));
        btnUpdateSelected->setText(tr2i18n("Update Selected", nullptr));
        textLabel1->setText(tr2i18n("Status:", nullptr));
        closeStatusButton->setText(tr2i18n("...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KEquityPriceUpdateDlg: public Ui_KEquityPriceUpdateDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KEQUITYPRICEUPDATEDLG_H

