/********************************************************************************
** Form generated from reading UI file 'accounts.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ACCOUNTS_H
#define UI_ACCOUNTS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kaccounttemplateselector.h"

QT_BEGIN_NAMESPACE

class Ui_Accounts
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel2;
    KAccountTemplateSelector *m_templateSelector;

    void setupUi(QWidget *Accounts)
    {
        if (Accounts->objectName().isEmpty())
            Accounts->setObjectName("Accounts");
        Accounts->resize(600, 550);
        vboxLayout = new QVBoxLayout(Accounts);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        qLabel2 = new QLabel(Accounts);
        qLabel2->setObjectName("qLabel2");
        qLabel2->setWordWrap(true);

        vboxLayout->addWidget(qLabel2);

        m_templateSelector = new KAccountTemplateSelector(Accounts);
        m_templateSelector->setObjectName("m_templateSelector");

        vboxLayout->addWidget(m_templateSelector);

        vboxLayout->setStretch(1, 5);

        retranslateUi(Accounts);

        QMetaObject::connectSlotsByName(Accounts);
    } // setupUi

    void retranslateUi(QWidget *Accounts)
    {
        qLabel2->setText(tr2i18n("Select groups of accounts that correspond to the ways that you will use KMyMoney. Each group you select will cause several accounts and categories to be created. Select the groups that are relevant to you. You can always create additional accounts manually later.", nullptr));
        (void)Accounts;
    } // retranslateUi

};

namespace Ui {
    class Accounts: public Ui_Accounts {};
} // namespace Ui

QT_END_NAMESPACE

#endif // ACCOUNTS_H

