/****************************************************************************
** Meta object code from reading C++ file 'kmymoneyview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/views/kmymoneyview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kmymoneyview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12KMyMoneyViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KMyMoneyView::qt_create_metaobjectdata<qt_meta_tag_ZN12KMyMoneyViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KMyMoneyView",
        "viewActivated",
        "",
        "View",
        "view",
        "accountSelectedForContextMenu",
        "MyMoneyAccount",
        "acc",
        "viewStateChanged",
        "enabled",
        "kmmFilePlugin",
        "action",
        "statusMsg",
        "txt",
        "statusProgress",
        "cnt",
        "base",
        "accountReconciled",
        "account",
        "date",
        "MyMoneyMoney",
        "startingBalance",
        "endingBalance",
        "transactionList",
        "accountSelected",
        "requestSelectionChange",
        "SelectedObjects",
        "selection",
        "requestCustomContextMenu",
        "eMenu::Menu",
        "type",
        "QPoint",
        "pos",
        "requestActionTrigger",
        "eMenu::Action",
        "settingsChanged",
        "addSharedActionButton",
        "QAction*",
        "defaultAction",
        "selectSharedActionButton",
        "onlinePluginsChanged",
        "QMap<QString,KMyMoneyPlugin::OnlinePlugin*>*",
        "plugins",
        "editTransaction",
        "id",
        "isEditing",
        "_q_selectionChangeRequested",
        "slotSettingsChanged",
        "updateActions",
        "selections",
        "slotSwitchView",
        "KPageWidgetItem*",
        "current",
        "previous",
        "switchToDefaultView",
        "slotRememberLastView",
        "switchView",
        "QWidget*",
        "viewWidget",
        "accountId",
        "journalEntryId"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'viewActivated'
        QtMocHelpers::SignalData<void(View)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'accountSelectedForContextMenu'
        QtMocHelpers::SignalData<void(const MyMoneyAccount &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'viewStateChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'kmmFilePlugin'
        QtMocHelpers::SignalData<void(unsigned int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 11 },
        }}),
        // Signal 'statusMsg'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'statusProgress'
        QtMocHelpers::SignalData<void(int, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::Int, 16 },
        }}),
        // Signal 'accountReconciled'
        QtMocHelpers::SignalData<void(const MyMoneyAccount &, const QDate &, const MyMoneyMoney &, const MyMoneyMoney &, const QStringList &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 18 }, { QMetaType::QDate, 19 }, { 0x80000000 | 20, 21 }, { 0x80000000 | 20, 22 },
            { QMetaType::QStringList, 23 },
        }}),
        // Signal 'accountSelected'
        QtMocHelpers::SignalData<void(const MyMoneyAccount &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 18 },
        }}),
        // Signal 'requestSelectionChange'
        QtMocHelpers::SignalData<void(const SelectedObjects &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Signal 'requestCustomContextMenu'
        QtMocHelpers::SignalData<void(eMenu::Menu, const QPoint &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 }, { 0x80000000 | 31, 32 },
        }}),
        // Signal 'requestActionTrigger'
        QtMocHelpers::SignalData<void(eMenu::Action)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 11 },
        }}),
        // Signal 'settingsChanged'
        QtMocHelpers::SignalData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'addSharedActionButton'
        QtMocHelpers::SignalData<void(eMenu::Action, QAction *)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 11 }, { 0x80000000 | 37, 38 },
        }}),
        // Signal 'selectSharedActionButton'
        QtMocHelpers::SignalData<void(eMenu::Action, QAction *)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 11 }, { 0x80000000 | 37, 38 },
        }}),
        // Signal 'onlinePluginsChanged'
        QtMocHelpers::SignalData<void(QMap<QString,KMyMoneyPlugin::OnlinePlugin*> *)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 42 },
        }}),
        // Signal 'editTransaction'
        QtMocHelpers::SignalData<void(void *, bool)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::VoidStar, 44 }, { QMetaType::Bool, 45 },
        }}),
        // Slot '_q_selectionChangeRequested'
        QtMocHelpers::SlotData<void(const SelectedObjects &)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'slotSettingsChanged'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateActions'
        QtMocHelpers::SlotData<void(const SelectedObjects &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 49 },
        }}),
        // Slot 'slotSwitchView'
        QtMocHelpers::SlotData<void(KPageWidgetItem *, KPageWidgetItem *)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 51, 52 }, { 0x80000000 | 51, 53 },
        }}),
        // Slot 'switchToDefaultView'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRememberLastView'
        QtMocHelpers::SlotData<void(View)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'switchView'
        QtMocHelpers::SlotData<void(QWidget *, const QString &, const QString &)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 57, 58 }, { QMetaType::QString, 59 }, { QMetaType::QString, 60 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KMyMoneyView, qt_meta_tag_ZN12KMyMoneyViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KMyMoneyView::staticMetaObject = { {
    QMetaObject::SuperData::link<KPageWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KMyMoneyViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KMyMoneyViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12KMyMoneyViewE_t>.metaTypes,
    nullptr
} };

void KMyMoneyView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KMyMoneyView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->viewActivated((*reinterpret_cast<std::add_pointer_t<View>>(_a[1]))); break;
        case 1: _t->accountSelectedForContextMenu((*reinterpret_cast<std::add_pointer_t<MyMoneyAccount>>(_a[1]))); break;
        case 2: _t->viewStateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->kmmFilePlugin((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 4: _t->statusMsg((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->statusProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->accountReconciled((*reinterpret_cast<std::add_pointer_t<MyMoneyAccount>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDate>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<MyMoneyMoney>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<MyMoneyMoney>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[5]))); break;
        case 7: _t->accountSelected((*reinterpret_cast<std::add_pointer_t<MyMoneyAccount>>(_a[1]))); break;
        case 8: _t->requestSelectionChange((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 9: _t->requestCustomContextMenu((*reinterpret_cast<std::add_pointer_t<eMenu::Menu>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 10: _t->requestActionTrigger((*reinterpret_cast<std::add_pointer_t<eMenu::Action>>(_a[1]))); break;
        case 11: _t->settingsChanged(); break;
        case 12: _t->addSharedActionButton((*reinterpret_cast<std::add_pointer_t<eMenu::Action>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[2]))); break;
        case 13: _t->selectSharedActionButton((*reinterpret_cast<std::add_pointer_t<eMenu::Action>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[2]))); break;
        case 14: _t->onlinePluginsChanged((*reinterpret_cast<std::add_pointer_t<QMap<QString,KMyMoneyPlugin::OnlinePlugin*>*>>(_a[1]))); break;
        case 15: _t->editTransaction((*reinterpret_cast<std::add_pointer_t<void*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 16: _t->d_func()->_q_selectionChangeRequested((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 17: _t->slotSettingsChanged(); break;
        case 18: _t->updateActions((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 19: _t->slotSwitchView((*reinterpret_cast<std::add_pointer_t<KPageWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KPageWidgetItem*>>(_a[2]))); break;
        case 20: _t->switchToDefaultView(); break;
        case 21: _t->slotRememberLastView((*reinterpret_cast<std::add_pointer_t<View>>(_a[1]))); break;
        case 22: _t->switchView((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KPageWidgetItem* >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(View )>(_a, &KMyMoneyView::viewActivated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(const MyMoneyAccount & )>(_a, &KMyMoneyView::accountSelectedForContextMenu, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(bool )>(_a, &KMyMoneyView::viewStateChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(unsigned int )>(_a, &KMyMoneyView::kmmFilePlugin, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(const QString & )>(_a, &KMyMoneyView::statusMsg, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(int , int )>(_a, &KMyMoneyView::statusProgress, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(const MyMoneyAccount & , const QDate & , const MyMoneyMoney & , const MyMoneyMoney & , const QStringList & )>(_a, &KMyMoneyView::accountReconciled, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(const MyMoneyAccount & )>(_a, &KMyMoneyView::accountSelected, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(const SelectedObjects & )>(_a, &KMyMoneyView::requestSelectionChange, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(eMenu::Menu , const QPoint & )>(_a, &KMyMoneyView::requestCustomContextMenu, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(eMenu::Action )>(_a, &KMyMoneyView::requestActionTrigger, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)()>(_a, &KMyMoneyView::settingsChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(eMenu::Action , QAction * )>(_a, &KMyMoneyView::addSharedActionButton, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(eMenu::Action , QAction * )>(_a, &KMyMoneyView::selectSharedActionButton, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(QMap<QString,KMyMoneyPlugin::OnlinePlugin*> * )>(_a, &KMyMoneyView::onlinePluginsChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (KMyMoneyView::*)(void * , bool )>(_a, &KMyMoneyView::editTransaction, 15))
            return;
    }
}

const QMetaObject *KMyMoneyView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KMyMoneyView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KMyMoneyViewE_t>.strings))
        return static_cast<void*>(this);
    return KPageWidget::qt_metacast(_clname);
}

int KMyMoneyView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPageWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void KMyMoneyView::viewActivated(View _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KMyMoneyView::accountSelectedForContextMenu(const MyMoneyAccount & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KMyMoneyView::viewStateChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KMyMoneyView::kmmFilePlugin(unsigned int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KMyMoneyView::statusMsg(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KMyMoneyView::statusProgress(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void KMyMoneyView::accountReconciled(const MyMoneyAccount & _t1, const QDate & _t2, const MyMoneyMoney & _t3, const MyMoneyMoney & _t4, const QStringList & _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 7
void KMyMoneyView::accountSelected(const MyMoneyAccount & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void KMyMoneyView::requestSelectionChange(const SelectedObjects & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void KMyMoneyView::requestCustomContextMenu(eMenu::Menu _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void KMyMoneyView::requestActionTrigger(eMenu::Action _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void KMyMoneyView::settingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void KMyMoneyView::addSharedActionButton(eMenu::Action _t1, QAction * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2);
}

// SIGNAL 13
void KMyMoneyView::selectSharedActionButton(eMenu::Action _t1, QAction * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2);
}

// SIGNAL 14
void KMyMoneyView::onlinePluginsChanged(QMap<QString,KMyMoneyPlugin::OnlinePlugin*> * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void KMyMoneyView::editTransaction(void * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1, _t2);
}
QT_WARNING_POP
