/********************************************************************************
** Form generated from reading UI file 'ktransactionfilter.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KTRANSACTIONFILTER_H
#define UI_KTRANSACTIONFILTER_H

#include <../widgets/kmymoneyaccountselector.h>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <amountedit.h>
#include <klocalizedstring.h>
#include "../widgets/kmymoneylineedit.h"
#include "kcombobox.h"
#include "klineedit.h"
#include "kmessagewidget.h"

QT_BEGIN_NAMESPACE

class Ui_KTransactionFilter
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *TextLabel1;
    QTabWidget *m_criteriaTab;
    QWidget *m_textTab;
    QGridLayout *gridLayout;
    QSpacerItem *Spacer20;
    KLineEdit *m_textEdit;
    KComboBox *m_textNegate;
    QCheckBox *m_regExp;
    QCheckBox *m_caseSensitive;
    QWidget *m_accountTab;
    QVBoxLayout *_2;
    KMyMoneyAccountSelector *m_accountsView;
    QWidget *m_dateTab;
    QVBoxLayout *dateTabLayout;
    QHBoxLayout *dateRangeLayout;
    QSpacerItem *Spacer25;
    QSpacerItem *Spacer22;
    QWidget *m_amountTab;
    QVBoxLayout *_3;
    QHBoxLayout *_4;
    QFrame *frame;
    QVBoxLayout *_5;
    QRadioButton *m_amountButton;
    QRadioButton *m_amountRangeButton;
    QGridLayout *_6;
    AmountEdit *m_amountToEdit;
    QLabel *TextLabel9;
    QLabel *TextLabel8;
    AmountEdit *m_amountEdit;
    QLabel *TextLabel7;
    AmountEdit *m_amountFromEdit;
    QSpacerItem *Spacer28;
    QSpacerItem *Spacer27;
    QWidget *m_categoryTab;
    QVBoxLayout *_7;
    KMessageWidget *m_categoriesHint;
    KMyMoneyAccountSelector *m_categoriesView;
    QWidget *m_tagTab;
    QHBoxLayout *_8;
    QVBoxLayout *_9;
    QTreeWidget *m_tagsView;
    QCheckBox *m_emptyTagsButton;
    QVBoxLayout *_10;
    QPushButton *m_allTagsButton;
    QPushButton *m_clearTagsButton;
    QSpacerItem *Spacer29;
    QWidget *m_payeeTab;
    QHBoxLayout *_11;
    QVBoxLayout *_12;
    QTreeWidget *m_payeesView;
    QCheckBox *m_emptyPayeesButton;
    QVBoxLayout *_13;
    QPushButton *m_allPayeesButton;
    QPushButton *m_clearPayeesButton;
    QSpacerItem *Spacer291;
    QWidget *m_detailsTab;
    QVBoxLayout *_14;
    QHBoxLayout *_15;
    QGridLayout *_16;
    QLabel *TextLabel11_2;
    QLabel *TextLabel10;
    KComboBox *m_stateBox;
    KComboBox *m_typeBox;
    KComboBox *m_validityBox;
    QLabel *TextLabel11;
    QSpacerItem *Spacer30;
    QHBoxLayout *_17;
    QFrame *frame_2;
    QVBoxLayout *_18;
    QRadioButton *m_nrButton;
    QRadioButton *m_nrRangeButton;
    QGridLayout *_19;
    QLabel *TextLabel12;
    QLabel *TextLabel14;
    KMyMoneyLineEdit *m_nrToEdit;
    KMyMoneyLineEdit *m_nrEdit;
    QLabel *TextLabel13;
    KMyMoneyLineEdit *m_nrFromEdit;
    QSpacerItem *Spacer31;
    QSpacerItem *Spacer32;
    QLabel *m_selectedCriteria;
    QLabel *m_transferWarning;
    QSpacerItem *spacer16;

    void setupUi(QWidget *KTransactionFilter)
    {
        if (KTransactionFilter->objectName().isEmpty())
            KTransactionFilter->setObjectName("KTransactionFilter");
        KTransactionFilter->resize(771, 481);
        KTransactionFilter->setProperty("sizeGripEnabled", QVariant(true));
        verticalLayout = new QVBoxLayout(KTransactionFilter);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        TextLabel1 = new QLabel(KTransactionFilter);
        TextLabel1->setObjectName("TextLabel1");
        TextLabel1->setWordWrap(false);

        verticalLayout->addWidget(TextLabel1);

        m_criteriaTab = new QTabWidget(KTransactionFilter);
        m_criteriaTab->setObjectName("m_criteriaTab");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(3);
        sizePolicy.setHeightForWidth(m_criteriaTab->sizePolicy().hasHeightForWidth());
        m_criteriaTab->setSizePolicy(sizePolicy);
        m_textTab = new QWidget();
        m_textTab->setObjectName("m_textTab");
        gridLayout = new QGridLayout(m_textTab);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        Spacer20 = new QSpacerItem(16, 83, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(Spacer20, 3, 2, 1, 1);

        m_textEdit = new KLineEdit(m_textTab);
        m_textEdit->setObjectName("m_textEdit");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_textEdit->sizePolicy().hasHeightForWidth());
        m_textEdit->setSizePolicy(sizePolicy1);
        m_textEdit->setMinimumSize(QSize(300, 0));
        m_textEdit->setProperty("showClearButton", QVariant(true));

        gridLayout->addWidget(m_textEdit, 0, 1, 1, 2);

        m_textNegate = new KComboBox(m_textTab);
        m_textNegate->addItem(QString());
        m_textNegate->addItem(QString());
        m_textNegate->setObjectName("m_textNegate");

        gridLayout->addWidget(m_textNegate, 0, 0, 1, 1);

        m_regExp = new QCheckBox(m_textTab);
        m_regExp->setObjectName("m_regExp");

        gridLayout->addWidget(m_regExp, 2, 0, 1, 3);

        m_caseSensitive = new QCheckBox(m_textTab);
        m_caseSensitive->setObjectName("m_caseSensitive");

        gridLayout->addWidget(m_caseSensitive, 1, 0, 1, 2);

        m_criteriaTab->addTab(m_textTab, QString());
        m_accountTab = new QWidget();
        m_accountTab->setObjectName("m_accountTab");
        _2 = new QVBoxLayout(m_accountTab);
        _2->setSpacing(6);
        _2->setContentsMargins(11, 11, 11, 11);
        _2->setObjectName("_2");
        _2->setContentsMargins(11, 11, 11, 11);
        m_accountsView = new KMyMoneyAccountSelector(m_accountTab);
        m_accountsView->setObjectName("m_accountsView");

        _2->addWidget(m_accountsView);

        m_criteriaTab->addTab(m_accountTab, QString());
        m_dateTab = new QWidget();
        m_dateTab->setObjectName("m_dateTab");
        dateTabLayout = new QVBoxLayout(m_dateTab);
        dateTabLayout->setSpacing(6);
        dateTabLayout->setContentsMargins(11, 11, 11, 11);
        dateTabLayout->setObjectName("dateTabLayout");
        dateRangeLayout = new QHBoxLayout();
        dateRangeLayout->setSpacing(6);
        dateRangeLayout->setObjectName("dateRangeLayout");
        Spacer25 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        dateRangeLayout->addItem(Spacer25);


        dateTabLayout->addLayout(dateRangeLayout);

        Spacer22 = new QSpacerItem(20, 30, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        dateTabLayout->addItem(Spacer22);

        m_criteriaTab->addTab(m_dateTab, QString());
        m_amountTab = new QWidget();
        m_amountTab->setObjectName("m_amountTab");
        _3 = new QVBoxLayout(m_amountTab);
        _3->setSpacing(6);
        _3->setContentsMargins(11, 11, 11, 11);
        _3->setObjectName("_3");
        _4 = new QHBoxLayout();
        _4->setSpacing(6);
        _4->setObjectName("_4");
        frame = new QFrame(m_amountTab);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::NoFrame);
        frame->setFrameShadow(QFrame::Sunken);
        _5 = new QVBoxLayout(frame);
        _5->setSpacing(6);
        _5->setContentsMargins(11, 11, 11, 11);
        _5->setObjectName("_5");
        _5->setContentsMargins(0, 0, 0, 0);
        m_amountButton = new QRadioButton(frame);
        m_amountButton->setObjectName("m_amountButton");

        _5->addWidget(m_amountButton);

        m_amountRangeButton = new QRadioButton(frame);
        m_amountRangeButton->setObjectName("m_amountRangeButton");

        _5->addWidget(m_amountRangeButton);


        _4->addWidget(frame);

        _6 = new QGridLayout();
        _6->setSpacing(6);
        _6->setObjectName("_6");
        m_amountToEdit = new AmountEdit(m_amountTab);
        m_amountToEdit->setObjectName("m_amountToEdit");
        m_amountToEdit->setMinimumSize(QSize(100, 0));

        _6->addWidget(m_amountToEdit, 2, 1, 1, 1);

        TextLabel9 = new QLabel(m_amountTab);
        TextLabel9->setObjectName("TextLabel9");
        TextLabel9->setWordWrap(false);

        _6->addWidget(TextLabel9, 0, 0, 1, 1);

        TextLabel8 = new QLabel(m_amountTab);
        TextLabel8->setObjectName("TextLabel8");
        TextLabel8->setWordWrap(false);

        _6->addWidget(TextLabel8, 2, 0, 1, 1);

        m_amountEdit = new AmountEdit(m_amountTab);
        m_amountEdit->setObjectName("m_amountEdit");
        m_amountEdit->setMinimumSize(QSize(100, 0));

        _6->addWidget(m_amountEdit, 0, 1, 1, 1);

        TextLabel7 = new QLabel(m_amountTab);
        TextLabel7->setObjectName("TextLabel7");
        TextLabel7->setWordWrap(false);

        _6->addWidget(TextLabel7, 1, 0, 1, 1);

        m_amountFromEdit = new AmountEdit(m_amountTab);
        m_amountFromEdit->setObjectName("m_amountFromEdit");
        m_amountFromEdit->setMinimumSize(QSize(100, 0));

        _6->addWidget(m_amountFromEdit, 1, 1, 1, 1);


        _4->addLayout(_6);

        Spacer28 = new QSpacerItem(16, 16, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _4->addItem(Spacer28);


        _3->addLayout(_4);

        Spacer27 = new QSpacerItem(16, 75, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        _3->addItem(Spacer27);

        m_criteriaTab->addTab(m_amountTab, QString());
        m_categoryTab = new QWidget();
        m_categoryTab->setObjectName("m_categoryTab");
        _7 = new QVBoxLayout(m_categoryTab);
        _7->setSpacing(6);
        _7->setContentsMargins(11, 11, 11, 11);
        _7->setObjectName("_7");
        m_categoriesHint = new KMessageWidget(m_categoryTab);
        m_categoriesHint->setObjectName("m_categoriesHint");
        m_categoriesHint->setWordWrap(true);
        m_categoriesHint->setCloseButtonVisible(false);

        _7->addWidget(m_categoriesHint);

        m_categoriesView = new KMyMoneyAccountSelector(m_categoryTab);
        m_categoriesView->setObjectName("m_categoriesView");

        _7->addWidget(m_categoriesView);

        m_criteriaTab->addTab(m_categoryTab, QString());
        m_tagTab = new QWidget();
        m_tagTab->setObjectName("m_tagTab");
        _8 = new QHBoxLayout(m_tagTab);
        _8->setSpacing(6);
        _8->setContentsMargins(11, 11, 11, 11);
        _8->setObjectName("_8");
        _8->setContentsMargins(11, 11, 11, 11);
        _9 = new QVBoxLayout();
        _9->setSpacing(6);
        _9->setObjectName("_9");
        _9->setContentsMargins(0, 0, 0, 0);
        m_tagsView = new QTreeWidget(m_tagTab);
        m_tagsView->setObjectName("m_tagsView");
        m_tagsView->setRootIsDecorated(false);

        _9->addWidget(m_tagsView);

        m_emptyTagsButton = new QCheckBox(m_tagTab);
        m_emptyTagsButton->setObjectName("m_emptyTagsButton");

        _9->addWidget(m_emptyTagsButton);


        _8->addLayout(_9);

        _10 = new QVBoxLayout();
        _10->setSpacing(6);
        _10->setObjectName("_10");
        _10->setContentsMargins(0, 0, 0, 0);
        m_allTagsButton = new QPushButton(m_tagTab);
        m_allTagsButton->setObjectName("m_allTagsButton");

        _10->addWidget(m_allTagsButton);

        m_clearTagsButton = new QPushButton(m_tagTab);
        m_clearTagsButton->setObjectName("m_clearTagsButton");

        _10->addWidget(m_clearTagsButton);

        Spacer29 = new QSpacerItem(0, 42, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        _10->addItem(Spacer29);


        _8->addLayout(_10);

        m_criteriaTab->addTab(m_tagTab, QString());
        m_payeeTab = new QWidget();
        m_payeeTab->setObjectName("m_payeeTab");
        _11 = new QHBoxLayout(m_payeeTab);
        _11->setSpacing(6);
        _11->setContentsMargins(11, 11, 11, 11);
        _11->setObjectName("_11");
        _11->setContentsMargins(11, 11, 11, 11);
        _12 = new QVBoxLayout();
        _12->setSpacing(6);
        _12->setObjectName("_12");
        _12->setContentsMargins(0, 0, 0, 0);
        m_payeesView = new QTreeWidget(m_payeeTab);
        m_payeesView->setObjectName("m_payeesView");
        m_payeesView->setRootIsDecorated(false);

        _12->addWidget(m_payeesView);

        m_emptyPayeesButton = new QCheckBox(m_payeeTab);
        m_emptyPayeesButton->setObjectName("m_emptyPayeesButton");

        _12->addWidget(m_emptyPayeesButton);


        _11->addLayout(_12);

        _13 = new QVBoxLayout();
        _13->setSpacing(6);
        _13->setObjectName("_13");
        _13->setContentsMargins(0, 0, 0, 0);
        m_allPayeesButton = new QPushButton(m_payeeTab);
        m_allPayeesButton->setObjectName("m_allPayeesButton");

        _13->addWidget(m_allPayeesButton);

        m_clearPayeesButton = new QPushButton(m_payeeTab);
        m_clearPayeesButton->setObjectName("m_clearPayeesButton");

        _13->addWidget(m_clearPayeesButton);

        Spacer291 = new QSpacerItem(0, 42, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        _13->addItem(Spacer291);


        _11->addLayout(_13);

        m_criteriaTab->addTab(m_payeeTab, QString());
        m_detailsTab = new QWidget();
        m_detailsTab->setObjectName("m_detailsTab");
        _14 = new QVBoxLayout(m_detailsTab);
        _14->setSpacing(6);
        _14->setContentsMargins(11, 11, 11, 11);
        _14->setObjectName("_14");
        _15 = new QHBoxLayout();
        _15->setSpacing(6);
        _15->setObjectName("_15");
        _16 = new QGridLayout();
        _16->setSpacing(6);
        _16->setObjectName("_16");
        TextLabel11_2 = new QLabel(m_detailsTab);
        TextLabel11_2->setObjectName("TextLabel11_2");
        TextLabel11_2->setWordWrap(false);

        _16->addWidget(TextLabel11_2, 2, 0, 1, 1);

        TextLabel10 = new QLabel(m_detailsTab);
        TextLabel10->setObjectName("TextLabel10");
        TextLabel10->setWordWrap(false);

        _16->addWidget(TextLabel10, 0, 0, 1, 1);

        m_stateBox = new KComboBox(m_detailsTab);
        m_stateBox->addItem(QString());
        m_stateBox->addItem(QString());
        m_stateBox->addItem(QString());
        m_stateBox->addItem(QString());
        m_stateBox->setObjectName("m_stateBox");

        _16->addWidget(m_stateBox, 1, 1, 1, 1);

        m_typeBox = new KComboBox(m_detailsTab);
        m_typeBox->addItem(QString());
        m_typeBox->addItem(QString());
        m_typeBox->addItem(QString());
        m_typeBox->addItem(QString());
        m_typeBox->setObjectName("m_typeBox");

        _16->addWidget(m_typeBox, 0, 1, 1, 1);

        m_validityBox = new KComboBox(m_detailsTab);
        m_validityBox->addItem(QString());
        m_validityBox->addItem(QString());
        m_validityBox->addItem(QString());
        m_validityBox->setObjectName("m_validityBox");

        _16->addWidget(m_validityBox, 2, 1, 1, 1);

        TextLabel11 = new QLabel(m_detailsTab);
        TextLabel11->setObjectName("TextLabel11");
        TextLabel11->setWordWrap(false);

        _16->addWidget(TextLabel11, 1, 0, 1, 1);


        _15->addLayout(_16);

        Spacer30 = new QSpacerItem(130, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _15->addItem(Spacer30);


        _14->addLayout(_15);

        _17 = new QHBoxLayout();
        _17->setSpacing(6);
        _17->setObjectName("_17");
        frame_2 = new QFrame(m_detailsTab);
        frame_2->setObjectName("frame_2");
        frame_2->setFrameShape(QFrame::NoFrame);
        _18 = new QVBoxLayout(frame_2);
        _18->setSpacing(6);
        _18->setContentsMargins(11, 11, 11, 11);
        _18->setObjectName("_18");
        m_nrButton = new QRadioButton(frame_2);
        m_nrButton->setObjectName("m_nrButton");

        _18->addWidget(m_nrButton);

        m_nrRangeButton = new QRadioButton(frame_2);
        m_nrRangeButton->setObjectName("m_nrRangeButton");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(m_nrRangeButton->sizePolicy().hasHeightForWidth());
        m_nrRangeButton->setSizePolicy(sizePolicy2);

        _18->addWidget(m_nrRangeButton);


        _17->addWidget(frame_2);

        _19 = new QGridLayout();
        _19->setSpacing(6);
        _19->setObjectName("_19");
        TextLabel12 = new QLabel(m_detailsTab);
        TextLabel12->setObjectName("TextLabel12");
        TextLabel12->setWordWrap(false);

        _19->addWidget(TextLabel12, 0, 0, 1, 1);

        TextLabel14 = new QLabel(m_detailsTab);
        TextLabel14->setObjectName("TextLabel14");
        TextLabel14->setWordWrap(false);

        _19->addWidget(TextLabel14, 2, 0, 1, 1);

        m_nrToEdit = new KMyMoneyLineEdit(m_detailsTab);
        m_nrToEdit->setObjectName("m_nrToEdit");

        _19->addWidget(m_nrToEdit, 2, 1, 1, 1);

        m_nrEdit = new KMyMoneyLineEdit(m_detailsTab);
        m_nrEdit->setObjectName("m_nrEdit");

        _19->addWidget(m_nrEdit, 0, 1, 1, 1);

        TextLabel13 = new QLabel(m_detailsTab);
        TextLabel13->setObjectName("TextLabel13");
        TextLabel13->setWordWrap(false);

        _19->addWidget(TextLabel13, 1, 0, 1, 1);

        m_nrFromEdit = new KMyMoneyLineEdit(m_detailsTab);
        m_nrFromEdit->setObjectName("m_nrFromEdit");

        _19->addWidget(m_nrFromEdit, 1, 1, 1, 1);


        _17->addLayout(_19);

        Spacer31 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _17->addItem(Spacer31);


        _14->addLayout(_17);

        Spacer32 = new QSpacerItem(16, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        _14->addItem(Spacer32);

        m_criteriaTab->addTab(m_detailsTab, QString());

        verticalLayout->addWidget(m_criteriaTab);

        m_selectedCriteria = new QLabel(KTransactionFilter);
        m_selectedCriteria->setObjectName("m_selectedCriteria");
        m_selectedCriteria->setAlignment(Qt::AlignTop);
        m_selectedCriteria->setWordWrap(false);

        verticalLayout->addWidget(m_selectedCriteria);

        m_transferWarning = new QLabel(KTransactionFilter);
        m_transferWarning->setObjectName("m_transferWarning");
        QFont font;
        font.setBold(true);
        m_transferWarning->setFont(font);
        m_transferWarning->setWordWrap(false);

        verticalLayout->addWidget(m_transferWarning);

        spacer16 = new QSpacerItem(20, 24, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacer16);


        retranslateUi(KTransactionFilter);

        m_criteriaTab->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KTransactionFilter);
    } // setupUi

    void retranslateUi(QWidget *KTransactionFilter)
    {
        KTransactionFilter->setWindowTitle(tr2i18n("Search transactions", nullptr));
        TextLabel1->setText(tr2i18n("Define your search criteria", nullptr));
        m_textNegate->setItemText(0, tr2i18n("Contains", nullptr));
        m_textNegate->setItemText(1, tr2i18n("Does not contain", nullptr));

        m_regExp->setText(tr2i18n("Treat text as regular expression", nullptr));
        m_caseSensitive->setText(tr2i18n("Case sensitive", nullptr));
        m_criteriaTab->setTabText(m_criteriaTab->indexOf(m_textTab), tr2i18n("Text", nullptr));
        m_criteriaTab->setTabText(m_criteriaTab->indexOf(m_accountTab), tr2i18n("Account", nullptr));
        m_criteriaTab->setTabText(m_criteriaTab->indexOf(m_dateTab), tr2i18n("Date", nullptr));
        m_amountButton->setText(tr2i18n("Search this amount", nullptr));
        m_amountRangeButton->setText(tr2i18n("Search amount in the range", nullptr));
        TextLabel9->setText(QString());
        TextLabel8->setText(tr2i18n("to", "to (amount)"));
        TextLabel7->setText(tr2i18n("from", "from (amount)"));
        m_criteriaTab->setTabText(m_criteriaTab->indexOf(m_amountTab), tr2i18n("Amount", nullptr));
        m_criteriaTab->setTabText(m_criteriaTab->indexOf(m_categoryTab), tr2i18n("Category", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = m_tagsView->headerItem();
        ___qtreewidgetitem->setText(0, tr2i18n("Tag", nullptr));
        m_emptyTagsButton->setText(tr2i18n("Select transactions without tags", nullptr));
        m_allTagsButton->setText(tr2i18n("Select all", nullptr));
        m_clearTagsButton->setText(tr2i18n("Select none", nullptr));
        m_criteriaTab->setTabText(m_criteriaTab->indexOf(m_tagTab), tr2i18n("Tag", nullptr));
        QTreeWidgetItem *___qtreewidgetitem1 = m_payeesView->headerItem();
        ___qtreewidgetitem1->setText(0, tr2i18n("Payee", nullptr));
        m_emptyPayeesButton->setText(tr2i18n("Select transactions without payees", nullptr));
        m_allPayeesButton->setText(tr2i18n("Select all", nullptr));
        m_clearPayeesButton->setText(tr2i18n("Select none", nullptr));
        m_criteriaTab->setTabText(m_criteriaTab->indexOf(m_payeeTab), tr2i18n("Payee", nullptr));
        TextLabel11_2->setText(tr2i18n("Validity", nullptr));
        TextLabel10->setText(tr2i18n("Type", nullptr));
        m_stateBox->setItemText(0, tr2i18n("All states", "@item reconciliation status"));
        m_stateBox->setItemText(1, tr2i18n("Not reconciled", "@item reconciliation status"));
        m_stateBox->setItemText(2, tr2i18n("Cleared", "@item reconciliation status"));
        m_stateBox->setItemText(3, tr2i18n("Reconciled", "@item reconciliation status"));

        m_typeBox->setItemText(0, tr2i18n("All types", nullptr));
        m_typeBox->setItemText(1, tr2i18n("Payments", nullptr));
        m_typeBox->setItemText(2, tr2i18n("Deposits", nullptr));
        m_typeBox->setItemText(3, tr2i18n("Transfers", nullptr));

        m_validityBox->setItemText(0, tr2i18n("Any transaction", nullptr));
        m_validityBox->setItemText(1, tr2i18n("Valid transaction", nullptr));
        m_validityBox->setItemText(2, tr2i18n("Invalid transaction", nullptr));

        TextLabel11->setText(tr2i18n("State", "@label reconciliation status"));
        m_nrButton->setText(tr2i18n("Search this number", nullptr));
        m_nrRangeButton->setText(tr2i18n("Search number in range", nullptr));
        TextLabel12->setText(QString());
        TextLabel14->setText(tr2i18n("to", "to (number)"));
        TextLabel13->setText(tr2i18n("from", "from (number)"));
        m_criteriaTab->setTabText(m_criteriaTab->indexOf(m_detailsTab), tr2i18n("Details", nullptr));
        m_selectedCriteria->setText(tr2i18n("text", nullptr));
        m_transferWarning->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class KTransactionFilter: public Ui_KTransactionFilter {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KTRANSACTIONFILTER_H

