/********************************************************************************
** Form generated from reading UI file 'firstpaymentwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FIRSTPAYMENTWIZARDPAGE_H
#define UI_FIRSTPAYMENTWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>
#include "kdatecombobox.h"
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_FirstPaymentWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *m_firstPaymentLabel;
    QSpacerItem *Spacer42;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel1_3;
    KMyMoneyDateEdit *m_firstDueDateEdit;
    QSpacerItem *Spacer43;
    QLabel *m_firstPaymentNote;

    void setupUi(QWizardPage *FirstPaymentWizardPage)
    {
        if (FirstPaymentWizardPage->objectName().isEmpty())
            FirstPaymentWizardPage->setObjectName("FirstPaymentWizardPage");
        FirstPaymentWizardPage->resize(256, 223);
        vboxLayout = new QVBoxLayout(FirstPaymentWizardPage);
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        m_firstPaymentLabel = new QLabel(FirstPaymentWizardPage);
        m_firstPaymentLabel->setObjectName("m_firstPaymentLabel");
        m_firstPaymentLabel->setAlignment(Qt::AlignTop);
        m_firstPaymentLabel->setWordWrap(true);

        vboxLayout->addWidget(m_firstPaymentLabel);

        Spacer42 = new QSpacerItem(0, 78, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer42);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        TextLabel1_3 = new QLabel(FirstPaymentWizardPage);
        TextLabel1_3->setObjectName("TextLabel1_3");
        TextLabel1_3->setWordWrap(false);

        hboxLayout->addWidget(TextLabel1_3);

        m_firstDueDateEdit = new KMyMoneyDateEdit(FirstPaymentWizardPage);
        m_firstDueDateEdit->setObjectName("m_firstDueDateEdit");

        hboxLayout->addWidget(m_firstDueDateEdit);


        vboxLayout->addLayout(hboxLayout);

        Spacer43 = new QSpacerItem(0, 28, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer43);

        m_firstPaymentNote = new QLabel(FirstPaymentWizardPage);
        m_firstPaymentNote->setObjectName("m_firstPaymentNote");
        m_firstPaymentNote->setAlignment(Qt::AlignBottom);
        m_firstPaymentNote->setWordWrap(true);

        vboxLayout->addWidget(m_firstPaymentNote);


        retranslateUi(FirstPaymentWizardPage);

        QMetaObject::connectSlotsByName(FirstPaymentWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *FirstPaymentWizardPage)
    {
        FirstPaymentWizardPage->setTitle(tr2i18n("Date of first payment", nullptr));
        m_firstPaymentLabel->setText(tr2i18n("1", nullptr));
        TextLabel1_3->setText(tr2i18n("First payment due on", nullptr));
        m_firstPaymentNote->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class FirstPaymentWizardPage: public Ui_FirstPaymentWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // FIRSTPAYMENTWIZARDPAGE_H

