/****************************************************************************
** Meta object code from reading C++ file 'kscheduledview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/views/kscheduledview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kscheduledview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14KScheduledViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KScheduledView::qt_create_metaobjectdata<qt_meta_tag_ZN14KScheduledViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KScheduledView",
        "enterOverdueSchedulesFinished",
        "",
        "eView::Schedules::Requester",
        "req",
        "slotEditSchedule",
        "QModelIndex",
        "index",
        "slotForceEditSchedule",
        "updateActions",
        "SelectedObjects",
        "selections",
        "slotListViewCollapsed",
        "idx",
        "slotListViewExpanded",
        "slotSetSelectedItem",
        "QItemSelection",
        "selected",
        "deselected",
        "slotNewSchedule",
        "slotDeleteSchedule",
        "slotDuplicateSchedule",
        "slotEnterSchedule",
        "slotSkipSchedule"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'enterOverdueSchedulesFinished'
        QtMocHelpers::SignalData<void(eView::Schedules::Requester)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotEditSchedule'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditSchedule'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotForceEditSchedule'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateActions'
        QtMocHelpers::SlotData<void(const SelectedObjects &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotListViewCollapsed'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 13 },
        }}),
        // Slot 'slotListViewExpanded'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 13 },
        }}),
        // Slot 'slotSetSelectedItem'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { 0x80000000 | 16, 18 },
        }}),
        // Slot 'slotNewSchedule'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDeleteSchedule'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDuplicateSchedule'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEnterSchedule'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSkipSchedule'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KScheduledView, qt_meta_tag_ZN14KScheduledViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KScheduledView::staticMetaObject = { {
    QMetaObject::SuperData::link<KMyMoneyViewBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KScheduledViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KScheduledViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14KScheduledViewE_t>.metaTypes,
    nullptr
} };

void KScheduledView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KScheduledView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->enterOverdueSchedulesFinished((*reinterpret_cast<std::add_pointer_t<eView::Schedules::Requester>>(_a[1]))); break;
        case 1: _t->slotEditSchedule(); break;
        case 2: _t->slotEditSchedule((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 3: _t->slotForceEditSchedule(); break;
        case 4: _t->updateActions((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 5: _t->slotListViewCollapsed((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 6: _t->slotListViewExpanded((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 7: _t->slotSetSelectedItem((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 8: _t->slotNewSchedule(); break;
        case 9: _t->slotDeleteSchedule(); break;
        case 10: _t->slotDuplicateSchedule(); break;
        case 11: _t->slotEnterSchedule(); break;
        case 12: _t->slotSkipSchedule(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KScheduledView::*)(eView::Schedules::Requester )>(_a, &KScheduledView::enterOverdueSchedulesFinished, 0))
            return;
    }
}

const QMetaObject *KScheduledView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KScheduledView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KScheduledViewE_t>.strings))
        return static_cast<void*>(this);
    return KMyMoneyViewBase::qt_metacast(_clname);
}

int KScheduledView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KMyMoneyViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void KScheduledView::enterOverdueSchedulesFinished(eView::Schedules::Requester _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
