/********************************************************************************
** Form generated from reading UI file 'knewequityentrydlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KNEWEQUITYENTRYDLG_H
#define UI_KNEWEQUITYENTRYDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <amountedit.h>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KNewEquityEntryDlg
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout;
    QLabel *lblEquityName;
    QLabel *lblType;
    KLineEdit *edtMarketSymbol;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel1_2;
    AmountEdit *edtFraction;
    QLabel *lblSymbol;
    QLabel *textLabel1;
    KComboBox *cmbInvestmentType;
    KLineEdit *edtEquityName;
    QSpacerItem *spacer6;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KNewEquityEntryDlg)
    {
        if (KNewEquityEntryDlg->objectName().isEmpty())
            KNewEquityEntryDlg->setObjectName("KNewEquityEntryDlg");
        KNewEquityEntryDlg->resize(417, 208);
        KNewEquityEntryDlg->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(KNewEquityEntryDlg);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        lblEquityName = new QLabel(KNewEquityEntryDlg);
        lblEquityName->setObjectName("lblEquityName");
        lblEquityName->setWordWrap(false);

        gridLayout->addWidget(lblEquityName, 0, 0, 1, 1);

        lblType = new QLabel(KNewEquityEntryDlg);
        lblType->setObjectName("lblType");
        lblType->setWordWrap(false);

        gridLayout->addWidget(lblType, 2, 0, 1, 1);

        edtMarketSymbol = new KLineEdit(KNewEquityEntryDlg);
        edtMarketSymbol->setObjectName("edtMarketSymbol");

        gridLayout->addWidget(edtMarketSymbol, 1, 1, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        textLabel1_2 = new QLabel(KNewEquityEntryDlg);
        textLabel1_2->setObjectName("textLabel1_2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textLabel1_2->sizePolicy().hasHeightForWidth());
        textLabel1_2->setSizePolicy(sizePolicy);
        textLabel1_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel1_2->setWordWrap(false);

        hboxLayout->addWidget(textLabel1_2);

        edtFraction = new AmountEdit(KNewEquityEntryDlg);
        edtFraction->setObjectName("edtFraction");

        hboxLayout->addWidget(edtFraction);


        gridLayout->addLayout(hboxLayout, 3, 1, 1, 1);

        lblSymbol = new QLabel(KNewEquityEntryDlg);
        lblSymbol->setObjectName("lblSymbol");
        lblSymbol->setWordWrap(false);

        gridLayout->addWidget(lblSymbol, 1, 0, 1, 1);

        textLabel1 = new QLabel(KNewEquityEntryDlg);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 3, 0, 1, 1);

        cmbInvestmentType = new KComboBox(KNewEquityEntryDlg);
        cmbInvestmentType->addItem(QString());
        cmbInvestmentType->addItem(QString());
        cmbInvestmentType->addItem(QString());
        cmbInvestmentType->setObjectName("cmbInvestmentType");

        gridLayout->addWidget(cmbInvestmentType, 2, 1, 1, 1);

        edtEquityName = new KLineEdit(KNewEquityEntryDlg);
        edtEquityName->setObjectName("edtEquityName");

        gridLayout->addWidget(edtEquityName, 0, 1, 1, 1);


        vboxLayout->addLayout(gridLayout);

        spacer6 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer6);

        buttonBox = new QDialogButtonBox(KNewEquityEntryDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);

        QWidget::setTabOrder(edtEquityName, edtMarketSymbol);
        QWidget::setTabOrder(edtMarketSymbol, cmbInvestmentType);

        retranslateUi(KNewEquityEntryDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KNewEquityEntryDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KNewEquityEntryDlg);
    } // setupUi

    void retranslateUi(QDialog *KNewEquityEntryDlg)
    {
        KNewEquityEntryDlg->setWindowTitle(tr2i18n("New Equity", nullptr));
        lblEquityName->setText(tr2i18n("Equity Name:", nullptr));
        lblType->setText(tr2i18n("Investment Type:", nullptr));
#if QT_CONFIG(tooltip)
        edtMarketSymbol->setToolTip(tr2i18n("Trading Symbol of the stock or mutual fund, not required.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        edtMarketSymbol->setWhatsThis(tr2i18n("Trading Symbol of the stock or mutual fund, not required.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1_2->setText(tr2i18n("1 /", nullptr));
        lblSymbol->setText(tr2i18n("Market Symbol:", nullptr));
        textLabel1->setText(tr2i18n("Smallest fraction:", nullptr));
        cmbInvestmentType->setItemText(0, tr2i18n("Stock", nullptr));
        cmbInvestmentType->setItemText(1, tr2i18n("Mutual Fund", nullptr));
        cmbInvestmentType->setItemText(2, tr2i18n("Bond", nullptr));

#if QT_CONFIG(tooltip)
        edtEquityName->setToolTip(tr2i18n("Name of the company, or mutual fund.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        edtEquityName->setWhatsThis(tr2i18n("Name of the company, or mutual fund.", nullptr));
#endif // QT_CONFIG(whatsthis)
    } // retranslateUi

};

namespace Ui {
    class KNewEquityEntryDlg: public Ui_KNewEquityEntryDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KNEWEQUITYENTRYDLG_H

