/********************************************************************************
** Form generated from reading UI file 'daterangedlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DATERANGEDLG_H
#define UI_DATERANGEDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kmymoneydateedit.h"
#include "kmymoneyperiodcombo.h"

QT_BEGIN_NAMESPACE

class Ui_DateRangeDlg
{
public:
    QGridLayout *gridLayout;
    QLabel *TextLabel4;
    KMyMoneyPeriodCombo *m_dateRange;
    QLabel *TextLabel5;
    KMyMoneyDateEdit *m_fromDate;
    QLabel *TextLabel6;
    KMyMoneyDateEdit *m_toDate;

    void setupUi(QWidget *DateRangeDlg)
    {
        if (DateRangeDlg->objectName().isEmpty())
            DateRangeDlg->setObjectName("DateRangeDlg");
        gridLayout = new QGridLayout(DateRangeDlg);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        TextLabel4 = new QLabel(DateRangeDlg);
        TextLabel4->setObjectName("TextLabel4");
        TextLabel4->setWordWrap(false);

        gridLayout->addWidget(TextLabel4, 0, 0, 1, 1);

        m_dateRange = new KMyMoneyPeriodCombo(DateRangeDlg);
        m_dateRange->setObjectName("m_dateRange");

        gridLayout->addWidget(m_dateRange, 0, 1, 1, 1);

        TextLabel5 = new QLabel(DateRangeDlg);
        TextLabel5->setObjectName("TextLabel5");
        TextLabel5->setWordWrap(false);

        gridLayout->addWidget(TextLabel5, 1, 0, 1, 1);

        m_fromDate = new KMyMoneyDateEdit(DateRangeDlg);
        m_fromDate->setObjectName("m_fromDate");

        gridLayout->addWidget(m_fromDate, 1, 1, 1, 1);

        TextLabel6 = new QLabel(DateRangeDlg);
        TextLabel6->setObjectName("TextLabel6");
        TextLabel6->setWordWrap(false);

        gridLayout->addWidget(TextLabel6, 2, 0, 1, 1);

        m_toDate = new KMyMoneyDateEdit(DateRangeDlg);
        m_toDate->setObjectName("m_toDate");

        gridLayout->addWidget(m_toDate, 2, 1, 1, 1);


        retranslateUi(DateRangeDlg);

        QMetaObject::connectSlotsByName(DateRangeDlg);
    } // setupUi

    void retranslateUi(QWidget *DateRangeDlg)
    {
        TextLabel4->setText(tr2i18n("Range", nullptr));
        TextLabel5->setText(tr2i18n("From", "From (Date)"));
        TextLabel6->setText(tr2i18n("To", "To (Date)"));
        (void)DateRangeDlg;
    } // retranslateUi

};

namespace Ui {
    class DateRangeDlg: public Ui_DateRangeDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // DATERANGEDLG_H

