/********************************************************************************
** Form generated from reading UI file 'reportcontrol.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REPORTCONTROL_H
#define UI_REPORTCONTROL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ReportControl
{
public:
    QHBoxLayout *hboxLayout;
    QToolButton *buttonChart;
    QToolButton *buttonConfigure;
    QToolButton *buttonNew;
    QToolButton *buttonExport;
    QToolButton *buttonDelete;

    void setupUi(QWidget *ReportControl)
    {
        if (ReportControl->objectName().isEmpty())
            ReportControl->setObjectName("ReportControl");
        ReportControl->resize(600, 36);
        hboxLayout = new QHBoxLayout(ReportControl);
        hboxLayout->setSpacing(1);
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(1, 1, 1, 1);
        buttonChart = new QToolButton(ReportControl);
        buttonChart->setObjectName("buttonChart");
        buttonChart->setMinimumSize(QSize(75, 0));

        hboxLayout->addWidget(buttonChart);

        buttonConfigure = new QToolButton(ReportControl);
        buttonConfigure->setObjectName("buttonConfigure");
        buttonConfigure->setMinimumSize(QSize(75, 0));

        hboxLayout->addWidget(buttonConfigure);

        buttonNew = new QToolButton(ReportControl);
        buttonNew->setObjectName("buttonNew");
        buttonNew->setMinimumSize(QSize(75, 0));

        hboxLayout->addWidget(buttonNew);

        buttonExport = new QToolButton(ReportControl);
        buttonExport->setObjectName("buttonExport");
        buttonExport->setMinimumSize(QSize(75, 0));

        hboxLayout->addWidget(buttonExport);

        buttonDelete = new QToolButton(ReportControl);
        buttonDelete->setObjectName("buttonDelete");
        buttonDelete->setMinimumSize(QSize(75, 0));

        hboxLayout->addWidget(buttonDelete);


        retranslateUi(ReportControl);

        QMetaObject::connectSlotsByName(ReportControl);
    } // setupUi

    void retranslateUi(QWidget *ReportControl)
    {
#if QT_CONFIG(tooltip)
        buttonChart->setToolTip(tr2i18n("Show the chart version of this report", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonChart->setText(tr2i18n("Chart", nullptr));
#if QT_CONFIG(tooltip)
        buttonConfigure->setToolTip(tr2i18n("Configure this report", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonConfigure->setText(tr2i18n("Configure", nullptr));
#if QT_CONFIG(tooltip)
        buttonNew->setToolTip(tr2i18n("Create a new report based on this one", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonNew->setText(tr2i18n("New", "@action create new report"));
#if QT_CONFIG(tooltip)
        buttonExport->setToolTip(tr2i18n("Export this report as an HTML or CSV file", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonExport->setText(tr2i18n("&Export", nullptr));
#if QT_CONFIG(tooltip)
        buttonDelete->setToolTip(tr2i18n("Permanently delete this report", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonDelete->setText(tr2i18n("Delete", nullptr));
        (void)ReportControl;
    } // retranslateUi

};

namespace Ui {
    class ReportControl: public Ui_ReportControl {};
} // namespace Ui

QT_END_NAMESPACE

#endif // REPORTCONTROL_H

