/********************************************************************************
** Form generated from reading UI file 'ktransactionselectdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KTRANSACTIONSELECTDLG_H
#define UI_KTRANSACTIONSELECTDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "ledgerview.h"

QT_BEGIN_NAMESPACE

class Ui_KTransactionSelectDlg
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    LedgerView *m_ledgerView;
    QVBoxLayout *verticalLayout;
    QSpacerItem *vs2;
    QToolButton *switchButton;
    QSpacerItem *vs1;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KTransactionSelectDlg)
    {
        if (KTransactionSelectDlg->objectName().isEmpty())
            KTransactionSelectDlg->setObjectName("KTransactionSelectDlg");
        KTransactionSelectDlg->resize(768, 326);
        verticalLayout_2 = new QVBoxLayout(KTransactionSelectDlg);
        verticalLayout_2->setObjectName("verticalLayout_2");
        label = new QLabel(KTransactionSelectDlg);
        label->setObjectName("label");
        label->setText(QString::fromUtf8("xxx"));
        label->setWordWrap(true);

        verticalLayout_2->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        m_ledgerView = new LedgerView(KTransactionSelectDlg);
        m_ledgerView->setObjectName("m_ledgerView");

        horizontalLayout->addWidget(m_ledgerView);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        vs2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(vs2);

        switchButton = new QToolButton(KTransactionSelectDlg);
        switchButton->setObjectName("switchButton");

        verticalLayout->addWidget(switchButton);

        vs1 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(vs1);


        horizontalLayout->addLayout(verticalLayout);


        verticalLayout_2->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(KTransactionSelectDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);


        retranslateUi(KTransactionSelectDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KTransactionSelectDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KTransactionSelectDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KTransactionSelectDlg);
    } // setupUi

    void retranslateUi(QDialog *KTransactionSelectDlg)
    {
        KTransactionSelectDlg->setWindowTitle(tr2i18n("Select Transaction", nullptr));
        switchButton->setText(tr2i18n("...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KTransactionSelectDlg: public Ui_KTransactionSelectDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KTRANSACTIONSELECTDLG_H

