/********************************************************************************
** Form generated from reading UI file 'kscheduledview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KSCHEDULEDVIEW_H
#define UI_KSCHEDULEDVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kmymoneytreeview.h"

QT_BEGIN_NAMESPACE

class Ui_KScheduledView
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QPushButton *m_qbuttonNew;
    QLineEdit *m_searchWidget;
    QFrame *line1;
    KMyMoneyTreeView *m_scheduleTree;

    void setupUi(QWidget *KScheduledView)
    {
        if (KScheduledView->objectName().isEmpty())
            KScheduledView->setObjectName("KScheduledView");
        KScheduledView->resize(759, 450);
        vboxLayout = new QVBoxLayout(KScheduledView);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_qbuttonNew = new QPushButton(KScheduledView);
        m_qbuttonNew->setObjectName("m_qbuttonNew");

        hboxLayout->addWidget(m_qbuttonNew);

        m_searchWidget = new QLineEdit(KScheduledView);
        m_searchWidget->setObjectName("m_searchWidget");
        m_searchWidget->setClearButtonEnabled(true);

        hboxLayout->addWidget(m_searchWidget);


        vboxLayout->addLayout(hboxLayout);

        line1 = new QFrame(KScheduledView);
        line1->setObjectName("line1");
        line1->setFrameShape(QFrame::HLine);
        line1->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(line1);

        m_scheduleTree = new KMyMoneyTreeView(KScheduledView);
        m_scheduleTree->setObjectName("m_scheduleTree");
        m_scheduleTree->setAlternatingRowColors(true);
        m_scheduleTree->setAllColumnsShowFocus(true);

        vboxLayout->addWidget(m_scheduleTree);


        retranslateUi(KScheduledView);

        QMetaObject::connectSlotsByName(KScheduledView);
    } // setupUi

    void retranslateUi(QWidget *KScheduledView)
    {
        m_qbuttonNew->setText(tr2i18n("New Scheduled transaction...", nullptr));
        m_searchWidget->setPlaceholderText(tr2i18n("Filter", "Placeholdertext in schedulesview"));
        (void)KScheduledView;
    } // retranslateUi

};

namespace Ui {
    class KScheduledView: public Ui_KScheduledView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KSCHEDULEDVIEW_H

