/********************************************************************************
** Form generated from reading UI file 'kinstitutionpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KINSTITUTIONPAGE_H
#define UI_KINSTITUTIONPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KInstitutionPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel1;
    QHBoxLayout *hboxLayout;
    QGridLayout *gridLayout;
    QLabel *textLabel4;
    KComboBox *m_institutionComboBox;
    QPushButton *m_newInstitutionButton;
    QLabel *textLabel5;
    KLineEdit *m_accountNumber;
    QLabel *textLabel1;
    KLineEdit *m_iban;
    QSpacerItem *spacer16;
    QSpacerItem *spacer17;

    void setupUi(QWidget *KInstitutionPage)
    {
        if (KInstitutionPage->objectName().isEmpty())
            KInstitutionPage->setObjectName("KInstitutionPage");
        KInstitutionPage->resize(586, 322);
        vboxLayout = new QVBoxLayout(KInstitutionPage);
        vboxLayout->setSpacing(-1);
        vboxLayout->setContentsMargins(10, 10, 10, 10);
        vboxLayout->setObjectName("vboxLayout");
        qLabel1 = new QLabel(KInstitutionPage);
        qLabel1->setObjectName("qLabel1");
        qLabel1->setFocusPolicy(Qt::NoFocus);
        qLabel1->setWordWrap(true);

        vboxLayout->addWidget(qLabel1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(-1);
        hboxLayout->setObjectName("hboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        textLabel4 = new QLabel(KInstitutionPage);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 0, 0, 1, 1);

        m_institutionComboBox = new KComboBox(KInstitutionPage);
        m_institutionComboBox->setObjectName("m_institutionComboBox");

        gridLayout->addWidget(m_institutionComboBox, 0, 1, 1, 1);

        m_newInstitutionButton = new QPushButton(KInstitutionPage);
        m_newInstitutionButton->setObjectName("m_newInstitutionButton");

        gridLayout->addWidget(m_newInstitutionButton, 0, 2, 1, 1);

        textLabel5 = new QLabel(KInstitutionPage);
        textLabel5->setObjectName("textLabel5");
        textLabel5->setWordWrap(false);

        gridLayout->addWidget(textLabel5, 1, 0, 1, 1);

        m_accountNumber = new KLineEdit(KInstitutionPage);
        m_accountNumber->setObjectName("m_accountNumber");

        gridLayout->addWidget(m_accountNumber, 1, 1, 1, 1);

        textLabel1 = new QLabel(KInstitutionPage);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 2, 0, 1, 1);

        m_iban = new KLineEdit(KInstitutionPage);
        m_iban->setObjectName("m_iban");

        gridLayout->addWidget(m_iban, 2, 1, 1, 2);


        hboxLayout->addLayout(gridLayout);

        spacer16 = new QSpacerItem(132, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer16);


        vboxLayout->addLayout(hboxLayout);

        spacer17 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer17);


        retranslateUi(KInstitutionPage);

        QMetaObject::connectSlotsByName(KInstitutionPage);
    } // setupUi

    void retranslateUi(QWidget *KInstitutionPage)
    {
        qLabel1->setText(tr2i18n("If this account is managed by an institution please select the institution from the list. If the institution does not exist yet, please choose the <b>New Institution</b> button to create it. Otherwise, leave this field empty.\n"
"Enter the account number used by the institution to identify the account.", nullptr));
        textLabel4->setText(tr2i18n("Institution", nullptr));
#if QT_CONFIG(tooltip)
        m_institutionComboBox->setToolTip(tr2i18n("Select the institution which manages this account or leave empty", nullptr));
#endif // QT_CONFIG(tooltip)
        m_newInstitutionButton->setText(tr2i18n("New Institution", nullptr));
        textLabel5->setText(tr2i18n("Account number", nullptr));
#if QT_CONFIG(whatsthis)
        m_accountNumber->setWhatsThis(tr2i18n("Accounts managed by an institution are usually referenced by a unique number the so called account number. Enter that number here.\n"
"\n"
"KMyMoney currently uses this number only for some online banking functions.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1->setText(tr2i18n("IBAN", nullptr));
#if QT_CONFIG(tooltip)
        m_iban->setToolTip(tr2i18n("Enter the International Bank Account Number into this field", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_iban->setWhatsThis(tr2i18n("You can enter the IBAN (International Bank Account Number) into this field if you know it. Usually, people at your institution can tell you the number or it is printed on your statements.\n"
"\n"
"See also https://en.wikipedia.org/wiki/International_Bank_Account_Number for more information.\n"
"\n"
"KMyMoney keeps this field only for documentation purposes and does not use it otherwise.", nullptr));
#endif // QT_CONFIG(whatsthis)
        (void)KInstitutionPage;
    } // retranslateUi

};

namespace Ui {
    class KInstitutionPage: public Ui_KInstitutionPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KINSTITUTIONPAGE_H

