/********************************************************************************
** Form generated from reading UI file 'kmmsearchwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KMMSEARCHWIDGET_H
#define UI_KMMSEARCHWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KMMSearchWidget
{
public:
    QHBoxLayout *horizontalLayout;
    QToolButton *m_closeButton;
    QToolButton *m_pinButton;
    QLineEdit *m_searchWidget;
    QComboBox *m_filterBox;

    void setupUi(QWidget *KMMSearchWidget)
    {
        if (KMMSearchWidget->objectName().isEmpty())
            KMMSearchWidget->setObjectName("KMMSearchWidget");
        KMMSearchWidget->resize(440, 34);
        horizontalLayout = new QHBoxLayout(KMMSearchWidget);
        horizontalLayout->setSpacing(2);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 1, 0, 1);
        m_closeButton = new QToolButton(KMMSearchWidget);
        m_closeButton->setObjectName("m_closeButton");
        m_closeButton->setAutoRaise(true);

        horizontalLayout->addWidget(m_closeButton);

        m_pinButton = new QToolButton(KMMSearchWidget);
        m_pinButton->setObjectName("m_pinButton");
        m_pinButton->setCheckable(true);
        m_pinButton->setAutoRaise(true);

        horizontalLayout->addWidget(m_pinButton);

        m_searchWidget = new QLineEdit(KMMSearchWidget);
        m_searchWidget->setObjectName("m_searchWidget");

        horizontalLayout->addWidget(m_searchWidget);

        m_filterBox = new QComboBox(KMMSearchWidget);
        m_filterBox->setObjectName("m_filterBox");
        m_filterBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        horizontalLayout->addWidget(m_filterBox);

        horizontalLayout->setStretch(2, 2);

        retranslateUi(KMMSearchWidget);

        QMetaObject::connectSlotsByName(KMMSearchWidget);
    } // setupUi

    void retranslateUi(QWidget *KMMSearchWidget)
    {
        KMMSearchWidget->setWindowTitle(tr2i18n("Form", nullptr));
        m_closeButton->setText(tr2i18n("...", nullptr));
#if QT_CONFIG(tooltip)
        m_pinButton->setToolTip(tr2i18n("Used to keep the filter widget visible. If not visible, it can be shown using Ctrl+F", nullptr));
#endif // QT_CONFIG(tooltip)
        m_pinButton->setText(tr2i18n("...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KMMSearchWidget: public Ui_KMMSearchWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KMMSEARCHWIDGET_H

