/********************************************************************************
** Form generated from reading UI file 'kendingbalancedlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KENDINGBALANCEDLG_H
#define UI_KENDINGBALANCEDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QWizard>
#include <klocalizedstring.h>
#include "amountedit.h"
#include "checkingstartwizardpage.h"
#include "checkingstatementinfowizardpage.h"
#include "interestchargecheckingswizardpage.h"
#include "kcombobox.h"
#include "kmymoneyaccountselector.h"
#include "kmymoneycategory.h"
#include "kmymoneydateedit.h"
#include "kmymoneypayeecombo.h"
#include "previouspostponewizardpage.h"

QT_BEGIN_NAMESPACE

class Ui_KEndingBalanceDlg
{
public:
    CheckingStartWizardPage *m_startPageCheckings;
    PreviousPostponeWizardPage *m_pagePreviousPostpone;
    CheckingStatementInfoWizardPage *m_statementInfoPageCheckings;
    InterestChargeCheckingsWizardPage *m_interestChargeCheckings;

    void setupUi(QWizard *KEndingBalanceDlg)
    {
        if (KEndingBalanceDlg->objectName().isEmpty())
            KEndingBalanceDlg->setObjectName("KEndingBalanceDlg");
        KEndingBalanceDlg->resize(589, 465);
        m_startPageCheckings = new CheckingStartWizardPage();
        m_startPageCheckings->setObjectName("m_startPageCheckings");
        KEndingBalanceDlg->addPage(m_startPageCheckings);
        m_pagePreviousPostpone = new PreviousPostponeWizardPage();
        m_pagePreviousPostpone->setObjectName("m_pagePreviousPostpone");
        KEndingBalanceDlg->addPage(m_pagePreviousPostpone);
        m_statementInfoPageCheckings = new CheckingStatementInfoWizardPage();
        m_statementInfoPageCheckings->setObjectName("m_statementInfoPageCheckings");
        KEndingBalanceDlg->addPage(m_statementInfoPageCheckings);
        m_interestChargeCheckings = new InterestChargeCheckingsWizardPage();
        m_interestChargeCheckings->setObjectName("m_interestChargeCheckings");
        KEndingBalanceDlg->addPage(m_interestChargeCheckings);

        retranslateUi(KEndingBalanceDlg);

        QMetaObject::connectSlotsByName(KEndingBalanceDlg);
    } // setupUi

    void retranslateUi(QWizard *KEndingBalanceDlg)
    {
        KEndingBalanceDlg->setWindowTitle(tr2i18n("Reconciliation Wizard", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KEndingBalanceDlg: public Ui_KEndingBalanceDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KENDINGBALANCEDLG_H

