/********************************************************************************
** Form generated from reading UI file 'schedulewizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCHEDULEWIZARDPAGE_H
#define UI_SCHEDULEWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>
#include <kmymoneyaccountselector.h>
#include "kdatecombobox.h"
#include "kmymoneycategory.h"
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_ScheduleWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel22_2;
    QSpacerItem *Spacer67_3;
    QGridLayout *gridLayout;
    KMyMoneyAccountSelector *m_paymentAccountEdit;
    KMyMoneyDateEdit *m_nextDueDateEdit;
    QLabel *m_dueLabel;
    QLabel *TextLabel2_5;
    QSpacerItem *Spacer68_2;

    void setupUi(QWizardPage *ScheduleWizardPage)
    {
        if (ScheduleWizardPage->objectName().isEmpty())
            ScheduleWizardPage->setObjectName("ScheduleWizardPage");
        vboxLayout = new QVBoxLayout(ScheduleWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel22_2 = new QLabel(ScheduleWizardPage);
        TextLabel22_2->setObjectName("TextLabel22_2");
        TextLabel22_2->setAlignment(Qt::AlignTop);
        TextLabel22_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel22_2);

        Spacer67_3 = new QSpacerItem(0, 23, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer67_3);

        gridLayout = new QGridLayout();
#ifndef UI_Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName("gridLayout");
        m_paymentAccountEdit = new KMyMoneyAccountSelector(ScheduleWizardPage);
        m_paymentAccountEdit->setObjectName("m_paymentAccountEdit");

        gridLayout->addWidget(m_paymentAccountEdit, 1, 1, 1, 1);

        m_nextDueDateEdit = new KMyMoneyDateEdit(ScheduleWizardPage);
        m_nextDueDateEdit->setObjectName("m_nextDueDateEdit");

        gridLayout->addWidget(m_nextDueDateEdit, 0, 1, 1, 1);

        m_dueLabel = new QLabel(ScheduleWizardPage);
        m_dueLabel->setObjectName("m_dueLabel");
        m_dueLabel->setWordWrap(false);

        gridLayout->addWidget(m_dueLabel, 0, 0, 1, 1);

        TextLabel2_5 = new QLabel(ScheduleWizardPage);
        TextLabel2_5->setObjectName("TextLabel2_5");
        TextLabel2_5->setAlignment(Qt::AlignTop);
        TextLabel2_5->setWordWrap(false);

        gridLayout->addWidget(TextLabel2_5, 1, 0, 1, 1);


        vboxLayout->addLayout(gridLayout);

        Spacer68_2 = new QSpacerItem(0, 24, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer68_2);


        retranslateUi(ScheduleWizardPage);

        QMetaObject::connectSlotsByName(ScheduleWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *ScheduleWizardPage)
    {
        TextLabel22_2->setText(tr2i18n("KMyMoney will create a schedule for this payment and reminds you whenever a payment must be made.<p>\n"
"If you selected to record all payments this date has already been supplied. If you selected to record only this years payments, then the <b>First payment due date</b> is the date of the first payment made in this year.", nullptr));
        m_dueLabel->setText(tr2i18n("First payment due on:", nullptr));
        TextLabel2_5->setText(tr2i18n("Make payment from/to:", nullptr));
        (void)ScheduleWizardPage;
    } // retranslateUi

};

namespace Ui {
    class ScheduleWizardPage: public Ui_ScheduleWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SCHEDULEWIZARDPAGE_H

