/********************************************************************************
** Form generated from reading UI file 'securitydlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SECURITYDLG_H
#define UI_SECURITYDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SecurityDlg
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label_title;
    QLabel *label_info;
    QGridLayout *gridLayout;
    QComboBox *cbSecurity;
    QLineEdit *leNewSymbol;
    QLineEdit *leNewName;
    QLabel *lblSecurity;
    QLabel *lblNewSymbol;
    QLabel *lblNewName;
    QCheckBox *cbDontAsk;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *SecurityDlg)
    {
        if (SecurityDlg->objectName().isEmpty())
            SecurityDlg->setObjectName("SecurityDlg");
        SecurityDlg->resize(300, 280);
        verticalLayout = new QVBoxLayout(SecurityDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        label_title = new QLabel(SecurityDlg);
        label_title->setObjectName("label_title");
        QFont font;
        font.setBold(true);
        label_title->setFont(font);
        label_title->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(label_title);

        label_info = new QLabel(SecurityDlg);
        label_info->setObjectName("label_info");
        label_info->setAlignment(Qt::AlignCenter);
        label_info->setWordWrap(true);

        verticalLayout->addWidget(label_info);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        cbSecurity = new QComboBox(SecurityDlg);
        cbSecurity->setObjectName("cbSecurity");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(cbSecurity->sizePolicy().hasHeightForWidth());
        cbSecurity->setSizePolicy(sizePolicy);

        gridLayout->addWidget(cbSecurity, 0, 1, 1, 1);

        leNewSymbol = new QLineEdit(SecurityDlg);
        leNewSymbol->setObjectName("leNewSymbol");
        sizePolicy.setHeightForWidth(leNewSymbol->sizePolicy().hasHeightForWidth());
        leNewSymbol->setSizePolicy(sizePolicy);

        gridLayout->addWidget(leNewSymbol, 1, 1, 1, 1);

        leNewName = new QLineEdit(SecurityDlg);
        leNewName->setObjectName("leNewName");
        sizePolicy.setHeightForWidth(leNewName->sizePolicy().hasHeightForWidth());
        leNewName->setSizePolicy(sizePolicy);

        gridLayout->addWidget(leNewName, 2, 1, 1, 1);

        lblSecurity = new QLabel(SecurityDlg);
        lblSecurity->setObjectName("lblSecurity");

        gridLayout->addWidget(lblSecurity, 0, 0, 1, 1);

        lblNewSymbol = new QLabel(SecurityDlg);
        lblNewSymbol->setObjectName("lblNewSymbol");

        gridLayout->addWidget(lblNewSymbol, 1, 0, 1, 1);

        lblNewName = new QLabel(SecurityDlg);
        lblNewName->setObjectName("lblNewName");

        gridLayout->addWidget(lblNewName, 2, 0, 1, 1);

        cbDontAsk = new QCheckBox(SecurityDlg);
        cbDontAsk->setObjectName("cbDontAsk");
        sizePolicy.setHeightForWidth(cbDontAsk->sizePolicy().hasHeightForWidth());
        cbDontAsk->setSizePolicy(sizePolicy);
        cbDontAsk->setLayoutDirection(Qt::RightToLeft);

        gridLayout->addWidget(cbDontAsk, 3, 1, 1, 1, Qt::AlignLeft);


        verticalLayout->addLayout(gridLayout);

        buttonBox = new QDialogButtonBox(SecurityDlg);
        buttonBox->setObjectName("buttonBox");
        sizePolicy.setHeightForWidth(buttonBox->sizePolicy().hasHeightForWidth());
        buttonBox->setSizePolicy(sizePolicy);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
        buttonBox->setCenterButtons(true);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(SecurityDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, SecurityDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, SecurityDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(SecurityDlg);
    } // setupUi

    void retranslateUi(QDialog *SecurityDlg)
    {
        SecurityDlg->setWindowTitle(tr2i18n("Security", nullptr));
        label_title->setText(tr2i18n("Missing name and symbol", nullptr));
        label_info->setText(tr2i18n("You need security to import into,\n"
"so please select existing security or enter new security name and symbol.", nullptr));
        lblSecurity->setText(tr2i18n("Existing security", nullptr));
        lblNewSymbol->setText(tr2i18n("New symbol", nullptr));
        lblNewName->setText(tr2i18n("New name", nullptr));
        cbDontAsk->setText(tr2i18n("Do not ask next time", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SecurityDlg: public Ui_SecurityDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SECURITYDLG_H

