/****************************************************************************
** Meta object code from reading C++ file 'amountedit.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/widgets/amountedit.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'amountedit.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10AmountEditE_t {};
} // unnamed namespace

template <> constexpr inline auto AmountEdit::qt_create_metaobjectdata<qt_meta_tag_ZN10AmountEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AmountEdit",
        "amountChanged",
        "",
        "validatedTextChanged",
        "text",
        "displayStateChanged",
        "MultiCurrencyEdit::DisplayState",
        "state",
        "calculatorButtonVisibilityChanged",
        "show",
        "allowEmptyChanged",
        "allowed",
        "readOnlyChanged",
        "readOnly",
        "validityChanged",
        "isValid",
        "theTextChanged",
        "slotCalculatorResult",
        "slotCalculatorOpen",
        "slotCalculatorClose",
        "setCalculatorButtonVisible",
        "setDisplayState",
        "setReadOnly",
        "ro",
        "setEnabled",
        "enable",
        "calculatorButtonVisibility",
        "allowEmpty",
        "value",
        "MyMoneyMoney",
        "valid"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'amountChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'validatedTextChanged'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'displayStateChanged'
        QtMocHelpers::SignalData<void(MultiCurrencyEdit::DisplayState)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'calculatorButtonVisibilityChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'allowEmptyChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'readOnlyChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'validityChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'theTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'slotCalculatorResult'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCalculatorOpen'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCalculatorClose'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'setCalculatorButtonVisible'
        QtMocHelpers::SlotData<void(const bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'setDisplayState'
        QtMocHelpers::SlotData<void(MultiCurrencyEdit::DisplayState)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setReadOnly'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Slot 'setEnabled'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'calculatorButtonVisibility'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'allowEmpty'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'readOnly'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'value'
        QtMocHelpers::PropertyData<MyMoneyMoney>(28, 0x80000000 | 29, QMC::Readable | QMC::Writable | QMC::Scriptable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::User, 0),
        // property 'valid'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::Readable | QMC::Scriptable, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AmountEdit, qt_meta_tag_ZN10AmountEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AmountEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QLineEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10AmountEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10AmountEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10AmountEditE_t>.metaTypes,
    nullptr
} };

void AmountEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AmountEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->amountChanged(); break;
        case 1: _t->validatedTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->displayStateChanged((*reinterpret_cast<std::add_pointer_t<MultiCurrencyEdit::DisplayState>>(_a[1]))); break;
        case 3: _t->calculatorButtonVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->allowEmptyChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->readOnlyChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->validityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->theTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->slotCalculatorResult(); break;
        case 9: _t->slotCalculatorOpen(); break;
        case 10: _t->slotCalculatorClose(); break;
        case 11: _t->setCalculatorButtonVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->setDisplayState((*reinterpret_cast<std::add_pointer_t<MultiCurrencyEdit::DisplayState>>(_a[1]))); break;
        case 13: _t->setReadOnly((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->setEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AmountEdit::*)()>(_a, &AmountEdit::amountChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AmountEdit::*)(const QString & )>(_a, &AmountEdit::validatedTextChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AmountEdit::*)(MultiCurrencyEdit::DisplayState )>(_a, &AmountEdit::displayStateChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AmountEdit::*)(bool )>(_a, &AmountEdit::calculatorButtonVisibilityChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AmountEdit::*)(bool )>(_a, &AmountEdit::allowEmptyChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AmountEdit::*)(bool )>(_a, &AmountEdit::readOnlyChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AmountEdit::*)(bool )>(_a, &AmountEdit::validityChanged, 6))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< MyMoneyMoney >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isCalculatorButtonVisible(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isEmptyAllowed(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isReadOnly(); break;
        case 3: *reinterpret_cast<MyMoneyMoney*>(_v) = _t->value(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isValid(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCalculatorButtonVisible(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setAllowEmpty(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setReadOnly(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setValue(*reinterpret_cast<MyMoneyMoney*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *AmountEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AmountEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10AmountEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "MultiCurrencyEdit"))
        return static_cast< MultiCurrencyEdit*>(this);
    return QLineEdit::qt_metacast(_clname);
}

int AmountEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void AmountEdit::amountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void AmountEdit::validatedTextChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void AmountEdit::displayStateChanged(MultiCurrencyEdit::DisplayState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void AmountEdit::calculatorButtonVisibilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void AmountEdit::allowEmptyChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void AmountEdit::readOnlyChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void AmountEdit::validityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
