/*
    SPDX-FileCopyrightText: 2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#ifndef REPORTSVIEWENUMS_H
#define REPORTSVIEWENUMS_H

namespace eWidgetPlugin {
enum WidgetType {
    NetWorthForecast,
    NetWorthForecastWithArgs, // for forecast view
    Budget,
};
}

#endif
