/********************************************************************************
** Form generated from reading UI file 'kloanschedulepage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KLOANSCHEDULEPAGE_H
#define UI_KLOANSCHEDULEPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kdatecombobox.h"
#include "kmymoneycategory.h"
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_KLoanSchedulePage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel2;
    QHBoxLayout *hboxLayout;
    QGridLayout *gridLayout;
    QLabel *textLabel7;
    KMyMoneyCategory *m_interestCategory;
    QLabel *textLabel8;
    KMyMoneyCategory *m_paymentAccount;
    QLabel *textLabel1;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacer89;
    KMyMoneyDateEdit *m_firstPaymentDueDate;
    QSpacerItem *spacer4;
    QSpacerItem *spacer88;

    void setupUi(QWidget *KLoanSchedulePage)
    {
        if (KLoanSchedulePage->objectName().isEmpty())
            KLoanSchedulePage->setObjectName("KLoanSchedulePage");
        KLoanSchedulePage->resize(545, 369);
        vboxLayout = new QVBoxLayout(KLoanSchedulePage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        qLabel2 = new QLabel(KLoanSchedulePage);
        qLabel2->setObjectName("qLabel2");
        qLabel2->setFocusPolicy(Qt::NoFocus);
        qLabel2->setWordWrap(true);

        vboxLayout->addWidget(qLabel2);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        textLabel7 = new QLabel(KLoanSchedulePage);
        textLabel7->setObjectName("textLabel7");
        textLabel7->setWordWrap(false);

        gridLayout->addWidget(textLabel7, 0, 0, 1, 1);

        m_interestCategory = new KMyMoneyCategory(KLoanSchedulePage);
        m_interestCategory->setObjectName("m_interestCategory");

        gridLayout->addWidget(m_interestCategory, 0, 1, 1, 1);

        textLabel8 = new QLabel(KLoanSchedulePage);
        textLabel8->setObjectName("textLabel8");
        textLabel8->setWordWrap(false);

        gridLayout->addWidget(textLabel8, 1, 0, 1, 1);

        m_paymentAccount = new KMyMoneyCategory(KLoanSchedulePage);
        m_paymentAccount->setObjectName("m_paymentAccount");

        gridLayout->addWidget(m_paymentAccount, 1, 1, 1, 1);

        textLabel1 = new QLabel(KLoanSchedulePage);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 2, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        spacer89 = new QSpacerItem(61, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacer89);

        m_firstPaymentDueDate = new KMyMoneyDateEdit(KLoanSchedulePage);
        m_firstPaymentDueDate->setObjectName("m_firstPaymentDueDate");

        hboxLayout1->addWidget(m_firstPaymentDueDate);


        gridLayout->addLayout(hboxLayout1, 2, 1, 1, 1);


        hboxLayout->addLayout(gridLayout);

        spacer4 = new QSpacerItem(41, 20, QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer4);


        vboxLayout->addLayout(hboxLayout);

        spacer88 = new QSpacerItem(20, 21, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer88);


        retranslateUi(KLoanSchedulePage);

        QMetaObject::connectSlotsByName(KLoanSchedulePage);
    } // setupUi

    void retranslateUi(QWidget *KLoanSchedulePage)
    {
        qLabel2->setText(tr2i18n("KMyMoney creates a schedule for this payment and reminds you whenever a payment must be made. Please select the account to/from which payments will be made and the category the interest will be assigned to.<p>\n"
"If you selected to record all payments this date has already been supplied. If you selected to record only this years payments, then the <b>First payment due date</b> is the date of the first payment made in this year.", nullptr));
        textLabel7->setText(tr2i18n("Interest category", nullptr));
#if QT_CONFIG(whatsthis)
        m_interestCategory->setWhatsThis(tr2i18n("The category the interest part of the payment will be assigned to. If you borrow money you usually have to pay interest, so this should be an expense category. If you lend the money, you receive the interest. In this case, select an income category here.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel8->setText(tr2i18n("Payment account", nullptr));
#if QT_CONFIG(whatsthis)
        m_paymentAccount->setWhatsThis(tr2i18n("Select the account from which you make your periodical payments or where you receive regular payments in case you lend the money. In most cases, this is a checking account.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1->setText(tr2i18n("First payment due on", nullptr));
        (void)KLoanSchedulePage;
    } // retranslateUi

};

namespace Ui {
    class KLoanSchedulePage: public Ui_KLoanSchedulePage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KLOANSCHEDULEPAGE_H

