/********************************************************************************
** Form generated from reading UI file 'kpreferencepage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPREFERENCEPAGE_H
#define UI_KPREFERENCEPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KPreferencePage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *qLabel1;
    QHBoxLayout *hboxLayout;
    QCheckBox *m_openConfigAfterFinished;
    QSpacerItem *spacer2;
    QLabel *qLabel2;

    void setupUi(QWidget *KPreferencePage)
    {
        if (KPreferencePage->objectName().isEmpty())
            KPreferencePage->setObjectName("KPreferencePage");
        KPreferencePage->resize(600, 288);
        vboxLayout = new QVBoxLayout(KPreferencePage);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        qLabel1 = new QLabel(KPreferencePage);
        qLabel1->setObjectName("qLabel1");
        qLabel1->setFocusPolicy(Qt::NoFocus);
        qLabel1->setWordWrap(true);

        vboxLayout->addWidget(qLabel1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_openConfigAfterFinished = new QCheckBox(KPreferencePage);
        m_openConfigAfterFinished->setObjectName("m_openConfigAfterFinished");

        hboxLayout->addWidget(m_openConfigAfterFinished);


        vboxLayout->addLayout(hboxLayout);

        spacer2 = new QSpacerItem(20, 31, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer2);

        qLabel2 = new QLabel(KPreferencePage);
        qLabel2->setObjectName("qLabel2");
        qLabel2->setFocusPolicy(Qt::NoFocus);
        qLabel2->setWordWrap(true);

        vboxLayout->addWidget(qLabel2);


        retranslateUi(KPreferencePage);

        QMetaObject::connectSlotsByName(KPreferencePage);
    } // setupUi

    void retranslateUi(QWidget *KPreferencePage)
    {
        qLabel1->setText(tr2i18n("If you would like to review or change any preference settings for KMyMoney after finishing this wizard, you can do so by checking this box. If not just click <b>Next</b>.", nullptr));
        m_openConfigAfterFinished->setText(tr2i18n("Configure preferences after finishing the wizard.", nullptr));
        qLabel2->setText(tr2i18n("You can change the configuration options at any time using the configuration dialog. Just select the <b>Configure KMyMoney...</b> option in the <b>Settings</b> menu.", nullptr));
        (void)KPreferencePage;
    } // retranslateUi

};

namespace Ui {
    class KPreferencePage: public Ui_KPreferencePage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KPREFERENCEPAGE_H

