/********************************************************************************
** Form generated from reading UI file 'investtransactioneditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INVESTTRANSACTIONEDITOR_H
#define UI_INVESTTRANSACTIONEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "amountedit.h"
#include "kdatecombobox.h"
#include "kmessagewidget.h"
#include "kmymoneyaccountcombo.h"
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_InvestTransactionEditor
{
public:
    QGridLayout *gridLayout;
    AmountEdit *priceAmountEdit;
    AmountEdit *totalAmountEdit;
    QLabel *interestAmountLabel;
    QHBoxLayout *horizontalLayout_4;
    KMyMoneyDateEdit *dateEdit;
    QSpacerItem *hs2;
    QLabel *assetAccountLabel;
    QLabel *securityLabel;
    QLabel *feesAmountLabel;
    QLabel *statusLabel;
    KMyMoneyAccountCombo *feesCombo;
    QComboBox *statusCombo;
    QLabel *priceLabel;
    QLabel *memoLabel;
    AmountEdit *interestAmountEdit;
    QLabel *sharesLabel;
    QComboBox *activityCombo;
    KMyMoneyAccountCombo *assetAccountCombo;
    QLabel *feesLabel;
    QLabel *totalLabel;
    QPlainTextEdit *memoEdit;
    QLabel *interestLabel;
    QLabel *dateLabel;
    QComboBox *securityAccountCombo;
    KMyMoneyAccountCombo *interestCombo;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QToolButton *enterButton;
    QToolButton *cancelButton;
    QLabel *activityLabel;
    AmountEdit *feesAmountEdit;
    AmountEdit *sharesAmountEdit;
    QSpacerItem *verticalSpacer;
    QSpacerItem *verticalSpacer_2;
    KMessageWidget *infoMessage;

    void setupUi(QWidget *InvestTransactionEditor)
    {
        if (InvestTransactionEditor->objectName().isEmpty())
            InvestTransactionEditor->setObjectName("InvestTransactionEditor");
        InvestTransactionEditor->resize(660, 362);
        gridLayout = new QGridLayout(InvestTransactionEditor);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setVerticalSpacing(2);
        priceAmountEdit = new AmountEdit(InvestTransactionEditor);
        priceAmountEdit->setObjectName("priceAmountEdit");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(priceAmountEdit->sizePolicy().hasHeightForWidth());
        priceAmountEdit->setSizePolicy(sizePolicy);
        priceAmountEdit->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(priceAmountEdit, 4, 3, 1, 1);

        totalAmountEdit = new AmountEdit(InvestTransactionEditor);
        totalAmountEdit->setObjectName("totalAmountEdit");
        sizePolicy.setHeightForWidth(totalAmountEdit->sizePolicy().hasHeightForWidth());
        totalAmountEdit->setSizePolicy(sizePolicy);
        totalAmountEdit->setReadOnly(true);
        totalAmountEdit->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(totalAmountEdit, 9, 3, 1, 1);

        interestAmountLabel = new QLabel(InvestTransactionEditor);
        interestAmountLabel->setObjectName("interestAmountLabel");

        gridLayout->addWidget(interestAmountLabel, 6, 2, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        dateEdit = new KMyMoneyDateEdit(InvestTransactionEditor);
        dateEdit->setObjectName("dateEdit");
        dateEdit->setProperty("kmm_taborder", QVariant(true));

        horizontalLayout_4->addWidget(dateEdit);

        hs2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(hs2);


        gridLayout->addLayout(horizontalLayout_4, 2, 3, 1, 1);

        assetAccountLabel = new QLabel(InvestTransactionEditor);
        assetAccountLabel->setObjectName("assetAccountLabel");

        gridLayout->addWidget(assetAccountLabel, 4, 0, 1, 1);

        securityLabel = new QLabel(InvestTransactionEditor);
        securityLabel->setObjectName("securityLabel");

        gridLayout->addWidget(securityLabel, 3, 0, 1, 1);

        feesAmountLabel = new QLabel(InvestTransactionEditor);
        feesAmountLabel->setObjectName("feesAmountLabel");

        gridLayout->addWidget(feesAmountLabel, 5, 2, 1, 1);

        statusLabel = new QLabel(InvestTransactionEditor);
        statusLabel->setObjectName("statusLabel");

        gridLayout->addWidget(statusLabel, 10, 2, 1, 1);

        feesCombo = new KMyMoneyAccountCombo(InvestTransactionEditor);
        feesCombo->setObjectName("feesCombo");
        feesCombo->setEditable(true);
        feesCombo->setIconSize(QSize(0, 0));
        feesCombo->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(feesCombo, 5, 1, 1, 1);

        statusCombo = new QComboBox(InvestTransactionEditor);
        statusCombo->addItem(QString());
        statusCombo->setObjectName("statusCombo");
        statusCombo->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(statusCombo, 10, 3, 1, 1);

        priceLabel = new QLabel(InvestTransactionEditor);
        priceLabel->setObjectName("priceLabel");
        priceLabel->setText(QString::fromUtf8("Price"));

        gridLayout->addWidget(priceLabel, 4, 2, 1, 1);

        memoLabel = new QLabel(InvestTransactionEditor);
        memoLabel->setObjectName("memoLabel");
        memoLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout->addWidget(memoLabel, 8, 0, 1, 1);

        interestAmountEdit = new AmountEdit(InvestTransactionEditor);
        interestAmountEdit->setObjectName("interestAmountEdit");
        sizePolicy.setHeightForWidth(interestAmountEdit->sizePolicy().hasHeightForWidth());
        interestAmountEdit->setSizePolicy(sizePolicy);
        interestAmountEdit->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(interestAmountEdit, 6, 3, 1, 1);

        sharesLabel = new QLabel(InvestTransactionEditor);
        sharesLabel->setObjectName("sharesLabel");
        sharesLabel->setText(QString::fromUtf8("Shares"));

        gridLayout->addWidget(sharesLabel, 3, 2, 1, 1);

        activityCombo = new QComboBox(InvestTransactionEditor);
        activityCombo->setObjectName("activityCombo");
        activityCombo->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(activityCombo, 2, 1, 1, 1);

        assetAccountCombo = new KMyMoneyAccountCombo(InvestTransactionEditor);
        assetAccountCombo->setObjectName("assetAccountCombo");
        assetAccountCombo->setEditable(true);
        assetAccountCombo->setIconSize(QSize(0, 0));
        assetAccountCombo->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(assetAccountCombo, 4, 1, 1, 1);

        feesLabel = new QLabel(InvestTransactionEditor);
        feesLabel->setObjectName("feesLabel");

        gridLayout->addWidget(feesLabel, 5, 0, 1, 1);

        totalLabel = new QLabel(InvestTransactionEditor);
        totalLabel->setObjectName("totalLabel");

        gridLayout->addWidget(totalLabel, 9, 2, 1, 1);

        memoEdit = new QPlainTextEdit(InvestTransactionEditor);
        memoEdit->setObjectName("memoEdit");
        memoEdit->setTabChangesFocus(true);
        memoEdit->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(memoEdit, 8, 1, 4, 1);

        interestLabel = new QLabel(InvestTransactionEditor);
        interestLabel->setObjectName("interestLabel");

        gridLayout->addWidget(interestLabel, 6, 0, 1, 1);

        dateLabel = new QLabel(InvestTransactionEditor);
        dateLabel->setObjectName("dateLabel");

        gridLayout->addWidget(dateLabel, 2, 2, 1, 1);

        securityAccountCombo = new QComboBox(InvestTransactionEditor);
        securityAccountCombo->setObjectName("securityAccountCombo");
        securityAccountCombo->setEditable(true);
        securityAccountCombo->setIconSize(QSize(0, 0));
        securityAccountCombo->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(securityAccountCombo, 3, 1, 1, 1);

        interestCombo = new KMyMoneyAccountCombo(InvestTransactionEditor);
        interestCombo->setObjectName("interestCombo");
        interestCombo->setEditable(true);
        interestCombo->setIconSize(QSize(0, 0));
        interestCombo->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(interestCombo, 6, 1, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        enterButton = new QToolButton(InvestTransactionEditor);
        enterButton->setObjectName("enterButton");
        enterButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        enterButton->setProperty("kmm_taborder", QVariant(true));

        horizontalLayout->addWidget(enterButton);

        cancelButton = new QToolButton(InvestTransactionEditor);
        cancelButton->setObjectName("cancelButton");
        cancelButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        cancelButton->setProperty("kmm_taborder", QVariant(true));

        horizontalLayout->addWidget(cancelButton);


        gridLayout->addLayout(horizontalLayout, 11, 2, 1, 2);

        activityLabel = new QLabel(InvestTransactionEditor);
        activityLabel->setObjectName("activityLabel");

        gridLayout->addWidget(activityLabel, 2, 0, 1, 1);

        feesAmountEdit = new AmountEdit(InvestTransactionEditor);
        feesAmountEdit->setObjectName("feesAmountEdit");
        sizePolicy.setHeightForWidth(feesAmountEdit->sizePolicy().hasHeightForWidth());
        feesAmountEdit->setSizePolicy(sizePolicy);
        feesAmountEdit->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(feesAmountEdit, 5, 3, 1, 1);

        sharesAmountEdit = new AmountEdit(InvestTransactionEditor);
        sharesAmountEdit->setObjectName("sharesAmountEdit");
        sizePolicy.setHeightForWidth(sharesAmountEdit->sizePolicy().hasHeightForWidth());
        sharesAmountEdit->setSizePolicy(sizePolicy);
        sharesAmountEdit->setProperty("kmm_taborder", QVariant(true));

        gridLayout->addWidget(sharesAmountEdit, 3, 3, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 7, 0, 1, 2);

        verticalSpacer_2 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer_2, 8, 2, 1, 2);

        infoMessage = new KMessageWidget(InvestTransactionEditor);
        infoMessage->setObjectName("infoMessage");

        gridLayout->addWidget(infoMessage, 1, 0, 1, 4);

        gridLayout->setColumnStretch(1, 5);
        QWidget::setTabOrder(activityCombo, dateEdit);
        QWidget::setTabOrder(dateEdit, securityAccountCombo);
        QWidget::setTabOrder(securityAccountCombo, sharesAmountEdit);
        QWidget::setTabOrder(sharesAmountEdit, assetAccountCombo);
        QWidget::setTabOrder(assetAccountCombo, priceAmountEdit);
        QWidget::setTabOrder(priceAmountEdit, feesCombo);
        QWidget::setTabOrder(feesCombo, feesAmountEdit);
        QWidget::setTabOrder(feesAmountEdit, interestCombo);
        QWidget::setTabOrder(interestCombo, interestAmountEdit);
        QWidget::setTabOrder(interestAmountEdit, memoEdit);
        QWidget::setTabOrder(memoEdit, totalAmountEdit);
        QWidget::setTabOrder(totalAmountEdit, statusCombo);
        QWidget::setTabOrder(statusCombo, enterButton);
        QWidget::setTabOrder(enterButton, cancelButton);

        retranslateUi(InvestTransactionEditor);

        QMetaObject::connectSlotsByName(InvestTransactionEditor);
    } // setupUi

    void retranslateUi(QWidget *InvestTransactionEditor)
    {
#if QT_CONFIG(tooltip)
        priceAmountEdit->setToolTip(tr2i18n("Price information for this transaction", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
        priceAmountEdit->setText(QString());
        priceAmountEdit->setPlaceholderText(tr2i18n("Price/share", nullptr));
        totalAmountEdit->setText(QString());
        totalAmountEdit->setPlaceholderText(tr2i18n("Total", nullptr));
        interestAmountLabel->setText(tr2i18n("Interest", nullptr));
#if QT_CONFIG(tooltip)
        dateEdit->setToolTip(tr2i18n("The posting date of the transaction.", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
        assetAccountLabel->setText(tr2i18n("Account", nullptr));
        securityLabel->setText(tr2i18n("Security", nullptr));
        feesAmountLabel->setText(tr2i18n("Fees", nullptr));
        statusLabel->setText(tr2i18n("Status", nullptr));
#if QT_CONFIG(tooltip)
        feesCombo->setToolTip(tr2i18n("Category for fees", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
        statusCombo->setItemText(0, tr2i18n("Cleared", nullptr));

        memoLabel->setText(tr2i18n("Memo", nullptr));
#if QT_CONFIG(tooltip)
        interestAmountEdit->setToolTip(tr2i18n("Amount of interest", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
        interestAmountEdit->setText(QString());
        interestAmountEdit->setPlaceholderText(tr2i18n("Interest", nullptr));
#if QT_CONFIG(tooltip)
        assetAccountCombo->setToolTip(tr2i18n("Asset or brokerage account", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
        feesLabel->setText(tr2i18n("Fees", nullptr));
        totalLabel->setText(tr2i18n("Total", nullptr));
        interestLabel->setText(tr2i18n("Interest", nullptr));
        dateLabel->setText(tr2i18n("Date", nullptr));
#if QT_CONFIG(tooltip)
        securityAccountCombo->setToolTip(tr2i18n("Security for this transaction", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        interestCombo->setToolTip(tr2i18n("Category for interest", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
        enterButton->setText(tr2i18n("Enter", nullptr));
        cancelButton->setText(tr2i18n("Cancel", nullptr));
        activityLabel->setText(tr2i18n("Activity", nullptr));
#if QT_CONFIG(tooltip)
        feesAmountEdit->setToolTip(tr2i18n("Amount of fees", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
        feesAmountEdit->setText(QString());
        feesAmountEdit->setPlaceholderText(tr2i18n("Fees", nullptr));
        sharesAmountEdit->setText(QString());
        sharesAmountEdit->setPlaceholderText(tr2i18n("Shares", nullptr));
        (void)InvestTransactionEditor;
    } // retranslateUi

};

namespace Ui {
    class InvestTransactionEditor: public Ui_InvestTransactionEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // INVESTTRANSACTIONEDITOR_H

