/********************************************************************************
** Form generated from reading UI file 'kmymoneybriefschedule.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KMYMONEYBRIEFSCHEDULE_H
#define UI_KMYMONEYBRIEFSCHEDULE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "klineedit.h"
#include "ktextedit.h"

QT_BEGIN_NAMESPACE

class Ui_KMyMoneyBriefSchedule
{
public:
    QVBoxLayout *vboxLayout;
    QFrame *frame3;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout;
    QToolButton *m_prevButton;
    QSpacerItem *spacer1;
    QLabel *m_indexLabel;
    QSpacerItem *spacer2;
    QToolButton *m_nextButton;
    QGridLayout *gridLayout;
    QLabel *textLabel1;
    QLabel *textLabel2;
    KLineEdit *m_name;
    KLineEdit *m_account;
    KLineEdit *m_type;
    QLabel *textLabel1_2;
    KTextEdit *m_details;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *Spacer3;
    QHBoxLayout *hboxLayout2;
    QPushButton *m_buttonEnter;
    QPushButton *m_skipButton;
    QPushButton *m_closeButton;
    QSpacerItem *Spacer4;

    void setupUi(QWidget *KMyMoneyBriefSchedule)
    {
        if (KMyMoneyBriefSchedule->objectName().isEmpty())
            KMyMoneyBriefSchedule->setObjectName("KMyMoneyBriefSchedule");
        KMyMoneyBriefSchedule->resize(341, 339);
        vboxLayout = new QVBoxLayout(KMyMoneyBriefSchedule);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(1, 1, 1, 1);
        frame3 = new QFrame(KMyMoneyBriefSchedule);
        frame3->setObjectName("frame3");
        frame3->setFrameShape(QFrame::WinPanel);
        frame3->setFrameShadow(QFrame::Raised);
        frame3->setLineWidth(2);
        frame3->setMidLineWidth(100);
        vboxLayout1 = new QVBoxLayout(frame3);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        m_prevButton = new QToolButton(frame3);
        m_prevButton->setObjectName("m_prevButton");

        hboxLayout->addWidget(m_prevButton);

        spacer1 = new QSpacerItem(40, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer1);

        m_indexLabel = new QLabel(frame3);
        m_indexLabel->setObjectName("m_indexLabel");
        m_indexLabel->setWordWrap(false);

        hboxLayout->addWidget(m_indexLabel);

        spacer2 = new QSpacerItem(70, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer2);

        m_nextButton = new QToolButton(frame3);
        m_nextButton->setObjectName("m_nextButton");

        hboxLayout->addWidget(m_nextButton);


        vboxLayout1->addLayout(hboxLayout);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        textLabel1 = new QLabel(frame3);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setMinimumSize(QSize(60, 0));
        textLabel1->setMaximumSize(QSize(60, 32767));
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 1, 0, 1, 1);

        textLabel2 = new QLabel(frame3);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setMinimumSize(QSize(60, 0));
        textLabel2->setMaximumSize(QSize(60, 32767));
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 2, 0, 1, 1);

        m_name = new KLineEdit(frame3);
        m_name->setObjectName("m_name");
        m_name->setEnabled(true);
        m_name->setReadOnly(true);

        gridLayout->addWidget(m_name, 1, 1, 1, 1);

        m_account = new KLineEdit(frame3);
        m_account->setObjectName("m_account");
        m_account->setEnabled(true);
        m_account->setReadOnly(true);

        gridLayout->addWidget(m_account, 0, 1, 1, 1);

        m_type = new KLineEdit(frame3);
        m_type->setObjectName("m_type");
        m_type->setEnabled(true);
        m_type->setReadOnly(true);

        gridLayout->addWidget(m_type, 2, 1, 1, 1);

        textLabel1_2 = new QLabel(frame3);
        textLabel1_2->setObjectName("textLabel1_2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textLabel1_2->sizePolicy().hasHeightForWidth());
        textLabel1_2->setSizePolicy(sizePolicy);
        textLabel1_2->setMinimumSize(QSize(60, 0));
        textLabel1_2->setWordWrap(false);

        gridLayout->addWidget(textLabel1_2, 0, 0, 1, 1);


        vboxLayout1->addLayout(gridLayout);

        m_details = new KTextEdit(frame3);
        m_details->setObjectName("m_details");
        m_details->setEnabled(true);
        m_details->setReadOnly(true);

        vboxLayout1->addWidget(m_details);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        Spacer3 = new QSpacerItem(35, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(Spacer3);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        m_buttonEnter = new QPushButton(frame3);
        m_buttonEnter->setObjectName("m_buttonEnter");

        hboxLayout2->addWidget(m_buttonEnter);

        m_skipButton = new QPushButton(frame3);
        m_skipButton->setObjectName("m_skipButton");

        hboxLayout2->addWidget(m_skipButton);

        m_closeButton = new QPushButton(frame3);
        m_closeButton->setObjectName("m_closeButton");
        m_closeButton->setProperty("stdItem", QVariant(13));

        hboxLayout2->addWidget(m_closeButton);


        hboxLayout1->addLayout(hboxLayout2);

        Spacer4 = new QSpacerItem(43, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(Spacer4);


        vboxLayout1->addLayout(hboxLayout1);


        vboxLayout->addWidget(frame3);


        retranslateUi(KMyMoneyBriefSchedule);

        QMetaObject::connectSlotsByName(KMyMoneyBriefSchedule);
    } // setupUi

    void retranslateUi(QWidget *KMyMoneyBriefSchedule)
    {
        KMyMoneyBriefSchedule->setWindowTitle(tr2i18n("Schedules", nullptr));
        m_prevButton->setText(tr2i18n("...", nullptr));
        m_indexLabel->setText(tr2i18n("n of n", nullptr));
        m_nextButton->setText(tr2i18n("...", nullptr));
        textLabel1->setText(tr2i18n("Name:", "@label name of the schedule"));
        textLabel2->setText(tr2i18n("Type:", nullptr));
        textLabel1_2->setText(tr2i18n("Account:", nullptr));
        m_details->setProperty("text", QVariant(tr2i18n("1", nullptr)));
#if QT_CONFIG(tooltip)
        m_buttonEnter->setToolTip(tr2i18n("Record this transaction into the register", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_buttonEnter->setWhatsThis(tr2i18n("Use this button to record this transaction", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_buttonEnter->setText(tr2i18n("Enter...", nullptr));
#if QT_CONFIG(tooltip)
        m_skipButton->setToolTip(tr2i18n("Skip this transaction", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_skipButton->setWhatsThis(tr2i18n("Use this button to skip this transaction", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_skipButton->setText(tr2i18n("&Skip", nullptr));
        m_closeButton->setText(tr2i18n("&Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KMyMoneyBriefSchedule: public Ui_KMyMoneyBriefSchedule {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KMYMONEYBRIEFSCHEDULE_H

