/********************************************************************************
** Form generated from reading UI file 'sepacredittransferedit.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SEPACREDITTRANSFEREDIT_H
#define UI_SEPACREDITTRANSFEREDIT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <amountedit.h>
#include <klocalizedstring.h>
#include "klineedit.h"
#include "kmymoneytextedit.h"
#include "kmymoneyvalidationfeedback.h"
#include "widgets/payeeidentifier/ibanbic/kbicedit.h"
#include "widgets/payeeidentifier/ibanbic/kibanlineedit.h"

QT_BEGIN_NAMESPACE

class Ui_sepaCreditTransferEdit
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label_2;
    KLineEdit *beneficiaryName;
    KMyMoneyValidationFeedback *feedbackName;
    QLabel *label_iban;
    KIbanLineEdit *beneficiaryIban;
    KMyMoneyValidationFeedback *feedbackIban;
    QLabel *label_bic;
    KBicEdit *beneficiaryBankCode;
    KMyMoneyValidationFeedback *feedbackBic;
    QHBoxLayout *horizontalLayout;
    QLabel *label_7;
    QSpacerItem *horizontalSpacer_2;
    AmountEdit *value;
    KMyMoneyValidationFeedback *feedbackAmount;
    QLabel *label;
    KLineEdit *sepaReference;
    KMyMoneyValidationFeedback *feedbackReference;
    QLabel *label_4;
    KMyMoneyTextEdit *purpose;
    KMyMoneyValidationFeedback *feedbackPurpose;

    void setupUi(QWidget *sepaCreditTransferEdit)
    {
        if (sepaCreditTransferEdit->objectName().isEmpty())
            sepaCreditTransferEdit->setObjectName("sepaCreditTransferEdit");
        sepaCreditTransferEdit->resize(340, 438);
        verticalLayout = new QVBoxLayout(sepaCreditTransferEdit);
        verticalLayout->setObjectName("verticalLayout");
        label_2 = new QLabel(sepaCreditTransferEdit);
        label_2->setObjectName("label_2");

        verticalLayout->addWidget(label_2);

        beneficiaryName = new KLineEdit(sepaCreditTransferEdit);
        beneficiaryName->setObjectName("beneficiaryName");

        verticalLayout->addWidget(beneficiaryName);

        feedbackName = new KMyMoneyValidationFeedback(sepaCreditTransferEdit);
        feedbackName->setObjectName("feedbackName");

        verticalLayout->addWidget(feedbackName);

        label_iban = new QLabel(sepaCreditTransferEdit);
        label_iban->setObjectName("label_iban");

        verticalLayout->addWidget(label_iban);

        beneficiaryIban = new KIbanLineEdit(sepaCreditTransferEdit);
        beneficiaryIban->setObjectName("beneficiaryIban");

        verticalLayout->addWidget(beneficiaryIban);

        feedbackIban = new KMyMoneyValidationFeedback(sepaCreditTransferEdit);
        feedbackIban->setObjectName("feedbackIban");

        verticalLayout->addWidget(feedbackIban);

        label_bic = new QLabel(sepaCreditTransferEdit);
        label_bic->setObjectName("label_bic");

        verticalLayout->addWidget(label_bic);

        beneficiaryBankCode = new KBicEdit(sepaCreditTransferEdit);
        beneficiaryBankCode->setObjectName("beneficiaryBankCode");

        verticalLayout->addWidget(beneficiaryBankCode);

        feedbackBic = new KMyMoneyValidationFeedback(sepaCreditTransferEdit);
        feedbackBic->setObjectName("feedbackBic");

        verticalLayout->addWidget(feedbackBic);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label_7 = new QLabel(sepaCreditTransferEdit);
        label_7->setObjectName("label_7");

        horizontalLayout->addWidget(label_7);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        value = new AmountEdit(sepaCreditTransferEdit);
        value->setObjectName("value");
        value->setMinimumSize(QSize(100, 0));
        value->setProperty("resetButtonVisibility", QVariant(false));

        horizontalLayout->addWidget(value);


        verticalLayout->addLayout(horizontalLayout);

        feedbackAmount = new KMyMoneyValidationFeedback(sepaCreditTransferEdit);
        feedbackAmount->setObjectName("feedbackAmount");

        verticalLayout->addWidget(feedbackAmount);

        label = new QLabel(sepaCreditTransferEdit);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        sepaReference = new KLineEdit(sepaCreditTransferEdit);
        sepaReference->setObjectName("sepaReference");

        verticalLayout->addWidget(sepaReference);

        feedbackReference = new KMyMoneyValidationFeedback(sepaCreditTransferEdit);
        feedbackReference->setObjectName("feedbackReference");

        verticalLayout->addWidget(feedbackReference);

        label_4 = new QLabel(sepaCreditTransferEdit);
        label_4->setObjectName("label_4");

        verticalLayout->addWidget(label_4);

        purpose = new KMyMoneyTextEdit(sepaCreditTransferEdit);
        purpose->setObjectName("purpose");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(purpose->sizePolicy().hasHeightForWidth());
        purpose->setSizePolicy(sizePolicy);
        purpose->setMinimumSize(QSize(70, 30));
        purpose->setTabChangesFocus(true);
        purpose->setLineWrapMode(QTextEdit::FixedColumnWidth);
        purpose->setLineWrapColumnOrWidth(27);
        purpose->setAcceptRichText(false);

        verticalLayout->addWidget(purpose);

        feedbackPurpose = new KMyMoneyValidationFeedback(sepaCreditTransferEdit);
        feedbackPurpose->setObjectName("feedbackPurpose");

        verticalLayout->addWidget(feedbackPurpose);

        label->raise();
        purpose->raise();
        label_4->raise();
        label_2->raise();
        beneficiaryName->raise();
        label_bic->raise();
        label_iban->raise();
        beneficiaryBankCode->raise();
        feedbackName->raise();
        feedbackBic->raise();
        feedbackAmount->raise();
        feedbackReference->raise();
        feedbackPurpose->raise();
        sepaReference->raise();
        feedbackIban->raise();
        beneficiaryIban->raise();
#if QT_CONFIG(shortcut)
        label_2->setBuddy(beneficiaryName);
        label->setBuddy(sepaReference);
        label_4->setBuddy(purpose);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(beneficiaryName, beneficiaryIban);
        QWidget::setTabOrder(beneficiaryIban, beneficiaryBankCode);
        QWidget::setTabOrder(beneficiaryBankCode, sepaReference);
        QWidget::setTabOrder(sepaReference, purpose);

        retranslateUi(sepaCreditTransferEdit);

        QMetaObject::connectSlotsByName(sepaCreditTransferEdit);
    } // setupUi

    void retranslateUi(QWidget *sepaCreditTransferEdit)
    {
        label_2->setText(tr2i18n("Beneficiary &Name", nullptr));
        label_iban->setText(tr2i18n("IBAN", nullptr));
        label_bic->setText(tr2i18n("BIC", nullptr));
        label_7->setText(tr2i18n("Amount", nullptr));
        label->setText(tr2i18n("End-to-end reference", nullptr));
        label_4->setText(tr2i18n("Purpose", nullptr));
        (void)sepaCreditTransferEdit;
    } // retranslateUi

};

namespace Ui {
    class sepaCreditTransferEdit: public Ui_sepaCreditTransferEdit {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SEPACREDITTRANSFEREDIT_H

