/********************************************************************************
** Form generated from reading UI file 'kofxdirectconnectdlgdecl.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KOFXDIRECTCONNECTDLGDECL_H
#define UI_KOFXDIRECTCONNECTDLGDECL_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KOfxDirectConnectDlgDecl
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *textLabel1;
    QProgressBar *kProgress1;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KOfxDirectConnectDlgDecl)
    {
        if (KOfxDirectConnectDlgDecl->objectName().isEmpty())
            KOfxDirectConnectDlgDecl->setObjectName("KOfxDirectConnectDlgDecl");
        KOfxDirectConnectDlgDecl->resize(511, 108);
        KOfxDirectConnectDlgDecl->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(KOfxDirectConnectDlgDecl);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        textLabel1 = new QLabel(KOfxDirectConnectDlgDecl);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        vboxLayout->addWidget(textLabel1);

        kProgress1 = new QProgressBar(KOfxDirectConnectDlgDecl);
        kProgress1->setObjectName("kProgress1");

        vboxLayout->addWidget(kProgress1);

        buttonBox = new QDialogButtonBox(KOfxDirectConnectDlgDecl);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel);
        buttonBox->setCenterButtons(true);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(KOfxDirectConnectDlgDecl);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KOfxDirectConnectDlgDecl, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KOfxDirectConnectDlgDecl);
    } // setupUi

    void retranslateUi(QDialog *KOfxDirectConnectDlgDecl)
    {
        KOfxDirectConnectDlgDecl->setWindowTitle(tr2i18n("OFX Direct Connect", nullptr));
        textLabel1->setText(tr2i18n("Contacting bank...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KOfxDirectConnectDlgDecl: public Ui_KOfxDirectConnectDlgDecl {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KOFXDIRECTCONNECTDLGDECL_H

