/********************************************************************************
** Form generated from reading UI file 'transactiondlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TRANSACTIONDLG_H
#define UI_TRANSACTIONDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_TransactionDlg
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label_title;
    QLabel *label_info;
    QTableWidget *tableWidget;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_left;
    QLabel *label_2;
    QComboBox *cbActionTypes;
    QSpacerItem *horizontalSpacer_right;
    QLabel *label;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *TransactionDlg)
    {
        if (TransactionDlg->objectName().isEmpty())
            TransactionDlg->setObjectName("TransactionDlg");
        TransactionDlg->resize(400, 280);
        verticalLayout = new QVBoxLayout(TransactionDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        label_title = new QLabel(TransactionDlg);
        label_title->setObjectName("label_title");
        QFont font;
        font.setBold(true);
        label_title->setFont(font);
        label_title->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(label_title);

        label_info = new QLabel(TransactionDlg);
        label_info->setObjectName("label_info");
        label_info->setAlignment(Qt::AlignCenter);
        label_info->setWordWrap(true);

        verticalLayout->addWidget(label_info);

        tableWidget = new QTableWidget(TransactionDlg);
        if (tableWidget->rowCount() < 1)
            tableWidget->setRowCount(1);
        tableWidget->setObjectName("tableWidget");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tableWidget->sizePolicy().hasHeightForWidth());
        tableWidget->setSizePolicy(sizePolicy);
        tableWidget->setRowCount(1);

        verticalLayout->addWidget(tableWidget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer_left = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_left);

        label_2 = new QLabel(TransactionDlg);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        cbActionTypes = new QComboBox(TransactionDlg);
        cbActionTypes->addItem(QString());
        cbActionTypes->addItem(QString());
        cbActionTypes->addItem(QString());
        cbActionTypes->addItem(QString());
        cbActionTypes->addItem(QString());
        cbActionTypes->addItem(QString());
        cbActionTypes->addItem(QString());
        cbActionTypes->setObjectName("cbActionTypes");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(cbActionTypes->sizePolicy().hasHeightForWidth());
        cbActionTypes->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(cbActionTypes);

        horizontalSpacer_right = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_right);


        verticalLayout->addLayout(horizontalLayout);

        label = new QLabel(TransactionDlg);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        buttonBox = new QDialogButtonBox(TransactionDlg);
        buttonBox->setObjectName("buttonBox");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(buttonBox->sizePolicy().hasHeightForWidth());
        buttonBox->setSizePolicy(sizePolicy2);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
        buttonBox->setCenterButtons(true);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(TransactionDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, TransactionDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, TransactionDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(TransactionDlg);
    } // setupUi

    void retranslateUi(QDialog *TransactionDlg)
    {
        TransactionDlg->setWindowTitle(tr2i18n("Transaction Type", nullptr));
        label_title->setText(tr2i18n("Invalid Transaction Type", nullptr));
        label_info->setText(tr2i18n("<center>The transaction below has an either unrecognised or invalid type.</center>\n"
"<center>Please select an appropriate entry, if available.</center>\n"
"<center>Otherwise, click Cancel to abort.</center>", nullptr));
        label_2->setText(tr2i18n("Select Transaction Type", nullptr));
        cbActionTypes->setItemText(0, tr2i18n("Buy Shares", "description of investment activity"));
        cbActionTypes->setItemText(1, tr2i18n("Sell Shares", "description of investment activity"));
        cbActionTypes->setItemText(2, tr2i18n("Dividend", "description of investment activity"));
        cbActionTypes->setItemText(3, tr2i18n("Reinvest Dividend", "description of investment activity"));
        cbActionTypes->setItemText(4, tr2i18n("Add Shares", "description of investment activity"));
        cbActionTypes->setItemText(5, tr2i18n("Remove Shares", "description of investment activity"));
        cbActionTypes->setItemText(6, tr2i18n("Interest", nullptr));

#if QT_CONFIG(tooltip)
        cbActionTypes->setToolTip(tr2i18n("Select new investment type", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class TransactionDlg: public Ui_TransactionDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TRANSACTIONDLG_H

