/********************************************************************************
** Form generated from reading UI file 'konlinebankingsetupwizard.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KONLINEBANKINGSETUPWIZARD_H
#define UI_KONLINEBANKINGSETUPWIZARD_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <QtWidgets/QWizard>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "klineedit.h"
#include "kmessagewidget.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_KOnlineBankingSetupWizard
{
public:
    QWizardPage *FIPage;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout;
    QLabel *m_instructionLabel;
    QTabWidget *m_selectionTab;
    QWidget *autoTab;
    QVBoxLayout *vboxLayout1;
    QListWidget *m_listFi;
    QWidget *manualTab;
    QVBoxLayout *vboxLayout2;
    QGridLayout *gridLayout;
    QLabel *textLabel1_5;
    QLabel *textLabel4;
    KLineEdit *m_userAgent;
    KLineEdit *m_bankName;
    QLabel *label;
    KLineEdit *m_fid;
    KUrlRequester *m_url;
    QLabel *textLabel2_4;
    QSpacerItem *spacer3;
    QWizardPage *LoginPage;
    QGridLayout *gridLayout1;
    QLabel *qLabel1;
    QLabel *textLabel1_4;
    KLineEdit *m_editUsername;
    QLabel *textLabel2_3;
    KLineEdit *m_editPassword;
    QCheckBox *m_storePassword;
    QLabel *textLabel1_2_3;
    QHBoxLayout *horizontalLayout;
    KComboBox *m_applicationCombo;
    KLineEdit *m_applicationEdit;
    QSpacerItem *spacer4_2;
    QLabel *textLabel1_6;
    QHBoxLayout *horizontalLayout_2;
    KComboBox *m_headerVersionCombo;
    KLineEdit *m_headerVersionEdit;
    QSpacerItem *horizontalSpacer;
    QTextBrowser *m_textDetails;
    QLabel *m_clientUidLabel;
    KLineEdit *m_editClientUid;
    QLabel *textLabel3;
    KMessageWidget *m_problemMessages;
    QWizardPage *AccountPage;
    QHBoxLayout *hboxLayout1;
    QVBoxLayout *vboxLayout3;
    QLabel *textLabel2_2;
    QTreeWidget *m_listAccount;
    QWizardPage *WizardPage;
    QHBoxLayout *hboxLayout2;
    QVBoxLayout *vboxLayout4;
    QLabel *m_labelFinal;

    void setupUi(QWizard *KOnlineBankingSetupWizard)
    {
        if (KOnlineBankingSetupWizard->objectName().isEmpty())
            KOnlineBankingSetupWizard->setObjectName("KOnlineBankingSetupWizard");
        KOnlineBankingSetupWizard->resize(595, 560);
        KOnlineBankingSetupWizard->setWizardStyle(QWizard::ModernStyle);
        FIPage = new QWizardPage();
        FIPage->setObjectName("FIPage");
        hboxLayout = new QHBoxLayout(FIPage);
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout = new QVBoxLayout();
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        m_instructionLabel = new QLabel(FIPage);
        m_instructionLabel->setObjectName("m_instructionLabel");
        m_instructionLabel->setAlignment(Qt::AlignVCenter);
        m_instructionLabel->setWordWrap(true);

        vboxLayout->addWidget(m_instructionLabel);

        m_selectionTab = new QTabWidget(FIPage);
        m_selectionTab->setObjectName("m_selectionTab");
        autoTab = new QWidget();
        autoTab->setObjectName("autoTab");
        vboxLayout1 = new QVBoxLayout(autoTab);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        m_listFi = new QListWidget(autoTab);
        m_listFi->setObjectName("m_listFi");

        vboxLayout1->addWidget(m_listFi);

        m_selectionTab->addTab(autoTab, QString());
        manualTab = new QWidget();
        manualTab->setObjectName("manualTab");
        vboxLayout2 = new QVBoxLayout(manualTab);
        vboxLayout2->setSpacing(6);
        vboxLayout2->setContentsMargins(11, 11, 11, 11);
        vboxLayout2->setObjectName("vboxLayout2");
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        textLabel1_5 = new QLabel(manualTab);
        textLabel1_5->setObjectName("textLabel1_5");
        textLabel1_5->setWordWrap(false);

        gridLayout->addWidget(textLabel1_5, 1, 0, 1, 1);

        textLabel4 = new QLabel(manualTab);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 0, 0, 1, 1);

        m_userAgent = new KLineEdit(manualTab);
        m_userAgent->setObjectName("m_userAgent");

        gridLayout->addWidget(m_userAgent, 3, 1, 1, 1);

        m_bankName = new KLineEdit(manualTab);
        m_bankName->setObjectName("m_bankName");

        gridLayout->addWidget(m_bankName, 0, 1, 1, 1);

        label = new QLabel(manualTab);
        label->setObjectName("label");

        gridLayout->addWidget(label, 3, 0, 1, 1);

        m_fid = new KLineEdit(manualTab);
        m_fid->setObjectName("m_fid");

        gridLayout->addWidget(m_fid, 1, 1, 1, 1);

        m_url = new KUrlRequester(manualTab);
        m_url->setObjectName("m_url");

        gridLayout->addWidget(m_url, 2, 1, 1, 1);

        textLabel2_4 = new QLabel(manualTab);
        textLabel2_4->setObjectName("textLabel2_4");
        textLabel2_4->setWordWrap(false);

        gridLayout->addWidget(textLabel2_4, 2, 0, 1, 1);


        vboxLayout2->addLayout(gridLayout);

        spacer3 = new QSpacerItem(20, 90, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout2->addItem(spacer3);

        m_selectionTab->addTab(manualTab, QString());

        vboxLayout->addWidget(m_selectionTab);


        hboxLayout->addLayout(vboxLayout);

        KOnlineBankingSetupWizard->addPage(FIPage);
        LoginPage = new QWizardPage();
        LoginPage->setObjectName("LoginPage");
        gridLayout1 = new QGridLayout(LoginPage);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName("gridLayout1");
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        qLabel1 = new QLabel(LoginPage);
        qLabel1->setObjectName("qLabel1");
        qLabel1->setFocusPolicy(Qt::NoFocus);
        qLabel1->setWordWrap(true);

        gridLayout1->addWidget(qLabel1, 0, 0, 1, 2);

        textLabel1_4 = new QLabel(LoginPage);
        textLabel1_4->setObjectName("textLabel1_4");
        textLabel1_4->setWordWrap(false);

        gridLayout1->addWidget(textLabel1_4, 1, 0, 1, 1);

        m_editUsername = new KLineEdit(LoginPage);
        m_editUsername->setObjectName("m_editUsername");

        gridLayout1->addWidget(m_editUsername, 1, 1, 1, 1);

        textLabel2_3 = new QLabel(LoginPage);
        textLabel2_3->setObjectName("textLabel2_3");
        textLabel2_3->setWordWrap(false);

        gridLayout1->addWidget(textLabel2_3, 2, 0, 1, 1);

        m_editPassword = new KLineEdit(LoginPage);
        m_editPassword->setObjectName("m_editPassword");

        gridLayout1->addWidget(m_editPassword, 2, 1, 1, 1);

        m_storePassword = new QCheckBox(LoginPage);
        m_storePassword->setObjectName("m_storePassword");

        gridLayout1->addWidget(m_storePassword, 4, 1, 1, 1);

        textLabel1_2_3 = new QLabel(LoginPage);
        textLabel1_2_3->setObjectName("textLabel1_2_3");
        textLabel1_2_3->setWordWrap(false);

        gridLayout1->addWidget(textLabel1_2_3, 5, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        m_applicationCombo = new KComboBox(LoginPage);
        m_applicationCombo->setObjectName("m_applicationCombo");

        horizontalLayout->addWidget(m_applicationCombo);

        m_applicationEdit = new KLineEdit(LoginPage);
        m_applicationEdit->setObjectName("m_applicationEdit");

        horizontalLayout->addWidget(m_applicationEdit);

        spacer4_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(spacer4_2);


        gridLayout1->addLayout(horizontalLayout, 5, 1, 1, 1);

        textLabel1_6 = new QLabel(LoginPage);
        textLabel1_6->setObjectName("textLabel1_6");
        textLabel1_6->setWordWrap(false);

        gridLayout1->addWidget(textLabel1_6, 6, 0, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        m_headerVersionCombo = new KComboBox(LoginPage);
        m_headerVersionCombo->setObjectName("m_headerVersionCombo");

        horizontalLayout_2->addWidget(m_headerVersionCombo);

        m_headerVersionEdit = new KLineEdit(LoginPage);
        m_headerVersionEdit->setObjectName("m_headerVersionEdit");

        horizontalLayout_2->addWidget(m_headerVersionEdit);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        gridLayout1->addLayout(horizontalLayout_2, 6, 1, 1, 1);

        m_textDetails = new QTextBrowser(LoginPage);
        m_textDetails->setObjectName("m_textDetails");
        m_textDetails->setEnabled(true);
        m_textDetails->setFocusPolicy(Qt::NoFocus);

        gridLayout1->addWidget(m_textDetails, 10, 0, 1, 2);

        m_clientUidLabel = new QLabel(LoginPage);
        m_clientUidLabel->setObjectName("m_clientUidLabel");

        gridLayout1->addWidget(m_clientUidLabel, 3, 0, 1, 1);

        m_editClientUid = new KLineEdit(LoginPage);
        m_editClientUid->setObjectName("m_editClientUid");

        gridLayout1->addWidget(m_editClientUid, 3, 1, 1, 1);

        textLabel3 = new QLabel(LoginPage);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout1->addWidget(textLabel3, 7, 0, 1, 1);

        m_problemMessages = new KMessageWidget(LoginPage);
        m_problemMessages->setObjectName("m_problemMessages");

        gridLayout1->addWidget(m_problemMessages, 8, 0, 1, 2);

        KOnlineBankingSetupWizard->addPage(LoginPage);
        AccountPage = new QWizardPage();
        AccountPage->setObjectName("AccountPage");
        hboxLayout1 = new QHBoxLayout(AccountPage);
        hboxLayout1->setSpacing(6);
        hboxLayout1->setContentsMargins(11, 11, 11, 11);
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        vboxLayout3 = new QVBoxLayout();
        vboxLayout3->setSpacing(6);
        vboxLayout3->setObjectName("vboxLayout3");
        textLabel2_2 = new QLabel(AccountPage);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setAlignment(Qt::AlignVCenter);
        textLabel2_2->setWordWrap(true);

        vboxLayout3->addWidget(textLabel2_2);

        m_listAccount = new QTreeWidget(AccountPage);
        m_listAccount->setObjectName("m_listAccount");

        vboxLayout3->addWidget(m_listAccount);


        hboxLayout1->addLayout(vboxLayout3);

        KOnlineBankingSetupWizard->addPage(AccountPage);
        WizardPage = new QWizardPage();
        WizardPage->setObjectName("WizardPage");
        hboxLayout2 = new QHBoxLayout(WizardPage);
        hboxLayout2->setSpacing(6);
        hboxLayout2->setContentsMargins(11, 11, 11, 11);
        hboxLayout2->setObjectName("hboxLayout2");
        vboxLayout4 = new QVBoxLayout();
        vboxLayout4->setSpacing(6);
        vboxLayout4->setObjectName("vboxLayout4");
        m_labelFinal = new QLabel(WizardPage);
        m_labelFinal->setObjectName("m_labelFinal");
        m_labelFinal->setAlignment(Qt::AlignVCenter);
        m_labelFinal->setWordWrap(true);

        vboxLayout4->addWidget(m_labelFinal);


        hboxLayout2->addLayout(vboxLayout4);

        KOnlineBankingSetupWizard->addPage(WizardPage);
        QWidget::setTabOrder(m_selectionTab, m_listFi);
        QWidget::setTabOrder(m_listFi, m_bankName);
        QWidget::setTabOrder(m_bankName, m_fid);
        QWidget::setTabOrder(m_fid, m_url);
        QWidget::setTabOrder(m_url, m_userAgent);
        QWidget::setTabOrder(m_userAgent, m_editUsername);
        QWidget::setTabOrder(m_editUsername, m_editPassword);
        QWidget::setTabOrder(m_editPassword, m_editClientUid);
        QWidget::setTabOrder(m_editClientUid, m_storePassword);
        QWidget::setTabOrder(m_storePassword, m_applicationCombo);
        QWidget::setTabOrder(m_applicationCombo, m_applicationEdit);
        QWidget::setTabOrder(m_applicationEdit, m_headerVersionCombo);
        QWidget::setTabOrder(m_headerVersionCombo, m_headerVersionEdit);
        QWidget::setTabOrder(m_headerVersionEdit, m_listAccount);

        retranslateUi(KOnlineBankingSetupWizard);

        m_selectionTab->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(KOnlineBankingSetupWizard);
    } // setupUi

    void retranslateUi(QWizard *KOnlineBankingSetupWizard)
    {
        KOnlineBankingSetupWizard->setWindowTitle(tr2i18n("Online Banking Account Setup", nullptr));
        FIPage->setTitle(tr2i18n("Select Financial Institution", nullptr));
        m_instructionLabel->setText(tr2i18n("Please select your financial institution from the list below...", nullptr));
        m_selectionTab->setTabText(m_selectionTab->indexOf(autoTab), tr2i18n("Automatic", "@title type of online banking connection"));
        textLabel1_5->setText(tr2i18n("FID", nullptr));
        textLabel4->setText(tr2i18n("Org", nullptr));
        label->setText(tr2i18n("UserAgent", nullptr));
        textLabel2_4->setText(tr2i18n("URL", nullptr));
        m_selectionTab->setTabText(m_selectionTab->indexOf(manualTab), tr2i18n("Manual", "@title type of online banking connection"));
        LoginPage->setTitle(tr2i18n("Enter Login Details", nullptr));
        qLabel1->setText(tr2i18n("Please enter the username and password you use to log into this bank for online banking.  Please note that many banks require a separate signup, and assign a separate PIN or password just for online banking from home.", nullptr));
        textLabel1_4->setText(tr2i18n("Username", "@label account username"));
        textLabel2_3->setText(tr2i18n("Password", nullptr));
        m_storePassword->setText(tr2i18n("Store password", nullptr));
        textLabel1_2_3->setText(tr2i18n("Identify as", nullptr));
#if QT_CONFIG(whatsthis)
        m_applicationEdit->setWhatsThis(tr2i18n("Expert option to enter the four digit version information of the application", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1_6->setText(tr2i18n("Header Version", nullptr));
        m_clientUidLabel->setText(tr2i18n("Client UID", nullptr));
        textLabel3->setText(tr2i18n("Connection Details", nullptr));
        AccountPage->setTitle(tr2i18n("Select Account", nullptr));
        textLabel2_2->setText(tr2i18n("Please select the account from your financial institution from the list below which matches this account.", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = m_listAccount->headerItem();
        ___qtreewidgetitem->setText(3, tr2i18n("Branch", nullptr));
        ___qtreewidgetitem->setText(2, tr2i18n("Bank", nullptr));
        ___qtreewidgetitem->setText(1, tr2i18n("Type", nullptr));
        ___qtreewidgetitem->setText(0, tr2i18n("Number", nullptr));
        WizardPage->setTitle(tr2i18n("Finish", nullptr));
        m_labelFinal->setText(tr2i18n("Congratulations: you have successfully set up your bank for online banking via OFX.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KOnlineBankingSetupWizard: public Ui_KOnlineBankingSetupWizard {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KONLINEBANKINGSETUPWIZARD_H

