/********************************************************************************
** Form generated from reading UI file 'previouspostponewizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREVIOUSPOSTPONEWIZARDPAGE_H
#define UI_PREVIOUSPOSTPONEWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_PreviousPostponeWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel8;

    void setupUi(QWizardPage *PreviousPostponeWizardPage)
    {
        if (PreviousPostponeWizardPage->objectName().isEmpty())
            PreviousPostponeWizardPage->setObjectName("PreviousPostponeWizardPage");
        PreviousPostponeWizardPage->resize(577, 388);
        vboxLayout = new QVBoxLayout(PreviousPostponeWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel8 = new QLabel(PreviousPostponeWizardPage);
        TextLabel8->setObjectName("TextLabel8");
        TextLabel8->setAlignment(Qt::AlignTop);
        TextLabel8->setWordWrap(true);

        vboxLayout->addWidget(TextLabel8);


        retranslateUi(PreviousPostponeWizardPage);

        QMetaObject::connectSlotsByName(PreviousPostponeWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *PreviousPostponeWizardPage)
    {
        TextLabel8->setText(tr2i18n("You have previously postponed the reconciliation of this account. If you have entered charges or interests the last time you started reconciling this account you can modify these in the ledger later on.\n"
"\n"
"It is important, that you continue with the same statement you used when you postponed the reconciliation.\n"
"\n"
"All information you have entered into this wizard will be shown and all transactions that you already cleared are marked with a 'C'.", nullptr));
        (void)PreviousPostponeWizardPage;
    } // retranslateUi

};

namespace Ui {
    class PreviousPostponeWizardPage: public Ui_PreviousPostponeWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PREVIOUSPOSTPONEWIZARDPAGE_H

