/********************************************************************************
** Form generated from reading UI file 'namewizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NAMEWIZARDPAGE_H
#define UI_NAMEWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>
#include <kmymoneylineedit.h>
#include "kmymoneypayeecombo.h"

QT_BEGIN_NAMESPACE

class Ui_NameWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel4_2;
    QSpacerItem *Spacer15_2;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel5;
    KMyMoneyLineEdit *m_nameEdit;
    QSpacerItem *Spacer19;
    QLabel *m_generalReceiverText;
    QHBoxLayout *hboxLayout1;
    QLabel *m_receiverLabel;
    KMyMoneyPayeeCombo *m_payeeEdit;
    QSpacerItem *Spacer21;

    void setupUi(QWizardPage *NameWizardPage)
    {
        if (NameWizardPage->objectName().isEmpty())
            NameWizardPage->setObjectName("NameWizardPage");
        vboxLayout = new QVBoxLayout(NameWizardPage);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel4_2 = new QLabel(NameWizardPage);
        TextLabel4_2->setObjectName("TextLabel4_2");
        TextLabel4_2->setAlignment(Qt::AlignTop);
        TextLabel4_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel4_2);

        Spacer15_2 = new QSpacerItem(16, 56, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer15_2);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        TextLabel5 = new QLabel(NameWizardPage);
        TextLabel5->setObjectName("TextLabel5");
        TextLabel5->setWordWrap(false);

        hboxLayout->addWidget(TextLabel5);

        m_nameEdit = new KMyMoneyLineEdit(NameWizardPage);
        m_nameEdit->setObjectName("m_nameEdit");

        hboxLayout->addWidget(m_nameEdit);


        vboxLayout->addLayout(hboxLayout);

        Spacer19 = new QSpacerItem(16, 56, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer19);

        m_generalReceiverText = new QLabel(NameWizardPage);
        m_generalReceiverText->setObjectName("m_generalReceiverText");
        m_generalReceiverText->setText(QString::fromUtf8("TextLabel6"));
        m_generalReceiverText->setWordWrap(false);

        vboxLayout->addWidget(m_generalReceiverText);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        m_receiverLabel = new QLabel(NameWizardPage);
        m_receiverLabel->setObjectName("m_receiverLabel");
        m_receiverLabel->setText(QString::fromUtf8("TextLabel7"));
        m_receiverLabel->setWordWrap(false);

        hboxLayout1->addWidget(m_receiverLabel);

        m_payeeEdit = new KMyMoneyPayeeCombo(NameWizardPage);
        m_payeeEdit->setObjectName("m_payeeEdit");

        hboxLayout1->addWidget(m_payeeEdit);


        vboxLayout->addLayout(hboxLayout1);

        Spacer21 = new QSpacerItem(16, 56, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer21);


        retranslateUi(NameWizardPage);

        QMetaObject::connectSlotsByName(NameWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *NameWizardPage)
    {
        TextLabel4_2->setText(tr2i18n("\n"
"How do you want to call this loan? Examples for names are 'car loan', 'school loan', 'home owner loan'.", nullptr));
        TextLabel5->setText(tr2i18n("Name", "@label loan name"));
        (void)NameWizardPage;
    } // retranslateUi

};

namespace Ui {
    class NameWizardPage: public Ui_NameWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // NAMEWIZARDPAGE_H

