/****************************************************************************
** Meta object code from reading C++ file 'kcurrencyeditdlg.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/dialogs/kcurrencyeditdlg.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kcurrencyeditdlg.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16KCurrencyEditDlgE_t {};
} // unnamed namespace

template <> constexpr inline auto KCurrencyEditDlg::qt_create_metaobjectdata<qt_meta_tag_ZN16KCurrencyEditDlgE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KCurrencyEditDlg",
        "slotSelectCurrency",
        "",
        "id",
        "QTreeWidgetItem*",
        "citem",
        "pitem",
        "item",
        "slotItemSelectionChanged",
        "slotShowCurrencyMenu",
        "QPoint",
        "p",
        "slotLoadCurrencies",
        "slotUpdateCurrency",
        "column",
        "finishCtor",
        "slotSelectBaseCurrency",
        "slotAddCurrency",
        "slotRemoveCurrency",
        "slotRemoveUnusedCurrency",
        "slotEditCurrency",
        "slotNewCurrency",
        "slotRenameCurrency",
        "slotDeleteCurrency",
        "slotSetBaseCurrency"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotSelectCurrency'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'slotSelectCurrency'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, QTreeWidgetItem *)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 4, 6 },
        }}),
        // Slot 'slotSelectCurrency'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 7 },
        }}),
        // Slot 'slotItemSelectionChanged'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotShowCurrencyMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotLoadCurrencies'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotUpdateCurrency'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, int)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Int, 14 },
        }}),
        // Slot 'slotUpdateCurrency'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, QTreeWidgetItem *)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 4, 6 },
        }}),
        // Slot 'finishCtor'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSelectBaseCurrency'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddCurrency'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRemoveCurrency'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRemoveUnusedCurrency'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEditCurrency'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotNewCurrency'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRenameCurrency'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDeleteCurrency'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSetBaseCurrency'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KCurrencyEditDlg, qt_meta_tag_ZN16KCurrencyEditDlgE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KCurrencyEditDlg::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KCurrencyEditDlgE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KCurrencyEditDlgE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16KCurrencyEditDlgE_t>.metaTypes,
    nullptr
} };

void KCurrencyEditDlg::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KCurrencyEditDlg *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotSelectCurrency((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->slotSelectCurrency((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[2]))); break;
        case 2: _t->slotSelectCurrency((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        case 3: _t->slotItemSelectionChanged(); break;
        case 4: _t->slotShowCurrencyMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 5: _t->slotLoadCurrencies(); break;
        case 6: _t->slotUpdateCurrency((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 7: _t->slotUpdateCurrency((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[2]))); break;
        case 8: _t->finishCtor(); break;
        case 9: _t->slotSelectBaseCurrency(); break;
        case 10: _t->slotAddCurrency(); break;
        case 11: _t->slotRemoveCurrency(); break;
        case 12: _t->slotRemoveUnusedCurrency(); break;
        case 13: _t->slotEditCurrency(); break;
        case 14: _t->slotNewCurrency(); break;
        case 15: _t->slotRenameCurrency(); break;
        case 16: _t->slotDeleteCurrency(); break;
        case 17: _t->slotSetBaseCurrency(); break;
        default: ;
        }
    }
}

const QMetaObject *KCurrencyEditDlg::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KCurrencyEditDlg::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KCurrencyEditDlgE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int KCurrencyEditDlg::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}
QT_WARNING_POP
