/********************************************************************************
** Form generated from reading UI file 'kpayeesview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPAYEESVIEW_H
#define UI_KPAYEESVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "../widgets/kmymoneyaccountcombo.h"
#include "kcombobox.h"
#include "keditlistwidget.h"
#include "klineedit.h"
#include "kmmemptyview.h"
#include "kmymoneyaccountcombo.h"
#include "kpayeeidentifierview.h"
#include "ktextedit.h"
#include "ledgerview.h"

QT_BEGIN_NAMESPACE

class Ui_KPayeesView
{
public:
    QVBoxLayout *verticalLayout;
    QSplitter *m_splitter;
    QGroupBox *groupBox1;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QToolButton *m_newButton;
    QToolButton *m_renameButton;
    QToolButton *m_deleteButton;
    QToolButton *m_mergeButton;
    QHBoxLayout *m_listTopHLayout;
    QLineEdit *m_searchWidget;
    KComboBox *m_filterBox;
    KMMEmptyListView *m_payees;
    QTabWidget *m_tabWidget;
    QWidget *transactionTab;
    QVBoxLayout *vboxLayout1;
    LedgerView *m_register;
    QLabel *m_balanceLabel;
    QWidget *addressTab;
    QGridLayout *gridLayout;
    QLabel *TextLabel7;
    QLabel *TextLabel5_1;
    QLabel *TextLabel5_2;
    QLabel *TextLabel5;
    QLabel *TextLabel6;
    QLabel *TextLabel4_2;
    KLineEdit *emailEdit;
    QLabel *TextLabel4;
    QPushButton *m_sendMail;
    KLineEdit *telephoneEdit;
    KLineEdit *payeecityEdit;
    KLineEdit *payeestateEdit;
    KLineEdit *postcodeEdit;
    KTextEdit *addressEdit;
    KTextEdit *notesEdit;
    QSpacerItem *spacer6;
    QWidget *matchingTab;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *matchTypeCombo;
    QCheckBox *checkMatchIgnoreCase;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_4;
    KEditListWidget *matchKeyEditList;
    QWidget *defaultAccountTab;
    QVBoxLayout *vboxLayout2;
    QCheckBox *checkEnableDefaultCategory;
    QGridLayout *gridLayout1;
    QLabel *labelDefaultCategory;
    KMyMoneyAccountCombo *comboDefaultCategory;
    QPushButton *buttonSuggestACategory;
    QSpacerItem *spacer5;
    QWidget *tabPayeeIdentifiers;
    QVBoxLayout *verticalLayout_5;
    KPayeeIdentifierView *payeeIdentifiers;
    QWidget *tabLinks;
    QGridLayout *gridLayout_3;
    QVBoxLayout *verticalLayout_7;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout_5;
    QGridLayout *gridLayout_6;
    KLineEdit *idPatternEdit;
    QLabel *idPatternLabel;
    QLabel *urlTemplateLabel;
    KLineEdit *urlTemplateEdit;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout_7;
    QGridLayout *gridLayout_8;
    QLabel *matchingCheckLabel;
    QLabel *label_1;
    QLabel *label_2;
    KLineEdit *matchingCheckEdit;
    QLabel *label_3;
    QLabel *matchingCheckURL;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *m_helpButton;
    QSpacerItem *spacer14;
    QPushButton *m_syncAddressbook;
    QPushButton *m_updateButton;

    void setupUi(QWidget *KPayeesView)
    {
        if (KPayeesView->objectName().isEmpty())
            KPayeesView->setObjectName("KPayeesView");
        KPayeesView->resize(885, 567);
        verticalLayout = new QVBoxLayout(KPayeesView);
        verticalLayout->setSpacing(4);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        m_splitter = new QSplitter(KPayeesView);
        m_splitter->setObjectName("m_splitter");
        m_splitter->setOrientation(Qt::Horizontal);
        m_splitter->setChildrenCollapsible(false);
        groupBox1 = new QGroupBox(m_splitter);
        groupBox1->setObjectName("groupBox1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(groupBox1->sizePolicy().hasHeightForWidth());
        groupBox1->setSizePolicy(sizePolicy);
        vboxLayout = new QVBoxLayout(groupBox1);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_newButton = new QToolButton(groupBox1);
        m_newButton->setObjectName("m_newButton");
        m_newButton->setToolButtonStyle(Qt::ToolButtonIconOnly);

        hboxLayout->addWidget(m_newButton);

        m_renameButton = new QToolButton(groupBox1);
        m_renameButton->setObjectName("m_renameButton");
        m_renameButton->setToolButtonStyle(Qt::ToolButtonIconOnly);

        hboxLayout->addWidget(m_renameButton);

        m_deleteButton = new QToolButton(groupBox1);
        m_deleteButton->setObjectName("m_deleteButton");

        hboxLayout->addWidget(m_deleteButton);

        m_mergeButton = new QToolButton(groupBox1);
        m_mergeButton->setObjectName("m_mergeButton");

        hboxLayout->addWidget(m_mergeButton);


        vboxLayout->addLayout(hboxLayout);

        m_listTopHLayout = new QHBoxLayout();
        m_listTopHLayout->setSpacing(6);
        m_listTopHLayout->setObjectName("m_listTopHLayout");
        m_searchWidget = new QLineEdit(groupBox1);
        m_searchWidget->setObjectName("m_searchWidget");

        m_listTopHLayout->addWidget(m_searchWidget);

        m_filterBox = new KComboBox(groupBox1);
        m_filterBox->setObjectName("m_filterBox");

        m_listTopHLayout->addWidget(m_filterBox);


        vboxLayout->addLayout(m_listTopHLayout);

        m_payees = new KMMEmptyListView(groupBox1);
        m_payees->setObjectName("m_payees");
        m_payees->setAlternatingRowColors(true);

        vboxLayout->addWidget(m_payees);

        m_splitter->addWidget(groupBox1);
        m_tabWidget = new QTabWidget(m_splitter);
        m_tabWidget->setObjectName("m_tabWidget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(2);
        sizePolicy1.setVerticalStretch(1);
        sizePolicy1.setHeightForWidth(m_tabWidget->sizePolicy().hasHeightForWidth());
        m_tabWidget->setSizePolicy(sizePolicy1);
        transactionTab = new QWidget();
        transactionTab->setObjectName("transactionTab");
        vboxLayout1 = new QVBoxLayout(transactionTab);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        m_register = new LedgerView(transactionTab);
        m_register->setObjectName("m_register");

        vboxLayout1->addWidget(m_register);

        m_balanceLabel = new QLabel(transactionTab);
        m_balanceLabel->setObjectName("m_balanceLabel");
        m_balanceLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        m_balanceLabel->setWordWrap(false);

        vboxLayout1->addWidget(m_balanceLabel);

        m_tabWidget->addTab(transactionTab, QString());
        addressTab = new QWidget();
        addressTab->setObjectName("addressTab");
        gridLayout = new QGridLayout(addressTab);
        gridLayout->setSpacing(4);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(4, 4, 4, 4);
        TextLabel7 = new QLabel(addressTab);
        TextLabel7->setObjectName("TextLabel7");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(TextLabel7->sizePolicy().hasHeightForWidth());
        TextLabel7->setSizePolicy(sizePolicy2);
        TextLabel7->setMinimumSize(QSize(90, 0));
        TextLabel7->setWordWrap(false);

        gridLayout->addWidget(TextLabel7, 5, 0, 1, 1);

        TextLabel5_1 = new QLabel(addressTab);
        TextLabel5_1->setObjectName("TextLabel5_1");
        sizePolicy2.setHeightForWidth(TextLabel5_1->sizePolicy().hasHeightForWidth());
        TextLabel5_1->setSizePolicy(sizePolicy2);
        TextLabel5_1->setMinimumSize(QSize(90, 0));
        TextLabel5_1->setWordWrap(false);

        gridLayout->addWidget(TextLabel5_1, 1, 0, 1, 1);

        TextLabel5_2 = new QLabel(addressTab);
        TextLabel5_2->setObjectName("TextLabel5_2");
        sizePolicy2.setHeightForWidth(TextLabel5_2->sizePolicy().hasHeightForWidth());
        TextLabel5_2->setSizePolicy(sizePolicy2);
        TextLabel5_2->setMinimumSize(QSize(90, 0));
        TextLabel5_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel5_2, 2, 0, 1, 1);

        TextLabel5 = new QLabel(addressTab);
        TextLabel5->setObjectName("TextLabel5");
        sizePolicy2.setHeightForWidth(TextLabel5->sizePolicy().hasHeightForWidth());
        TextLabel5->setSizePolicy(sizePolicy2);
        TextLabel5->setMinimumSize(QSize(90, 0));
        TextLabel5->setWordWrap(false);

        gridLayout->addWidget(TextLabel5, 3, 0, 1, 1);

        TextLabel6 = new QLabel(addressTab);
        TextLabel6->setObjectName("TextLabel6");
        sizePolicy2.setHeightForWidth(TextLabel6->sizePolicy().hasHeightForWidth());
        TextLabel6->setSizePolicy(sizePolicy2);
        TextLabel6->setMinimumSize(QSize(90, 0));
        TextLabel6->setWordWrap(false);

        gridLayout->addWidget(TextLabel6, 4, 0, 1, 1);

        TextLabel4_2 = new QLabel(addressTab);
        TextLabel4_2->setObjectName("TextLabel4_2");
        sizePolicy2.setHeightForWidth(TextLabel4_2->sizePolicy().hasHeightForWidth());
        TextLabel4_2->setSizePolicy(sizePolicy2);
        TextLabel4_2->setMinimumSize(QSize(90, 0));
        TextLabel4_2->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        TextLabel4_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel4_2, 6, 0, 1, 1);

        emailEdit = new KLineEdit(addressTab);
        emailEdit->setObjectName("emailEdit");

        gridLayout->addWidget(emailEdit, 5, 1, 1, 1);

        TextLabel4 = new QLabel(addressTab);
        TextLabel4->setObjectName("TextLabel4");
        sizePolicy2.setHeightForWidth(TextLabel4->sizePolicy().hasHeightForWidth());
        TextLabel4->setSizePolicy(sizePolicy2);
        TextLabel4->setMinimumSize(QSize(90, 0));
        TextLabel4->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        TextLabel4->setWordWrap(false);

        gridLayout->addWidget(TextLabel4, 0, 0, 1, 1);

        m_sendMail = new QPushButton(addressTab);
        m_sendMail->setObjectName("m_sendMail");

        gridLayout->addWidget(m_sendMail, 5, 2, 1, 1);

        telephoneEdit = new KLineEdit(addressTab);
        telephoneEdit->setObjectName("telephoneEdit");

        gridLayout->addWidget(telephoneEdit, 4, 1, 1, 2);

        payeecityEdit = new KLineEdit(addressTab);
        payeecityEdit->setObjectName("payeecityEdit");

        gridLayout->addWidget(payeecityEdit, 1, 1, 1, 2);

        payeestateEdit = new KLineEdit(addressTab);
        payeestateEdit->setObjectName("payeestateEdit");

        gridLayout->addWidget(payeestateEdit, 2, 1, 1, 2);

        postcodeEdit = new KLineEdit(addressTab);
        postcodeEdit->setObjectName("postcodeEdit");

        gridLayout->addWidget(postcodeEdit, 3, 1, 1, 2);

        addressEdit = new KTextEdit(addressTab);
        addressEdit->setObjectName("addressEdit");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(1);
        sizePolicy3.setHeightForWidth(addressEdit->sizePolicy().hasHeightForWidth());
        addressEdit->setSizePolicy(sizePolicy3);

        gridLayout->addWidget(addressEdit, 0, 1, 1, 2);

        notesEdit = new KTextEdit(addressTab);
        notesEdit->setObjectName("notesEdit");
        sizePolicy3.setHeightForWidth(notesEdit->sizePolicy().hasHeightForWidth());
        notesEdit->setSizePolicy(sizePolicy3);

        gridLayout->addWidget(notesEdit, 6, 1, 1, 2);

        spacer6 = new QSpacerItem(21, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacer6, 0, 0, 1, 1);

        m_tabWidget->addTab(addressTab, QString());
        matchingTab = new QWidget();
        matchingTab->setObjectName("matchingTab");
        verticalLayout_6 = new QVBoxLayout(matchingTab);
        verticalLayout_6->setSpacing(6);
        verticalLayout_6->setContentsMargins(11, 11, 11, 11);
        verticalLayout_6->setObjectName("verticalLayout_6");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(matchingTab);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        matchTypeCombo = new QComboBox(matchingTab);
        matchTypeCombo->setObjectName("matchTypeCombo");

        horizontalLayout->addWidget(matchTypeCombo);

        checkMatchIgnoreCase = new QCheckBox(matchingTab);
        checkMatchIgnoreCase->setObjectName("checkMatchIgnoreCase");

        horizontalLayout->addWidget(checkMatchIgnoreCase);

        horizontalLayout->setStretch(1, 1);
        horizontalLayout->setStretch(2, 1);

        verticalLayout_6->addLayout(horizontalLayout);

        groupBox = new QGroupBox(matchingTab);
        groupBox->setObjectName("groupBox");
        verticalLayout_4 = new QVBoxLayout(groupBox);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName("verticalLayout_4");
        matchKeyEditList = new KEditListWidget(groupBox);
        matchKeyEditList->setObjectName("matchKeyEditList");
        sizePolicy3.setHeightForWidth(matchKeyEditList->sizePolicy().hasHeightForWidth());
        matchKeyEditList->setSizePolicy(sizePolicy3);

        verticalLayout_4->addWidget(matchKeyEditList);


        verticalLayout_6->addWidget(groupBox);

        m_tabWidget->addTab(matchingTab, QString());
        defaultAccountTab = new QWidget();
        defaultAccountTab->setObjectName("defaultAccountTab");
        vboxLayout2 = new QVBoxLayout(defaultAccountTab);
        vboxLayout2->setSpacing(6);
        vboxLayout2->setContentsMargins(11, 11, 11, 11);
        vboxLayout2->setObjectName("vboxLayout2");
        checkEnableDefaultCategory = new QCheckBox(defaultAccountTab);
        checkEnableDefaultCategory->setObjectName("checkEnableDefaultCategory");

        vboxLayout2->addWidget(checkEnableDefaultCategory);

        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setObjectName("gridLayout1");
        labelDefaultCategory = new QLabel(defaultAccountTab);
        labelDefaultCategory->setObjectName("labelDefaultCategory");
        labelDefaultCategory->setWordWrap(false);

        gridLayout1->addWidget(labelDefaultCategory, 0, 0, 1, 1);

        comboDefaultCategory = new KMyMoneyAccountCombo(defaultAccountTab);
        comboDefaultCategory->setObjectName("comboDefaultCategory");

        gridLayout1->addWidget(comboDefaultCategory, 0, 1, 1, 1);

        buttonSuggestACategory = new QPushButton(defaultAccountTab);
        buttonSuggestACategory->setObjectName("buttonSuggestACategory");

        gridLayout1->addWidget(buttonSuggestACategory, 1, 1, 1, 1);


        vboxLayout2->addLayout(gridLayout1);

        spacer5 = new QSpacerItem(20, 31, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout2->addItem(spacer5);

        m_tabWidget->addTab(defaultAccountTab, QString());
        tabPayeeIdentifiers = new QWidget();
        tabPayeeIdentifiers->setObjectName("tabPayeeIdentifiers");
        verticalLayout_5 = new QVBoxLayout(tabPayeeIdentifiers);
        verticalLayout_5->setSpacing(6);
        verticalLayout_5->setContentsMargins(11, 11, 11, 11);
        verticalLayout_5->setObjectName("verticalLayout_5");
        payeeIdentifiers = new KPayeeIdentifierView(tabPayeeIdentifiers);
        payeeIdentifiers->setObjectName("payeeIdentifiers");

        verticalLayout_5->addWidget(payeeIdentifiers);

        m_tabWidget->addTab(tabPayeeIdentifiers, QString());
        tabLinks = new QWidget();
        tabLinks->setObjectName("tabLinks");
        gridLayout_3 = new QGridLayout(tabLinks);
        gridLayout_3->setSpacing(6);
        gridLayout_3->setContentsMargins(11, 11, 11, 11);
        gridLayout_3->setObjectName("gridLayout_3");
        verticalLayout_7 = new QVBoxLayout();
        verticalLayout_7->setSpacing(6);
        verticalLayout_7->setObjectName("verticalLayout_7");
        groupBox_2 = new QGroupBox(tabLinks);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout_5 = new QGridLayout(groupBox_2);
        gridLayout_5->setSpacing(6);
        gridLayout_5->setContentsMargins(11, 11, 11, 11);
        gridLayout_5->setObjectName("gridLayout_5");
        gridLayout_6 = new QGridLayout();
        gridLayout_6->setSpacing(6);
        gridLayout_6->setObjectName("gridLayout_6");
        idPatternEdit = new KLineEdit(groupBox_2);
        idPatternEdit->setObjectName("idPatternEdit");

        gridLayout_6->addWidget(idPatternEdit, 0, 1, 1, 1);

        idPatternLabel = new QLabel(groupBox_2);
        idPatternLabel->setObjectName("idPatternLabel");

        gridLayout_6->addWidget(idPatternLabel, 0, 0, 1, 1);

        urlTemplateLabel = new QLabel(groupBox_2);
        urlTemplateLabel->setObjectName("urlTemplateLabel");

        gridLayout_6->addWidget(urlTemplateLabel, 1, 0, 1, 1);

        urlTemplateEdit = new KLineEdit(groupBox_2);
        urlTemplateEdit->setObjectName("urlTemplateEdit");

        gridLayout_6->addWidget(urlTemplateEdit, 1, 1, 1, 1);


        gridLayout_5->addLayout(gridLayout_6, 0, 0, 1, 1);


        verticalLayout_7->addWidget(groupBox_2);

        groupBox_3 = new QGroupBox(tabLinks);
        groupBox_3->setObjectName("groupBox_3");
        gridLayout_7 = new QGridLayout(groupBox_3);
        gridLayout_7->setSpacing(6);
        gridLayout_7->setContentsMargins(11, 11, 11, 11);
        gridLayout_7->setObjectName("gridLayout_7");
        gridLayout_8 = new QGridLayout();
        gridLayout_8->setSpacing(6);
        gridLayout_8->setObjectName("gridLayout_8");
        matchingCheckLabel = new QLabel(groupBox_3);
        matchingCheckLabel->setObjectName("matchingCheckLabel");

        gridLayout_8->addWidget(matchingCheckLabel, 1, 1, 1, 1);

        label_1 = new QLabel(groupBox_3);
        label_1->setObjectName("label_1");

        gridLayout_8->addWidget(label_1, 0, 0, 1, 1);

        label_2 = new QLabel(groupBox_3);
        label_2->setObjectName("label_2");

        gridLayout_8->addWidget(label_2, 1, 0, 1, 1);

        matchingCheckEdit = new KLineEdit(groupBox_3);
        matchingCheckEdit->setObjectName("matchingCheckEdit");

        gridLayout_8->addWidget(matchingCheckEdit, 0, 1, 1, 1);

        label_3 = new QLabel(groupBox_3);
        label_3->setObjectName("label_3");

        gridLayout_8->addWidget(label_3, 2, 0, 1, 1);

        matchingCheckURL = new QLabel(groupBox_3);
        matchingCheckURL->setObjectName("matchingCheckURL");

        gridLayout_8->addWidget(matchingCheckURL, 2, 1, 1, 1);


        gridLayout_7->addLayout(gridLayout_8, 0, 0, 1, 1);


        verticalLayout_7->addWidget(groupBox_3);


        gridLayout_3->addLayout(verticalLayout_7, 0, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer, 1, 0, 1, 1);

        m_tabWidget->addTab(tabLinks, QString());
        m_splitter->addWidget(m_tabWidget);

        verticalLayout->addWidget(m_splitter);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        m_helpButton = new QPushButton(KPayeesView);
        m_helpButton->setObjectName("m_helpButton");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(m_helpButton->sizePolicy().hasHeightForWidth());
        m_helpButton->setSizePolicy(sizePolicy4);

        horizontalLayout_2->addWidget(m_helpButton);

        spacer14 = new QSpacerItem(260, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(spacer14);

        m_syncAddressbook = new QPushButton(KPayeesView);
        m_syncAddressbook->setObjectName("m_syncAddressbook");

        horizontalLayout_2->addWidget(m_syncAddressbook);

        m_updateButton = new QPushButton(KPayeesView);
        m_updateButton->setObjectName("m_updateButton");
        sizePolicy4.setHeightForWidth(m_updateButton->sizePolicy().hasHeightForWidth());
        m_updateButton->setSizePolicy(sizePolicy4);

        horizontalLayout_2->addWidget(m_updateButton);


        verticalLayout->addLayout(horizontalLayout_2);

        QWidget::setTabOrder(m_tabWidget, m_filterBox);
        QWidget::setTabOrder(m_filterBox, addressEdit);
        QWidget::setTabOrder(addressEdit, payeecityEdit);
        QWidget::setTabOrder(payeecityEdit, payeestateEdit);
        QWidget::setTabOrder(payeestateEdit, postcodeEdit);
        QWidget::setTabOrder(postcodeEdit, telephoneEdit);
        QWidget::setTabOrder(telephoneEdit, emailEdit);
        QWidget::setTabOrder(emailEdit, m_sendMail);
        QWidget::setTabOrder(m_sendMail, notesEdit);
        QWidget::setTabOrder(notesEdit, matchTypeCombo);
        QWidget::setTabOrder(matchTypeCombo, checkMatchIgnoreCase);
        QWidget::setTabOrder(checkMatchIgnoreCase, checkEnableDefaultCategory);
        QWidget::setTabOrder(checkEnableDefaultCategory, comboDefaultCategory);
        QWidget::setTabOrder(comboDefaultCategory, buttonSuggestACategory);
        QWidget::setTabOrder(buttonSuggestACategory, m_helpButton);
        QWidget::setTabOrder(m_helpButton, m_syncAddressbook);
        QWidget::setTabOrder(m_syncAddressbook, m_updateButton);

        retranslateUi(KPayeesView);

        m_tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KPayeesView);
    } // setupUi

    void retranslateUi(QWidget *KPayeesView)
    {
        groupBox1->setTitle(tr2i18n("Your payees", nullptr));
        m_newButton->setText(tr2i18n("New", nullptr));
        m_renameButton->setText(tr2i18n("Ren", nullptr));
        m_deleteButton->setText(tr2i18n("Del", nullptr));
        m_mergeButton->setText(tr2i18n("Merge", nullptr));
        m_balanceLabel->setText(tr2i18n("Balance: ", nullptr));
        m_tabWidget->setTabText(m_tabWidget->indexOf(transactionTab), tr2i18n("Transactions", nullptr));
        TextLabel7->setText(tr2i18n("E-Mail:", "E-mail address"));
        TextLabel5_1->setText(tr2i18n("City:", nullptr));
        TextLabel5_2->setText(tr2i18n("State:", nullptr));
        TextLabel5->setText(tr2i18n("Postal Code:", nullptr));
        TextLabel6->setText(tr2i18n("Telephone/Fax:", nullptr));
        TextLabel4_2->setText(tr2i18n("Notes", nullptr));
        TextLabel4->setText(tr2i18n("Address:", "Street address"));
#if QT_CONFIG(tooltip)
        m_sendMail->setToolTip(tr2i18n("Creates new e-mail to your payee.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_sendMail->setWhatsThis(tr2i18n("Use this to create new e-mail to your payee.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_sendMail->setText(tr2i18n("Send", nullptr));
        m_tabWidget->setTabText(m_tabWidget->indexOf(addressTab), tr2i18n("Address", "Street address"));
        label->setText(tr2i18n("Match method", nullptr));
        checkMatchIgnoreCase->setText(tr2i18n("Ignore Case", nullptr));
        groupBox->setTitle(tr2i18n("List of matching names", nullptr));
#if QT_CONFIG(whatsthis)
        matchKeyEditList->setWhatsThis(tr2i18n("This list contains the names that will match this payee if a transaction is imported from an external source. Keep in mind, that you can specify regular expressions here.", nullptr));
#endif // QT_CONFIG(whatsthis)
        matchKeyEditList->setProperty("title", QVariant(tr2i18n("List of matching names", nullptr)));
        m_tabWidget->setTabText(m_tabWidget->indexOf(matchingTab), tr2i18n("Matching", nullptr));
        checkEnableDefaultCategory->setText(tr2i18n("Use the default category for new transactions with this payee", nullptr));
        labelDefaultCategory->setText(tr2i18n("Default category:", nullptr));
        buttonSuggestACategory->setText(tr2i18n("Suggest a category", nullptr));
        m_tabWidget->setTabText(m_tabWidget->indexOf(defaultAccountTab), tr2i18n("Default Category", nullptr));
        m_tabWidget->setTabText(m_tabWidget->indexOf(tabPayeeIdentifiers), tr2i18n("Account Numbers", nullptr));
        groupBox_2->setTitle(tr2i18n("Matching setup", nullptr));
        idPatternEdit->setText(tr2i18n("\\d+-\\d+-\\d+", nullptr));
        idPatternLabel->setText(tr2i18n("Identifier pattern", nullptr));
        urlTemplateLabel->setText(tr2i18n("Url template", nullptr));
        groupBox_3->setTitle(tr2i18n("Matching check", nullptr));
        matchingCheckLabel->setText(QString());
        label_1->setText(tr2i18n("Enter identifier", nullptr));
        label_2->setText(tr2i18n("Captured result", nullptr));
        label_3->setText(tr2i18n("Resulting URL", nullptr));
        matchingCheckURL->setText(tr2i18n("TextLabel", nullptr));
        m_tabWidget->setTabText(m_tabWidget->indexOf(tabLinks), tr2i18n("Link Matching", nullptr));
        m_helpButton->setText(tr2i18n("Help", nullptr));
#if QT_CONFIG(tooltip)
        m_syncAddressbook->setToolTip(tr2i18n("Fetches the payee's data from your addressbook.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_syncAddressbook->setWhatsThis(tr2i18n("Use this to fetch payee's data.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_syncAddressbook->setText(tr2i18n("Sync", nullptr));
#if QT_CONFIG(tooltip)
        m_updateButton->setToolTip(tr2i18n("Accepts the entered data and stores it", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_updateButton->setWhatsThis(tr2i18n("Use this to accept the modified data.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_updateButton->setText(tr2i18n("Update", nullptr));
        (void)KPayeesView;
    } // retranslateUi

};

namespace Ui {
    class KPayeesView: public Ui_KPayeesView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KPAYEESVIEW_H

