/********************************************************************************
** Form generated from reading UI file 'kupdatestockpricedlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KUPDATESTOCKPRICEDLG_H
#define UI_KUPDATESTOCKPRICEDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "kmymoneycurrencyselector.h"
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_KUpdateStockPriceDlg
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    KMyMoneyDateEdit *m_date;
    QSpacerItem *spacer25;
    QHBoxLayout *hboxLayout1;
    KMyMoneySecuritySelector *m_currency;
    QSpacerItem *spacer26;
    QLabel *textLabel2;
    QLabel *lblDate;
    QLabel *textLabel1;
    QHBoxLayout *hboxLayout2;
    KMyMoneySecuritySelector *m_security;
    QSpacerItem *spacer27;
    QSpacerItem *spacer8;
    QFrame *line6;
    QDialogButtonBox *m_buttonBox;

    void setupUi(QDialog *KUpdateStockPriceDlg)
    {
        if (KUpdateStockPriceDlg->objectName().isEmpty())
            KUpdateStockPriceDlg->setObjectName("KUpdateStockPriceDlg");
        KUpdateStockPriceDlg->resize(457, 194);
        vboxLayout = new QVBoxLayout(KUpdateStockPriceDlg);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_date = new KMyMoneyDateEdit(KUpdateStockPriceDlg);
        m_date->setObjectName("m_date");

        hboxLayout->addWidget(m_date);

        spacer25 = new QSpacerItem(71, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer25);


        gridLayout->addLayout(hboxLayout, 2, 1, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        m_currency = new KMyMoneySecuritySelector(KUpdateStockPriceDlg);
        m_currency->setObjectName("m_currency");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_currency->sizePolicy().hasHeightForWidth());
        m_currency->setSizePolicy(sizePolicy);

        hboxLayout1->addWidget(m_currency);

        spacer26 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacer26);


        gridLayout->addLayout(hboxLayout1, 1, 1, 1, 1);

        textLabel2 = new QLabel(KUpdateStockPriceDlg);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 1, 0, 1, 1);

        lblDate = new QLabel(KUpdateStockPriceDlg);
        lblDate->setObjectName("lblDate");
        lblDate->setWordWrap(false);

        gridLayout->addWidget(lblDate, 2, 0, 1, 1);

        textLabel1 = new QLabel(KUpdateStockPriceDlg);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        m_security = new KMyMoneySecuritySelector(KUpdateStockPriceDlg);
        m_security->setObjectName("m_security");
        sizePolicy.setHeightForWidth(m_security->sizePolicy().hasHeightForWidth());
        m_security->setSizePolicy(sizePolicy);

        hboxLayout2->addWidget(m_security);

        spacer27 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout2->addItem(spacer27);


        gridLayout->addLayout(hboxLayout2, 0, 1, 1, 1);


        vboxLayout->addLayout(gridLayout);

        spacer8 = new QSpacerItem(31, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer8);

        line6 = new QFrame(KUpdateStockPriceDlg);
        line6->setObjectName("line6");
        line6->setFrameShape(QFrame::HLine);
        line6->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(line6);

        m_buttonBox = new QDialogButtonBox(KUpdateStockPriceDlg);
        m_buttonBox->setObjectName("m_buttonBox");
        m_buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(m_buttonBox);


        retranslateUi(KUpdateStockPriceDlg);
        QObject::connect(m_buttonBox, &QDialogButtonBox::accepted, KUpdateStockPriceDlg, qOverload<>(&QDialog::accept));
        QObject::connect(m_buttonBox, &QDialogButtonBox::rejected, KUpdateStockPriceDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KUpdateStockPriceDlg);
    } // setupUi

    void retranslateUi(QDialog *KUpdateStockPriceDlg)
    {
        KUpdateStockPriceDlg->setWindowTitle(tr2i18n("New price entry", nullptr));
        textLabel2->setText(tr2i18n("Currency", nullptr));
        lblDate->setText(tr2i18n("Date:", nullptr));
        textLabel1->setText(tr2i18n("Security", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KUpdateStockPriceDlg: public Ui_KUpdateStockPriceDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KUPDATESTOCKPRICEDLG_H

