/********************************************************************************
** Form generated from reading UI file 'additionalfeeswizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ADDITIONALFEESWIZARDPAGE_H
#define UI_ADDITIONALFEESWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_AdditionalFeesWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel21_2;
    QSpacerItem *Spacer58;
    QFrame *Frame9;
    QGridLayout *gridLayout;
    QLabel *TextLabel21_1_3;
    QLabel *m_periodicPayment;
    QLabel *m_additionalCost;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel21_1_2;
    QPushButton *m_additionalFeeButton;
    QLabel *m_basePayment;
    QLabel *TextLabel21_1;
    QSpacerItem *spacer173_2;
    QSpacerItem *spacer173;
    QSpacerItem *Spacer59;
    QLabel *TextLabel21_2_2;
    QSpacerItem *Spacer60;

    void setupUi(QWizardPage *AdditionalFeesWizardPage)
    {
        if (AdditionalFeesWizardPage->objectName().isEmpty())
            AdditionalFeesWizardPage->setObjectName("AdditionalFeesWizardPage");
        vboxLayout = new QVBoxLayout(AdditionalFeesWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel21_2 = new QLabel(AdditionalFeesWizardPage);
        TextLabel21_2->setObjectName("TextLabel21_2");
        TextLabel21_2->setAlignment(Qt::AlignTop);
        TextLabel21_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel21_2);

        Spacer58 = new QSpacerItem(0, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer58);

        Frame9 = new QFrame(AdditionalFeesWizardPage);
        Frame9->setObjectName("Frame9");
        Frame9->setFrameShape(QFrame::StyledPanel);
        Frame9->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(Frame9);
#ifndef UI_Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        TextLabel21_1_3 = new QLabel(Frame9);
        TextLabel21_1_3->setObjectName("TextLabel21_1_3");
        TextLabel21_1_3->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel21_1_3->setWordWrap(false);

        gridLayout->addWidget(TextLabel21_1_3, 2, 1, 1, 1);

        m_periodicPayment = new QLabel(Frame9);
        m_periodicPayment->setObjectName("m_periodicPayment");
        QFont font;
        font.setBold(true);
        m_periodicPayment->setFont(font);
        m_periodicPayment->setTextFormat(Qt::PlainText);
        m_periodicPayment->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        m_periodicPayment->setWordWrap(false);

        gridLayout->addWidget(m_periodicPayment, 2, 2, 1, 1);

        m_additionalCost = new QLabel(Frame9);
        m_additionalCost->setObjectName("m_additionalCost");
        m_additionalCost->setFont(font);
        m_additionalCost->setTextFormat(Qt::PlainText);
        m_additionalCost->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        m_additionalCost->setWordWrap(false);

        gridLayout->addWidget(m_additionalCost, 1, 2, 1, 1);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        TextLabel21_1_2 = new QLabel(Frame9);
        TextLabel21_1_2->setObjectName("TextLabel21_1_2");
        TextLabel21_1_2->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel21_1_2->setWordWrap(false);

        hboxLayout->addWidget(TextLabel21_1_2);

        m_additionalFeeButton = new QPushButton(Frame9);
        m_additionalFeeButton->setObjectName("m_additionalFeeButton");

        hboxLayout->addWidget(m_additionalFeeButton);


        gridLayout->addLayout(hboxLayout, 1, 1, 1, 1);

        m_basePayment = new QLabel(Frame9);
        m_basePayment->setObjectName("m_basePayment");
        m_basePayment->setFont(font);
        m_basePayment->setTextFormat(Qt::PlainText);
        m_basePayment->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        m_basePayment->setWordWrap(false);

        gridLayout->addWidget(m_basePayment, 0, 2, 1, 1);

        TextLabel21_1 = new QLabel(Frame9);
        TextLabel21_1->setObjectName("TextLabel21_1");
        TextLabel21_1->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel21_1->setWordWrap(false);

        gridLayout->addWidget(TextLabel21_1, 0, 1, 1, 1);

        spacer173_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacer173_2, 1, 0, 1, 1);

        spacer173 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacer173, 1, 3, 1, 1);


        vboxLayout->addWidget(Frame9);

        Spacer59 = new QSpacerItem(0, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer59);

        TextLabel21_2_2 = new QLabel(AdditionalFeesWizardPage);
        TextLabel21_2_2->setObjectName("TextLabel21_2_2");
        TextLabel21_2_2->setAlignment(Qt::AlignTop);
        TextLabel21_2_2->setWordWrap(true);

        vboxLayout->addWidget(TextLabel21_2_2);

        Spacer60 = new QSpacerItem(0, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer60);


        retranslateUi(AdditionalFeesWizardPage);

        QMetaObject::connectSlotsByName(AdditionalFeesWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *AdditionalFeesWizardPage)
    {
        TextLabel21_2->setText(tr2i18n("\n"
"If your regular payment contains any additional fees, click on the button \"Additional fees\" to enter them.", nullptr));
        TextLabel21_1_3->setText(tr2i18n("= periodical payment:", nullptr));
        m_periodicPayment->setText(tr2i18n("1", nullptr));
        m_additionalCost->setText(tr2i18n("1", nullptr));
        TextLabel21_1_2->setText(tr2i18n("+", nullptr));
        m_additionalFeeButton->setText(QString());
        m_basePayment->setText(tr2i18n("1", nullptr));
        TextLabel21_1->setText(tr2i18n("Principal + Interest:", nullptr));
        TextLabel21_2_2->setText(tr2i18n("If no additional fees are included in your periodical payment or you have entered all such fees, then click on \"Next\".", nullptr));
        (void)AdditionalFeesWizardPage;
    } // retranslateUi

};

namespace Ui {
    class AdditionalFeesWizardPage: public Ui_AdditionalFeesWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // ADDITIONALFEESWIZARDPAGE_H

