/********************************************************************************
** Form generated from reading UI file 'editpersonaldatadlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITPERSONALDATADLG_H
#define UI_EDITPERSONALDATADLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_EditPersonalDataDlg
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel1_2;
    QFrame *Line1;
    QGridLayout *gridLayout;
    QLabel *TextLabel4;
    KLineEdit *userNameEdit;
    KLineEdit *countyEdit;
    KLineEdit *townEdit;
    QLabel *TextLabel2;
    KLineEdit *streetEdit;
    QLabel *TextLabel7;
    QLabel *TextLabel1;
    QLabel *TextLabel6;
    QLabel *TextLabel3;
    KLineEdit *emailEdit;
    QLabel *TextLabel5;
    KLineEdit *telephoneEdit;
    KLineEdit *postcodeEdit;
    QHBoxLayout *hboxLayout1;
    QPushButton *kabcBtn;
    QSpacerItem *spacer11;
    QSpacerItem *spacer2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *EditPersonalDataDlg)
    {
        if (EditPersonalDataDlg->objectName().isEmpty())
            EditPersonalDataDlg->setObjectName("EditPersonalDataDlg");
        EditPersonalDataDlg->resize(546, 461);
        EditPersonalDataDlg->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(EditPersonalDataDlg);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        TextLabel1_2 = new QLabel(EditPersonalDataDlg);
        TextLabel1_2->setObjectName("TextLabel1_2");
        TextLabel1_2->setWordWrap(false);

        hboxLayout->addWidget(TextLabel1_2);


        vboxLayout->addLayout(hboxLayout);

        Line1 = new QFrame(EditPersonalDataDlg);
        Line1->setObjectName("Line1");
        Line1->setFrameShape(QFrame::HLine);
        Line1->setFrameShadow(QFrame::Raised);
        Line1->setLineWidth(1);

        vboxLayout->addWidget(Line1);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        TextLabel4 = new QLabel(EditPersonalDataDlg);
        TextLabel4->setObjectName("TextLabel4");
        TextLabel4->setMinimumSize(QSize(100, 0));
        TextLabel4->setWordWrap(false);

        gridLayout->addWidget(TextLabel4, 3, 0, 1, 1);

        userNameEdit = new KLineEdit(EditPersonalDataDlg);
        userNameEdit->setObjectName("userNameEdit");

        gridLayout->addWidget(userNameEdit, 0, 1, 1, 1);

        countyEdit = new KLineEdit(EditPersonalDataDlg);
        countyEdit->setObjectName("countyEdit");

        gridLayout->addWidget(countyEdit, 3, 1, 1, 1);

        townEdit = new KLineEdit(EditPersonalDataDlg);
        townEdit->setObjectName("townEdit");

        gridLayout->addWidget(townEdit, 2, 1, 1, 1);

        TextLabel2 = new QLabel(EditPersonalDataDlg);
        TextLabel2->setObjectName("TextLabel2");
        TextLabel2->setMinimumSize(QSize(100, 0));
        TextLabel2->setWordWrap(false);

        gridLayout->addWidget(TextLabel2, 1, 0, 1, 1);

        streetEdit = new KLineEdit(EditPersonalDataDlg);
        streetEdit->setObjectName("streetEdit");

        gridLayout->addWidget(streetEdit, 1, 1, 1, 1);

        TextLabel7 = new QLabel(EditPersonalDataDlg);
        TextLabel7->setObjectName("TextLabel7");
        TextLabel7->setMinimumSize(QSize(100, 0));
        TextLabel7->setWordWrap(false);

        gridLayout->addWidget(TextLabel7, 6, 0, 1, 1);

        TextLabel1 = new QLabel(EditPersonalDataDlg);
        TextLabel1->setObjectName("TextLabel1");
        TextLabel1->setWordWrap(false);

        gridLayout->addWidget(TextLabel1, 0, 0, 1, 1);

        TextLabel6 = new QLabel(EditPersonalDataDlg);
        TextLabel6->setObjectName("TextLabel6");
        TextLabel6->setMinimumSize(QSize(100, 0));
        TextLabel6->setWordWrap(false);

        gridLayout->addWidget(TextLabel6, 5, 0, 1, 1);

        TextLabel3 = new QLabel(EditPersonalDataDlg);
        TextLabel3->setObjectName("TextLabel3");
        TextLabel3->setMinimumSize(QSize(100, 0));
        TextLabel3->setWordWrap(false);

        gridLayout->addWidget(TextLabel3, 2, 0, 1, 1);

        emailEdit = new KLineEdit(EditPersonalDataDlg);
        emailEdit->setObjectName("emailEdit");

        gridLayout->addWidget(emailEdit, 6, 1, 1, 1);

        TextLabel5 = new QLabel(EditPersonalDataDlg);
        TextLabel5->setObjectName("TextLabel5");
        TextLabel5->setMinimumSize(QSize(100, 0));
        TextLabel5->setWordWrap(false);

        gridLayout->addWidget(TextLabel5, 4, 0, 1, 1);

        telephoneEdit = new KLineEdit(EditPersonalDataDlg);
        telephoneEdit->setObjectName("telephoneEdit");

        gridLayout->addWidget(telephoneEdit, 5, 1, 1, 1);

        postcodeEdit = new KLineEdit(EditPersonalDataDlg);
        postcodeEdit->setObjectName("postcodeEdit");

        gridLayout->addWidget(postcodeEdit, 4, 1, 1, 1);


        vboxLayout->addLayout(gridLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        kabcBtn = new QPushButton(EditPersonalDataDlg);
        kabcBtn->setObjectName("kabcBtn");

        hboxLayout1->addWidget(kabcBtn);

        spacer11 = new QSpacerItem(305, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacer11);


        vboxLayout->addLayout(hboxLayout1);

        spacer2 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer2);

        buttonBox = new QDialogButtonBox(EditPersonalDataDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);

        QWidget::setTabOrder(userNameEdit, streetEdit);
        QWidget::setTabOrder(streetEdit, townEdit);
        QWidget::setTabOrder(townEdit, countyEdit);
        QWidget::setTabOrder(countyEdit, postcodeEdit);
        QWidget::setTabOrder(postcodeEdit, telephoneEdit);
        QWidget::setTabOrder(telephoneEdit, emailEdit);
        QWidget::setTabOrder(emailEdit, kabcBtn);

        retranslateUi(EditPersonalDataDlg);

        QMetaObject::connectSlotsByName(EditPersonalDataDlg);
    } // setupUi

    void retranslateUi(QDialog *EditPersonalDataDlg)
    {
        EditPersonalDataDlg->setWindowTitle(tr2i18n("New File Dialog", nullptr));
        TextLabel1_2->setText(tr2i18n("Use this dialog to enter personal information about yourself.\n"
"\n"
"All information is optional and is provided to personalize\n"
"your KMyMoney file.", nullptr));
        TextLabel4->setText(tr2i18n("County/State:", nullptr));
        TextLabel2->setText(tr2i18n("Street:", nullptr));
        TextLabel7->setText(tr2i18n("Email:", "Email address"));
        TextLabel1->setText(tr2i18n("Your name:", nullptr));
        TextLabel6->setText(tr2i18n("Telephone:", nullptr));
        TextLabel3->setText(tr2i18n("Town:", nullptr));
        TextLabel5->setText(tr2i18n("Postal Code:", nullptr));
        kabcBtn->setText(tr2i18n("Load from Addressbook", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EditPersonalDataDlg: public Ui_EditPersonalDataDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // EDITPERSONALDATADLG_H

