/********************************************************************************
** Form generated from reading UI file 'ksettingsgeneral.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KSETTINGSGENERAL_H
#define UI_KSETTINGSGENERAL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "kdatecombobox.h"
#include "klineedit.h"
#include "kmymoneydateedit.h"

QT_BEGIN_NAMESPACE

class Ui_KSettingsGeneral
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *tabWidget2;
    QWidget *tab;
    QVBoxLayout *verticalLayout_6;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_3;
    QCheckBox *kcfg_AutoSaveFile;
    QWidget *m_periodFrame;
    QHBoxLayout *horizontalLayout_2;
    QSpinBox *kcfg_AutoSavePeriod;
    QLabel *textLabel3;
    QSpacerItem *spacer3;
    QCheckBox *kcfg_AutoSaveOnClose;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_11;
    QHBoxLayout *horizontalLayout_6;
    QGridLayout *gridLayout;
    QLineEdit *kcfg_BackupExtension;
    QSpinBox *kcfg_AutoBackupCopies;
    QLabel *label_2;
    QLabel *label_5;
    QSpacerItem *horizontalSpacer;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout;
    QLabel *textLabel1_2_2;
    QSpinBox *kcfg_FiscalYearBeginDay;
    KComboBox *kcfg_FiscalYearBegin;
    QSpacerItem *spacer11;
    QGroupBox *groupBox_7;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label;
    KLineEdit *kcfg_ExternalCalculator;
    QSpacerItem *spacer3_2;
    QWidget *tab2;
    QVBoxLayout *verticalLayout_7;
    QGroupBox *kcfg_StartLastViewSelected;
    QVBoxLayout *verticalLayout_8;
    QRadioButton *radioButton1;
    QRadioButton *radioButton2;
    QGroupBox *kcfg_viewType;
    QGridLayout *gridLayout1;
    QRadioButton *radioButton3;
    QRadioButton *radioButton4;
    QRadioButton *radioButton5;
    QCheckBox *kcfg_syncLedgerInvestment;
    QSpacerItem *spacer7;
    QWidget *TabPage;
    QVBoxLayout *verticalLayout_5;
    QGroupBox *groupBox_6;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *kcfg_HideUnusedCategory;
    QCheckBox *kcfg_ExpertMode;
    QCheckBox *kcfg_ShowCategoriesInAccountsView;
    QCheckBox *kcfg_HideZeroBalanceAccountsHome;
    QCheckBox *kcfg_HideZeroBalanceAccounts;
    QCheckBox *kcfg_HideZeroBalanceEquities;
    QGroupBox *groupBox_5;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *kcfg_HideFinishedSchedules;
    QGroupBox *groupBox_4;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel1_3;
    KMyMoneyDateEdit *kcfg_StartDate;
    QSpacerItem *spacer10;
    QCheckBox *kcfg_HideReconciledTransactions;
    QSpacerItem *spacer4;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_9;
    QGroupBox *groupBox_8;
    QVBoxLayout *verticalLayout_10;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_3;
    QLineEdit *kcfg_logPath;
    QPushButton *choosePath;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label_4;
    QCheckBox *kcfg_logImportedStatements;
    QCheckBox *kcfg_logOfxTransactions;
    QSpacerItem *verticalSpacer;
    QSpacerItem *verticalSpacer1;

    void setupUi(QWidget *KSettingsGeneral)
    {
        if (KSettingsGeneral->objectName().isEmpty())
            KSettingsGeneral->setObjectName("KSettingsGeneral");
        KSettingsGeneral->resize(592, 482);
        vboxLayout = new QVBoxLayout(KSettingsGeneral);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        tabWidget2 = new QTabWidget(KSettingsGeneral);
        tabWidget2->setObjectName("tabWidget2");
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_6 = new QVBoxLayout(tab);
        verticalLayout_6->setSpacing(6);
        verticalLayout_6->setContentsMargins(11, 11, 11, 11);
        verticalLayout_6->setObjectName("verticalLayout_6");
        groupBox_2 = new QGroupBox(tab);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setSpacing(0);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        kcfg_AutoSaveFile = new QCheckBox(groupBox_2);
        kcfg_AutoSaveFile->setObjectName("kcfg_AutoSaveFile");

        horizontalLayout_3->addWidget(kcfg_AutoSaveFile);

        m_periodFrame = new QWidget(groupBox_2);
        m_periodFrame->setObjectName("m_periodFrame");
        m_periodFrame->setEnabled(false);
        horizontalLayout_2 = new QHBoxLayout(m_periodFrame);
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setContentsMargins(11, 11, 11, 11);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        kcfg_AutoSavePeriod = new QSpinBox(m_periodFrame);
        kcfg_AutoSavePeriod->setObjectName("kcfg_AutoSavePeriod");
        kcfg_AutoSavePeriod->setMaximum(60);

        horizontalLayout_2->addWidget(kcfg_AutoSavePeriod);

        textLabel3 = new QLabel(m_periodFrame);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        horizontalLayout_2->addWidget(textLabel3);


        horizontalLayout_3->addWidget(m_periodFrame);

        spacer3 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(spacer3);


        verticalLayout->addLayout(horizontalLayout_3);

        kcfg_AutoSaveOnClose = new QCheckBox(groupBox_2);
        kcfg_AutoSaveOnClose->setObjectName("kcfg_AutoSaveOnClose");

        verticalLayout->addWidget(kcfg_AutoSaveOnClose);


        verticalLayout_6->addWidget(groupBox_2);

        groupBox_3 = new QGroupBox(tab);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_11 = new QVBoxLayout(groupBox_3);
        verticalLayout_11->setSpacing(6);
        verticalLayout_11->setContentsMargins(11, 11, 11, 11);
        verticalLayout_11->setObjectName("verticalLayout_11");
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setSpacing(6);
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        kcfg_BackupExtension = new QLineEdit(groupBox_3);
        kcfg_BackupExtension->setObjectName("kcfg_BackupExtension");
        kcfg_BackupExtension->setClearButtonEnabled(true);

        gridLayout->addWidget(kcfg_BackupExtension, 1, 0, 1, 1);

        kcfg_AutoBackupCopies = new QSpinBox(groupBox_3);
        kcfg_AutoBackupCopies->setObjectName("kcfg_AutoBackupCopies");
        kcfg_AutoBackupCopies->setFrame(true);
        kcfg_AutoBackupCopies->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        kcfg_AutoBackupCopies->setMaximum(20);
        kcfg_AutoBackupCopies->setSingleStep(1);

        gridLayout->addWidget(kcfg_AutoBackupCopies, 0, 0, 1, 1);

        label_2 = new QLabel(groupBox_3);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 1, 1, 1);

        label_5 = new QLabel(groupBox_3);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 0, 1, 1, 1);


        horizontalLayout_6->addLayout(gridLayout);

        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer);


        verticalLayout_11->addLayout(horizontalLayout_6);


        verticalLayout_6->addWidget(groupBox_3);

        groupBox = new QGroupBox(tab);
        groupBox->setObjectName("groupBox");
        horizontalLayout = new QHBoxLayout(groupBox);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setContentsMargins(11, 11, 11, 11);
        horizontalLayout->setObjectName("horizontalLayout");
        textLabel1_2_2 = new QLabel(groupBox);
        textLabel1_2_2->setObjectName("textLabel1_2_2");
        textLabel1_2_2->setWordWrap(false);

        horizontalLayout->addWidget(textLabel1_2_2);

        kcfg_FiscalYearBeginDay = new QSpinBox(groupBox);
        kcfg_FiscalYearBeginDay->setObjectName("kcfg_FiscalYearBeginDay");
        kcfg_FiscalYearBeginDay->setMaximum(366);

        horizontalLayout->addWidget(kcfg_FiscalYearBeginDay);

        kcfg_FiscalYearBegin = new KComboBox(groupBox);
        kcfg_FiscalYearBegin->addItem(QString());
        kcfg_FiscalYearBegin->addItem(QString());
        kcfg_FiscalYearBegin->addItem(QString());
        kcfg_FiscalYearBegin->addItem(QString());
        kcfg_FiscalYearBegin->addItem(QString());
        kcfg_FiscalYearBegin->addItem(QString());
        kcfg_FiscalYearBegin->addItem(QString());
        kcfg_FiscalYearBegin->addItem(QString());
        kcfg_FiscalYearBegin->addItem(QString());
        kcfg_FiscalYearBegin->addItem(QString());
        kcfg_FiscalYearBegin->addItem(QString());
        kcfg_FiscalYearBegin->addItem(QString());
        kcfg_FiscalYearBegin->setObjectName("kcfg_FiscalYearBegin");

        horizontalLayout->addWidget(kcfg_FiscalYearBegin);

        spacer11 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(spacer11);


        verticalLayout_6->addWidget(groupBox);

        groupBox_7 = new QGroupBox(tab);
        groupBox_7->setObjectName("groupBox_7");
        horizontalLayout_5 = new QHBoxLayout(groupBox_7);
        horizontalLayout_5->setSpacing(6);
        horizontalLayout_5->setContentsMargins(11, 11, 11, 11);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label = new QLabel(groupBox_7);
        label->setObjectName("label");

        horizontalLayout_5->addWidget(label);

        kcfg_ExternalCalculator = new KLineEdit(groupBox_7);
        kcfg_ExternalCalculator->setObjectName("kcfg_ExternalCalculator");

        horizontalLayout_5->addWidget(kcfg_ExternalCalculator);


        verticalLayout_6->addWidget(groupBox_7);

        spacer3_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_6->addItem(spacer3_2);

        tabWidget2->addTab(tab, QString());
        tab2 = new QWidget();
        tab2->setObjectName("tab2");
        verticalLayout_7 = new QVBoxLayout(tab2);
        verticalLayout_7->setSpacing(6);
        verticalLayout_7->setContentsMargins(11, 11, 11, 11);
        verticalLayout_7->setObjectName("verticalLayout_7");
        kcfg_StartLastViewSelected = new QGroupBox(tab2);
        kcfg_StartLastViewSelected->setObjectName("kcfg_StartLastViewSelected");
        verticalLayout_8 = new QVBoxLayout(kcfg_StartLastViewSelected);
        verticalLayout_8->setSpacing(6);
        verticalLayout_8->setContentsMargins(11, 11, 11, 11);
        verticalLayout_8->setObjectName("verticalLayout_8");
        radioButton1 = new QRadioButton(kcfg_StartLastViewSelected);
        radioButton1->setObjectName("radioButton1");
        radioButton1->setChecked(true);

        verticalLayout_8->addWidget(radioButton1);

        radioButton2 = new QRadioButton(kcfg_StartLastViewSelected);
        radioButton2->setObjectName("radioButton2");

        verticalLayout_8->addWidget(radioButton2);


        verticalLayout_7->addWidget(kcfg_StartLastViewSelected);

        kcfg_viewType = new QGroupBox(tab2);
        kcfg_viewType->setObjectName("kcfg_viewType");
        gridLayout1 = new QGridLayout(kcfg_viewType);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName("gridLayout1");
        radioButton3 = new QRadioButton(kcfg_viewType);
        radioButton3->setObjectName("radioButton3");
        radioButton3->setChecked(true);

        gridLayout1->addWidget(radioButton3, 0, 0, 1, 1);

        radioButton4 = new QRadioButton(kcfg_viewType);
        radioButton4->setObjectName("radioButton4");

        gridLayout1->addWidget(radioButton4, 0, 1, 1, 1);

        radioButton5 = new QRadioButton(kcfg_viewType);
        radioButton5->setObjectName("radioButton5");

        gridLayout1->addWidget(radioButton5, 0, 2, 1, 1);


        verticalLayout_7->addWidget(kcfg_viewType);

        kcfg_syncLedgerInvestment = new QCheckBox(tab2);
        kcfg_syncLedgerInvestment->setObjectName("kcfg_syncLedgerInvestment");

        verticalLayout_7->addWidget(kcfg_syncLedgerInvestment);

        spacer7 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(spacer7);

        tabWidget2->addTab(tab2, QString());
        TabPage = new QWidget();
        TabPage->setObjectName("TabPage");
        verticalLayout_5 = new QVBoxLayout(TabPage);
        verticalLayout_5->setSpacing(6);
        verticalLayout_5->setContentsMargins(11, 11, 11, 11);
        verticalLayout_5->setObjectName("verticalLayout_5");
        groupBox_6 = new QGroupBox(TabPage);
        groupBox_6->setObjectName("groupBox_6");
        verticalLayout_3 = new QVBoxLayout(groupBox_6);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        kcfg_HideUnusedCategory = new QCheckBox(groupBox_6);
        kcfg_HideUnusedCategory->setObjectName("kcfg_HideUnusedCategory");

        verticalLayout_3->addWidget(kcfg_HideUnusedCategory);

        kcfg_ExpertMode = new QCheckBox(groupBox_6);
        kcfg_ExpertMode->setObjectName("kcfg_ExpertMode");

        verticalLayout_3->addWidget(kcfg_ExpertMode);

        kcfg_ShowCategoriesInAccountsView = new QCheckBox(groupBox_6);
        kcfg_ShowCategoriesInAccountsView->setObjectName("kcfg_ShowCategoriesInAccountsView");

        verticalLayout_3->addWidget(kcfg_ShowCategoriesInAccountsView);

        kcfg_HideZeroBalanceAccountsHome = new QCheckBox(groupBox_6);
        kcfg_HideZeroBalanceAccountsHome->setObjectName("kcfg_HideZeroBalanceAccountsHome");

        verticalLayout_3->addWidget(kcfg_HideZeroBalanceAccountsHome);

        kcfg_HideZeroBalanceAccounts = new QCheckBox(groupBox_6);
        kcfg_HideZeroBalanceAccounts->setObjectName("kcfg_HideZeroBalanceAccounts");

        verticalLayout_3->addWidget(kcfg_HideZeroBalanceAccounts);

        kcfg_HideZeroBalanceEquities = new QCheckBox(groupBox_6);
        kcfg_HideZeroBalanceEquities->setObjectName("kcfg_HideZeroBalanceEquities");

        verticalLayout_3->addWidget(kcfg_HideZeroBalanceEquities);


        verticalLayout_5->addWidget(groupBox_6);

        groupBox_5 = new QGroupBox(TabPage);
        groupBox_5->setObjectName("groupBox_5");
        verticalLayout_4 = new QVBoxLayout(groupBox_5);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName("verticalLayout_4");
        kcfg_HideFinishedSchedules = new QCheckBox(groupBox_5);
        kcfg_HideFinishedSchedules->setObjectName("kcfg_HideFinishedSchedules");

        verticalLayout_4->addWidget(kcfg_HideFinishedSchedules);


        verticalLayout_5->addWidget(groupBox_5);

        groupBox_4 = new QGroupBox(TabPage);
        groupBox_4->setObjectName("groupBox_4");
        verticalLayout_2 = new QVBoxLayout(groupBox_4);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        textLabel1_3 = new QLabel(groupBox_4);
        textLabel1_3->setObjectName("textLabel1_3");
        textLabel1_3->setWordWrap(false);

        hboxLayout->addWidget(textLabel1_3);

        kcfg_StartDate = new KMyMoneyDateEdit(groupBox_4);
        kcfg_StartDate->setObjectName("kcfg_StartDate");

        hboxLayout->addWidget(kcfg_StartDate);

        spacer10 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer10);


        verticalLayout_2->addLayout(hboxLayout);

        kcfg_HideReconciledTransactions = new QCheckBox(groupBox_4);
        kcfg_HideReconciledTransactions->setObjectName("kcfg_HideReconciledTransactions");

        verticalLayout_2->addWidget(kcfg_HideReconciledTransactions);


        verticalLayout_5->addWidget(groupBox_4);

        spacer4 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(spacer4);

        tabWidget2->addTab(TabPage, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_9 = new QVBoxLayout(tab_2);
        verticalLayout_9->setSpacing(6);
        verticalLayout_9->setContentsMargins(11, 11, 11, 11);
        verticalLayout_9->setObjectName("verticalLayout_9");
        groupBox_8 = new QGroupBox(tab_2);
        groupBox_8->setObjectName("groupBox_8");
        verticalLayout_10 = new QVBoxLayout(groupBox_8);
        verticalLayout_10->setSpacing(6);
        verticalLayout_10->setContentsMargins(11, 11, 11, 11);
        verticalLayout_10->setObjectName("verticalLayout_10");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(6);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_3 = new QLabel(groupBox_8);
        label_3->setObjectName("label_3");

        horizontalLayout_4->addWidget(label_3);

        kcfg_logPath = new QLineEdit(groupBox_8);
        kcfg_logPath->setObjectName("kcfg_logPath");
        kcfg_logPath->setReadOnly(true);

        horizontalLayout_4->addWidget(kcfg_logPath);

        choosePath = new QPushButton(groupBox_8);
        choosePath->setObjectName("choosePath");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(choosePath->sizePolicy().hasHeightForWidth());
        choosePath->setSizePolicy(sizePolicy);
        choosePath->setMaximumSize(QSize(32, 32));

        horizontalLayout_4->addWidget(choosePath);

        horizontalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_2);


        verticalLayout_10->addLayout(horizontalLayout_4);

        label_4 = new QLabel(groupBox_8);
        label_4->setObjectName("label_4");
        label_4->setWordWrap(true);

        verticalLayout_10->addWidget(label_4);

        kcfg_logImportedStatements = new QCheckBox(groupBox_8);
        kcfg_logImportedStatements->setObjectName("kcfg_logImportedStatements");

        verticalLayout_10->addWidget(kcfg_logImportedStatements);

        kcfg_logOfxTransactions = new QCheckBox(groupBox_8);
        kcfg_logOfxTransactions->setObjectName("kcfg_logOfxTransactions");

        verticalLayout_10->addWidget(kcfg_logOfxTransactions);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_10->addItem(verticalSpacer);


        verticalLayout_9->addWidget(groupBox_8);

        verticalSpacer1 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_9->addItem(verticalSpacer1);

        tabWidget2->addTab(tab_2, QString());

        vboxLayout->addWidget(tabWidget2);


        retranslateUi(KSettingsGeneral);
        QObject::connect(kcfg_AutoSaveFile, &QCheckBox::toggled, m_periodFrame, &QWidget::setEnabled);

        tabWidget2->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KSettingsGeneral);
    } // setupUi

    void retranslateUi(QWidget *KSettingsGeneral)
    {
        KSettingsGeneral->setWindowTitle(tr2i18n("General Settings", nullptr));
        groupBox_2->setTitle(tr2i18n("Autosave options", nullptr));
        kcfg_AutoSaveFile->setText(tr2i18n("Autosave periodically", nullptr));
        textLabel3->setText(tr2i18n("minutes", nullptr));
        kcfg_AutoSaveOnClose->setText(tr2i18n("Autosave when file is modified upon close", nullptr));
        groupBox_3->setTitle(tr2i18n("Backup options", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_BackupExtension->setToolTip(tr2i18n("Enter a character or string that will be added to backup files. Default is ~", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        kcfg_AutoBackupCopies->setToolTip(tr2i18n("Setup number of backups to keep (local file only)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_AutoBackupCopies->setWhatsThis(tr2i18n("Whenever the current data is saved into a local file, KMyMoney keeps the selected number of previous states of the file.\n"
"\n"
"Set it to 0 to turn the feature off.", nullptr));
#endif // QT_CONFIG(whatsthis)
        label_2->setText(tr2i18n("Extension for backup files", nullptr));
        label_5->setText(tr2i18n("Number of backups to keep (0=off)", nullptr));
        groupBox->setTitle(tr2i18n("Fiscal Year", nullptr));
        textLabel1_2_2->setText(tr2i18n("Your fiscal year starts on", nullptr));
        kcfg_FiscalYearBegin->setItemText(0, tr2i18n("January", nullptr));
        kcfg_FiscalYearBegin->setItemText(1, tr2i18n("February", nullptr));
        kcfg_FiscalYearBegin->setItemText(2, tr2i18n("March", nullptr));
        kcfg_FiscalYearBegin->setItemText(3, tr2i18n("April", nullptr));
        kcfg_FiscalYearBegin->setItemText(4, tr2i18n("May", nullptr));
        kcfg_FiscalYearBegin->setItemText(5, tr2i18n("June", nullptr));
        kcfg_FiscalYearBegin->setItemText(6, tr2i18n("July", nullptr));
        kcfg_FiscalYearBegin->setItemText(7, tr2i18n("August", "@item the month August"));
        kcfg_FiscalYearBegin->setItemText(8, tr2i18n("September", nullptr));
        kcfg_FiscalYearBegin->setItemText(9, tr2i18n("October", nullptr));
        kcfg_FiscalYearBegin->setItemText(10, tr2i18n("November", nullptr));
        kcfg_FiscalYearBegin->setItemText(11, tr2i18n("December", nullptr));

        groupBox_7->setTitle(tr2i18n("External programs", nullptr));
        label->setText(tr2i18n("Calculator", nullptr));
        tabWidget2->setTabText(tabWidget2->indexOf(tab), tr2i18n("Global", "@title global settings"));
        kcfg_StartLastViewSelected->setTitle(tr2i18n("Startup page options", nullptr));
        radioButton1->setText(tr2i18n("Start with ho&mepage", nullptr));
        radioButton2->setText(tr2i18n("Start with last selected view", nullptr));
        kcfg_viewType->setTitle(tr2i18n("Type of the KMyMoney view", nullptr));
        radioButton3->setText(tr2i18n("List", nullptr));
        radioButton4->setText(tr2i18n("Tree", nullptr));
        radioButton5->setText(tr2i18n("Tabbed", nullptr));
        kcfg_syncLedgerInvestment->setText(tr2i18n("Synchronize account selection of ledger and investment view", nullptr));
        tabWidget2->setTabText(tabWidget2->indexOf(tab2), tr2i18n("Views", nullptr));
        groupBox_6->setTitle(tr2i18n("Accounts / Categories", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_HideUnusedCategory->setWhatsThis(tr2i18n("This option hides all categories in the categories view that are not used in at least a single transaction. They are still shown in the category selection lists.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_HideUnusedCategory->setText(tr2i18n("Do not show unused categories", nullptr));
        kcfg_ExpertMode->setText(tr2i18n("Show equity accounts", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_ShowCategoriesInAccountsView->setWhatsThis(tr2i18n("This option will display the categories in the accounts view also.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_ShowCategoriesInAccountsView->setText(tr2i18n("Show categories in the accounts list view", nullptr));
        kcfg_HideZeroBalanceAccountsHome->setText(tr2i18n("Hide accounts with a zero balance from Home", nullptr));
        kcfg_HideZeroBalanceAccounts->setText(tr2i18n("Hide accounts with a zero balance from Accounts", nullptr));
        kcfg_HideZeroBalanceEquities->setText(tr2i18n("Do not show zero balance equities", nullptr));
        groupBox_5->setTitle(tr2i18n("Schedules", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_HideFinishedSchedules->setWhatsThis(tr2i18n("This option hides all finished schedules in the schedules view.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_HideFinishedSchedules->setText(tr2i18n("Do not show finished schedules", nullptr));
        groupBox_4->setTitle(tr2i18n("Transactions", nullptr));
        textLabel1_3->setText(tr2i18n("Do not show transactions prior to", nullptr));
#if QT_CONFIG(whatsthis)
        kcfg_HideReconciledTransactions->setWhatsThis(tr2i18n("This option hides all reconciled transactions in the ledger view.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_HideReconciledTransactions->setText(tr2i18n("Do not show reconciled transactions", nullptr));
        tabWidget2->setTabText(tabWidget2->indexOf(TabPage), tr2i18n("Filter", nullptr));
        groupBox_8->setTitle(tr2i18n("Logging", nullptr));
        label_3->setText(tr2i18n("Log path", nullptr));
        choosePath->setText(tr2i18n("...", nullptr));
        label_4->setText(tr2i18n("On choosing a log file path please keep in mind that log files may contain sensitive data (e.g. passwords in clear-text etc).", nullptr));
        kcfg_logImportedStatements->setText(tr2i18n("Log imported statements", nullptr));
        kcfg_logOfxTransactions->setText(tr2i18n("Log OFX transactions", nullptr));
        tabWidget2->setTabText(tabWidget2->indexOf(tab_2), tr2i18n("Support", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KSettingsGeneral: public Ui_KSettingsGeneral {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KSETTINGSGENERAL_H

