/********************************************************************************
** Form generated from reading UI file 'phototandialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PHOTOTANDIALOG_H
#define UI_PHOTOTANDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGraphicsView>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_photoTanDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QGraphicsView *graphicsView;
    QTextBrowser *infoText;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QSpacerItem *horizontalSpacer;
    QLineEdit *tanInput;
    QDialogButtonBox *buttonBox;

    void setupUi(QWidget *photoTanDialog)
    {
        if (photoTanDialog->objectName().isEmpty())
            photoTanDialog->setObjectName("photoTanDialog");
        photoTanDialog->resize(596, 420);
        verticalLayout = new QVBoxLayout(photoTanDialog);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        graphicsView = new QGraphicsView(photoTanDialog);
        graphicsView->setObjectName("graphicsView");

        gridLayout->addWidget(graphicsView, 1, 0, 1, 1);

        infoText = new QTextBrowser(photoTanDialog);
        infoText->setObjectName("infoText");
        infoText->setStyleSheet(QString::fromUtf8("background:transparent"));
        infoText->setFrameShape(QFrame::NoFrame);
        infoText->setFrameShadow(QFrame::Plain);
        infoText->setReadOnly(true);
        infoText->setAcceptRichText(false);

        gridLayout->addWidget(infoText, 0, 1, 2, 1);

        gridLayout->setColumnStretch(0, 10);

        verticalLayout->addLayout(gridLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(photoTanDialog);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        tanInput = new QLineEdit(photoTanDialog);
        tanInput->setObjectName("tanInput");

        horizontalLayout->addWidget(tanInput);


        verticalLayout->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(photoTanDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label->setBuddy(tanInput);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(tanInput, buttonBox);
        QWidget::setTabOrder(buttonBox, infoText);

        retranslateUi(photoTanDialog);

        QMetaObject::connectSlotsByName(photoTanDialog);
    } // setupUi

    void retranslateUi(QWidget *photoTanDialog)
    {
        photoTanDialog->setWindowTitle(tr2i18n("Order confirmation", nullptr));
        label->setText(tr2i18n("To confirm this order enter the tan displayed by your &generator", nullptr));
    } // retranslateUi

};

namespace Ui {
    class photoTanDialog: public Ui_photoTanDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PHOTOTANDIALOG_H

