/********************************************************************************
** Form generated from reading UI file 'kcurrencyeditdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KCURRENCYEDITDLG_H
#define UI_KCURRENCYEDITDLG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KCurrencyEditDlg
{
public:
    QVBoxLayout *verticalLayout;
    QTreeWidget *m_currencyList;
    QGridLayout *gridLayout;
    QPushButton *m_removeCurrencyButton;
    QPushButton *m_addCurrencyButton;
    QPushButton *m_editCurrencyButton;
    QPushButton *m_removeUnusedCurrencyButton;
    QPushButton *m_closeButton;
    QPushButton *m_selectBaseCurrencyButton;
    QPushButton *m_newCurrencyButton;

    void setupUi(QDialog *KCurrencyEditDlg)
    {
        if (KCurrencyEditDlg->objectName().isEmpty())
            KCurrencyEditDlg->setObjectName("KCurrencyEditDlg");
        KCurrencyEditDlg->resize(400, 516);
        KCurrencyEditDlg->setSizeGripEnabled(true);
        verticalLayout = new QVBoxLayout(KCurrencyEditDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        m_currencyList = new QTreeWidget(KCurrencyEditDlg);
        m_currencyList->setObjectName("m_currencyList");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(m_currencyList->sizePolicy().hasHeightForWidth());
        m_currencyList->setSizePolicy(sizePolicy);
        m_currencyList->setContextMenuPolicy(Qt::CustomContextMenu);
        m_currencyList->setSelectionMode(QAbstractItemView::ExtendedSelection);
        m_currencyList->setRootIsDecorated(false);
        m_currencyList->setAllColumnsShowFocus(true);

        verticalLayout->addWidget(m_currencyList);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        m_removeCurrencyButton = new QPushButton(KCurrencyEditDlg);
        m_removeCurrencyButton->setObjectName("m_removeCurrencyButton");
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        m_removeCurrencyButton->setIcon(icon);

        gridLayout->addWidget(m_removeCurrencyButton, 1, 2, 1, 1);

        m_addCurrencyButton = new QPushButton(KCurrencyEditDlg);
        m_addCurrencyButton->setObjectName("m_addCurrencyButton");
        QIcon icon1;
        iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        m_addCurrencyButton->setIcon(icon1);

        gridLayout->addWidget(m_addCurrencyButton, 1, 0, 1, 1);

        m_editCurrencyButton = new QPushButton(KCurrencyEditDlg);
        m_editCurrencyButton->setObjectName("m_editCurrencyButton");

        gridLayout->addWidget(m_editCurrencyButton, 1, 3, 1, 1);

        m_removeUnusedCurrencyButton = new QPushButton(KCurrencyEditDlg);
        m_removeUnusedCurrencyButton->setObjectName("m_removeUnusedCurrencyButton");
        QIcon icon2;
        iconThemeName = QString::fromUtf8("edit-clear");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        }
        m_removeUnusedCurrencyButton->setIcon(icon2);

        gridLayout->addWidget(m_removeUnusedCurrencyButton, 2, 0, 1, 4);

        m_closeButton = new QPushButton(KCurrencyEditDlg);
        m_closeButton->setObjectName("m_closeButton");

        gridLayout->addWidget(m_closeButton, 4, 3, 1, 1);

        m_selectBaseCurrencyButton = new QPushButton(KCurrencyEditDlg);
        m_selectBaseCurrencyButton->setObjectName("m_selectBaseCurrencyButton");

        gridLayout->addWidget(m_selectBaseCurrencyButton, 4, 0, 1, 3);

        m_newCurrencyButton = new QPushButton(KCurrencyEditDlg);
        m_newCurrencyButton->setObjectName("m_newCurrencyButton");

        gridLayout->addWidget(m_newCurrencyButton, 1, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        QWidget::setTabOrder(m_currencyList, m_addCurrencyButton);
        QWidget::setTabOrder(m_addCurrencyButton, m_newCurrencyButton);
        QWidget::setTabOrder(m_newCurrencyButton, m_removeCurrencyButton);
        QWidget::setTabOrder(m_removeCurrencyButton, m_editCurrencyButton);
        QWidget::setTabOrder(m_editCurrencyButton, m_removeUnusedCurrencyButton);
        QWidget::setTabOrder(m_removeUnusedCurrencyButton, m_selectBaseCurrencyButton);
        QWidget::setTabOrder(m_selectBaseCurrencyButton, m_closeButton);

        retranslateUi(KCurrencyEditDlg);
        QObject::connect(m_closeButton, &QPushButton::clicked, KCurrencyEditDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(KCurrencyEditDlg);
    } // setupUi

    void retranslateUi(QDialog *KCurrencyEditDlg)
    {
        KCurrencyEditDlg->setWindowTitle(tr2i18n("Currencies", "@title:window"));
        QTreeWidgetItem *___qtreewidgetitem = m_currencyList->headerItem();
        ___qtreewidgetitem->setText(2, tr2i18n("Symbol", "@title:column currency symbol"));
        ___qtreewidgetitem->setText(1, tr2i18n("ISO 4217 ID", "@title:column currency ISO4217 code"));
        ___qtreewidgetitem->setText(0, tr2i18n("Name", "@title:column currency name"));
        m_removeCurrencyButton->setText(tr2i18n("Remove", "@action:button Remove currency"));
        m_addCurrencyButton->setText(tr2i18n("Add", "@action:button Add currency from existing list"));
        m_editCurrencyButton->setText(tr2i18n("Edit", "@action:button Edit currency"));
        m_removeUnusedCurrencyButton->setText(tr2i18n("Remove unused currencies", "@action:button"));
        m_closeButton->setText(tr2i18n("Close", "@action:button Close currency dialog"));
        m_selectBaseCurrencyButton->setText(tr2i18n("Select as base currency", "@action:button"));
        m_newCurrencyButton->setText(tr2i18n("New", "@action:button Create new currency"));
    } // retranslateUi

};

namespace Ui {
    class KCurrencyEditDlg: public Ui_KCurrencyEditDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KCURRENCYEDITDLG_H

