/********************************************************************************
** Form generated from reading UI file 'transactionsortoption.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TRANSACTIONSORTOPTION_H
#define UI_TRANSACTIONSORTOPTION_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_TransactionSortOption
{
public:
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout;
    QListWidget *m_availableList;
    QVBoxLayout *vboxLayout1;
    QSpacerItem *spacer3;
    QPushButton *m_addButton;
    QPushButton *m_removeButton;
    QSpacerItem *spacer4;
    QVBoxLayout *vboxLayout2;
    QListWidget *m_selectedList;
    QVBoxLayout *vboxLayout3;
    QSpacerItem *spacer5;
    QPushButton *m_upButton;
    QPushButton *m_downButton;
    QSpacerItem *spacer6;

    void setupUi(QWidget *TransactionSortOption)
    {
        if (TransactionSortOption->objectName().isEmpty())
            TransactionSortOption->setObjectName("TransactionSortOption");
        TransactionSortOption->resize(486, 230);
        hboxLayout = new QHBoxLayout(TransactionSortOption);
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName("hboxLayout");
        vboxLayout = new QVBoxLayout();
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        m_availableList = new QListWidget(TransactionSortOption);
        m_availableList->setObjectName("m_availableList");

        vboxLayout->addWidget(m_availableList);


        hboxLayout->addLayout(vboxLayout);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName("vboxLayout1");
        spacer3 = new QSpacerItem(20, 67, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacer3);

        m_addButton = new QPushButton(TransactionSortOption);
        m_addButton->setObjectName("m_addButton");

        vboxLayout1->addWidget(m_addButton);

        m_removeButton = new QPushButton(TransactionSortOption);
        m_removeButton->setObjectName("m_removeButton");

        vboxLayout1->addWidget(m_removeButton);

        spacer4 = new QSpacerItem(21, 67, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacer4);


        hboxLayout->addLayout(vboxLayout1);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(6);
        vboxLayout2->setObjectName("vboxLayout2");
        m_selectedList = new QListWidget(TransactionSortOption);
        m_selectedList->setObjectName("m_selectedList");

        vboxLayout2->addWidget(m_selectedList);


        hboxLayout->addLayout(vboxLayout2);

        vboxLayout3 = new QVBoxLayout();
        vboxLayout3->setSpacing(6);
        vboxLayout3->setObjectName("vboxLayout3");
        spacer5 = new QSpacerItem(20, 67, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout3->addItem(spacer5);

        m_upButton = new QPushButton(TransactionSortOption);
        m_upButton->setObjectName("m_upButton");

        vboxLayout3->addWidget(m_upButton);

        m_downButton = new QPushButton(TransactionSortOption);
        m_downButton->setObjectName("m_downButton");

        vboxLayout3->addWidget(m_downButton);

        spacer6 = new QSpacerItem(20, 67, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout3->addItem(spacer6);


        hboxLayout->addLayout(vboxLayout3);


        retranslateUi(TransactionSortOption);

        QMetaObject::connectSlotsByName(TransactionSortOption);
    } // setupUi

    void retranslateUi(QWidget *TransactionSortOption)
    {
        m_addButton->setText(QString());
        m_removeButton->setText(QString());
        m_upButton->setText(QString());
        m_downButton->setText(QString());
        (void)TransactionSortOption;
    } // retranslateUi

};

namespace Ui {
    class TransactionSortOption: public Ui_TransactionSortOption {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TRANSACTIONSORTOPTION_H

