/****************************************************************************
** Meta object code from reading C++ file 'ledgerview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kmymoney-5.2.2/kmymoney/views/ledgerview.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ledgerview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10LedgerViewE_t {};
} // unnamed namespace

template <> constexpr inline auto LedgerView::qt_create_metaobjectdata<qt_meta_tag_ZN10LedgerViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LedgerView",
        "requestCustomContextMenu",
        "",
        "eMenu::Menu",
        "type",
        "QPoint",
        "pos",
        "transactionSelectionChanged",
        "SelectedObjects",
        "selection",
        "transactionSelected",
        "QModelIndex",
        "idx",
        "aboutToStartEdit",
        "aboutToFinishEdit",
        "sectionResized",
        "QWidget*",
        "view",
        "configGroupName",
        "section",
        "oldSize",
        "newSize",
        "sectionMoved",
        "oldIndex",
        "newIndex",
        "requestView",
        "viewWidget",
        "accountId",
        "journalEntryId",
        "settingsChanged",
        "sortOrderChanged",
        "QList<int>",
        "sortOrder",
        "modifySortOrder",
        "editTransaction",
        "id",
        "isEditing",
        "ensureCurrentItemIsVisible",
        "edit",
        "index",
        "slotSettingsChanged",
        "resizeSection",
        "moveSection",
        "reset",
        "setSelectedJournalEntries",
        "journalEntryIds",
        "selectionChanged",
        "QItemSelection",
        "selected",
        "deselected",
        "closeEditor",
        "editor",
        "QAbstractItemDelegate::EndEditHint",
        "hint",
        "transactionEditorDestroyed",
        "currentChanged",
        "current",
        "previous",
        "resizeEditorRow",
        "adjustDetailColumn",
        "newViewportWidth",
        "informOtherViews",
        "slotMoveToAccount",
        "reselectAfterModelReset"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestCustomContextMenu'
        QtMocHelpers::SignalData<void(eMenu::Menu, const QPoint &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'transactionSelectionChanged'
        QtMocHelpers::SignalData<void(const SelectedObjects &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'transactionSelected'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'aboutToStartEdit'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aboutToFinishEdit'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sectionResized'
        QtMocHelpers::SignalData<void(QWidget *, const QString &, int, int, int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::QString, 18 }, { QMetaType::Int, 19 }, { QMetaType::Int, 20 },
            { QMetaType::Int, 21 },
        }}),
        // Signal 'sectionMoved'
        QtMocHelpers::SignalData<void(QWidget *, int, int, int)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::Int, 19 }, { QMetaType::Int, 23 }, { QMetaType::Int, 24 },
        }}),
        // Signal 'requestView'
        QtMocHelpers::SignalData<void(QWidget *, const QString &, const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 26 }, { QMetaType::QString, 27 }, { QMetaType::QString, 28 },
        }}),
        // Signal 'settingsChanged'
        QtMocHelpers::SignalData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sortOrderChanged'
        QtMocHelpers::SignalData<void(QList<int>)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Signal 'modifySortOrder'
        QtMocHelpers::SignalData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'editTransaction'
        QtMocHelpers::SignalData<void(void *, bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::VoidStar, 35 }, { QMetaType::Bool, 36 },
        }}),
        // Slot 'ensureCurrentItemIsVisible'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'edit'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 39 },
        }}),
        // Slot 'slotSettingsChanged'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resizeSection'
        QtMocHelpers::SlotData<void(QWidget *, const QString &, int, int, int)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::QString, 18 }, { QMetaType::Int, 19 }, { QMetaType::Int, 20 },
            { QMetaType::Int, 21 },
        }}),
        // Slot 'moveSection'
        QtMocHelpers::SlotData<void(QWidget *, int, int, int)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::Int, 19 }, { QMetaType::Int, 23 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'reset'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSelectedJournalEntries'
        QtMocHelpers::SlotData<void(const QStringList &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 45 },
        }}),
        // Slot 'selectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(46, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 47, 48 }, { 0x80000000 | 47, 49 },
        }}),
        // Slot 'closeEditor'
        QtMocHelpers::SlotData<void(QWidget *, QAbstractItemDelegate::EndEditHint)>(50, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 16, 51 }, { 0x80000000 | 52, 53 },
        }}),
        // Slot 'transactionEditorDestroyed'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'currentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(55, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 11, 56 }, { 0x80000000 | 11, 57 },
        }}),
        // Slot 'resizeEditorRow'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'adjustDetailColumn'
        QtMocHelpers::SlotData<void(int, bool)>(59, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 60 }, { QMetaType::Bool, 61 },
        }}),
        // Slot 'slotMoveToAccount'
        QtMocHelpers::SlotData<void(const QString &)>(62, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 27 },
        }}),
        // Slot 'reselectAfterModelReset'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LedgerView, qt_meta_tag_ZN10LedgerViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LedgerView::staticMetaObject = { {
    QMetaObject::SuperData::link<QTableView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10LedgerViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10LedgerViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10LedgerViewE_t>.metaTypes,
    nullptr
} };

void LedgerView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LedgerView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestCustomContextMenu((*reinterpret_cast<std::add_pointer_t<eMenu::Menu>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 1: _t->transactionSelectionChanged((*reinterpret_cast<std::add_pointer_t<SelectedObjects>>(_a[1]))); break;
        case 2: _t->transactionSelected((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 3: _t->aboutToStartEdit(); break;
        case 4: _t->aboutToFinishEdit(); break;
        case 5: _t->sectionResized((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 6: _t->sectionMoved((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 7: _t->requestView((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 8: _t->settingsChanged(); break;
        case 9: _t->sortOrderChanged((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 10: _t->modifySortOrder(); break;
        case 11: _t->editTransaction((*reinterpret_cast<std::add_pointer_t<void*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 12: _t->ensureCurrentItemIsVisible(); break;
        case 13: _t->edit((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 14: _t->slotSettingsChanged(); break;
        case 15: _t->resizeSection((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 16: _t->moveSection((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 17: _t->reset(); break;
        case 18: _t->setSelectedJournalEntries((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 19: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 20: _t->closeEditor((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAbstractItemDelegate::EndEditHint>>(_a[2]))); break;
        case 21: _t->transactionEditorDestroyed(); break;
        case 22: _t->currentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 23: _t->resizeEditorRow(); break;
        case 24: _t->adjustDetailColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 25: _t->slotMoveToAccount((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 26: _t->reselectAfterModelReset(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LedgerView::*)(eMenu::Menu , const QPoint & )>(_a, &LedgerView::requestCustomContextMenu, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerView::*)(const SelectedObjects & )>(_a, &LedgerView::transactionSelectionChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerView::*)(const QModelIndex & )>(_a, &LedgerView::transactionSelected, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerView::*)()>(_a, &LedgerView::aboutToStartEdit, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerView::*)()>(_a, &LedgerView::aboutToFinishEdit, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerView::*)(QWidget * , const QString & , int , int , int )>(_a, &LedgerView::sectionResized, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerView::*)(QWidget * , int , int , int )>(_a, &LedgerView::sectionMoved, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerView::*)(QWidget * , const QString & , const QString & )>(_a, &LedgerView::requestView, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerView::*)()>(_a, &LedgerView::settingsChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerView::*)(QList<int> )>(_a, &LedgerView::sortOrderChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerView::*)()>(_a, &LedgerView::modifySortOrder, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (LedgerView::*)(void * , bool )>(_a, &LedgerView::editTransaction, 11))
            return;
    }
}

const QMetaObject *LedgerView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LedgerView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10LedgerViewE_t>.strings))
        return static_cast<void*>(this);
    return QTableView::qt_metacast(_clname);
}

int LedgerView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTableView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void LedgerView::requestCustomContextMenu(eMenu::Menu _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void LedgerView::transactionSelectionChanged(const SelectedObjects & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void LedgerView::transactionSelected(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void LedgerView::aboutToStartEdit()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void LedgerView::aboutToFinishEdit()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void LedgerView::sectionResized(QWidget * _t1, const QString & _t2, int _t3, int _t4, int _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 6
void LedgerView::sectionMoved(QWidget * _t1, int _t2, int _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 7
void LedgerView::requestView(QWidget * _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3);
}

// SIGNAL 8
void LedgerView::settingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void LedgerView::sortOrderChanged(QList<int> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void LedgerView::modifySortOrder()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void LedgerView::editTransaction(void * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}
QT_WARNING_POP
