/********************************************************************************
** Form generated from reading UI file 'loanattributeswizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOANATTRIBUTESWIZARDPAGE_H
#define UI_LOANATTRIBUTESWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <klocalizedstring.h>
#include "kcombobox.h"

QT_BEGIN_NAMESPACE

class Ui_LoanAttributesWizardPage
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *TextLabel1_3;
    KComboBox *m_qcomboboxInstitutions;
    QPushButton *m_qbuttonNew;
    QLabel *label;
    QLineEdit *m_accountNameEdit;
    QSpacerItem *Spacer43;

    void setupUi(QWizardPage *LoanAttributesWizardPage)
    {
        if (LoanAttributesWizardPage->objectName().isEmpty())
            LoanAttributesWizardPage->setObjectName("LoanAttributesWizardPage");
        LoanAttributesWizardPage->resize(462, 176);
        verticalLayout = new QVBoxLayout(LoanAttributesWizardPage);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        TextLabel1_3 = new QLabel(LoanAttributesWizardPage);
        TextLabel1_3->setObjectName("TextLabel1_3");
        TextLabel1_3->setWordWrap(false);

        gridLayout->addWidget(TextLabel1_3, 0, 0, 1, 1);

        m_qcomboboxInstitutions = new KComboBox(LoanAttributesWizardPage);
        m_qcomboboxInstitutions->setObjectName("m_qcomboboxInstitutions");
        m_qcomboboxInstitutions->setInsertPolicy(QComboBox::InsertAlphabetically);

        gridLayout->addWidget(m_qcomboboxInstitutions, 0, 1, 1, 1);

        m_qbuttonNew = new QPushButton(LoanAttributesWizardPage);
        m_qbuttonNew->setObjectName("m_qbuttonNew");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_qbuttonNew->sizePolicy().hasHeightForWidth());
        m_qbuttonNew->setSizePolicy(sizePolicy);

        gridLayout->addWidget(m_qbuttonNew, 0, 2, 1, 1);

        label = new QLabel(LoanAttributesWizardPage);
        label->setObjectName("label");

        gridLayout->addWidget(label, 1, 0, 1, 1);

        m_accountNameEdit = new QLineEdit(LoanAttributesWizardPage);
        m_accountNameEdit->setObjectName("m_accountNameEdit");

        gridLayout->addWidget(m_accountNameEdit, 1, 1, 1, 2);

        gridLayout->setColumnStretch(0, 1);
        gridLayout->setColumnStretch(1, 2);

        verticalLayout->addLayout(gridLayout);

        Spacer43 = new QSpacerItem(0, 28, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(Spacer43);

#if QT_CONFIG(shortcut)
        TextLabel1_3->setBuddy(m_qcomboboxInstitutions);
#endif // QT_CONFIG(shortcut)

        retranslateUi(LoanAttributesWizardPage);

        QMetaObject::connectSlotsByName(LoanAttributesWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *LoanAttributesWizardPage)
    {
        TextLabel1_3->setText(tr2i18n("Institution", nullptr));
        m_qbuttonNew->setText(tr2i18n("New...", "create new institution"));
        label->setText(tr2i18n("Name", nullptr));
        (void)LoanAttributesWizardPage;
    } // retranslateUi

};

namespace Ui {
    class LoanAttributesWizardPage: public Ui_LoanAttributesWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // LOANATTRIBUTESWIZARDPAGE_H

