/********************************************************************************
** Form generated from reading UI file 'reporttabgeneral.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REPORTTABGENERAL_H
#define UI_REPORTTABGENERAL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_ReportTabGeneral
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel6;
    KLineEdit *m_editName;
    QHBoxLayout *hboxLayout1;
    QLabel *textLabel7;
    KLineEdit *m_editComment;
    QCheckBox *m_checkCurrency;
    QCheckBox *m_checkFavorite;
    QCheckBox *m_skipZero;
    QSpacerItem *spacer15;

    void setupUi(QWidget *ReportTabGeneral)
    {
        if (ReportTabGeneral->objectName().isEmpty())
            ReportTabGeneral->setObjectName("ReportTabGeneral");
        ReportTabGeneral->resize(600, 176);
        vboxLayout = new QVBoxLayout(ReportTabGeneral);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        textLabel6 = new QLabel(ReportTabGeneral);
        textLabel6->setObjectName("textLabel6");
        textLabel6->setWordWrap(false);

        hboxLayout->addWidget(textLabel6);

        m_editName = new KLineEdit(ReportTabGeneral);
        m_editName->setObjectName("m_editName");

        hboxLayout->addWidget(m_editName);


        vboxLayout->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        textLabel7 = new QLabel(ReportTabGeneral);
        textLabel7->setObjectName("textLabel7");
        textLabel7->setWordWrap(false);

        hboxLayout1->addWidget(textLabel7);

        m_editComment = new KLineEdit(ReportTabGeneral);
        m_editComment->setObjectName("m_editComment");

        hboxLayout1->addWidget(m_editComment);


        vboxLayout->addLayout(hboxLayout1);

        m_checkCurrency = new QCheckBox(ReportTabGeneral);
        m_checkCurrency->setObjectName("m_checkCurrency");

        vboxLayout->addWidget(m_checkCurrency);

        m_checkFavorite = new QCheckBox(ReportTabGeneral);
        m_checkFavorite->setObjectName("m_checkFavorite");

        vboxLayout->addWidget(m_checkFavorite);

        m_skipZero = new QCheckBox(ReportTabGeneral);
        m_skipZero->setObjectName("m_skipZero");

        vboxLayout->addWidget(m_skipZero);

        spacer15 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer15);


        retranslateUi(ReportTabGeneral);

        QMetaObject::connectSlotsByName(ReportTabGeneral);
    } // setupUi

    void retranslateUi(QWidget *ReportTabGeneral)
    {
        ReportTabGeneral->setWindowTitle(tr2i18n("Report Tab", nullptr));
#if QT_CONFIG(tooltip)
        ReportTabGeneral->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        ReportTabGeneral->setWhatsThis(tr2i18n("<p>On this tab, you set the basic properties of this report.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel6->setText(tr2i18n("Report Name", nullptr));
#if QT_CONFIG(tooltip)
        m_editName->setToolTip(tr2i18n("<p>Choose a name for this report.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel7->setText(tr2i18n("Comment", nullptr));
#if QT_CONFIG(tooltip)
        m_editComment->setToolTip(tr2i18n("<p>Enter a comment to help you remember the details of this report.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_checkCurrency->setToolTip(tr2i18n("<p>Select this option to convert all values in the report to your base currency.</p><p>Leave it unchecked if you would like to see values in their original currency.</p><p>If currencies are not converted, then subtotals will not be shown.</p>", "Convert 'em!!"));
#endif // QT_CONFIG(tooltip)
        m_checkCurrency->setText(tr2i18n("Convert values to base currency", nullptr));
#if QT_CONFIG(tooltip)
        m_checkFavorite->setToolTip(tr2i18n("<p>Select this option to notate this report as one of your favorites.</p><p>All your favorite reports are grouped in one place on the report list for easy access.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_checkFavorite->setText(tr2i18n("Mark as a favorite report", nullptr));
#if QT_CONFIG(tooltip)
        m_skipZero->setToolTip(tr2i18n("\n"
"              <p>\n"
"              This option is for investments reports only which\n"
"              show prices instead of balances as all other reports do.\n"
"              </p>\n"
"              <p>\n"
"              Select this option to include prices only if there is an actual price for the date.\n"
"              If not, 0 will be shown. In the graph, the value will be skipped.\n"
"              </p>\n"
"              <p>\n"
"              If this option is off the last existing price is shown for a period, if\n"
"              it is on, in a table the value is '0' shown and in a chart a linear\n"
"              interpolation for the missing values will be performed.\n"
"              <br>Example:\n"
"              <br>There are prices for January and March, but there is no price for\n"
"              February.\n"
"              <ul>\n"
"              <li><b>OFF</b>: shows the price for February as the last price of\n"
"              January\n"
"              <li><b>ON</b>: in a table the value"
                        " is '0', in a chart a linear\n"
"              interpolation for the February-price will be performed\n"
"              (so it makes a kind of average-value using the January- and the\n"
"              March-price in the chart)\n"
"              </ul>\n"
"              </p>\n"
"            ", nullptr));
#endif // QT_CONFIG(tooltip)
        m_skipZero->setText(tr2i18n("Skip value if price is zero", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ReportTabGeneral: public Ui_ReportTabGeneral {};
} // namespace Ui

QT_END_NAMESPACE

#endif // REPORTTABGENERAL_H

