/********************************************************************************
** Form generated from reading UI file 'importoption.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMPORTOPTION_H
#define UI_IMPORTOPTION_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTimeEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"

QT_BEGIN_NAMESPACE

class Ui_ImportOption
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *buttonGroupBox1;
    QGridLayout *gridLayout;
    QComboBox *m_uniqueIdSource;
    QLabel *label;
    QLabel *label_3;
    QLabel *label_4;
    QCheckBox *m_invertAmount;
    QCheckBox *m_fixBuySellSignage;
    QLabel *label_2;
    QLabel *textLabel3_2;
    KComboBox *m_preferName;
    QHBoxLayout *hl2;
    KComboBox *m_timestampOffsetSign;
    QTimeEdit *m_timestampOffset;
    QSpacerItem *horizontalSpacer;
    QLabel *label_5;
    QComboBox *m_preferredPrice;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *ImportOption)
    {
        if (ImportOption->objectName().isEmpty())
            ImportOption->setObjectName("ImportOption");
        ImportOption->resize(596, 248);
        verticalLayout = new QVBoxLayout(ImportOption);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        buttonGroupBox1 = new QGroupBox(ImportOption);
        buttonGroupBox1->setObjectName("buttonGroupBox1");
        gridLayout = new QGridLayout(buttonGroupBox1);
        gridLayout->setObjectName("gridLayout");
        m_uniqueIdSource = new QComboBox(buttonGroupBox1);
        m_uniqueIdSource->addItem(QString());
        m_uniqueIdSource->addItem(QString());
        m_uniqueIdSource->setObjectName("m_uniqueIdSource");

        gridLayout->addWidget(m_uniqueIdSource, 1, 1, 1, 1);

        label = new QLabel(buttonGroupBox1);
        label->setObjectName("label");

        gridLayout->addWidget(label, 3, 0, 1, 1);

        label_3 = new QLabel(buttonGroupBox1);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 4, 0, 1, 1);

        label_4 = new QLabel(buttonGroupBox1);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 5, 0, 1, 1);

        m_invertAmount = new QCheckBox(buttonGroupBox1);
        m_invertAmount->setObjectName("m_invertAmount");

        gridLayout->addWidget(m_invertAmount, 4, 1, 1, 1);

        m_fixBuySellSignage = new QCheckBox(buttonGroupBox1);
        m_fixBuySellSignage->setObjectName("m_fixBuySellSignage");

        gridLayout->addWidget(m_fixBuySellSignage, 5, 1, 1, 1);

        label_2 = new QLabel(buttonGroupBox1);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        textLabel3_2 = new QLabel(buttonGroupBox1);
        textLabel3_2->setObjectName("textLabel3_2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textLabel3_2->sizePolicy().hasHeightForWidth());
        textLabel3_2->setSizePolicy(sizePolicy);
        textLabel3_2->setWordWrap(false);

        gridLayout->addWidget(textLabel3_2, 0, 0, 1, 1);

        m_preferName = new KComboBox(buttonGroupBox1);
        m_preferName->addItem(QString::fromUtf8("PAYEEID"));
        m_preferName->addItem(QString::fromUtf8("NAME"));
        m_preferName->addItem(QString::fromUtf8("MEMO"));
        m_preferName->setObjectName("m_preferName");

        gridLayout->addWidget(m_preferName, 0, 1, 1, 1);

        hl2 = new QHBoxLayout();
        hl2->setObjectName("hl2");
        m_timestampOffsetSign = new KComboBox(buttonGroupBox1);
        m_timestampOffsetSign->addItem(QString());
        m_timestampOffsetSign->addItem(QString());
        m_timestampOffsetSign->setObjectName("m_timestampOffsetSign");

        hl2->addWidget(m_timestampOffsetSign);

        m_timestampOffset = new QTimeEdit(buttonGroupBox1);
        m_timestampOffset->setObjectName("m_timestampOffset");

        hl2->addWidget(m_timestampOffset);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hl2->addItem(horizontalSpacer);

        hl2->setStretch(1, 1);

        gridLayout->addLayout(hl2, 3, 1, 1, 1);

        label_5 = new QLabel(buttonGroupBox1);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 2, 0, 1, 1);

        m_preferredPrice = new QComboBox(buttonGroupBox1);
        m_preferredPrice->addItem(QString());
        m_preferredPrice->addItem(QString());
        m_preferredPrice->setObjectName("m_preferredPrice");

        gridLayout->addWidget(m_preferredPrice, 2, 1, 1, 1);


        verticalLayout->addWidget(buttonGroupBox1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(ImportOption);

        QMetaObject::connectSlotsByName(ImportOption);
    } // setupUi

    void retranslateUi(QWidget *ImportOption)
    {
        buttonGroupBox1->setTitle(tr2i18n("Import options", nullptr));
        m_uniqueIdSource->setItemText(0, tr2i18n("OFX FITID", nullptr));
        m_uniqueIdSource->setItemText(1, tr2i18n("KMyMoney Hash", nullptr));

        label->setText(tr2i18n("Timezone offset", nullptr));
        label_3->setText(tr2i18n("Invert sign of transaction amount", nullptr));
        label_4->setText(tr2i18n("Fix sign of the investment transaction amount and quantity", nullptr));
#if QT_CONFIG(tooltip)
        m_invertAmount->setToolTip(tr2i18n("<html><head/><body><p>Check this if you need to invert the sign of imported transaction amounts, e.g. if you need to reverse charges and payments because your institution provides them inverted.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_invertAmount->setText(QString());
#if QT_CONFIG(tooltip)
        m_fixBuySellSignage->setToolTip(tr2i18n("<html><head/><body><p>Check this if the investment transactions after importing have incorrect Buy/Sell type assigned, which may be caused by your institution not applying the correct negative/positive sign to the share amount or quantity value.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_fixBuySellSignage->setText(QString());
        label_2->setText(tr2i18n("Method to detect duplicate transactions during import", nullptr));
        textLabel3_2->setText(tr2i18n("Payee's name is based on contents of the OFX tag", nullptr));

        m_timestampOffsetSign->setItemText(0, tr2i18n("+", nullptr));
        m_timestampOffsetSign->setItemText(1, tr2i18n("-", nullptr));

        label_5->setText(tr2i18n("Price is derived from", nullptr));
        m_preferredPrice->setItemText(0, tr2i18n("Price information", nullptr));
        m_preferredPrice->setItemText(1, tr2i18n("Value and number of shares", nullptr));

        (void)ImportOption;
    } // retranslateUi

};

namespace Ui {
    class ImportOption: public Ui_ImportOption {};
} // namespace Ui

QT_END_NAMESPACE

#endif // IMPORTOPTION_H

