/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kmymoney.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef KMYMONEYADAPTOR_H
#define KMYMONEYADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "kmymoney.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.kde.kmymoney
 */
class KmymoneyAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.kmymoney")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.kde.kmymoney\">\n"
"    <method name=\"filename\">\n"
"      <arg direction=\"out\" type=\"s\"/>\n"
"    </method>\n"
"    <method name=\"webConnect\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"url\"/>\n"
"      <arg direction=\"in\" type=\"ay\" name=\"asn_id\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    KmymoneyAdaptor(KMyMoneyApp *parent);
    ~KmymoneyAdaptor() override;

    inline KMyMoneyApp *parent() const
    { return static_cast<KMyMoneyApp *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    QString filename();
    void webConnect(const QString &url, const QByteArray &asn_id);
Q_SIGNALS: // SIGNALS
};

#endif
