/********************************************************************************
** Form generated from reading UI file 'konlinejoboutboxview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KONLINEJOBOUTBOXVIEW_H
#define UI_KONLINEJOBOUTBOXVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kmmemptyview.h"
#include "kmmsearchwidget.h"

QT_BEGIN_NAMESPACE

class Ui_KOnlineJobOutboxView
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout;
    KMMEmptyTreeView *m_onlineJobView;
    KMMSearchWidget *m_searchWidget;
    QToolButton *m_buttonSend;
    QToolButton *m_buttonNewCreditTransfer;
    QToolButton *m_buttonEdit;
    QToolButton *m_buttonRemove;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KOnlineJobOutboxView)
    {
        if (KOnlineJobOutboxView->objectName().isEmpty())
            KOnlineJobOutboxView->setObjectName("KOnlineJobOutboxView");
        KOnlineJobOutboxView->resize(483, 374);
        gridLayout = new QGridLayout(KOnlineJobOutboxView);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        m_onlineJobView = new KMMEmptyTreeView(KOnlineJobOutboxView);
        m_onlineJobView->setObjectName("m_onlineJobView");
        m_onlineJobView->setAlternatingRowColors(true);
        m_onlineJobView->setSelectionMode(QAbstractItemView::ExtendedSelection);
        m_onlineJobView->setRootIsDecorated(false);
        m_onlineJobView->setItemsExpandable(false);
        m_onlineJobView->setExpandsOnDoubleClick(false);

        verticalLayout->addWidget(m_onlineJobView);

        m_searchWidget = new KMMSearchWidget(KOnlineJobOutboxView);
        m_searchWidget->setObjectName("m_searchWidget");

        verticalLayout->addWidget(m_searchWidget);


        gridLayout->addLayout(verticalLayout, 0, 0, 5, 1);

        m_buttonSend = new QToolButton(KOnlineJobOutboxView);
        m_buttonSend->setObjectName("m_buttonSend");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_buttonSend->sizePolicy().hasHeightForWidth());
        m_buttonSend->setSizePolicy(sizePolicy);
        m_buttonSend->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        gridLayout->addWidget(m_buttonSend, 0, 1, 1, 1);

        m_buttonNewCreditTransfer = new QToolButton(KOnlineJobOutboxView);
        m_buttonNewCreditTransfer->setObjectName("m_buttonNewCreditTransfer");
        m_buttonNewCreditTransfer->setEnabled(false);
        sizePolicy.setHeightForWidth(m_buttonNewCreditTransfer->sizePolicy().hasHeightForWidth());
        m_buttonNewCreditTransfer->setSizePolicy(sizePolicy);
        m_buttonNewCreditTransfer->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        gridLayout->addWidget(m_buttonNewCreditTransfer, 1, 1, 1, 1);

        m_buttonEdit = new QToolButton(KOnlineJobOutboxView);
        m_buttonEdit->setObjectName("m_buttonEdit");
        sizePolicy.setHeightForWidth(m_buttonEdit->sizePolicy().hasHeightForWidth());
        m_buttonEdit->setSizePolicy(sizePolicy);
        m_buttonEdit->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        gridLayout->addWidget(m_buttonEdit, 2, 1, 1, 1);

        m_buttonRemove = new QToolButton(KOnlineJobOutboxView);
        m_buttonRemove->setObjectName("m_buttonRemove");
        sizePolicy.setHeightForWidth(m_buttonRemove->sizePolicy().hasHeightForWidth());
        m_buttonRemove->setSizePolicy(sizePolicy);
        m_buttonRemove->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        gridLayout->addWidget(m_buttonRemove, 3, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 208, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 4, 1, 1, 1);

        gridLayout->setColumnStretch(0, 2);

        retranslateUi(KOnlineJobOutboxView);

        QMetaObject::connectSlotsByName(KOnlineJobOutboxView);
    } // setupUi

    void retranslateUi(QWidget *KOnlineJobOutboxView)
    {
        m_buttonSend->setText(tr2i18n("Send", nullptr));
        m_buttonNewCreditTransfer->setText(tr2i18n("New credit transfer", nullptr));
        m_buttonEdit->setText(tr2i18n("Edit", nullptr));
        m_buttonRemove->setText(tr2i18n("Remove", nullptr));
        (void)KOnlineJobOutboxView;
    } // retranslateUi

};

namespace Ui {
    class KOnlineJobOutboxView: public Ui_KOnlineJobOutboxView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KONLINEJOBOUTBOXVIEW_H

