/********************************************************************************
** Form generated from reading UI file 'kinvestmentview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KINVESTMENTVIEW_H
#define UI_KINVESTMENTVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "kmmemptyview.h"
#include "kmymoneyaccountcombo.h"

QT_BEGIN_NAMESPACE

class Ui_KInvestmentView
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *m_tab;
    QWidget *m_equitiesTab;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel1;
    KMyMoneyAccountCombo *m_accountComboBox;
    QToolButton *m_ledgerButton;
    QSpacerItem *spacer1;
    QHBoxLayout *hboxLayout1;
    KMMEmptyTreeView *m_equitiesTree;
    QWidget *m_securitiesTab;
    QHBoxLayout *hboxLayout2;
    QVBoxLayout *m_securitiesLayout;
    QLineEdit *m_searchSecurities;
    KMMEmptyTreeView *m_securitiesTree;

    void setupUi(QWidget *KInvestmentView)
    {
        if (KInvestmentView->objectName().isEmpty())
            KInvestmentView->setObjectName("KInvestmentView");
        KInvestmentView->resize(439, 274);
        vboxLayout = new QVBoxLayout(KInvestmentView);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName("vboxLayout");
        m_tab = new QTabWidget(KInvestmentView);
        m_tab->setObjectName("m_tab");
        m_equitiesTab = new QWidget();
        m_equitiesTab->setObjectName("m_equitiesTab");
        vboxLayout1 = new QVBoxLayout(m_equitiesTab);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        vboxLayout1->setObjectName("vboxLayout1");
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        textLabel1 = new QLabel(m_equitiesTab);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        hboxLayout->addWidget(textLabel1);

        m_accountComboBox = new KMyMoneyAccountCombo(m_equitiesTab);
        m_accountComboBox->setObjectName("m_accountComboBox");

        hboxLayout->addWidget(m_accountComboBox);

        m_ledgerButton = new QToolButton(m_equitiesTab);
        m_ledgerButton->setObjectName("m_ledgerButton");

        hboxLayout->addWidget(m_ledgerButton);

        spacer1 = new QSpacerItem(280, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer1);

        hboxLayout->setStretch(0, 70);
        hboxLayout->setStretch(1, 250);
        hboxLayout->setStretch(3, 300);

        vboxLayout1->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        m_equitiesTree = new KMMEmptyTreeView(m_equitiesTab);
        m_equitiesTree->setObjectName("m_equitiesTree");
        m_equitiesTree->setContextMenuPolicy(Qt::CustomContextMenu);
        m_equitiesTree->setEditTriggers(QAbstractItemView::DoubleClicked);
        m_equitiesTree->setProperty("showDropIndicator", QVariant(false));
        m_equitiesTree->setAlternatingRowColors(true);
        m_equitiesTree->setIndentation(0);
        m_equitiesTree->setRootIsDecorated(false);
        m_equitiesTree->setItemsExpandable(false);
        m_equitiesTree->setSortingEnabled(true);
        m_equitiesTree->setAllColumnsShowFocus(true);

        hboxLayout1->addWidget(m_equitiesTree);


        vboxLayout1->addLayout(hboxLayout1);

        m_tab->addTab(m_equitiesTab, QString());
        m_securitiesTab = new QWidget();
        m_securitiesTab->setObjectName("m_securitiesTab");
        hboxLayout2 = new QHBoxLayout(m_securitiesTab);
        hboxLayout2->setSpacing(6);
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName("hboxLayout2");
        m_securitiesLayout = new QVBoxLayout();
        m_securitiesLayout->setSpacing(6);
        m_securitiesLayout->setObjectName("m_securitiesLayout");
        m_searchSecurities = new QLineEdit(m_securitiesTab);
        m_searchSecurities->setObjectName("m_searchSecurities");
        m_searchSecurities->setClearButtonEnabled(true);

        m_securitiesLayout->addWidget(m_searchSecurities);

        m_securitiesTree = new KMMEmptyTreeView(m_securitiesTab);
        m_securitiesTree->setObjectName("m_securitiesTree");
        m_securitiesTree->setContextMenuPolicy(Qt::CustomContextMenu);
        m_securitiesTree->setEditTriggers(QAbstractItemView::DoubleClicked);
        m_securitiesTree->setProperty("showDropIndicator", QVariant(false));
        m_securitiesTree->setAlternatingRowColors(true);
        m_securitiesTree->setIndentation(0);
        m_securitiesTree->setRootIsDecorated(false);
        m_securitiesTree->setItemsExpandable(false);
        m_securitiesTree->setSortingEnabled(true);
        m_securitiesTree->setAllColumnsShowFocus(true);
        m_securitiesTree->setExpandsOnDoubleClick(false);

        m_securitiesLayout->addWidget(m_securitiesTree);


        hboxLayout2->addLayout(m_securitiesLayout);

        m_tab->addTab(m_securitiesTab, QString());

        vboxLayout->addWidget(m_tab);


        retranslateUi(KInvestmentView);

        m_tab->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KInvestmentView);
    } // setupUi

    void retranslateUi(QWidget *KInvestmentView)
    {
        KInvestmentView->setWindowTitle(tr2i18n("Investment Summary", nullptr));
        textLabel1->setText(tr2i18n("Select investment account:", nullptr));
        m_ledgerButton->setText(tr2i18n("Goto ledger", nullptr));
#if QT_CONFIG(tooltip)
        m_equitiesTree->setToolTip(tr2i18n("Summary of the equities contained in this account, showing your holdings and their most recent price.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_tab->setTabText(m_tab->indexOf(m_equitiesTab), tr2i18n("Equities", nullptr));
        m_searchSecurities->setPlaceholderText(tr2i18n("Search...", nullptr));
        m_tab->setTabText(m_tab->indexOf(m_securitiesTab), tr2i18n("Securities", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KInvestmentView: public Ui_KInvestmentView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KINVESTMENTVIEW_H

