/********************************************************************************
** Form generated from reading UI file 'reporttabrange.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REPORTTABRANGE_H
#define UI_REPORTTABRANGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"

QT_BEGIN_NAMESPACE

class Ui_ReportTabRange
{
public:
    QHBoxLayout *horizontalLayout;
    QGroupBox *dateRangeBox;
    QGridLayout *dateRangeGrid;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_6;
    KComboBox *m_comboColumns;
    QSpacerItem *verticalSpacer;
    QGroupBox *dataRangeBox;
    QGridLayout *dataRangeLayout;
    QLabel *lblDataRangeStart;
    QLabel *lblDataMajorTick;
    QLineEdit *m_dataMinorTick;
    QSpacerItem *verticalSpacer_2;
    QLineEdit *m_dataRangeEnd;
    QLabel *label;
    QComboBox *m_dataLock;
    QLabel *lblDataMinorTick;
    QLineEdit *m_dataMajorTick;
    QLabel *lblDataRangeEnd;
    QLineEdit *m_dataRangeStart;
    QSpinBox *m_yLabelsPrecision;
    QLabel *lblLabelsPrecision;

    void setupUi(QWidget *ReportTabRange)
    {
        if (ReportTabRange->objectName().isEmpty())
            ReportTabRange->setObjectName("ReportTabRange");
        ReportTabRange->resize(600, 288);
        horizontalLayout = new QHBoxLayout(ReportTabRange);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setContentsMargins(11, 11, 11, 11);
        horizontalLayout->setObjectName("horizontalLayout");
        dateRangeBox = new QGroupBox(ReportTabRange);
        dateRangeBox->setObjectName("dateRangeBox");
        dateRangeGrid = new QGridLayout(dateRangeBox);
        dateRangeGrid->setSpacing(6);
        dateRangeGrid->setContentsMargins(11, 11, 11, 11);
        dateRangeGrid->setObjectName("dateRangeGrid");
        dateRangeGrid->setVerticalSpacing(0);
        dateRangeGrid->setContentsMargins(-1, 0, -1, -1);
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_3->setContentsMargins(6, 0, 6, 0);
        label_6 = new QLabel(dateRangeBox);
        label_6->setObjectName("label_6");

        horizontalLayout_3->addWidget(label_6);

        m_comboColumns = new KComboBox(dateRangeBox);
        m_comboColumns->addItem(QString());
        m_comboColumns->addItem(QString());
        m_comboColumns->addItem(QString());
        m_comboColumns->addItem(QString());
        m_comboColumns->addItem(QString());
        m_comboColumns->addItem(QString());
        m_comboColumns->setObjectName("m_comboColumns");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_comboColumns->sizePolicy().hasHeightForWidth());
        m_comboColumns->setSizePolicy(sizePolicy);

        horizontalLayout_3->addWidget(m_comboColumns);


        dateRangeGrid->addLayout(horizontalLayout_3, 1, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        dateRangeGrid->addItem(verticalSpacer, 2, 0, 1, 1);


        horizontalLayout->addWidget(dateRangeBox);

        dataRangeBox = new QGroupBox(ReportTabRange);
        dataRangeBox->setObjectName("dataRangeBox");
        dataRangeLayout = new QGridLayout(dataRangeBox);
        dataRangeLayout->setSpacing(6);
        dataRangeLayout->setContentsMargins(11, 11, 11, 11);
        dataRangeLayout->setObjectName("dataRangeLayout");
        dataRangeLayout->setContentsMargins(12, 12, -1, -1);
        lblDataRangeStart = new QLabel(dataRangeBox);
        lblDataRangeStart->setObjectName("lblDataRangeStart");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(lblDataRangeStart->sizePolicy().hasHeightForWidth());
        lblDataRangeStart->setSizePolicy(sizePolicy1);

        dataRangeLayout->addWidget(lblDataRangeStart, 3, 0, 1, 1);

        lblDataMajorTick = new QLabel(dataRangeBox);
        lblDataMajorTick->setObjectName("lblDataMajorTick");

        dataRangeLayout->addWidget(lblDataMajorTick, 5, 0, 1, 1);

        m_dataMinorTick = new QLineEdit(dataRangeBox);
        m_dataMinorTick->setObjectName("m_dataMinorTick");

        dataRangeLayout->addWidget(m_dataMinorTick, 6, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        dataRangeLayout->addItem(verticalSpacer_2, 8, 1, 1, 1);

        m_dataRangeEnd = new QLineEdit(dataRangeBox);
        m_dataRangeEnd->setObjectName("m_dataRangeEnd");

        dataRangeLayout->addWidget(m_dataRangeEnd, 4, 1, 1, 1);

        label = new QLabel(dataRangeBox);
        label->setObjectName("label");

        dataRangeLayout->addWidget(label, 0, 0, 1, 1);

        m_dataLock = new QComboBox(dataRangeBox);
        m_dataLock->addItem(QString());
        m_dataLock->addItem(QString());
        m_dataLock->setObjectName("m_dataLock");

        dataRangeLayout->addWidget(m_dataLock, 0, 1, 1, 1);

        lblDataMinorTick = new QLabel(dataRangeBox);
        lblDataMinorTick->setObjectName("lblDataMinorTick");

        dataRangeLayout->addWidget(lblDataMinorTick, 6, 0, 1, 1);

        m_dataMajorTick = new QLineEdit(dataRangeBox);
        m_dataMajorTick->setObjectName("m_dataMajorTick");

        dataRangeLayout->addWidget(m_dataMajorTick, 5, 1, 1, 1);

        lblDataRangeEnd = new QLabel(dataRangeBox);
        lblDataRangeEnd->setObjectName("lblDataRangeEnd");

        dataRangeLayout->addWidget(lblDataRangeEnd, 4, 0, 1, 1);

        m_dataRangeStart = new QLineEdit(dataRangeBox);
        m_dataRangeStart->setObjectName("m_dataRangeStart");

        dataRangeLayout->addWidget(m_dataRangeStart, 3, 1, 1, 1);

        m_yLabelsPrecision = new QSpinBox(dataRangeBox);
        m_yLabelsPrecision->setObjectName("m_yLabelsPrecision");
        m_yLabelsPrecision->setMinimum(2);
        m_yLabelsPrecision->setMaximum(10);

        dataRangeLayout->addWidget(m_yLabelsPrecision, 7, 1, 1, 1);

        lblLabelsPrecision = new QLabel(dataRangeBox);
        lblLabelsPrecision->setObjectName("lblLabelsPrecision");

        dataRangeLayout->addWidget(lblLabelsPrecision, 7, 0, 1, 1);


        horizontalLayout->addWidget(dataRangeBox);

        horizontalLayout->setStretch(0, 1);
        horizontalLayout->setStretch(1, 1);
        QWidget::setTabOrder(m_dataLock, m_dataRangeStart);
        QWidget::setTabOrder(m_dataRangeStart, m_dataRangeEnd);
        QWidget::setTabOrder(m_dataRangeEnd, m_dataMajorTick);
        QWidget::setTabOrder(m_dataMajorTick, m_dataMinorTick);
        QWidget::setTabOrder(m_dataMinorTick, m_yLabelsPrecision);
        QWidget::setTabOrder(m_yLabelsPrecision, m_comboColumns);

        retranslateUi(ReportTabRange);

        QMetaObject::connectSlotsByName(ReportTabRange);
    } // setupUi

    void retranslateUi(QWidget *ReportTabRange)
    {
        ReportTabRange->setWindowTitle(tr2i18n("Chart Tab", nullptr));
#if QT_CONFIG(tooltip)
        ReportTabRange->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        ReportTabRange->setWhatsThis(tr2i18n("<p>On this tab, you configure the chart drawn for this report.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        dateRangeBox->setTitle(tr2i18n("Date range", nullptr));
        label_6->setText(tr2i18n("Ticks", nullptr));
        m_comboColumns->setItemText(0, tr2i18n("Daily", "@item the columns will display daily data"));
        m_comboColumns->setItemText(1, tr2i18n("Weekly", "@item the columns will display weekly data"));
        m_comboColumns->setItemText(2, tr2i18n("Monthly", "@item the columns will display monthly data"));
        m_comboColumns->setItemText(3, tr2i18n("Bi-Monthly", "@item the columns will display bi-monthly data"));
        m_comboColumns->setItemText(4, tr2i18n("Quarterly", "@item the columns will display quarterly data"));
        m_comboColumns->setItemText(5, tr2i18n("Yearly", "@item the columns will display yearly data"));

#if QT_CONFIG(tooltip)
        m_comboColumns->setToolTip(tr2i18n("<p>Choose how large of a time period each column should encompass</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        dataRangeBox->setTitle(tr2i18n("Data range", nullptr));
        lblDataRangeStart->setText(tr2i18n("From", "From (Data)"));
        lblDataMajorTick->setText(tr2i18n("Major tick length", nullptr));
        label->setText(tr2i18n("Range", nullptr));
        m_dataLock->setItemText(0, tr2i18n("Automatic", nullptr));
        m_dataLock->setItemText(1, tr2i18n("User defined", nullptr));

        lblDataMinorTick->setText(tr2i18n("Minor tick length", nullptr));
        lblDataRangeEnd->setText(tr2i18n("To", "To (Data)"));
        lblLabelsPrecision->setText(tr2i18n("Labels precision", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ReportTabRange: public Ui_ReportTabRange {};
} // namespace Ui

QT_END_NAMESPACE

#endif // REPORTTABRANGE_H

