/********************************************************************************
** Form generated from reading UI file 'kcurrencyeditordlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KCURRENCYEDITORDLG_H
#define UI_KCURRENCYEDITORDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KCurrencyEditorDlg
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *lblIsoCode;
    QLabel *lblSymbol;
    QLabel *lblCashFraction;
    QLabel *lblAccountFraction;
    QLabel *lblRemainder;
    QLabel *lblpricePrecision;
    QSpinBox *spbPricePrecision;
    QLabel *lblName;
    QLineEdit *leName;
    QLineEdit *leIsoCode;
    QLineEdit *leSymbol;
    QLineEdit *leAccountFraction;
    QLineEdit *leCashFraction;
    QComboBox *comboRoundingMethod;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *KCurrencyEditorDlg)
    {
        if (KCurrencyEditorDlg->objectName().isEmpty())
            KCurrencyEditorDlg->setObjectName("KCurrencyEditorDlg");
        KCurrencyEditorDlg->resize(303, 314);
        KCurrencyEditorDlg->setSizeGripEnabled(true);
        verticalLayout = new QVBoxLayout(KCurrencyEditorDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        formLayout = new QFormLayout();
        formLayout->setSpacing(6);
        formLayout->setObjectName("formLayout");
        lblIsoCode = new QLabel(KCurrencyEditorDlg);
        lblIsoCode->setObjectName("lblIsoCode");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, lblIsoCode);

        lblSymbol = new QLabel(KCurrencyEditorDlg);
        lblSymbol->setObjectName("lblSymbol");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, lblSymbol);

        lblCashFraction = new QLabel(KCurrencyEditorDlg);
        lblCashFraction->setObjectName("lblCashFraction");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, lblCashFraction);

        lblAccountFraction = new QLabel(KCurrencyEditorDlg);
        lblAccountFraction->setObjectName("lblAccountFraction");

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, lblAccountFraction);

        lblRemainder = new QLabel(KCurrencyEditorDlg);
        lblRemainder->setObjectName("lblRemainder");

        formLayout->setWidget(5, QFormLayout::ItemRole::LabelRole, lblRemainder);

        lblpricePrecision = new QLabel(KCurrencyEditorDlg);
        lblpricePrecision->setObjectName("lblpricePrecision");

        formLayout->setWidget(6, QFormLayout::ItemRole::LabelRole, lblpricePrecision);

        spbPricePrecision = new QSpinBox(KCurrencyEditorDlg);
        spbPricePrecision->setObjectName("spbPricePrecision");
        spbPricePrecision->setMinimum(2);
        spbPricePrecision->setMaximum(20);
        spbPricePrecision->setValue(4);

        formLayout->setWidget(6, QFormLayout::ItemRole::FieldRole, spbPricePrecision);

        lblName = new QLabel(KCurrencyEditorDlg);
        lblName->setObjectName("lblName");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, lblName);

        leName = new QLineEdit(KCurrencyEditorDlg);
        leName->setObjectName("leName");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, leName);

        leIsoCode = new QLineEdit(KCurrencyEditorDlg);
        leIsoCode->setObjectName("leIsoCode");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, leIsoCode);

        leSymbol = new QLineEdit(KCurrencyEditorDlg);
        leSymbol->setObjectName("leSymbol");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, leSymbol);

        leAccountFraction = new QLineEdit(KCurrencyEditorDlg);
        leAccountFraction->setObjectName("leAccountFraction");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, leAccountFraction);

        leCashFraction = new QLineEdit(KCurrencyEditorDlg);
        leCashFraction->setObjectName("leCashFraction");

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, leCashFraction);

        comboRoundingMethod = new QComboBox(KCurrencyEditorDlg);
        comboRoundingMethod->addItem(QString());
        comboRoundingMethod->addItem(QString());
        comboRoundingMethod->addItem(QString());
        comboRoundingMethod->addItem(QString());
        comboRoundingMethod->addItem(QString());
        comboRoundingMethod->addItem(QString());
        comboRoundingMethod->addItem(QString());
        comboRoundingMethod->addItem(QString());
        comboRoundingMethod->setObjectName("comboRoundingMethod");

        formLayout->setWidget(5, QFormLayout::ItemRole::FieldRole, comboRoundingMethod);


        verticalLayout->addLayout(formLayout);

        buttonBox = new QDialogButtonBox(KCurrencyEditorDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(leIsoCode, leName);
        QWidget::setTabOrder(leName, leSymbol);
        QWidget::setTabOrder(leSymbol, leAccountFraction);
        QWidget::setTabOrder(leAccountFraction, leCashFraction);
        QWidget::setTabOrder(leCashFraction, comboRoundingMethod);
        QWidget::setTabOrder(comboRoundingMethod, spbPricePrecision);

        retranslateUi(KCurrencyEditorDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, KCurrencyEditorDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, KCurrencyEditorDlg, qOverload<>(&QDialog::reject));

        comboRoundingMethod->setCurrentIndex(7);


        QMetaObject::connectSlotsByName(KCurrencyEditorDlg);
    } // setupUi

    void retranslateUi(QDialog *KCurrencyEditorDlg)
    {
        KCurrencyEditorDlg->setWindowTitle(tr2i18n("Currency editor", nullptr));
        lblIsoCode->setText(tr2i18n("ISO Code", nullptr));
        lblSymbol->setText(tr2i18n("Symbol:", nullptr));
        lblCashFraction->setText(tr2i18n("Smallest account unit:", nullptr));
        lblAccountFraction->setText(tr2i18n("Smallest cash unit:", nullptr));
        lblRemainder->setText(tr2i18n("Rounding method", nullptr));
        lblpricePrecision->setText(tr2i18n("Price precision:", nullptr));
#if QT_CONFIG(tooltip)
        spbPricePrecision->setToolTip(tr2i18n("Enter the price precision for this currency to be used for exchange rates.", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
        lblName->setText(tr2i18n("Name", nullptr));
#if QT_CONFIG(tooltip)
        leName->setToolTip(tr2i18n("Enter the name of the currency here (e.g. Euro)", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        leIsoCode->setToolTip(tr2i18n("<html><head/><body><p>Enter the ISO4217 code for the currency here (e.g. EUR for the Euro). This cannot be changed once the currency is created.</p></body></html>", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        leSymbol->setToolTip(tr2i18n("Enter the monetary symbol of the currency here (e.g. \342\202\254 for the Euro)", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        leAccountFraction->setToolTip(tr2i18n("Enter the smallest fraction of the currency used for bank accounts here. This cannot be changed once the currency is created.", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        leCashFraction->setToolTip(tr2i18n("Enter the smallest fraction of the currency used for cash accounts here. This cannot be changed once the currency is created.", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
        comboRoundingMethod->setItemText(0, tr2i18n("Never", nullptr));
        comboRoundingMethod->setItemText(1, tr2i18n("Floor", nullptr));
        comboRoundingMethod->setItemText(2, tr2i18n("Ceil", nullptr));
        comboRoundingMethod->setItemText(3, tr2i18n("Truncate", nullptr));
        comboRoundingMethod->setItemText(4, tr2i18n("Promote", nullptr));
        comboRoundingMethod->setItemText(5, tr2i18n("Half Down", nullptr));
        comboRoundingMethod->setItemText(6, tr2i18n("Half Up", nullptr));
        comboRoundingMethod->setItemText(7, tr2i18n("Round", nullptr));

#if QT_CONFIG(tooltip)
        comboRoundingMethod->setToolTip(tr2i18n("<html><head/><body><p>Enter the rounding method used for this currency.  See <span style=\" font-weight:600;\">What's this</span> for more details.</p></body></html>", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        comboRoundingMethod->setWhatsThis(tr2i18n("<html><head/><body><p><span style=\" font-weight:600;\">RoundNever</span></p><p>Don't do any rounding, simply truncate and print a warning in case of a remainder. Otherwise the same as RoundTrunc.</p><p><span style=\" font-weight:600;\">RoundFloor</span></p><p>Round to the largest integral value not greater than the value. E.g. 0.5 -&gt; 0.0 and -0.5 -&gt; -1.0</p><p><span style=\" font-weight:600;\">RoundCeil</span></p><p>Round to the smallest integral value not less than the value. E.g. 0.5 -&gt; 1.0 and -0.5 -&gt; -0.0</p><p><span style=\" font-weight:600;\">RoundTruncate</span></p><p>No rounding, simply truncate any fraction</p><p><span style=\" font-weight:600;\">RoundPromote</span></p><p>Use RoundCeil for positive and RoundFloor for negative values. E.g. 0.5 -&gt; 1.0 and -0.5 -&gt; -1.0</p><p><span style=\" font-weight:600;\">RoundHalfDown</span></p><p>Round up or down with the following constraints: 0.1 .. 0.5 -&gt; 0.0 and 0.6 .. 0.9 -&gt; 1.0</p><p><span style=\" font-weight:600;\">RoundHalfUp</span>"
                        "</p><p>Round up or down with the following constraints: 0.1 .. 0.4 -&gt; 0.0 and 0.5 .. 0.9 -&gt; 1.0</p><p><span style=\" font-weight:600;\">RoundRound</span></p><p>Use RoundHalfDown for 0.1 .. 0.4 and RoundHalfUp for 0.6 .. 0.9. Use RoundHalfUp for 0.5 in case the resulting numerator is odd, RoundHalfDown in case the resulting numerator is even. E.g. 0.5 -&gt; 0.0 and 1.5 -&gt; 2.0</p></body></html>", "@info:whatsthis"));
#endif // QT_CONFIG(whatsthis)
    } // retranslateUi

};

namespace Ui {
    class KCurrencyEditorDlg: public Ui_KCurrencyEditorDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KCURRENCYEDITORDLG_H

