/********************************************************************************
** Form generated from reading UI file 'knewaccountdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KNEWACCOUNTDLG_H
#define UI_KNEWACCOUNTDLG_H

#include <../widgets/kmymoneyaccountselector.h>
#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <amountedit.h>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "kdatecombobox.h"
#include "klineedit.h"
#include "kmessagewidget.h"
#include "kmymoneyaccounttreeview.h"
#include "kmymoneycurrencyselector.h"
#include "kmymoneydateedit.h"
#include "kmymoneygeneralcombo.h"
#include "ktextedit.h"

QT_BEGIN_NAMESPACE

class Ui_KNewAccountDlg
{
public:
    QVBoxLayout *verticalLayout_6;
    QTabWidget *m_tab;
    QWidget *m_generalTab;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel1;
    KLineEdit *accountNameEdit;
    QGroupBox *groupBox4;
    QGridLayout *gridLayout;
    KMyMoneyCurrencySelector *m_currency;
    QLabel *m_currencyText;
    KComboBox *typeCombo;
    QLabel *textLabel2;
    QLabel *m_openingDateLabel;
    KMyMoneyDateEdit *m_openingDateEdit;
    QLabel *m_openingBalanceLabel;
    AmountEdit *m_openingBalanceEdit;
    QGroupBox *groupBox6;
    QVBoxLayout *vboxLayout1;
    QGridLayout *gridLayout1;
    QLabel *TextLabel5;
    QLabel *textLabel1_2;
    KTextEdit *descriptionEdit;
    QHBoxLayout *hboxLayout1;
    KLineEdit *m_lastCheckNumberUsed;
    QSpacerItem *spacer10_2;
    QGridLayout *gridLayout2;
    QCheckBox *m_qcheckboxNoVat;
    QCheckBox *m_costCenterRequiredCheckBox;
    QCheckBox *m_qcheckboxPreferred;
    QHBoxLayout *hboxLayout2;
    QLabel *m_priceModeLabel;
    KMyMoneyGeneralCombo *m_priceMode;
    QCheckBox *m_qcheckboxOpeningBalance;
    QSpacerItem *spacer9;
    QWidget *m_institutionTab;
    QVBoxLayout *vboxLayout2;
    QGroupBox *m_institutionBox;
    QVBoxLayout *vboxLayout3;
    QHBoxLayout *hboxLayout3;
    KComboBox *m_qcomboboxInstitutions;
    QPushButton *m_qbuttonNew;
    QGridLayout *gridLayout3;
    KLineEdit *m_urlEdit;
    QLabel *TextLabel2_2;
    QLabel *m_protocolLabel;
    QLabel *m_websiteLabel;
    QLabel *textLabel1_4;
    QLabel *textLabel3;
    QToolButton *m_iconButton;
    KLineEdit *accountNoEdit;
    KLineEdit *ibanEdit;
    QLabel *m_bicValue;
    KMessageWidget *m_messageWidget;
    QSpacerItem *spacer8_2;
    QWidget *m_hierarchyTab;
    QVBoxLayout *vboxLayout4;
    QLabel *m_subAccountLabel;
    KMyMoneyAccountTreeView *m_parentAccounts;
    QWidget *m_optionsTab;
    QVBoxLayout *verticalLayout_8;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_13;
    QGroupBox *m_limitsGroupBox;
    QVBoxLayout *verticalLayout_10;
    QLabel *qLabel1;
    QGridLayout *_2;
    QLabel *textLabel2_2;
    AmountEdit *m_minBalanceAbsoluteEdit;
    QLabel *textLabel3_2;
    QLabel *m_maxCreditLabel;
    AmountEdit *m_maxCreditEarlyEdit;
    AmountEdit *m_maxCreditAbsoluteEdit;
    AmountEdit *m_minBalanceEarlyEdit;
    QLabel *m_minBalanceLabel;
    QFrame *line2;
    QLabel *textLabel4;
    QGroupBox *m_taxGroupBox;
    QVBoxLayout *verticalLayout_11;
    QGroupBox *groupBox3;
    QHBoxLayout *horizontalLayout_2;
    QFrame *m_vatCategoryFrame;
    QVBoxLayout *verticalLayout_5;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *m_vatCategory;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QFrame *m_vatPercentageFrame;
    QVBoxLayout *verticalLayout;
    QLabel *textLabel1;
    AmountEdit *m_vatRate;
    QSpacerItem *spacer17;
    QSpacerItem *spacer6;
    QFrame *m_vatAssignmentFrame;
    QGridLayout *_3;
    QCheckBox *m_vatAssignment;
    QWidget *widget;
    QVBoxLayout *verticalLayout_3;
    KMyMoneyAccountSelector *m_vatAccount;
    QVBoxLayout *_4;
    QGroupBox *m_amountGroupBox;
    QVBoxLayout *vboxLayout5;
    QFrame *frame;
    QVBoxLayout *vboxLayout6;
    QRadioButton *m_grossAmount;
    QRadioButton *m_netAmount;
    QSpacerItem *spacer7;
    QCheckBox *m_qcheckboxTax;
    QGroupBox *m_importGroupBox;
    QVBoxLayout *verticalLayout_12;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label;
    QComboBox *m_payeeCreation;
    QSpacerItem *horizontalSpacer;
    QGroupBox *m_budgetGroupBox;
    QVBoxLayout *verticalLayout_7;
    QComboBox *m_budgetInclusion;
    QFrame *line1;
    QDialogButtonBox *buttonBox;
    QButtonGroup *m_amountGroup;

    void setupUi(QDialog *KNewAccountDlg)
    {
        if (KNewAccountDlg->objectName().isEmpty())
            KNewAccountDlg->setObjectName("KNewAccountDlg");
        KNewAccountDlg->resize(670, 569);
        KNewAccountDlg->setSizeGripEnabled(true);
        verticalLayout_6 = new QVBoxLayout(KNewAccountDlg);
        verticalLayout_6->setSpacing(6);
        verticalLayout_6->setContentsMargins(11, 11, 11, 11);
        verticalLayout_6->setObjectName("verticalLayout_6");
        m_tab = new QTabWidget(KNewAccountDlg);
        m_tab->setObjectName("m_tab");
        m_generalTab = new QWidget();
        m_generalTab->setObjectName("m_generalTab");
        vboxLayout = new QVBoxLayout(m_generalTab);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        TextLabel1 = new QLabel(m_generalTab);
        TextLabel1->setObjectName("TextLabel1");
        TextLabel1->setMinimumSize(QSize(60, 0));
        TextLabel1->setMaximumSize(QSize(60, 32767));
        TextLabel1->setWordWrap(false);

        hboxLayout->addWidget(TextLabel1);

        accountNameEdit = new KLineEdit(m_generalTab);
        accountNameEdit->setObjectName("accountNameEdit");

        hboxLayout->addWidget(accountNameEdit);


        vboxLayout->addLayout(hboxLayout);

        groupBox4 = new QGroupBox(m_generalTab);
        groupBox4->setObjectName("groupBox4");
        gridLayout = new QGridLayout(groupBox4);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(6, 6, 6, 6);
        m_currency = new KMyMoneyCurrencySelector(groupBox4);
        m_currency->setObjectName("m_currency");

        gridLayout->addWidget(m_currency, 1, 1, 1, 1);

        m_currencyText = new QLabel(groupBox4);
        m_currencyText->setObjectName("m_currencyText");
        m_currencyText->setWordWrap(false);

        gridLayout->addWidget(m_currencyText, 1, 0, 1, 1);

        typeCombo = new KComboBox(groupBox4);
        typeCombo->setObjectName("typeCombo");
        typeCombo->setEnabled(true);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(typeCombo->sizePolicy().hasHeightForWidth());
        typeCombo->setSizePolicy(sizePolicy);

        gridLayout->addWidget(typeCombo, 0, 1, 1, 1);

        textLabel2 = new QLabel(groupBox4);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 0, 0, 1, 1);

        m_openingDateLabel = new QLabel(groupBox4);
        m_openingDateLabel->setObjectName("m_openingDateLabel");
        m_openingDateLabel->setMinimumSize(QSize(0, 0));
        m_openingDateLabel->setWordWrap(false);

        gridLayout->addWidget(m_openingDateLabel, 0, 2, 1, 1);

        m_openingDateEdit = new KMyMoneyDateEdit(groupBox4);
        m_openingDateEdit->setObjectName("m_openingDateEdit");

        gridLayout->addWidget(m_openingDateEdit, 0, 3, 1, 1);

        m_openingBalanceLabel = new QLabel(groupBox4);
        m_openingBalanceLabel->setObjectName("m_openingBalanceLabel");
        m_openingBalanceLabel->setWordWrap(false);

        gridLayout->addWidget(m_openingBalanceLabel, 1, 2, 1, 1);

        m_openingBalanceEdit = new AmountEdit(groupBox4);
        m_openingBalanceEdit->setObjectName("m_openingBalanceEdit");

        gridLayout->addWidget(m_openingBalanceEdit, 1, 3, 1, 1);


        vboxLayout->addWidget(groupBox4);

        groupBox6 = new QGroupBox(m_generalTab);
        groupBox6->setObjectName("groupBox6");
        vboxLayout1 = new QVBoxLayout(groupBox6);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setObjectName("gridLayout1");
        TextLabel5 = new QLabel(groupBox6);
        TextLabel5->setObjectName("TextLabel5");
        TextLabel5->setMinimumSize(QSize(0, 0));
        TextLabel5->setMaximumSize(QSize(32767, 32767));
        TextLabel5->setAlignment(Qt::AlignTop);
        TextLabel5->setWordWrap(false);

        gridLayout1->addWidget(TextLabel5, 1, 0, 1, 1);

        textLabel1_2 = new QLabel(groupBox6);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        gridLayout1->addWidget(textLabel1_2, 0, 0, 1, 1);

        descriptionEdit = new KTextEdit(groupBox6);
        descriptionEdit->setObjectName("descriptionEdit");

        gridLayout1->addWidget(descriptionEdit, 1, 1, 1, 2);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        m_lastCheckNumberUsed = new KLineEdit(groupBox6);
        m_lastCheckNumberUsed->setObjectName("m_lastCheckNumberUsed");

        hboxLayout1->addWidget(m_lastCheckNumberUsed);

        spacer10_2 = new QSpacerItem(140, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacer10_2);


        gridLayout1->addLayout(hboxLayout1, 0, 1, 1, 2);


        vboxLayout1->addLayout(gridLayout1);

        gridLayout2 = new QGridLayout();
        gridLayout2->setSpacing(6);
        gridLayout2->setObjectName("gridLayout2");
        m_qcheckboxNoVat = new QCheckBox(groupBox6);
        m_qcheckboxNoVat->setObjectName("m_qcheckboxNoVat");

        gridLayout2->addWidget(m_qcheckboxNoVat, 1, 0, 1, 1);

        m_costCenterRequiredCheckBox = new QCheckBox(groupBox6);
        m_costCenterRequiredCheckBox->setObjectName("m_costCenterRequiredCheckBox");

        gridLayout2->addWidget(m_costCenterRequiredCheckBox, 2, 0, 1, 1);

        m_qcheckboxPreferred = new QCheckBox(groupBox6);
        m_qcheckboxPreferred->setObjectName("m_qcheckboxPreferred");

        gridLayout2->addWidget(m_qcheckboxPreferred, 0, 0, 1, 1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        m_priceModeLabel = new QLabel(groupBox6);
        m_priceModeLabel->setObjectName("m_priceModeLabel");
        m_priceModeLabel->setWordWrap(false);

        hboxLayout2->addWidget(m_priceModeLabel);

        m_priceMode = new KMyMoneyGeneralCombo(groupBox6);
        m_priceMode->setObjectName("m_priceMode");
        m_priceMode->setEnabled(false);
        sizePolicy.setHeightForWidth(m_priceMode->sizePolicy().hasHeightForWidth());
        m_priceMode->setSizePolicy(sizePolicy);

        hboxLayout2->addWidget(m_priceMode);


        gridLayout2->addLayout(hboxLayout2, 0, 1, 1, 1);

        m_qcheckboxOpeningBalance = new QCheckBox(groupBox6);
        m_qcheckboxOpeningBalance->setObjectName("m_qcheckboxOpeningBalance");
        m_qcheckboxOpeningBalance->setEnabled(true);

        gridLayout2->addWidget(m_qcheckboxOpeningBalance, 1, 1, 1, 1);


        vboxLayout1->addLayout(gridLayout2);


        vboxLayout->addWidget(groupBox6);

        spacer9 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer9);

        m_tab->addTab(m_generalTab, QString());
        m_institutionTab = new QWidget();
        m_institutionTab->setObjectName("m_institutionTab");
        vboxLayout2 = new QVBoxLayout(m_institutionTab);
        vboxLayout2->setSpacing(6);
        vboxLayout2->setContentsMargins(11, 11, 11, 11);
        vboxLayout2->setObjectName("vboxLayout2");
        m_institutionBox = new QGroupBox(m_institutionTab);
        m_institutionBox->setObjectName("m_institutionBox");
        vboxLayout3 = new QVBoxLayout(m_institutionBox);
        vboxLayout3->setSpacing(6);
        vboxLayout3->setContentsMargins(11, 11, 11, 11);
        vboxLayout3->setObjectName("vboxLayout3");
        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setSpacing(6);
        hboxLayout3->setObjectName("hboxLayout3");
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        m_qcomboboxInstitutions = new KComboBox(m_institutionBox);
        m_qcomboboxInstitutions->setObjectName("m_qcomboboxInstitutions");

        hboxLayout3->addWidget(m_qcomboboxInstitutions);

        m_qbuttonNew = new QPushButton(m_institutionBox);
        m_qbuttonNew->setObjectName("m_qbuttonNew");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_qbuttonNew->sizePolicy().hasHeightForWidth());
        m_qbuttonNew->setSizePolicy(sizePolicy1);

        hboxLayout3->addWidget(m_qbuttonNew);


        vboxLayout3->addLayout(hboxLayout3);

        gridLayout3 = new QGridLayout();
        gridLayout3->setSpacing(6);
        gridLayout3->setObjectName("gridLayout3");
        m_urlEdit = new KLineEdit(m_institutionBox);
        m_urlEdit->setObjectName("m_urlEdit");

        gridLayout3->addWidget(m_urlEdit, 3, 2, 1, 1);

        TextLabel2_2 = new QLabel(m_institutionBox);
        TextLabel2_2->setObjectName("TextLabel2_2");
        TextLabel2_2->setWordWrap(false);

        gridLayout3->addWidget(TextLabel2_2, 2, 0, 1, 1);

        m_protocolLabel = new QLabel(m_institutionBox);
        m_protocolLabel->setObjectName("m_protocolLabel");
        m_protocolLabel->setText(QString::fromUtf8("https://"));

        gridLayout3->addWidget(m_protocolLabel, 3, 1, 1, 1);

        m_websiteLabel = new QLabel(m_institutionBox);
        m_websiteLabel->setObjectName("m_websiteLabel");

        gridLayout3->addWidget(m_websiteLabel, 3, 0, 1, 1);

        textLabel1_4 = new QLabel(m_institutionBox);
        textLabel1_4->setObjectName("textLabel1_4");
        textLabel1_4->setWordWrap(false);

        gridLayout3->addWidget(textLabel1_4, 0, 0, 1, 1);

        textLabel3 = new QLabel(m_institutionBox);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout3->addWidget(textLabel3, 1, 0, 1, 1);

        m_iconButton = new QToolButton(m_institutionBox);
        m_iconButton->setObjectName("m_iconButton");
        m_iconButton->setEnabled(false);

        gridLayout3->addWidget(m_iconButton, 3, 3, 1, 1);

        accountNoEdit = new KLineEdit(m_institutionBox);
        accountNoEdit->setObjectName("accountNoEdit");

        gridLayout3->addWidget(accountNoEdit, 2, 1, 1, 3);

        ibanEdit = new KLineEdit(m_institutionBox);
        ibanEdit->setObjectName("ibanEdit");

        gridLayout3->addWidget(ibanEdit, 1, 1, 1, 3);

        m_bicValue = new QLabel(m_institutionBox);
        m_bicValue->setObjectName("m_bicValue");
        m_bicValue->setWordWrap(false);

        gridLayout3->addWidget(m_bicValue, 0, 1, 1, 3);


        vboxLayout3->addLayout(gridLayout3);


        vboxLayout2->addWidget(m_institutionBox);

        m_messageWidget = new KMessageWidget(m_institutionTab);
        m_messageWidget->setObjectName("m_messageWidget");

        vboxLayout2->addWidget(m_messageWidget);

        spacer8_2 = new QSpacerItem(21, 30, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout2->addItem(spacer8_2);

        m_tab->addTab(m_institutionTab, QString());
        m_hierarchyTab = new QWidget();
        m_hierarchyTab->setObjectName("m_hierarchyTab");
        vboxLayout4 = new QVBoxLayout(m_hierarchyTab);
        vboxLayout4->setSpacing(6);
        vboxLayout4->setContentsMargins(11, 11, 11, 11);
        vboxLayout4->setObjectName("vboxLayout4");
        m_subAccountLabel = new QLabel(m_hierarchyTab);
        m_subAccountLabel->setObjectName("m_subAccountLabel");
        m_subAccountLabel->setWordWrap(false);

        vboxLayout4->addWidget(m_subAccountLabel);

        m_parentAccounts = new KMyMoneyAccountTreeView(m_hierarchyTab);
        m_parentAccounts->setObjectName("m_parentAccounts");

        vboxLayout4->addWidget(m_parentAccounts);

        m_tab->addTab(m_hierarchyTab, QString());
        m_optionsTab = new QWidget();
        m_optionsTab->setObjectName("m_optionsTab");
        verticalLayout_8 = new QVBoxLayout(m_optionsTab);
        verticalLayout_8->setSpacing(6);
        verticalLayout_8->setContentsMargins(11, 11, 11, 11);
        verticalLayout_8->setObjectName("verticalLayout_8");
        scrollArea = new QScrollArea(m_optionsTab);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 613, 634));
        verticalLayout_13 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_13->setSpacing(6);
        verticalLayout_13->setContentsMargins(11, 11, 11, 11);
        verticalLayout_13->setObjectName("verticalLayout_13");
        verticalLayout_13->setContentsMargins(6, 6, 6, 6);
        m_limitsGroupBox = new QGroupBox(scrollAreaWidgetContents);
        m_limitsGroupBox->setObjectName("m_limitsGroupBox");
        verticalLayout_10 = new QVBoxLayout(m_limitsGroupBox);
        verticalLayout_10->setSpacing(6);
        verticalLayout_10->setContentsMargins(11, 11, 11, 11);
        verticalLayout_10->setObjectName("verticalLayout_10");
        verticalLayout_10->setContentsMargins(-1, -1, -1, 0);
        qLabel1 = new QLabel(m_limitsGroupBox);
        qLabel1->setObjectName("qLabel1");
        qLabel1->setWordWrap(true);

        verticalLayout_10->addWidget(qLabel1);

        _2 = new QGridLayout();
        _2->setSpacing(6);
        _2->setObjectName("_2");
        textLabel2_2 = new QLabel(m_limitsGroupBox);
        textLabel2_2->setObjectName("textLabel2_2");
        textLabel2_2->setWordWrap(false);

        _2->addWidget(textLabel2_2, 0, 0, 1, 1);

        m_minBalanceAbsoluteEdit = new AmountEdit(m_limitsGroupBox);
        m_minBalanceAbsoluteEdit->setObjectName("m_minBalanceAbsoluteEdit");
        m_minBalanceAbsoluteEdit->setProperty("resetButtonVisibility", QVariant(false));

        _2->addWidget(m_minBalanceAbsoluteEdit, 2, 2, 1, 1);

        textLabel3_2 = new QLabel(m_limitsGroupBox);
        textLabel3_2->setObjectName("textLabel3_2");
        textLabel3_2->setWordWrap(false);

        _2->addWidget(textLabel3_2, 0, 2, 1, 1);

        m_maxCreditLabel = new QLabel(m_limitsGroupBox);
        m_maxCreditLabel->setObjectName("m_maxCreditLabel");
        m_maxCreditLabel->setWordWrap(false);

        _2->addWidget(m_maxCreditLabel, 3, 0, 1, 1);

        m_maxCreditEarlyEdit = new AmountEdit(m_limitsGroupBox);
        m_maxCreditEarlyEdit->setObjectName("m_maxCreditEarlyEdit");
        m_maxCreditEarlyEdit->setProperty("resetButtonVisibility", QVariant(false));

        _2->addWidget(m_maxCreditEarlyEdit, 3, 1, 1, 1);

        m_maxCreditAbsoluteEdit = new AmountEdit(m_limitsGroupBox);
        m_maxCreditAbsoluteEdit->setObjectName("m_maxCreditAbsoluteEdit");
        m_maxCreditAbsoluteEdit->setProperty("resetButtonVisibility", QVariant(false));

        _2->addWidget(m_maxCreditAbsoluteEdit, 3, 2, 1, 1);

        m_minBalanceEarlyEdit = new AmountEdit(m_limitsGroupBox);
        m_minBalanceEarlyEdit->setObjectName("m_minBalanceEarlyEdit");
        m_minBalanceEarlyEdit->setProperty("resetButtonVisibility", QVariant(false));

        _2->addWidget(m_minBalanceEarlyEdit, 2, 1, 1, 1);

        m_minBalanceLabel = new QLabel(m_limitsGroupBox);
        m_minBalanceLabel->setObjectName("m_minBalanceLabel");
        m_minBalanceLabel->setWordWrap(false);

        _2->addWidget(m_minBalanceLabel, 2, 0, 1, 1);

        line2 = new QFrame(m_limitsGroupBox);
        line2->setObjectName("line2");
        line2->setFrameShape(QFrame::HLine);
        line2->setFrameShadow(QFrame::Sunken);

        _2->addWidget(line2, 1, 0, 1, 3);

        textLabel4 = new QLabel(m_limitsGroupBox);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        _2->addWidget(textLabel4, 0, 1, 1, 1);


        verticalLayout_10->addLayout(_2);


        verticalLayout_13->addWidget(m_limitsGroupBox);

        m_taxGroupBox = new QGroupBox(scrollAreaWidgetContents);
        m_taxGroupBox->setObjectName("m_taxGroupBox");
        verticalLayout_11 = new QVBoxLayout(m_taxGroupBox);
        verticalLayout_11->setSpacing(6);
        verticalLayout_11->setContentsMargins(11, 11, 11, 11);
        verticalLayout_11->setObjectName("verticalLayout_11");
        groupBox3 = new QGroupBox(m_taxGroupBox);
        groupBox3->setObjectName("groupBox3");
        horizontalLayout_2 = new QHBoxLayout(groupBox3);
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setContentsMargins(11, 11, 11, 11);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(-1, -1, -1, 0);
        m_vatCategoryFrame = new QFrame(groupBox3);
        m_vatCategoryFrame->setObjectName("m_vatCategoryFrame");
        m_vatCategoryFrame->setFrameShape(QFrame::NoFrame);
        m_vatCategoryFrame->setFrameShadow(QFrame::Plain);
        verticalLayout_5 = new QVBoxLayout(m_vatCategoryFrame);
        verticalLayout_5->setSpacing(6);
        verticalLayout_5->setContentsMargins(11, 11, 11, 11);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setObjectName("verticalLayout_4");
        m_vatCategory = new QCheckBox(m_vatCategoryFrame);
        m_vatCategory->setObjectName("m_vatCategory");

        verticalLayout_4->addWidget(m_vatCategory);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        m_vatPercentageFrame = new QFrame(m_vatCategoryFrame);
        m_vatPercentageFrame->setObjectName("m_vatPercentageFrame");
        m_vatPercentageFrame->setFrameShape(QFrame::NoFrame);
        m_vatPercentageFrame->setFrameShadow(QFrame::Plain);
        verticalLayout = new QVBoxLayout(m_vatPercentageFrame);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        textLabel1 = new QLabel(m_vatPercentageFrame);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        verticalLayout->addWidget(textLabel1);

        m_vatRate = new AmountEdit(m_vatPercentageFrame);
        m_vatRate->setObjectName("m_vatRate");

        verticalLayout->addWidget(m_vatRate);


        horizontalLayout->addWidget(m_vatPercentageFrame);

        spacer17 = new QSpacerItem(21, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(spacer17);


        verticalLayout_2->addLayout(horizontalLayout);

        spacer6 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(spacer6);


        verticalLayout_4->addLayout(verticalLayout_2);


        verticalLayout_5->addLayout(verticalLayout_4);


        horizontalLayout_2->addWidget(m_vatCategoryFrame);

        m_vatAssignmentFrame = new QFrame(groupBox3);
        m_vatAssignmentFrame->setObjectName("m_vatAssignmentFrame");
        m_vatAssignmentFrame->setFrameShape(QFrame::NoFrame);
        m_vatAssignmentFrame->setFrameShadow(QFrame::Plain);
        _3 = new QGridLayout(m_vatAssignmentFrame);
        _3->setSpacing(6);
        _3->setContentsMargins(11, 11, 11, 11);
        _3->setObjectName("_3");
        _3->setContentsMargins(0, 0, 0, 0);
        m_vatAssignment = new QCheckBox(m_vatAssignmentFrame);
        m_vatAssignment->setObjectName("m_vatAssignment");

        _3->addWidget(m_vatAssignment, 0, 0, 1, 1);

        widget = new QWidget(m_vatAssignmentFrame);
        widget->setObjectName("widget");
        verticalLayout_3 = new QVBoxLayout(widget);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        m_vatAccount = new KMyMoneyAccountSelector(widget);
        m_vatAccount->setObjectName("m_vatAccount");

        verticalLayout_3->addWidget(m_vatAccount);


        _3->addWidget(widget, 1, 0, 1, 1);

        _4 = new QVBoxLayout();
        _4->setSpacing(6);
        _4->setObjectName("_4");
        m_amountGroupBox = new QGroupBox(m_vatAssignmentFrame);
        m_amountGroupBox->setObjectName("m_amountGroupBox");
        m_amountGroupBox->setMinimumSize(QSize(0, 0));
        m_amountGroupBox->setMaximumSize(QSize(32767, 32767));
        vboxLayout5 = new QVBoxLayout(m_amountGroupBox);
        vboxLayout5->setSpacing(6);
        vboxLayout5->setContentsMargins(11, 11, 11, 11);
        vboxLayout5->setObjectName("vboxLayout5");
        frame = new QFrame(m_amountGroupBox);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Sunken);
        vboxLayout6 = new QVBoxLayout(frame);
        vboxLayout6->setSpacing(6);
        vboxLayout6->setContentsMargins(11, 11, 11, 11);
        vboxLayout6->setObjectName("vboxLayout6");
        m_grossAmount = new QRadioButton(frame);
        m_amountGroup = new QButtonGroup(KNewAccountDlg);
        m_amountGroup->setObjectName("m_amountGroup");
        m_amountGroup->addButton(m_grossAmount);
        m_grossAmount->setObjectName("m_grossAmount");
        m_grossAmount->setChecked(true);

        vboxLayout6->addWidget(m_grossAmount);

        m_netAmount = new QRadioButton(frame);
        m_amountGroup->addButton(m_netAmount);
        m_netAmount->setObjectName("m_netAmount");

        vboxLayout6->addWidget(m_netAmount);


        vboxLayout5->addWidget(frame);


        _4->addWidget(m_amountGroupBox);

        spacer7 = new QSpacerItem(20, 21, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        _4->addItem(spacer7);


        _3->addLayout(_4, 1, 1, 1, 1);


        horizontalLayout_2->addWidget(m_vatAssignmentFrame);


        verticalLayout_11->addWidget(groupBox3);

        m_qcheckboxTax = new QCheckBox(m_taxGroupBox);
        m_qcheckboxTax->setObjectName("m_qcheckboxTax");

        verticalLayout_11->addWidget(m_qcheckboxTax);


        verticalLayout_13->addWidget(m_taxGroupBox);

        m_importGroupBox = new QGroupBox(scrollAreaWidgetContents);
        m_importGroupBox->setObjectName("m_importGroupBox");
        verticalLayout_12 = new QVBoxLayout(m_importGroupBox);
        verticalLayout_12->setSpacing(6);
        verticalLayout_12->setContentsMargins(11, 11, 11, 11);
        verticalLayout_12->setObjectName("verticalLayout_12");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label = new QLabel(m_importGroupBox);
        label->setObjectName("label");

        horizontalLayout_3->addWidget(label);

        m_payeeCreation = new QComboBox(m_importGroupBox);
        m_payeeCreation->addItem(QString());
        m_payeeCreation->addItem(QString());
        m_payeeCreation->addItem(QString());
        m_payeeCreation->addItem(QString());
        m_payeeCreation->setObjectName("m_payeeCreation");

        horizontalLayout_3->addWidget(m_payeeCreation);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);


        verticalLayout_12->addLayout(horizontalLayout_3);


        verticalLayout_13->addWidget(m_importGroupBox);

        m_budgetGroupBox = new QGroupBox(scrollAreaWidgetContents);
        m_budgetGroupBox->setObjectName("m_budgetGroupBox");
        verticalLayout_7 = new QVBoxLayout(m_budgetGroupBox);
        verticalLayout_7->setSpacing(6);
        verticalLayout_7->setContentsMargins(11, 11, 11, 11);
        verticalLayout_7->setObjectName("verticalLayout_7");
        m_budgetInclusion = new QComboBox(m_budgetGroupBox);
        m_budgetInclusion->addItem(QString());
        m_budgetInclusion->addItem(QString());
        m_budgetInclusion->addItem(QString());
        m_budgetInclusion->setObjectName("m_budgetInclusion");

        verticalLayout_7->addWidget(m_budgetInclusion);


        verticalLayout_13->addWidget(m_budgetGroupBox);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_8->addWidget(scrollArea);

        m_tab->addTab(m_optionsTab, QString());

        verticalLayout_6->addWidget(m_tab);

        line1 = new QFrame(KNewAccountDlg);
        line1->setObjectName("line1");
        line1->setFrameShape(QFrame::HLine);
        line1->setFrameShadow(QFrame::Sunken);

        verticalLayout_6->addWidget(line1);

        buttonBox = new QDialogButtonBox(KNewAccountDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_6->addWidget(buttonBox);

        QWidget::setTabOrder(m_tab, accountNameEdit);
        QWidget::setTabOrder(accountNameEdit, typeCombo);
        QWidget::setTabOrder(typeCombo, m_openingDateEdit);
        QWidget::setTabOrder(m_openingDateEdit, m_currency);
        QWidget::setTabOrder(m_currency, m_openingBalanceEdit);
        QWidget::setTabOrder(m_openingBalanceEdit, m_lastCheckNumberUsed);
        QWidget::setTabOrder(m_lastCheckNumberUsed, descriptionEdit);
        QWidget::setTabOrder(descriptionEdit, m_qcheckboxPreferred);
        QWidget::setTabOrder(m_qcheckboxPreferred, m_qcheckboxNoVat);
        QWidget::setTabOrder(m_qcheckboxNoVat, m_qcheckboxOpeningBalance);
        QWidget::setTabOrder(m_qcheckboxOpeningBalance, m_costCenterRequiredCheckBox);
        QWidget::setTabOrder(m_costCenterRequiredCheckBox, m_qcomboboxInstitutions);
        QWidget::setTabOrder(m_qcomboboxInstitutions, m_qbuttonNew);
        QWidget::setTabOrder(m_qbuttonNew, ibanEdit);
        QWidget::setTabOrder(ibanEdit, accountNoEdit);
        QWidget::setTabOrder(accountNoEdit, m_urlEdit);
        QWidget::setTabOrder(m_urlEdit, m_iconButton);
        QWidget::setTabOrder(m_iconButton, m_parentAccounts);
        QWidget::setTabOrder(m_parentAccounts, scrollArea);
        QWidget::setTabOrder(scrollArea, m_minBalanceEarlyEdit);
        QWidget::setTabOrder(m_minBalanceEarlyEdit, m_minBalanceAbsoluteEdit);
        QWidget::setTabOrder(m_minBalanceAbsoluteEdit, m_maxCreditEarlyEdit);
        QWidget::setTabOrder(m_maxCreditEarlyEdit, m_maxCreditAbsoluteEdit);
        QWidget::setTabOrder(m_maxCreditAbsoluteEdit, m_vatCategory);
        QWidget::setTabOrder(m_vatCategory, m_vatRate);
        QWidget::setTabOrder(m_vatRate, m_vatAssignment);
        QWidget::setTabOrder(m_vatAssignment, m_grossAmount);
        QWidget::setTabOrder(m_grossAmount, m_netAmount);
        QWidget::setTabOrder(m_netAmount, m_qcheckboxTax);
        QWidget::setTabOrder(m_qcheckboxTax, m_payeeCreation);
        QWidget::setTabOrder(m_payeeCreation, m_budgetInclusion);

        retranslateUi(KNewAccountDlg);

        m_tab->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KNewAccountDlg);
    } // setupUi

    void retranslateUi(QDialog *KNewAccountDlg)
    {
        KNewAccountDlg->setWindowTitle(tr2i18n("New Account Dialog", nullptr));
        TextLabel1->setText(tr2i18n("Name:", "account name"));
        groupBox4->setTitle(tr2i18n("Opening information", nullptr));
        m_currencyText->setText(tr2i18n("Currency:", nullptr));
        textLabel2->setText(tr2i18n("Type:", nullptr));
        m_openingDateLabel->setText(tr2i18n("Date:", nullptr));
        m_openingBalanceLabel->setText(tr2i18n("Balance:", nullptr));
        groupBox6->setTitle(tr2i18n("Options", nullptr));
        TextLabel5->setText(tr2i18n("Notes:", nullptr));
        textLabel1_2->setText(tr2i18n("Last check number:", nullptr));
        m_qcheckboxNoVat->setText(tr2i18n("No auto-VAT-assignment", nullptr));
        m_costCenterRequiredCheckBox->setText(tr2i18n("Cost Center assignment required", nullptr));
        m_qcheckboxPreferred->setText(tr2i18n("Preferred Account", nullptr));
        m_priceModeLabel->setText(tr2i18n("Price entry", nullptr));
#if QT_CONFIG(tooltip)
        m_qcheckboxOpeningBalance->setToolTip(tr2i18n("<html><head/><body><p>Flag this account to be an opening balance account</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        m_qcheckboxOpeningBalance->setText(tr2i18n("Opening Balance Account", nullptr));
        m_tab->setTabText(m_tab->indexOf(m_generalTab), tr2i18n("General", "Account general settings"));
        m_institutionBox->setTitle(tr2i18n("Institution", nullptr));
        m_qbuttonNew->setText(tr2i18n("New...", "create new institution"));
        TextLabel2_2->setText(tr2i18n("Number:", nullptr));
        m_websiteLabel->setText(tr2i18n("Website", nullptr));
        textLabel1_4->setText(tr2i18n("BIC", nullptr));
        textLabel3->setText(tr2i18n("IBAN", nullptr));
        m_iconButton->setText(tr2i18n("...", nullptr));
        m_bicValue->setText(QString());
        m_tab->setTabText(m_tab->indexOf(m_institutionTab), tr2i18n("Institution", nullptr));
        m_subAccountLabel->setText(tr2i18n("Subaccount", nullptr));
        m_tab->setTabText(m_tab->indexOf(m_hierarchyTab), tr2i18n("Hierarchy", nullptr));
        m_limitsGroupBox->setTitle(tr2i18n("Limits", nullptr));
        qLabel1->setText(tr2i18n("This section allows to setup certain limits. KMyMoney will warn you when the account balance reaches a limit.", nullptr));
        textLabel2_2->setText(tr2i18n("Type", nullptr));
        textLabel3_2->setText(tr2i18n("Absolute limit", nullptr));
        m_maxCreditLabel->setText(tr2i18n("Maximum credit limit", nullptr));
        m_minBalanceLabel->setText(tr2i18n("Minimum balance", nullptr));
        textLabel4->setText(tr2i18n("Early warning", nullptr));
        m_taxGroupBox->setTitle(tr2i18n("Tax options", nullptr));
        groupBox3->setTitle(tr2i18n("VAT details", nullptr));
        m_vatCategory->setText(tr2i18n("VAT category", nullptr));
        textLabel1->setText(tr2i18n("VAT percentage", nullptr));
        m_vatAssignment->setText(tr2i18n("Enable auto VAT assignment", nullptr));
        m_amountGroupBox->setTitle(tr2i18n("Amount entered", nullptr));
        m_grossAmount->setText(tr2i18n("Gross amount", nullptr));
        m_netAmount->setText(tr2i18n("Net a&mount", nullptr));
        m_qcheckboxTax->setText(tr2i18n("Include on Tax Reports", nullptr));
        m_importGroupBox->setTitle(tr2i18n("Import options", nullptr));
        label->setText(tr2i18n("Automatic Payee creation", nullptr));
        m_payeeCreation->setItemText(0, tr2i18n("Application Default", nullptr));
        m_payeeCreation->setItemText(1, tr2i18n("Never create", nullptr));
        m_payeeCreation->setItemText(2, tr2i18n("Ask user", nullptr));
        m_payeeCreation->setItemText(3, tr2i18n("Create automatically", nullptr));

        m_budgetGroupBox->setTitle(tr2i18n("Budget options", nullptr));
        m_budgetInclusion->setItemText(0, tr2i18n("Don't include in budgets", "@item:inlistbox Don't include this account in budgets"));
        m_budgetInclusion->setItemText(1, tr2i18n("Include in budgets as expense", "@item:inlistbox Include this account in budgets as expense"));
        m_budgetInclusion->setItemText(2, tr2i18n("Include in budgets as income", "@item:inlistbox Include this account in budgets as income"));

        m_tab->setTabText(m_tab->indexOf(m_optionsTab), tr2i18n("Options", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KNewAccountDlg: public Ui_KNewAccountDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KNEWACCOUNTDLG_H

