/********************************************************************************
** Form generated from reading UI file 'splitadjustdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SPLITADJUSTDIALOG_H
#define UI_SPLITADJUSTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SplitAdjustDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *explanation;
    QLabel *question;
    QFrame *frame;
    QVBoxLayout *vboxLayout;
    QRadioButton *continueBtn;
    QRadioButton *changeBtn;
    QRadioButton *distributeBtn;
    QRadioButton *leaveBtn;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *SplitAdjustDialog)
    {
        if (SplitAdjustDialog->objectName().isEmpty())
            SplitAdjustDialog->setObjectName("SplitAdjustDialog");
        SplitAdjustDialog->resize(400, 240);
        verticalLayout = new QVBoxLayout(SplitAdjustDialog);
        verticalLayout->setObjectName("verticalLayout");
        explanation = new QLabel(SplitAdjustDialog);
        explanation->setObjectName("explanation");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(explanation->sizePolicy().hasHeightForWidth());
        explanation->setSizePolicy(sizePolicy);
        explanation->setFocusPolicy(Qt::NoFocus);
        explanation->setText(QString::fromUtf8("xxx"));
        explanation->setWordWrap(true);

        verticalLayout->addWidget(explanation);

        question = new QLabel(SplitAdjustDialog);
        question->setObjectName("question");
        question->setEnabled(true);
        question->setWordWrap(false);

        verticalLayout->addWidget(question);

        frame = new QFrame(SplitAdjustDialog);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::NoFrame);
        vboxLayout = new QVBoxLayout(frame);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        continueBtn = new QRadioButton(frame);
        continueBtn->setObjectName("continueBtn");
        continueBtn->setChecked(true);

        vboxLayout->addWidget(continueBtn);

        changeBtn = new QRadioButton(frame);
        changeBtn->setObjectName("changeBtn");
        changeBtn->setText(QString::fromUtf8("Change total amount of transaction to %1."));

        vboxLayout->addWidget(changeBtn);

        distributeBtn = new QRadioButton(frame);
        distributeBtn->setObjectName("distributeBtn");
        distributeBtn->setEnabled(false);
        distributeBtn->setText(QString::fromUtf8("Distribute difference of %1 among all splits."));

        vboxLayout->addWidget(distributeBtn);

        leaveBtn = new QRadioButton(frame);
        leaveBtn->setObjectName("leaveBtn");
        leaveBtn->setText(QString::fromUtf8("Leave %1 unassigned."));

        vboxLayout->addWidget(leaveBtn);


        verticalLayout->addWidget(frame);

        verticalSpacer = new QSpacerItem(20, 2, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(SplitAdjustDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(SplitAdjustDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, SplitAdjustDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, SplitAdjustDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(SplitAdjustDialog);
    } // setupUi

    void retranslateUi(QDialog *SplitAdjustDialog)
    {
        SplitAdjustDialog->setWindowTitle(tr2i18n("Dialog", nullptr));
        question->setText(tr2i18n("How do you want to proceed?", nullptr));
        continueBtn->setText(tr2i18n("Continue to edit splits", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SplitAdjustDialog: public Ui_SplitAdjustDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SPLITADJUSTDIALOG_H

