/********************************************************************************
** Form generated from reading UI file 'knewinvestmentwizard.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KNEWINVESTMENTWIZARD_H
#define UI_KNEWINVESTMENTWIZARD_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QWizard>
#include <kinvestmentdetailswizardpage.h>
#include <klocalizedstring.h>
#include "kinvestmenttypewizardpage.h"
#include "konlineupdatewizardpage.h"

QT_BEGIN_NAMESPACE

class Ui_KNewInvestmentWizard
{
public:
    KInvestmentTypeWizardPage *m_investmentTypePage;
    KInvestmentDetailsWizardPage *m_investmentDetailsPage;
    KOnlineUpdateWizardPage *m_onlineUpdatePage;

    void setupUi(QWizard *KNewInvestmentWizard)
    {
        if (KNewInvestmentWizard->objectName().isEmpty())
            KNewInvestmentWizard->setObjectName("KNewInvestmentWizard");
        KNewInvestmentWizard->resize(584, 527);
        KNewInvestmentWizard->setWizardStyle(QWizard::ModernStyle);
        m_investmentTypePage = new KInvestmentTypeWizardPage();
        m_investmentTypePage->setObjectName("m_investmentTypePage");
        KNewInvestmentWizard->addPage(m_investmentTypePage);
        m_investmentDetailsPage = new KInvestmentDetailsWizardPage();
        m_investmentDetailsPage->setObjectName("m_investmentDetailsPage");
        KNewInvestmentWizard->addPage(m_investmentDetailsPage);
        m_onlineUpdatePage = new KOnlineUpdateWizardPage();
        m_onlineUpdatePage->setObjectName("m_onlineUpdatePage");
        KNewInvestmentWizard->addPage(m_onlineUpdatePage);

        retranslateUi(KNewInvestmentWizard);

        QMetaObject::connectSlotsByName(KNewInvestmentWizard);
    } // setupUi

    void retranslateUi(QWizard *KNewInvestmentWizard)
    {
        KNewInvestmentWizard->setWindowTitle(tr2i18n("New Investment wizard", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KNewInvestmentWizard: public Ui_KNewInvestmentWizard {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KNEWINVESTMENTWIZARD_H

