/********************************************************************************
** Form generated from reading UI file 'ksettingsonlinequotes.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KSETTINGSONLINEQUOTES_H
#define UI_KSETTINGSONLINEQUOTES_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KSettingsOnlineQuotes
{
public:
    QVBoxLayout *vboxLayout;
    QListWidget *m_quoteSourceList;
    QGroupBox *groupParsing;
    QGridLayout *gridLayout;
    QLabel *textLabel4_2;
    QLabel *textLabel2;
    KLineEdit *m_editCSVURL;
    KLineEdit *m_editIdentifier;
    QLabel *textLabel3;
    KLineEdit *m_editDate;
    QLabel *textLabel4;
    QLabel *textLabel1;
    QLabel *label;
    KLineEdit *m_editURL;
    KLineEdit *m_editDateFormat;
    QComboBox *m_editIdentifyBy;
    KLineEdit *m_editPrice;
    QCheckBox *m_skipStripping;
    QLabel *label_2;
    QHBoxLayout *hboxLayout;
    QPushButton *m_newButton;
    QPushButton *m_deleteButton;
    QSpacerItem *spacer6;
    QPushButton *m_dumpCSVProfile;
    QPushButton *m_updateButton;

    void setupUi(QWidget *KSettingsOnlineQuotes)
    {
        if (KSettingsOnlineQuotes->objectName().isEmpty())
            KSettingsOnlineQuotes->setObjectName("KSettingsOnlineQuotes");
        KSettingsOnlineQuotes->resize(512, 592);
        KSettingsOnlineQuotes->setMinimumSize(QSize(0, 0));
        vboxLayout = new QVBoxLayout(KSettingsOnlineQuotes);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        m_quoteSourceList = new QListWidget(KSettingsOnlineQuotes);
        m_quoteSourceList->setObjectName("m_quoteSourceList");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_quoteSourceList->sizePolicy().hasHeightForWidth());
        m_quoteSourceList->setSizePolicy(sizePolicy);

        vboxLayout->addWidget(m_quoteSourceList);

        groupParsing = new QGroupBox(KSettingsOnlineQuotes);
        groupParsing->setObjectName("groupParsing");
        gridLayout = new QGridLayout(groupParsing);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        textLabel4_2 = new QLabel(groupParsing);
        textLabel4_2->setObjectName("textLabel4_2");
        textLabel4_2->setWordWrap(false);

        gridLayout->addWidget(textLabel4_2, 7, 0, 1, 1);

        textLabel2 = new QLabel(groupParsing);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 2, 0, 1, 1);

        m_editCSVURL = new KLineEdit(groupParsing);
        m_editCSVURL->setObjectName("m_editCSVURL");

        gridLayout->addWidget(m_editCSVURL, 1, 1, 1, 2);

        m_editIdentifier = new KLineEdit(groupParsing);
        m_editIdentifier->setObjectName("m_editIdentifier");

        gridLayout->addWidget(m_editIdentifier, 2, 1, 1, 2);

        textLabel3 = new QLabel(groupParsing);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout->addWidget(textLabel3, 4, 0, 1, 1);

        m_editDate = new KLineEdit(groupParsing);
        m_editDate->setObjectName("m_editDate");

        gridLayout->addWidget(m_editDate, 5, 1, 1, 2);

        textLabel4 = new QLabel(groupParsing);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 5, 0, 1, 1);

        textLabel1 = new QLabel(groupParsing);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        label = new QLabel(groupParsing);
        label->setObjectName("label");

        gridLayout->addWidget(label, 1, 0, 1, 1);

        m_editURL = new KLineEdit(groupParsing);
        m_editURL->setObjectName("m_editURL");

        gridLayout->addWidget(m_editURL, 0, 1, 1, 2);

        m_editDateFormat = new KLineEdit(groupParsing);
        m_editDateFormat->setObjectName("m_editDateFormat");

        gridLayout->addWidget(m_editDateFormat, 7, 1, 1, 1);

        m_editIdentifyBy = new QComboBox(groupParsing);
        m_editIdentifyBy->setObjectName("m_editIdentifyBy");

        gridLayout->addWidget(m_editIdentifyBy, 3, 1, 1, 2);

        m_editPrice = new KLineEdit(groupParsing);
        m_editPrice->setObjectName("m_editPrice");

        gridLayout->addWidget(m_editPrice, 4, 1, 1, 2);

        m_skipStripping = new QCheckBox(groupParsing);
        m_skipStripping->setObjectName("m_skipStripping");

        gridLayout->addWidget(m_skipStripping, 7, 2, 1, 1);

        label_2 = new QLabel(groupParsing);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 3, 0, 1, 1);


        vboxLayout->addWidget(groupParsing);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        m_newButton = new QPushButton(KSettingsOnlineQuotes);
        m_newButton->setObjectName("m_newButton");

        hboxLayout->addWidget(m_newButton);

        m_deleteButton = new QPushButton(KSettingsOnlineQuotes);
        m_deleteButton->setObjectName("m_deleteButton");

        hboxLayout->addWidget(m_deleteButton);

        spacer6 = new QSpacerItem(240, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer6);

        m_dumpCSVProfile = new QPushButton(KSettingsOnlineQuotes);
        m_dumpCSVProfile->setObjectName("m_dumpCSVProfile");

        hboxLayout->addWidget(m_dumpCSVProfile);

        m_updateButton = new QPushButton(KSettingsOnlineQuotes);
        m_updateButton->setObjectName("m_updateButton");

        hboxLayout->addWidget(m_updateButton);


        vboxLayout->addLayout(hboxLayout);


        retranslateUi(KSettingsOnlineQuotes);

        QMetaObject::connectSlotsByName(KSettingsOnlineQuotes);
    } // setupUi

    void retranslateUi(QWidget *KSettingsOnlineQuotes)
    {
        KSettingsOnlineQuotes->setWindowTitle(tr2i18n("Online Quotes", nullptr));
#if QT_CONFIG(whatsthis)
        groupParsing->setWhatsThis(tr2i18n("<i>Enter regular expressions which can be used to parse the data returned from the URL entered above.  The symbol, price, and date must be found in the quote data to be usable. You may also try the KMyMoney user's mailinglist at <a href=\"mailto:kmymoney@kde.org\">kmymoney@kde.org</a> to find what settings work for other users in your country.</i>", nullptr));
#endif // QT_CONFIG(whatsthis)
        groupParsing->setTitle(tr2i18n("Details", nullptr));
        textLabel4_2->setText(tr2i18n("Date Format", nullptr));
        textLabel2->setText(tr2i18n("Identifier", nullptr));
#if QT_CONFIG(tooltip)
        m_editIdentifier->setToolTip(tr2i18n("Regular Expression to extract the identifier from the downloaded data", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel3->setText(tr2i18n("Price", nullptr));
#if QT_CONFIG(tooltip)
        m_editDate->setToolTip(tr2i18n("Regular Expression to extract the date from the downloaded data", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel4->setText(tr2i18n("Date", nullptr));
        textLabel1->setText(tr2i18n("URL", nullptr));
        label->setText(tr2i18n("CSV URL", nullptr));
#if QT_CONFIG(tooltip)
        m_editURL->setToolTip(tr2i18n("URL to be used to download the quote", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_editURL->setWhatsThis(tr2i18n("Enter the URL from which stock quotes will be fetched.  <b>%1</b> will be replaced with the symbol for the security being quoted. For currency conversions, <b>%2</b> will be replaced with the currency to be quoted and <b>%1</b> with the currency the quote is based on.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        m_editDateFormat->setToolTip(tr2i18n("Regular Expression to extract the date from the downloaded data", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_editPrice->setToolTip(tr2i18n("Regular Expression to extract the price from the downloaded data", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_skipStripping->setWhatsThis(tr2i18n("<p>For easier processing of the data returned by the online source, KMyMoney usually strips unused parts before it is parsed with the regular expressions. If matching of the fields relies on those items, then use this option to turn stripping off.</p>\n"
"\n"
"<p>The following items are usually removed by stripping:\n"
"\n"
"<ul>\n"
"<li>HTML tags such as <b>&lt;tag&gt;</b></li>\n"
"<li>& encoded characters such as <b>&amp;nbsp;</b></li>\n"
"<li>duplicate whitespace</li>\n"
"</ul>\n"
"</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_skipStripping->setText(tr2i18n("Skip HTML stripping", nullptr));
        label_2->setText(tr2i18n("Identify by", nullptr));
#if QT_CONFIG(tooltip)
        m_newButton->setToolTip(tr2i18n("Create a new source entry for online quotes", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_newButton->setWhatsThis(tr2i18n("Use this to create a new entry for online quotes", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_newButton->setText(tr2i18n("New", "@action create new quote source"));
#if QT_CONFIG(tooltip)
        m_deleteButton->setToolTip(tr2i18n("Delete the selected source entry", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_deleteButton->setWhatsThis(tr2i18n("Use this to delete the selected online source entry", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_deleteButton->setText(tr2i18n("Delete", nullptr));
#if QT_CONFIG(tooltip)
        m_dumpCSVProfile->setToolTip(tr2i18n("Dumps CSV profile used for importing downloaded prices, which can be customized by user in CSV Importer.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_dumpCSVProfile->setText(tr2i18n("Dump CSV", nullptr));
#if QT_CONFIG(tooltip)
        m_updateButton->setToolTip(tr2i18n("Accepts the entered data and stores it", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        m_updateButton->setWhatsThis(tr2i18n("Use this to accept the modified data.", nullptr));
#endif // QT_CONFIG(whatsthis)
        m_updateButton->setText(tr2i18n("Update", "@action update quotes from online quote source"));
    } // retranslateUi

};

namespace Ui {
    class KSettingsOnlineQuotes: public Ui_KSettingsOnlineQuotes {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KSETTINGSONLINEQUOTES_H

