/********************************************************************************
** Form generated from reading UI file 'bankingwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BANKINGWIZARDPAGE_H
#define UI_BANKINGWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_BankingPage
{
public:
    QVBoxLayout *verticalLayout_4;
    QLabel *header;
    QFrame *line;
    QHBoxLayout *hl_selectors;
    QSpacerItem *leftSpacer;
    QVBoxLayout *vl_left;
    QGridLayout *gl_fields;
    QLabel *labelBnk_number;
    QComboBox *m_numberCol;
    QToolButton *m_numberColClear;
    QLabel *labelBnk_date;
    QComboBox *m_dateCol;
    QToolButton *m_dateColClear;
    QLabel *labelBnk_payee;
    QComboBox *m_payeeCol;
    QToolButton *m_payeeColClear;
    QLabel *labelBnk_category;
    QComboBox *m_categoryCol;
    QToolButton *m_categoryColClear;
    QLabel *labelBnk_memo;
    QComboBox *m_memoCol;
    QLabel *m_currentMemoColumns;
    QToolButton *m_memoColClear;
    QLabel *labelBnk_balance;
    QComboBox *m_balanceCol;
    QToolButton *m_balanceColClear;
    QSpacerItem *verticalSpacer_2;
    QSpacerItem *midSpace;
    QTabWidget *m_amountTabWidget;
    QWidget *amountTab;
    QVBoxLayout *verticalLayout_3;
    QGridLayout *gl_amount;
    QLabel *labelBnk_amount;
    QComboBox *m_amountCol;
    QToolButton *m_amountColClear;
    QLabel *labelBnk_indicator;
    QComboBox *m_creditDebitIndicatorCol;
    QToolButton *m_creditDebitIndicatorColClear;
    QLabel *labelBnk_debit;
    QLineEdit *m_creditIndicator;
    QLabel *labelBnk_credit;
    QLineEdit *m_debitIndicator;
    QLabel *labelBnk_opposite;
    QCheckBox *m_oppositeSigns;
    QSpacerItem *verticalSpacer_3;
    QWidget *debitCreditTab;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gl_debitcredit;
    QLabel *labelBnk_debits;
    QComboBox *m_debitCol;
    QToolButton *m_debitColClear;
    QLabel *labelBnk_credits;
    QComboBox *m_creditCol;
    QToolButton *m_creditColClear;
    QSpacerItem *verticalSpacer;
    QSpacerItem *rightSpacer;
    QHBoxLayout *hl_bottom;
    QSpacerItem *horizontalSpacer;
    QPushButton *m_clear;

    void setupUi(QWidget *BankingPage)
    {
        if (BankingPage->objectName().isEmpty())
            BankingPage->setObjectName("BankingPage");
        BankingPage->resize(698, 402);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(BankingPage->sizePolicy().hasHeightForWidth());
        BankingPage->setSizePolicy(sizePolicy);
        verticalLayout_4 = new QVBoxLayout(BankingPage);
        verticalLayout_4->setObjectName("verticalLayout_4");
        header = new QLabel(BankingPage);
        header->setObjectName("header");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(header->sizePolicy().hasHeightForWidth());
        header->setSizePolicy(sizePolicy1);
        header->setAlignment(Qt::AlignCenter);
        header->setWordWrap(true);

        verticalLayout_4->addWidget(header);

        line = new QFrame(BankingPage);
        line->setObjectName("line");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(line->sizePolicy().hasHeightForWidth());
        line->setSizePolicy(sizePolicy2);
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_4->addWidget(line);

        hl_selectors = new QHBoxLayout();
        hl_selectors->setObjectName("hl_selectors");
        leftSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hl_selectors->addItem(leftSpacer);

        vl_left = new QVBoxLayout();
        vl_left->setObjectName("vl_left");
        gl_fields = new QGridLayout();
        gl_fields->setObjectName("gl_fields");
        labelBnk_number = new QLabel(BankingPage);
        labelBnk_number->setObjectName("labelBnk_number");
        labelBnk_number->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_fields->addWidget(labelBnk_number, 0, 0, 1, 1);

        m_numberCol = new QComboBox(BankingPage);
        m_numberCol->setObjectName("m_numberCol");
        sizePolicy1.setHeightForWidth(m_numberCol->sizePolicy().hasHeightForWidth());
        m_numberCol->setSizePolicy(sizePolicy1);
        m_numberCol->setMinimumSize(QSize(0, 0));
        m_numberCol->setEditable(false);
        m_numberCol->setMaxVisibleItems(12);

        gl_fields->addWidget(m_numberCol, 0, 1, 1, 1);

        m_numberColClear = new QToolButton(BankingPage);
        m_numberColClear->setObjectName("m_numberColClear");

        gl_fields->addWidget(m_numberColClear, 0, 2, 1, 1);

        labelBnk_date = new QLabel(BankingPage);
        labelBnk_date->setObjectName("labelBnk_date");
        labelBnk_date->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_fields->addWidget(labelBnk_date, 1, 0, 1, 1);

        m_dateCol = new QComboBox(BankingPage);
        m_dateCol->setObjectName("m_dateCol");
        sizePolicy1.setHeightForWidth(m_dateCol->sizePolicy().hasHeightForWidth());
        m_dateCol->setSizePolicy(sizePolicy1);
        m_dateCol->setMinimumSize(QSize(0, 0));
        m_dateCol->setMaxVisibleItems(12);

        gl_fields->addWidget(m_dateCol, 1, 1, 1, 1);

        m_dateColClear = new QToolButton(BankingPage);
        m_dateColClear->setObjectName("m_dateColClear");

        gl_fields->addWidget(m_dateColClear, 1, 2, 1, 1);

        labelBnk_payee = new QLabel(BankingPage);
        labelBnk_payee->setObjectName("labelBnk_payee");
        labelBnk_payee->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_fields->addWidget(labelBnk_payee, 2, 0, 1, 1);

        m_payeeCol = new QComboBox(BankingPage);
        m_payeeCol->setObjectName("m_payeeCol");
        sizePolicy1.setHeightForWidth(m_payeeCol->sizePolicy().hasHeightForWidth());
        m_payeeCol->setSizePolicy(sizePolicy1);
        m_payeeCol->setMaxVisibleItems(12);

        gl_fields->addWidget(m_payeeCol, 2, 1, 1, 1);

        m_payeeColClear = new QToolButton(BankingPage);
        m_payeeColClear->setObjectName("m_payeeColClear");

        gl_fields->addWidget(m_payeeColClear, 2, 2, 1, 1);

        labelBnk_category = new QLabel(BankingPage);
        labelBnk_category->setObjectName("labelBnk_category");
        labelBnk_category->setMaximumSize(QSize(16777215, 16777215));
        labelBnk_category->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_fields->addWidget(labelBnk_category, 3, 0, 1, 1);

        m_categoryCol = new QComboBox(BankingPage);
        m_categoryCol->setObjectName("m_categoryCol");
        sizePolicy1.setHeightForWidth(m_categoryCol->sizePolicy().hasHeightForWidth());
        m_categoryCol->setSizePolicy(sizePolicy1);
        m_categoryCol->setMaxVisibleItems(12);

        gl_fields->addWidget(m_categoryCol, 3, 1, 1, 1);

        m_categoryColClear = new QToolButton(BankingPage);
        m_categoryColClear->setObjectName("m_categoryColClear");

        gl_fields->addWidget(m_categoryColClear, 3, 2, 1, 1);

        labelBnk_memo = new QLabel(BankingPage);
        labelBnk_memo->setObjectName("labelBnk_memo");
        labelBnk_memo->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_fields->addWidget(labelBnk_memo, 4, 0, 1, 1);

        m_memoCol = new QComboBox(BankingPage);
        m_memoCol->setObjectName("m_memoCol");
        sizePolicy1.setHeightForWidth(m_memoCol->sizePolicy().hasHeightForWidth());
        m_memoCol->setSizePolicy(sizePolicy1);
        m_memoCol->setMaxVisibleItems(12);

        gl_fields->addWidget(m_memoCol, 4, 1, 1, 1);

        m_currentMemoColumns = new QLabel(BankingPage);
        m_currentMemoColumns->setObjectName("m_currentMemoColumns");
        m_currentMemoColumns->setMinimumSize(QSize(0, 0));
        m_currentMemoColumns->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_fields->addWidget(m_currentMemoColumns, 5, 0, 1, 2);

        m_memoColClear = new QToolButton(BankingPage);
        m_memoColClear->setObjectName("m_memoColClear");

        gl_fields->addWidget(m_memoColClear, 5, 2, 1, 1);

        labelBnk_balance = new QLabel(BankingPage);
        labelBnk_balance->setObjectName("labelBnk_balance");
        labelBnk_balance->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_fields->addWidget(labelBnk_balance, 6, 0, 1, 1);

        m_balanceCol = new QComboBox(BankingPage);
        m_balanceCol->setObjectName("m_balanceCol");
        sizePolicy1.setHeightForWidth(m_balanceCol->sizePolicy().hasHeightForWidth());
        m_balanceCol->setSizePolicy(sizePolicy1);

        gl_fields->addWidget(m_balanceCol, 6, 1, 1, 1);

        m_balanceColClear = new QToolButton(BankingPage);
        m_balanceColClear->setObjectName("m_balanceColClear");

        gl_fields->addWidget(m_balanceColClear, 6, 2, 1, 1);


        vl_left->addLayout(gl_fields);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vl_left->addItem(verticalSpacer_2);


        hl_selectors->addLayout(vl_left);

        midSpace = new QSpacerItem(13, 29, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        hl_selectors->addItem(midSpace);

        m_amountTabWidget = new QTabWidget(BankingPage);
        m_amountTabWidget->setObjectName("m_amountTabWidget");
        amountTab = new QWidget();
        amountTab->setObjectName("amountTab");
        verticalLayout_3 = new QVBoxLayout(amountTab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        gl_amount = new QGridLayout();
        gl_amount->setObjectName("gl_amount");
        labelBnk_amount = new QLabel(amountTab);
        labelBnk_amount->setObjectName("labelBnk_amount");
        labelBnk_amount->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_amount->addWidget(labelBnk_amount, 0, 0, 1, 1);

        m_amountCol = new QComboBox(amountTab);
        m_amountCol->setObjectName("m_amountCol");
        sizePolicy1.setHeightForWidth(m_amountCol->sizePolicy().hasHeightForWidth());
        m_amountCol->setSizePolicy(sizePolicy1);
        m_amountCol->setMaxVisibleItems(12);

        gl_amount->addWidget(m_amountCol, 0, 1, 1, 1);

        m_amountColClear = new QToolButton(amountTab);
        m_amountColClear->setObjectName("m_amountColClear");

        gl_amount->addWidget(m_amountColClear, 0, 2, 1, 1);

        labelBnk_indicator = new QLabel(amountTab);
        labelBnk_indicator->setObjectName("labelBnk_indicator");
        labelBnk_indicator->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_amount->addWidget(labelBnk_indicator, 1, 0, 1, 1);

        m_creditDebitIndicatorCol = new QComboBox(amountTab);
        m_creditDebitIndicatorCol->setObjectName("m_creditDebitIndicatorCol");

        gl_amount->addWidget(m_creditDebitIndicatorCol, 1, 1, 1, 1);

        m_creditDebitIndicatorColClear = new QToolButton(amountTab);
        m_creditDebitIndicatorColClear->setObjectName("m_creditDebitIndicatorColClear");

        gl_amount->addWidget(m_creditDebitIndicatorColClear, 1, 2, 1, 1);

        labelBnk_debit = new QLabel(amountTab);
        labelBnk_debit->setObjectName("labelBnk_debit");
        labelBnk_debit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_amount->addWidget(labelBnk_debit, 2, 0, 1, 1);

        m_creditIndicator = new QLineEdit(amountTab);
        m_creditIndicator->setObjectName("m_creditIndicator");
        m_creditIndicator->setClearButtonEnabled(true);

        gl_amount->addWidget(m_creditIndicator, 2, 1, 1, 2);

        labelBnk_credit = new QLabel(amountTab);
        labelBnk_credit->setObjectName("labelBnk_credit");
        labelBnk_credit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_amount->addWidget(labelBnk_credit, 3, 0, 1, 1);

        m_debitIndicator = new QLineEdit(amountTab);
        m_debitIndicator->setObjectName("m_debitIndicator");
        m_debitIndicator->setClearButtonEnabled(true);

        gl_amount->addWidget(m_debitIndicator, 3, 1, 1, 2);

        labelBnk_opposite = new QLabel(amountTab);
        labelBnk_opposite->setObjectName("labelBnk_opposite");
        labelBnk_opposite->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_amount->addWidget(labelBnk_opposite, 4, 0, 1, 1);

        m_oppositeSigns = new QCheckBox(amountTab);
        m_oppositeSigns->setObjectName("m_oppositeSigns");
        sizePolicy1.setHeightForWidth(m_oppositeSigns->sizePolicy().hasHeightForWidth());
        m_oppositeSigns->setSizePolicy(sizePolicy1);
        m_oppositeSigns->setLayoutDirection(Qt::LeftToRight);

        gl_amount->addWidget(m_oppositeSigns, 4, 1, 1, 2);


        verticalLayout_3->addLayout(gl_amount);

        verticalSpacer_3 = new QSpacerItem(20, 64, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        m_amountTabWidget->addTab(amountTab, QString());
        debitCreditTab = new QWidget();
        debitCreditTab->setObjectName("debitCreditTab");
        verticalLayout_2 = new QVBoxLayout(debitCreditTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        gl_debitcredit = new QGridLayout();
        gl_debitcredit->setObjectName("gl_debitcredit");
        labelBnk_debits = new QLabel(debitCreditTab);
        labelBnk_debits->setObjectName("labelBnk_debits");
        labelBnk_debits->setEnabled(true);
        labelBnk_debits->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_debitcredit->addWidget(labelBnk_debits, 0, 0, 1, 1);

        m_debitCol = new QComboBox(debitCreditTab);
        m_debitCol->setObjectName("m_debitCol");
        m_debitCol->setEnabled(true);
        sizePolicy1.setHeightForWidth(m_debitCol->sizePolicy().hasHeightForWidth());
        m_debitCol->setSizePolicy(sizePolicy1);
        m_debitCol->setMaxVisibleItems(12);

        gl_debitcredit->addWidget(m_debitCol, 0, 1, 1, 1);

        m_debitColClear = new QToolButton(debitCreditTab);
        m_debitColClear->setObjectName("m_debitColClear");

        gl_debitcredit->addWidget(m_debitColClear, 0, 2, 1, 1);

        labelBnk_credits = new QLabel(debitCreditTab);
        labelBnk_credits->setObjectName("labelBnk_credits");
        labelBnk_credits->setEnabled(true);
        labelBnk_credits->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gl_debitcredit->addWidget(labelBnk_credits, 1, 0, 1, 1);

        m_creditCol = new QComboBox(debitCreditTab);
        m_creditCol->setObjectName("m_creditCol");
        sizePolicy1.setHeightForWidth(m_creditCol->sizePolicy().hasHeightForWidth());
        m_creditCol->setSizePolicy(sizePolicy1);
        m_creditCol->setMaxVisibleItems(12);

        gl_debitcredit->addWidget(m_creditCol, 1, 1, 1, 1);

        m_creditColClear = new QToolButton(debitCreditTab);
        m_creditColClear->setObjectName("m_creditColClear");

        gl_debitcredit->addWidget(m_creditColClear, 1, 2, 1, 1);


        verticalLayout_2->addLayout(gl_debitcredit);

        verticalSpacer = new QSpacerItem(20, 38, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        m_amountTabWidget->addTab(debitCreditTab, QString());

        hl_selectors->addWidget(m_amountTabWidget);

        rightSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hl_selectors->addItem(rightSpacer);


        verticalLayout_4->addLayout(hl_selectors);

        hl_bottom = new QHBoxLayout();
        hl_bottom->setObjectName("hl_bottom");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hl_bottom->addItem(horizontalSpacer);

        m_clear = new QPushButton(BankingPage);
        m_clear->setObjectName("m_clear");
        sizePolicy1.setHeightForWidth(m_clear->sizePolicy().hasHeightForWidth());
        m_clear->setSizePolicy(sizePolicy1);

        hl_bottom->addWidget(m_clear);


        verticalLayout_4->addLayout(hl_bottom);

#if QT_CONFIG(shortcut)
        labelBnk_number->setBuddy(m_numberCol);
        labelBnk_date->setBuddy(m_dateCol);
        labelBnk_payee->setBuddy(m_payeeCol);
        labelBnk_category->setBuddy(m_categoryCol);
        labelBnk_memo->setBuddy(m_memoCol);
        labelBnk_balance->setBuddy(m_balanceCol);
        labelBnk_amount->setBuddy(m_amountCol);
        labelBnk_indicator->setBuddy(m_creditDebitIndicatorCol);
        labelBnk_debits->setBuddy(m_debitCol);
        labelBnk_credits->setBuddy(m_creditCol);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(m_numberCol, m_numberColClear);
        QWidget::setTabOrder(m_numberColClear, m_dateCol);
        QWidget::setTabOrder(m_dateCol, m_dateColClear);
        QWidget::setTabOrder(m_dateColClear, m_payeeCol);
        QWidget::setTabOrder(m_payeeCol, m_payeeColClear);
        QWidget::setTabOrder(m_payeeColClear, m_categoryCol);
        QWidget::setTabOrder(m_categoryCol, m_categoryColClear);
        QWidget::setTabOrder(m_categoryColClear, m_memoCol);
        QWidget::setTabOrder(m_memoCol, m_memoColClear);
        QWidget::setTabOrder(m_memoColClear, m_balanceCol);
        QWidget::setTabOrder(m_balanceCol, m_balanceColClear);
        QWidget::setTabOrder(m_balanceColClear, m_amountTabWidget);
        QWidget::setTabOrder(m_amountTabWidget, m_amountCol);
        QWidget::setTabOrder(m_amountCol, m_amountColClear);
        QWidget::setTabOrder(m_amountColClear, m_creditDebitIndicatorCol);
        QWidget::setTabOrder(m_creditDebitIndicatorCol, m_creditDebitIndicatorColClear);
        QWidget::setTabOrder(m_creditDebitIndicatorColClear, m_creditIndicator);
        QWidget::setTabOrder(m_creditIndicator, m_debitIndicator);
        QWidget::setTabOrder(m_debitIndicator, m_oppositeSigns);
        QWidget::setTabOrder(m_oppositeSigns, m_debitCol);
        QWidget::setTabOrder(m_debitCol, m_debitColClear);
        QWidget::setTabOrder(m_debitColClear, m_creditCol);
        QWidget::setTabOrder(m_creditCol, m_creditColClear);

        retranslateUi(BankingPage);

        m_amountTabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(BankingPage);
    } // setupUi

    void retranslateUi(QWidget *BankingPage)
    {
        BankingPage->setWindowTitle(tr2i18n("Banking Wizard Page", nullptr));
        header->setText(tr2i18n("Please select the appropriate columns to use,\n"
" corresponding to your data.", nullptr));
#if QT_CONFIG(accessibility)
        labelBnk_number->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelBnk_number->setText(tr2i18n("Number", "column containing number field"));
#if QT_CONFIG(tooltip)
        m_numberCol->setToolTip(tr2i18n("Select column containing number field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_numberColClear->setToolTip(tr2i18n("Clear number field selection.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_numberColClear->setText(tr2i18n("...", nullptr));
#if QT_CONFIG(accessibility)
        labelBnk_date->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelBnk_date->setText(tr2i18n("Date", nullptr));
#if QT_CONFIG(tooltip)
        m_dateCol->setToolTip(tr2i18n("Select column containing date field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_dateColClear->setToolTip(tr2i18n("Clear date field selection.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_dateColClear->setText(tr2i18n("...", nullptr));
#if QT_CONFIG(accessibility)
        labelBnk_payee->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelBnk_payee->setText(tr2i18n("Payee/Description", "column containing payee or description field"));
#if QT_CONFIG(tooltip)
        m_payeeCol->setToolTip(tr2i18n("Select column containing payee or description field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_payeeColClear->setToolTip(tr2i18n("Clear payee field selection.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_payeeColClear->setText(tr2i18n("...", nullptr));
#if QT_CONFIG(accessibility)
        labelBnk_category->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelBnk_category->setText(tr2i18n("Category", "column containing number field"));
#if QT_CONFIG(tooltip)
        m_categoryCol->setToolTip(tr2i18n("Select column containing category field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_categoryColClear->setToolTip(tr2i18n("Clear category field selection.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_categoryColClear->setText(tr2i18n("...", nullptr));
#if QT_CONFIG(accessibility)
        labelBnk_memo->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelBnk_memo->setText(tr2i18n("Memo", nullptr));
#if QT_CONFIG(tooltip)
        m_memoCol->setToolTip(tr2i18n("Select column containing memo field.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_currentMemoColumns->setText(tr2i18n("Selection", nullptr));
#if QT_CONFIG(tooltip)
        m_memoColClear->setToolTip(tr2i18n("Clear memo field selection.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_memoColClear->setText(tr2i18n("...", nullptr));
        labelBnk_balance->setText(tr2i18n("Balance", nullptr));
#if QT_CONFIG(tooltip)
        m_balanceCol->setToolTip(tr2i18n("Select column containing balance field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_balanceColClear->setToolTip(tr2i18n("Clear balance field selection.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_balanceColClear->setText(tr2i18n("...", nullptr));
#if QT_CONFIG(accessibility)
        labelBnk_amount->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelBnk_amount->setText(tr2i18n("Amount", "location of amount column"));
#if QT_CONFIG(tooltip)
        m_amountCol->setToolTip(tr2i18n("Select column containing amount field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_amountColClear->setToolTip(tr2i18n("Clear amount field selection.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_amountColClear->setText(tr2i18n("...", nullptr));
        labelBnk_indicator->setText(tr2i18n("Debit/Credit Indicator", nullptr));
#if QT_CONFIG(tooltip)
        m_creditDebitIndicatorCol->setToolTip(tr2i18n("Select column containing indicator field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_creditDebitIndicatorColClear->setToolTip(tr2i18n("Clear indicator field selection.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_creditDebitIndicatorColClear->setText(tr2i18n("...", nullptr));
        labelBnk_debit->setText(tr2i18n("Indicator for Credit", nullptr));
#if QT_CONFIG(tooltip)
        m_creditIndicator->setToolTip(tr2i18n("Enter text that is used to indicate that amount is a credit.", nullptr));
#endif // QT_CONFIG(tooltip)
        labelBnk_credit->setText(tr2i18n("Indicator for Debit", nullptr));
#if QT_CONFIG(tooltip)
        m_debitIndicator->setToolTip(tr2i18n("Enter text that is used to indicate that amount is a debit.", nullptr));
#endif // QT_CONFIG(tooltip)
        labelBnk_opposite->setText(tr2i18n("Opposite signs", nullptr));
#if QT_CONFIG(tooltip)
        m_oppositeSigns->setToolTip(tr2i18n("Select if your amount column has signs improperly set.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_oppositeSigns->setText(QString());
        m_amountTabWidget->setTabText(m_amountTabWidget->indexOf(amountTab), tr2i18n("Amount", nullptr));
#if QT_CONFIG(accessibility)
        labelBnk_debits->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelBnk_debits->setText(tr2i18n("Debit", "location of debit column"));
#if QT_CONFIG(tooltip)
        m_debitCol->setToolTip(tr2i18n("Select column containing debit field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_debitColClear->setToolTip(tr2i18n("Clear debit field selection.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_debitColClear->setText(tr2i18n("...", nullptr));
#if QT_CONFIG(accessibility)
        labelBnk_credits->setAccessibleName(QString());
#endif // QT_CONFIG(accessibility)
        labelBnk_credits->setText(tr2i18n("Credit", "location of credit column"));
#if QT_CONFIG(tooltip)
        m_creditCol->setToolTip(tr2i18n("Select column containing credit field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_creditColClear->setToolTip(tr2i18n("Clear credit field selection.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_creditColClear->setText(tr2i18n("...", nullptr));
        m_amountTabWidget->setTabText(m_amountTabWidget->indexOf(debitCreditTab), tr2i18n("Debit/credit", nullptr));
#if QT_CONFIG(tooltip)
        m_clear->setToolTip(tr2i18n("Clear all selected column and indicator entries", nullptr));
#endif // QT_CONFIG(tooltip)
        m_clear->setText(tr2i18n("Clear all", "Clear all column assignments"));
    } // retranslateUi

};

namespace Ui {
    class BankingPage: public Ui_BankingPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // BANKINGWIZARDPAGE_H

