/********************************************************************************
** Form generated from reading UI file 'loanamountwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOANAMOUNTWIZARDPAGE_H
#define UI_LOANAMOUNTWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <amountedit.h>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_LoanAmountWizardPage
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *m_balanceLabel;
    QSpacerItem *Spacer68;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel16;
    AmountEdit *m_loanAmountEdit;
    QFrame *Frame3;
    QGridLayout *gridLayout;
    QLabel *TextLabel6;
    QLabel *TextLabel8;
    QLabel *TextLabel9;
    QLabel *TextLabel10;
    QLabel *TextLabel11;
    QLabel *m_loanAmount1;
    QLabel *m_interestRate1;
    QLabel *m_duration1;
    QLabel *m_payment1;
    QLabel *m_balloon1;

    void setupUi(QWizardPage *LoanAmountWizardPage)
    {
        if (LoanAmountWizardPage->objectName().isEmpty())
            LoanAmountWizardPage->setObjectName("LoanAmountWizardPage");
        vboxLayout = new QVBoxLayout(LoanAmountWizardPage);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        m_balanceLabel = new QLabel(LoanAmountWizardPage);
        m_balanceLabel->setObjectName("m_balanceLabel");
        m_balanceLabel->setAlignment(Qt::AlignTop);
        m_balanceLabel->setWordWrap(true);

        vboxLayout->addWidget(m_balanceLabel);

        Spacer68 = new QSpacerItem(0, 15, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(Spacer68);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        TextLabel16 = new QLabel(LoanAmountWizardPage);
        TextLabel16->setObjectName("TextLabel16");
        TextLabel16->setWordWrap(false);

        hboxLayout->addWidget(TextLabel16);

        m_loanAmountEdit = new AmountEdit(LoanAmountWizardPage);
        m_loanAmountEdit->setObjectName("m_loanAmountEdit");

        hboxLayout->addWidget(m_loanAmountEdit);


        vboxLayout->addLayout(hboxLayout);

        Frame3 = new QFrame(LoanAmountWizardPage);
        Frame3->setObjectName("Frame3");
        Frame3->setFrameShape(QFrame::StyledPanel);
        Frame3->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(Frame3);
#ifndef UI_Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        TextLabel6 = new QLabel(Frame3);
        TextLabel6->setObjectName("TextLabel6");
        TextLabel6->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel6->setWordWrap(true);

        gridLayout->addWidget(TextLabel6, 0, 0, 1, 1);

        TextLabel8 = new QLabel(Frame3);
        TextLabel8->setObjectName("TextLabel8");
        TextLabel8->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel8->setWordWrap(false);

        gridLayout->addWidget(TextLabel8, 1, 0, 1, 1);

        TextLabel9 = new QLabel(Frame3);
        TextLabel9->setObjectName("TextLabel9");
        TextLabel9->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel9->setWordWrap(false);

        gridLayout->addWidget(TextLabel9, 2, 0, 1, 1);

        TextLabel10 = new QLabel(Frame3);
        TextLabel10->setObjectName("TextLabel10");
        TextLabel10->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel10->setWordWrap(false);

        gridLayout->addWidget(TextLabel10, 3, 0, 1, 1);

        TextLabel11 = new QLabel(Frame3);
        TextLabel11->setObjectName("TextLabel11");
        TextLabel11->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
        TextLabel11->setWordWrap(false);

        gridLayout->addWidget(TextLabel11, 4, 0, 1, 1);

        m_loanAmount1 = new QLabel(Frame3);
        m_loanAmount1->setObjectName("m_loanAmount1");
        QFont font;
        font.setBold(true);
        m_loanAmount1->setFont(font);
        m_loanAmount1->setTextFormat(Qt::PlainText);
        m_loanAmount1->setWordWrap(false);

        gridLayout->addWidget(m_loanAmount1, 0, 1, 1, 1);

        m_interestRate1 = new QLabel(Frame3);
        m_interestRate1->setObjectName("m_interestRate1");
        m_interestRate1->setFont(font);
        m_interestRate1->setTextFormat(Qt::PlainText);
        m_interestRate1->setWordWrap(false);

        gridLayout->addWidget(m_interestRate1, 1, 1, 1, 1);

        m_duration1 = new QLabel(Frame3);
        m_duration1->setObjectName("m_duration1");
        m_duration1->setFont(font);
        m_duration1->setTextFormat(Qt::PlainText);
        m_duration1->setWordWrap(false);

        gridLayout->addWidget(m_duration1, 2, 1, 1, 1);

        m_payment1 = new QLabel(Frame3);
        m_payment1->setObjectName("m_payment1");
        m_payment1->setFont(font);
        m_payment1->setTextFormat(Qt::PlainText);
        m_payment1->setWordWrap(false);

        gridLayout->addWidget(m_payment1, 3, 1, 1, 1);

        m_balloon1 = new QLabel(Frame3);
        m_balloon1->setObjectName("m_balloon1");
        m_balloon1->setFont(font);
        m_balloon1->setTextFormat(Qt::PlainText);
        m_balloon1->setWordWrap(false);

        gridLayout->addWidget(m_balloon1, 4, 1, 1, 1);


        vboxLayout->addWidget(Frame3);


        retranslateUi(LoanAmountWizardPage);

        QMetaObject::connectSlotsByName(LoanAmountWizardPage);
    } // setupUi

    void retranslateUi(QWizardPage *LoanAmountWizardPage)
    {
        m_balanceLabel->setText(tr2i18n("1", nullptr));
        TextLabel16->setText(tr2i18n("Loan amount", nullptr));
        TextLabel6->setText(tr2i18n("Loan amount:", nullptr));
        TextLabel8->setText(tr2i18n("Interest rate:", nullptr));
        TextLabel9->setText(tr2i18n("Term:", nullptr));
        TextLabel10->setText(tr2i18n("Principal + Interest:", nullptr));
        TextLabel11->setText(tr2i18n("Final amortization payment", nullptr));
        m_loanAmount1->setText(tr2i18n("1", nullptr));
        m_interestRate1->setText(tr2i18n("1", nullptr));
        m_duration1->setText(tr2i18n("1", nullptr));
        m_payment1->setText(tr2i18n("1", nullptr));
        m_balloon1->setText(tr2i18n("1", nullptr));
        (void)LoanAmountWizardPage;
    } // retranslateUi

};

namespace Ui {
    class LoanAmountWizardPage: public Ui_LoanAmountWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // LOANAMOUNTWIZARDPAGE_H

