/****************************************************************************
** Meta object code from reading C++ file 'kreportconfigurationfilterdlg.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../kmymoney-5.2.2/kmymoney/plugins/views/reports/kreportconfigurationfilterdlg.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kreportconfigurationfilterdlg.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN29KReportConfigurationFilterDlgE_t {};
} // unnamed namespace

template <> constexpr inline auto KReportConfigurationFilterDlg::qt_create_metaobjectdata<qt_meta_tag_ZN29KReportConfigurationFilterDlgE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KReportConfigurationFilterDlg",
        "slotRowTypeChanged",
        "",
        "slotColumnTypeChanged",
        "slotReset",
        "slotConvertCurrencyChanged",
        "state",
        "slotSearch",
        "slotShowHelp",
        "slotUpdateCheckTransfers",
        "slotUpdateColumnsCombo",
        "idx",
        "slotLogAxisChanged",
        "slotNegExpensesChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotRowTypeChanged'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotColumnTypeChanged'
        QtMocHelpers::SlotData<void(int)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotReset'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotConvertCurrencyChanged'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'slotSearch'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotShowHelp'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotUpdateCheckTransfers'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotUpdateColumnsCombo'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotUpdateColumnsCombo'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'slotLogAxisChanged'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'slotNegExpensesChanged'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KReportConfigurationFilterDlg, qt_meta_tag_ZN29KReportConfigurationFilterDlgE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KReportConfigurationFilterDlg::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29KReportConfigurationFilterDlgE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29KReportConfigurationFilterDlgE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29KReportConfigurationFilterDlgE_t>.metaTypes,
    nullptr
} };

void KReportConfigurationFilterDlg::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KReportConfigurationFilterDlg *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotRowTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->slotColumnTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->slotReset(); break;
        case 3: _t->slotConvertCurrencyChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->slotSearch(); break;
        case 5: _t->slotShowHelp(); break;
        case 6: _t->slotUpdateCheckTransfers(); break;
        case 7: _t->slotUpdateColumnsCombo(); break;
        case 8: _t->slotUpdateColumnsCombo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->slotLogAxisChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->slotNegExpensesChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KReportConfigurationFilterDlg::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KReportConfigurationFilterDlg::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29KReportConfigurationFilterDlgE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int KReportConfigurationFilterDlg::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
